@@ -1,6 +1,6 @@
-/* $Id: about.hpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: about.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -26,9 +26,9 @@ class config;
 namespace about
 {
 
-void show_about(display &disp, std::string campaign="");
+void show_about(display &disp, const std::string &campaign = std::string());
 void set_about(const config& cfg);
-std::vector<std::string> get_text(std::string campaign="");
+std::vector<std::string> get_text(const std::string &campaign = std::string());
 
 }
 
