@@ -1,7 +1,7 @@
-/* $Id: addon_management.cpp 35863 2009-05-24 09:54:22Z soliton $ */
+/* $Id: addon_management.cpp 46627 2010-09-20 08:10:30Z shadowmaster $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
-                 2008 - 2009 by Ignacio R. Morelle <shadowm2006@gmail.com>
+                 2008 - 2010 by Ignacio R. Morelle <shadowm2006@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -26,22 +26,27 @@
 #include "gui/dialogs/addon_connect.hpp"
 #include "gui/dialogs/addon_list.hpp"
 #include "gui/dialogs/message.hpp"
+#include "gui/dialogs/transient_message.hpp"
+#include "gui/widgets/settings.hpp"
 #include "gui/widgets/window.hpp"
+#include "log.hpp"
 #include "marked-up_text.hpp"
 #include "serialization/parser.hpp"
 #include "version.hpp"
 #include "wml_separators.hpp"
 #include "formula_string_utils.hpp"
 
+static lg::log_domain log_config("config");
+#define ERR_CFG LOG_STREAM(err , log_config)
+#define LOG_CFG LOG_STREAM(info, log_config)
+#define WRN_CFG LOG_STREAM(warn, log_config)
 
-#define DEFAULT_CAMPAIGND_PORT				15003
+static lg::log_domain log_filesystem("filesystem");
+#define ERR_FS  LOG_STREAM(err , log_filesystem)
 
-#define ERR_CFG LOG_STREAM(err , config)
-#define LOG_CFG LOG_STREAM(info, config)
-#define WRN_CFG LOG_STREAM(warn, config)
-#define ERR_FS  LOG_STREAM(err , filesystem)
-#define ERR_NET LOG_STREAM(err , network)
-#define LOG_NET LOG_STREAM(info, network)
+static lg::log_domain log_network("network");
+#define ERR_NET LOG_STREAM(err , log_network)
+#define LOG_NET LOG_STREAM(info, log_network)
 
 void get_addon_info(const std::string& addon_name, config& cfg)
 {
@@ -110,6 +115,16 @@ std::vector<std::string> available_addons()
 			res.push_back(*i);
 		}
 	}
+	for(std::vector<std::string>::const_iterator i = files.begin(); i != files.end(); ++i) {
+		const size_t length = i->size() - 4;
+		if (i->rfind(".cfg", length) != length) continue;
+		const std::string name = i->substr(0, length);
+		// Continue if there is a dir (which we already processed).
+		if (std::find(dirs.begin(), dirs.end(), name) != dirs.end()) continue;
+		if (std::find(files.begin(), files.end(), name + ".pbl") != files.end()) {
+			res.push_back(name);
+		}
+	}
 
 	return res;
 }
@@ -156,6 +171,8 @@ namespace {
 		   UNIX platforms */
 		files.push_back(".*");
 		dirs.push_back(".*");
+		/* MacOS X metadata-like cruft (http://floatingsun.net/2007/02/07/whats-with-__macosx-in-zip-files/) */
+		dirs.push_back("__MACOSX");
 
 		files.push_back("#*#");
 		files.push_back("*~");
@@ -201,6 +218,7 @@ static std::pair<std::vector<std::string>, std::vector<std::string> > read_ignor
 	std::istream *stream = istream_file(ign_file);
 	std::string line;
 	while (std::getline(*stream, line)) {
+		utils::strip(line);
 		const size_t l = line.size();
 		if (line[l - 1] == '/') { // directory; we strip the last /
 			patterns.second.push_back(line.substr(0, l - 1));
@@ -218,6 +236,11 @@ static void archive_file(const std::string& path, const std::string& fname, conf
 	cfg["contents"] = encode_binary(strip_cr(read_file(path + '/' + fname),is_cfg));
 }
 
+inline bool looks_like_pbl(const std::string& file)
+{
+	return utils::wildcard_string_match(utils::lowercase(file), "*.pbl");
+}
+
 static void archive_dir(const std::string& path, const std::string& dirname, config& cfg, std::pair<std::vector<std::string>, std::vector<std::string> >& ignore_patterns)
 {
 	cfg["name"] = dirname;
@@ -226,7 +249,7 @@ static void archive_dir(const std::string& path, const std::string& dirname, con
 	std::vector<std::string> files, dirs;
 	get_files_in_dir(dir,&files,&dirs);
 	for(std::vector<std::string>::const_iterator i = files.begin(); i != files.end(); ++i) {
-		bool valid = true;
+		bool valid = !looks_like_pbl(*i);
 		for(std::vector<std::string>::const_iterator p = ignore_patterns.first.begin(); p != ignore_patterns.first.end(); ++p) {
 			if (utils::wildcard_string_match(*i, *p)) {
 				valid = false;
@@ -281,14 +304,12 @@ static void unarchive_dir(const std::string& path, const config& cfg)
 
 	make_directory(dir);
 
-	const config::child_list& dirs = cfg.get_children("dir");
-	for(config::child_list::const_iterator i = dirs.begin(); i != dirs.end(); ++i) {
-		unarchive_dir(dir,**i);
+	foreach (const config &d, cfg.child_range("dir")) {
+		unarchive_dir(dir, d);
 	}
 
-	const config::child_list& files = cfg.get_children("file");
-	for(config::child_list::const_iterator j = files.begin(); j != files.end(); ++j) {
-		unarchive_file(dir,**j);
+	foreach (const config &f, cfg.child_range("file")) {
+		unarchive_file(dir, f);
 	}
 }
 
@@ -299,6 +320,46 @@ void unarchive_addon(const config& cfg)
 }
 
 namespace {
+	/** Class to handle showing add-on descriptions. */
+	class display_description : public gui::dialog_button_action
+	{
+		display& disp_;
+		std::vector<std::string> titles_, desc_;
+		gui::filter_textbox* filter_;
+
+	public:
+		display_description(display& disp, std::vector<std::string> const& titles, std::vector<std::string> const& descriptions, gui::filter_textbox* filter)
+			: disp_(disp)
+			, titles_(titles)
+			, desc_(descriptions)
+			, filter_(filter)
+		{}
+
+		virtual gui::dialog_button_action::RESULT button_pressed(int filter_choice)
+		{
+			assert(filter_ != NULL);
+			const int menu_selection = filter_->get_index(filter_choice);
+			if(menu_selection < 0) { return gui::CONTINUE_DIALOG; }
+			size_t const uchoice = static_cast<size_t>(menu_selection);
+
+			std::string text;
+			std::string title;
+
+			if(uchoice >= desc_.size()) {
+				text = _("No description available.");
+			}
+			else {
+				title = titles_[uchoice];
+				text = desc_[uchoice];
+			}
+
+			gui2::show_transient_message(disp_.video(), title, text);
+
+			return gui::CONTINUE_DIALOG;
+		}
+	};
+
+
 	/**
 	 * Strip the ".cfg" extension and replace "_" with " " for display.
 	 *
@@ -414,6 +475,8 @@ namespace {
 				return _("addon_type^MP campaign");
 			case ADDON_MEDIA:
 				return _("addon_type^Resources");
+			case ADDON_OTHER:
+				return _("addon_type^Other");
 			default:
 				return _("addon_type^(unknown)");
 		}
@@ -427,7 +490,8 @@ namespace {
 	 *
 	 * @returns       true if dependencies are met; false otherwise.
 	 */
-	bool addon_dependencies_met(game_display& disp, const std::vector<std::string>& deplist)
+	bool addon_dependencies_met(game_display &disp,
+		const std::vector<std::string> &deplist, const std::string &addon_title)
 	{
 		const std::vector<std::string>& installed = installed_addons();
 		std::vector<std::string>::const_iterator i;
@@ -445,14 +509,21 @@ namespace {
 		// the missing dependencies.
 		if (!missing.empty()) {
 			const std::string msg_title    = _("Dependencies");
-			const std::string msg_entrytxt = _n("This add-on depends upon the following add-on which you have not installed yet:",
-												"This add-on depends upon the following add-ons which you have not installed yet:",
-												count_missing);
-			/* GCC-3.3 needs a temp var otherwise compilation fails */
-			gui::dialog dlg(disp, msg_title, msg_entrytxt + "\n" + missing +
-			                "\n" + _("Do you want to download this add-on? (You will still have to install the dependencies in order to play.)"), gui::OK_CANCEL);
-			if (dlg.show())
+			utils::string_map symbols;
+			symbols["addon_title"] = addon_title;
+			const std::string msg_entrytxt = utils::interpolate_variables_into_string(
+				_n("$addon_title depends upon the following add-on which you have not installed yet:",
+				"$addon_title depends upon the following add-ons which you have not installed yet:",
+				count_missing), &symbols);
+			std::string msg_reminder = utils::interpolate_variables_into_string(_("Do you still want to download $addon_title|? (You will have to install all the dependencies in order to play.)"), &symbols);
+
+			if(!(gui2::show_message(disp.video()
+					, msg_title
+					, msg_entrytxt + "\n \n" + missing + "\n \n" + msg_reminder
+					, gui2::tmessage::ok_cancel_buttons) == gui2::twindow::OK)) {
+
 				return false;
+			}
 		}
 		return true;
 	}
@@ -465,20 +536,22 @@ namespace {
 		config data;
 		sock = network::receive_data(data,sock,5000);
 		if(!sock) {
-			gui::show_error_message(disp, _("Connection timed out"));
+			gui2::show_error_message(disp.video(), _("Connection timed out"));
 			return;
-		} else if(data.child("error")) {
+		} else if (const config &c = data.child("error")) {
 			std::string error_message = _("The server responded with an error: \"$error|\"");
 			utils::string_map symbols;
-			symbols["error"] = (*data.child("error"))["message"].str();
+			symbols["error"] = c["message"].str();
 			error_message = utils::interpolate_variables_into_string(error_message, &symbols);
-			gui::show_error_message(disp, error_message);
+			gui2::show_error_message(disp.video(), error_message);
 			return;
-		} else if(data.child("message")) {
-			/* GCC-3.3 needs a temp var otherwise compilation fails */
-			gui::dialog dlg(disp,_("Terms"),(*data.child("message"))["message"],gui::OK_CANCEL);
-			const int res = dlg.show();
-			if(res != 0) {
+		} else if (const config &c = data.child("message")) {
+
+			if(gui2::show_message(disp.video()
+					,_("Terms")
+					, c["message"]
+					, gui2::tmessage::ok_cancel_buttons) != gui2::twindow::OK) {
+
 				return;
 			}
 		}
@@ -513,13 +586,11 @@ namespace {
 		sock = dialogs::network_send_dialog(disp,_("Sending add-on"),data,sock);
 		if(!sock) {
 			return;
-		} else if(data.child("error")) {
-			gui::show_error_message(disp, _("The server responded with an error: \"") +
-									(*data.child("error"))["message"].str() + '"');
-		} else if(data.child("message")) {
-			/* GCC-3.3 needs a temp var otherwise compilation fails */
-			gui::message_dialog dlg(disp,_("Response"),(*data.child("message"))["message"]);
-			dlg.show();
+		} else if (const config &c = data.child("error")) {
+			gui2::show_error_message(disp.video(), _("The server responded with an error: \"") +
+			                        c["message"].str() + '"');
+		} else if (const config &c = data.child("message")) {
+			gui2::show_transient_message(disp.video(), _("Response"), c["message"]);
 		}
 	}
 
@@ -539,14 +610,12 @@ namespace {
 
 		sock = network::receive_data(data,sock,5000);
 		if(!sock) {
-			gui::show_error_message(disp, _("Connection timed out"));
-		} else if(data.child("error")) {
-			gui::show_error_message(disp, _("The server responded with an error: \"") +
-									(*data.child("error"))["message"].str() + '"');
-		} else if(data.child("message")) {
-			/* GCC-3.3 needs a temp var otherwise compilation fails */
-			gui::message_dialog dlg(disp,_("Response"),(*data.child("message"))["message"]);
-			dlg.show();
+			gui2::show_error_message(disp.video(), _("Connection timed out"));
+		} else if (const config &c = data.child("error")) {
+			gui2::show_error_message(disp.video(), _("The server responded with an error: \"") +
+			                        c["message"].str() + '"');
+		} else if (const config &c = data.child("message")) {
+			gui2::show_transient_message(disp.video(), _("Response"), c["message"]);
 		}
 	}
 
@@ -559,12 +628,12 @@ namespace {
 	                   bool show_result = true)
 	{
 		// Get all dependencies of the addon/campaign selected for download.
-		const config * const selected_campaign = addons_tree.find_child("campaign", "name", addon_id);
-		assert(selected_campaign != NULL);
+		const config &selected_campaign = addons_tree.find_child("campaign", "name", addon_id);
+		assert(selected_campaign);
 		// Get all dependencies which are not already installed.
 		// TODO: Somehow determine if the version is outdated.
-		std::vector<std::string> dependencies = utils::split((*selected_campaign)["dependencies"]);
-		if (!addon_dependencies_met(disp,dependencies)) return false;
+		std::vector<std::string> dependencies = utils::split(selected_campaign["dependencies"]);
+		if (!addon_dependencies_met(disp, dependencies, addon_title)) return false;
 		// Proceed to download and install
 		config request;
 		request.add_child("request_campaign")["name"] = addon_id;
@@ -582,20 +651,19 @@ namespace {
 			return false;
 		}
 
-		config const * const dlerror = cfg.child("error");
-		if(dlerror != NULL) {
-			gui::show_error_message(disp, (*dlerror)["message"]);
+		if (config const &dlerror = cfg.child("error")) {
+			gui2::show_error_message(disp.video(), dlerror["message"]);
 			return false;
 		}
 
 		if(!check_names_legal(cfg)) {
-			gui::show_error_message(disp, _("The add-on has an invalid file or directory name and can not be installed."));
+			gui2::show_error_message(disp.video(), _("The add-on has an invalid file or directory name and can not be installed."));
 			return false;
 		}
 
 		// add revision info to the addon archive
-		config* maindir = cfg.find_child("dir", "name", addon_id);
-		if(maindir == NULL) {
+		config *maindir = &cfg.find_child("dir", "name", addon_id);
+		if (!*maindir) {
 			LOG_CFG << "downloaded addon '" << addon_id << "' is missing its own directory, creating...\n";
 			maindir = &cfg.add_child("dir");
 			(*maindir)["name"] = addon_id;
@@ -639,9 +707,7 @@ namespace {
 		if(show_result) {
 			const std::string& message =
 				utils::interpolate_variables_into_string(_("The add-on '$addon_title|' has been successfully installed."), &syms);
-			/* GCC-3.3 needs a temp var otherwise compilation fails */
-			gui::message_dialog dlg(disp, _("Add-on Installed"), message);
-			dlg.show();
+			gui2::show_transient_message(disp.video(), _("Add-on Installed"), message);
 		}
 
 		if(do_refresh != NULL)
@@ -649,11 +715,36 @@ namespace {
 		return true;
 	}
 
-	void addons_update_dlg(game_display& disp, config& cfg, const config::child_list& remote_addons_list,
+	void do_addon_icon_fixups(std::string& icon, const std::string& addon_name)
+	{
+		static const std::string default_icon = "misc/blank-hex.png";
+		static const std::string debug_default_icon = "misc/missing-image.png";
+
+		if(icon.empty()) {
+			ERR_CFG << "remote add-on '" << addon_name << "' has no icon\n";
+			icon = default_icon;
+		}
+		else if(!image::exists(icon)) {
+			ERR_CFG << "remote add-on '" << addon_name << "' has an icon which cannot be found: '" << icon << "'\n";
+			if(game_config::debug) {
+				icon = debug_default_icon;
+			}
+			else {
+				icon = default_icon;
+			}
+		}
+		else if(icon.find("units/") != std::string::npos && icon.find_first_of('~') == std::string::npos) {
+			// a hack to prevent magenta icons, because they look awful
+			WRN_CFG << "remote add-on '" << addon_name << "' has an icon without TC/RC specification\n";
+			icon.append("~RC(magenta>red)");
+		}
+	}
+
+	void addons_update_dlg(game_display &disp, config &cfg, const config::const_child_itors &remote_addons_list,
 	                       const network::manager& net_manager, const network::connection& sock,
 	                       bool* do_refresh)
 	{
-		std::vector< config* > remote_matches_cfgs;
+		std::vector<const config *> remote_matches_cfgs;
 		std::vector< std::string > safe_matches;
 		std::vector< std::string > unsafe_matches;
 		std::ostringstream unsafe_list;
@@ -664,10 +755,11 @@ namespace {
 		std::vector<version_info> safe_local_versions;
 		std::vector<version_info> unsafe_local_versions;
 		std::map<std::string, version_info> remote_version_map;
-		foreach(config* const remote_addon, remote_addons_list) {
+		foreach (const config &remote_addon, remote_addons_list)
+		{
 			if(remote_addon == NULL) continue; // shouldn't happen...
-			const std::string& name = (*remote_addon)["name"];
-			const std::string& version = (*remote_addon)["version"];
+			const std::string& name = remote_addon["name"];
+			const std::string& version = remote_addon["version"];
 			try {
 				remote_version_map.insert(std::make_pair(name, version_info(version)));
 			} catch(version_info::not_sane_exception const&) {
@@ -688,7 +780,7 @@ namespace {
 						} else {
 							safe_matches.push_back(name);
 							safe_local_versions.push_back(local_version);
-							remote_matches_cfgs.push_back(remote_addon);
+							remote_matches_cfgs.push_back(&remote_addon);
 						}
 					}
 				}
@@ -703,15 +795,17 @@ namespace {
 				"An outdated local add-on has publishing information attached. It will not be offered for updating.",
 				"Some outdated local add-ons have publishing information attached. They will not be offered for updating.",
 				unsafe_matches.size());
-			/* GCC-3.3 needs a temp var otherwise compilation fails */
-			gui::dialog dlg(disp, warn_title, warn_entrytxt + unsafe_list.str(), gui::MESSAGE);
-			dlg.show();
+
+			gui2::show_transient_message(disp.video()
+					, warn_title
+					, warn_entrytxt + unsafe_list.str());
 		}
 
 		if(safe_matches.empty()) {
-			gui::dialog dlg(disp, _("No add-ons to update"), _("Could not find any updated add-ons on this server."),
-			                gui::MESSAGE);
-			dlg.show();
+
+			gui2::show_transient_message(disp.video()
+					, _("No add-ons to update")
+					, _("Could not find any updated add-ons on this server."));
 			return;
 		}
 
@@ -761,10 +855,7 @@ namespace {
 			newversions.push_back(newver);
 
 			std::string icon = c["icon"];
-			if(icon.find("units/") != std::string::npos &&
-				icon.find_first_of('~') == std::string::npos) {
-				icon.append("~RC(magenta>red)");
-			}
+			do_addon_icon_fixups(icon, name);
 
 			const std::string text_columns =
 				title + COLUMN_SEPARATOR +
@@ -821,12 +912,12 @@ namespace {
 		bool result = true;
 		std::vector<std::string> failed_titles;
 
-		assert(cfg.child("campaigns") != NULL);
+		assert(cfg.child("campaigns"));
 
 		if(upd_all) {
 			for(size_t i = 0; i < addons.size() && i < remote_matches_cfgs.size(); ++i)
 			{
-				if(!install_addon(disp, * cfg.child("campaigns"), addons[i], titles[i],
+				if (!install_addon(disp, cfg.child("campaigns"), addons[i], titles[i],
 				                   types[i], uploads[i], newversions[i], net_manager, sock,
 				                   do_refresh, false)) {
 					result=false;
@@ -835,7 +926,7 @@ namespace {
 			}
 		} else {
 			const size_t i = static_cast<size_t>(index);
-			if(!install_addon(disp, * cfg.child("campaigns"), addons[i], titles[i],
+			if (!install_addon(disp, cfg.child("campaigns"), addons[i], titles[i],
 				               types[i], uploads[i], newversions[i], net_manager, sock,
 				               do_refresh, false)) {
 				result=false;
@@ -855,33 +946,32 @@ namespace {
 				_n("The following add-on could not be downloaded or updated successfully:",
 				   "The following add-ons could not be downloaded or updated successfully:",
 				   failed_titles.size()) + failed_titles_list_fmt;
-			/* GCC-3.3 needs a temp var otherwise compilation fails */
-			gui::dialog err_dlg(disp, err_title, err_message, gui::MESSAGE);
-			err_dlg.show();
+
+			gui2::show_message(disp.video(), err_title, err_message);
 			return;
 		}
 
 		const std::string msg_title = _("Update succeeded");
 		const std::string msg_message = !upd_all ? _("Add-on updated successfully.") :
 		                                           _("All add-ons updated successfully.");
-		/* GCC-3.3 needs a temp var otherwise compilation fails */
-		gui::dialog msg_dlg(disp, msg_title, msg_message, gui::MESSAGE);
-		msg_dlg.show();
+
+		gui2::show_message(disp.video(), msg_title, msg_message);
 	}
 
-	void download_addons(game_display& disp, std::string remote_host, bool update_mode, bool* do_refresh)
+	void download_addons(game_display& disp, const std::string& remote_address,
+			bool update_mode, bool* do_refresh, int old_index = 0)
 	{
 		const std::vector<std::string> address_components =
-			utils::split(remote_host, ':');
+			utils::split(remote_address, ':');
 		if(address_components.empty()) {
 			return;
 		}
 
 		const std::string old_host = preferences::campaign_server();
 		const int remote_port = lexical_cast_default<int>(address_components.back(),
-		                                                  DEFAULT_CAMPAIGND_PORT);
-		remote_host = address_components.front();
-		preferences::set_campaign_server(remote_host);
+		                                                  default_campaignd_port);
+		std::string remote_host = address_components.front();
+		preferences::set_campaign_server(remote_address);
 
 		try {
 			const network::manager net_manager;
@@ -889,7 +979,7 @@ namespace {
 				dialogs::network_connect_dialog(disp, _("Connecting to add-ons server..."),
 				                                remote_host, remote_port);
 			if(!sock) {
-				gui::show_error_message(disp, _("Could not connect to host."));
+				gui2::show_error_message(disp.video(), _("Could not connect to host."));
 				preferences::set_campaign_server(old_host);
 				return;
 			}
@@ -902,26 +992,25 @@ namespace {
 				return;
 			}
 
-			config const * const error = cfg.child("error");
-			if(error != NULL) {
-				gui::show_error_message(disp, (*error)["message"]);
+			if (config const &error = cfg.child("error")) {
+				gui2::show_error_message(disp.video(), error["message"]);
 				return;
 			}
 
-			config const * const addons_tree = cfg.child("campaigns");
-			if(addons_tree == NULL) {
-				gui::show_error_message(disp, _("An error occurred while communicating with the server."));
+			config const &addons_tree = cfg.child("campaigns");
+			if (!addons_tree) {
+				gui2::show_error_message(disp.video(), _("An error occurred while communicating with the server."));
 				return;
 			}
 
-			const config::child_list& addon_cfgs = addons_tree->get_children("campaign");
+			const config::const_child_itors &addon_cfgs = addons_tree.child_range("campaign");
 			if(update_mode) {
 				addons_update_dlg(disp, cfg, addon_cfgs, net_manager, sock, do_refresh);
 				return;
 			}
 
 			// column contents
-			std::vector<std::string> addons, titles, versions, uploads, types, options, options_to_filter;
+			std::vector<std::string> addons, titles, versions, uploads, types, options, options_to_filter, descriptions;
 			std::vector<int> sizes;
 
 			std::string sep(1, COLUMN_SEPARATOR);
@@ -935,9 +1024,8 @@ namespace {
 
 			std::vector< std::string > delete_options;
 
-			foreach(const config* i, addon_cfgs) {
-				const config& c = *i;
-
+			foreach(const config &c, addon_cfgs)
+			{
 				const std::string& name = c["name"];
 				const std::string& downloads = c["downloads"].str();
 				const std::string& size = c["size"];
@@ -949,6 +1037,7 @@ namespace {
 				addons.push_back(name);
 				versions.push_back(c["version"]);
 				uploads.push_back(c["uploads"]);
+				descriptions.push_back(c["description"]);
 				types.push_back(type_str);
 
 				if(std::count(publish_options.begin(), publish_options.end(), name) != 0) {
@@ -964,20 +1053,13 @@ namespace {
 
 				std::string version = c["version"], author = c["author"];
 
-				utils::truncate_as_wstring(title, 20);
-				utils::truncate_as_wstring(version, 12);
-				utils::truncate_as_wstring(author, 16);
-
 				//add negative sizes to reverse the sort order
 				sizes.push_back(-atoi(size.c_str()));
 
 				std::string icon = c["icon"];
-				if(icon.find("units/") != std::string::npos &&
-				   icon.find_first_of('~') == std::string::npos) {
-					//a hack to prevent magenta icons, because they look awful
-					icon.append("~RC(magenta>red)");
-				}
-				const std::string text_columns =
+				do_addon_icon_fixups(icon, name);
+
+				std::string text_columns =
 					title + COLUMN_SEPARATOR +
 					version + COLUMN_SEPARATOR +
 					author + COLUMN_SEPARATOR +
@@ -985,9 +1067,23 @@ namespace {
 					downloads + COLUMN_SEPARATOR +
 					sizef + COLUMN_SEPARATOR;
 
-				options.push_back(IMAGE_PREFIX + icon + COLUMN_SEPARATOR + text_columns);
 				// icon paths shouldn't be filtered!
-				options_to_filter.push_back(text_columns);
+				options_to_filter.push_back(text_columns + descriptions.back());
+
+				utils::truncate_as_wstring(title, 20);
+				utils::truncate_as_wstring(version, 12);
+				utils::truncate_as_wstring(author, 16);
+
+				text_columns =
+					IMAGE_PREFIX + icon + COLUMN_SEPARATOR +
+					title + COLUMN_SEPARATOR +
+					version + COLUMN_SEPARATOR +
+					author + COLUMN_SEPARATOR +
+					type_label_str + COLUMN_SEPARATOR +
+					downloads + COLUMN_SEPARATOR +
+					sizef + COLUMN_SEPARATOR;
+
+				options.push_back(text_columns);
 			}
 
 			std::string pub_option_text, del_option_text;
@@ -1006,13 +1102,13 @@ namespace {
 			}
 
 			if(addons.empty() && publish_options.empty()) {
-				gui::show_error_message(disp, _("There are no add-ons available for download from this server."));
+				gui2::show_error_message(disp.video(), _("There are no add-ons available for download from this server."));
 				return;
 			}
 
 			int index = -1;
 			if(gui2::new_widgets) {
-				gui2::taddon_list dlg(*addons_tree);
+				gui2::taddon_list dlg(addons_tree);
 				dlg.show(disp.video());
 			} else {
 
@@ -1035,6 +1131,14 @@ namespace {
 				_("Filter: "), options, options_to_filter, 1, addon_dialog, 300);
 				addon_dialog.set_textbox(filter);
 
+				display_description description_helper(disp, titles, descriptions, filter);
+
+				gui::dialog_button* description = new gui::dialog_button(disp.video(), _("Description"), gui::button::TYPE_PRESS, gui::CONTINUE_DIALOG, &description_helper);
+				addon_dialog.add_button(description, gui::dialog::BUTTON_EXTRA);
+
+				// Scroll the menu to the previous selection
+				addon_menu->move_selection(old_index);
+
 				index = addon_dialog.show();
 				index = filter->get_index(index);
 			}
@@ -1058,18 +1162,21 @@ namespace {
 			}
 
 			// Handle download
-			install_addon(disp, * addons_tree, addons[index], titles[index], types[index],
+			install_addon(disp, addons_tree, addons[index], titles[index], types[index],
 			              uploads[index], versions[index], net_manager, sock, do_refresh);
 
+			// Show the dialog again, and position it on the same item installed
+			download_addons(disp, remote_address, update_mode, do_refresh, index);
+
 		} catch(config::error& e) {
 			ERR_CFG << "config::error thrown during transaction with add-on server; \""<< e.message << "\"\n";
-			gui::show_error_message(disp, _("Network communication error."));
+			gui2::show_error_message(disp.video(), _("Network communication error."));
 		} catch(network::error& e) {
 			ERR_NET << "network::error thrown during transaction with add-on server; \""<< e.message << "\"\n";
-			gui::show_error_message(disp, _("Remote host disconnected."));
+			gui2::show_error_message(disp.video(), _("Remote host disconnected."));
 		} catch(io_exception& e) {
 			ERR_FS << "io_exception thrown while installing an addon; \"" << e.what() << "\"\n";
-			gui::show_error_message(disp, _("A problem occurred when trying to create the files necessary to install this add-on."));
+			gui2::show_error_message(disp.video(), _("A problem occurred when trying to create the files necessary to install this add-on."));
 		} catch(twml_exception& e) {
 			e.show(disp);
 		}
@@ -1086,8 +1193,7 @@ namespace {
 		prepare_addons_list_for_display(addons, addon_dirs, parentdir);
 
 		if (addons.empty()) {
-			/** @todo should use a dialog which always shows the close button. */
-			gui2::show_message(disp.video(), _("Error"),
+			gui2::show_error_message(disp.video(),
 				_("You have no add-ons installed."));
 			return;
 		}
@@ -1116,10 +1222,12 @@ namespace {
 			utils::string_map symbols;
 			symbols["addon"] = addons.at(index);
 			confirm_message = utils::interpolate_variables_into_string(confirm_message, &symbols);
-			/* GCC-3.3 needs a temp var otherwise compilation fails */
-			gui::dialog confirm_dlg(disp, _("Confirm"), confirm_message, gui::YES_NO);
-			res = confirm_dlg.show();
-		} while (res != 0);
+
+			res = gui2::show_message(disp.video()
+					, _("Confirm")
+					, confirm_message
+					, gui2::tmessage::yes_no_buttons);
+		} while (res != gui2::twindow::OK);
 
 		// Put underscores back in the name
 		std::string addon_id = addons.at(index);
@@ -1133,22 +1241,19 @@ namespace {
 			utils::string_map symbols;
 			symbols["addon"] = addons.at(index);
 			message = utils::interpolate_variables_into_string(message, &symbols);
-			/* GCC-3.3 needs a temp var otherwise compilation fails */
-			gui::dialog dlg(disp, _("Add-on deleted"), message, gui::OK_ONLY);
-			dlg.show();
+			gui2::show_transient_message(disp.video()
+					, _("Add-on deleted")
+					, message);
 
 			if(should_reload_cfg != NULL)
 				*should_reload_cfg = true;
 		}
 		else
 		{
-			const std::string err_msg_title = _("Error");
 			std::string err_msg = _("Add-on could not be deleted properly:");
 			err_msg += '\n';
 			err_msg += removal_log;
-			/* GCC-3.3 needs a temp var otherwise compilation fails */
-			gui::dialog dlg2(disp, err_msg_title, err_msg, gui::OK_ONLY);
-			dlg2.show();
+			gui2::show_error_message(disp.video(), err_msg);
 		}
 	}
 
@@ -1165,10 +1270,13 @@ void manage_addons(game_display& disp)
 	bool do_refresh = false;
 	std::string remote_host;
 	const std::string default_host = preferences::campaign_server();
+	const bool have_addons = !installed_addons().empty();
 
 	gui2::taddon_connect addon_dlg;
 
 	addon_dlg.set_host_name(default_host);
+	addon_dlg.set_allow_remove(have_addons);
+	addon_dlg.set_allow_updates(have_addons);
 	addon_dlg.show(disp.video());
 
 	res = addon_dlg.get_retval();
@@ -1221,24 +1329,28 @@ void refresh_addon_version_info_cache()
 	foreach(std::string const& info_file, addon_info_files) {
 		assert(i < addons.size());
 
-		config cfg;
 		std::string const& addon = addons[i];
 		++i;
 
 		if(file_exists(info_file)) {
 			scoped_istream stream = istream_file(info_file);
+			config cfg;
 			read(cfg, *stream);
 
-			config const* const info_cfg = cfg.child("info");
-			if(info_cfg == NULL) {
+			config const &info_cfg = cfg.child("info");
+			if (!info_cfg) {
 				continue;
 			}
-			std::string const& version = (*info_cfg)["version"];
+			std::string const& version = info_cfg["version"];
 			LOG_CFG << "caching add-on version info: " << addon << " [" << version << "]\n";
 			version_info_cache.insert(std::make_pair(addon, version_info(version)));
 		}
-		else {
-			WRN_CFG << "add-on '" << addon << "' has no _info.cfg; cannot read version info\n";
+		// Don't print the warning if the user is clearly the author
+		else if (!file_exists(parentd+"/"+addon+".pbl")
+			  && !file_exists(parentd+"/"+addon+"/_server.pbl")
+			  && !file_exists(parentd+"/"+addon+"/.svn")
+			  && !file_exists(parentd+"/"+addon+"/.git")) {
+				WRN_CFG << "add-on '" << addon << "' has no _info.cfg; cannot read version info\n";
 		}
 	}
 }
