@@ -1,6 +1,6 @@
-/* $Id: animated.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: animated.hpp 41222 2010-02-14 22:18:06Z boucman $ */
 /*
-   Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2010 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -12,8 +12,10 @@
    See the COPYING file for more details.
 */
 
-//! @file animated.hpp
-//! Animate units.
+/**
+ * @file animated.hpp
+ * Animate units.
+ */
 
 #ifndef ANIMATED_IMAGE_H_INCLUDED
 #define ANIMATED_IMAGE_H_INCLUDED
@@ -47,13 +49,18 @@ public:
 	animated(const std::vector<frame_description> &cfg, int start_time = 0,bool force_change =false);
 
 
-	//! Adds a frame to an animation.
+	/** Adds a frame to an animation. */
 	void add_frame(int duration, const T& value,bool force_change =false);
 
-	//! Starts an animation cycle.
-	//! The first frame of the animation to start may be set
-	//! to any value by using a start_time different to 0.
+	/**
+	 * Starts an animation cycle.
+	 *
+	 * The first frame of the animation to start may be set to any value by
+	 * using a start_time different to 0.
+	 */
 	void start_animation(int start_time, bool cycles=false);
+	void pause_animation(){ started_ =false;};
+	void restart_animation(){if(start_tick_) started_ = true;};
 
 	int get_begin_time() const;
 	int get_end_time() const;
@@ -67,32 +74,31 @@ public:
 
 	bool cycles() const {return cycles_;};
 
-	//! Returns true if the current animation was finished.
+	/** Returns true if the current animation was finished. */
 	bool animation_finished() const;
 	bool animation_finished_potential() const;
 	int get_animation_time() const;
 	int get_animation_time_potential() const;
 
-	const int get_animation_duration() const;
+	int get_animation_duration() const;
 	const T& get_current_frame() const;
-	const int get_current_frame_begin_time() const;
-	const int get_current_frame_end_time() const;
-	const int get_current_frame_duration() const;
-	const int get_current_frame_time() const;
+	int get_current_frame_begin_time() const;
+	int get_current_frame_end_time() const;
+	int get_current_frame_duration() const;
+	int get_current_frame_time() const;
 	const T& get_first_frame() const;
 	const T& get_last_frame() const;
 	int get_frames_count() const;
 	void force_change() {does_not_change_ = false ; }
-	const bool does_not_change() const {return does_not_change_;}
+	bool does_not_change() const {return does_not_change_;}
 
 	static const T void_value_; //MSVC: the frame constructor below requires this to be public
 
 protected:
 friend class unit_animation;
 	int starting_frame_time_;
-        // backward compatibility for teleport anims
-        void remove_frames_until(int starting_time); 
-        void remove_frames_after(int ending_time); 
+        void remove_frames_until(int starting_time);
+        void set_end_time(int ending_time);
 
 private:
 	struct frame
