@@ -1,6 +1,6 @@
-/* $Id: builder.cpp 23963 2008-02-20 00:16:11Z esr $ */
+/* $Id: builder.cpp 40968 2010-01-31 12:47:38Z boucman $ */
 /*
-   Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2010 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -12,25 +12,26 @@
    See the COPYING file for more details.
 */
 
-//! @file builder.cpp
-//! Terrain builder.
+/**
+ * @file builder.cpp
+ * Terrain builder.
+ */
 
 #include "global.hpp"
 
-#include "array.hpp"
 #include "builder.hpp"
 #include "config.hpp"
+#include "foreach.hpp"
 #include "log.hpp"
-#include "pathutils.hpp"
-#include "terrain.hpp"
-#include "util.hpp"
+#include "map.hpp"
 #include "serialization/string_utils.hpp"
+#include "image.hpp"
 
-#include <cassert>
 #include <climits>
 
-#define ERR_NG LOG_STREAM(err, engine)
-#define DEBUG_NG LOG_STREAM(info, engine)
+static lg::log_domain log_engine("engine");
+#define ERR_NG LOG_STREAM(err, log_engine)
+#define WRN_NG LOG_STREAM(warn, log_engine)
 
 /** The tile width used when using basex and basey. This is not,
  * necessarily, the tile width in pixels, this is totally
@@ -48,12 +49,17 @@ static const int UNITPOS = 36 + 18;
  */
 static const int BASE_Y_INTERVAL = 100000;
 
-terrain_builder::rule_image::rule_image(int layer, int x, int y, bool global_image) :
+terrain_builder::building_ruleset terrain_builder::building_rules_;
+const config* terrain_builder::rules_cfg_ = NULL;
+
+terrain_builder::rule_image::rule_image(int layer, int x, int y, bool global_image, int cx, int cy) :
 	layer(layer),
 	basex(x),
 	basey(y),
 	variants(),
-	global_image(global_image)
+	global_image(global_image),
+	center_x(cx),
+	center_y(cy)
 {}
 
 terrain_builder::tile::tile() :
@@ -112,53 +118,99 @@ void terrain_builder::tile::clear()
 
 void terrain_builder::tilemap::reset()
 {
-	for(std::vector<tile>::iterator it = map_.begin(); it != map_.end(); ++it)
+	for(std::vector<tile>::iterator it = tiles_.begin(); it != tiles_.end(); ++it)
 		it->clear();
 }
 
-bool terrain_builder::tilemap::on_map(const gamemap::location &loc) const
+void terrain_builder::tilemap::reload(int x, int y)
+{
+	x_ = x;
+	y_ = y;
+    std::vector<terrain_builder::tile> new_tiles((x + 4) * (y + 4));
+    tiles_.swap(new_tiles);
+}
+
+bool terrain_builder::tilemap::on_map(const map_location &loc) const
 {
 	if(loc.x < -2 || loc.y < -2 || loc.x > (x_ + 1) || loc.y > (y_ + 1)) {
 		return false;
 	}
 
 	return true;
-	
+
 }
 
-terrain_builder::tile& terrain_builder::tilemap::operator[](const gamemap::location &loc)
+terrain_builder::tile& terrain_builder::tilemap::operator[](const map_location &loc)
 {
 	assert(on_map(loc));
 
-	return map_[(loc.x + 2) + (loc.y + 2) * (x_ + 4)];
+	return tiles_[(loc.x + 2) + (loc.y + 2) * (x_ + 4)];
 }
 
-const terrain_builder::tile& terrain_builder::tilemap::operator[] (const gamemap::location &loc) const
+const terrain_builder::tile& terrain_builder::tilemap::operator[] (const map_location &loc) const
 {
 	assert(on_map(loc));
 
-	return map_[(loc.x + 2) + (loc.y + 2) * (x_ + 4)];
+	return tiles_[(loc.x + 2) + (loc.y + 2) * (x_ + 4)];
 }
 
-terrain_builder::terrain_builder(const config& cfg, const config& level,
-		const gamemap& map, const std::string& offmap_image) :
-	map_(map), 
-	tile_map_(map.w(), map.h()),
-	terrain_by_type_(),
-	building_rules_()
+terrain_builder::terrain_builder(const config& level,
+		const gamemap* m, const std::string& offmap_image) :
+	map_(m),
+	tile_map_(map().w(), map().h()),
+	terrain_by_type_()
 {
-	// Make sure there's nothing left in the cache,
-	// since it might give problems (or not?)
-	//image::flush_cache();
+	image::precache_file_existence("terrain/");
 
-	parse_config(cfg);
+	if(building_rules_.empty() && rules_cfg_){
+		// parse global terrain rules
+		parse_global_config(*rules_cfg_);
+	} else {
+		// use cached global rules but clear local rules
+		flush_local_rules();
+	}
+
+	// parse local rules
 	parse_config(level);
 	add_off_map_rule(offmap_image);
 
 	build_terrains();
 }
 
-const terrain_builder::imagelist *terrain_builder::get_terrain_at(const gamemap::location &loc,
+void terrain_builder::flush_local_rules()
+{
+	building_ruleset::iterator i = building_rules_.begin();
+	for(; i != building_rules_.end();){
+		if(i->second.local)
+			building_rules_.erase(i++);
+		else
+			++i;
+	}
+}
+
+void terrain_builder::set_terrain_rules_cfg(const config& cfg)
+{
+	rules_cfg_ = &cfg;
+	// use the swap trick to clear the rules cache and get a fresh one.
+	// because simple clear() seems to cause some progressive memory degradation.
+	building_ruleset empty;
+	std::swap(building_rules_, empty);
+}
+
+void terrain_builder::reload_map()
+{
+	tile_map_.reload(map().w(), map().h());
+	terrain_by_type_.clear();
+	build_terrains();
+}
+
+void terrain_builder::change_map(const gamemap* m)
+{
+	map_ = m;
+	reload_map();
+}
+
+const terrain_builder::imagelist *terrain_builder::get_terrain_at(const map_location &loc,
 		const std::string &tod, const ADJACENT_TERRAIN_TYPE terrain_type)
 {
 	if(!tile_map_.on_map(loc))
@@ -184,7 +236,7 @@ const terrain_builder::imagelist *terrain_builder::get_terrain_at(const gamemap:
 	return NULL;
 }
 
-bool terrain_builder::update_animation(const gamemap::location &loc)
+bool terrain_builder::update_animation(const map_location &loc)
 {
 	if(!tile_map_.on_map(loc))
 		return false;
@@ -210,8 +262,8 @@ bool terrain_builder::update_animation(const gamemap::location &loc)
 	return changed;
 }
 
-//! @todo TODO: rename this function
-void terrain_builder::rebuild_terrain(const gamemap::location &loc)
+/** @todo TODO: rename this function */
+void terrain_builder::rebuild_terrain(const map_location &loc)
 {
 	if (tile_map_.on_map(loc)) {
 		tile& btile = tile_map_[loc];
@@ -219,11 +271,21 @@ void terrain_builder::rebuild_terrain(const gamemap::location &loc)
 		btile.images_foreground.clear();
 		btile.images_background.clear();
 		const std::string filename =
-			map_.get_terrain_info(map_.get_terrain(loc)).minimap_image();
+			map().get_terrain_info(map().get_terrain(loc)).minimap_image();
 		animated<image::locator> img_loc;
 		img_loc.add_frame(100,image::locator("terrain/" + filename + ".png"));
 		img_loc.start_animation(0, true);
 		btile.images_background.push_back(img_loc);
+
+		//Combine base and overlay image if neccessary
+		if(map().get_terrain_info(map().get_terrain(loc)).is_combined()) {
+			const std::string filename_ovl =
+				map().get_terrain_info(map().get_terrain(loc)).minimap_image_overlay();
+			animated<image::locator> img_loc_ovl;
+			img_loc_ovl.add_frame(100,image::locator("terrain/" + filename_ovl + ".png"));
+			img_loc_ovl.start_animation(0, true);
+			btile.images_background.push_back(img_loc_ovl);
+		}
 	}
 }
 
@@ -256,7 +318,9 @@ bool terrain_builder::rule_valid(const building_rule &rule) const
 				std::string s = variant->second.image_string;
 				s = s.substr(0, s.find_first_of(",:"));
 
-				if(!image::exists("terrain/" + s + ".png"))
+				// we already precached file existence in the constructor
+				// but only for filenames not using ".."
+				if(!image::exists("terrain/" + s + ".png", s.find("..") == std::string::npos))
 					return false;
 			}
 		}
@@ -296,7 +360,7 @@ bool terrain_builder::start_animation(building_rule &rule)
 						time = 100;
 					}
 					if(image->global_image) {
-						image_vector.push_back(animated<image::locator>::frame_description(time,image::locator("terrain/" + str + ".png",constraint->second.loc)));
+						image_vector.push_back(animated<image::locator>::frame_description(time,image::locator("terrain/" + str + ".png",constraint->second.loc, image->center_x, image->center_y)));
 					} else {
 						image_vector.push_back(animated<image::locator>::frame_description(time,image::locator("terrain/" + str + ".png")));
 					}
@@ -436,13 +500,13 @@ void terrain_builder::replace_token(terrain_builder::building_rule &rule, const
 		// Transforms attributes
 		std::vector<std::string>::iterator flag;
 
-		for(flag = cons->second.set_flag.begin(); flag != cons->second.set_flag.end(); flag++) {
+		for(flag = cons->second.set_flag.begin(); flag != cons->second.set_flag.end(); ++flag) {
 			replace_token(*flag, token, replacement);
 		}
-		for(flag = cons->second.no_flag.begin(); flag != cons->second.no_flag.end(); flag++) {
+		for(flag = cons->second.no_flag.begin(); flag != cons->second.no_flag.end(); ++flag) {
 			replace_token(*flag, token, replacement);
 		}
-		for(flag = cons->second.has_flag.begin(); flag != cons->second.has_flag.end(); flag++) {
+		for(flag = cons->second.has_flag.begin(); flag != cons->second.has_flag.end(); ++flag) {
 			replace_token(*flag, token, replacement);
 		}
 		replace_token(cons->second.images, token, replacement);
@@ -461,7 +525,7 @@ terrain_builder::building_rule terrain_builder::rotate_rule(const terrain_builde
 	}
 	ret.location_constraints = rule.location_constraints;
 	ret.probability = rule.probability;
-	ret.precedence = rule.precedence;
+	ret.local = rule.local;
 
 	constraint_set tmp_cons;
 	constraint_set::const_iterator cons;
@@ -477,8 +541,8 @@ terrain_builder::building_rule terrain_builder::rotate_rule(const terrain_builde
 
 	constraint_set::iterator cons2;
 	for(cons2 = tmp_cons.begin(); cons2 != tmp_cons.end(); ++cons2) {
-		minx = minimum<int>(cons2->second.loc.x, minx);
-		miny = minimum<int>(2*cons2->second.loc.y + (cons2->second.loc.x & 1), miny);
+		minx = std::min<int>(cons2->second.loc.x, minx);
+		miny = std::min<int>(2*cons2->second.loc.y + (cons2->second.loc.x & 1), miny);
 	}
 
 	if((miny & 1) && (minx & 1) && (minx < 0))
@@ -488,14 +552,14 @@ terrain_builder::building_rule terrain_builder::rotate_rule(const terrain_builde
 
 	for(cons2 = tmp_cons.begin(); cons2 != tmp_cons.end(); ++cons2) {
 		// Adjusts positions
-		cons2->second.loc += gamemap::location(-minx, -((miny-1)/2));
+		cons2->second.loc.legacy_sum_assign(map_location(-minx, -((miny-1)/2)));
 		ret.constraints[cons2->second.loc] = cons2->second;
 	}
 
 	for(int i = 0; i < 6; ++i) {
 		int a = (angle+i) % 6;
 		std::string token = "@R";
-		push_back(token,'0' + i);
+		token.push_back('0' + i);
 		replace_token(ret, token, rot[a]);
 	}
 
@@ -504,20 +568,17 @@ terrain_builder::building_rule terrain_builder::rotate_rule(const terrain_builde
 
 void terrain_builder::add_images_from_config(rule_imagelist& images, const config &cfg, bool global, int dx, int dy)
 {
-	const config::child_list& cimages = cfg.get_children("image");
-
-
-	for(config::child_list::const_iterator img = cimages.begin(); img != cimages.end(); ++img) {
-
-		const std::string &name = (**img)["name"];
-		const int layer = lexical_cast_default<int>((**img)["layer"], 0);
+	foreach (const config &img, cfg.child_range("image"))
+	{
+		const std::string &name = img["name"];
+		const int layer = lexical_cast_default<int>(img["layer"], 0);
 
 		int basex = 0, basey = 0;
-		if((**img)["base"].empty()) {
+		if (img["base"].empty()) {
 			basex = TILEWIDTH / 2 + dx;
 			basey = TILEWIDTH / 2 + dy;
 		} else {
-			std::vector<std::string> base = utils::split((**img)["base"]);
+			std::vector<std::string> base = utils::split(img["base"]);
 
 			if(base.size() >= 2) {
 				basex = atoi(base[0].c_str());
@@ -525,19 +586,26 @@ void terrain_builder::add_images_from_config(rule_imagelist& images, const confi
 			}
 		}
 
-		images.push_back(rule_image(layer, basex - dx, basey - dy, global));
+		int center_x = -1, center_y = -1;
+		if (!img["center"].empty()) {
+			std::vector<std::string> center = utils::split(img["center"]);
 
+			if(center.size() >= 2) {
+				center_x = atoi(center[0].c_str());
+				center_y = atoi(center[1].c_str());
+			}
+		}
+
+		images.push_back(rule_image(layer, basex - dx, basey - dy, global, center_x, center_y));
 
 		// Adds the main (default) variant of the image, if present
 		images.back().variants.insert(std::pair<std::string, rule_image_variant>("", rule_image_variant(name,"")));
 
 		// Adds the other variants of the image
-		const config::child_list& variants = (**img).get_children("variant");
-
-		for(config::child_list::const_iterator variant = variants.begin();
-				variant != variants.end(); ++variant) {
-			const std::string &name = (**variant)["name"];
-			const std::string &tod = (**variant)["tod"];
+		foreach (const config &variant, img.child_range("variant"))
+		{
+			const std::string &name = variant["name"];
+			const std::string &tod = variant["tod"];
 
 			images.back().variants.insert(std::pair<std::string, rule_image_variant>(tod, rule_image_variant(name,tod)));
 
@@ -547,7 +615,7 @@ void terrain_builder::add_images_from_config(rule_imagelist& images, const confi
 
 void terrain_builder::add_constraints(
 		terrain_builder::constraint_set& constraints,
-		const gamemap::location& loc,
+		const map_location& loc,
 		const t_translation::t_match& type, const config& global_images)
 {
 	if(constraints.find(loc) == constraints.end()) {
@@ -565,7 +633,7 @@ void terrain_builder::add_constraints(
 }
 
 void terrain_builder::add_constraints(terrain_builder::constraint_set &constraints,
-		const gamemap::location& loc, const config& cfg, const config& global_images)
+		const map_location& loc, const config& cfg, const config& global_images)
 
 {
 	add_constraints(constraints, loc, t_translation::t_match(cfg["type"], t_translation::WILDCARD), global_images);
@@ -613,12 +681,12 @@ void terrain_builder::parse_mapstring(const std::string &mapstring,
 				// Dots are simple placeholders,
 				// which do not represent actual terrains.
 			} else if (terrain.overlay != 0 ) {
-				anchors.insert(std::pair<int, gamemap::location>(terrain.overlay, gamemap::location(x, y)));
+				anchors.insert(std::pair<int, map_location>(terrain.overlay, map_location(x, y)));
 			} else if (terrain.base == t_translation::TB_STAR) {
-				add_constraints(br.constraints, gamemap::location(x, y), t_translation::STAR, global_images);
+				add_constraints(br.constraints, map_location(x, y), t_translation::STAR, global_images);
 			} else {
 					ERR_NG << "Invalid terrain (" << t_translation::write_terrain_code(terrain) << ") in builder map\n";
-					assert(false); 
+					assert(false);
 					return;
 			}
 		x += 2;
@@ -634,116 +702,120 @@ void terrain_builder::parse_mapstring(const std::string &mapstring,
 	}
 }
 
-void terrain_builder::add_rule(building_ruleset& rules, building_rule &rule)
+void terrain_builder::add_rule(building_ruleset& rules, building_rule &rule, int precedence)
 {
 	if(rule_valid(rule)) {
 		start_animation(rule);
-		rules.insert(std::pair<int, building_rule>(rule.precedence, rule));
+		rules.insert(std::pair<int, building_rule>(precedence, rule));
 	}
 }
 
-void terrain_builder::add_rotated_rules(building_ruleset& rules, building_rule& tpl, const std::string &rotations)
+void terrain_builder::add_rotated_rules(building_ruleset& rules, building_rule& tpl, int precedence, const std::string &rotations )
 {
 	if(rotations.empty()) {
 		// Adds the parsed built terrain to the list
 
-		add_rule(rules, tpl);
+		add_rule(rules, tpl, precedence);
 	} else {
 		const std::vector<std::string>& rot = utils::split(rotations, ',');
 
-		for(size_t angle = 0; angle < rot.size(); angle++) {
+		for(size_t angle = 0; angle < rot.size(); ++angle) {
 			building_rule rule = rotate_rule(tpl, angle, rot);
-			add_rule(rules, rule);
+			add_rule(rules, rule, precedence);
 		}
 	}
 }
 
-void terrain_builder::parse_config(const config &cfg)
+void terrain_builder::parse_config(const config &cfg, bool local)
 {
 	log_scope("terrain_builder::parse_config");
 
 	// Parses the list of building rules (BRs)
-	const config::child_list& brs = cfg.get_children("terrain_graphics");
-
-	for(config::child_list::const_iterator br = brs.begin(); br != brs.end(); ++br) {
+	foreach (const config &br, cfg.child_range("terrain_graphics"))
+	{
 		building_rule pbr; // Parsed Building rule
+		pbr.local = local;
 
 		// add_images_from_config(pbr.images, **br);
 
-		if(!((**br)["x"].empty() || (**br)["y"].empty()))
-			pbr.location_constraints = gamemap::location(atoi((**br)["x"].c_str()), atoi((**br)["y"].c_str()));
+		if(!br["x"].empty() && !br["y"].empty())
+			pbr.location_constraints =
+				map_location(atoi(br["x"].c_str()) - 1, atoi(br["y"].c_str()) - 1);
 
-		pbr.probability = (**br)["probability"].empty() ? -1 : atoi((**br)["probability"].c_str());
-		pbr.precedence = (**br)["precedence"].empty() ? 0 : atoi((**br)["precedence"].c_str());
+		pbr.probability = br["probability"].empty() ? -1 : atoi(br["probability"].c_str());
 
 		// Mapping anchor indices to anchor locations.
 		anchormap anchors;
 
 		// Parse the map= , if there is one (and fill the anchors list)
-		parse_mapstring((**br)["map"], pbr, anchors, **br);
+		parse_mapstring(br["map"], pbr, anchors, br);
 
 		// Parses the terrain constraints (TCs)
-		config::child_list tcs((*br)->get_children("tile"));
-
-		for(config::child_list::const_iterator tc = tcs.begin(); tc != tcs.end(); tc++) {
+		foreach (const config &tc, br.child_range("tile"))
+		{
 			// Adds the terrain constraint to the current built terrain's list
 			// of terrain constraints, if it does not exist.
-			gamemap::location loc;
-			if((**tc)["x"].size()) {
-				loc.x = atoi((**tc)["x"].c_str());
+			map_location loc;
+			if (!tc["x"].empty()) {
+				loc.x = atoi(tc["x"].c_str());
 			}
-			if((**tc)["y"].size()) {
-				loc.y = atoi((**tc)["y"].c_str());
+			if (!tc["y"].empty()) {
+				loc.y = atoi(tc["y"].c_str());
 			}
-			if(!(**tc)["loc"].empty()) {
-				std::vector<std::string> sloc = utils::split((**tc)["loc"]);
+			if (!tc["loc"].empty()) {
+				std::vector<std::string> sloc = utils::split(tc["loc"]);
 				if(sloc.size() == 2) {
 					loc.x = atoi(sloc[0].c_str());
 					loc.y = atoi(sloc[1].c_str());
 				}
 			}
 			if(loc.valid()) {
-				add_constraints(pbr.constraints, loc, **tc, **br);
+				add_constraints(pbr.constraints, loc, tc, br);
 			}
-			if((**tc)["pos"].size()) {
-				int pos = atoi((**tc)["pos"].c_str());
+			if (!tc["pos"].empty()) {
+				int pos = atoi(tc["pos"].c_str());
 				if(anchors.find(pos) == anchors.end()) {
-					LOG_STREAM(warn, engine) << "Invalid anchor!\n";
+					WRN_NG << "Invalid anchor!\n";
 					continue;
 				}
 
 				std::pair<anchormap::const_iterator, anchormap::const_iterator> range =
 					anchors.equal_range(pos);
 
-				for(; range.first != range.second; range.first++) {
+				for(; range.first != range.second; ++range.first) {
 					loc = range.first->second;
-					add_constraints(pbr.constraints, loc, **tc, **br);
+					add_constraints(pbr.constraints, loc, tc, br);
 				}
 			}
 		}
 
-		const std::string global_set_flag = (**br)["set_flag"];
-		const std::string global_no_flag = (**br)["no_flag"];
-		const std::string global_has_flag = (**br)["has_flag"];
+		const std::vector<std::string> global_set_flag = utils::split(br["set_flag"]);
+		const std::vector<std::string> global_no_flag = utils::split(br["no_flag"]);
+		const std::vector<std::string> global_has_flag = utils::split(br["has_flag"]);
 
 		for(constraint_set::iterator constraint = pbr.constraints.begin(); constraint != pbr.constraints.end();
-		    constraint++) {
+		    ++constraint) {
 
 			if(global_set_flag.size())
-				constraint->second.set_flag.push_back(global_set_flag);
+				constraint->second.set_flag.insert(constraint->second.set_flag.end(),
+						global_set_flag.begin(), global_set_flag.end());
 
 			if(global_no_flag.size())
-				constraint->second.no_flag.push_back(global_no_flag);
+				constraint->second.no_flag.insert(constraint->second.no_flag.end(),
+						global_no_flag.begin(), global_no_flag.end());
 
 			if(global_has_flag.size())
-				constraint->second.has_flag.push_back(global_has_flag);
+				constraint->second.has_flag.insert(constraint->second.has_flag.end(),
+						global_has_flag.begin(), global_has_flag.end());
 
 		}
 
 		// Handles rotations
-		const std::string rotations = (**br)["rotations"];
+		const std::string &rotations = br["rotations"];
 
-		add_rotated_rules(building_rules_, pbr, rotations);
+		int precedence = lexical_cast_default<int>(br["precedence"],0);
+
+		add_rotated_rules(building_rules_, pbr, precedence, rotations);
 
 	}
 
@@ -753,10 +825,9 @@ void terrain_builder::parse_config(const config &cfg)
 
 	building_ruleset::const_iterator rule;
 	for(rule = building_rules_.begin(); rule != building_rules_.end(); ++rule) {
-		std::cerr << ">> New rule: image_background = " 
+		std::cerr << ">> New rule: image_background = "
 			<< "\n>> Location " << rule->second.location_constraints
-			<< "\n>> Probability " << rule->second.probability 
-			<< "\n>> Precedence " << rule->second.precedence << '\n';
+			<< "\n>> Probability " << rule->second.probability
 
 		for(constraint_set::const_iterator constraint = rule->second.constraints.begin();
 		    constraint != rule->second.constraints.end(); ++constraint) {
@@ -785,30 +856,27 @@ void terrain_builder::add_off_map_rule(const std::string& image)
 	// Build a config object
 	config cfg;
 
-	cfg.add_child("terrain_graphics");
-	config *item = cfg.child("terrain_graphics");
+	config &item = cfg.add_child("terrain_graphics");
 
-	(*item).add_child("tile");
-	config *tile = (*item).child("tile");
-	(*tile)["x"] = "0";
-	(*tile)["y"] = "0";
-	(*tile)["type"] = t_translation::write_terrain_code(t_translation::OFF_MAP_USER);
+	config &tile = item.add_child("tile");
+	tile["x"] = "0";
+	tile["y"] = "0";
+	tile["type"] = t_translation::write_terrain_code(t_translation::OFF_MAP_USER);
 
-	(*tile).add_child("image");
-	config *tile_image = (*tile).child("image");
-	(*tile_image)["layer"] = "-1000";
-	(*tile_image)["name"] = image;
+	config &tile_image = tile.add_child("image");
+	tile_image["layer"] = "-1000";
+	tile_image["name"] = image;
 
-	(*item)["probability"] = "100";
-	(*item)["no_flag"] = "base";
-	(*item)["set_flag"] = "base";
+	item["probability"] = "100";
+	item["no_flag"] = "base";
+	item["set_flag"] = "base";
 
 	// Parse the object
 	parse_config(cfg);
 }
 
 bool terrain_builder::rule_matches(const terrain_builder::building_rule &rule,
-		const gamemap::location &loc, const int rule_index, const constraint_set::const_iterator type_checked) const
+		const map_location &loc, const int rule_index, const constraint_set::const_iterator type_checked) const
 {
 	if(rule.location_constraints.valid() && rule.location_constraints != loc) {
 		return false;
@@ -830,17 +898,17 @@ bool terrain_builder::rule_matches(const terrain_builder::building_rule &rule,
 			cons != rule.constraints.end(); ++cons) {
 
 		// Translated location
-		const gamemap::location tloc = loc + cons->second.loc;
+		const map_location tloc = loc.legacy_sum(cons->second.loc);
 
 		if(!tile_map_.on_map(tloc)) {
 			return false;
 		}
 
-		//std::cout << "testing..." << builder_letter(map_.get_terrain(tloc))
+		//std::cout << "testing..." << builder_letter(map().get_terrain(tloc))
 
 		// check if terrain matches except if we already know that it does
 		if(cons != type_checked &&
-				!terrain_matches(map_.get_terrain(tloc), cons->second.terrain_types_match)) {
+				!terrain_matches(map().get_terrain(tloc), cons->second.terrain_types_match)) {
 			return false;
 		}
 
@@ -866,13 +934,13 @@ bool terrain_builder::rule_matches(const terrain_builder::building_rule &rule,
 	return true;
 }
 
-void terrain_builder::apply_rule(const terrain_builder::building_rule &rule, const gamemap::location &loc)
+void terrain_builder::apply_rule(const terrain_builder::building_rule &rule, const map_location &loc)
 {
 	for(constraint_set::const_iterator constraint = rule.constraints.begin();
 			constraint != rule.constraints.end(); ++constraint) {
 
 		rule_imagelist::const_iterator img;
-		const gamemap::location tloc = loc + constraint->second.loc;
+		const map_location tloc = loc.legacy_sum(constraint->second.loc);
 		if(!tile_map_.on_map(tloc)) {
 			return;
 		}
@@ -889,7 +957,7 @@ void terrain_builder::apply_rule(const terrain_builder::building_rule &rule, con
 
 		// Sets flags
 		for(std::vector<std::string>::const_iterator itor = constraint->second.set_flag.begin();
-				itor != constraint->second.set_flag.end(); itor++) {
+				itor != constraint->second.set_flag.end(); ++itor) {
 			btile.flags.insert(*itor);
 		}
 
@@ -901,10 +969,10 @@ void terrain_builder::build_terrains()
 	log_scope("terrain_builder::build_terrains");
 
 	// Builds the terrain_by_type_ cache
-	for(int x = -2; x <= map_.w(); ++x) {
-		for(int y = -2; y <= map_.h(); ++y) {
-			const gamemap::location loc(x,y);
-			const t_translation::t_terrain t = map_.get_terrain(loc);
+	for(int x = -2; x <= map().w(); ++x) {
+		for(int y = -2; y <= map().h(); ++y) {
+			const map_location loc(x,y);
+			const t_translation::t_terrain t = map().get_terrain(loc);
 
 			terrain_by_type_[t].push_back(loc);
 		}
@@ -917,11 +985,6 @@ void terrain_builder::build_terrains()
 
 		const building_rule& rule = r->second;
 
-		if (rule.location_constraints.valid()) {
-			apply_rule(rule, rule.location_constraints);
-			continue;
-		}
-
 		// Find the constraint that contains the less terrain of all terrain rules.
 		// We will keep a track of the matching terrains of this constraint
 		// and later try to apply the rule only on them
@@ -937,7 +1000,7 @@ void terrain_builder::build_terrains()
 			size_t constraint_size = 0;
 
 			for (terrain_by_type_map::iterator type_it = terrain_by_type_.begin();
-					 type_it != terrain_by_type_.end(); type_it++) {
+					 type_it != terrain_by_type_.end(); ++type_it) {
 
 				const t_translation::t_terrain t = type_it->first;
 				if (terrain_matches(t, match)) {
@@ -966,11 +1029,11 @@ void terrain_builder::build_terrains()
 		for(t_translation::t_list::const_iterator t = min_types.begin();
 				t != min_types.end(); ++t) {
 
-			const std::vector<gamemap::location>* locations = &terrain_by_type_[*t];
+			const std::vector<map_location>* locations = &terrain_by_type_[*t];
 
-			for(std::vector<gamemap::location>::const_iterator itor = locations->begin();
+			for(std::vector<map_location>::const_iterator itor = locations->begin();
 					itor != locations->end(); ++itor) {
-				const gamemap::location loc = *itor - min_constraint->second.loc;
+				const map_location loc = itor->legacy_difference(min_constraint->second.loc);
 
 				if(rule_matches(rule, loc, rule_index, min_constraint)) {
 					apply_rule(rule, loc);
@@ -978,6 +1041,6 @@ void terrain_builder::build_terrains()
 			}
 		}
 
-		rule_index++;
+		++rule_index;
 	}
 }
