@@ -1,6 +1,6 @@
-/* $Id: builder.hpp 23963 2008-02-20 00:16:11Z esr $ */
+/* $Id: builder.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2010 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -12,21 +12,24 @@
    See the COPYING file for more details.
 */
 
-//! @file builder.hpp
-//! Definitions for the terrain builder.
+/**
+ * @file builder.hpp
+ * Definitions for the terrain builder.
+ */
 
 #ifndef BUILDER_H_INCLUDED
 #define BUILDER_H_INCLUDED
 
 #include "animated.hpp"
-#include "image.hpp"
-#include "map.hpp"
+#include "map_location.hpp"
+#include "terrain_translation.hpp"
 
 #include <map>
 #include <set>
 
 class config;
-
+class gamemap;
+namespace image{ class locator; }
 /**
  * The class terrain_builder is constructed from a config object, and a
  * gamemap object. On construction, it parses the configuration and extracts
@@ -42,10 +45,16 @@ class config;
 class terrain_builder
 {
 public:
-	//! Used as a parameter for the get_terrain_at function.
+	/** Used as a parameter for the get_terrain_at function. */
 	enum ADJACENT_TERRAIN_TYPE {
-			ADJACENT_BACKGROUND,	//!< Represents terrains which are to be drawn behind unit sprites
-			ADJACENT_FOREGROUND	//!< Represents terrains which are to be drawn in front of them.
+			ADJACENT_BACKGROUND,	/**<
+									 * Represents terrains which are to be
+									 * drawn behind unit sprites
+									 */
+			ADJACENT_FOREGROUND	    /**<
+									 * Represents terrains which are to be
+									 * drawn in front of them.
+									 */
 	};
 
 	/** A shorthand typedef for a list of animated image locators,
@@ -55,8 +64,6 @@ public:
 
 	/** Constructor for the terrain_builder class.
 	 *
-	 * @param cfg			The main grame configuration object, where the
-	 *						[terrain_graphics] rule reside.
 	 * @param level		A level (scenario)-specific configuration file,
 	 *		    containing scenario-specific [terrain_graphics] rules.
 	 * @param map			A properly-initialized gamemap object representing
@@ -66,8 +73,26 @@ public:
 	 *						This image automatically gets the 'terrain/' prefix
 	 *						and '.png' suffix
 	 */
-	terrain_builder(const config& cfg, const config &level,
-		const gamemap& map, const std::string& offmap_image);
+	terrain_builder(const config &level, const gamemap* map,
+			const std::string& offmap_image);
+
+	/**  Set the config where we will parse the global terrain rules.
+	 *   This also flushes the terrain rules cache.
+	 *
+	 * @param cfg			The main grame configuration object, where the
+	 *						[terrain_graphics] rule reside.
+	 */
+	static void set_terrain_rules_cfg(const config& cfg);
+
+	const gamemap& map() const { return *map_; }
+
+	/**
+	 * Updates internals that cache map size. This should be called when the map
+	 * size has changed.
+	 */
+	void reload_map();
+
+	void change_map(const gamemap* m);
 
 	/** Returns a vector of strings representing the images to load & blit
 	 * together to get the built content for this tile.
@@ -84,7 +109,7 @@ public:
 	 * @return      Returns a pointer list of animated images corresponding
 	 *              to the parameters, or NULL if there is none.
 	 */
-	const imagelist *get_terrain_at(const gamemap::location &loc,
+	const imagelist *get_terrain_at(const map_location &loc,
 			const std::string &tod, ADJACENT_TERRAIN_TYPE const terrain_type);
 
 	/** Updates the animation at a given tile.
@@ -94,7 +119,7 @@ public:
 	 *
 	 * @retval      true: this tile must be redrawn.
 	 */
-	bool update_animation(const gamemap::location &loc);
+	bool update_animation(const map_location &loc);
 
 	/** Performs a "quick-rebuild" of the terrain in a given location.
 	 * The "quick-rebuild" is no proper rebuild: it only clears the
@@ -103,7 +128,7 @@ public:
 	 *
 	 * @param loc   the location where to rebuild terrains
 	 */
-	void rebuild_terrain(const gamemap::location &loc);
+	void rebuild_terrain(const map_location &loc);
 
 	/** Performs a complete rebuild of the list of terrain graphics
 	 * attached to a map.
@@ -121,7 +146,7 @@ public:
 		rule_image_variant(const std::string &image_string, const std::string &tod) :
 			image_string(image_string),
 			image(),
-			tod(tod) 
+			tod(tod)
 			{};
 
 		/** A string representing either the filename for an image, or
@@ -169,7 +194,7 @@ public:
 	 * The rule_image structure represents one such image.
 	 */
 	struct rule_image {
-		rule_image(int layer, int x, int y, bool global_image=false);
+		rule_image(int layer, int x, int y, bool global_image=false, int center_x=-1, int center_y=-1);
 
 		/** The layer of the image for horizontal layering */
 		int layer;
@@ -186,6 +211,10 @@ public:
 		 * [terrain_graphics] tag, set to false if it was defined as a
 		 * child of a [tile] tag */
 		bool global_image;
+
+		/** The position where the center of the image base should be
+		 */
+		int center_x, center_y;
 	};
 
 	/**
@@ -199,7 +228,7 @@ public:
 	 */
 	struct terrain_constraint
 	{
-		terrain_constraint() : 
+		terrain_constraint() :
 			loc(),
 			terrain_types_match(),
 			set_flag(),
@@ -208,7 +237,7 @@ public:
 			images()
 			{};
 
-		terrain_constraint(gamemap::location loc) : 
+		terrain_constraint(map_location loc) :
 			loc(loc),
 			terrain_types_match(),
 			set_flag(),
@@ -217,7 +246,7 @@ public:
 			images()
 			{};
 
-		gamemap::location loc;
+		map_location loc;
 		t_translation::t_match terrain_types_match;
 		std::vector<std::string> set_flag;
 		std::vector<std::string> no_flag;
@@ -290,7 +319,7 @@ private:
 	/**
 	 * The list of constraints attached to a terrain_graphics WML rule.
 	 */
-	typedef std::map<gamemap::location, terrain_constraint> constraint_set;
+	typedef std::map<map_location, terrain_constraint> constraint_set;
 
 	/**
 	 * The in-memory representation of a [terrain_graphics] WML rule.
@@ -301,7 +330,7 @@ private:
 			constraints(),
 			location_constraints(),
 			probability(0),
-			precedence(0)
+			local(false)
 		{}
 
 		/**
@@ -311,10 +340,10 @@ private:
 
 		/**
 		 * The location on which this map may match.
-		 * Set to a valid gamemap::location if the "x" and "y" parameters
+		 * Set to a valid map_location if the "x" and "y" parameters
 		 * of the [terrain_graphics] rule are set.
 		 */
-		gamemap::location location_constraints;
+		map_location location_constraints;
 
 		/**
 		 * The probability of this rule to match, when all conditions
@@ -324,12 +353,9 @@ private:
 		int probability;
 
 		/**
-		 * The precedence of this rule. Used to order rules differently
-		 * that the order in which they appear.
-		 * Defined if the "precedence" parameter of the
-		 * [terrain_graphics] element is set.
+		 * Indicate if the rule is only for this scenario
 		 */
-		int precedence;
+		bool local;
 	};
 
 	/**
@@ -341,8 +367,8 @@ private:
 		/**
 		 * Constructs a tilemap of dimensions x * y
 		 */
-		tilemap(int x, int y) : 
-				map_((x + 4) * (y + 4)),
+		tilemap(int x, int y) :
+				tiles_((x + 4) * (y + 4)),
 				x_(x),
 				y_(y)
 			{}
@@ -356,11 +382,11 @@ private:
 		 * @return		A reference to the tile at this location.
 		 *
 		 */
-		tile &operator[](const gamemap::location &loc);
+		tile &operator[](const map_location &loc);
 		/**
 		 * a const variant of operator[]
 		 */
-		const tile &operator[] (const gamemap::location &loc) const;
+		const tile &operator[] (const map_location &loc) const;
 
 		/**
 		 * Tests if a location is on the map.
@@ -369,15 +395,20 @@ private:
 		 *
 		 * @return		true if loc is on the map, false otherwise.
 		 */
-		bool on_map(const gamemap::location &loc) const;
+		bool on_map(const map_location &loc) const;
 
 		/**
 		 * Resets the whole tile map
 		 */
 		void reset();
+
+		/**
+		 * Rebuilds the map to a new set of dimensions
+		 */
+		void reload(int x, int y);
 	private:
 		/** The map */
-		std::vector<tile> map_;
+		std::vector<tile> tiles_;
 		/** The x dimension of the map */
 		int x_;
 		/** The y dimension of the map */
@@ -485,11 +516,11 @@ private:
 	 * (has_flag, set_flag and no_flag) contained in all constraints
 	 * of the building_rule.
 	 *
-	 * @param s            The building_rule  in which to do the replacement
+	 * @param rule         The building_rule in which to do the replacement
 	 * @param token        The token to substitute
 	 * @param replacement  The replacement string
 	 */
-	void replace_token(building_rule &s, const std::string &token,
+	void replace_token(building_rule &rule, const std::string &token,
 			const std::string& replacement);
 
 	/**
@@ -562,7 +593,7 @@ private:
 	 *                      describing rule-global images.
 	 */
 	void add_constraints(constraint_set& constraints,
-			const gamemap::location &loc, const t_translation::t_match& type,
+			const map_location &loc, const t_translation::t_match& type,
 			const config& global_images);
 
 	/**
@@ -577,10 +608,10 @@ private:
 	 *                      describing rule-global images.
 	 */
 	void add_constraints(constraint_set& constraints,
-			const gamemap::location &loc, const config &cfg,
+			const map_location &loc, const config &cfg,
 			const config& global_images);
 
-	typedef std::multimap<int, gamemap::location> anchormap;
+	typedef std::multimap<int, map_location> anchormap;
 
 	/**
 	 * Parses a map string (the map= element of a [terrain_graphics] rule,
@@ -599,10 +630,11 @@ private:
 	/**
 	 * Adds a rule to a ruleset. Checks for validity before adding the rule.
 	 *
-	 * @param rules   The ruleset into which to add the rules.
-	 * @param rule    The rule to add.
+	 * @param rules      The ruleset into which to add the rules.
+	 * @param rule       The rule to add.
+	 * @param precedence The precedence specified in [terrain_graphics].
 	 */
-	void add_rule(building_ruleset& rules, building_rule &rule);
+	void add_rule(building_ruleset& rules, building_rule &rule, int precedence);
 
 	/**
 	 * Adds a set of rules to a ruleset, from a template rule which spans
@@ -610,20 +642,25 @@ private:
 	 *
 	 * @param rules      The ruleset into which to add the rules.
 	 * @param tpl        The template rule
+	 * @param precedence The precedence specified in [terrain_graphics].
 	 * @param rotations  A comma-separated string containing the
 	 *					 6 values for replacing rotation
 	 *					 template strings @verbatim (@Rn) @endverbatim
 	 */
-	void add_rotated_rules(building_ruleset& rules, building_rule& tpl, const std::string &rotations);
+	void add_rotated_rules(building_ruleset& rules, building_rule& tpl,
+			int precedence, const std::string &rotations);
 
 	/**
 	 * Parses a configuration object containing [terrain_graphics] rules,
 	 * and fills the building_rules_ member of the current class according
 	 * to those.
 	 *
-	 * @param cfg		The configuration object to parse.
+	 * @param cfg       The configuration object to parse.
+	 * @param local     Mark the rules as local only.
 	 */
-	void parse_config(const config &cfg);
+	void parse_config(const config &cfg, bool local = true);
+
+	void parse_global_config(const config &cfg) { parse_config(cfg, false); }
 
 	/**
 	 * Adds a builder rule for the _off^_usr tile, this tile only has 1 image.
@@ -632,6 +669,8 @@ private:
 	 */
 	void add_off_map_rule(const std::string& image);
 
+	void flush_local_rules();
+
 	/**
 	 * Checks whether a terrain code matches a given list of terrain codes.
 	 *
@@ -673,7 +712,7 @@ private:
 	 * @param type_checked The constraint which we already know that its
 	 *                  terrain types matches.
 	 */
-	bool rule_matches(const building_rule &rule, const gamemap::location &loc,
+	bool rule_matches(const building_rule &rule, const map_location &loc,
 			const int rule_index, const constraint_set::const_iterator type_checked) const;
 
 	/**
@@ -684,7 +723,7 @@ private:
 	 * @param rule      The rule to apply
 	 * @param loc       The location to which to apply the rule.
 	 */
-	void apply_rule(const building_rule &rule, const gamemap::location &loc);
+	void apply_rule(const building_rule &rule, const map_location &loc);
 
 	/**
 	 * Calculates the list of terrains, and fills the tile_map_ member,
@@ -693,9 +732,10 @@ private:
 	void build_terrains();
 
 	/**
-	 * A reference to the gamemap class used in the current level.
+	 * A pointer to the gamemap class used in the current level.
 	 */
-	const gamemap& map_;
+	const gamemap* map_;
+
 	/**
 	 * The tile_map_ for the current level, which is filled by the
 	 * build_terrains_ method to contain "tiles" representing images
@@ -706,15 +746,18 @@ private:
 	/**
 	 * Shorthand typedef for a map associating a list of locations to a terrain type.
 	 */
-	typedef std::map<t_translation::t_terrain, std::vector<gamemap::location> > terrain_by_type_map;
+	typedef std::map<t_translation::t_terrain, std::vector<map_location> > terrain_by_type_map;
 
 	/**
 	 * A map representing all locations whose terrain is of a given type.
 	 */
 	terrain_by_type_map terrain_by_type_;
 
-	building_ruleset building_rules_;
+	/** Parsed terrain rules. Cached between instances */
+	static building_ruleset building_rules_;
 
+	/** Config used to parse global terrain rules */
+	static const config* rules_cfg_;
 };
 
 #endif
