@@ -1,6 +1,6 @@
-/* $Id: campaign_server.cpp 35864 2009-05-24 09:54:26Z soliton $ */
+/* $Id: campaign_server.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -20,15 +20,18 @@
  */
 
 #include "filesystem.hpp"
+#include "foreach.hpp"
 #include "log.hpp"
 #include "network_worker.hpp"
 #include "serialization/binary_or_text.hpp"
 #include "serialization/binary_wml.hpp"
 #include "serialization/parser.hpp"
+#include "serialization/string_utils.hpp"
 #include "game_config.hpp"
 #include "addon_checks.hpp"
 #include "version.hpp"
 #include "server/input_stream.hpp"
+#include "util.hpp"
 
 #include <csignal>
 
@@ -40,7 +43,8 @@
 #include <errno.h>
 #endif
 
-#define LOG_CS lg::err(lg::network, false)
+static lg::log_domain log_network("network");
+#define LOG_CS if (lg::err.dont_log(log_network)) ; else lg::err(log_network, false)
 
 //compatibility code for MS compilers
 #ifndef SIGHUP
@@ -51,7 +55,7 @@
 static void exit_sighup(int signal) {
 	assert(signal == SIGHUP);
 	LOG_CS << "SIGHUP caught, exiting without cleanup immediately.\n";
-	exit(-1);
+	exit(128 + SIGHUP);
 }
 
 static void exit_sigint(int signal) {
@@ -63,11 +67,9 @@ static void exit_sigint(int signal) {
 static void exit_sigterm(int signal) {
 	assert(signal == SIGTERM);
 	LOG_CS << "SIGTERM caught, exiting without cleanup immediately.\n";
-	exit(-1);
+	exit(128 + SIGTERM);
 }
 
-#define DEFAULT_CAMPAIGND_PORT          15003
-
 namespace {
 
 	config construct_message(const std::string& msg)
@@ -104,8 +106,8 @@ namespace {
 			void fire(const std::string& hook, const std::string& addon);
 			void convert_binary_to_gzip();
 			int load_config(); // return the server port
-			const config& campaigns() const { return *cfg_.child("campaigns"); }
-			config& campaigns() { return *cfg_.child("campaigns"); }
+			const config &campaigns() const { return cfg_.child("campaigns"); }
+			config &campaigns() { return cfg_.child("campaigns"); }
 			config cfg_;
 			const std::string file_;
 			const network::manager net_manager_;
@@ -140,7 +142,7 @@ namespace {
 
 			// execute the script, we run is a separate thread and share the
 			// output which will make the logging look ugly.
-			execlp(script.c_str(), script.c_str(), addon.c_str(), (char *)NULL);
+			execlp(script.c_str(), script.c_str(), addon.c_str(), static_cast<char *>(NULL));
 
 			// exec() and family never return; if they do, we have a problem
 			std::cerr << "ERROR: exec failed with errno " << errno << " for addon " << addon
@@ -164,7 +166,7 @@ namespace {
  		/** Seems like compression level above 6 is waste of cpu cycle */
  		compress_level_ = lexical_cast_default<int>(cfg_["compress_level"],6);
  		cfg_["compress_level"] = lexical_cast<std::string>(compress_level_);
-		return lexical_cast_default<int>(cfg_["port"], DEFAULT_CAMPAIGND_PORT);
+		return lexical_cast_default<int>(cfg_["port"], default_campaignd_port);
 	}
 
 	campaign_server::campaign_server(const std::string& cfgfile,
@@ -194,14 +196,13 @@ namespace {
 
 	void find_translations(const config& cfg, config& campaign)
 	{
-		const config::child_list& dirs = cfg.get_children("dir");
-		for(config::child_list::const_iterator i = dirs.begin(); i != dirs.end(); ++i) {
-			if((const t_string)(**i)["name"] == "LC_MESSAGES") {
-				config* language = &campaign.add_child("translation");
-				(*language)["language"] = cfg["name"];
-			}
-			else {
-				find_translations(**i, campaign);
+		foreach (const config &dir, cfg.child_range("dir"))
+		{
+			if (dir["name"] == "LC_MESSAGES") {
+				config &language = campaign.add_child("translation");
+				language["language"] = cfg["name"];
+			} else {
+				find_translations(dir, campaign);
 			}
 		}
 	}
@@ -214,33 +215,35 @@ namespace {
 	// again.
 	std::string check_python_scripts(config &data, std::string filename)
 	{
-		std::vector<config *> python_scripts = find_scripts(data, ".py");
-		if (!python_scripts.empty()) {
-			// Campaign contains python scripts.
-			config old_campaign;
-			scoped_istream stream = istream_file(filename);
-			read_gz(old_campaign, *stream);
-			std::vector<config *> old_scripts = find_scripts(old_campaign, ".py");
-			std::string script_names = "";
-			std::vector<config *>::iterator i, j;
-			// Go through all newly uploaded python scripts.
-			for (i = python_scripts.begin(); i != python_scripts.end(); ++i) {
-				bool already = false;
-				// Compare to existing, approved scripts.
-				for (j = old_scripts.begin(); j != old_scripts.end(); ++j) {
-					if ((**i)["contents"] != (**j)["contents"]) continue;
-					already = true;
-					break;
-				}
-				if (!already) {
-					script_names += "\n" + (**i)["name"];
-					(**i)["name"] += ".unchecked";
-				}
+		std::vector<config *> python_scripts;
+		find_scripts(data, ".py", python_scripts);
+		if (python_scripts.empty()) return std::string();
+		// Campaign contains python scripts.
+		config old_campaign;
+		scoped_istream stream = istream_file(filename);
+		read_gz(old_campaign, *stream);
+		std::vector<config *> old_scripts;
+		find_scripts(old_campaign, ".py", old_scripts);
+		std::string script_names;
+		// Go through all newly uploaded python scripts.
+		foreach (config *i, python_scripts)
+		{
+			bool already = false;
+			// Compare to existing, approved scripts.
+			foreach (const config *j, old_scripts)
+			{
+				if ((*i)["contents"] != (*j)["contents"]) continue;
+				already = true;
+				break;
+			}
+			if (!already) {
+				script_names += "\n" + (*i)["name"];
+				(*i)["name"] += ".unchecked";
 			}
-			if (script_names != "")
-				return "\nScripts awaiting approval:\n" + script_names;
 		}
-		return "";
+		if (!script_names.empty())
+			return "\nScripts awaiting approval:\n" + script_names;
+		return std::string();
 	}
 
 	// Go through all .py.unchecked files in the given campaign, and rename them to
@@ -249,61 +252,61 @@ namespace {
 	// command.
 	std::string validate_all_python_scripts(config &data)
 	{
-		std::vector<config *> python_scripts = find_scripts(data, ".py.unchecked");
-		if (!python_scripts.empty()) {
-			// Campaign contains unchecked python scripts.
-			std::string script_names = "";
-			std::vector<config *>::iterator i;
-			// Go through all unchecked python scripts.
-			for (i = python_scripts.begin(); i != python_scripts.end(); ++i) {
-				std::string name = (**i)["name"];
-				name.resize(name.length() - 10);
-				(**i)["name"] = name;
-				script_names += "\n" + name;
-			}
-			return script_names;
+		std::vector<config *> python_scripts;
+		find_scripts(data, ".py.unchecked", python_scripts);
+		if (python_scripts.empty()) return std::string();
+		// Campaign contains unchecked python scripts.
+		std::string script_names;
+		foreach (config *i, python_scripts)
+		{
+			std::string name = (*i)["name"];
+			name.resize(name.length() - 10);
+			(*i)["name"] = name;
+			script_names += "\n" + name;
 		}
-		return "";
+		return script_names;
 	}
 
 	// Add a file COPYING.txt with the GPL to an uploaded campaign.
 	void add_license(config &data)
 	{
-		config *dir = data.find_child("dir", "name", data["campaign_name"]);
+		config &dir = data.find_child("dir", "name", data["campaign_name"]);
 		// No top-level directory? Hm..
 		if (!dir) return;
 
 		// Don't add if it already exists.
-		if (dir->find_child("file", "name", "COPYING.txt")) return;
-		if (dir->find_child("file", "name", "COPYING")) return;
-		if (dir->find_child("file", "name", "copying.txt")) return;
-		if (dir->find_child("file", "name", "Copying.txt")) return;
-		if (dir->find_child("file", "name", "COPYING.TXT")) return;
+		if (dir.find_child("file", "name", "COPYING.txt")) return;
+		if (dir.find_child("file", "name", "COPYING")) return;
+		if (dir.find_child("file", "name", "copying.txt")) return;
+		if (dir.find_child("file", "name", "Copying.txt")) return;
+		if (dir.find_child("file", "name", "COPYING.TXT")) return;
 
 		// Copy over COPYING.txt
 		std::string contents = read_file("COPYING.txt");
-		config &copying = dir->add_child("file");
-		copying["name"] = "COPYING.txt";
-		copying["contents"] = contents;
-
 		if (contents.empty()) {
 			LOG_CS << "Could not find COPYING.txt, path is \""
 				<< game_config::path << "\"\n";
+			return;
 		}
+		config &copying = dir.add_child("file");
+		copying["name"] = "COPYING.txt";
+		copying["contents"] = contents;
+
 	}
  	void campaign_server::convert_binary_to_gzip()
  	{
  		if (cfg_["converted_to_gzipped_data"] != "yes")
  		{
 			// Convert all addons to gzip
- 			config::child_list camps = campaigns().get_children("campaign");
- 			LOG_CS << "Converting all stored addons to gzip format. Number of addons: " << camps.size() <<"\n";
+			config::const_child_itors camps = campaigns().child_range("campaign");
+			LOG_CS << "Converting all stored addons to gzip format. Number of addons: "
+				<< std::distance(camps.first, camps.second) << '\n';
 
- 			for (config::child_list::iterator itor = camps.begin();
- 					itor != camps.end(); ++itor)
- 			{
- 				LOG_CS << "Converting " << (**itor)["name"] << "\n";
- 				scoped_istream binary_stream = istream_file((**itor)["filename"]);
+			foreach (const config &cm, camps)
+			{
+				LOG_CS << "Converting " << cm["name"] << '\n';
+				std::string filename = cm["filename"];
+				scoped_istream binary_stream = istream_file(filename);
  				config data;
  				if (binary_stream->peek() == 31) //This is gzip file allready
  				{
@@ -312,7 +315,7 @@ namespace {
  				}
  				read_compressed(data, *binary_stream);
 
- 				scoped_ostream gzip_stream = ostream_file((**itor)["filename"]);
+				scoped_ostream gzip_stream = ostream_file(filename);
 				config_writer writer(*gzip_stream, true, compress_level_);
  				writer.write(data);
  			}
@@ -322,20 +325,22 @@ namespace {
 		if (cfg_["encoded"] != "yes")
 		{
 			// Convert all addons to gzip
- 			config::child_list camps = campaigns().get_children("campaign");
- 			LOG_CS << "Encoding all stored campaigns. Number of addons: " << camps.size() <<"\n";
+			config::const_child_itors camps = campaigns().child_range("campaign");
+			LOG_CS << "Encoding all stored addons. Number of addons: "
+				<< std::distance(camps.first, camps.second) << '\n';
+
+			foreach (const config &cm, camps)
+			{
+				LOG_CS << "Encoding " << cm["name"] << '\n';
+				std::string filename = cm["filename"], newfilename = filename + ".new";
 
- 			for (config::child_list::iterator itor = camps.begin();
- 					itor != camps.end(); ++itor)
- 			{
- 				LOG_CS << "Encoding " << (**itor)["name"] << "\n";
 				{
-					scoped_istream in_file = istream_file((**itor)["filename"]);
+					scoped_istream in_file = istream_file(filename);
 					boost::iostreams::filtering_stream<boost::iostreams::input> in_filter;
 					in_filter.push(boost::iostreams::gzip_decompressor());
 					in_filter.push(*in_file);
 
-					scoped_ostream out_file = ostream_file((**itor)["filename"] + ".new");
+					scoped_ostream out_file = ostream_file(newfilename);
 					boost::iostreams::filtering_stream<boost::iostreams::output> out_filter;
 					out_filter.push(boost::iostreams::gzip_compressor(boost::iostreams::gzip_params(compress_level_)));
 					out_filter.push(*out_file);
@@ -352,11 +357,10 @@ namespace {
 						}
 						c = in_filter.get();
 					}
-
-					std::remove((**itor)["filename"].c_str());
-					std::rename(((**itor)["filename"]+".new").c_str(),((**itor)["filename"]).c_str());
-
 				}
+
+				std::remove(filename.c_str());
+				std::rename(newfilename.c_str(), filename.c_str());
 			}
 
 			cfg_["encoded"] = "yes";
@@ -397,161 +401,162 @@ namespace {
 
 				config data;
 				while((sock = network::receive_data(data, 0, &gzipped)) != network::null_connection) {
-					if(const config* req = data.child("request_campaign_list")) {
+					if (const config &req = data.child("request_campaign_list"))
+					{
 						LOG_CS << "sending campaign list to " << network::ip_address(sock) << (gzipped?" using gzip":"");
 						time_t epoch = time(NULL);
 						config campaign_list;
-						(campaign_list)["timestamp"] = lexical_cast<std::string>(epoch);
-						if((const t_string)(*req)["times_relative_to"] != "now") {
+						campaign_list["timestamp"] = lexical_cast<std::string>(epoch);
+						if (req["times_relative_to"] != "now") {
 							epoch = 0;
 						}
-						int before_flag = 0;
+
+						bool before_flag = false;
 						time_t before = epoch;
-						if((const t_string)(*req)["before"] != "") {
-							try {
-								before = before + lexical_cast<time_t>((*req)["before"]);
-								before_flag = 1;
-							}
-							catch(bad_lexical_cast) {
-							}
-						}
-						int after_flag = 0;
+						try {
+							before = before + lexical_cast<time_t>(req["before"]);
+							before_flag = true;
+						} catch(bad_lexical_cast) {}
+
+						bool after_flag = false;
 						time_t after = epoch;
-						if((const t_string)(*req)["after"] != "") {
-							try {
-								after = after + lexical_cast<time_t>((*req)["after"]);
-								after_flag = 1;
-							}
-							catch(bad_lexical_cast) {
-							}
-						}
-						config::child_list cmps = campaigns().get_children("campaign");
-						for(config::child_list::iterator i = cmps.begin(); i != cmps.end(); ++i) {
-							if((const t_string)(*req)["name"] != "" && (*req)["name"] != (**i)["name"]) continue;
-							if(before_flag && ((const t_string)(**i)["timestamp"] == "" || lexical_cast_default<time_t>((**i)["timestamp"],0) >= before)) continue;
-							if(after_flag && ((const t_string)(**i)["timestamp"] == "" || lexical_cast_default<time_t>((**i)["timestamp"],0) <= after)) continue;
-							int found = 1;
-							if((const t_string)(*req)["language"] != "") {
-								found = 0;
-								config::child_list translation = (**i).get_children("translation");
-								for(config::child_list::iterator j = translation.begin(); j != translation.end(); ++j) {
-									if((*req)["language"] == (**j)["language"]) {
-										found = 1;
+						try {
+							after = after + lexical_cast<time_t>(req["after"]);
+							after_flag = true;
+						} catch(bad_lexical_cast) {}
+
+						std::string name = req["name"], lang = req["language"];
+						foreach (const config &i, campaigns().child_range("campaign"))
+						{
+							if (!name.empty() && name != i["name"]) continue;
+							std::string tm = i["timestamp"];
+							if (before_flag && (tm.empty() || lexical_cast_default<time_t>(tm, 0) >= before)) continue;
+							if (after_flag && (tm.empty() || lexical_cast_default<time_t>(tm, 0) <= after)) continue;
+							if (!lang.empty()) {
+								bool found = false;
+								foreach (const config &j, i.child_range("translation")) {
+									if (j["language"] == lang) {
+										found = true;
 										break;
 									}
 								}
+								if (!found) continue;
 							}
-							if(found == 0) continue;
-							campaign_list.add_child("campaign", (**i));
+							campaign_list.add_child("campaign", i);
 						}
-						cmps = campaign_list.get_children("campaign");
-						for(config::child_list::iterator j = cmps.begin(); j != cmps.end(); ++j) {
-							(**j)["passphrase"] = "";
-							(**j)["upload_ip"] = "";
-							(**j)["email"] = "";
+
+						foreach (config &j, campaign_list.child_range("campaign")) {
+							j["passphrase"] = t_string();
+							j["upload_ip"] = t_string();
+							j["email"] = t_string();
 						}
 
 						config response;
 						response.add_child("campaigns",campaign_list);
 						std::cerr << " size: " << (network::send_data(response, sock, gzipped)/1024) << "kb\n";
-					} else if(const config* req = data.child("request_campaign")) {
-						LOG_CS << "sending campaign '" << (*req)["name"] << "' to " << network::ip_address(sock)  << (gzipped?" using gzip":"");
-						config* const campaign = campaigns().find_child("campaign","name",(*req)["name"]);
-						if(campaign == NULL) {
-							network::send_data(construct_error("Add-on '" + (*req)["name"] + "'not found."), sock, gzipped);
+					}
+					else if (const config &req = data.child("request_campaign"))
+					{
+						LOG_CS << "sending campaign '" << req["name"] << "' to " << network::ip_address(sock)  << (gzipped?" using gzip":"");
+						config &campaign = campaigns().find_child("campaign", "name", req["name"]);
+						if (!campaign) {
+							network::send_data(construct_error("Add-on '" + req["name"] + "'not found."), sock, gzipped);
 						} else {
  							if (gzipped)
  							{
-								std::cerr << " size: " << (file_size((*campaign)["filename"])/1024) << "kb\n";
- 								network::send_file((*campaign)["filename"], sock);
+								std::cerr << " size: " << (file_size(campaign["filename"])/1024) << "kb\n";
+								network::send_file(campaign["filename"], sock);
  							} else {
- 								scoped_istream stream = istream_file((*campaign)["filename"]);
+								scoped_istream stream = istream_file(campaign["filename"]);
  								config cfg;
  								read_gz(cfg, *stream);
 								std::cerr << " size: " <<
 									(network::send_data(cfg, sock, false)/1024)
 									<< "kb\n";
 							}
-							const int downloads = lexical_cast_default<int>((*campaign)["downloads"],0)+1;
-							(*campaign)["downloads"] = lexical_cast<std::string>(downloads);
+							const int downloads = lexical_cast_default<int>(campaign["downloads"], 0) + 1;
+							campaign["downloads"] = lexical_cast<std::string>(downloads);
 						}
 
-					} else if(data.child("request_terms") != NULL) {
+					}
+					else if (data.child("request_terms"))
+					{
 						LOG_CS << "sending terms " << network::ip_address(sock) << "\n";
 						network::send_data(construct_message("All add-ons uploaded to this server must be licensed under the terms of the GNU General Public License (GPL). By uploading content to this server, you certify that you have the right to place the content under the conditions of the GPL, and choose to do so."), sock, gzipped);
 						LOG_CS << " Done\n";
-					} else if(config* upload = data.child("upload")) {
-						LOG_CS << "uploading campaign '" << (*upload)["name"] << "' from " << network::ip_address(sock) << ".\n";
-						config* data = upload->child("data");
-						const std::string& name = (*upload)["name"];
+					}
+					else if (config &upload = data.child("upload"))
+					{
+						LOG_CS << "uploading campaign '" << upload["name"] << "' from " << network::ip_address(sock) << ".\n";
+						config &data = upload.child("data");
+						const std::string& name = upload["name"];
 						std::string lc_name(name.size(), ' ');
 						std::transform(name.begin(), name.end(), lc_name.begin(), tolower);
-						const config::child_list& campaign_list = campaigns().get_children("campaign");
-						config* campaign = NULL;
-						for (config::child_list::const_iterator i = campaign_list.begin(); i != campaign_list.end(); ++i) {
-							if (utils::lowercase((**i)["name"]) == lc_name) {
-								campaign = *i;
+						config *campaign = NULL;
+						foreach (config &c, campaigns().child_range("campaign")) {
+							if (utils::lowercase(c["name"]) == lc_name) {
+								campaign = &c;
 								break;
 							}
 						}
-						if(data == NULL) {
+						if (!data) {
 							LOG_CS << "Upload aborted - no add-on data.\n";
 							network::send_data(construct_error("Add-on rejected: No add-on data was supplied."), sock, gzipped);
-						} else if(addon_name_legal((*upload)["name"]) == false) {
+						} else if (!addon_name_legal(upload["name"])) {
 							LOG_CS << "Upload aborted - invalid add-on name.\n";
 							network::send_data(construct_error("Add-on rejected: The name of the add-on is invalid."), sock, gzipped);
-						} else if(((*upload)["title"]).empty()) {
+						} else if (upload["title"].empty()) {
 							LOG_CS << "Upload aborted - no add-on title specified.\n";
 							network::send_data(construct_error("Add-on rejected: You did not specify the title of the add-on in the pbl file!"), sock, gzipped);
-						} else if (get_addon_type((*upload)["type"]) == ADDON_UNKNOWN) {
+						} else if (get_addon_type(upload["type"]) == ADDON_UNKNOWN) {
 							LOG_CS << "Upload aborted - unknown add-on type specified.\n";
 							network::send_data(construct_error("Add-on rejected: You did not specify a known type for the add-on in the pbl file! (See PblWML: wiki.wesnoth.org/PblWML)"), sock, gzipped);
-						} else if(((*upload)["author"]).empty()) {
+						} else if (upload["author"].empty()) {
 							LOG_CS << "Upload aborted - no add-on author specified.\n";
 							network::send_data(construct_error("Add-on rejected: You did not specify the author(s) of the add-on in the pbl file!"), sock, gzipped);
-						} else if(((*upload)["version"]).empty()) {
+						} else if (upload["version"].empty()) {
 							LOG_CS << "Upload aborted - no add-on version specified.\n";
 							network::send_data(construct_error("Add-on rejected: You did not specify the version of the add-on in the pbl file!"), sock, gzipped);
-						} else if(((*upload)["description"]).empty()) {
+						} else if (upload["description"].empty()) {
 							LOG_CS << "Upload aborted - no add-on description specified.\n";
 							network::send_data(construct_error("Add-on rejected: You did not specify a description of the add-on in the pbl file!"), sock, gzipped);
-						} else if(check_names_legal(*data) == false) {
+						} else if (!check_names_legal(data)) {
 							LOG_CS << "Upload aborted - invalid file names in add-on data.\n";
 							network::send_data(construct_error("Add-on rejected: The add-on contains an illegal file or directory name."
 									" File or directory names may not contain any of the following characters: '/ \\ : ~'"), sock, gzipped);
-						} else if(campaign != NULL && (*campaign)["passphrase"] != (*upload)["passphrase"]) {
+						} else if (campaign && (*campaign)["passphrase"] != upload["passphrase"]) {
 							// the user password failed, now test for the master password, in master password
 							// mode the upload behaves different since it's only intended to update translations.
 							// In a later version the translations will be separated from the addon.
 							LOG_CS << "Upload is admin upload.\n";
-							if(campaigns()["master_password"] != ""
-									&& campaigns()["master_password"] == (*upload)["passphrase"]) {
-
+							if (!campaigns()["master_password"].empty()
+							    && campaigns()["master_password"] == upload["passphrase"])
+							{
 								std::string message = "Add-on accepted.";
 
-								if (!version_info((*upload)["version"]).good()) {
+								if (!version_info(upload["version"]).good()) {
 									message += "\n#Note: The version you specified is invalid. This addon will be ignored for automatic update checks.";
 								}
 
 								std::string filename = (*campaign)["filename"];
-								(*data)["title"] = (*campaign)["title"];
-								(*data)["name"] = "";
-								(*data)["campaign_name"] = (*campaign)["name"];
-								(*data)["author"] = (*campaign)["author"];
-								(*data)["description"] = (*campaign)["description"];
-								(*data)["version"] = (*campaign)["version"];
-								(*data)["timestamp"] = (*campaign)["timestamp"];
-								(*data)["icon"] = (*campaign)["icon"];
-								(*data)["translate"] = (*campaign)["translate"];
-								(*data)["type"] = (*campaign)["type"];
+								data["title"] = (*campaign)["title"];
+								data["name"] = "";
+								data["campaign_name"] = (*campaign)["name"];
+								data["author"] = (*campaign)["author"];
+								data["description"] = (*campaign)["description"];
+								data["version"] = (*campaign)["version"];
+								data["timestamp"] = (*campaign)["timestamp"];
+								data["icon"] = (*campaign)["icon"];
+								data["translate"] = (*campaign)["translate"];
+								data["type"] = (*campaign)["type"];
 								(*campaign).clear_children("translation");
-								find_translations(*data, *campaign);
+								find_translations(data, *campaign);
 
-								add_license(*data);
+								add_license(data);
 								{
 									scoped_ostream campaign_file = ostream_file(filename);
 									config_writer writer(*campaign_file, true, compress_level_);
-									writer.write(*data);
+									writer.write(data);
 								}
 //								write_compressed(*campaign_file, *data);
 
@@ -569,7 +574,7 @@ namespace {
 							LOG_CS << "Upload is owner upload.\n";
 							std::string message = "Add-on accepted.";
 
-							if (!version_info((*upload)["version"]).good()) {
+							if (!version_info(upload["version"]).good()) {
 								message += "\n<255,255,0>Note: The version you specified is invalid. This add-on will be ignored for automatic update checks.";
 							}
 
@@ -577,19 +582,19 @@ namespace {
 								campaign = &campaigns().add_child("campaign");
 							}
 
-							(*campaign)["title"] = (*upload)["title"];
-							(*campaign)["name"] = (*upload)["name"];
-							(*campaign)["filename"] = "data/" + (*upload)["name"];
-							(*campaign)["passphrase"] = (*upload)["passphrase"];
-							(*campaign)["author"] = (*upload)["author"];
-							(*campaign)["description"] = (*upload)["description"];
-							(*campaign)["version"] = (*upload)["version"];
-							(*campaign)["icon"] = (*upload)["icon"];
-							(*campaign)["translate"] = (*upload)["translate"];
-							(*campaign)["dependencies"] = (*upload)["dependencies"];
+							(*campaign)["title"] = upload["title"];
+							(*campaign)["name"] = upload["name"];
+							(*campaign)["filename"] = "data/" + upload["name"];
+							(*campaign)["passphrase"] = upload["passphrase"];
+							(*campaign)["author"] = upload["author"];
+							(*campaign)["description"] = upload["description"];
+							(*campaign)["version"] = upload["version"];
+							(*campaign)["icon"] = upload["icon"];
+							(*campaign)["translate"] = upload["translate"];
+							(*campaign)["dependencies"] = upload["dependencies"];
 							(*campaign)["upload_ip"] = network::ip_address(sock);
-							(*campaign)["type"] = (*upload)["type"];
-							(*campaign)["email"] = (*upload)["email"];
+							(*campaign)["type"] = upload["type"];
+							(*campaign)["email"] = upload["email"];
 
 							if((*campaign)["downloads"].empty()) {
 								(*campaign)["downloads"] = "0";
@@ -600,29 +605,29 @@ namespace {
 							(*campaign)["uploads"] = lexical_cast<std::string>(uploads);
 
 							std::string filename = (*campaign)["filename"];
-							(*data)["title"] = (*campaign)["title"];
-							(*data)["name"] = "";
-							(*data)["campaign_name"] = (*campaign)["name"];
-							(*data)["author"] = (*campaign)["author"];
-							(*data)["description"] = (*campaign)["description"];
-							(*data)["version"] = (*campaign)["version"];
-							(*data)["timestamp"] = (*campaign)["timestamp"];
-							(*data)["icon"] = (*campaign)["icon"];
-							(*data)["type"] = (*campaign)["type"];
+							data["title"] = (*campaign)["title"];
+							data["name"] = "";
+							data["campaign_name"] = (*campaign)["name"];
+							data["author"] = (*campaign)["author"];
+							data["description"] = (*campaign)["description"];
+							data["version"] = (*campaign)["version"];
+							data["timestamp"] = (*campaign)["timestamp"];
+							data["icon"] = (*campaign)["icon"];
+							data["type"] = (*campaign)["type"];
 							(*campaign).clear_children("translation");
-							find_translations(*data, *campaign);
+							find_translations(data, *campaign);
 
 							// Campaigns which have python="allowed" are not checked
 							if ((*campaign)["python"] != "allowed") {
-								message += check_python_scripts(*data, filename);
+								message += check_python_scripts(data, filename);
 							}
 
-							add_license(*data);
+							add_license(data);
 
 							{
 								scoped_ostream campaign_file = ostream_file(filename);
 								config_writer writer(*campaign_file, true, compress_level_);
-								writer.write(*data);
+								writer.write(data);
 							}
 //							write_compressed(*campaign_file, *data);
 
@@ -632,78 +637,89 @@ namespace {
 							write(*cfgfile, cfg_);
 							network::send_data(construct_message(message), sock, gzipped);
 
-							fire("hook_post_upload", (*upload)["name"]);
+							fire("hook_post_upload", upload["name"]);
 						}
-					} else if(const config* erase = data.child("delete")) {
-						LOG_CS << "deleting campaign '" << (*erase)["name"] << "' requested from " << network::ip_address(sock) << "\n";
-						config* const campaign = campaigns().find_child("campaign","name",(*erase)["name"]);
-						if(campaign == NULL) {
+					}
+					else if (const config &erase = data.child("delete"))
+					{
+						LOG_CS << "deleting campaign '" << erase["name"] << "' requested from " << network::ip_address(sock) << "\n";
+						const config &campaign = campaigns().find_child("campaign", "name", erase["name"]);
+						if (!campaign) {
 							network::send_data(construct_error("The add-on does not exist."), sock, gzipped);
 							continue;
 						}
 
-						if((*campaign)["passphrase"] != (*erase)["passphrase"]
-								&& (campaigns()["master_password"] == ""
-								|| campaigns()["master_password"] != (*erase)["passphrase"])) {
-
+						if (campaign["passphrase"] != erase["passphrase"]
+								&& (campaigns()["master_password"].empty()
+								|| campaigns()["master_password"] != erase["passphrase"]))
+						{
 							network::send_data(construct_error("The passphrase is incorrect."), sock, gzipped);
 							continue;
 						}
 
 						//erase the campaign
-						write_file((*campaign)["filename"],"");
-						remove((*campaign)["filename"].c_str());
+						write_file(campaign["filename"], std::string());
+						remove(campaign["filename"].c_str());
 
-						const config::child_list& campaigns_list = campaigns().get_children("campaign");
-						const size_t index = std::find(campaigns_list.begin(),campaigns_list.end(),campaign) - campaigns_list.begin();
-						campaigns().remove_child("campaign",index);
+						config::child_itors itors = campaigns().child_range("campaign");
+						for (size_t index = 0; itors.first != itors.second;
+						     ++index, ++itors.first)
+						{
+							if (&campaign == &*itors.first) {
+								campaigns().remove_child("campaign", index);
+								break;
+							}
+						}
 						scoped_ostream cfgfile = ostream_file(file_);
 						write(*cfgfile, cfg_);
 						network::send_data(construct_message("Add-on deleted."), sock, gzipped);
 
-						fire("hook_post_erase", (*erase)["name"]);
-
-					} else if(const config* cpass = data.child("change_passphrase")) {
-						config* campaign = campaigns().find_child("campaign","name",(*cpass)["name"]);
-						if(campaign == NULL) {
+						fire("hook_post_erase", erase["name"]);
+					}
+					else if (const config &cpass = data.child("change_passphrase"))
+					{
+						config &campaign = campaigns().find_child("campaign", "name", cpass["name"]);
+						if (!campaign) {
 							network::send_data(construct_error("No add-on with that name exists."), sock, gzipped);
-						} else if((*campaign)["passphrase"] != (*cpass)["passphrase"]) {
+						} else if (campaign["passphrase"] != cpass["passphrase"]) {
 							network::send_data(construct_error("Your old passphrase was incorrect."), sock, gzipped);
-						} else if((const t_string)(*cpass)["new_passphrase"] == "") {
+						} else if (cpass["new_passphrase"].empty()) {
 							network::send_data(construct_error("No new passphrase was supplied."), sock, gzipped);
 						} else {
-							(*campaign)["passphrase"] = (*cpass)["new_passphrase"];
+							campaign["passphrase"] = cpass["new_passphrase"];
 							scoped_ostream cfgfile = ostream_file(file_);
 							write(*cfgfile, cfg_);
 							network::send_data(construct_message("Passphrase changed."), sock, gzipped);
 						}
-					} else if(const config* cvalidate = data.child("validate_scripts")) {
-						config* campaign = campaigns().find_child("campaign","name",(*cvalidate)["name"]);
-						if(campaign == NULL) {
+					}
+					else if (const config &cvalidate = data.child("validate_scripts"))
+					{
+						config &campaign = campaigns().find_child("campaign", "name", cvalidate["name"]);
+						if (!campaign) {
 							network::send_data(construct_error(
 										"No add-on with that name exists."), sock, gzipped);
-						} else if(campaigns()["master_password"] == "") {
+						} else if (campaigns()["master_password"].empty()) {
 							network::send_data(construct_error(
 										"Sever does not allow scripts."), sock, gzipped);
-						} else if (campaigns()["master_password"] != (*cvalidate)["master_password"]) {
+						} else if (campaigns()["master_password"] != cvalidate["master_password"]) {
 							network::send_data(construct_error(
 										"Password was incorrect."), sock, gzipped);
 						} else {
 							// Read the campaign from disk.
 							config campaign_file;
-							scoped_istream stream = istream_file((*campaign)["filename"]);
+							scoped_istream stream = istream_file(campaign["filename"]);
 							read_gz(campaign_file, *stream);
 							std::string scripts = validate_all_python_scripts(campaign_file);
 							if (!scripts.empty()) {
 								// Write the campaign with changed filenames back to disk
 								{
-									scoped_ostream ostream = ostream_file((*campaign)["filename"]);
+									scoped_ostream ostream = ostream_file(campaign["filename"]);
 									config_writer writer(*ostream, true, compress_level_);
 									writer.write(campaign_file);
 								}
 //								write_compressed(*ostream, campaign_file);
-								(*campaign)["size"] = lexical_cast<std::string>(
-										file_size((*campaign)["filename"]));
+								campaign["size"] = lexical_cast<std::string>(
+										file_size(campaign["filename"]));
 
 								network::send_data(construct_message("The following scripts have been validated: " +
 											scripts), sock, gzipped);
