@@ -1,6 +1,6 @@
-/* $Id: cavegen.cpp 32318 2009-01-26 17:50:26Z jhinrichs $ */
+/* $Id: cavegen.cpp 40856 2010-01-25 15:37:35Z esr $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -20,14 +20,16 @@
 #include "global.hpp"
 
 #include "cavegen.hpp"
+#include "foreach.hpp"
 #include "log.hpp"
 #include "map.hpp"
-#include "pathfind.hpp"
+#include "pathfind/pathfind.hpp"
+#include "util.hpp"
 
+static lg::log_domain log_engine("engine");
+#define LOG_NG LOG_STREAM(info, log_engine)
 
-#define LOG_NG LOG_STREAM(info, engine)
-
-cave_map_generator::cave_map_generator(const config* cfg) :
+cave_map_generator::cave_map_generator(const config &cfg) :
 	wall_(t_translation::CAVE_WALL),
 	clear_(t_translation::CAVE),
 	village_(t_translation::UNDERGROUND_VILLAGE),
@@ -39,33 +41,25 @@ cave_map_generator::cave_map_generator(const config* cfg) :
 	chambers_(),
 	passages_(),
 	res_(),
-	cfg_(cfg),
+	cfg_(cfg ? cfg : config()),
 	width_(50),
 	height_(50),
 	village_density_(0),
 	flipx_(false),
 	flipy_(false)
 {
-	if(cfg_ == NULL) {
-		static const config default_cfg;
-		cfg_ = &default_cfg;
-	}
+	width_ = atoi(cfg_["map_width"].c_str());
+	height_ = atoi(cfg_["map_height"].c_str());
 
-	width_ = atoi((*cfg_)["map_width"].c_str());
-	height_ = atoi((*cfg_)["map_height"].c_str());
-	width_ += 2 * gamemap::default_border;
-	height_ += 2 * gamemap::default_border;
-
-	village_density_ = atoi((*cfg_)["village_density"].c_str());
+	village_density_ = atoi(cfg_["village_density"].c_str());
 
 	const int r = rand()%100;
-	const int chance = atoi((*cfg_)["flipx_chance"].c_str());
+	const int chance = atoi(cfg_["flipx_chance"].c_str());
 
 	flipx_ = r < chance;
 
 	LOG_NG << "flipx: " << r << " < " << chance << " = " << (flipx_ ? "true" : "false") << "\n";
-	flipy_ = (rand()%100) < atoi((*cfg_)["flipy_chance"].c_str());
-
+	flipy_ = (rand()%100) < atoi(cfg_["flipy_chance"].c_str());
 }
 
 std::string cave_map_generator::config_name() const
@@ -99,14 +93,14 @@ std::string cave_map_generator::create_map(const std::vector<std::string>& args)
 
 config cave_map_generator::create_scenario(const std::vector<std::string>& /*args*/)
 {
-	map_ = t_translation::t_map(width_, t_translation::t_list(height_, wall_));
+	map_ = t_translation::t_map(width_ + 2 * gamemap::default_border,
+		t_translation::t_list(height_ + 2 * gamemap::default_border, wall_));
 	chambers_.clear();
 	passages_.clear();
 
 	res_.clear();
-	const config* const settings = cfg_->child("settings");
-	if(settings != NULL) {
-		res_ = *settings;
+	if (const config &settings = cfg_.child("settings")) {
+		res_ = settings;
 	}
 
 	LOG_NG << "creating scenario....\n";
@@ -151,20 +145,20 @@ void cave_map_generator::build_chamber(map_location loc, std::set<map_location>&
 
 void cave_map_generator::generate_chambers()
 {
-	const config::child_list& chambers = cfg_->get_children("chamber");
-	for(config::child_list::const_iterator i = chambers.begin(); i != chambers.end(); ++i) {
+	foreach (const config &ch, cfg_.child_range("chamber"))
+	{
 		// If there is only a chance of the chamber appearing, deal with that here.
-		const std::string& chance = (**i)["chance"];
-		if(chance != "" && (rand()%100) < atoi(chance.c_str())) {
+		const std::string &chance = ch["chance"];
+		if (!chance.empty() && (rand() % 100) < atoi(chance.c_str())) {
 			continue;
 		}
 
-		const std::string& xpos = (**i)["x"];
-		const std::string& ypos = (**i)["y"];
+		const std::string &xpos = ch["x"];
+		const std::string &ypos = ch["y"];
 
 		size_t min_xpos = 0, min_ypos = 0, max_xpos = width_, max_ypos = height_;
 
-		if(xpos != "") {
+		if (!xpos.empty()) {
 			const std::vector<std::string>& items = utils::split(xpos, '-');
 			if(items.empty() == false) {
 				min_xpos = atoi(items.front().c_str()) - 1;
@@ -172,7 +166,7 @@ void cave_map_generator::generate_chambers()
 			}
 		}
 
-		if(ypos != "") {
+		if (!ypos.empty()) {
 			const std::vector<std::string>& items = utils::split(ypos, '-');
 			if(items.empty() == false) {
 				min_ypos = atoi(items.front().c_str()) - 1;
@@ -183,34 +177,26 @@ void cave_map_generator::generate_chambers()
 		const size_t x = translate_x(min_xpos + (rand()%(max_xpos-min_xpos)));
 		const size_t y = translate_y(min_ypos + (rand()%(max_ypos-min_ypos)));
 
-		const std::string& size = (**i)["size"];
-		size_t chamber_size = 3;
-		if(size != "") {
-			chamber_size = atoi(size.c_str());
-		}
-
-		const std::string& jagged = (**i)["jagged"];
-		size_t jagged_edges = 0;
-		if(jagged != "") {
-			jagged_edges = atoi(jagged.c_str());
-		}
+		size_t chamber_size = lexical_cast_default(ch["size"], 3);
+		size_t jagged_edges = lexical_cast_default(ch["jagged"], 0);
 
 		chamber new_chamber;
 		new_chamber.center = map_location(x,y);
 		build_chamber(new_chamber.center,new_chamber.locs,chamber_size,jagged_edges);
 
-		new_chamber.items = (**i).child("items");
+		const config &items = ch.child("items");
+		new_chamber.items = items ? &items : NULL;
 
-		const std::string& id = (**i)["id"];
-		if(id != "") {
+		const std::string &id = ch["id"];
+		if (!id.empty()) {
 			chamber_ids_[id] = chambers_.size();
 		}
 
 		chambers_.push_back(new_chamber);
 
-		const config::child_list& passages = (**i).get_children("passage");
-		for(config::child_list::const_iterator p = passages.begin(); p != passages.end(); ++p) {
-			const std::string& dst = (**p)["destination"];
+		foreach (const config &p, ch.child_range("passage"))
+		{
+			const std::string &dst = p["destination"];
 
 			// Find the destination of this passage
 			const std::map<std::string,size_t>::const_iterator itor = chamber_ids_.find(dst);
@@ -219,7 +205,7 @@ void cave_map_generator::generate_chambers()
 
 			assert(itor->second < chambers_.size());
 
-			passages_.push_back(passage(new_chamber.center,chambers_[itor->second].center,**p));
+			passages_.push_back(passage(new_chamber.center, chambers_[itor->second].center, p));
 		}
 	}
 }
@@ -230,26 +216,17 @@ void cave_map_generator::place_chamber(const chamber& c)
 		set_terrain(*i,clear_);
 	}
 
-	if(c.items != NULL) {
-		place_items(c,c.items->ordered_begin(),c.items->ordered_end());
-	}
-}
-
-void cave_map_generator::place_items(const chamber& c, config::all_children_iterator i1, config::all_children_iterator i2)
-{
-	if(c.locs.empty()) {
-		return;
-	}
+	if (c.items == NULL || c.locs.empty()) return;
 
 	size_t index = 0;
-	while(i1 != i2) {
-		const std::string& key = *(*i1).first;
-		config cfg = *(*i1).second;
-		config* const filter = cfg.child("filter");
-		config* const object = cfg.child("object");
+	foreach (const config::any_child &it, c.items->all_children_range())
+	{
+		config cfg = it.cfg;
+		config &filter = cfg.child("filter");
 		config* object_filter = NULL;
-		if(object != NULL) {
-			object_filter = object->child("filter");
+		if (config &object = cfg.child("object")) {
+			if (config &of = object.child("filter"))
+				object_filter = &of;
 		}
 
 		if(!utils::string_bool(cfg["same_location_as_previous"])) {
@@ -260,22 +237,20 @@ void cave_map_generator::place_items(const chamber& c, config::all_children_iter
 		std::set<map_location>::const_iterator loc = c.locs.begin();
 		std::advance(loc,index);
 
-		char xbuf[50];
-		snprintf(xbuf,sizeof(xbuf),"%d",loc->x+1);
-		cfg.values["x"] = xbuf;
-		if(filter != NULL) {
-			(*filter)["x"] = xbuf;
+		std::string xbuf = str_cast(loc->x + 1);
+		cfg["x"] = xbuf;
+		if (filter) {
+			filter["x"] = xbuf;
 		}
 
 		if(object_filter != NULL) {
 			(*object_filter)["x"] = xbuf;
 		}
 
-		char ybuf[50];
-		snprintf(ybuf,sizeof(ybuf),"%d",loc->y+1);
-		cfg.values["y"] = ybuf;
-		if(filter != NULL) {
-			(*filter)["y"] = ybuf;
+		std::string ybuf = str_cast(loc->y + 1);
+		cfg["y"] = ybuf;
+		if (filter) {
+			filter["y"] = ybuf;
 		}
 
 		if(object_filter != NULL) {
@@ -283,11 +258,11 @@ void cave_map_generator::place_items(const chamber& c, config::all_children_iter
 		}
 
 		// If this is a side, place a castle for the side
-		if(key == "side" && !utils::string_bool(cfg["no_castle"])) {
+		if (it.key == "side" && !utils::string_bool(cfg["no_castle"])) {
 			place_castle(cfg["side"],*loc);
 		}
 
-		res_.add_child(key,cfg);
+		res_.add_child(it.key, cfg);
 
 		if(!loc_var.empty()) {
 			config &temp = res_.add_child("event");
@@ -299,19 +274,17 @@ void cave_map_generator::place_items(const chamber& c, config::all_children_iter
 			ycfg["name"] = loc_var + "_y";
 			ycfg["value"] = ybuf;
 		}
-
-		++i1;
 	}
 }
 
-struct passage_path_calculator : cost_calculator
+struct passage_path_calculator : pathfind::cost_calculator
 {
 	passage_path_calculator(const t_translation::t_map& mapdata,
 	t_translation::t_terrain wall, double laziness, size_t windiness):
 		map_(mapdata), wall_(wall), laziness_(laziness), windiness_(windiness)
 	{}
 
-	virtual double cost(const map_location& src,const map_location& loc, const double so_far) const;
+	virtual double cost(const map_location& loc, const double so_far) const;
 private:
 	const t_translation::t_map& map_;
 	t_translation::t_terrain wall_;
@@ -319,13 +292,10 @@ private:
 	size_t windiness_;
 };
 
-double passage_path_calculator::cost(const map_location& /*src*/,const map_location& loc, const double) const
+double passage_path_calculator::cost(const map_location& loc, const double) const
 {
-	assert(loc.x >= 0 && loc.y >= 0 && size_t(loc.x) < map_.size() &&
-	        !map_.empty() && size_t(loc.y) < map_.front().size());
-
 	double res = 1.0;
-	if(map_[loc.x][loc.y] == wall_) {
+	if (map_[loc.x + gamemap::default_border][loc.y + gamemap::default_border] == wall_) {
 		res = laziness_;
 	}
 
@@ -349,7 +319,7 @@ void cave_map_generator::place_passage(const passage& p)
 
 	passage_path_calculator calc(map_,wall_,laziness,windiness);
 
-	const paths::route rt = a_star_search(p.src, p.dst, 10000.0, &calc, width_, height_);
+	pathfind::plain_route rt = a_star_search(p.src, p.dst, 10000.0, &calc, width_, height_);
 
 	const size_t width = std::max<size_t>(1,atoi(p.cfg["width"].c_str()));
 
@@ -366,14 +336,12 @@ void cave_map_generator::place_passage(const passage& p)
 
 void cave_map_generator::set_terrain(map_location loc, t_translation::t_terrain t)
 {
-	if(on_board(loc)) {
-		if(t == clear_ &&
-                (rand() % 1000) < static_cast<long>(village_density_)) {
-
+	if (on_board(loc)) {
+		if (t == clear_ && (rand() % 1000) < village_density_) {
 			t = village_;
 		}
 
-		t_translation::t_terrain& c = map_[loc.x][loc.y];
+		t_translation::t_terrain& c = map_[loc.x + gamemap::default_border][loc.y + gamemap::default_border];
 		if(c == clear_ || c == wall_ || c == village_) {
 			c = t;
 		}
@@ -386,7 +354,8 @@ void cave_map_generator::place_castle(const std::string& side, map_location loc)
 	if(starting_position != -1) {
 		set_terrain(loc, keep_);
 
-		const struct t_translation::coordinate coord = {loc.x, loc.y};
+		t_translation::coordinate coord =
+			{ loc.x + gamemap::default_border, loc.y + gamemap::default_border };
 		starting_positions_[starting_position] = coord;
 	}
 
