@@ -1,7 +1,7 @@
-/* $Id: config.cpp 33687 2009-03-15 16:50:42Z mordante $ */
+/* $Id: config.cpp 42009 2010-04-05 14:16:17Z silene $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2010 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -21,12 +21,36 @@
 #include "global.hpp"
 
 #include "config.hpp"
+#include "foreach.hpp"
 #include "log.hpp"
+#include "serialization/string_utils.hpp"
+#include "util.hpp"
 
 #include <cstring>
 #include <deque>
 
-#define ERR_CF LOG_STREAM(err, config)
+static lg::log_domain log_config("config");
+#define ERR_CF LOG_STREAM(err, log_config)
+
+config config::invalid;
+
+const char* config::diff_track_attribute = "__diff_track";
+
+int *volatile null_pointer = 0;
+
+void config::check_valid() const
+{
+	if (*this) return;
+	*null_pointer = 0;
+	throw error("Mandatory WML child missing yet untested for. Please report.");
+}
+
+void config::check_valid(const config &cfg) const
+{
+	if (*this && cfg) return;
+	*null_pointer = 0;
+	throw error("Mandatory WML child missing yet untested for. Please report.");
+}
 
 config::config() : values(), children(), ordered_children()
 {
@@ -34,6 +58,7 @@ config::config() : values(), children(), ordered_children()
 
 config::config(const config& cfg) : values(), children(), ordered_children()
 {
+	cfg.check_valid();
 	append(cfg);
 }
 
@@ -53,18 +78,31 @@ config& config::operator=(const config& cfg)
 		return *this;
 	}
 
+	check_valid(cfg);
+
 	clear();
-
 	append(cfg);
-
 	return *this;
 }
 
+bool config::has_attribute(const std::string &key) const
+{
+	check_valid();
+	return values.find(key) != values.end();
+}
+
+void config::remove_attribute(const std::string &key)
+{
+	check_valid();
+	values.erase(key);
+}
+
 void config::append(const config& cfg)
 {
-	for(all_children_iterator i = cfg.ordered_begin(); i != cfg.ordered_end(); ++i) {
-		const std::pair<const std::string*,const config*>& value = *i;
-		add_child(*value.first,*value.second);
+	check_valid(cfg);
+
+	foreach (const any_child &value, cfg.all_children_range()) {
+		add_child(value.key, value.cfg);
 	}
 
 	for(string_map::const_iterator j = cfg.values.begin(); j != cfg.values.end(); ++j) {
@@ -74,6 +112,8 @@ void config::append(const config& cfg)
 
 void config::merge_children(const std::string& key)
 {
+	check_valid();
+
 	config merged_children;
 	const child_list& children = get_children(key);
 	if(children.size() < 2) {
@@ -88,30 +128,72 @@ void config::merge_children(const std::string& key)
 	add_child(key,merged_children);
 }
 
-config::child_itors config::child_range(const std::string& key)
+void config::merge_children_by_attribute(const std::string& key, const std::string& attribute)
 {
+	check_valid();
+
+	std::map<std::string,config> merged_children_map;
+	const child_list& children = get_children(key);
+	if(children.size() < 2) {
+		return;
+	}
+
+	for(child_list::const_iterator i = children.begin(); i != children.end(); ++i) {
+		const std::string& value = (**i)[attribute];
+		std::map<std::string,config>::iterator m = merged_children_map.find(value);
+		if ( m!=merged_children_map.end() ) {
+			m->second.append(**i);
+		} else {
+			merged_children_map.insert(make_pair(value,**i));
+		}
+	}
+
+	clear_children(key);
+	typedef std::map<std::string,config> config_map;
+	foreach (const config_map::value_type &i, merged_children_map) {
+		add_child(key,i.second);
+	}
+}
+
+config::child_itors_bak config::child_range_bak(const std::string& key)
+{
+	check_valid();
+
 	child_map::iterator i = children.find(key);
 	if(i != children.end()) {
-		return child_itors(i->second.begin(),i->second.end());
+		return child_itors_bak(i->second.begin(),i->second.end());
 	} else {
 		static std::vector<config*> dummy;
-		return child_itors(dummy.begin(),dummy.end());
+		return child_itors_bak(dummy.begin(),dummy.end());
 	}
 }
 
+config::child_itors config::child_range(const std::string& key)
+{
+	check_valid();
+
+	child_map::iterator i = children.find(key);
+	static child_list dummy;
+	child_list *p = &dummy;
+	if (i != children.end()) p = &i->second;
+	return child_itors(child_iterator(p->begin()), child_iterator(p->end()));
+}
+
 config::const_child_itors config::child_range(const std::string& key) const
 {
+	check_valid();
+
 	child_map::const_iterator i = children.find(key);
-	if(i != children.end()) {
-		return const_child_itors(i->second.begin(),i->second.end());
-	} else {
-		static const std::vector<config*> dummy;
-		return const_child_itors(dummy.begin(),dummy.end());
-	}
+	static child_list dummy;
+	const child_list *p = &dummy;
+	if (i != children.end()) p = &i->second;
+	return const_child_itors(const_child_iterator(p->begin()), const_child_iterator(p->end()));
 }
 
 size_t config::child_count(const std::string& key) const
 {
+	check_valid();
+
 	child_map::const_iterator i = children.find(key);
 	if(i != children.end()) {
 		return i->second.size();
@@ -121,6 +203,8 @@ size_t config::child_count(const std::string& key) const
 
 const config::child_list& config::get_children(const std::string& key) const
 {
+	check_valid();
+
 	const child_map::const_iterator i = children.find(key);
 	if(i != children.end()) {
 		return i->second;
@@ -130,30 +214,54 @@ const config::child_list& config::get_children(const std::string& key) const
 	}
 }
 
-const config::child_map& config::all_children() const { return children; }
-
-config* config::child(const std::string& key)
+config &config::child(const std::string& key)
 {
+	check_valid();
+
 	const child_map::const_iterator i = children.find(key);
 	if(i != children.end() && i->second.empty() == false) {
-		return i->second.front();
+		return *i->second.front();
 	} else {
-		return NULL;
+		return invalid;
 	}
 }
 
-const config* config::child(const std::string& key) const
+const config &config::child(const std::string& key) const
 {
+	check_valid();
+
 	const child_map::const_iterator i = children.find(key);
 	if(i != children.end() && i->second.empty() == false) {
-		return i->second.front();
+		return *i->second.front();
 	} else {
-		return NULL;
+		return invalid;
 	}
 }
 
+config config::child_or_empty(const std::string& key) const
+{
+	check_valid();
+
+	child_map::const_iterator i = children.find(key);
+	if (i != children.end() && !i->second.empty())
+		return *i->second.front();
+
+	return config();
+}
+
+config &config::child_or_add(const std::string &key)
+{
+	child_map::const_iterator i = children.find(key);
+	if (i != children.end() && !i->second.empty())
+		return *i->second.front();
+
+	return add_child(key);
+}
+
 config& config::add_child(const std::string& key)
 {
+	check_valid();
+
 	child_list& v = children[key];
 	v.push_back(new config());
 	ordered_children.push_back(child_pos(children.find(key),v.size()-1));
@@ -162,6 +270,8 @@ config& config::add_child(const std::string& key)
 
 config& config::add_child(const std::string& key, const config& val)
 {
+	check_valid(val);
+
 	child_list& v = children[key];
 	v.push_back(new config(val));
 	ordered_children.push_back(child_pos(children.find(key),v.size()-1));
@@ -170,6 +280,8 @@ config& config::add_child(const std::string& key, const config& val)
 
 config& config::add_child_at(const std::string& key, const config& val, size_t index)
 {
+	check_valid(val);
+
 	child_list& v = children[key];
 	if(index > v.size()) {
 		throw error("illegal index to add child at");
@@ -212,11 +324,13 @@ private:
 
 void config::clear_children(const std::string& key)
 {
+	check_valid();
+
 	ordered_children.erase(std::remove_if(ordered_children.begin(),ordered_children.end(),remove_ordered(key)),ordered_children.end());
 
 	child_map::iterator i = children.find(key);
 	if (i != children.end()) {
-		for (child_iterator c = i->second.begin(); c != i->second.end(); c++) {
+		for (child_list::iterator c = i->second.begin(); c != i->second.end(); ++c) {
 			delete *c;
 		}
 		children.erase(i);
@@ -225,20 +339,23 @@ void config::clear_children(const std::string& key)
 
 void config::recursive_clear_value(const std::string& key)
 {
+	check_valid();
+
 	values.erase(key);
 
-	for (all_children_iterator it = ordered_begin(); it != ordered_end(); ++it)
-	{
-		const_cast<config*>(it->second)->recursive_clear_value(key);
+	foreach (const any_child &value, all_children_range()) {
+		const_cast<config *>(&value.cfg)->recursive_clear_value(key);
 	}
 }
 
 config::all_children_iterator config::erase(const config::all_children_iterator& i)
 {
+	check_valid();
+
 	config* found_config = NULL;
 	std::vector<child_pos>::iterator erase_pos, j, j_end = ordered_children.end();
 	for(j = ordered_children.begin(); j != j_end; ++j) {
-		if(i.get_key() == j->pos->first) {
+		if (i->key == j->pos->first) {
 			if(i.get_index() == j->index) {
 				erase_pos = j;
 				found_config = *(j->pos->second.begin() + j->index);
@@ -248,12 +365,8 @@ config::all_children_iterator config::erase(const config::all_children_iterator&
 			}
 		}
 	}
-	child_list& vec = children[i.get_key()];
-	if(!found_config || erase_pos->index >= vec.size()) {
-		ERR_CF << "Error: attempting to delete non-existing child: "
-			<< i.get_key() << "[" << i.get_index() << "]\n";
-		return ordered_end();
-	}
+	child_list& vec = children[i->key];
+	assert(found_config && erase_pos->index < vec.size());
 	delete found_config;
 	vec.erase(vec.begin()+i.get_index());
 	return all_children_iterator(ordered_children.erase(erase_pos));
@@ -261,6 +374,8 @@ config::all_children_iterator config::erase(const config::all_children_iterator&
 
 void config::remove_child(const std::string& key, size_t index)
 {
+	check_valid();
+
 	// Remove from the ordering
 	const child_pos pos(children.find(key),index);
 	ordered_children.erase(std::remove(ordered_children.begin(),ordered_children.end(),pos),ordered_children.end());
@@ -287,6 +402,8 @@ void config::remove_child(const std::string& key, size_t index)
 
 t_string& config::operator[](const std::string& key)
 {
+	check_valid();
+
 	return values[key];
 }
 
@@ -297,6 +414,8 @@ const t_string& config::operator[](const std::string& key) const
 
 const t_string& config::get_attribute(const std::string& key) const
 {
+	check_valid();
+
 	const string_map::const_iterator i = values.find(key);
 	if(i != values.end()) {
 		return i->second;
@@ -306,6 +425,26 @@ const t_string& config::get_attribute(const std::string& key) const
 	}
 }
 
+void config::merge_attributes(const config &cfg)
+{
+	check_valid(cfg);
+
+	assert(this != &cfg);
+	for (string_map::const_iterator i = cfg.values.begin(),
+	     i_end = cfg.values.end(); i != i_end; ++i)
+	{
+		values[i->first] = i->second;
+	}
+}
+
+config::const_attr_itors config::attribute_range() const
+{
+	check_valid();
+
+	return const_attr_itors(const_attribute_iterator(values.begin()),
+	                        const_attribute_iterator(values.end()));
+}
+
 namespace {
 
 struct config_has_value {
@@ -321,41 +460,49 @@ private:
 
 } // end namespace
 
-config* config::find_child(const std::string& key,
+config &config::find_child(const std::string& key,
                            const std::string& name,
                            const t_string& value)
 {
+	check_valid();
+
 	const child_map::iterator i = children.find(key);
 	if(i == children.end())
-		return NULL;
+		return invalid;
 
 	const child_list::iterator j = std::find_if(i->second.begin(),
 	                                            i->second.end(),
 	                                            config_has_value(name,value));
 	if(j != i->second.end())
-		return *j;
+		return **j;
 	else
-		return NULL;
+		return invalid;
 }
 
-const config* config::find_child(const std::string& key,
+const config &config::find_child(const std::string& key,
                                  const std::string& name,
                                  const t_string& value) const
 {
-	const child_map::const_iterator i = children.find(key);
-	if(i == children.end())
-		return NULL;
-
-	const child_list::const_iterator j = std::find_if(
-	                                            i->second.begin(),
-	                                            i->second.end(),
-	                                            config_has_value(name,value));
-	if(j != i->second.end())
-		return *j;
-	else
-		return NULL;
+	return const_cast<config *>(this)->find_child(key, name, value);
 }
 
+const config& config::find_child_recursive(const std::string& key,
+                                 const std::string& name,
+                                 const t_string& value) const
+{
+	const config& res = this->find_child(key, name, value);
+	if (res)
+		return res;
+
+	foreach (const any_child &child, all_children_range()) {
+		const config& res2 = child.cfg.find_child_recursive(key, name, value);
+
+		if (res2)
+			return res2;
+	}
+
+	return invalid;
+}
 namespace {
 	/**
 	 * Helper struct for iterative config clearing.
@@ -377,6 +524,8 @@ namespace {
 
 void config::clear()
 {
+	// No validity check for this function.
+
 	if (!children.empty()) {
 		//start with this node, the first entry in the child map,
 		//zeroeth element in that entry
@@ -425,43 +574,14 @@ void config::clear()
 
 bool config::empty() const
 {
+	check_valid();
+
 	return children.empty() && values.empty();
 }
 
-config::all_children_iterator::all_children_iterator(config::all_children_iterator::Itor i) : i_(i)
-{}
-
-config::all_children_iterator& config::all_children_iterator::operator++()
+config::any_child config::all_children_iterator::operator*() const
 {
-	++i_;
-	return *this;
-}
-
-config::all_children_iterator config::all_children_iterator::operator++(int)
-{
-	config::all_children_iterator i = *this;
-	++i_;
-	return i;
-}
-
-std::pair<const std::string*,const config*> config::all_children_iterator::operator*() const
-{
-	return std::pair<const std::string*,const config*>(&(i_->pos->first),i_->pos->second[i_->index]);
-}
-
-config::all_children_iterator::pointer config::all_children_iterator::operator->() const
-{
-	return pointer(new std::pair<const std::string*,const config*>(&(i_->pos->first),i_->pos->second[i_->index]));
-}
-
-const std::string& config::all_children_iterator::get_key() const
-{
-	return i_->pos->first;
-}
-
-const config& config::all_children_iterator::get_child() const
-{
-	return *(i_->pos->second[i_->index]);
+	return any_child(&i_->pos->first, i_->pos->second[i_->index]);
 }
 
 size_t config::all_children_iterator::get_index() const
@@ -469,16 +589,6 @@ size_t config::all_children_iterator::get_index() const
 	return i_->index;
 }
 
-bool config::all_children_iterator::operator==(all_children_iterator i) const
-{
-	return i_ == i.i_;
-}
-
-bool config::all_children_iterator::operator!=(all_children_iterator i) const
-{
-	return i_ != i.i_;
-}
-
 config::all_children_iterator config::ordered_begin() const
 {
 	return all_children_iterator(ordered_children.begin());
@@ -489,8 +599,17 @@ config::all_children_iterator config::ordered_end() const
 	return all_children_iterator(ordered_children.end());
 }
 
+config::all_children_itors config::all_children_range() const
+{
+	return all_children_itors(
+		all_children_iterator(ordered_children.begin()),
+		all_children_iterator(ordered_children.end()));
+}
+
 config config::get_diff(const config& c) const
 {
+	check_valid(c);
+
 	config res;
 	get_diff(c, res);
 	return res;
@@ -498,6 +617,9 @@ config config::get_diff(const config& c) const
 
 void config::get_diff(const config& c, config& res) const
 {
+	check_valid(c);
+	check_valid(res);
+
 	config* inserts = NULL;
 
 	string_map::const_iterator i;
@@ -600,64 +722,105 @@ void config::get_diff(const config& c, config& res) const
 	}
 }
 
-void config::apply_diff(const config& diff)
+void config::apply_diff(const config& diff, bool track /* = false */)
 {
-	const config* const inserts = diff.child("insert");
-	if(inserts != NULL) {
-		for(string_map::const_iterator i = inserts->values.begin(); i != inserts->values.end(); ++i) {
-			values[i->first] = i->second;
+	check_valid(diff);
+
+	if (track) values[diff_track_attribute] = "modified";
+
+	if (const config &inserts = diff.child("insert")) {
+		foreach (const attribute &v, inserts.attribute_range()) {
+			values[v.first] = v.second;
 		}
 	}
 
-	const config* const deletes = diff.child("delete");
-	if(deletes != NULL) {
-		for(string_map::const_iterator i = deletes->values.begin(); i != deletes->values.end(); ++i) {
-			values.erase(i->first);
+	if (const config &deletes = diff.child("delete")) {
+		foreach (const attribute &v, deletes.attribute_range()) {
+			values.erase(v.first);
 		}
 	}
 
-	const child_list& child_changes = diff.get_children("change_child");
-	child_list::const_iterator i;
-	for(i = child_changes.begin(); i != child_changes.end(); ++i) {
-		const size_t index = lexical_cast<size_t>((**i)["index"].str());
-		for(all_children_iterator j = (*i)->ordered_begin(); j != (*i)->ordered_end(); ++j) {
-			const std::pair<const std::string*,const config*> item = *j;
-
-			if(item.first->empty()) {
+	foreach (const config &i, diff.child_range("change_child"))
+	{
+		const size_t index = lexical_cast<size_t>(i["index"].str());
+		foreach (const any_child &item, i.all_children_range())
+		{
+			if (item.key.empty()) {
 				continue;
 			}
 
-			const child_map::iterator itor = children.find(*item.first);
+			const child_map::iterator itor = children.find(item.key);
 			if(itor == children.end() || index >= itor->second.size()) {
-				throw error("error in diff: could not find element '" + *item.first + "'");
+				throw error("error in diff: could not find element '" + item.key + "'");
 			}
 
-			itor->second[index]->apply_diff(*item.second);
+			itor->second[index]->apply_diff(item.cfg, track);
 		}
 	}
 
-	const child_list& child_inserts = diff.get_children("insert_child");
-	for(i = child_inserts.begin(); i != child_inserts.end(); ++i) {
-		const size_t index = lexical_cast<size_t>((**i)["index"].str());
-		for(all_children_iterator j = (*i)->ordered_begin(); j != (*i)->ordered_end(); ++j) {
-			const std::pair<const std::string*,const config*> item = *j;
-			add_child_at(*item.first,*item.second,index);
+	foreach (const config &i, diff.child_range("insert_child"))
+	{
+		const size_t index = lexical_cast<size_t>(i["index"].str());
+		foreach (const any_child &item, i.all_children_range()) {
+			config& inserted = add_child_at(item.key, item.cfg, index);
+			if (track) inserted[diff_track_attribute] = "new";
 		}
 	}
 
-	const child_list& child_deletes = diff.get_children("delete_child");
-	for(i = child_deletes.begin(); i != child_deletes.end(); ++i) {
-		const size_t index = lexical_cast<size_t>((**i)["index"].str());
-		for(all_children_iterator j = (*i)->ordered_begin(); j != (*i)->ordered_end(); ++j) {
-			const std::pair<const std::string*,const config*> item = *j;
-
-			remove_child(*item.first,index);
+	foreach (const config &i, diff.child_range("delete_child"))
+	{
+		const size_t index = lexical_cast<size_t>(i["index"].str());
+		foreach (const any_child &item, i.all_children_range()) {
+			if (!track) {
+				remove_child(item.key, index);
+			} else {
+				const child_map::iterator itor = children.find(item.key);
+				if(itor == children.end() || index >= itor->second.size()) {
+					throw error("error in diff: could not find element '" + item.key + "'");
+				}
+				itor->second[index]->values[diff_track_attribute] = "deleted";
+			}
 		}
 	}
 }
 
+void config::clear_diff_track(const config& diff)
+{
+	remove_attribute(diff_track_attribute);
+	foreach (const config &i, diff.child_range("delete_child"))
+	{
+		const size_t index = lexical_cast<size_t>(i["index"].str());
+		foreach (const any_child &item, i.all_children_range()) {
+			remove_child(item.key, index);
+		}
+	}
+
+	foreach (const config &i, diff.child_range("change_child"))
+	{
+		const size_t index = lexical_cast<size_t>(i["index"].str());
+		foreach (const any_child &item, i.all_children_range())
+		{
+			if (item.key.empty()) {
+				continue;
+			}
+
+			const child_map::iterator itor = children.find(item.key);
+			if(itor == children.end() || index >= itor->second.size()) {
+				throw error("error in diff: could not find element '" + item.key + "'");
+			}
+
+			itor->second[index]->clear_diff_track(item.cfg);
+		}
+	}
+	foreach (const any_child &value, all_children_range()) {
+		const_cast<config *>(&value.cfg)->remove_attribute(diff_track_attribute);
+	}
+}
+
 void config::merge_with(const config& c)
 {
+	check_valid(c);
+
 	std::map<std::string, unsigned> visitations;
 
 	// Merge attributes first
@@ -689,28 +852,10 @@ void config::merge_with(const config& c)
 	}
 }
 
-void config::merge_and_keep(const config& c)
-{
-	// Merge not existing attributes first
-	string_map::const_iterator attrib_it, attrib_end = c.values.end();
-	for(attrib_it = c.values.begin(); attrib_it != attrib_end; ++attrib_it) {
-	    if (values[attrib_it->first] == "")
-            values[attrib_it->first] = attrib_it->second;
-	}
-
-	// Now merge unshared tags
-	for(child_map::const_iterator i = c.children.begin(); i != c.children.end(); ++i) {
-		const std::string& tag = i->first;
-		child_map::const_iterator j = children.find(tag);
-		if (j == children.end()) {
-		    for (size_t count=0; count < i->second.size(); count++)
-                add_child(tag, *i->second[count]);
-		}
-	}
-}
-
 bool config::matches(const config &filter) const
 {
+	check_valid(filter);
+
 	// First match values. all values should match.
 	for(string_map::const_iterator j = filter.values.begin(); j != filter.values.end(); ++j) {
 		if(!this->values.count(j->first)) return false;
@@ -732,19 +877,20 @@ bool config::matches(const config &filter) const
 	}
 
 	// Now, match the kids
-	for(all_children_iterator i2 = filter.ordered_begin(); i2 != filter.ordered_end(); ++i2) {
-		if(*(*i2).first == "not") continue;
-		child_list interesting_children = get_children(*(*i2).first);
+	foreach (const any_child &i, filter.all_children_range())
+	{
+		if (i.key == "not") continue;
 		bool found = false;
-		for(child_list::iterator j2 = interesting_children.begin(); j2 != interesting_children.end(); ++j2) {
-			if((*j2)->matches(*(*i2).second)) {
+		foreach (const config &j, child_range(i.key)) {
+			if (j.matches(i.cfg)) {
 				found = true;
+				break;
 			}
 		}
 		if(!found) return false;
 	}
 	child_list negative_children = filter.get_children("not");
-	for(child_list::iterator j3 = negative_children.begin() ; j3 != negative_children.end() ; j3++) {
+	for(child_list::iterator j3 = negative_children.begin() ; j3 != negative_children.end() ; ++j3) {
 		if(matches(**j3)) return false;
 	}
 	return true;
@@ -767,40 +913,30 @@ void config::prune() {
 	}
 }
 
-void config::reset_translation() const
-{
-	for(string_map::const_iterator val = values.begin(); val != values.end(); ++val) {
-		val->second.reset_translation();
-	}
-
-	for(child_map::const_iterator list = children.begin(); list != children.end(); ++list) {
-		for(child_list::const_iterator child = list->second.begin();
-				child != list->second.end(); ++child) {
-			(*child)->reset_translation();
-		}
-	}
-}
-
 std::string config::debug() const
 {
+	check_valid();
+
 	std::ostringstream outstream;
 	outstream << *this;
 	return outstream.str();
 }
 
-std::ostream& operator << (std::ostream& outstream, const config& cfg) {
+std::ostream& operator << (std::ostream& outstream, const config& cfg)
+{
 	static int i = 0;
 	i++;
-	for(string_map::const_iterator val = cfg.values.begin(); val != cfg.values.end(); ++val) {
+	foreach (const config::attribute &val, cfg.attribute_range()) {
 		for (int j = 0; j < i-1; j++){ outstream << char(9); }
-		outstream << val->first << " = " << val->second << "\n";
+		outstream << val.first << " = " << val.second << '\n';
 	}
-	for(config::all_children_iterator list = cfg.ordered_begin(); list != cfg.ordered_end(); ++list) {
-		{ for (int j = 0; j < i-1; j++){ outstream << char(9); } }
-		outstream << "[" << *(*list).first << "]\n";
-		outstream << *(*list).second;
-		{ for (int j = 0; j < i-1; j++){ outstream << char(9); } }
-		outstream << "[/" << *(*list).first << "]\n";
+	foreach (const config::any_child &child, cfg.all_children_range())
+	{
+		for (int j = 0; j < i - 1; ++j) outstream << char(9);
+		outstream << "[" << child.key << "]\n";
+		outstream << child.cfg;
+		for (int j = 0; j < i - 1; ++j) outstream << char(9);
+		outstream << "[/" << child.key << "]\n";
 	}
 	i--;
     return outstream;
@@ -808,6 +944,8 @@ std::ostream& operator << (std::ostream& outstream, const config& cfg) {
 
 std::string config::hash() const
 {
+	check_valid();
+
 	static const unsigned int hash_length = 128;
 	static const char hash_string[] =
 		"+-,.<>0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
@@ -869,6 +1007,8 @@ std::string config::hash() const
 
 void config::swap(config& cfg)
 {
+	check_valid(cfg);
+
 	values.swap(cfg.values);
 	children.swap(cfg.children);
 	ordered_children.swap(cfg.ordered_children);
@@ -876,28 +1016,19 @@ void config::swap(config& cfg)
 
 bool operator==(const config& a, const config& b)
 {
+	a.check_valid(b);
+
 	if (a.values != b.values)
 		return false;
 
-	config::all_children_iterator x = a.ordered_begin(), y = b.ordered_begin();
-	while(x != a.ordered_end() && y != b.ordered_end()) {
-		const std::pair<const std::string*,const config*> val1 = *x;
-		const std::pair<const std::string*,const config*> val2 = *y;
-
-		if(*val1.first != *val2.first || *val1.second != *val2.second) {
+	config::all_children_itors x = a.all_children_range(), y = b.all_children_range();
+	for (; x.first != x.second && y.first != y.second; ++x.first, ++y.first) {
+		if (x.first->key != y.first->key || x.first->cfg != y.first->cfg) {
 			return false;
 		}
-
-		++x;
-		++y;
 	}
 
-	return x == a.ordered_end() && y == b.ordered_end();
-}
-
-bool operator!=(const config& a, const config& b)
-{
-	return !operator==(a,b);
+	return x.first == x.second && y.first == y.second;
 }
 
 //#define TEST_CONFIG
