@@ -1,6 +1,6 @@
-/* $Id: construct_dialog.cpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: construct_dialog.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2006 - 2009 by Patrick Parker <patrick_x99@hotmail.com>
+   Copyright (C) 2006 - 2010 by Patrick Parker <patrick_x99@hotmail.com>
    wesnoth widget Copyright (C) 2003-5 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -19,17 +19,18 @@
 
 #include "construct_dialog.hpp"
 #include "display.hpp"
+#include "game_config.hpp"
 #include "gettext.hpp"
 #include "sound.hpp"
 #include "log.hpp"
 #include "marked-up_text.hpp"
 
 
-
-#define ERR_DP LOG_STREAM(err, display)
-#define LOG_DP LOG_STREAM(info, display)
-#define DBG_DP LOG_STREAM(debug, display)
-#define ERR_G  LOG_STREAM(err, general)
+static lg::log_domain log_display("display");
+#define ERR_DP LOG_STREAM(err, log_display)
+#define LOG_DP LOG_STREAM(info, log_display)
+#define DBG_DP LOG_STREAM(debug, log_display)
+#define ERR_G  LOG_STREAM(err, lg::general)
 
 namespace gui {
 
@@ -247,6 +248,7 @@ void dialog::set_menu_items(const std::vector<std::string> &menu_items)
 		set_menu(menu_items);
 	} else {
 		menu_->set_items(menu_items);
+		menu_->reset_selection();
 
 		for(pp_iterator i = preview_panes_.begin(); i != preview_panes_.end(); ++i) {
 			(**i).set_selection(menu_->selection());
@@ -904,7 +906,6 @@ void filter_textbox::handle_text_changed(const wide_string& text) {
 	}
 
 	dialog_.set_menu_items(filtered_items_);
-	dialog_.get_menu().reset_selection();
 }
 
 int message_dialog::show(msecs minimum_lifetime)
