@@ -1,6 +1,6 @@
-/* $Id: dialogs.cpp 33775 2009-03-17 20:26:18Z soliton $ */
+/* $Id: dialogs.cpp 41192 2010-02-13 17:54:41Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,9 @@
 #include "global.hpp"
 
 #include "dialogs.hpp"
+#include "foreach.hpp"
 #include "game_events.hpp"
+#include "game_display.hpp"
 #include "game_preferences.hpp"
 #include "gettext.hpp"
 #include "help.hpp"
@@ -33,11 +35,15 @@
 #include "mouse_handler_base.hpp"
 #include "minimap.hpp"
 #include "replay.hpp"
+#include "resources.hpp"
+#include "savegame.hpp"
 #include "thread.hpp"
 #include "wml_separators.hpp"
 #include "widgets/progressbar.hpp"
 #include "wml_exception.hpp"
 #include "formula_string_utils.hpp"
+#include "gui/dialogs/game_save.hpp"
+#include "gui/dialogs/transient_message.hpp"
 
 
 //#ifdef _WIN32
@@ -46,23 +52,25 @@
 
 #include <clocale>
 
-#define LOG_NG LOG_STREAM(info, engine)
-#define LOG_DP LOG_STREAM(info, display)
-#define ERR_G  LOG_STREAM(err, general)
-#define ERR_CF LOG_STREAM(err, config)
+static lg::log_domain log_engine("engine");
+#define LOG_NG LOG_STREAM(info, log_engine)
+#define ERR_NG LOG_STREAM(err, log_engine)
+
+static lg::log_domain log_display("display");
+#define LOG_DP LOG_STREAM(info, log_display)
+
+#define ERR_G  LOG_STREAM(err, lg::general)
+
+static lg::log_domain log_config("config");
+#define ERR_CF LOG_STREAM(err, log_config)
 
 namespace dialogs
 {
 
-void advance_unit(const gamemap& map,
-                  unit_map& units,
-                  map_location loc,
-                  game_display& gui,
-		  bool random_choice,
-		  const bool add_replay_event)
+void advance_unit(const map_location &loc, bool random_choice, bool add_replay_event)
 {
-	unit_map::iterator u = units.find(loc);
-	if(u == units.end() || u->second.advances() == false)
+	unit_map::iterator u = resources::units->find(loc);
+	if (u == resources::units->end() || u->second.advances() == false)
 		return;
 
 	LOG_DP << "advance_unit: " << u->second.type_id() << "\n";
@@ -73,7 +81,7 @@ void advance_unit(const gamemap& map,
 
 	std::vector<unit> sample_units;
 	for(std::vector<std::string>::const_iterator op = options.begin(); op != options.end(); ++op) {
-		sample_units.push_back(::get_advanced_unit(units,loc,*op));
+		sample_units.push_back(::get_advanced_unit(u->second, *op));
 		const unit& type = sample_units.back();
 
 #ifdef LOW_MEM
@@ -84,20 +92,20 @@ void advance_unit(const gamemap& map,
 		preferences::encountered_units().insert(*op);
 	}
 
-	const config::child_list& mod_options = u->second.get_modification_advances();
 	bool always_display = false;
-	for(config::child_list::const_iterator mod = mod_options.begin(); mod != mod_options.end(); ++mod) {
-		if (utils::string_bool((**mod)["always_display"])) always_display = true;
-		sample_units.push_back(::get_advanced_unit(units,loc,u->second.type_id()));
-		sample_units.back().add_modification("advance",**mod);
+	foreach (const config &mod, u->second.get_modification_advances())
+	{
+		if (utils::string_bool(mod["always_display"])) always_display = true;
+		sample_units.push_back(::get_advanced_unit(u->second, u->second.type_id()));
+		sample_units.back().add_modification("advance", mod);
 		const unit& type = sample_units.back();
-		if((**mod)["image"].str().size()){
-		  lang_options.push_back(IMAGE_PREFIX + (**mod)["image"].str() + COLUMN_SEPARATOR + (**mod)["description"].str());
-		}else{
+		if (!mod["image"].empty()) {
+			lang_options.push_back(IMAGE_PREFIX + mod["image"].str() + COLUMN_SEPARATOR + mod["description"].str());
+		} else {
 #ifdef LOW_MEM
-		  lang_options.push_back(IMAGE_PREFIX + type.absolute_image() + COLUMN_SEPARATOR + (**mod)["description"].str());
+			lang_options.push_back(IMAGE_PREFIX + type.absolute_image() + COLUMN_SEPARATOR + mod["description"].str());
 #else
-		  lang_options.push_back(IMAGE_PREFIX + type.absolute_image() + u->second.image_mods() + COLUMN_SEPARATOR + (**mod)["description"].str());
+			lang_options.push_back(IMAGE_PREFIX + type.absolute_image() + u->second.image_mods() + COLUMN_SEPARATOR + mod["description"].str());
 #endif
 		}
 	}
@@ -112,11 +120,11 @@ void advance_unit(const gamemap& map,
 		res = rand()%lang_options.size();
 	} else if(lang_options.size() > 1 || always_display) {
 
-		units_list_preview_pane unit_preview(gui,&map,sample_units);
+		units_list_preview_pane unit_preview(sample_units);
 		std::vector<gui::preview_pane*> preview_panes;
 		preview_panes.push_back(&unit_preview);
 
-		gui::dialog advances = gui::dialog(gui,
+		gui::dialog advances = gui::dialog(*resources::screen,
 				      _("Advance Unit"),
 		                      _("What should our victorious unit become?"),
 		                      gui::OK_ONLY);
@@ -132,38 +140,38 @@ void advance_unit(const gamemap& map,
 	recorder.choose_option(res);
 
 	LOG_DP << "animating advancement...\n";
-	animate_unit_advancement(units,loc,gui,size_t(res));
+	animate_unit_advancement(loc, size_t(res));
 
 	// In some rare cases the unit can have enough XP to advance again,
 	// so try to do that.
 	// Make sure that we don't enter an infinite level loop.
-	u = units.find(loc);
-	if(u != units.end()) {
+	u = resources::units->find(loc);
+	if (u != resources::units->end()) {
 		// Level 10 unit gives 80 XP and the highest mainline is level 5
 		if(u->second.experience() < 81) {
 			// For all leveling up we have to add advancement to replay here because replay
-			// doesn't handle multi advancemnet
-			advance_unit(map, units, loc, gui, random_choice, true);
+			// doesn't handle cascading advancement since it just calls animate_unit_advancement().
+			advance_unit(loc, random_choice, true);
 		} else {
-			LOG_STREAM(err, config) << "Unit has an too high amount of " << u->second.experience()
+			ERR_CF << "Unit has an too high amount of " << u->second.experience()
 				<< " XP left, cascade leveling disabled\n";
 		}
 	} else {
-		LOG_STREAM(err, engine) << "Unit advanced no longer exists\n";
+		ERR_NG << "Unit advanced no longer exists\n";
 	}
 }
 
-bool animate_unit_advancement(unit_map& units, map_location loc, game_display& gui, size_t choice)
+bool animate_unit_advancement(const map_location &loc, size_t choice)
 {
 	const events::command_disabler cmd_disabler;
 
-	unit_map::iterator u = units.find(loc);
-	if(u == units.end() || u->second.advances() == false) {
+	unit_map::iterator u = resources::units->find(loc);
+	if (u == resources::units->end() || u->second.advances() == false) {
 		return false;
 	}
 
 	const std::vector<std::string>& options = u->second.advances_to();
-	const config::child_list& mod_options = u->second.get_modification_advances();
+	std::vector<config> mod_options = u->second.get_modification_advances();
 
 	if(choice >= options.size() + mod_options.size()) {
 		return false;
@@ -172,144 +180,58 @@ bool animate_unit_advancement(unit_map& units, map_location loc, game_display& g
 	// When the unit advances, it fades to white, and then switches
 	// to the new unit, then fades back to the normal colour
 
-	if(!gui.video().update_locked()) {
+	if (!resources::screen->video().update_locked()) {
 		unit_animator animator;
-		animator.add_animation(&u->second,"levelout",u->first);
+		bool with_bars = true;
+		animator.add_animation(&u->second,"levelout", u->first, map_location(), 0, with_bars);
 		animator.start_animations();
 		animator.wait_for_end();
 	}
 
 	if(choice < options.size()) {
-		const std::string& chosen_unit = options[choice];
-		::advance_unit(units,loc,chosen_unit);
+		// chosen_unit is not a reference, since the unit may disappear at any moment.
+		std::string chosen_unit = options[choice];
+		::advance_unit(loc, chosen_unit);
 	} else {
 		unit amla_unit(u->second);
-		config mod_option(*mod_options[choice - options.size()]);
+		const config &mod_option = mod_options[choice - options.size()];
 
 		LOG_NG << "firing advance event (AMLA)\n";
 		game_events::fire("advance",loc);
 
 		amla_unit.get_experience(-amla_unit.max_experience()); // subtract xp required
 		amla_unit.add_modification("advance",mod_option);
-		units.replace(loc, amla_unit);
+		resources::units->replace(loc, amla_unit);
 
 		LOG_NG << "firing post_advance event (AMLA)\n";
 		game_events::fire("post_advance",loc);
 	}
 
-	u = units.find(loc);
-	gui.invalidate_unit();
+	u = resources::units->find(loc);
+	resources::screen->invalidate_unit();
 
-	if(u != units.end() && !gui.video().update_locked()) {
+	if (u != resources::units->end() && !resources::screen->video().update_locked()) {
 		unit_animator animator;
-		animator.add_animation(&u->second,"levelin",u->first);
+		animator.add_animation(&u->second,"levelin",u->first, map_location(), 0, true);
 		animator.start_animations();
 		animator.wait_for_end();
 		animator.set_all_standing();
-		gui.invalidate(loc);
-		gui.draw();
+		resources::screen->invalidate(loc);
+		resources::screen->draw();
 		events::pump();
 	}
 
-	gui.invalidate_all();
-	gui.draw();
+	resources::screen->invalidate_all();
+	resources::screen->draw();
 
 	return true;
 }
 
-void show_objectives(game_display& disp, const config& level, const std::string& objectives)
+void show_objectives(const config &level, const std::string &objectives)
 {
 	static const std::string no_objectives(_("No objectives available"));
-	const std::string& name = level["name"];
-	std::string campaign_name = std::string(level["campaign"]);
-	replace_underbar2space(campaign_name);
-
-	gui::message_dialog(disp, "", "*~" + name +
-			(campaign_name.empty() ? "\n" : " - " + campaign_name + "\n") +
-	                (objectives.empty() ? no_objectives : objectives)
-	                ).show();
-}
-
-bool is_illegal_file_char(char c)
-{
-	return c == '/' || c == '\\' || c == ':'
- 	#ifdef _WIN32
-	|| c == '?' || c == '|' || c == '<' || c == '>' || c == '*' || c == '"'
-	#endif
-	;
-}
-
-int get_save_name(display & disp,const std::string& message, const std::string& txt_label,
-				  std::string* fname, gui::DIALOG_TYPE dialog_type, const std::string& title,
-				  const bool has_exit_button, const bool ask_for_filename)
-{
-	static int quit_prompt = 0;
-	const std::string& tmp_title = (title.empty()) ? _("Save Game") : title;
-	bool ignore_opt = false;
-	int overwrite=0;
-	int res=0;
-	bool ask = ask_for_filename;
-	do {
-		if (ask) {
-			gui::dialog d(disp, tmp_title, message, dialog_type);
-			d.set_textbox(txt_label, *fname);
-			if(has_exit_button) {
-				d.add_button(new gui::dialog_button(disp.video(), _("Quit Game"),
-					gui::button::TYPE_PRESS, 2), gui::dialog::BUTTON_STANDARD);
-				if(quit_prompt < 0) {
-					res = 1;
-				} else if(quit_prompt > 5) {
-					d.add_button(new gui::dialog_button(disp.video(), _("Ignore All"),
-						gui::button::TYPE_CHECK), gui::dialog::BUTTON_CHECKBOX);
-					res = d.show();
-					ignore_opt = d.option_checked();
-				} else {
-					res = d.show();
-					if(res == 1) {
-						++quit_prompt;
-					} else {
-						quit_prompt = 0;
-					}
-				}
-			} else {
-				res = d.show();
-			}
-			*fname = d.textbox_text();
-		} else {
-			ask = true;
-		}
-
-		if (std::count_if(fname->begin(),fname->end(),is_illegal_file_char)) {
-			gui::message_dialog(disp, _("Error"),
-				_("Save names may not contain colons, slashes, or backslashes. "
-				"Please choose a different name.")).show();
-			overwrite = 1;
-			continue;
-		}
-
-		if (is_gzip_file(*fname)) {
-			gui::message_dialog(disp, _("Error"),
-				_("Save names should not end on '.gz'. "
-				"Please choose a different name.")).show();
-			overwrite = 1;
-			continue;
-		}
-
-		if (res == 0 && save_game_exists(*fname)) {
-			std::stringstream s;
-			s << _("Save already exists. Do you want to overwrite it?")
-			  << std::endl << _("Name: ") << *fname;
-			overwrite = gui::dialog(disp,_("Overwrite?"),
-			    s.str(), gui::YES_NO).show();
-		} else {
-			overwrite = 0;
-		}
-	} while ((res == 0) && (overwrite != 0));
-
-	if(ignore_opt) {
-		quit_prompt = -1;
-	}
-	return res;
+	gui2::show_transient_message(resources::screen->video(), level["name"],
+		(objectives.empty() ? no_objectives : objectives), true);
 }
 
 namespace {
@@ -318,12 +240,12 @@ namespace {
 class delete_save : public gui::dialog_button_action
 {
 public:
-	delete_save(display& disp, gui::filter_textbox& filter, std::vector<save_info>& saves, std::vector<config*>& save_summaries) : disp_(disp), saves_(saves), summaries_(save_summaries), filter_(filter) {}
+	delete_save(display& disp, gui::filter_textbox& filter, std::vector<savegame::save_info>& saves, std::vector<config*>& save_summaries) : disp_(disp), saves_(saves), summaries_(save_summaries), filter_(filter) {}
 private:
 	gui::dialog_button_action::RESULT button_pressed(int menu_selection);
 
 	display& disp_;
-	std::vector<save_info>& saves_;
+	std::vector<savegame::save_info>& saves_;
 	std::vector<config*>& summaries_;
 	gui::filter_textbox& filter_;
 };
@@ -354,7 +276,7 @@ gui::dialog_button_action::RESULT delete_save::button_pressed(int menu_selection
 		filter_.delete_item(menu_selection);
 
 		// Delete the file
-		delete_game(saves_[index].name);
+		savegame::manager::delete_game(saves_[index].name);
 
 		// Remove it from the list of saves
 		saves_.erase(saves_.begin() + index);
@@ -375,13 +297,13 @@ class save_preview_pane : public gui::preview_pane
 {
 public:
 	save_preview_pane(CVideo &video, const config& game_config, gamemap* map,
-			const std::vector<save_info>& info,
+			const std::vector<savegame::save_info>& info,
 			const std::vector<config*>& summaries,
 			const gui::filter_textbox& textbox) :
 		gui::preview_pane(video),
 		game_config_(&game_config),
 		map_(map), info_(&info),
-		summaries_(&summaries),
+		summaries_(summaries),
 		index_(0),
 		map_cache_(),
 		textbox_(textbox)
@@ -401,8 +323,8 @@ public:
 private:
 	const config* game_config_;
 	gamemap* map_;
-	const std::vector<save_info>* info_;
-	const std::vector<config*>* summaries_;
+	const std::vector<savegame::save_info>* info_;
+	const std::vector<config*>& summaries_;
 	int index_;
 	std::map<std::string,surface> map_cache_;
 	const gui::filter_textbox& textbox_;
@@ -410,16 +332,16 @@ private:
 
 void save_preview_pane::draw_contents()
 {
-	if (size_t(index_) >= summaries_->size() || info_->size() != summaries_->size()) {
+	if (size_t(index_) >= summaries_.size() || info_->size() != summaries_.size()) {
 		return;
 	}
 
 	std::string dummy;
-	config& summary = *(*summaries_)[index_];
+	config& summary = *summaries_[index_];
 	if (summary["label"] == ""){
 		try {
-			load_game_summary((*info_)[index_].name, summary, &dummy);
-			*(*summaries_)[index_] = summary;
+			savegame::manager::load_summary((*info_)[index_].name, summary, &dummy);
+			*summaries_[index_] = summary;
 		} catch(game::load_game_failed&) {
 			summary["corrupt"] = "yes";
 		}
@@ -435,13 +357,13 @@ void save_preview_pane::draw_contents()
 
 	int ypos = area.y;
 
-	const unit_type_data::unit_type_map::const_iterator leader = unit_type_data::types().find_unit_type(summary["leader"]);
-	if(unit_type_data::types().unit_type_exists(summary["leader"])) {
-
+	const unit_type *leader = unit_types.find(summary["leader"]);
+	if (leader)
+	{
 #ifdef LOW_MEM
-		const surface image(image::get_image(leader->second.image()));
+		const surface image(image::get_image(leader->image()));
 #else
-		const surface image(image::get_image(leader->second.image() + "~RC(" + leader->second.flag_rgb() + ">1)"));
+		const surface image(image::get_image(leader->image() + "~RC(" + leader->flag_rgb() + ">1)"));
 #endif
 
 		if(image != NULL) {
@@ -454,14 +376,14 @@ void save_preview_pane::draw_contents()
 
 	std::string map_data = summary["map_data"];
 	if(map_data.empty()) {
-		const config* const scenario = game_config_->find_child(summary["campaign_type"],"id",summary["scenario"]);
-		if(scenario != NULL && scenario->find_child("side","shroud","yes") == NULL) {
-			map_data = (*scenario)["map_data"];
-			if(map_data.empty() && (*scenario)["map"].empty() == false) {
+		const config &scenario = game_config_->find_child(summary["campaign_type"], "id", summary["scenario"]);
+		if (scenario && !scenario.find_child("side", "shroud", "yes")) {
+			map_data = scenario["map_data"];
+			if (map_data.empty() && !scenario["map"].empty()) {
 				try {
-					map_data = read_map((*scenario)["map"]);
+					map_data = read_map(scenario["map"]);
 				} catch(io_exception& e) {
-					ERR_G << "could not read map '" << (*scenario)["map"] << "': " << e.what() << "\n";
+					ERR_G << "could not read map '" << scenario["map"] << "': " << e.what() << '\n';
 				}
 			}
 		}
@@ -500,9 +422,8 @@ void save_preview_pane::draw_contents()
 		SDL_BlitSurface(map_surf,NULL,screen,&map_rect);
 	}
 
-	char* old_locale = std::setlocale(LC_TIME, get_locale().localename.c_str());
 	char time_buf[256] = {0};
-	const save_info& save = (*info_)[index_];
+	const savegame::save_info& save = (*info_)[index_];
 	tm* tm_l = localtime(&save.time_modified);
 	if (tm_l) {
 		const size_t res = strftime(time_buf,sizeof(time_buf),_("%a %b %d %H:%M %Y"),tm_l);
@@ -513,15 +434,10 @@ void save_preview_pane::draw_contents()
 		LOG_NG << "localtime() returned null for time " << save.time_modified << ", save " << save.name;
 	}
 
-	if(old_locale) {
-		std::setlocale(LC_TIME, old_locale);
-	}
-
 	std::stringstream str;
 
-	// Escape all special characters in filenames
-	std::string name = (*info_)[index_].name;
-	str << font::BOLD_TEXT << utils::escape(name) << "\n" << time_buf;
+	str << font::BOLD_TEXT << font::NULL_MARKUP
+		<< (*info_)[index_].name << '\n' << time_buf;
 
 	const std::string& campaign_type = summary["campaign_type"];
 	if(utils::string_bool(summary["corrupt"], false)) {
@@ -531,7 +447,11 @@ void save_preview_pane::draw_contents()
 
 		if(campaign_type == "scenario") {
 			const std::string campaign_id = summary["campaign"];
-			const config* campaign = campaign_id.empty() ? NULL : game_config_->find_child("campaign", "id", campaign_id);
+			const config *campaign = NULL;
+			if (!campaign_id.empty()) {
+				if (const config &c = game_config_->find_child("campaign", "id", campaign_id))
+					campaign = &c;
+			}
 			utils::string_map symbols;
 			if (campaign != NULL) {
 				symbols["campaign_name"] = (*campaign)["name"];
@@ -621,23 +541,26 @@ std::string format_time_summary(time_t t)
 
 std::string load_game_dialog(display& disp, const config& game_config, bool* show_replay, bool* cancel_orders)
 {
-	std::vector<save_info> games;
+	std::vector<savegame::save_info> games;
 	{
 		cursor::setter cur(cursor::WAIT);
-		games = get_saves_list();
+		games = savegame::manager::get_saves_list();
 	}
 
 	if(games.empty()) {
-		gui::message_dialog(disp,
+		gui2::show_transient_message(disp.video(),
 		                 _("No Saved Games"),
-				 _("There are no saved games to load.\n\n(Games are saved automatically when you complete a scenario)")).show();
+				 _("There are no saved games to load.\n\n(Games are saved automatically when you complete a scenario)"));
 		return "";
 	}
 
 	std::vector<config*> summaries;
-	std::vector<save_info>::const_iterator i;
+	std::vector<savegame::save_info>::const_iterator i;
+	//FIXME: parent_to_child is not used yet
+	std::map<std::string,std::string> parent_to_child;
 	for(i = games.begin(); i != games.end(); ++i) {
-		config& cfg = save_summary(i->name);
+		config& cfg = savegame::save_index::save_summary(i->name);
+		parent_to_child[cfg["parent"]] = i->name;
 		summaries.push_back(&cfg);
 	}
 
@@ -695,7 +618,7 @@ std::string load_game_dialog(display& disp, const config& game_config, bool* sho
 
 	int res = lmenu.show();
 
-	write_save_index();
+	savegame::save_index::write_save_index();
 
 	if(res == -1)
 		return "";
@@ -742,15 +665,15 @@ unit_preview_pane::details::details() :
 {
 }
 
-unit_preview_pane::unit_preview_pane(game_display& disp, const gamemap* map,
-		const gui::filter_textbox* filter, TYPE type, bool on_left_side)
-				    : gui::preview_pane(disp.video()), disp_(disp), map_(map), index_(0),
-				      details_button_(disp.video(), _("Profile"),
+unit_preview_pane::unit_preview_pane(const gui::filter_textbox *filter, TYPE type, bool on_left_side) :
+	gui::preview_pane(resources::screen->video()), index_(0),
+	details_button_(resources::screen->video(), _("Profile"),
 				      gui::button::TYPE_PRESS,"lite_small", gui::button::MINIMUM_SPACE),
 				      filter_(filter), weapons_(type == SHOW_ALL), left_(on_left_side)
 {
 	unsigned w = font::relative_size(weapons_ ? 200 : 190);
-	unsigned h = font::relative_size(weapons_ ? 370 : 140);
+// advance test
+	unsigned h = font::relative_size(weapons_ ? 440 : 140);
 	set_measurements(w, h);
 }
 
@@ -781,7 +704,7 @@ void unit_preview_pane::set_selection(int index)
 	if(index != index_) {
 		index_ = index;
 		set_dirty();
-		if(map_ != NULL && index >= 0) {
+		if (index >= 0) {
 			details_button_.set_dirty();
 		}
 	}
@@ -818,18 +741,16 @@ void unit_preview_pane::draw_contents()
 	}
 
 	// Place the 'unit profile' button
-	if(map_ != NULL) {
-		const SDL_Rect button_loc = {right_align ? area.x : area.x + area.w - details_button_.location().w,
+	const SDL_Rect button_loc = {right_align ? area.x : area.x + area.w - details_button_.location().w,
 		                             image_rect.y + image_rect.h,
 		                             details_button_.location().w,details_button_.location().h};
-		details_button_.set_location(button_loc);
-	}
+	details_button_.set_location(button_loc);
 
 	SDL_Rect description_rect = {image_rect.x,image_rect.y+image_rect.h+details_button_.location().h,0,0};
 
 	if(det.name.empty() == false) {
 		std::stringstream desc;
-		desc << font::NORMAL_TEXT << det.name;
+		desc << font::BOLD_TEXT << det.name;
 		const std::string description = desc.str();
 		description_rect = font::text_area(description, font::SIZE_NORMAL);
 		description_rect = font::draw_text(&video(), area,
@@ -840,13 +761,14 @@ void unit_preview_pane::draw_contents()
 	}
 
 	std::stringstream text;
-	text << det.type_name << "\n"
-		<< det.race << "\n"
-		<< font::BOLD_TEXT << _("level") << " " << det.level << "\n"
+	text << font::unit_type << det.type_name << "\n"
+		<< font::race
+		<< (right_align && !weapons_ ? det.race+"  " : "  "+det.race) << "\n"
+		<< _("level") << " " << det.level << "\n"
 		<< det.alignment << "\n"
 		<< det.traits << "\n";
 
-	for(std::vector<t_string>::const_iterator a = det.abilities.begin(); a != det.abilities.end(); a++) {
+	for(std::vector<t_string>::const_iterator a = det.abilities.begin(); a != det.abilities.end(); ++a) {
 		if(a != det.abilities.begin()) {
 			text << ", ";
 		}
@@ -871,19 +793,19 @@ void unit_preview_pane::draw_contents()
 			//at_it->set_specials_context(map_location(),u);
 
 			// see generate_report() in generate_report.cpp
-			text << "<245,230,193>" << at_it->name()
+			text << font::weapon << at_it->name()
 				<< " (" << gettext(at_it->type().c_str()) << ")\n";
 
 			std::string special = at_it->weapon_specials(true);
 			if (!special.empty()) {
-				text << "<166,146,117>  " << special << "\n";
+				text << font::weapon_details << "  " << special << "\n";
 			}
 			std::string accuracy = at_it->accuracy_parry_description();
 			if(accuracy.empty() == false) {
 				accuracy += " ";
 			}
 
-			text << "<166,146,117>  " << at_it->damage() << "-" << at_it->num_attacks()
+			text << font::weapon_details << "  " << at_it->damage() << "-" << at_it->num_attacks()
 				<< " " << accuracy << "-- " << _(at_it->range().c_str()) << "\n";
 		}
 	}
@@ -892,36 +814,37 @@ void unit_preview_pane::draw_contents()
 	const std::vector<std::string> lines = utils::split(text.str(), '\n',
 		utils::STRIP_SPACES & !utils::REMOVE_EMPTY);
 
-	SDL_Rect cur_area = area;
+
+	int ypos = area.y;
 
 	if(weapons_) {
-		cur_area.y += image_rect.h + description_rect.h + details_button_.location().h;
+		ypos += image_rect.h + description_rect.h + details_button_.location().h;
 	}
 
 	for(std::vector<std::string>::const_iterator line = lines.begin(); line != lines.end(); ++line) {
-		int xpos = cur_area.x;
+		int xpos = area.x;
 		if(right_align && !weapons_) {
 			const SDL_Rect& line_area = font::text_area(*line,font::SIZE_SMALL);
-			xpos = cur_area.x + cur_area.w - line_area.w;
+			// right align, but if too long, don't hide line's beginning
+			if (line_area.w < area.w)
+				xpos = area.x + area.w - line_area.w;
 		}
 
-		cur_area = font::draw_text(&video(),location(),font::SIZE_SMALL,font::NORMAL_COLOUR,*line,xpos,cur_area.y);
-		cur_area.y += cur_area.h;
+		SDL_Rect cur_area = font::draw_text(&video(),location(),font::SIZE_SMALL,font::NORMAL_COLOUR,*line,xpos,ypos);
+		ypos += cur_area.h;
 	}
 }
 
-units_list_preview_pane::units_list_preview_pane(game_display& disp,
-		const gamemap* map, const unit& u, TYPE type, bool on_left_side) :
-	unit_preview_pane(disp, map, NULL, type, on_left_side),
+units_list_preview_pane::units_list_preview_pane(const unit &u, TYPE type, bool on_left_side) :
+	unit_preview_pane(NULL, type, on_left_side),
 	units_(&unit_store_),
 	unit_store_(1, u)
 {
 }
 
-units_list_preview_pane::units_list_preview_pane(game_display& disp,
-		const gamemap* map, std::vector<unit>& units,
+units_list_preview_pane::units_list_preview_pane(std::vector<unit> &units,
 		const gui::filter_textbox* filter, TYPE type, bool on_left_side) :
-	unit_preview_pane(disp, map, filter, type, on_left_side),
+	unit_preview_pane(filter, type, on_left_side),
 	units_(&units),
 	unit_store_()
 {
@@ -949,7 +872,7 @@ const unit_preview_pane::details units_list_preview_pane::get_details() const
 	det.traits = u.traits_description();
 
 	//we filter to remove the tooltips (increment by 2)
-	const std::vector<std::string>& abilities = u.unit_ability_tooltips();
+	const std::vector<std::string> &abilities = u.ability_tooltips(true);
 	for(std::vector<std::string>::const_iterator a = abilities.begin();
 		 a != abilities.end(); a+=2) {
 		det.abilities.push_back(*a);
@@ -972,15 +895,15 @@ const unit_preview_pane::details units_list_preview_pane::get_details() const
 
 void units_list_preview_pane::process_event()
 {
-	if(map_ != NULL && details_button_.pressed() && index_ >= 0 && index_ < int(size())) {
-		show_unit_description(disp_, (*units_)[index_]);
+	if (details_button_.pressed() && index_ >= 0 && index_ < int(size())) {
+		show_unit_description((*units_)[index_]);
 	}
 }
 
-unit_types_preview_pane::unit_types_preview_pane(game_display& disp, const gamemap* map,
+unit_types_preview_pane::unit_types_preview_pane(
 					std::vector<const unit_type*>& unit_types, const gui::filter_textbox* filter,
 					int side, TYPE type, bool on_left_side)
-					: unit_preview_pane(disp, map, filter, type, on_left_side),
+	: unit_preview_pane(filter, type, on_left_side),
 					  unit_types_(&unit_types), side_(side)
 {}
 
@@ -998,7 +921,7 @@ const unit_types_preview_pane::details unit_types_preview_pane::get_details() co
 		return det;
 
     //FIXME: There should be a better way to deal with this
-    unit_type_data::types().find_unit_type(t->id(), unit_type::WITHOUT_ANIMATIONS);
+	unit_types.find(t->id(), unit_type::WITHOUT_ANIMATIONS);
 
 	std::string mod = "~RC(" + t->flag_rgb() + ">" + team::get_side_colour_index(side_) + ")";
 	det.image = image::get_image(t->image()+mod);
@@ -1008,28 +931,25 @@ const unit_types_preview_pane::details unit_types_preview_pane::get_details() co
 	det.level = t->level();
 	det.alignment = unit_type::alignment_description(t->alignment(), t->genders().front());
 
-	race_map const& rcm = unit_type_data::types().races();
-	race_map::const_iterator ri = rcm.find(t->race());
-	if(ri != rcm.end()) {
-		assert(t->genders().empty() != true);
-		det.race = (*ri).second.name(t->genders().front());
+	if (const unit_race *r = unit_types.find_race(t->race())) {
+		assert(!t->genders().empty());
+		det.race = r->name(t->genders().front());
 	}
 
 	//FIXME: This probably must be move into a unit_type function
-	const std::vector<config*> traits = t->possible_traits();
-	for(std::vector<config*>::const_iterator i = traits.begin(); i != traits.end(); i++) {
-		if((**(i))["availability"] == "musthave") {
-			std::string gender_string = (!t->genders().empty() && t->genders().front()== unit_race::FEMALE) ? "female_name" : "male_name";
-			t_string name = (**i)[gender_string];
-			if (name.empty()) {
-				name = (**i)["name"];
-			}
-			if (!name.empty()) {
-				if (i != traits.begin()) {
-					det.traits += ", ";
-				}
-				det.traits += name;
+	foreach (const config &tr, t->possible_traits())
+	{
+		if (tr["availability"] != "musthave") continue;
+		std::string gender_string = (!t->genders().empty() && t->genders().front()== unit_race::FEMALE) ? "female_name" : "male_name";
+		t_string name = tr[gender_string];
+		if (name.empty()) {
+			name = tr["name"];
+		}
+		if (!name.empty()) {
+			if (!det.traits.empty()) {
+				det.traits += ", ";
 			}
+			det.traits += name;
 		}
 	}
 
@@ -1046,10 +966,10 @@ const unit_types_preview_pane::details unit_types_preview_pane::get_details() co
 	// Check if AMLA color is needed
 	// FIXME: not sure if it's fully accurate (but not very important for unit_type)
 	// xp_color also need a simpler function for doing this
-	const config::child_list& advances = t->modification_advancements();
-	for(config::child_list::const_iterator j = advances.begin(); j != advances.end(); ++j) {
-		if (!utils::string_bool((**j)["strict_amla"]) || !t->can_advance()) {
-			det.xp_color = "<100,0,150>"; // from unit::xp_color()
+	foreach (const config &adv, t->modification_advancements())
+	{
+		if (!utils::string_bool(adv["strict_amla"]) || !t->can_advance()) {
+			det.xp_color = "<170,0,255>"; // from unit::xp_color()
 			break;
 		}
 	}
@@ -1063,113 +983,27 @@ const unit_types_preview_pane::details unit_types_preview_pane::get_details() co
 
 void unit_types_preview_pane::process_event()
 {
-	if(map_ != NULL && details_button_.pressed() && index_ >= 0 && index_ < int(size())) {
+	if (details_button_.pressed() && index_ >= 0 && index_ < int(size())) {
 		const unit_type* type = (*unit_types_)[index_];
 		if (type != NULL)
-			show_unit_description(disp_, *type);
+			show_unit_description(*type);
 	}
 }
 
 
-void show_unit_description(game_display &disp, const unit& u)
+void show_unit_description(const unit &u)
 {
 	const unit_type* t = u.type();
 	if (t != NULL)
-		show_unit_description(disp, *t);
+		show_unit_description(*t);
 	else
 		// can't find type, try open the id page to have feedback and unit error page
-	  help::show_unit_help(disp, u.type_id());
+	  help::show_unit_help(*resources::screen, u.type_id());
 }
 
-void show_unit_description(game_display &disp, const unit_type& t)
+void show_unit_description(const unit_type &t)
 {
-	help::show_unit_help(disp, t.id(), t.hide_help());
-}
-
-
-namespace {
-	static const int campaign_preview_border = font::relative_size(10);
-}
-
-campaign_preview_pane::campaign_preview_pane(CVideo &video,std::vector<std::pair<std::string,std::string> >* desc) : gui::preview_pane(video),descriptions_(desc),index_(0)
-{
-// size of the campaign info window with the campaign description and image in pixel
-#ifdef USE_TINY_GUI
-	set_measurements(160, 200);
-#else
-	set_measurements(430, 440);
-#endif
-}
-
-bool campaign_preview_pane::show_above() const { return false; }
-bool campaign_preview_pane::left_side() const { return false; }
-
-void campaign_preview_pane::set_selection(int index)
-{
-	index = std::min<int>(descriptions_->size()-1,index);
-	if(index != index_ && index >= 0) {
-		index_ = index;
-		set_dirty();
-	}
-}
-
-void campaign_preview_pane::draw_contents()
-{
-	if (size_t(index_) >= descriptions_->size()) {
-		return;
-	}
-
-	const SDL_Rect area = {
-		location().x+campaign_preview_border,
-		location().y,
-		location().w-campaign_preview_border*2,
-		location().h };
-
-	/* background frame */
-	gui::dialog_frame f(video(), "", gui::dialog_frame::preview_style, false);
-	f.layout(area);
-	f.draw_background();
-	f.draw_border();
-
-	/* description text */
-	std::string desc_text;
-	try {
-		desc_text = font::word_wrap_text((*descriptions_)[index_].first,
-			font::SIZE_SMALL, area.w - 2 * campaign_preview_border);
-	} catch (utils::invalid_utf8_exception&) {
-		LOG_STREAM(err, engine) << "Invalid utf-8 found, campaign description is ignored.\n";
-	}
-	const std::vector<std::string> lines = utils::split(desc_text, '\n',utils::STRIP_SPACES);
-	SDL_Rect txt_area = { area.x+campaign_preview_border,area.y+campaign_preview_border,0,0 };
-
-	for(std::vector<std::string>::const_iterator line = lines.begin(); line != lines.end(); ++line) {
-	  txt_area = font::draw_text(&video(),location(),font::SIZE_SMALL,font::NORMAL_COLOUR,*line,txt_area.x,txt_area.y);
-		txt_area.y += txt_area.h;
-	}
-
-	/* description image */
-	surface img(NULL);
-	const std::string desc_img_name = (*descriptions_)[index_].second;
-	if(!desc_img_name.empty()) {
-		img.assign(image::get_image(desc_img_name));
-	}
-	if (!img.null()) {
-		SDL_Rect src_rect,dst_rect;
-		int max_height = area.h-(txt_area.h+txt_area.y-area.y);
-
-		src_rect.x = src_rect.y = 0;
-		src_rect.w = std::min<int>(area.w,img->w);
-		src_rect.h = std::min<int>(max_height,img->h);
-		dst_rect.x = area.x+(area.w-src_rect.w)/2;
-		dst_rect.y = txt_area.y+((max_height-src_rect.h)*8)/13;
-		if(dst_rect.y - txt_area.h - txt_area.y >= 120) {
-			//for really tall dialogs, just put it under the text
-			dst_rect.y = txt_area.y + font::get_max_height(font::SIZE_SMALL)*5;
-		}
-
-		SDL_BlitSurface(img,&src_rect,video().getSurface(),&dst_rect);
-
-	}
+	help::show_unit_help(*resources::screen, t.id(), t.hide_help());
 }
 
 static network::connection network_data_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num, network::statistics (*get_stats)(network::connection handle))
