@@ -1,6 +1,6 @@
-/* $Id: dialogs.hpp 32964 2009-02-21 14:05:24Z ilor $ */
+/* $Id: dialogs.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@ class unit_type;
 #include "map_location.hpp"
 #include "construct_dialog.hpp"
 #include "events.hpp"
+#include "network.hpp"
 
 #include "widgets/button.hpp"
 
@@ -40,28 +41,15 @@ namespace dialogs {
  * map (when replacing the unit that is being advanced) will possibly
  * invalidate the reference.
  *
- * The game only expects an advancement to be triggered by a fight, it the
+ * The game only expects an advancement to be triggered by a fight, if the
  * cause for advancement is different (eg unstore_unit) the add_replay_event
  * should be set.
  */
-void advance_unit(const gamemap& map,unit_map& units, map_location loc,
-				  game_display& gui, bool random_choice=false, const bool add_replay_event=false);
+void advance_unit(const map_location &loc, bool random_choice = false, bool add_replay_event = false);
 
-bool animate_unit_advancement(unit_map& units, map_location loc, game_display& gui, size_t choice);
+bool animate_unit_advancement(const map_location &loc, size_t choice);
 
-void show_objectives(game_display& disp, const config& level, const std::string& objectives);
-
-/** check if a character is valid for a filename. */
-bool is_illegal_file_char(char c);
-
-/**
- * Ask user if I should really save the game and what name I should use returns
- * 0 if user wants to save the game.
- */
-int get_save_name(display & disp,const std::string& message, const std::string& txt_label,
-				  std::string* fname, gui::DIALOG_TYPE dialog_type=gui::YES_NO,
-				  const std::string& title="", const bool has_exit_button=false,
-				  const bool ask_for_filename=true);
+void show_objectives(const config &level, const std::string &objectives);
 
 /**
  * Allow user to select the game they want to load. Returns the name of the
@@ -91,8 +79,8 @@ public:
 		std::vector<attack_type> attacks;
 	};
 
-	unit_preview_pane(game_display &disp, const gamemap* map, const gui::filter_textbox* filter=NULL,
-			TYPE type=SHOW_ALL, bool left_side=true);
+	unit_preview_pane(const gui::filter_textbox *filter = NULL,
+			TYPE type = SHOW_ALL, bool left_side = true);
 
 	bool show_above() const;
 	bool left_side() const;
@@ -101,8 +89,6 @@ public:
 	handler_vector handler_members();
 
 protected:
-	game_display& disp_;
-	const gamemap* map_;
 	int index_;
 	gui::button details_button_;
 
@@ -121,9 +107,8 @@ private:
 class units_list_preview_pane : public dialogs::unit_preview_pane
 {
 public:
-	units_list_preview_pane(game_display &disp, const gamemap* map,
-			const unit& u, TYPE type=SHOW_ALL, bool left_side=true);
-	units_list_preview_pane(game_display &disp, const gamemap* map,
+	units_list_preview_pane(const unit &u, TYPE type = SHOW_ALL, bool left_side = true);
+	units_list_preview_pane(
 			std::vector<unit>& units, const gui::filter_textbox* filter=NULL,
 			TYPE type=SHOW_ALL, bool left_side=true);
 
@@ -140,7 +125,7 @@ private:
 class unit_types_preview_pane : public dialogs::unit_preview_pane
 {
 public:
-	unit_types_preview_pane(game_display &disp, const gamemap* map,
+	unit_types_preview_pane(
 			std::vector<const unit_type*>& unit_types, const gui::filter_textbox* filterbox=NULL,
 			int side = 1, TYPE type=SHOW_ALL, bool left_side=true);
 
@@ -154,25 +139,8 @@ private:
 };
 
 
-void show_unit_description(game_display &disp, const unit_type& t);
-void show_unit_description(game_display &disp, const unit& u);
-
-
-class campaign_preview_pane : public gui::preview_pane
-{
-public:
-	campaign_preview_pane(CVideo &video,std::vector<std::pair<std::string,std::string> >* descriptions);
-
-	bool show_above() const;
-	bool left_side() const;
-	void set_selection(int index);
-
-private:
-	void draw_contents();
-
-	const std::vector<std::pair<std::string,std::string> >* descriptions_;
-	int index_;
-};
+void show_unit_description(const unit_type &t);
+void show_unit_description(const unit &u);
 
 network::connection network_send_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num=0);
 network::connection network_receive_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num=0);
