@@ -1,6 +1,6 @@
-/* $Id: display.cpp 33767 2009-03-17 19:19:34Z alink $ */
+/* $Id: display.cpp 42541 2010-05-10 22:52:59Z ai0867 $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,6 @@
 
 #include "global.hpp"
 
-#include "actions.hpp"
 #include "builder.hpp"
 #include "cursor.hpp"
 #include "display.hpp"
@@ -36,9 +35,9 @@
 #include "map.hpp"
 #include "map_label.hpp"
 #include "minimap.hpp"
-#include "pathfind.hpp"
 #include "preferences.hpp"
 #include "sdl_utils.hpp"
+#include "text.hpp"
 #include "tooltips.hpp"
 
 #include "SDL_image.h"
@@ -53,9 +52,10 @@
 #include <iostream>
 #include <sstream>
 
-#define ERR_DP LOG_STREAM(err, display)
-#define INFO_DP LOG_STREAM(info, display)
-#define DBG_DP LOG_STREAM(debug, display)
+static lg::log_domain log_display("display");
+#define ERR_DP LOG_STREAM(err, log_display)
+#define LOG_DP LOG_STREAM(info, log_display)
+#define DBG_DP LOG_STREAM(debug, log_display)
 
 namespace {
 #ifdef USE_TINY_GUI
@@ -76,7 +76,7 @@ namespace {
 
 int display::last_zoom_ = SmallZoom;
 
-display::display(CVideo& video, const gamemap* map, const config& theme_cfg, const config& cfg, const config& level) :
+display::display(CVideo& video, const gamemap* map, const config& theme_cfg, const config& level) :
 	screen_(video),
 	map_(map),
 	viewpoint_(NULL),
@@ -84,7 +84,7 @@ display::display(CVideo& video, const gamemap* map, const config& theme_cfg, con
 	ypos_(0),
 	theme_(theme_cfg, screen_area()),
 	zoom_(DefaultZoom),
-	builder_(new terrain_builder(cfg, level, map, theme_.border().tile_image)),
+	builder_(new terrain_builder(level, map, theme_.border().tile_image)),
 	minimap_(NULL),
 	minimap_location_(empty_rect),
 	redrawMinimap_(false),
@@ -106,12 +106,10 @@ display::display(CVideo& video, const gamemap* map, const config& theme_cfg, con
 	buttons_(),
 	invalidated_(),
 	previous_invalidated_(),
-	hex_overlay_(),
 	selected_hex_overlay_(0),
 	mouseover_hex_overlay_(0),
 	selectedHex_(),
 	mouseoverHex_(),
-	highlighted_locations_(),
 	keys_(),
 #if TDRAWING_BUFFER_USES_VECTOR
 	drawing_buffer_(LAYER_LAST_LAYER),
@@ -120,6 +118,8 @@ display::display(CVideo& video, const gamemap* map, const config& theme_cfg, con
 #endif
 	map_screenshot_(false),
 	fps_handle_(0),
+	invalidated_hexes_(0),
+	drawn_hexes_(0),
 	idle_anim_(preferences::idle_anim()),
 	idle_anim_rate_(1.0),
 	map_screenshot_surf_(NULL),
@@ -359,19 +359,19 @@ const map_location display::pixel_position_to_hex(int x, int y) const
 void display::rect_of_hexes::iterator::operator++()
 {
 	if (loc_.y < rect_.bottom[loc_.x & 1])
-		loc_.y++;
+		++loc_.y;
 	else {
-		loc_.x++;
+		++loc_.x;
 		loc_.y = rect_.top[loc_.x & 1];
 	}
 }
 
 // begin is top left, and end is after bottom right
-display::rect_of_hexes::iterator display::rect_of_hexes::begin()
+display::rect_of_hexes::iterator display::rect_of_hexes::begin() const
 {
 	return iterator(map_location(left, top[left & 1]), *this);
 }
-display::rect_of_hexes::iterator display::rect_of_hexes::end()
+display::rect_of_hexes::iterator display::rect_of_hexes::end() const
 {
 	return iterator(map_location(right+1, top[(right+1) & 1]), *this);
 }
@@ -380,6 +380,17 @@ const display::rect_of_hexes display::hexes_under_rect(const SDL_Rect& r) const
 {
 	rect_of_hexes res;
 
+	if (r.w<=0 || r.h<=0) {
+		// empty rect, return dummy values giving begin=end
+		res.left = 0;
+		res.right = -1; // end is right+1
+		res.top[0] = 0;
+		res.top[1] = 0;
+		res.bottom[0] = 0;
+		res.bottom[1] = 0;
+		return res;
+	}
+
 	SDL_Rect map_rect = map_area();
 	// translate rect coordinates from screen-based to map_area-based
 	int x = xpos_ - map_rect.x + r.x;
@@ -634,7 +645,7 @@ std::vector<std::string> display::get_fog_shroud_graphics(const map_location& lo
 }
 
 std::vector<surface> display::get_terrain_images(const map_location &loc,
-						     const std::string timeid,
+						     const std::string& timeid,
 		image::TYPE image_type,
 		ADJACENT_TERRAIN_TYPE terrain_type)
 {
@@ -672,7 +683,7 @@ std::vector<surface> display::get_terrain_images(const map_location &loc,
 		for(std::vector<animated<image::locator> >::const_iterator it =
 				terrains->begin(); it != terrains->end(); ++it) {
 
-			image::locator image = preferences::animate_map() ?
+			const image::locator& image = preferences::animate_map() ?
 				it->get_current_frame() : it->get_first_frame();
 
 			// We prevent ToD colouring and brightening of off-map tiles,
@@ -863,7 +874,7 @@ void display::flip()
 
 void display::update_display()
 {
-	if(screen_.update_locked()) {
+	if (screen_.update_locked()) {
 		return;
 	}
 
@@ -871,8 +882,8 @@ void display::update_display()
 		static int last_sample = SDL_GetTicks();
 		static int frames = 0;
 		++frames;
-
-		if(frames == 10) {
+		const int sample_freq = 10;
+		if(frames == sample_freq) {
 			const int this_sample = SDL_GetTicks();
 
 			const int fps = (frames*1000)/(this_sample - last_sample);
@@ -884,7 +895,15 @@ void display::update_display()
 				fps_handle_ = 0;
 			}
 			std::ostringstream stream;
-			stream << fps << "fps";
+			stream << "fps: " << fps;
+			if (game_config::debug) {
+				stream << "\nhex: " << drawn_hexes_*1.0/sample_freq;
+				if (drawn_hexes_ != invalidated_hexes_)
+					stream << " (" << (invalidated_hexes_-drawn_hexes_)*1.0/sample_freq << ")";
+			}
+			drawn_hexes_ = 0;
+			invalidated_hexes_ = 0;
+
 			fps_handle_ = font::add_floating_label(stream.str(),12,
 				benchmark ? font::BAD_COLOUR : font::NORMAL_COLOUR,
 				10,100,0,0,-1,screen_area(),font::LEFT_ALIGN);
@@ -892,6 +911,8 @@ void display::update_display()
 	} else if(fps_handle_ != 0) {
 		font::remove_floating_label(fps_handle_);
 		fps_handle_ = 0;
+		drawn_hexes_ = 0;
+		invalidated_hexes_ = 0;
 	}
 
 	flip();
@@ -1039,8 +1060,8 @@ void display::draw_text_in_hex(const map_location& loc,
 	const int y = get_location_y(loc) - text_surf->h/2
 	              + static_cast<int>(y_in_hex* hex_size());
 
-	for (int dy=-1; dy <= 1; dy++) {
-		for (int dx=-1; dx <= 1; dx++) {
+	for (int dy=-1; dy <= 1; ++dy) {
+		for (int dx=-1; dx <= 1; ++dx) {
 			if (dx!=0 || dy!=0) {
 				drawing_buffer_add(layer, loc, tblit(x + dx, y + dy, back_surf));
 			}
@@ -1049,16 +1070,6 @@ void display::draw_text_in_hex(const map_location& loc,
 	drawing_buffer_add(layer, loc, tblit(x, y, text_surf));
 }
 
-void display::clear_hex_overlay(const map_location& loc)
-{
-	if(! hex_overlay_.empty()) {
-		std::map<map_location, surface>::iterator itor = hex_overlay_.find(loc);
-		if(itor != hex_overlay_.end()) {
-			hex_overlay_.erase(itor);
-		}
-	}
-}
-
 void display::render_unit_image(int x, int y,const display::tdrawing_layer drawing_layer,
 		const map_location& loc, surface image,
 		bool hreverse, bool greyscale, fixed_t alpha,
@@ -1269,33 +1280,7 @@ void display::enable_menu(const std::string& item, bool enable)
 	}
 }
 
-void display::add_highlighted_loc(const map_location &hex)
-{
-	// Only invalidate and insert if this is a new addition,
-	// for efficiency.
-	if (highlighted_locations_.find(hex) == highlighted_locations_.end()) {
-		highlighted_locations_.insert(hex);
-		invalidate(hex);
-	}
-}
-
-void display::clear_highlighted_locs()
-{
-	invalidate(highlighted_locations_);
-	highlighted_locations_.clear();
-}
-
-void display::remove_highlighted_loc(const map_location &hex)
-{
-	std::set<map_location>::iterator it = highlighted_locations_.find(hex);
-	// Only invalidate and remove if the hex was found, for efficiency.
-	if (it != highlighted_locations_.end()) {
-		highlighted_locations_.erase(it);
-		invalidate(hex);
-	}
-}
-
-void display::announce(const std::string message, const SDL_Color& colour)
+void display::announce(const std::string& message, const SDL_Color& colour)
 {
 	font::add_floating_label(message,
 				 font::SIZE_XLARGE,
@@ -1420,7 +1405,7 @@ void display::draw_minimap()
                    box_color, screen);
 }
 
-void display::scroll(int xmove, int ymove)
+bool display::scroll(int xmove, int ymove)
 {
 	const int orig_x = xpos_;
 	const int orig_y = ypos_;
@@ -1432,7 +1417,7 @@ void display::scroll(int xmove, int ymove)
 
 	// Only invalidate if we've actually moved
 	if(dx == 0 && dy == 0)
-		return;
+		return false;
 
 	labels().scroll(dx, dy);
 	font::scroll_floating_labels(dx, dy);
@@ -1479,6 +1464,7 @@ void display::scroll(int xmove, int ymove)
 	update_rect(map_area());
 
 	redrawMinimap_ = true;
+	return true;
 }
 
 void display::set_zoom(int amount)
@@ -1524,13 +1510,23 @@ void display::set_default_zoom()
 	}
 }
 
-bool display::tile_on_screen(const map_location& loc)
+bool display::tile_fully_on_screen(const map_location& loc)
 {
 	int x = get_location_x(loc);
 	int y = get_location_y(loc);
 	return !outside_area(map_area(), x, y);
 }
 
+bool display::tile_nearly_on_screen(const map_location& loc) const
+{
+	int x = get_location_x(loc);
+	int y = get_location_y(loc);
+	const SDL_Rect &area = map_area();
+	int hw = hex_width(), hs = hex_size();
+	return x + hs >= area.x - hw && x < area.x + area.w + hw &&
+	       y + hs >= area.y - hs && y < area.y + area.h + hs;
+}
+
 void display::scroll_to_xy(int screenxpos, int screenypos, SCROLL_TYPE scroll_type, bool force)
 {
 	if(!force && !preferences::scroll_to_action()) return;
@@ -1645,9 +1641,8 @@ void display::scroll_to_tiles(const std::vector<map_location>& locs,
 	int miny = 0;
 	int maxy = 0;
 	bool valid = false;
-	bool first_tile_on_screen = false;
 
-	for(std::vector<map_location>::const_iterator itor = locs.begin(); itor != locs.end() ; itor++) {
+	for(std::vector<map_location>::const_iterator itor = locs.begin(); itor != locs.end() ; ++itor) {
 		if(get_map().on_board(*itor) == false) continue;
 		if(check_fogged && fogged(*itor)) continue;
 
@@ -1660,7 +1655,6 @@ void display::scroll_to_tiles(const std::vector<map_location>& locs,
 			miny = y;
 			maxy = y;
 			valid = true;
-			first_tile_on_screen = !outside_area(map_area(), x, y);
 		} else {
 			int minx_new = std::min<int>(minx,x);
 			int miny_new = std::min<int>(miny,y);
@@ -1718,8 +1712,8 @@ void display::scroll_to_tiles(const std::vector<map_location>& locs,
 
 		// we do not want to be only inside the screen rect, but center a bit more
 		double inside_frac = 0.5; // 0.0 = always center the target, 1.0 = scroll the minimum distance
-		w = (int)(w * inside_frac);
-		h = (int)(h * inside_frac);
+		w = static_cast<int>(w * inside_frac);
+		h = static_cast<int>(h * inside_frac);
 
 		// shrink the rectangle by the size of the locations rectangle we found
 		// such that the new task to fit a point into a rectangle instead of rectangle into rectangle
@@ -1865,7 +1859,7 @@ void display::redraw_everything()
 	int ticks2 = SDL_GetTicks();
 	draw(true,true);
 	int ticks3 = SDL_GetTicks();
-	INFO_DP << "invalidate and draw: " << (ticks3 - ticks2) << " and " << (ticks2 - ticks1) << "\n";
+	LOG_DP << "invalidate and draw: " << (ticks3 - ticks2) << " and " << (ticks2 - ticks1) << "\n";
 
 	foreach (boost::function<void(display&)> f, redraw_observers_) {
 		f(*this);
@@ -1957,17 +1951,18 @@ void display::draw_invalidated() {
 		int xpos = get_location_x(loc);
 		int ypos = get_location_y(loc);
 		const bool on_map = get_map().on_board(loc);
-		const bool off_map_tile = (get_map().get_terrain(loc) == t_translation::OFF_MAP_USER);
 		SDL_Rect hex_rect = {xpos, ypos, zoom_, zoom_};
 		if(!rects_overlap(hex_rect,clip_rect)) {
 			continue;
 		}
 		draw_hex(loc);
+		drawn_hexes_+=1;
 		// If the tile is at the border, we start to blend it
-		if(!on_map && !off_map_tile) {
+		if(!on_map) {
 			 draw_border(loc, xpos, ypos);
 		}
 	}
+	invalidated_hexes_ += invalidated_.size();
 }
 
 void display::draw_hex(const map_location& loc) {
@@ -1991,14 +1986,6 @@ void display::draw_hex(const map_location& loc) {
 		}
 	}
 
-
-	// Add the top layer overlay surfaces
-	if(!hex_overlay_.empty()) {
-		std::map<map_location, surface>::const_iterator itor = hex_overlay_.find(loc);
-		if(itor != hex_overlay_.end())
-			drawing_buffer_add(LAYER_TERRAIN_TMP_BG, loc, tblit(xpos, ypos, itor->second));
-	}
-
 	// Paint selection and mouseover overlays
 	if(loc == selectedHex_ && on_map && selected_hex_overlay_ != NULL) {
 		drawing_buffer_add(LAYER_TERRAIN_TMP_BG, loc, tblit(xpos, ypos, selected_hex_overlay_));
@@ -2106,163 +2093,166 @@ void display:: set_report_content(const reports::TYPE which_report, const std::s
 	report_[which_report] = content;
 }
 
-void display::refresh_report(reports::TYPE report_num, reports::report report,
-			     bool brighten)
+void display::refresh_report(reports::TYPE report_num, reports::report report)
 {
 	const theme::status_item* const item = theme_.get_status_item(reports::report_name(report_num));
-	if(item != NULL) {
-		SDL_Rect& rect = reportRects_[report_num];
-		const SDL_Rect& new_rect = item->location(screen_area());
-
-		// Report and its location is unchanged since last time. Do nothing.
-		if(rect == new_rect && reports_[report_num] == report) {
-			return;
-		}
-
-		reports_[report_num] = report;
-
-		surface& surf = reportSurfaces_[report_num];
-
-		if(surf != NULL) {
-			SDL_BlitSurface(surf,NULL,screen_.getSurface(),&rect);
-			update_rect(rect);
-		}
-		// If the rectangle has just changed, assign the surface to it
-		if(new_rect != rect || surf == NULL) {
-			surf.assign(NULL);
-			rect = new_rect;
-
-			// If the rectangle is present, and we are blitting text,
-			// then we need to backup the surface.
-			// (Images generally won't need backing up,
-			// unless they are transperant, but that is done later).
-			if(rect.w > 0 && rect.h > 0) {
-				surf.assign(get_surface_portion(screen_.getSurface(),rect));
-				if(reportSurfaces_[report_num] == NULL) {
-					ERR_DP << "Could not backup background for report!\n";
-				}
-			}
-
-			update_rect(rect);
-		}
-
-		tooltips::clear_tooltips(rect);
-
-		SDL_Rect area = rect;
-
-		int x = rect.x, y = rect.y;
-
-		if(!report.empty()) {
-			// Add prefix, postfix elements.
-			// Make sure that they get the same tooltip
-			// as the guys around them.
-			std::stringstream temp;
-			Uint32 RGB = item->font_rgb();
-			int red   = (RGB & 0x00FF0000)>>16;
-			int green = (RGB & 0x0000FF00)>>8;
-			int blue  = (RGB & 0x000000FF);
-
-			static const std::string c_start="<";
-			static const std::string c_sep=",";
-			static const std::string c_end=">";
-			std::stringstream color;
-			color<< c_start << red << c_sep << green << c_sep << blue << c_end;
-			std::string str;
-
-			str = item->prefix();
-			if(str.empty() == false) {
-			  report.insert(report.begin(), reports::element(str,"",report.begin()->tooltip));
-			}
-			str = item->postfix();
-			if(str.empty() == false) {
-			  report.push_back(reports::element(str,"",report.end()->tooltip));
-			}
-			// Loop through and display each report element
-			size_t tallest = 0;
-			int image_count = 0;
-			bool used_ellipsis=false;
-			std::stringstream ellipsis_tooltip;
-			SDL_Rect ellipsis_area =rect;
-			for(reports::report::iterator i = report.begin(); i != report.end(); ++i) {
-			  temp.str("");
-				if(i->text.empty() == false) {
-				  if(used_ellipsis == false) {
-					// Draw a text element
-				        if(item->font_rgb_set()) {
-						temp <<color.str();
-					}
-					temp << i->text;
-					str = temp.str();
-					area = font::draw_text(&screen_,rect,item->font_size(),font::NORMAL_COLOUR,str,x,y);
-					if(area.h > tallest) {
-						tallest = area.h;
-					}
-					if(i->text[i->text.size() - 1] == '\n') {
-						x = rect.x;
-						y += tallest;
-						tallest = 0;
-					} else {
-						x += area.w;
-					}
-				  }
-				} else if(i->image.get_filename().empty() == false) {
-				  if(used_ellipsis == false) {
-					// Draw an image element
-					surface img(image::get_image(i->image));
-
-					if(img == NULL) {
-						ERR_DP << "could not find image for report: '" << i->image.get_filename() << "'\n";
-						continue;
-					}
-
-					if(rect.w + rect.x - x < img->w && image_count) {
-					  // We have more than one image, and this one doesn't fit.
-					  img=surface(image::get_image(game_config::ellipsis_image));
-					  used_ellipsis=true;
-					}
-
-					area.x = x;
-					area.y = y;
-					area.w = std::min<int>(rect.w + rect.x - x, img->w);
-					area.h = std::min<int>(rect.h + rect.y - y, img->h);
-					draw_image_for_report(img, area);
-
-					if(brighten) {
-						surface tod_bright(image::get_image(game_config:: tod_bright_image));
-						if(tod_bright != NULL) {
-							draw_image_for_report(tod_bright,area);
-						}
-					}
-
-					image_count++;
-					if(area.h > tallest) {
-						tallest = area.h;
-					}
-
-					if(! used_ellipsis) {
-						x += area.w;
-					} else {
-						ellipsis_area = area;
-					}
-				  }
-				} else {
-					// No text or image, skip this element
-					continue;
-				}
-				if(i->tooltip.empty() == false) {
-					if(! used_ellipsis) {
-						tooltips::add_tooltip(area,i->tooltip);
-					} else { // Collect all tooltips for the ellipsis
-						ellipsis_tooltip<<i->tooltip<<"\n";
-					}
-				}
-			}
-			if(used_ellipsis) {
-				tooltips::add_tooltip(ellipsis_area,ellipsis_tooltip.str());
-			}
-		}
-	} else {
+	if (!item) {
 		reportSurfaces_[report_num].assign(NULL);
+		return;
+	}
+
+	SDL_Rect &rect = reportRects_[report_num];
+	const SDL_Rect &new_rect = item->location(screen_area());
+
+	// Report and its location is unchanged since last time. Do nothing.
+	if (rect == new_rect && reports_[report_num] == report) {
+		return;
+	}
+
+	reports_[report_num] = report;
+
+	surface &surf = reportSurfaces_[report_num];
+
+	if (surf) {
+		SDL_BlitSurface(surf, NULL, screen_.getSurface(), &rect);
+		update_rect(rect);
+	}
+
+	// If the rectangle has just changed, assign the surface to it
+	if (new_rect != rect || !surf)
+	{
+		surf.assign(NULL);
+		rect = new_rect;
+
+		// If the rectangle is present, and we are blitting text,
+		// then we need to backup the surface.
+		// (Images generally won't need backing up,
+		// unless they are transperant, but that is done later).
+		if (rect.w > 0 && rect.h > 0) {
+			surf.assign(get_surface_portion(screen_.getSurface(), rect));
+			if (reportSurfaces_[report_num] == NULL) {
+				ERR_DP << "Could not backup background for report!\n";
+			}
+		}
+		update_rect(rect);
+	}
+
+	tooltips::clear_tooltips(rect);
+
+	if (report.empty()) return;
+
+	int x = rect.x, y = rect.y;
+
+	// Add prefix, postfix elements.
+	// Make sure that they get the same tooltip
+	// as the guys around them.
+	std::string str = item->prefix();
+	if (!str.empty()) {
+		report.insert(report.begin(), reports::element(str, "", report.begin()->tooltip));
+	}
+	str = item->postfix();
+	if (!str.empty()) {
+		report.push_back(reports::element(str, "", report.back().tooltip));
+	}
+
+	// Loop through and display each report element.
+	int tallest = 0;
+	int image_count = 0;
+	bool used_ellipsis = false;
+	std::ostringstream ellipsis_tooltip;
+	SDL_Rect ellipsis_area = rect;
+
+	foreach (const reports::element &e, report)
+	{
+		SDL_Rect area = { x, y, rect.w + rect.x - x, rect.h + rect.y - y };
+		if (area.h <= 0) break;
+
+		if (!e.text.empty())
+		{
+			if (used_ellipsis) goto skip_element;
+
+			// Draw a text element.
+			font::ttext text;
+			if (item->font_rgb_set()) {
+				text.set_foreground_colour(item->font_rgb());
+			}
+			std::string t = e.text;
+			bool eol = false;
+			if (t[t.size() - 1] == '\n') {
+				eol = true;
+				t = t.substr(0, t.size() - 1);
+			}
+			text.set_font_size(item->font_size());
+			text.set_text(t, true);
+			text.set_maximum_width(area.w);
+			text.set_maximum_height(area.h, false);
+			surface s = text.render();
+			screen_.blit_surface(x, y, s);
+			area.w = s->w;
+			area.h = s->h;
+			if (area.h > tallest) {
+				tallest = area.h;
+			}
+			if (eol) {
+				x = rect.x;
+				y += tallest;
+				tallest = 0;
+			} else {
+				x += area.w;
+			}
+		}
+		else if (!e.image.get_filename().empty())
+		{
+			if (used_ellipsis) goto skip_element;
+
+			// Draw an image element.
+			surface img(image::get_image(e.image));
+
+			if (!img) {
+				ERR_DP << "could not find image for report: '" << e.image.get_filename() << "'\n";
+				continue;
+			}
+
+			if (area.w < img->w && image_count) {
+				// We have more than one image, and this one doesn't fit.
+				img = surface(image::get_image(game_config::ellipsis_image));
+				used_ellipsis = true;
+			}
+
+			if (img->w < area.w) area.w = img->w;
+			if (img->h < area.h) area.h = img->h;
+			draw_image_for_report(img, area);
+
+			++image_count;
+			if (area.h > tallest) {
+				tallest = area.h;
+			}
+
+			if (!used_ellipsis) {
+				x += area.w;
+			} else {
+				ellipsis_area = area;
+			}
+		}
+		else
+		{
+			// No text nor image, skip this element
+			continue;
+		}
+
+		skip_element:
+		if (!e.tooltip.empty()) {
+			if (!used_ellipsis) {
+				tooltips::add_tooltip(area, e.tooltip);
+			} else {
+				// Collect all tooltips for the ellipsis.
+				ellipsis_tooltip << e.tooltip << '\n';
+			}
+		}
+	}
+
+	if (used_ellipsis) {
+		tooltips::add_tooltip(ellipsis_area, ellipsis_tooltip.str());
 	}
 }
 
@@ -2293,6 +2283,29 @@ bool display::invalidate(const std::set<map_location>& locs)
 	return ret;
 }
 
+bool display::propagate_invalidation(const std::set<map_location>& locs)
+{
+	if(invalidateAll_)
+		return false;
+
+	if(locs.size()<=1)
+		return false; // propagation never needed
+
+	// search the first hex invalidated (if any)
+	std::set<map_location>::const_iterator i = locs.begin();
+	for(; i != locs.end() && invalidated_.count(*i) == 0 ; ++i) {}
+
+	if (i == locs.end())
+		return false; // no invalidation, don't propagate
+
+	// propagate invalidation
+	// 'i' is already in, but I suspect that splitting the range is bad
+	// especially because locs are often adjacents
+	size_t previous_size = invalidated_.size();
+	invalidated_.insert(locs.begin(), locs.end());
+	return previous_size < invalidated_.size();
+}
+
 bool display::invalidate_visible_locations_in_rect(const SDL_Rect& rect)
 {
 	return invalidate_locations_in_rect(intersect_rects(map_area(),rect));
@@ -2304,41 +2317,25 @@ bool display::invalidate_locations_in_rect(const SDL_Rect& rect)
 		return false;
 
 	bool result = false;
-	rect_of_hexes hexes = hexes_under_rect(rect);
-	rect_of_hexes::iterator i = hexes.begin(), end = hexes.end();
-	for (;i != end; ++i) {
-		result |= invalidate(*i);
+	foreach (const map_location &loc, hexes_under_rect(rect)) {
+		result |= invalidate(loc);
 	}
 	return result;
 }
 
-bool display::rectangle_need_update(const SDL_Rect& rect) const
+void display::invalidate_animations()
 {
-	if(invalidateAll_)
-		return true;
-
-	rect_of_hexes hexes = hexes_under_rect(rect);
-	rect_of_hexes::iterator i = hexes.begin(), end = hexes.end();
-	for (;i != end; ++i) {
-		if(invalidated_.find(*i) != invalidated_.end())
-			return true;
+	if (!preferences::animate_map()) {
+		return;
 	}
 
-	return false;
-}
-
-void display::invalidate_animations() {
-	if (preferences::animate_map()) {
-		rect_of_hexes hexes = get_visible_hexes();
-		rect_of_hexes::iterator i = hexes.begin(), end = hexes.end();
-		for (;i != end; ++i) {
-			if (!shrouded(*i)) {
-				if (builder_->update_animation(*i)) {
-					invalidate(*i);
-				} else {
-					invalidate_animations_location(*i);
-				}
-			}
+	foreach (const map_location &loc, get_visible_hexes())
+	{
+		if (shrouded(loc)) continue;
+		if (builder_->update_animation(loc)) {
+			invalidate(loc);
+		} else {
+			invalidate_animations_location(loc);
 		}
 	}
 }
