@@ -1,6 +1,6 @@
-/* $Id: editor_layout.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: editor_layout.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-  Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+  Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
   This program is free software; you can redistribute it and/or modify
@@ -12,23 +12,24 @@
   See the COPYING file for more details.
 */
 
-//! @file editor/editor_layout.hpp 
-//!
+/** @file editor/editor_layout.hpp */
 
 #ifndef EDITOR_LAYOUT_H_INCLUDED
 #define EDITOR_LAYOUT_H_INCLUDED
 
 #include "global.hpp"
 
-#include "display.hpp"
+class display;
 
-namespace map_editor {
+namespace editor {
 
-//! Size specifications for the map editor.
+/** Size specifications for the map editor. */
 struct size_specs {
-	//! Initialize the values to dummie-values that will 
-	//! avoid floating point errors if calculations are made 
-	//! before the sizes are adjusted through adjust_sizes().
+	/**
+	 * Initialize the values to dummie-values that will avoid floating point
+	 * errors if calculations are made before the sizes are adjusted through
+	 * adjust_sizes().
+	 */
 	size_specs();
 	size_t terrain_size;
 	size_t terrain_padding;
@@ -43,7 +44,7 @@ struct size_specs {
 	size_t brush_padding;
 };
 
-//! Adjust the internal size specifications to fit the display.
+/** Adjust the internal size specifications to fit the display. */
 void adjust_sizes(const display &disp, size_specs &sizes);
 
 }
