@@ -1,6 +1,6 @@
-/* $Id: editor_palettes.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: editor_palettes.cpp 46326 2010-09-09 04:14:18Z shadowmaster $ */
 /*
-  Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+  Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
   This program is free software; you can redistribute it and/or modify
@@ -12,92 +12,126 @@
   See the COPYING file for more details.
 */
 
-//! @file editor/editor_palettes.cpp
-//! Manage the terrain-palette in the editor.
+/**
+ * Manage the terrain-palette in the editor.
+ * Note: this is a near-straight rip from the old editor.
+*/
 
-#include "SDL.h"
-#include "SDL_keysym.h"
+#define GETTEXT_DOMAIN "wesnoth-editor"
 
+#include "editor_common.hpp"
 #include "editor_palettes.hpp"
-#include "editor_layout.hpp"
-#include "../config.hpp"
-#include "../sdl_utils.hpp"
-#include "../serialization/string_utils.hpp"
-#include "../image.hpp"
-#include "../reports.hpp"
+
+#include "../foreach.hpp"
 #include "../gettext.hpp"
+#include "../serialization/string_utils.hpp"
+#include "../sound.hpp"
 #include "../tooltips.hpp"
-#include "../util.hpp"
-#include "../video.hpp"
 
-#include <cassert>
+namespace editor {
 
-namespace map_editor {
-
-bool is_invalid_terrain(t_translation::t_terrain c) {
-	return (c == t_translation::VOID_TERRAIN || c == t_translation::FOGGED);
+static bool is_valid_terrain(t_translation::t_terrain c) {
+	return !(c == t_translation::VOID_TERRAIN || c == t_translation::FOGGED);
 }
 
 terrain_group::terrain_group(const config& cfg, display& gui):
 	id(cfg["id"]), name(cfg["name"]),
-	button(gui.video(), "", gui::button::TYPE_CHECK, cfg["icon"])
+	button(gui.video(), "", gui::button::TYPE_CHECK, cfg["icon"]),
+	core(utils::string_bool(cfg["core"], false))
 {
 }
 
 terrain_palette::terrain_palette(display &gui, const size_specs &sizes,
-								 const gamemap &map, const config& cfg)
-	: gui::widget(gui.video()), size_specs_(sizes), gui_(gui), tstart_(0),
-	  checked_group_btn_(0), map_(map),
-	  top_button_(gui.video(), "", gui::button::TYPE_PRESS, "uparrow-button"),
-	  bot_button_(gui.video(), "", gui::button::TYPE_PRESS, "downarrow-button")
+								 const config& cfg,
+								 t_translation::t_terrain& fore,
+								 t_translation::t_terrain& back)
+	: gui::widget(gui.video())
+	, size_specs_(sizes)
+	, gui_(gui)
+	, tstart_(0)
+	, terrain_map_()
+	, terrains_()
+	, terrain_groups_()
+	, non_core_terrains_()
+	, checked_group_btn_(0)
+	, top_button_(gui.video(), "", gui::button::TYPE_PRESS, "uparrow-button")
+	, bot_button_(gui.video(), "", gui::button::TYPE_PRESS, "downarrow-button")
+	, button_x_()
+	, top_button_y_()
+	, bot_button_y_()
+	, nterrains_()
+	, terrain_start_()
+	, selected_fg_terrain_(fore)
+	, selected_bg_terrain_(back)
 {
-
 	// Get the available terrains temporary in terrains_
-	terrains_ = map_.get_terrain_list();
-	terrains_.erase(std::remove_if(terrains_.begin(), terrains_.end(), is_invalid_terrain),
-					terrains_.end());
+	terrains_ = map().get_terrain_list();
+
+	//move "invalid" terrains to the end
+	std::stable_partition(terrains_.begin(), terrains_.end(), is_valid_terrain);
 
 	// Get the available groups and add them to the structure
-    const config::child_list& groups = cfg.get_children("editor_group");
-	config::child_list::const_iterator g_itor = groups.begin();
-	for(; g_itor != groups.end(); ++ g_itor) {
-		terrain_groups_.push_back(terrain_group(**g_itor, gui));
-
-		// By default the 'all'-button is pressed
-		if(terrain_groups_.back().id == "all") {
-			terrain_groups_.back().button.set_check(true);
-			checked_group_btn_ = &terrain_groups_.back().button;
+	std::set<std::string> group_names;
+	foreach (const config &g, cfg.child_range("editor_group"))
+	{
+		if (group_names.find(g["id"]) == group_names.end()) {
+			terrain_groups_.push_back(terrain_group(g, gui));
+			group_names.insert(terrain_groups_.back().id);
+			// By default the 'all'-button is pressed
+			if(terrain_groups_.back().id == "all") {
+				terrain_groups_.back().button.set_check(true);
+				checked_group_btn_ = &terrain_groups_.back().button;
+			}
 		}
 	}
+	std::map<std::string, terrain_group*> id_to_group;
+	foreach (terrain_group& tg, terrain_groups_) {
+		id_to_group.insert(std::make_pair(tg.id, &tg));
+	}
 	// The rest of the code assumes this is a valid pointer
 	assert(checked_group_btn_ != 0);
 
-	// Add the groups for all terrains to the map
-	t_translation::t_list::const_iterator t_itor = terrains_.begin();
-	for(; t_itor != terrains_.end(); ++t_itor) {
-		// Add the terrain to the requested groups
-		const std::vector<std::string>& key =
-			utils::split(map_.get_terrain_info(*t_itor).editor_group());
+	// add the groups for all terrains to the map
+	foreach (const t_translation::t_terrain& t, terrains_) {
+		const terrain_type& t_info = map().get_terrain_info(t);
+		DBG_ED << "Palette: processing terrain " << t_info.name()
+			<< "(" << t_info.number() << ")"
+			<< ": " << t_info.editor_group() << "\n";
 
-		for(std::vector<std::string>::const_iterator k_itor = key.begin();
-				k_itor != key.end(); ++k_itor)
-		 {
-			terrain_map_[*k_itor].push_back(*t_itor);
+		// don't display terrains that were automatically created from base+overlay
+		if (t_info.is_combined()) continue;
+		// nor display terrains that have hide_in_editor=true
+		if (t_info.hide_in_editor()) continue;
+
+		// add the terrain to the requested groups
+		const std::vector<std::string>& keys = utils::split(t_info.editor_group());
+		bool core = false;
+		foreach (const std::string& k, keys) {
+			terrain_map_[k].push_back(t);
+			std::map<std::string, terrain_group*>::iterator i = id_to_group.find(k);
+			if (i != id_to_group.end()) {
+				if (i->second->core) {
+					core = true;
+				}
+			}
+		}
+		// A terrain is considered core iff it appears in at least
+		// one core terrain group
+		if (core) {
+			// Add the terrain to the default group
+			terrain_map_["all"].push_back(t);
+		} else {
+			non_core_terrains_.insert(t);
 		}
 
-		// Add the terrain to the default group
-		terrain_map_["all"].push_back(*t_itor);
 	}
+	typedef std::pair<std::string, t_translation::t_list> map_pair;
 
 	// Set the default group
 	terrains_ = terrain_map_["all"];
 
 	if(terrains_.empty()) {
-		std::cerr << "No terrain found.\n";
-	}
-	else {
-		selected_fg_terrain_ = terrains_[0];
-		selected_bg_terrain_ = terrains_[0];
+		ERR_ED << "No terrain found.\n";
 	}
 	update_report();
 	//adjust_size();
@@ -110,7 +144,7 @@ void terrain_palette::adjust_size() {
 	const size_t button_palette_padding = 8;
 
 	// Values for the group buttons fully hardcoded for now
-	//! @todo will be fixed later
+	/** @todo will be fixed later */
 	const size_t group_button_height   = 24;
 	const size_t group_button_padding  =  2;
 	const size_t group_buttons_per_row =  5;
@@ -135,7 +169,7 @@ void terrain_palette::adjust_size() {
 		static_cast<unsigned> (space_for_terrains / size_specs_.terrain_space) *
 		size_specs_.terrain_width;
 	const unsigned total_terrains = num_terrains();
-	nterrains_ = minimum<int>(terrains_fitting, total_terrains);
+	nterrains_ = std::min<int>(terrains_fitting, total_terrains);
 	bot_button_y_ = size_specs_.palette_y + (nterrains_ / size_specs_.terrain_width) * size_specs_.terrain_space + \
 		button_palette_padding * size_specs_.terrain_width + button_height + group_height;
 	top_button_.set_location(button_x_, top_button_y_);
@@ -210,7 +244,7 @@ void terrain_palette::set_group(const std::string& id)
 {
 	terrains_ = terrain_map_[id];
 	if(terrains_.empty()) {
-		std::cerr << "No terrain found.\n";
+		ERR_ED << "No terrain found.\n";
 	}
 	scroll_top();
 }
@@ -243,10 +277,16 @@ void terrain_palette::select_bg_terrain(t_translation::t_terrain terrain)
 	}
 }
 
-/**
- * After the language is changed, the selected terrains needs an update.
- */
-void terrain_palette::update_selected_terrains(void)
+void terrain_palette::swap()
+{
+	std::swap(selected_fg_terrain_, selected_bg_terrain_);
+	set_dirty();
+	update_report();
+}
+
+
+/** After the language is changed, the selected terrains needs an update. */
+void terrain_palette::update_selected_terrains()
 {
 	set_dirty();
 	update_report();
@@ -255,15 +295,15 @@ void terrain_palette::update_selected_terrains(void)
 std::string terrain_palette::get_terrain_string(const t_translation::t_terrain t)
 {
 	std::stringstream str;
-	const std::string& name = map_.get_terrain_info(t).name();
-	const t_translation::t_list& underlying = map_.underlying_union_terrain(t);
+	const std::string& name = map().get_terrain_info(t).name();
+	const t_translation::t_list& underlying = map().underlying_union_terrain(t);
 	str << name;
 	if(underlying.size() != 1 || underlying[0] != t) {
 		str << " (";
 		for(t_translation::t_list::const_iterator i = underlying.begin();
 			i != underlying.end(); ++i) {
 
-			str << map_.get_terrain_info(*i).name();
+			str << map().get_terrain_info(*i).name();
 			if(i+1 != underlying.end()) {
 				str << ",";
 			}
@@ -275,7 +315,7 @@ std::string terrain_palette::get_terrain_string(const t_translation::t_terrain t
 
 void terrain_palette::left_mouse_click(const int mousex, const int mousey) {
 	int tselect = tile_selected(mousex, mousey);
-	if(tselect >= 0) {
+	if(tselect >= 0 && (tstart_+tselect) < terrains_.size()) {
 		select_fg_terrain(terrains_[tstart_+tselect]);
 		gui_.invalidate_game_status();
 	}
@@ -283,7 +323,7 @@ void terrain_palette::left_mouse_click(const int mousex, const int mousey) {
 
 void terrain_palette::right_mouse_click(const int mousex, const int mousey) {
 	int tselect = tile_selected(mousex, mousey);
-	if(tselect >= 0) {
+	if(tselect >= 0 && (tstart_+tselect) < terrains_.size()) {
 		select_bg_terrain(terrains_[tstart_+tselect]);
 		gui_.invalidate_game_status();
 	}
@@ -339,25 +379,29 @@ void terrain_palette::handle_event(const SDL_Event& event) {
 }
 
 void terrain_palette::draw(bool force) {
-
 	if (top_button_.pressed()) {
 		scroll_up();
 	}
 	if (bot_button_.pressed()) {
 		scroll_down();
 	}
-	for(size_t i = 0; i < terrain_groups_.size(); ++i) {
-		if(terrain_groups_[i].button.pressed()) {
-			if(&terrain_groups_[i].button == checked_group_btn_) {
-				checked_group_btn_->set_check(true);
-			} else {
-				checked_group_btn_->set_check(false);
-				checked_group_btn_ = &terrain_groups_[i].button;
-				set_group(terrain_groups_[i].id);
-			}
+
+	foreach (terrain_group& g, terrain_groups_) {
+		if (g.button.pressed()) {
+			checked_group_btn_ = &g.button;
+			set_group(g.id);
 			break;
 		}
 	}
+
+	foreach (terrain_group& g, terrain_groups_) {
+		if (&g.button == checked_group_btn_) {
+			g.button.set_check(true);
+		} else {
+			g.button.set_check(false);
+		}
+	}
+
 	if (!dirty() && !force) {
 		return;
 	}
@@ -369,13 +413,59 @@ void terrain_palette::draw(bool force) {
 	}
 	const SDL_Rect &loc = location();
 	int y = terrain_start_;
+	SDL_Rect palrect;
+	palrect.x = loc.x;
+	palrect.y = terrain_start_;
+	palrect.w = size_specs_.palette_w;
+	palrect.h = size_specs_.palette_h;
+	tooltips::clear_tooltips(palrect);
 	for(unsigned int counter = starting; counter < ending; counter++){
 		const t_translation::t_terrain terrain = terrains_[counter];
-		const std::string filename = "terrain/" + map_.get_terrain_info(terrain).editor_image() + ".png";
+		const t_translation::t_terrain base_terrain = map().get_terrain_info(terrain).default_base();
+
+		const int counter_from_zero = counter - starting;
+		SDL_Rect dstrect;
+		dstrect.x = loc.x + (counter_from_zero % size_specs_.terrain_width) * size_specs_.terrain_space;
+		dstrect.y = y;
+		dstrect.w = size_specs_.terrain_size;
+		dstrect.h = size_specs_.terrain_size;
+		std::stringstream tooltip_text;
+
+		//Draw default base for overlay terrains
+		if(base_terrain != t_translation::NONE_TERRAIN) {
+			const std::string base_filename = "terrain/" + map().get_terrain_info(base_terrain).editor_image() + ".png";
+			surface base_image(image::get_image(base_filename));
+
+			if(base_image == NULL) {
+				tooltip_text << "BASE IMAGE NOT FOUND\n";
+				ERR_ED << "image for terrain " << counter << ": '" << base_filename << "' not found\n";
+				base_image = image::get_image("misc/missing-image.png");
+				if (base_image == NULL) {
+					ERR_ED << "Placeholder image not found\n";
+					return;
+				}
+			}
+
+			if(static_cast<unsigned>(base_image->w) != size_specs_.terrain_size ||
+			   static_cast<unsigned>(base_image->h) != size_specs_.terrain_size) {
+
+				base_image.assign(scale_surface(base_image,
+				   size_specs_.terrain_size, size_specs_.terrain_size));
+			}
+
+			SDL_BlitSurface(base_image, NULL, screen, &dstrect);
+		}
+
+		const std::string filename = "terrain/" + map().get_terrain_info(terrain).editor_image() + ".png";
 		surface image(image::get_image(filename));
 		if(image == NULL) {
-			std::cerr << "image for terrain " << counter << ": '" << filename << "' not found\n";
-			return;
+			tooltip_text << "IMAGE NOT FOUND\n";
+			ERR_ED << "image for terrain " << counter << ": '" << filename << "' not found\n";
+			image = image::get_image("misc/missing-image.png");
+			if (image == NULL) {
+				ERR_ED << "Placeholder image not found\n";
+				return;
+			}
 		}
 
 		if(static_cast<unsigned>(image->w) != size_specs_.terrain_size ||
@@ -385,14 +475,8 @@ void terrain_palette::draw(bool force) {
 				size_specs_.terrain_size, size_specs_.terrain_size));
 		}
 
-		const int counter_from_zero = counter - starting;
-		SDL_Rect dstrect;
-		dstrect.x = loc.x + (counter_from_zero % size_specs_.terrain_width) * size_specs_.terrain_space;
-		dstrect.y = y;
-		dstrect.w = image->w;
-		dstrect.h = image->h;
-
 		SDL_BlitSurface(image, NULL, screen, &dstrect);
+
 		SDL_Surface* const screen = gui_.video().getSurface();
 		Uint32 color;
 		if (terrain == selected_bg_terrain() && terrain == selected_fg_terrain()) {
@@ -408,6 +492,22 @@ void terrain_palette::draw(bool force) {
 			color = SDL_MapRGB(screen->format,0x00,0x00,0x00);
 		}
 		draw_rectangle(dstrect.x, dstrect.y, image->w, image->h, color, screen);
+
+		bool is_core = non_core_terrains_.find(terrain) == non_core_terrains_.end();
+		SDL_Color tip_color = font::NORMAL_COLOUR;
+		if (!is_core) {
+			SDL_Color red = {0xff, 0, 0, 0};
+			tip_color = red;
+		}
+		tooltip_text << map().get_terrain_string(terrain);
+		if (gui_.get_draw_terrain_codes()) {
+			tooltip_text << " - " << terrain;
+		}
+		if (!is_core) {
+			tooltip_text << " " << _("(non-core)") << "\n";
+			tooltip_text << _("Will not work in game without extra care.");
+		}
+		tooltips::add_tooltip(dstrect, tooltip_text.str(), tip_color);
 		if (counter_from_zero % size_specs_.terrain_width == size_specs_.terrain_width - 1)
 			y += size_specs_.terrain_space;
 	}
@@ -456,33 +556,32 @@ void terrain_palette::load_tooltips()
 //	restorer_.restore();
 // }
 
-brush_bar::brush_bar(display &gui, const size_specs &sizes)
-	: gui::widget(gui.video()), size_specs_(sizes), gui_(gui), selected_(0), total_brush_(3),
-	  size_(30) {
+brush_bar::brush_bar(display &gui, const size_specs &sizes,
+	std::vector<brush>& brushes, brush** the_brush)
+: gui::widget(gui.video()), size_specs_(sizes), gui_(gui),
+selected_(0), brushes_(brushes), the_brush_(the_brush),
+size_(30) {
 	adjust_size();
 }
 
 void brush_bar::adjust_size() {// TODO
 	set_location(size_specs_.brush_x, size_specs_.brush_y);
-	set_measurements(size_ * total_brush_ + (total_brush_ - 1) * size_specs_.brush_padding, size_);
+	set_measurements(size_ * brushes_.size() + (brushes_.size() - 1) * size_specs_.brush_padding, size_);
 	set_dirty();
 }
 
 unsigned int brush_bar::selected_brush_size() {
-	return selected_ + 1;
-}
-
-void brush_bar::select_brush_size(int new_size) {
-	assert(new_size > 0 && new_size <= total_brush_);
-	selected_ = new_size - 1;
+	return selected_;
 }
 
 void brush_bar::left_mouse_click(const int mousex, const int mousey) {
+	sound::play_UI_sound(game_config::sounds::button_press);
 	int index = selected_index(mousex, mousey);
 	if(index >= 0) {
-		if (static_cast<unsigned>(index) != selected_) {
+		if (static_cast<size_t>(index) != selected_) {
 			set_dirty();
 			selected_ = index;
+			*the_brush_ = &brushes_[index];
 		}
 	}
 }
@@ -528,26 +627,20 @@ void brush_bar::draw(bool force) {
 	// Everything will be redrawn even though only one little part may
 	// have changed, but that happens so seldom so we'll settle with this.
 	SDL_Surface* const screen = gui_.video().getSurface();
-	for (int i = 1; i <= total_brush_; i++) {
-		std::stringstream filename;
-		filename << "editor/brush-" << i << ".png";
-		surface image(image::get_image(filename.str()));
+	for (size_t i = 0; i < brushes_.size(); i++) {
+		std::string filename = brushes_[i].image();
+		surface image(image::get_image(filename));
 		if (image == NULL) {
-			std::cerr << "Image " << filename.str() << " not found." << std::endl;
+			ERR_ED << "Image " << filename << " not found." << std::endl;
 			continue;
 		}
-		if (static_cast<unsigned>(image->w) != size_ ||
-				static_cast<unsigned>(image->h) != size_) {
-
+		if (static_cast<unsigned>(image->w) != size_
+		|| static_cast<unsigned>(image->h) != size_) {
 			image.assign(scale_surface(image, size_, size_));
 		}
-		SDL_Rect dstrect;
-		dstrect.x = x;
-		dstrect.y = size_specs_.brush_y;
-		dstrect.w = image->w;
-		dstrect.h = image->h;
+		SDL_Rect dstrect = {x, size_specs_.brush_y, image->w, image->h};
 		SDL_BlitSurface(image, NULL, screen, &dstrect);
-		const Uint32 color = static_cast<unsigned>(i) == selected_brush_size() ?
+		const Uint32 color = i == selected_brush_size() ?
 			SDL_MapRGB(screen->format,0xFF,0x00,0x00) :
 			SDL_MapRGB(screen->format,0x00,0x00,0x00);
 		draw_rectangle(dstrect.x, dstrect.y, image->w, image->h, color, screen);
@@ -557,28 +650,25 @@ void brush_bar::draw(bool force) {
 	set_dirty(false);
 }
 
-int brush_bar::selected_index(const int x, const int y) const {
+int brush_bar::selected_index(int x, int y) const {
 	const int bar_x = size_specs_.brush_x;
 	const int bar_y = size_specs_.brush_y;
 
-	if ((x < bar_x || static_cast<unsigned>(x) > bar_x + size_ * total_brush_ +
-			total_brush_ * size_specs_.brush_padding) ||
-		    (y < bar_y || static_cast<unsigned>(y) > bar_y + size_)) {
+	if ((x < bar_x || static_cast<size_t>(x) > bar_x + size_ * brushes_.size() +
+	                  brushes_.size() * size_specs_.brush_padding) ||
+	    (y < bar_y || static_cast<size_t>(y) > bar_y + size_)) {
 
 		return -1;
 	}
 
-	for(int i = 0; i < total_brush_; i++) {
-		const int px = bar_x + size_ * i + i * size_specs_.brush_padding;
-
-		if(x >= px && static_cast<unsigned>(x) <= px + size_ &&
-				y >= bar_y && static_cast<unsigned>(y) <= bar_y + size_) {
-
+	for(size_t i = 0; i <  brushes_.size(); i++) {
+		int px = bar_x + size_ * i + i * size_specs_.brush_padding;
+		if (x >= px && x <= px + static_cast<int>(size_) && y >= bar_y && y <= bar_y + static_cast<int>(size_)) {
 			return i;
 		}
 	}
 	return -1;
 }
 
-} // end namespace map_editor
+} // end namespace editor
 
