@@ -1,6 +1,6 @@
-/* $Id: events.cpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: events.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -30,10 +30,7 @@
 #include <utility>
 #include <vector>
 
-#define ERR_GEN LOG_STREAM(err, general)
-#define INFO_GEN LOG_STREAM(info, general)
-
-unsigned input_blocker::instance_count = 0; //static initialization
+#define ERR_GEN LOG_STREAM(err, lg::general)
 
 namespace events
 {
@@ -194,7 +191,7 @@ void handler::join()
 	//instruct members to join
 	handler_vector members = handler_members();
 	if(!members.empty()) {
-		for(handler_vector::iterator i = members.begin(); i != members.end(); i++) {
+		for(handler_vector::iterator i = members.begin(); i != members.end(); ++i) {
 			(*i)->join();
 		}
 	}
@@ -204,7 +201,7 @@ void handler::leave()
 {
 	handler_vector members = handler_members();
 	if(!members.empty()) {
-		for(handler_vector::iterator i = members.begin(); i != members.end(); i++) {
+		for(handler_vector::iterator i = members.begin(); i != members.end(); ++i) {
 			(*i)->leave();
 		}
 	} else {
@@ -491,28 +488,3 @@ int pump_info::ticks(unsigned *refresh_counter, unsigned refresh_rate) {
 }
 
 } //end events namespace
-
-input_blocker::input_blocker()
-{
-	SDL_EventState(SDL_KEYDOWN, SDL_IGNORE);
-	SDL_EventState(SDL_KEYUP, SDL_IGNORE);
-	SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_IGNORE);
-	SDL_EventState(SDL_MOUSEBUTTONUP, SDL_IGNORE);
-	SDL_EventState(SDL_JOYBUTTONDOWN, SDL_IGNORE);
-	SDL_EventState(SDL_JOYBUTTONUP, SDL_IGNORE);
-	instance_count++;
-}
-
-input_blocker::~input_blocker()
-{
-	instance_count--;
-	if(instance_count == 0) {
-		events::discard(INPUT_MASK);
-		SDL_EventState(SDL_KEYDOWN, SDL_ENABLE);
-		SDL_EventState(SDL_KEYUP, SDL_ENABLE);
-		SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_ENABLE);
-		SDL_EventState(SDL_MOUSEBUTTONUP, SDL_ENABLE);
-		SDL_EventState(SDL_JOYBUTTONDOWN, SDL_ENABLE);
-		SDL_EventState(SDL_JOYBUTTONUP, SDL_ENABLE);
-	}
-}
