@@ -1,6 +1,6 @@
-/* $Id: filesystem.cpp 34025 2009-03-22 12:39:12Z mordante $ */
+/* $Id: filesystem.cpp 44825 2010-07-28 20:25:31Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -57,16 +57,25 @@ BPath be_path;
 #include "filesystem.hpp"
 #include "foreach.hpp"
 #include "game_config.hpp"
+#include "game_preferences.hpp"
 #include "log.hpp"
 #include "loadscreen.hpp"
+#include "scoped_resource.hpp"
 
-#define DBG_FS LOG_STREAM(debug, filesystem)
-#define LOG_FS LOG_STREAM(info, filesystem)
-#define WRN_FS LOG_STREAM(warn, filesystem)
-#define ERR_FS LOG_STREAM(err, filesystem)
+static lg::log_domain log_filesystem("filesystem");
+#define DBG_FS LOG_STREAM(debug, log_filesystem)
+#define LOG_FS LOG_STREAM(info, log_filesystem)
+#define WRN_FS LOG_STREAM(warn, log_filesystem)
+#define ERR_FS LOG_STREAM(err, log_filesystem)
 
 namespace {
 	const mode_t AccessMode = 00770;
+
+	// These are the filenames that get special processing
+	const std::string maincfg_filename = "_main.cfg";
+	const std::string finalcfg_filename = "_final.cfg";
+	const std::string initialcfg_filename = "_initial.cfg";
+
 }
 
 #ifdef __APPLE__
@@ -80,14 +89,7 @@ bool ends_with(const std::string& str, const std::string& suffix)
 	return str.size() >= suffix.size() && std::equal(suffix.begin(),suffix.end(),str.end()-suffix.size());
 }
 
-// These are the filenames that get special processing
-#define MAINCFG 	"_main.cfg"
-#define FINALCFG	"_final.cfg"
-#define INITIALCFG	"_initial.cfg"
-
-// Don't pass directory as reference, it seems to break on
-// arklinux with GCC-4.3.
-void get_files_in_dir(const std::string directory,
+void get_files_in_dir(const std::string &directory,
 					  std::vector<std::string>* files,
 					  std::vector<std::string>* dirs,
 					  file_name_option mode,
@@ -100,7 +102,7 @@ void get_files_in_dir(const std::string directory,
 	// on the wesnoth path
 #ifndef __AMIGAOS4__
 	if(!directory.empty() && directory[0] != '/' && !game_config::path.empty()){
-		const std::string& dir = game_config::path + "/" + directory;
+		std::string dir = game_config::path + "/" + directory;
 		if(is_directory(dir)) {
 			get_files_in_dir(dir,files,dirs,mode,filter,reorder,checksum);
 			return;
@@ -118,9 +120,9 @@ void get_files_in_dir(const std::string directory,
 			|| (directory[directory.size()-1]==':')
 #endif /* __AMIGAOS4__ */
 		)
-			maincfg = directory + MAINCFG;
+			maincfg = directory + maincfg_filename;
 		else
-			maincfg = (directory + "/") + MAINCFG;
+			maincfg = (directory + "/") + maincfg_filename;
 
 		if (::stat(maincfg.c_str(), &st) != -1) {
 			LOG_FS << "_main.cfg found : " << maincfg << '\n';
@@ -128,7 +130,7 @@ void get_files_in_dir(const std::string directory,
 				if (mode == ENTIRE_FILE_PATH)
 					files->push_back(maincfg);
 				else
-					files->push_back(MAINCFG);
+					files->push_back(maincfg_filename);
 			}
 			return;
 		}
@@ -196,20 +198,20 @@ void get_files_in_dir(const std::string directory,
 					continue;
 
 				if (reorder == DO_REORDER &&
-						::stat((fullname+"/"+MAINCFG).c_str(), &st)!=-1 &&
+						::stat((fullname+"/"+maincfg_filename).c_str(), &st)!=-1 &&
 						S_ISREG(st.st_mode)) {
 					LOG_FS << "_main.cfg found : ";
 					if (files != NULL) {
 						if (mode == ENTIRE_FILE_PATH) {
-							files->push_back(fullname + "/" + MAINCFG);
-							LOG_FS << fullname << "/" << MAINCFG << '\n';
+							files->push_back(fullname + "/" + maincfg_filename);
+							LOG_FS << fullname << "/" << maincfg_filename << '\n';
 						} else {
-							files->push_back(basename + "/" + MAINCFG);
-							LOG_FS << basename << "/" << MAINCFG << '\n';
+							files->push_back(basename + "/" + maincfg_filename);
+							LOG_FS << basename << "/" << maincfg_filename << '\n';
 					}
 					} else {
 					// Show what I consider strange
-						LOG_FS << fullname << "/" << MAINCFG << " not used now but skip the directory \n";
+						LOG_FS << fullname << "/" << maincfg_filename << " not used now but skip the directory \n";
 					}
 				} else if (dirs != NULL) {
 					if (mode == ENTIRE_FILE_PATH)
@@ -230,18 +232,18 @@ void get_files_in_dir(const std::string directory,
 		std::sort(dirs->begin(),dirs->end());
 
 	if (files != NULL && reorder == DO_REORDER) {
-		// move FINALCFG, if present, to the end of the vector
+		// move finalcfg_filename, if present, to the end of the vector
 		for (unsigned int i = 0; i < files->size(); i++) {
-			if (ends_with((*files)[i], "/" FINALCFG)) {
+			if (ends_with((*files)[i], "/" + finalcfg_filename)) {
 				files->push_back((*files)[i]);
 				files->erase(files->begin()+i);
 				break;
 			}
 		}
-		// move INITIALCFG, if present, to the beginning of the vector
+		// move initialcfg_filename, if present, to the beginning of the vector
 		int foundit = -1;
 		for (unsigned int i = 0; i < files->size(); i++)
-			if (ends_with((*files)[i], "/" INITIALCFG)) {
+			if (ends_with((*files)[i], "/" + initialcfg_filename)) {
 				foundit = i;
 				break;
 			}
@@ -278,19 +280,10 @@ std::string get_cache_dir()
 
 std::string get_addon_campaigns_dir()
 {
-	const std::string dir_path = get_user_data_dir() + "/data/campaigns";
+	const std::string dir_path = get_user_data_dir() + "/data/add-ons";
 	return get_dir(dir_path);
 }
 
-std::vector< std::string > get_addon_dirs()
-{
-	std::vector< std::string > ret;
-
-	ret.push_back(get_addon_campaigns_dir());
-
-	return ret;
-}
-
 std::string get_intl_dir()
 {
 #ifdef _WIN32
@@ -531,12 +524,7 @@ static std::string setup_user_data_dir()
 	_mkdir((game_config::preferences_dir + "/editor").c_str());
 	_mkdir((game_config::preferences_dir + "/editor/maps").c_str());
 	_mkdir((game_config::preferences_dir + "/data").c_str());
-	_mkdir((game_config::preferences_dir + "/data/ais").c_str());
-	_mkdir((game_config::preferences_dir + "/data/campaigns").c_str());
-	_mkdir((game_config::preferences_dir + "/data/multiplayer").c_str());
-	_mkdir((game_config::preferences_dir + "/data/maps").c_str());
-	_mkdir((game_config::preferences_dir + "/data/maps/multiplayer").c_str());
-	_mkdir((game_config::preferences_dir + "/data/units").c_str());
+	_mkdir((game_config::preferences_dir + "/data/add-ons").c_str());
 	_mkdir((game_config::preferences_dir + "/saves").c_str());
 
 	return game_config::preferences_dir;
@@ -556,12 +544,7 @@ static std::string setup_user_data_dir()
 		BEOS_CREATE_PREFERENCES_SUBDIR("editor");
 		BEOS_CREATE_PREFERENCES_SUBDIR("editor/maps");
 		BEOS_CREATE_PREFERENCES_SUBDIR("data");
-		BEOS_CREATE_PREFERENCES_SUBDIR("data/ais");
-		BEOS_CREATE_PREFERENCES_SUBDIR("data/campaigns");
-		BEOS_CREATE_PREFERENCES_SUBDIR("data/multiplayer");
-		BEOS_CREATE_PREFERENCES_SUBDIR("data/maps");
-		BEOS_CREATE_PREFERENCES_SUBDIR("data/maps/multiplayer");
-		BEOS_CREATE_PREFERENCES_SUBDIR("data/units");
+		BEOS_CREATE_PREFERENCES_SUBDIR("data/add-ons");
 		BEOS_CREATE_PREFERENCES_SUBDIR("saves");
 	#undef BEOS_CREATE_PREFERENCES_SUBDIR
 	}
@@ -582,12 +565,7 @@ static std::string setup_user_data_dir()
 	create_directory_if_missing(dir_path + "/editor");
 	create_directory_if_missing(dir_path + "/editor/maps");
 	create_directory_if_missing(dir_path + "/data");
-	create_directory_if_missing(dir_path + "/data/ais");
-	create_directory_if_missing(dir_path + "/data/campaigns");
-	create_directory_if_missing(dir_path + "/data/multiplayer");
-	create_directory_if_missing(dir_path + "/data/maps");
-	create_directory_if_missing(dir_path + "/data/maps/multiplayer");
-	create_directory_if_missing(dir_path + "/data/units");
+	create_directory_if_missing(dir_path + "/data/add-ons");
 	create_directory_if_missing(dir_path + "/saves");
 
 	return dir_path;
@@ -926,9 +904,9 @@ void binary_paths_manager::set_paths(const config& cfg)
 	cleanup();
 	init_binary_paths();
 
-	foreach (const config *item, cfg.get_children("binary_path"))
+	foreach (const config &bp, cfg.child_range("binary_path"))
 	{
-		std::string path = (*item)["path"].str();
+		std::string path = bp["path"].str();
 		if (path.find("..") != std::string::npos) {
 			ERR_FS << "Invalid binary path '" << path << "'\n";
 			continue;
@@ -983,7 +961,7 @@ const std::vector<std::string>& get_binary_paths(const std::string& type)
 	}
 
 	// not found in "/type" directory, try main directory
-	res.push_back(get_user_data_dir());
+	res.push_back(get_user_data_dir() + "/");
 
 	if(!game_config::path.empty())
 		res.push_back(game_config::path+"/");
@@ -1005,7 +983,7 @@ std::string get_binary_file_location(const std::string& type, const std::string&
 	std::string::size_type pos = filename.rfind("../");
 	if (pos != std::string::npos) {
 		std::string nf = filename.substr(pos + 3);
-		WRN_FS << "Illegal path '" << filename << "' replaced by '" << nf << "'\n";
+		LOG_FS << "Illegal path '" << filename << "' replaced by '" << nf << "'\n";
 		return get_binary_file_location(type, nf);
 	}
 
@@ -1074,17 +1052,13 @@ std::string get_wml_location(const std::string &filename, const std::string &cur
 
 	bool already_found = false;
 
-	if (filename[0] == '~' || filename[0] == '@')
+	if (filename[0] == '~')
 	{
-		// If the filename starts with '~' or '@', look in the user data directory.
+		// If the filename starts with '~', look in the user data directory.
 		result = get_user_data_dir() + "/data/" + filename.substr(1);
 		DBG_FS << "  trying '" << result << "'\n";
 
 		already_found = file_exists(result) || is_directory(result);
-
-		// If it starts with '@', look also in the standard data directory.
-		if (filename[0] == '@' && !already_found && !game_config::path.empty())
-			result = game_config::path + "/data/" + filename.substr(1);
 	}
 	else if (filename.size() >= 2 && filename[0] == '.' && filename[1] == '/')
 	{
@@ -1109,6 +1083,43 @@ std::string get_wml_location(const std::string &filename, const std::string &cur
 	return result;
 }
 
+std::string get_short_wml_path(const std::string &filename)
+{
+	std::string match = get_user_data_dir() + "/data/";
+	if (filename.find(match) == 0) {
+		return "~" + filename.substr(match.size());
+	}
+	match = game_config::path + "/data/";
+	if (filename.find(match) == 0) {
+		return filename.substr(match.size());
+	}
+	return filename;
+}
+
+std::string get_program_invocation(const std::string& program_name) {
+#ifdef DEBUG
+  #ifdef _WIN32
+	const char *program_suffix = "-debug.exe";
+  #else
+	const char *program_suffix = "-debug";
+  #endif
+#else
+  #ifdef _WIN32
+	const char *program_suffix = ".exe";
+  #else
+	const char *program_suffix = "";
+  #endif
+#endif
+
+	const std::string real_program_name(program_name + program_suffix);
+	if(game_config::wesnoth_program_dir.empty()) return real_program_name;
+#ifdef _WIN32
+	return game_config::wesnoth_program_dir + "\\" + real_program_name;
+#else
+	return game_config::wesnoth_program_dir + "/" + real_program_name;
+#endif
+}
+
 static bool is_path_sep(char c)
 {
 #ifdef _WIN32
@@ -1147,7 +1158,7 @@ std::string normalize_path(const std::string &p1)
 	for (int i = 0, i_end = p2.size(); i <= i_end; ++i)
 	{
 		std::string &last = components[components.size() - 1];
-		char c = p2[i];
+		char c = p2.c_str()[i];
 		if (is_path_sep(c) || c == 0)
 		{
 			if (last == ".")
