@@ -1,6 +1,6 @@
-/* $Id: filesystem.hpp 33329 2009-03-03 22:43:55Z silene $ */
+/* $Id: filesystem.hpp 44825 2010-07-28 20:25:31Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -53,7 +53,7 @@ enum file_reorder_option { DONT_REORDER, DO_REORDER };
  * reorder: triggers the special handling of _main.cfg and _final.cfg
  * checksum: can be used to store checksum info
  */
-void get_files_in_dir(const std::string dir,
+void get_files_in_dir(const std::string &dir,
                       std::vector<std::string>* files,
                       std::vector<std::string>* dirs=NULL,
                       file_name_option mode = FILE_NAME_ONLY,
@@ -72,12 +72,6 @@ std::string get_intl_dir();
 std::string get_screenshot_dir();
 std::string get_addon_campaigns_dir();
 
-/**
- * Wrapper around get_mp_addons_dir() and get_addon_campaigns_dir() intended
- * for massive operations with add-ons, e.g. enumerating for publishing.
- */
-std::vector< std::string > get_addon_dirs();
-
 /**
  * Get the next free filename using "name + number (3 digits) + extension"
  * maximum 1000 files then start always giving 999
@@ -226,6 +220,20 @@ std::string get_binary_dir_location(const std::string &type, const std::string &
 std::string get_wml_location(const std::string &filename,
 	const std::string &current_dir = std::string());
 
+/**
+ * Returns a short path to @a filename, skipping the (user) data directory.
+ */
+std::string get_short_wml_path(const std::string &filename);
+
+/**
+ * Returns the appropriate invocation for a Wesnoth-related binary, assuming
+ * that it is located in the same directory as the running wesnoth binary.
+ * This is just a string-transformation based on argv[0], so the returned
+ * program is not guaranteed to actaully exist.  '-debug' variants are handled
+ * correctly.
+ */
+std::string get_program_invocation(const std::string &program_name);
+
 class scoped_istream {
 	std::istream *stream;
 public:
