@@ -1,6 +1,6 @@
-/* $Id: floating_textbox.cpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: floating_textbox.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -19,9 +19,13 @@
 #include "game_display.hpp"
 #include "game_preferences.hpp"
 #include "log.hpp"
+#include "widgets/textbox.hpp"
 
 #include <ctime>
 
+static lg::log_domain log_display("display");
+#define ERR_DP LOG_STREAM(err, log_display)
+
 namespace gui{
 	floating_textbox::floating_textbox() :
 		box_(NULL),
@@ -152,13 +156,13 @@ namespace gui{
 			} else {
 				std::string completion_list = utils::join(matches, ' ');
 				gui.add_chat_message(time(NULL), "", 0, completion_list,
-						game_display::MESSAGE_PRIVATE, false);
+						events::chat_handler::MESSAGE_PRIVATE, false);
 			}
 			box_->set_text(text);
 			break;
 		}
 		default:
-			LOG_STREAM(err, display) << "unknown textbox mode\n";
+			ERR_DP << "unknown textbox mode\n";
 		}
 	}
 }
