@@ -1,6 +1,6 @@
-/* $Id: floating_textbox.hpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: floating_textbox.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -19,7 +19,9 @@
 #include "global.hpp"
 
 #include "scoped_resource.hpp"
-#include "widgets/textbox.hpp"
+
+#include <string>
+#include <vector>
 
 class game_display;
 class team;
@@ -27,6 +29,9 @@ class unit_map;
 
 namespace gui{
 
+	class textbox;
+	class button;
+
 	enum TEXTBOX_MODE { TEXTBOX_NONE, TEXTBOX_SEARCH, TEXTBOX_MESSAGE,
 		        TEXTBOX_COMMAND, TEXTBOX_AI };
 
