@@ -1,7 +1,7 @@
-/* $Id: font.cpp 37708 2009-08-13 10:03:28Z loonycyborg $ */
+/* $Id: font.cpp 46300 2010-09-07 07:50:59Z silene $ */
 /* vim:set encoding=utf-8: */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -20,14 +20,16 @@
 #include "config.hpp"
 #include "filesystem.hpp"
 #include "font.hpp"
+#include "foreach.hpp"
 #include "game_config.hpp"
 #include "log.hpp"
+#include "marked-up_text.hpp"
+#include "text.hpp"
 #include "tooltips.hpp"
 #include "video.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
-#include "marked-up_text.hpp"
-#include "foreach.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <list>
 #include <set>
@@ -35,28 +37,25 @@
 
 #include <cairo-features.h>
 
+#ifdef CAIRO_HAS_WIN32_FONT
+#include <windows.h>
+#undef CAIRO_HAS_FT_FONT
+#endif
+
 #ifdef CAIRO_HAS_FT_FONT
 #include <fontconfig/fontconfig.h>
 #endif
 
-#ifdef CAIRO_HAS_WIN32_FONT
-#include <windows.h>
-#endif
-
-#define DBG_FT LOG_STREAM(debug, display)
-#define LOG_FT LOG_STREAM(info, display)
-#define WRN_FT LOG_STREAM(warn, display)
-#define ERR_FT LOG_STREAM(err, display)
+static lg::log_domain log_font("font");
+#define DBG_FT LOG_STREAM(debug, log_font)
+#define LOG_FT LOG_STREAM(info, log_font)
+#define WRN_FT LOG_STREAM(warn, log_font)
+#define ERR_FT LOG_STREAM(err, log_font)
 
 #ifdef	HAVE_FRIBIDI
-#include <fribidi/fribidi.h>
-
-#else
-
+#include <fribidi.h>
 #endif
 
-namespace {
-
 // Signed int. Negative values mean "no subset".
 typedef int subset_id;
 
@@ -76,57 +75,96 @@ struct font_id
 	int size;
 };
 
-std::map<font_id, TTF_Font*> font_table;
-std::vector<std::string> font_names;
+static std::map<font_id, TTF_Font*> font_table;
+static std::vector<std::string> font_names;
 
 struct text_chunk
 {
 	text_chunk(subset_id subset) :
 		subset(subset),
-		text(),
-		ucs2_text()
+		text()
 	{
 	}
 
-	text_chunk(subset_id subset, std::string const & text) :
-		subset(subset),
-		text(text),
-		ucs2_text()
-	{
-	}
-
-	text_chunk(subset_id subset, ucs2_string const & ucs2_text) :
-		subset(subset),
-		text(),
-		ucs2_text(ucs2_text)
-	{
-	}
-
-	text_chunk(subset_id subset, std::string const & text, ucs2_string const & ucs2_text) :
-		subset(subset),
-		text(text),
-		ucs2_text(ucs2_text)
-	{
-	}
-
-	bool operator==(text_chunk const & t) const { return subset == t.subset && ucs2_text == t.ucs2_text; }
+	bool operator==(text_chunk const & t) const { return subset == t.subset && text == t.text; }
 	bool operator!=(text_chunk const & t) const { return !operator==(t); }
 
 	subset_id subset;
-	//FIXME if we don't need the utf8 here remove it
 	std::string text;
-	ucs2_string ucs2_text;
 };
 
-std::vector<subset_id> font_map;
+struct char_block_map
+{
+	char_block_map()
+		: cbmap()
+	{
+	}
+
+	typedef std::pair<int, subset_id> block_t;
+	typedef std::map<int, block_t> cbmap_t;
+	cbmap_t cbmap;
+	/** Associates not-associated parts of a range with a new font. */
+	void insert(int first, int last, subset_id id)
+	{
+		if (first > last) return;
+		cbmap_t::iterator i = cbmap.lower_bound(first);
+		// At this point, either first <= i->first or i is past the end.
+		if (i != cbmap.begin()) {
+			cbmap_t::iterator j = i;
+			--j;
+			if (first <= j->second.first /* prev.last */) {
+				insert(j->second.first + 1, last, id);
+				return;
+			}
+		}
+		if (i != cbmap.end()) {
+			if (/* next.first */ i->first <= last) {
+				insert(first, i->first - 1, id);
+				return;
+			}
+		}
+		cbmap.insert(std::make_pair(first, block_t(last, id)));
+	}
+	/**
+	 * Compresses map by merging consecutive ranges with the same font, even
+	 * if there is some unassociated ranges inbetween.
+	 */
+	void compress()
+	{
+		LOG_FT << "Font map size before compression: " << cbmap.size() << " ranges\n";
+		cbmap_t::iterator i = cbmap.begin(), e = cbmap.end();
+		while (i != e) {
+			cbmap_t::iterator j = i;
+			++j;
+			if (j == e || i->second.second != j->second.second) {
+				i = j;
+				continue;
+			}
+			i->second.first = j->second.first;
+			cbmap.erase(j);
+		}
+		LOG_FT << "Font map size after compression: " << cbmap.size() << " ranges\n";
+	}
+	subset_id get_id(int ch)
+	{
+		cbmap_t::iterator i = cbmap.upper_bound(ch);
+		// At this point, either ch < i->first or i is past the end.
+		if (i != cbmap.begin()) {
+			--i;
+			if (ch <= i->second.first /* prev.last */)
+				return i->second.second;
+		}
+		return -1;
+	}
+};
+
+static char_block_map char_blocks;
 
 //cache sizes of small text
 typedef std::map<std::string,SDL_Rect> line_size_cache_map;
 
 //map of styles -> sizes -> cache
-std::map<int,std::map<int,line_size_cache_map> > line_size_cache;
-
-}
+static std::map<int,std::map<int,line_size_cache_map> > line_size_cache;
 
 //Splits the UTF-8 text into text_chunks using the same font.
 static std::vector<text_chunk> split_text(std::string const & utf8_text) {
@@ -138,21 +176,16 @@ static std::vector<text_chunk> split_text(std::string const & utf8_text) {
 
 	try {
 		utils::utf8_iterator ch(utf8_text);
-		if(size_t(*ch) < font_map.size() && font_map[size_t(*ch)] >= 0) {
-			current_chunk.subset = font_map[size_t(*ch)];
-		}
-		for(utils::utf8_iterator end = utils::utf8_iterator::end(utf8_text); ch != end; ++ch) {
-			if(size_t(*ch) < font_map.size() &&
-					font_map[size_t(*ch)] >= 0 &&
-					font_map[size_t(*ch)] != current_chunk.subset) {
-				//null-terminate ucs2_text so we can pass it to SDL_ttf later
-				current_chunk.ucs2_text.push_back(0);
+		int sub = char_blocks.get_id(*ch);
+		if (sub >= 0) current_chunk.subset = sub;
+		for(utils::utf8_iterator end = utils::utf8_iterator::end(utf8_text); ch != end; ++ch)
+		{
+			sub = char_blocks.get_id(*ch);
+			if (sub >= 0 && sub != current_chunk.subset) {
 				chunks.push_back(current_chunk);
-				current_chunk.text = "";
-				current_chunk.ucs2_text.clear();
-				current_chunk.subset = font_map[size_t(*ch)];
+				current_chunk.text.clear();
+				current_chunk.subset = sub;
 			}
-			current_chunk.ucs2_text.push_back(static_cast<Uint16>(*ch));
 			current_chunk.text.append(ch.substr().first, ch.substr().second);
 		}
 		if (!current_chunk.text.empty()) {
@@ -230,7 +263,7 @@ static void clear_fonts()
 
 	font_table.clear();
 	font_names.clear();
-	font_map.clear();
+	char_blocks.cbmap.clear();
 	line_size_cache.clear();
 }
 
@@ -312,11 +345,10 @@ void manager::update_font_path() const
 void manager::init() const
 {
 #ifdef CAIRO_HAS_FT_FONT
-	if(!FcConfigAppFontAddDir( FcConfigGetCurrent(),
-			reinterpret_cast<const FcChar8*>
-			((game_config::path + "/fonts/").c_str()))) {
-
-		ERR_FT << "Could not load the  true type fonts\n";
+	if (!FcConfigAppFontAddDir(FcConfigGetCurrent(),
+		reinterpret_cast<const FcChar8 *>((game_config::path + "/fonts").c_str())))
+	{
+		ERR_FT << "Could not load the true type fonts\n";
 		throw error();
 	}
 #endif
@@ -360,14 +392,14 @@ struct subset_descriptor
 	}
 
 	std::string name;
-	std::vector<std::pair<size_t, size_t> > present_codepoints;
+	typedef std::pair<int, int> range;
+	std::vector<range> present_codepoints;
 };
 
 //sets the font list to be used.
 static void set_font_list(const std::vector<subset_descriptor>& fontlist)
 {
 	clear_fonts();
-	font_map.reserve(0x10000);
 
 	std::vector<subset_descriptor>::const_iterator itor;
 	for(itor = fontlist.begin(); itor != fontlist.end(); ++itor) {
@@ -392,20 +424,11 @@ static void set_font_list(const std::vector<subset_descriptor>& fontlist)
 		const subset_id subset = font_names.size();
 		font_names.push_back(itor->name);
 
-		std::vector<std::pair<size_t,size_t> >::const_iterator cp_range;
-		for(cp_range = itor->present_codepoints.begin();
-				cp_range != itor->present_codepoints.end(); ++cp_range) {
-
-			size_t cp_max = std::max<size_t>(cp_range->first, cp_range->second);
-			if(cp_max >= font_map.size()) {
-				font_map.resize(cp_max+1, -1);
-			}
-			for(size_t cp = cp_range->first; cp <= cp_range->second; ++cp) {
-				if(font_map[cp] < 0)
-					font_map[cp] = subset;
-			}
+		foreach (const subset_descriptor::range &cp_range, itor->present_codepoints) {
+			char_blocks.insert(cp_range.first, cp_range.second, subset);
 		}
 	}
+	char_blocks.compress();
 }
 
 const SDL_Color NORMAL_COLOUR = {0xDD,0xDD,0xDD,0},
@@ -416,11 +439,11 @@ const SDL_Color NORMAL_COLOUR = {0xDD,0xDD,0xDD,0},
                 BLACK_COLOUR  = {0x00,0x00,0x00,0},
                 YELLOW_COLOUR = {0xFF,0xFF,0x00,0},
                 BUTTON_COLOUR = {0xBC,0xB0,0x88,0},
-                STONED_COLOUR = {0xA0,0xA0,0xA0,0},
+                PETRIFIED_COLOUR = {0xA0,0xA0,0xA0,0},
                 TITLE_COLOUR  = {0xBC,0xB0,0x88,0},
 				LABEL_COLOUR  = {0x6B,0x8C,0xFF,0},
 				BIGMAP_COLOUR = {0xFF,0xFF,0xFF,0};
-const SDL_Color DISABLED_COLOUR = inverse(STONED_COLOUR);
+const SDL_Color DISABLED_COLOUR = inverse(PETRIFIED_COLOUR);
 
 namespace {
 
@@ -467,16 +490,25 @@ private:
 void text_surface::bidi_cvt()
 {
 	char		*c_str = const_cast<char *>(str_.c_str());	// fribidi forgot const...
-	int		len = str_.length();
+	FriBidiStrIndex	len = str_.length();
 	FriBidiChar	*bidi_logical = new FriBidiChar[len + 2];
 	FriBidiChar	*bidi_visual = new FriBidiChar[len + 2];
 	char		*utf8str = new char[4*len + 1];	//assume worst case here (all 4 Byte characters)
 	FriBidiCharType	base_dir = FRIBIDI_TYPE_ON;
-	int n;
+	FriBidiStrIndex n;
 
+
+#ifdef	OLD_FRIBIDI
 	n = fribidi_utf8_to_unicode (c_str, len, bidi_logical);
+#else
+	n = fribidi_charset_to_unicode(FRIBIDI_CHAR_SET_UTF8, c_str, len, bidi_logical);
+#endif
 	fribidi_log2vis(bidi_logical, n, &base_dir, bidi_visual, NULL, NULL, NULL);
+#ifdef	OLD_FRIBIDI
 	fribidi_unicode_to_utf8 (bidi_visual, n, utf8str);
+#else
+	fribidi_unicode_to_charset(FRIBIDI_CHAR_SET_UTF8, bidi_visual, n, utf8str);
+#endif
 	is_rtl_ = base_dir == FRIBIDI_TYPE_RTL;
 	str_ = std::string(utf8str);
 	delete[] bidi_logical;
@@ -549,23 +581,15 @@ void text_surface::measure() const
 	w_ = 0;
 	h_ = 0;
 
-	for(std::vector<text_chunk>::iterator itor = chunks_.begin();
-			itor != chunks_.end(); ++itor) {
-
-		TTF_Font* ttfont = get_font(font_id(itor->subset, font_size_));
+	foreach (text_chunk const &chunk, chunks_)
+	{
+		TTF_Font* ttfont = get_font(font_id(chunk.subset, font_size_));
 		if(ttfont == NULL)
 			continue;
 		font_style_setter const style_setter(ttfont, style_);
 
-		int w;
-		int h;
-
-		if(itor->ucs2_text.back() != 0) {
-			itor->ucs2_text.push_back(0);
-		}
-
-		TTF_SizeUNICODE(ttfont,
-			static_cast<Uint16 const *>(&(itor->ucs2_text.front())), &w, &h);
+		int w, h;
+		TTF_SizeUTF8(ttfont, chunk.text.c_str(), &w, &h);
 		w_ += w;
 		h_ = std::max<int>(h_, h);
 	}
@@ -603,15 +627,14 @@ std::vector<surface> const &text_surface::get_surfaces() const
 	if(width() > max_text_line_width)
 		return surfs_;
 
-	for(std::vector<text_chunk>::const_iterator itor = chunks_.begin();
-			itor != chunks_.end(); ++itor) {
-		TTF_Font* ttfont = get_font(font_id(itor->subset, font_size_));
+	foreach (text_chunk const &chunk, chunks_)
+	{
+		TTF_Font* ttfont = get_font(font_id(chunk.subset, font_size_));
 		if (ttfont == NULL)
 			continue;
 		font_style_setter const style_setter(ttfont, style_);
 
-		surface s = surface(TTF_RenderUNICODE_Blended(ttfont,
-			static_cast<Uint16 const *>(&(itor->ucs2_text.front())), color_));
+		surface s = surface(TTF_RenderUTF8_Blended(ttfont, chunk.text.c_str(), color_));
 		if(!s.null())
 			surfs_.push_back(s);
 	}
@@ -853,16 +876,11 @@ SDL_Rect line_size(const std::string& line, int font_size, int style)
 }
 
 std::string make_text_ellipsis(const std::string &text, int font_size,
-		int max_width, bool with_tags, bool parse_for_style)
+	int max_width, int style)
 {
 	static const std::string ellipsis = "...";
 
-	SDL_Color unused_color;
-	int unused_int;
-	int style = TTF_STYLE_NORMAL;
-	if(parse_for_style) parse_markup(text.begin(), text.end(), &unused_int, &unused_color, &style);
-
-	if(line_width(with_tags ? text : del_tags(text), font_size, style) <= max_width)
+	if (line_width(text, font_size, style) <= max_width)
 		return text;
 	if(line_width(ellipsis, font_size, style) > max_width)
 		return "";
@@ -874,9 +892,8 @@ std::string make_text_ellipsis(const std::string &text, int font_size,
 	for(; itor != utils::utf8_iterator::end(text); ++itor) {
 		std::string tmp = current_substring;
 		tmp.append(itor.substr().first, itor.substr().second);
-		tmp += ellipsis;
 
-		if (line_width(with_tags ? tmp : del_tags(tmp), font_size, style) > max_width) {
+		if (line_width(tmp + ellipsis, font_size, style) > max_width) {
 			return current_substring + ellipsis;
 		}
 
@@ -896,11 +913,12 @@ class floating_label
 public:
 	floating_label(const std::string& text, int font_size, const SDL_Color& colour, const SDL_Color& bgcolour,
 			double xpos, double ypos, double xmove, double ymove, int lifetime, const SDL_Rect& clip_rect,
-			font::ALIGN align, int border_size, bool scroll_with_map)
+			font::ALIGN align, int border_size, bool scroll_with_map, bool use_markup)
 		: surf_(NULL), buf_(NULL), text_(text), font_size_(font_size), colour_(colour),
 		bgcolour_(bgcolour), bgalpha_(bgcolour.unused), xpos_(xpos), ypos_(ypos),
 		xmove_(xmove), ymove_(ymove), lifetime_(lifetime), clip_rect_(clip_rect),
-		alpha_change_(-255/lifetime), visible_(true), align_(align), border_(border_size), scroll_(scroll_with_map)
+		alpha_change_(-255 / lifetime), visible_(true), align_(align),
+		border_(border_size), scroll_(scroll_with_map), use_markup_(use_markup)
 	{}
 
 	void move(double xmove, double ymove);
@@ -933,7 +951,7 @@ private:
 	bool visible_;
 	font::ALIGN align_;
 	int border_;
-	bool scroll_;
+	bool scroll_, use_markup_;
 };
 
 typedef std::map<int,floating_label> label_map;
@@ -963,7 +981,18 @@ int floating_label::xpos(size_t width) const
 surface floating_label::create_surface()
 {
 	if (surf_.null()) {
-		surface foreground = font::render_text(text_, font_size_, colour_, 0, true);
+		font::ttext text;
+		text.set_foreground_colour((colour_.r << 24) | (colour_.g << 16) | (colour_.b << 8) | 255);
+		text.set_font_size(font_size_);
+		text.set_maximum_width(clip_rect_.w);
+		text.set_maximum_height(clip_rect_.h);
+		// If a colour is specified don't allow to override it with markup. (prevents faking map labels for example)
+		// FIXME: @todo Better would be to only ignore colour markup or reset the colour after set_text().
+		if (colour_ != font::LABEL_COLOUR) use_markup_ = false;
+		text.set_text(text_, use_markup_);
+		// Reset the maximum width, as we want the smallest bounding box.
+		text.set_maximum_width(text.get_width());
+		surface foreground = text.render();
 
 		if(foreground == NULL) {
 			ERR_FT << "could not create floating label's text" << std::endl;
@@ -1001,8 +1030,12 @@ surface floating_label::create_surface()
 		}
 		else {
 			// background is blurred shadow of the text
-			//TODO add a little extra space for the blur of letters with a lower part
-			surface background = shadow_image(foreground, false);
+			surface background = create_neutral_surface
+				(foreground->w + 4, foreground->h + 4);
+			SDL_FillRect(background, NULL, 0);
+			SDL_Rect r = { 2, 2, 0, 0 };
+			blit_surface(foreground, NULL, background, &r);
+			background = shadow_image(background, false);
 
 			if (background == NULL) {
 				ERR_FT << "could not create floating label's shadow" << std::endl;
@@ -1010,7 +1043,7 @@ surface floating_label::create_surface()
 				return surf_;
 			}
 			SDL_SetAlpha(foreground,SDL_SRCALPHA,SDL_ALPHA_OPAQUE);
-			blit_surface(foreground, NULL, background, NULL);
+			blit_surface(foreground, NULL, background, &r);
 			surf_ = create_optimized_surface(background);
 		}
 	}
@@ -1031,7 +1064,7 @@ void floating_label::draw(surface screen)
 	}
 
 	if(buf_ == NULL) {
-		buf_.assign(create_compatible_surface(surf_));
+		buf_.assign(create_compatible_surface(screen, surf_->w, surf_->h));
 		if(buf_ == NULL) {
 			return;
 		}
@@ -1077,7 +1110,8 @@ void floating_label::undraw(surface screen)
 namespace font {
 int add_floating_label(const std::string& text, int font_size, const SDL_Color& colour,
 		double xpos, double ypos, double xmove, double ymove, int lifetime, const SDL_Rect& clip_rect, ALIGN align,
-		const SDL_Color* bg_colour, int border_size, LABEL_SCROLL_MODE scroll_mode)
+		const SDL_Color *bg_colour, int border_size, LABEL_SCROLL_MODE scroll_mode,
+		bool use_markup)
 {
 	if(label_contexts.empty()) {
 		return 0;
@@ -1093,7 +1127,9 @@ int add_floating_label(const std::string& text, int font_size, const SDL_Color&
 	}
 
 	++label_id;
-	labels.insert(std::pair<int,floating_label>(label_id,floating_label(text,font_size,colour,bg,xpos,ypos,xmove,ymove,lifetime,clip_rect,align,border_size,scroll_mode == ANCHOR_LABEL_MAP)));
+	labels.insert(std::pair<int, floating_label>(label_id, floating_label(
+		text, font_size, colour, bg, xpos, ypos, xmove, ymove, lifetime, clip_rect,
+		align, border_size, scroll_mode == ANCHOR_LABEL_MAP, use_markup)));
 	label_contexts.top().insert(label_id);
 	return label_id;
 }
@@ -1220,15 +1256,16 @@ void undraw_floating_labels(surface screen)
 
 }
 
-static bool add_font_to_fontlist(config* fonts_config, std::vector<font::subset_descriptor>& fontlist, const std::string& name)
-	{
-		config* font = fonts_config->find_child("font", "name", name);
-		if(font == NULL)
-			return false;
+static bool add_font_to_fontlist(config &fonts_config,
+	std::vector<font::subset_descriptor>& fontlist, const std::string& name)
+{
+	config &font = fonts_config.find_child("font", "name", name);
+	if (!font)
+		return false;
 
 		fontlist.push_back(font::subset_descriptor());
 		fontlist.back().name = name;
-		std::vector<std::string> ranges = utils::split((*font)["codepoints"]);
+		std::vector<std::string> ranges = utils::split(font["codepoints"]);
 
 		for(std::vector<std::string>::const_iterator itor = ranges.begin();
 				itor != ranges.end(); ++itor) {
@@ -1268,18 +1305,17 @@ bool load_font_config()
 		return false;
 	}
 
-	config* fonts_config = cfg.child("fonts");
-	if(fonts_config == NULL)
+	config &fonts_config = cfg.child("fonts");
+	if (!fonts_config)
 		return false;
 
 	std::set<std::string> known_fonts;
-	const config::child_list fonts = fonts_config->get_children("font");
-	for (config::child_list::const_iterator child = fonts.begin(); child != fonts.end(); ++child) {
-		known_fonts.insert((**child)["name"]);
+	foreach (const config &font, fonts_config.child_range("font")) {
+		known_fonts.insert(font["name"]);
 	}
 
-	family_order = (*fonts_config)["family_order"];
-	const std::vector<std::string> font_order = utils::split((*fonts_config)["order"]);
+	family_order = fonts_config["family_order"];
+	const std::vector<std::string> font_order = utils::split(fonts_config["order"]);
 	std::vector<font::subset_descriptor> fontlist;
 	std::vector<std::string>::const_iterator font;
 	for(font = font_order.begin(); font != font_order.end(); ++font) {
