@@ -1,6 +1,6 @@
-/* $Id: font.hpp 35228 2009-04-26 08:38:25Z silene $ */
+/* $Id: font.hpp 46300 2010-09-07 07:50:59Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -20,10 +20,8 @@
 class CVideo;
 class t_string;
 #include "sdl_utils.hpp"
-#include "serialization/string_utils.hpp"
 
 #include <string>
-#include <vector>
 
 namespace font {
 
@@ -51,7 +49,7 @@ private:
 //various standard colours
 extern const SDL_Color NORMAL_COLOUR, GRAY_COLOUR, LOBBY_COLOUR, GOOD_COLOUR, BAD_COLOUR,
                        BLACK_COLOUR, YELLOW_COLOUR, BUTTON_COLOUR, BIGMAP_COLOUR,
-                       STONED_COLOUR, TITLE_COLOUR, DISABLED_COLOUR, LABEL_COLOUR;
+                       PETRIFIED_COLOUR, TITLE_COLOUR, DISABLED_COLOUR, LABEL_COLOUR;
 
 // font sizes, to be made theme parameters
 #ifdef USE_TINY_GUI
@@ -109,14 +107,11 @@ int line_width(const std::string& line, int font_size, int style=TTF_STYLE_NORMA
 ///
 SDL_Rect line_size(const std::string& line, int font_size, int style=TTF_STYLE_NORMAL);
 
-///
-/// If the text excedes the specified max width, end it with an ellipsis (...)
-/// The with_tags can probably always be set to false
-/// If parse_for_style is true we look for the style of the text (bold, etc.)
-/// before deleting its tags.
-///
+/**
+ * If the text excedes the specified max width, end it with an ellipsis (...)
+ */
 std::string make_text_ellipsis(const std::string& text, int font_size, int max_width,
-		bool with_tags = true, bool parse_for_style = false);
+	int style = TTF_STYLE_NORMAL);
 
 
 /// structure which will hide all current floating labels, and cause floating labels
@@ -145,7 +140,8 @@ int add_floating_label(const std::string& text, int font_size, const SDL_Color&
 		double xpos, double ypos, double xmove, double ymove, int lifetime,
 		const SDL_Rect& clip_rect, ALIGN alignment=CENTER_ALIGN,
 		const SDL_Color* bg_colour=NULL, int border_size=0,
-		LABEL_SCROLL_MODE scroll_mode=ANCHOR_LABEL_SCREEN);
+		LABEL_SCROLL_MODE scroll_mode = ANCHOR_LABEL_SCREEN,
+		bool use_markup = true);
 
 /// moves the floating label given by 'handle' by (xmove,ymove)
 void move_floating_label(int handle, double xmove, double ymove);
