@@ -1,4 +1,4 @@
-/* $Id: formula.cpp 33793 2009-03-18 01:16:11Z dragonking $ */
+/* $Id: formula.cpp 40549 2010-01-03 18:44:38Z mordante $ */
 /*
    Copyright (C) 2007 by David White <dave.net>
    Part of the Silver Tree Project
@@ -16,7 +16,7 @@
 #include <iostream>
 #include <set>
 
-//#include "foreach.hpp"
+#include "foreach.hpp"
 #include "formula_callable.hpp"
 #include "formula_function.hpp"
 #include "map_utils.hpp"
@@ -29,6 +29,7 @@ void formula_callable::set_value(const std::string& key, const variant& /*value*
 	std::cerr << "ERROR: cannot set key '" << key << "' on object\n";
 }
 
+
 map_formula_callable::map_formula_callable(
     	const formula_callable* fallback) :
 	formula_callable(false),
@@ -72,8 +73,12 @@ public:
 		: symbols_(symbols)
 	{}
 
+	virtual std::string str() const
+	{
+		return "{function_list_expression()}";
+	}
 private:
-	variant execute(const formula_callable& /*variables*/) const {
+	variant execute(const formula_callable& /*variables*/, formula_debugger * /*fdb*/) const {
 		std::vector<variant> res;
 		std::vector<std::string> function_names = builtin_function_names();
 		std::vector<std::string> more_function_names = symbols_->get_function_names();
@@ -94,17 +99,36 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
 		std::vector<variant> res;
 		res.reserve(items_.size());
 		for(std::vector<expression_ptr>::const_iterator i = items_.begin(); i != items_.end(); ++i) {
-			res.push_back((*i)->evaluate(variables));
+			res.push_back((*i)->evaluate(variables,fdb));
 		}
 
 		return variant(&res);
 	}
 
 	std::vector<expression_ptr> items_;
+
+	std::string str() const
+	{
+		std::stringstream s;
+		s << '[';
+		bool first_item = true;
+		foreach(expression_ptr a , items_) {
+			if (!first_item) {
+				s << ',';
+			} else {
+				first_item = false;
+			}
+			s << a->str();
+		}
+		s << ']';
+		return s.str();
+	}
+
+
 };
 
 class map_expression : public formula_expression {
@@ -113,13 +137,25 @@ public:
 	   : items_(items)
 	{}
 
+	virtual std::string str() const
+	{
+		std::stringstream s;
+		s << " [";
+		for(std::vector<expression_ptr>::const_iterator i = items_.begin(); ( i != items_.end() ) && ( i+1 != items_.end() ) ; i+=2) {
+			s << (*i)->str();
+			s << " -> ";
+			s << (*(i+1))->str();
+		}
+		s << " ]";
+		return s.str();
+	}
 private:
-	variant execute(const formula_callable& variables) const {
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
 		std::map<variant,variant> res;
 		for(std::vector<expression_ptr>::const_iterator i = items_.begin(); ( i != items_.end() ) && ( i+1 != items_.end() ) ; i+=2) {
-				variant key = (*i)->evaluate(variables);
-				variant value = (*(i+1))->evaluate(variables);
-				res[ key ] = value;
+			variant key = (*i)->evaluate(variables,fdb);
+			variant value = (*(i+1))->evaluate(variables,fdb);
+			res[ key ] = value;
 		}
 
 		return variant(&res);
@@ -131,7 +167,7 @@ private:
 class unary_operator_expression : public formula_expression {
 public:
 	unary_operator_expression(const std::string& op, expression_ptr arg) :
-		op_(),
+		op_(),op_str_(op),
 		operand_(arg)
 	{
 		if(op == "not") {
@@ -142,9 +178,16 @@ public:
 			throw formula_error("Illegal unary operator: '" + op + "'" , "", "", 0);
 		}
 	}
+
+	virtual std::string str() const {
+		std::stringstream s;
+		s << op_str_ << '('<< operand_->str() << ')';
+		return s.str();
+	}
+
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant res = operand_->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant res = operand_->evaluate(variables,fdb);
 		switch(op_) {
 		case NOT:
 			return res.as_bool() ? variant(0) : variant(1);
@@ -155,6 +198,7 @@ private:
 	}
 	enum OP { NOT, SUB };
 	OP op_;
+	std::string op_str_;
 	expression_ptr operand_;
 };
 
@@ -192,6 +236,7 @@ public:
 			return variant();
 		}
 	}
+
 };
 
 class dot_callable : public formula_callable {
@@ -221,21 +266,27 @@ public:
 	dot_expression(expression_ptr left, expression_ptr right)
 	   : left_(left), right_(right)
 	{}
+	std::string str() const
+	{
+		std::stringstream s;
+		s << left_->str() << "." << right_->str();
+		return s.str();
+	}
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant left = left_->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant left = left_->evaluate(variables,add_debug_info(fdb,0,"left."));
 		if(!left.is_callable()) {
 			if(left.is_list()) {
                                 list_callable list_call(left);
                                 dot_callable callable(variables, list_call);
-				return right_->evaluate(callable);
+				return right_->evaluate(callable,fdb);
 			}
 
 			return left;
 		}
 
                 dot_callable callable(variables, *left.as_callable());
-                return right_->evaluate(callable);
+                return right_->evaluate(callable,add_debug_info(fdb,1,".right"));
         }
 
 	expression_ptr left_, right_;
@@ -246,14 +297,21 @@ public:
 	square_bracket_expression(expression_ptr left, expression_ptr key)
 	   : left_(left), key_(key)
 	{}
+
+	std::string str() const
+	{
+		std::stringstream s;
+		s << left_->str() << '[' << key_->str() << ']';
+		return s.str();
+	}
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant left = left_->evaluate(variables);
-		const variant key = key_->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant left = left_->evaluate(variables,fdb);
+		const variant key = key_->evaluate(variables,fdb);
 		if(left.is_list() || left.is_map()) {
 			return left[ key ];
 		} else {
-			throw formula_error("Illegal usage of operator []", "", "", 0);
+			return variant();
 		}
 	}
 
@@ -264,7 +322,7 @@ class operator_expression : public formula_expression {
 public:
 	operator_expression(const std::string& op, expression_ptr left,
 	                             expression_ptr right)
-	  : op_(OP(op[0])), left_(left), right_(right)
+		: op_(OP(op[0])), op_str_(op), left_(left), right_(right)
 	{
 		if(op == ">=") {
 			op_ = GTE;
@@ -276,13 +334,27 @@ public:
 			op_ = AND;
 		} else if(op == "or") {
 			op_ = OR;
+		} else if(op == ".+") {
+			op_ = ADDL;
+		} else if(op == ".-") {
+			op_ = SUBL;
+		} else if(op == ".*") {
+			op_ = MULL;
+		} else if(op == "./") {
+			op_ = DIVL;
 		}
 	}
 
+	std::string str() const
+	{
+		std::stringstream s;
+		s << left_->str() << op_str_ << right_->str();
+		return s.str();
+	}
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant left = left_->evaluate(variables);
-		const variant right = right_->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant left = left_->evaluate(variables,add_debug_info(fdb,0,"left_OP"));
+		const variant right = right_->evaluate(variables,add_debug_info(fdb,1,"OP_right"));
 		switch(op_) {
 		case AND:
 			return left.as_bool() == false ? left : right;
@@ -298,6 +370,14 @@ private:
 			return left / right;
 		case POW:
 			return left ^ right;
+		case ADDL:
+			return left.list_elements_add(right);
+		case SUBL:
+			return left.list_elements_sub(right);
+		case MULL:
+			return left.list_elements_mul(right);
+		case DIVL:
+			return left.list_elements_div(right);
 		case EQ:
 			return left == right ? variant(1) : variant(0);
 		case NEQ:
@@ -327,23 +407,32 @@ private:
 	}
 
 	enum OP { AND, OR, NEQ, LTE, GTE, GT='>', LT='<', EQ='=',
-	          ADD='+', SUB='-', MUL='*', DIV='/', DICE='d', POW='^', MOD='%' };
+	          ADD='+', SUB='-', MUL='*', DIV='/', ADDL, SUBL, MULL, DIVL, DICE='d', POW='^', MOD='%' };
 
 	OP op_;
+	std::string op_str_;
 	expression_ptr left_, right_;
 };
 
 typedef std::map<std::string,expression_ptr> expr_table;
 typedef boost::shared_ptr<expr_table> expr_table_ptr;
+typedef std::map<std::string, variant> exp_table_evaluated;
 
 class where_variables: public formula_callable {
 public:
 	where_variables(const formula_callable &base,
 			expr_table_ptr table )
-		: formula_callable(false), base_(base), table_(table) { }
+		: formula_callable(false)
+		, base_(base)
+		, table_(table)
+		, evaluated_table_()
+	{
+	}
+
 private:
 	const formula_callable& base_;
 	expr_table_ptr table_;
+	mutable exp_table_evaluated evaluated_table_;
 
 	void get_inputs(std::vector<formula_input>* inputs) const {
 		for(expr_table::const_iterator i = table_->begin(); i != table_->end(); ++i) {
@@ -354,7 +443,13 @@ private:
 	variant get_value(const std::string& key) const {
 		expr_table::iterator i = table_->find(key);
 		if(i != table_->end()) {
-			return i->second->evaluate(base_);
+			exp_table_evaluated::const_iterator ev = evaluated_table_.find(key);
+			if( ev != evaluated_table_.end())
+				return ev->second;
+
+			variant v = i->second->evaluate(base_);
+			evaluated_table_[key] = v;
+			return v;
 		}
 		return base_.query_value(key);
 	}
@@ -367,13 +462,25 @@ public:
 		: body_(body), clauses_(clauses)
 	{}
 
+	std::string str() const
+	{
+		std::stringstream s;
+		s << "{where:(";
+		s << body_->str();
+		foreach (const expr_table::value_type &a, *clauses_) {
+			s << ", [" << a.first << "] -> ["<< a.second->str()<<"]";
+		}
+		s << ")}";
+		return s.str();
+	}
+
 private:
 	expression_ptr body_;
 	expr_table_ptr clauses_;
 
-	variant execute(const formula_callable& variables) const {
+	variant execute(const formula_callable& variables,formula_debugger *fdb) const {
 		where_variables wrapped_variables(variables, clauses_);
-		return body_->evaluate(wrapped_variables);
+		return body_->evaluate(wrapped_variables,fdb);
 	}
 };
 
@@ -382,8 +489,12 @@ class identifier_expression : public formula_expression {
 public:
 	explicit identifier_expression(const std::string& id) : id_(id)
 	{}
+	std::string str() const
+	{
+		return id_;
+	}
 private:
-	variant execute(const formula_callable& variables) const {
+	variant execute(const formula_callable& variables, formula_debugger * /*fdb*/) const {
 		return variables.query_value(id_);
 	}
 	std::string id_;
@@ -392,8 +503,11 @@ private:
 class null_expression : public formula_expression {
 public:
 	explicit null_expression() {};
+	std::string str() const {
+		return "";
+	}
 private:
-	variant execute(const formula_callable& /*variables*/) const {
+	variant execute(const formula_callable& /*variables*/, formula_debugger * /*fdb*/) const {
 		return variant();
 	}
 };
@@ -403,14 +517,39 @@ class integer_expression : public formula_expression {
 public:
 	explicit integer_expression(int i) : i_(i)
 	{}
+	std::string str() const
+	{
+		std::stringstream s;
+		s << i_;
+		return s.str();
+	}
 private:
-	variant execute(const formula_callable& /*variables*/) const {
+	variant execute(const formula_callable& /*variables*/, formula_debugger * /*fdb*/) const {
 		return variant(i_);
 	}
 
 	int i_;
 };
 
+class decimal_expression : public formula_expression {
+public:
+	explicit decimal_expression(int i, int f) : i_(i), f_(f)
+	{}
+
+	std::string str() const
+	{
+		std::stringstream s;
+		s << i_ << '.' << f_;
+		return s.str();
+	}
+private:
+	variant execute(const formula_callable& /*variables*/, formula_debugger * /*fdb*/) const {
+		return variant(i_ * 1000 + f_, variant::DECIMAL_VARIANT );
+	}
+
+	int i_, f_;
+};
+
 class string_expression : public formula_expression {
 public:
 	explicit string_expression(std::string str) :
@@ -438,15 +577,20 @@ public:
 
 		str_ = variant(str);
 	}
+
+	std::string str() const
+	{
+		return str_.as_string();
+	}
 private:
-	variant execute(const formula_callable& variables) const {
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
 		if(subs_.empty()) {
 			return str_;
 		} else {
 			std::string res = str_.as_string();
 			for(size_t i=0; i < subs_.size(); ++i) {
 				const substitution& sub = subs_[i];
-				const std::string str = sub.calculation->execute(variables).string_cast();
+				const std::string str = sub.calculation->evaluate(variables,fdb).string_cast();
 				res.insert(sub.pos, str);
 			}
 
@@ -678,6 +822,9 @@ expression_ptr parse_expression(const token* i1, const token* i2, function_symbo
 				++i1;
 			}
 			const std::string precond = "";
+			if(symbols == NULL) {
+				throw formula_error("Function symbol table required but not present", "",*i1->filename, i1->line_number);
+			}
 			symbols->add_formula_function(formula_name,
 					const_formula_ptr(new formula(beg, i1, symbols)),
 					formula::create_optional_formula(precond, symbols),
@@ -703,8 +850,8 @@ expression_ptr parse_expression(const token* i1, const token* i2, function_symbo
 			if( ( !operator_group ) && (op == NULL || operator_precedence(*op) >=
 							 operator_precedence(*i)) ) {
 				op = i;
-				operator_group = true;
 			}
+                        operator_group = true;
 		} else {
 			operator_group = false;
 		}
@@ -762,6 +909,30 @@ expression_ptr parse_expression(const token* i1, const token* i2, function_symbo
 			} else if(i1->type == TOKEN_INTEGER) {
 				int n = atoi(std::string(i1->begin,i1->end).c_str());
 				return expression_ptr(new integer_expression(n));
+			} else if(i1->type == TOKEN_DECIMAL) {
+				iterator dot = i1->begin;
+				while( *dot != '.' )
+					dot++;
+
+				int n = atoi(std::string(i1->begin,dot).c_str());
+
+				iterator end = i1->end;
+
+				if( end - dot > 4)
+					end = dot + 4;
+
+				++dot;
+
+				int f = 0;
+
+				int multiplicator = 100;
+				while( dot != end) {
+					f += (*dot - 48)*multiplicator;
+					multiplicator /= 10;
+					++dot;
+				}
+
+				return expression_ptr(new decimal_expression(n, f));
 			} else if(i1->type == TOKEN_STRING_LITERAL) {
 				return expression_ptr(new string_expression(std::string(i1->begin+1,i1->end-1)));
 			}
@@ -944,7 +1115,7 @@ formula::formula(const std::string& str, function_symbol_table* symbols) :
 		throw formula_error("Missing 'faiend', make sure each .fai file ends with it", "", "", 0);
 	}
 
-	if(tokens.size() != 0) {
+	if(!tokens.empty()) {
 		expr_ = parse_expression(&tokens[0],&tokens[0] + tokens.size(), symbols);
 	} else {
 		expr_ = expression_ptr(new null_expression());
@@ -962,20 +1133,20 @@ formula::formula(const token* i1, const token* i2, function_symbol_table* symbol
 	}
 }
 
-variant formula::execute(const formula_callable& variables) const
+variant formula::execute(const formula_callable& variables, formula_debugger *fdb) const
 {
 	try {
-		return expr_->evaluate(variables);
+		return expr_->evaluate(variables, fdb);
 	} catch(type_error& e) {
 		std::cerr << "formula type error: " << e.message << "\n";
 		return variant();
 	}
 }
 
-variant formula::execute() const
+variant formula::execute(formula_debugger *fdb) const
 {
 	static map_formula_callable null_callable;
-	return execute(null_callable);
+	return execute(null_callable,fdb);
 }
 
 }
@@ -1022,9 +1193,10 @@ class mock_party : public formula_callable {
 int main()
 {
 	srand(time(NULL));
-	mock_char c;
-	mock_party p;
 	try {
+		mock_char c;
+		mock_party p;
+
 		assert(formula("strength").execute(c).as_int() == 15);
 		assert(formula("17").execute(c).as_int() == 17);
 		assert(formula("strength/2 + agility").execute(c).as_int() == 19);
