@@ -1,6 +1,6 @@
-/* $Id: formula_function.cpp 33859 2009-03-19 20:52:41Z ivanovic $ */
+/* $Id: formula_function.cpp 40503 2010-01-01 17:12:05Z mordante $ */
 /*
-   Copyright (C) 2008 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2008 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -17,19 +17,70 @@
 
 //#include "foreach.hpp"
 #include "callable_objects.hpp"
+#include "foreach.hpp"
+#include "formula_debugger.hpp"
 #include "formula_function.hpp"
+#include "game_display.hpp"
 #include "log.hpp"
 
 
-#define DBG_NG LOG_STREAM(debug, engine)
-#define LOG_AI LOG_STREAM(info, formula_ai)
-#define WRN_AI LOG_STREAM(warn, formula_ai)
-#define ERR_AI LOG_STREAM(err, formula_ai)
+static lg::log_domain log_engine("engine");
+#define DBG_NG LOG_STREAM(debug, log_engine)
+static lg::log_domain log_scripting_formula("scripting/formula");
+#define LOG_AI LOG_STREAM(info, log_scripting_formula)
 
 namespace game_logic {
 
+
+std::string function_expression::str() const
+{
+	std::stringstream s;
+	s << get_name();
+	s << '(';
+	bool first_arg = true;
+	foreach(expression_ptr a , args()) {
+		if (!first_arg) {
+			s << ',';
+		} else {
+			first_arg = false;
+		}
+		s << a->str();
+		}
+	s << ')';
+	return s.str();
+}
+
 namespace {
 
+class debug_function : public function_expression {
+public:
+	explicit debug_function(const args_list& args)
+		: function_expression("debug",args, 0, 1)
+	{}
+private:
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		boost::shared_ptr<formula_debugger> fdbp;
+		bool need_wrapper = false;
+		if (fdb==NULL) {
+			fdbp = boost::shared_ptr<formula_debugger>(new formula_debugger());
+			fdb = &*fdbp;
+			need_wrapper = true;
+
+		}
+
+		if (args().size()==1) {
+			if (!need_wrapper) {
+				return args()[0]->evaluate(variables,fdb);
+			} else {
+				return wrapper_formula(args()[0]).evaluate(variables,fdb);
+			}
+		} else {
+			return wrapper_formula().evaluate(variables,fdb);
+		}
+	}
+};
+
+
 class dir_function : public function_expression {
 public:
 	explicit dir_function(const args_list& args)
@@ -37,8 +88,8 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		variant var = args()[0]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		variant var = args()[0]->evaluate(variables, fdb);
 		const formula_callable* callable = var.as_callable();
 		callable->add_ref();
 		std::vector<formula_input> inputs = callable->inputs();
@@ -55,13 +106,23 @@ private:
 class if_function : public function_expression {
 public:
 	explicit if_function(const args_list& args)
-	     : function_expression("if", args, 3, 3)
+	     : function_expression("if", args, 3, -1)
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		const int i = args()[0]->evaluate(variables).as_bool() ? 1 : 2;
-		return args()[i]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		for(size_t n = 0; n < args().size()-1; n += 2) {
+			if( args()[n]->evaluate(variables,fdb).as_bool() ) {
+				return args()[n+1]->evaluate(variables,fdb);
+			}
+		}
+
+		if((args().size()%2) != 0) {
+			return args().back()->evaluate(variables,fdb);
+		} else {
+			return variant();
+		}
+
 	}
 };
 
@@ -72,17 +133,17 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		variant var = args()[0]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		variant var = args()[0]->evaluate(variables,fdb);
 		for(size_t n = 1; n < args().size()-1; n += 2) {
-			variant val = args()[n]->evaluate(variables);
+			variant val = args()[n]->evaluate(variables,fdb);
 			if(val == var) {
-				return args()[n+1]->evaluate(variables);
+				return args()[n+1]->evaluate(variables,fdb);
 			}
 		}
 
 		if((args().size()%2) == 0) {
-			return args().back()->evaluate(variables);
+			return args().back()->evaluate(variables,fdb);
 		} else {
 			return variant();
 		}
@@ -96,8 +157,8 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		const int n = args()[0]->evaluate(variables).as_int();
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const int n = args()[0]->evaluate(variables,fdb).as_int();
 		return variant(n >= 0 ? n : -n);
 	}
 };
@@ -109,11 +170,11 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
 		bool found = false;
 		int res = 0;
 		for(size_t n = 0; n != args().size(); ++n) {
-			const variant v = args()[n]->evaluate(variables);
+			const variant v = args()[n]->evaluate(variables,fdb);
 			if(v.is_list()) {
 				for(size_t m = 0; m != v.num_elements(); ++m) {
 					if(!found || v[m].as_int() < res) {
@@ -140,11 +201,11 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
 		bool found = false;
 		int res = 0;
 		for(size_t n = 0; n != args().size(); ++n) {
-			const variant v = args()[n]->evaluate(variables);
+			const variant v = args()[n]->evaluate(variables,fdb);
 			if(v.is_list()) {
 				for(size_t m = 0; m != v.num_elements(); ++m) {
 					if(!found || v[m].as_int() > res) {
@@ -164,6 +225,40 @@ private:
 	}
 };
 
+class debug_float_function : public function_expression {
+public:
+        explicit debug_float_function(const args_list& args)
+          : function_expression("debug_float", args, 2, 3)
+        {}
+
+private:
+        variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+                const args_list& arguments = args();
+                const variant var0 = arguments[0]->evaluate(variables,fdb);
+                const variant var1 = arguments[1]->evaluate(variables,fdb);
+
+                const map_location location = convert_variant<location_callable>(var0)->loc();
+                std::string text;
+
+                if(arguments.size() == 2) {
+                        text = var1.to_debug_string();
+                        display_float(location,text);
+                        return var1;
+                } else {
+                        const variant var2 = arguments[2]->evaluate(variables,fdb);
+                        text = var1.string_cast() + var2.to_debug_string();
+                        display_float(location,text);
+                        return var2;
+                }
+
+        }
+
+        void display_float(const map_location& location, const std::string& text) const{
+                game_display::get_singleton()->float_label(location,text,255,0,0);
+        }
+};
+
+
 class debug_print_function : public function_expression {
 public:
 	explicit debug_print_function(const args_list& args)
@@ -171,20 +266,20 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant var1 = args()[0]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant var1 = args()[0]->evaluate(variables,fdb);
 
 		std::string str1,str2;
 
 		if( args().size() == 1)
 		{
-			str1 = var1.to_debug_string();
+			str1 = var1.to_debug_string(NULL, true);
 			LOG_AI << str1 << std::endl;
 			return var1;
 		} else {
 			str1 = var1.string_cast();
-			const variant var2 = args()[1]->evaluate(variables);
-			str2 = var2.to_debug_string();
+			const variant var2 = args()[1]->evaluate(variables,fdb);
+			str2 = var2.to_debug_string(NULL, true);
 			LOG_AI << str1 << str2 << std::endl;
 			return var2;
 		}
@@ -198,8 +293,8 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant map = args()[0]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant map = args()[0]->evaluate(variables,fdb);
 		return map.get_keys();
 	}
 };
@@ -211,8 +306,8 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant map = args()[0]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant map = args()[0]->evaluate(variables,fdb);
 		return map.get_values();
 	}
 };
@@ -220,12 +315,12 @@ private:
 class tolist_function : public function_expression {
 public:
 	explicit tolist_function(const args_list& args)
-	     : function_expression("values", args, 1, 1)
+	     : function_expression("tolist", args, 1, 1)
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant var = args()[0]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant var = args()[0]->evaluate(variables,fdb);
 
 		std::vector<variant> tmp;
 
@@ -241,21 +336,21 @@ private:
 class tomap_function : public function_expression {
 public:
 	explicit tomap_function(const args_list& args)
-	     : function_expression("values", args, 1, 2)
+	     : function_expression("tomap", args, 1, 2)
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant var_1 = args()[0]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant var_1 = args()[0]->evaluate(variables,fdb);
 
 		std::map<variant, variant> tmp;
 
 		if (args().size() == 2)
 		{
-			const variant var_2 = args()[1]->evaluate(variables);
+			const variant var_2 = args()[1]->evaluate(variables,fdb);
 			if ( var_1.num_elements() != var_2.num_elements() )
 				return variant();
-			for(size_t i = 0; i < var_1.num_elements(); i++ )
+			for(size_t i = 0; i < var_1.num_elements(); ++i )
 				tmp[ var_1[i] ] = var_2[i];
 		} else
 		{
@@ -280,11 +375,11 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant value = args()[0]->evaluate(variables);
-		const variant list = args()[1]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant value = args()[0]->evaluate(variables,fdb);
+		const variant list = args()[1]->evaluate(variables,fdb);
 
-		for(size_t i = 0; i < list.num_elements(); i++ ) {
+		for(size_t i = 0; i < list.num_elements(); ++i ) {
 			if( list[i] == value) {
 				return variant(i);
 			}
@@ -302,15 +397,15 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant items = args()[0]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant items = args()[0]->evaluate(variables,fdb);
 		variant max_value;
 		variant_iterator it = items.get_iterator();
 		variant_iterator max = variant_iterator();
 
 		if(args().size() == 2) {
 			for(it = items.begin(); it != items.end(); ++it) {
-				const variant val = args().back()->evaluate(formula_variant_callable_with_backup(*it, variables));
+				const variant val = args().back()->evaluate(formula_variant_callable_with_backup(*it, variables),fdb);
 				if(max == variant_iterator() || val > max_value) {
 					max = it;
 					max_value = val;
@@ -319,10 +414,10 @@ private:
 		} else {
 			map_formula_callable self_callable;
 			self_callable.add_ref();
-			const std::string self = args()[1]->evaluate(variables).as_string();
+			const std::string self = args()[1]->evaluate(variables,fdb).as_string();
 			for(it = items.begin(); it != items.end(); ++it) {
 				self_callable.add(self, *it);
-				const variant val = args().back()->evaluate(formula_callable_with_backup(self_callable, formula_variant_callable_with_backup(*it, variables)));
+				const variant val = args().back()->evaluate(formula_callable_with_backup(self_callable, formula_variant_callable_with_backup(*it, variables)),fdb);
 				if(max == variant_iterator() || val > max_value) {
 					max = it;
 					max_value = val;
@@ -345,8 +440,8 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		const int value = args()[0]->evaluate(variables).as_int()%1000;
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const int value = args()[0]->evaluate(variables,fdb).as_int()%1000;
 		const double angle = 2.0*3.141592653589*(static_cast<double>(value)/1000.0);
 		return variant(static_cast<int>(sin(angle)*1000.0));
 	}
@@ -393,8 +488,8 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		variant list = args()[0]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		variant list = args()[0]->evaluate(variables,fdb);
 		std::vector<variant> vars;
 		vars.reserve(list.num_elements());
 		for(size_t n = 0; n != list.num_elements(); ++n) {
@@ -418,9 +513,9 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		std::string str = args()[0]->evaluate(variables).as_string();
-		std::string key = args()[1]->evaluate(variables).as_string();
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		std::string str = args()[0]->evaluate(variables,fdb).as_string();
+		std::string key = args()[1]->evaluate(variables,fdb).as_string();
 
 		if (key.size() > str.size())
 			return variant(0);
@@ -430,6 +525,9 @@ private:
 		for(str_it = str.begin(); str_it != str.end() - (key.size()-1); ++str_it)
 		{
 			key_it = key.begin();
+			if((key_it) == key.end()) {
+				return variant(1);
+			}
 			tmp_it = str_it;
 
 			while( *tmp_it == *key_it)
@@ -451,17 +549,17 @@ public:
 	    : function_expression("filter", args, 2, 3)
 	{}
 private:
-	variant execute(const formula_callable& variables) const {
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
 		std::vector<variant> list_vars;
 		std::map<variant,variant> map_vars;
 
-		const variant items = args()[0]->evaluate(variables);
+		const variant items = args()[0]->evaluate(variables,fdb);
 
 		variant_iterator it = items.get_iterator();
 
 		if(args().size() == 2) {
 			for(it = items.begin(); it != items.end(); ++it)  {
-				const variant val = args()[1]->evaluate(formula_variant_callable_with_backup(*it, variables));
+				const variant val = args()[1]->evaluate(formula_variant_callable_with_backup(*it, variables),fdb);
 				if(val.as_bool()) {
 					if (items.is_map() )
 						map_vars[ (*it).get_member("key") ] = (*it).get_member("value");
@@ -472,10 +570,10 @@ private:
 		} else {
 			map_formula_callable self_callable;
 			self_callable.add_ref();
-			const std::string self = args()[1]->evaluate(variables).as_string();
+			const std::string self = args()[1]->evaluate(variables,fdb).as_string();
 			for(it = items.begin(); it != items.end(); ++it) {
 				self_callable.add(self, *it);
-				const variant val = args()[2]->evaluate(formula_callable_with_backup(self_callable, formula_variant_callable_with_backup(*it, variables)));
+				const variant val = args()[2]->evaluate(formula_callable_with_backup(self_callable, formula_variant_callable_with_backup(*it, variables)),fdb);
 				if(val.as_bool()) {
 					if (items.is_map() )
 						map_vars[ (*it).get_member("key") ] = (*it).get_member("value");
@@ -497,13 +595,13 @@ public:
 	{}
 
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant items = args()[0]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant items = args()[0]->evaluate(variables,fdb);
 
 		variant_iterator it = items.get_iterator();
 		if(args().size() == 2) {
 			for(it = items.begin(); it != items.end(); ++it) {
-				const variant val = args()[1]->evaluate(formula_variant_callable_with_backup(*it, variables));
+				const variant val = args()[1]->evaluate(formula_variant_callable_with_backup(*it, variables),fdb);
 				if(val.as_bool()) {
 					return *it;
 				}
@@ -511,10 +609,10 @@ private:
 		} else {
 			map_formula_callable self_callable;
 			self_callable.add_ref();
-			const std::string self = args()[1]->evaluate(variables).as_string();
+			const std::string self = args()[1]->evaluate(variables,fdb).as_string();
 			for(it = items.begin(); it != items.end(); ++it){
 				self_callable.add(self, *it);
-				const variant val = args().back()->evaluate(formula_callable_with_backup(self_callable, formula_variant_callable_with_backup(*it, variables)));
+				const variant val = args().back()->evaluate(formula_callable_with_backup(self_callable, formula_variant_callable_with_backup(*it, variables)),fdb);
 				if(val.as_bool()) {
 					return *it;
 				}
@@ -531,15 +629,15 @@ public:
 	    : function_expression("map", args, 2, 3)
 	{}
 private:
-	variant execute(const formula_callable& variables) const {
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
 		std::vector<variant> list_vars;
 		std::map<variant,variant> map_vars;
-		const variant items = args()[0]->evaluate(variables);
+		const variant items = args()[0]->evaluate(variables,fdb);
 
 		variant_iterator it = items.get_iterator();
 		if(args().size() == 2) {
 			for(it = items.begin(); it != items.end(); ++it) {
-				const variant val = args().back()->evaluate(formula_variant_callable_with_backup(*it, variables));
+				const variant val = args().back()->evaluate(formula_variant_callable_with_backup(*it, variables),fdb);
 				if (items.is_map() )
 					map_vars[ (*it).get_member("key") ] = val;
 				else
@@ -548,10 +646,10 @@ private:
 		} else {
 			map_formula_callable self_callable;
 			self_callable.add_ref();
-			const std::string self = args()[1]->evaluate(variables).as_string();
+			const std::string self = args()[1]->evaluate(variables,fdb).as_string();
 			for(it = items.begin(); it != items.end(); ++it) {
 				self_callable.add(self, *it);
-				const variant val = args().back()->evaluate(formula_callable_with_backup(self_callable, formula_variant_callable_with_backup(*it, variables)));
+				const variant val = args().back()->evaluate(formula_callable_with_backup(self_callable, formula_variant_callable_with_backup(*it, variables)),fdb);
 				if (items.is_map() )
 					map_vars[ (*it).get_member("key") ] = val;
 				else
@@ -564,15 +662,45 @@ private:
 	}
 };
 
+class reduce_function : public function_expression {
+public:
+	explicit reduce_function(const args_list& args)
+	    : function_expression("reduce", args, 2, 2)
+	{}
+private:
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		std::vector<variant> list_vars;
+		std::map<variant,variant> map_vars;
+		const variant items = args()[0]->evaluate(variables,fdb);
+
+		if(items.num_elements() == 0)
+			return variant();
+		else if(items.num_elements() == 1)
+			return items[0];
+
+		variant_iterator it = items.get_iterator();
+		it = items.begin();
+		variant res(*it);
+		map_formula_callable self_callable;
+		self_callable.add_ref();
+		for(++it; it != items.end(); ++it) {
+			self_callable.add("a", res);
+			self_callable.add("b", *it);
+			res = args().back()->evaluate(formula_callable_with_backup(self_callable, formula_variant_callable_with_backup(*it, variables)),fdb);
+		}
+		return res;
+	}
+};
+
 class sum_function : public function_expression {
 public:
 	explicit sum_function(const args_list& args)
 	    : function_expression("sum", args, 1, 2)
 	{}
 private:
-	variant execute(const formula_callable& variables) const {
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
 		variant res(0);
-		const variant items = args()[0]->evaluate(variables);
+		const variant items = args()[0]->evaluate(variables,fdb);
 		if (items.num_elements() > 0)
 		{
 			if (items[0].is_list() )
@@ -580,7 +708,7 @@ private:
 				std::vector<variant> tmp;
 				res = variant(&tmp);
 				if(args().size() >= 2) {
-					res = args()[1]->evaluate(variables);
+					res = args()[1]->evaluate(variables,fdb);
 					if(!res.is_list())
 						return variant();
 				}
@@ -589,14 +717,14 @@ private:
 				std::map<variant,variant> tmp;
 				res = variant(&tmp);
 				if(args().size() >= 2) {
-					res = args()[1]->evaluate(variables);
+					res = args()[1]->evaluate(variables,fdb);
 					if(!res.is_map())
 						return variant();
 				}
 			} else
 			{
 				if(args().size() >= 2) {
-					res = args()[1]->evaluate(variables);
+					res = args()[1]->evaluate(variables,fdb);
 				}
 			}
 		}
@@ -615,10 +743,13 @@ public:
 	    : function_expression("head", args, 1, 1)
 	{}
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant items = args()[0]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant items = args()[0]->evaluate(variables,fdb);
 		variant_iterator it = items.get_iterator();
 		it = items.begin();
+		if(it == items.end()) {
+			return variant();
+		}
 		return *it;
 	}
 };
@@ -629,8 +760,8 @@ public:
 	    : function_expression("size", args, 1, 1)
 	{}
 private:
-	variant execute(const formula_callable& variables) const {
-		const variant items = args()[0]->evaluate(variables);
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		const variant items = args()[0]->evaluate(variables,fdb);
 		return variant(static_cast<int>(items.num_elements()));
 	}
 };
@@ -638,22 +769,119 @@ private:
 class null_function : public function_expression {
 public:
 	explicit null_function(const args_list& args)
-	    : function_expression("null", args, 0, 0)
+	    : function_expression("null", args, 0, -1)
 	{}
 private:
-	variant execute(const formula_callable& /*variables*/) const {
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		if( args().size() != 0 ) {
+			for( size_t i = 0; i < args().size() ; ++i)
+				args()[i]->evaluate(variables,fdb);
+		}
+
 		return variant();
 	}
 };
 
+
+class ceil_function : public function_expression {
+public:
+	explicit ceil_function(const args_list& args)
+	    : function_expression("ceil", args, 1, 1)
+	{}
+private:
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		variant decimal = args()[0]->evaluate(variables,fdb);
+
+		int d = decimal.as_decimal();
+
+		int f = d%1000;
+
+		if( f > 0 ) {
+			d/=1000;
+			return variant( ++d );
+		} else if( f < 0 ) {
+			d/=1000;
+			return variant( --d );
+		} else {
+			d/=1000;
+			return variant( d );
+		}
+	}
+};
+
+class round_function : public function_expression {
+public:
+	explicit round_function(const args_list& args)
+	    : function_expression("round", args, 1, 1)
+	{}
+private:
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		variant decimal = args()[0]->evaluate(variables,fdb);
+
+		int d = decimal.as_decimal();
+
+		int f = d%1000;
+
+		if( f >= 500 ) {
+			d/=1000;
+			return variant( ++d );
+		} else if( f <= -500 ) {
+			d/=1000;
+			return variant( --d );
+		} else {
+			d/=1000;
+			return variant( d );
+		}
+	}
+};
+
+class floor_function : public function_expression {
+public:
+	explicit floor_function(const args_list& args)
+	    : function_expression("floor", args, 1, 1)
+	{}
+private:
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		variant decimal = args()[0]->evaluate(variables,fdb);
+
+		int d = decimal.as_decimal();
+
+		if( d%1000 != 0 ) {
+			d/=1000;
+			return variant( d );
+		} else {
+			d/=1000;
+			return variant( d );
+		}
+	}
+};
+
+
+
+class as_decimal_function : public function_expression {
+public:
+	explicit as_decimal_function(const args_list& args)
+	    : function_expression("as_decimal", args, 1, 1)
+	{}
+private:
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		variant decimal = args()[0]->evaluate(variables,fdb);
+
+		int d = decimal.as_decimal();
+
+		return variant( d , variant::DECIMAL_VARIANT );
+	}
+};
+
+
 class refcount_function : public function_expression {
 public:
 	explicit refcount_function(const args_list& args)
 	    : function_expression("refcount", args, 1, 1)
 	{}
 private:
-	variant execute(const formula_callable& variables) const {
-		return variant(args()[0]->evaluate(variables).refcount());
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		return variant(args()[0]->evaluate(variables,fdb).refcount());
 	}
 };
 
@@ -663,8 +891,10 @@ public:
 	  : function_expression("loc", args, 2, 2)
 	{}
 private:
-	variant execute(const formula_callable& variables) const {
-		return variant(new location_callable(map_location(args()[0]->evaluate(variables).as_int()-1, args()[1]->evaluate(variables).as_int()-1)));
+	variant execute(const formula_callable& variables, formula_debugger *fdb) const {
+		return variant(new location_callable(map_location(
+							     args()[0]->evaluate(variables,add_debug_info(fdb,0,"loc:x")).as_int()-1,
+							     args()[1]->evaluate(variables,add_debug_info(fdb,1,"loc:y")).as_int()-1)));
 	}
 };
 
@@ -701,7 +931,7 @@ formula_function_expression::formula_function_expression(const std::string& name
 	}
 }
 
-variant formula_function_expression::execute(const formula_callable& variables) const
+variant formula_function_expression::execute(const formula_callable& variables, formula_debugger *fdb) const
 {
 	static std::string indent;
 	indent += "  ";
@@ -709,7 +939,7 @@ variant formula_function_expression::execute(const formula_callable& variables)
 	const int begin_time = SDL_GetTicks();
 	map_formula_callable callable;
 	for(size_t n = 0; n != arg_names_.size(); ++n) {
-		variant var = args()[n]->evaluate(variables);
+		variant var = args()[n]->evaluate(variables,fdb);
 		callable.add(arg_names_[n], var);
 		if(static_cast<int>(n) == star_arg_) {
 			callable.set_fallback(var.as_callable());
@@ -717,15 +947,15 @@ variant formula_function_expression::execute(const formula_callable& variables)
 	}
 
 	if(precondition_) {
-		if(!precondition_->execute(callable).as_bool()) {
+		if(!precondition_->evaluate(callable,fdb).as_bool()) {
 			DBG_NG << "FAILED function precondition for function '" << formula_->str() << "' with arguments: ";
 			for(size_t n = 0; n != arg_names_.size(); ++n) {
-				DBG_NG << "  arg " << (n+1) << ": " << args()[n]->evaluate(variables).to_debug_string() << "\n";
+				DBG_NG << "  arg " << (n+1) << ": " << args()[n]->evaluate(variables,fdb).to_debug_string() << "\n";
 			}
 		}
 	}
 
-	variant res = formula_->execute(callable);
+	variant res = formula_->evaluate(callable,fdb);
 	const int taken = SDL_GetTicks() - begin_time;
 	DBG_NG << indent << "returning: " << taken << "\n";
 	indent.resize(indent.size() - 2);
@@ -756,7 +986,7 @@ expression_ptr function_symbol_table::create_function(const std::string& fn, con
 std::vector<std::string> function_symbol_table::get_function_names() const
 {
 	std::vector<std::string> res;
-	for(std::map<std::string, formula_function>::const_iterator iter = custom_formulas_.begin(); iter != custom_formulas_.end(); iter++ ) {
+	for(std::map<std::string, formula_function>::const_iterator iter = custom_formulas_.begin(); iter != custom_formulas_.end(); ++iter ) {
 		res.push_back((*iter).first);
 	}
 	return res;
@@ -787,6 +1017,7 @@ functions_map& get_functions_map() {
 
 	if(functions_table.empty()) {
 #define FUNCTION(name) functions_table[#name] = new function_creator<name##_function>();
+		FUNCTION(debug);
 		FUNCTION(dir);
 		FUNCTION(if);
 		FUNCTION(switch);
@@ -794,6 +1025,7 @@ functions_map& get_functions_map() {
 		FUNCTION(min);
 		FUNCTION(max);
 		FUNCTION(choose);
+                FUNCTION(debug_float);
 		FUNCTION(debug_print);
 		FUNCTION(wave);
 		FUNCTION(sort);
@@ -801,10 +1033,15 @@ functions_map& get_functions_map() {
 		FUNCTION(filter);
 		FUNCTION(find);
 		FUNCTION(map);
+		FUNCTION(reduce);
 		FUNCTION(sum);
 		FUNCTION(head);
 		FUNCTION(size);
 		FUNCTION(null);
+		FUNCTION(ceil);
+		FUNCTION(floor);
+		FUNCTION(round);
+		FUNCTION(as_decimal);
 		FUNCTION(refcount);
 		FUNCTION(loc);
 		FUNCTION(index_of);
@@ -831,7 +1068,7 @@ expression_ptr create_function(const std::string& fn,
 		}
 	}
 
-	DBG_NG << "FN: '" << fn << "' " << fn.size() << "\n";
+	//DBG_NG << "FN: '" << fn << "' " << fn.size() << "\n";
 
 	functions_map::const_iterator i = get_functions_map().find(fn);
 	if(i == get_functions_map().end()) {
