@@ -1,6 +1,6 @@
-/* $Id: formula_tokenizer.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: formula_tokenizer.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2007 - 2009 by David White <dave.net>
+   Copyright (C) 2007 - 2010 by David White <dave.net>
    Part of the Silver Tree Project
 
    This program is free software; you can redistribute it and/or modify
@@ -22,12 +22,12 @@ namespace formula_tokenizer
 typedef std::string::const_iterator iterator;
 
 enum TOKEN_TYPE { TOKEN_OPERATOR, TOKEN_STRING_LITERAL,
-		          TOKEN_IDENTIFIER, TOKEN_INTEGER,
-                  TOKEN_LPARENS, TOKEN_RPARENS,
-				  TOKEN_LSQUARE, TOKEN_RSQUARE,
-				  TOKEN_COMMA, TOKEN_SEMICOLON,
-				  TOKEN_WHITESPACE, TOKEN_EOL, TOKEN_KEYWORD,
-				  TOKEN_COMMENT, TOKEN_POINTER  };
+			TOKEN_IDENTIFIER, TOKEN_INTEGER, TOKEN_DECIMAL,
+			TOKEN_LPARENS, TOKEN_RPARENS,
+			TOKEN_LSQUARE, TOKEN_RSQUARE,
+			TOKEN_COMMA, TOKEN_SEMICOLON,
+			TOKEN_WHITESPACE, TOKEN_EOL, TOKEN_KEYWORD,
+			TOKEN_COMMENT, TOKEN_POINTER  };
 
 struct token {
 
@@ -40,6 +40,15 @@ struct token {
 	{
 	}
 
+	token(iterator& i1, iterator i2, TOKEN_TYPE type) :
+		type(type),
+		begin(i1),
+		end(i2),
+		line_number(1),
+		filename()
+	{
+	}
+
 	TOKEN_TYPE type;
 	iterator begin, end;
 	int line_number;
