@@ -1,6 +1,6 @@
-/* $Id: game_config.cpp 34194 2009-03-27 22:13:44Z mordante $ */
+/* $Id: game_config.cpp 41173 2010-02-12 20:26:25Z ilor $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,9 @@
 #include "global.hpp"
 #include "game_config.hpp"
 
+#include "color_range.hpp"
 #include "config.hpp"
+#include "foreach.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
 #include "util.hpp"
@@ -26,9 +28,9 @@
 #include "revision.hpp"
 #endif /* HAVE_REVISION */
 
-
-#define DBG_NG LOG_STREAM(debug, engine)
-#define ERR_NG LOG_STREAM(err, engine)
+static lg::log_domain log_engine("engine");
+#define DBG_NG LOG_STREAM(debug, log_engine)
+#define ERR_NG LOG_STREAM(err, log_engine)
 
 namespace game_config
 {
@@ -38,20 +40,18 @@ namespace game_config
 	int rest_heal_amount= 2;
 	int recall_cost = 20;
 	int kill_experience = 8;
-	unsigned lobby_refresh = 2000;
+	unsigned lobby_network_timer = 100;
+	unsigned lobby_refresh = 4000;
 	const int gold_carryover_percentage = 80;
-	const bool gold_carryover_add = false;
 	const std::string version = VERSION;
 #ifdef REVISION
 	const std::string revision = VERSION " (" REVISION ")";
 #else
 	const std::string revision = VERSION;
 #endif
-	std::string wesnothd_name;
+	std::string wesnoth_program_dir;
 	bool debug = false, editor = false, ignore_replay_errors = false, mp_debug = false, exit_at_end = false, no_delay = false, small_gui = false, disable_autosave = false;
 
-	bool use_dummylocales = false;
-
 	int cache_compression_level = 6;
 
 	std::string game_icon = "wesnoth-icon-small.png", game_title, game_logo, title_music, lobby_music;
@@ -72,13 +72,12 @@ namespace game_config
 	std::string flag_image = "flags/flag-1.png:150,flags/flag-2.png:150,flags/flag-3.png:150,flags/flag-4.png:150";
 	std::string flag_icon_image = "flags/flag_icon.png";
 	std::string flag_rgb = "flag_green";
-	std::vector<Uint32> defense_color_scale;
+	std::vector<Uint32> red_green_scale;
+	std::vector<Uint32> red_green_scale_text;
 
 	double hp_bar_scaling = 0.666;
 	double xp_bar_scaling = 0.5;
 
-	std::string cross_image = "misc/cross.png";
-
 	std::vector<std::string> foot_speed_prefix;
 	std::string foot_teleport_enter = "footsteps/teleport-in.png";
 	std::string foot_teleport_exit = "footsteps/teleport-out.png";
@@ -184,7 +183,6 @@ namespace game_config
 		ally_ball_image = v["ally_ball_image"];
 		flag_image = v["flag_image"];
 		flag_icon_image = v["flag_icon_image"];
-		cross_image = v["cross_image"];
 
 		hp_bar_scaling = lexical_cast_default<double>(v["hp_bar_scaling"]);
 		xp_bar_scaling = lexical_cast_default<double>(v["xp_bar_scaling"]);
@@ -211,9 +209,14 @@ namespace game_config
 		if( !flag_rgb.size()){
 			flag_rgb="flag_green";
 		}
-		defense_color_scale = string2rgb(v["defense_color_scale"]);
-		if (defense_color_scale.empty()) {
-			defense_color_scale.push_back(0x00FFFF00);
+
+		red_green_scale = string2rgb(v["red_green_scale"]);
+		if (red_green_scale.empty()) {
+			red_green_scale.push_back(0x00FFFF00);
+		}
+		red_green_scale_text = string2rgb(v["red_green_scale_text"]);
+		if (red_green_scale_text.empty()) {
+			red_green_scale_text.push_back(0x00FFFF00);
 		}
 
 		server_list.clear();
@@ -225,10 +228,6 @@ namespace game_config
 			sinf.address = (**server)["address"];
 			server_list.push_back(sinf);
 		}
-		if (!v["wesnothd_name"].empty())
-		{
-			game_config::wesnothd_name = v["wesnothd_name"];
-		}
 	}
 
 	void add_color_info(const config& v){
@@ -241,13 +240,13 @@ namespace game_config
 				team_rgb_name[id] = (**teamC)["name"];
 				//generate palette of same name;
 				std::vector<Uint32> tp = palette(team_rgb_range[id]);
-				if(tp.size()){
+				if(!tp.empty()){
 					team_rgb_colors.insert(std::make_pair(id,tp));
 					//if this is being used, output log of palette for artists use.
 					DBG_NG << "color palette creation:\n";
 					std::stringstream str;
 					str << id <<" = ";
-					for(std::vector<Uint32>::const_iterator r=tp.begin();r!=tp.end();r++){
+					for(std::vector<Uint32>::const_iterator r=tp.begin(); r!=tp.end(); ++r){
 						int red = ((*r) & 0x00FF0000)>>16;
 						int green = ((*r) & 0x0000FF00)>>8;
 						int blue = ((*r) & 0x000000FF);
@@ -263,12 +262,13 @@ namespace game_config
 
 		const config::child_list& colors = v.get_children("color_palette");
 		for(config::child_list::const_iterator cp = colors.begin(); cp != colors.end(); ++cp) {
-			for(string_map::const_iterator rgb_it = (*cp)->values.begin(); rgb_it != (*cp)->values.end(); ++rgb_it) {
+			foreach (const config::attribute &rgb, (*cp)->attribute_range())
+			{
 				try {
-					team_rgb_colors.insert(std::make_pair(rgb_it->first,string2rgb(rgb_it->second)));
+					team_rgb_colors.insert(std::make_pair(rgb.first, string2rgb(rgb.second)));
 				} catch(bad_lexical_cast&) {
 					//throw config::error(_("Invalid team color: ") + rgb_it->second);
-					ERR_NG << "Invalid team color: " << rgb_it->second << "\n";
+					ERR_NG << "Invalid team color: " << rgb.second << "\n";
 				}
 			}
 		}
@@ -307,22 +307,12 @@ namespace game_config
 		return i->second;
 	}
 
-	bool is_compatible_savegame_version(const std::string& v)
-	{
-		bool ret = v==version;
-		if(ret) {
-			return true;
-		} else {
-			// do not load if too old, if either the savegame or the current game
-			// has the version 'test' allow loading
-			try {
-				ret = !(v < min_savegame_version && test_version != v && test_version != version);
-			} catch(version_info::not_sane_exception&) {
-				ERR_NG << "invalid version_info from saved game: '" << v << "'\n";
-				ret = false;
-			}
-		}
-		return ret;
+	Uint32 red_to_green(int val, bool for_text){
+		const std::vector<Uint32>& color_scale =
+				for_text ? red_green_scale_text : red_green_scale;
+		val = std::max<int>(0, std::min<int>(val, 100));
+		int lvl = (color_scale.size()-1) * val / 100;
+		return color_scale[lvl];
 	}
 
 } // game_config
