@@ -1,6 +1,6 @@
-/* $Id: game_config.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: game_config.hpp 41173 2010-02-12 20:26:25Z ilor $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -16,10 +16,12 @@
 
 class config;
 class version_info;
+class color_range;
 
-#include "color_range.hpp"
 #include "tstring.hpp"
 
+#include <SDL_types.h>
+
 #include <string>
 #include <vector>
 #include <map>
@@ -33,26 +35,23 @@ namespace game_config
 	extern int rest_heal_amount;
 	extern int recall_cost;
 	extern int kill_experience;
+	extern unsigned lobby_network_timer;
 	extern unsigned lobby_refresh;
 	extern const std::string version;
 	extern const std::string revision;
 
-	extern std::string wesnothd_name;
+	inline int kill_xp(int level)
+	{
+		return level ? kill_experience * level : kill_experience / 2;
+	}
+
+	extern std::string wesnoth_program_dir;
 
 	/** Default percentage gold carried over to the next scenario. */
 	extern const int gold_carryover_percentage;
 
-	/**
-	 * If true the carried over gold is added to the start gold in the next
-	 * scenario, otherwise it uses the maximum of starting gold and carryover
-	 * gold.
-	 */
-	extern const bool gold_carryover_add;
-
 	extern bool debug, editor, ignore_replay_errors, mp_debug, exit_at_end, no_delay, small_gui, disable_autosave;
 
-	extern bool use_dummylocales;
-
 	extern int cache_compression_level;
 
 	extern std::string path;
@@ -68,7 +67,7 @@ namespace game_config
 	extern std::string game_icon, game_title, game_logo, title_music, lobby_music,
 		moved_ball_image, unmoved_ball_image, partmoved_ball_image,
 		enemy_ball_image, ally_ball_image, energy_image,
-		flag_image, flag_icon_image, cross_image,
+		flag_image, flag_icon_image,
 		terrain_mask_image, grid_image, unreachable_image, linger_image,
 		observer_image, tod_bright_image,
 		checked_menu_image, unchecked_menu_image, wml_menu_image, level_image,
@@ -77,7 +76,8 @@ namespace game_config
 	extern double hp_bar_scaling, xp_bar_scaling;
 
 	extern std::string flag_rgb;
-	extern std::vector<Uint32> defense_color_scale;
+	extern std::vector<Uint32> red_green_scale;
+	extern std::vector<Uint32> red_green_scale_text;
 
 	extern std::vector<std::string> foot_speed_prefix;
 	extern std::string foot_teleport_enter, foot_teleport_exit;
@@ -112,16 +112,18 @@ namespace game_config
 	const std::vector<Uint32>& tc_info(const std::string& name);
 	const color_range& color_info(const std::string& name);
 
+	/**
+	 * Return a color corresponding to the value val
+	 * red for val=0 to green for val=100, passing by yellow.
+	 * Colors are defined by [game_config] keys
+	 * red_green_scale and red_green_scale_text
+	 */
+
+	Uint32 red_to_green(int val, bool for_text = true);
+
 	extern const version_info wesnoth_version;
 	extern const version_info min_savegame_version;
 	extern const std::string  test_version;
-
-	/**
-	 * Tests whether the given version is compatible with the
-	 * running version, i.e. >= MIN_SAVEGAME_VERSION.
-	 * See wesconfig.h for the criteria used for this evaluation.
-	 */
-	bool is_compatible_savegame_version(const std::string& v);
 }
 
 #endif
