@@ -1,6 +1,6 @@
-/* $Id: game_display.cpp 35509 2009-05-09 16:25:14Z silene $ */
+/* $Id: game_display.cpp 42827 2010-05-24 14:37:58Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,20 @@
 
 #include "global.hpp"
 
+#include "game_display.hpp"
+
+#include "wesconfig.h"
+
+#ifdef HAVE_LIBDBUS
+#include <dbus/dbus.h>
+#endif
+
+#ifdef HAVE_GROWL
+#include <Growl/GrowlApplicationBridge-Carbon.h>
+#include <Carbon/Carbon.h>
+Growl_Delegate growl_obj;
+#endif
+
 #include "actions.hpp"
 #include "foreach.hpp"
 #include "halo.hpp"
@@ -27,28 +41,29 @@
 #include "map_label.hpp"
 #include "marked-up_text.hpp"
 #include "game_preferences.hpp"
-#include "gamestatus.hpp"
+#include "tod_manager.hpp"
 #include "sound.hpp"
 
+static lg::log_domain log_display("display");
+#define ERR_DP LOG_STREAM(err, log_display)
+#define LOG_DP LOG_STREAM(info, log_display)
 
-
-#define ERR_DP LOG_STREAM(err, display)
-#define INFO_DP LOG_STREAM(info, display)
+static lg::log_domain log_engine("engine");
+#define ERR_NG LOG_STREAM(err, log_engine)
 
 std::map<map_location,fixed_t> game_display::debugHighlights_;
 
 game_display::game_display(unit_map& units, CVideo& video, const gamemap& map,
-		const gamestatus& status, const std::vector<team>& t,
-		const config& theme_cfg, const config& cfg, const config& level) :
-		display(video, &map, theme_cfg, cfg, level),
+		const tod_manager& tod, const std::vector<team>& t,
+		const config& theme_cfg, const config& level) :
+		display(video, &map, theme_cfg, level),
 		units_(units),
 		temp_unit_(NULL),
-		temp_unit_loc_(),
 		attack_indicator_src_(),
 		attack_indicator_dst_(),
 		energy_bar_rects_(),
 		route_(),
-		status_(status),
+		tod_manager_(tod),
 		teams_(t),
 		level_(level),
 		invalidateUnit_(true),
@@ -88,7 +103,7 @@ game_display::game_display(unit_map& units, CVideo& video, const gamemap& map,
 			flag = game_config::flag_image;
 		}
 
-		LOG_STREAM(info, display) << "Adding flag for team " << i << " from animation " << flag << "\n";
+		LOG_DP << "Adding flag for team " << i << " from animation " << flag << "\n";
 
 		// Must recolor flag image
 		animated<image::locator> temp_anim;
@@ -125,10 +140,10 @@ game_display* game_display::create_dummy_display(CVideo& video)
 	static unit_map dummy_umap;
 	static config dummy_cfg;
 	static gamemap dummy_map(dummy_cfg, "");
-	static gamestatus dummy_status(dummy_cfg, 0);
+	static tod_manager dummy_tod(dummy_cfg, 0);
 	static std::vector<team> dummy_teams;
-	return new game_display(dummy_umap, video, dummy_map, dummy_status,
-			dummy_teams, dummy_cfg, dummy_cfg, dummy_cfg);
+	return new game_display(dummy_umap, video, dummy_map, dummy_tod,
+			dummy_teams, dummy_cfg, dummy_cfg);
 }
 
 game_display::~game_display()
@@ -140,10 +155,10 @@ game_display::~game_display()
 
 void game_display::new_turn()
 {
-	const time_of_day& tod = status_.get_time_of_day();
+	const time_of_day& tod = tod_manager_.get_time_of_day();
 
 	if( !first_turn_) {
-		const time_of_day& old_tod = status_.get_previous_time_of_day();
+		const time_of_day& old_tod = tod_manager_.get_previous_time_of_day();
 
 		if(old_tod.image_mask != tod.image_mask) {
 			const surface old_mask(image::get_image(old_tod.image_mask,image::UNMASKED));
@@ -189,7 +204,7 @@ void game_display::new_turn()
 
 void game_display::adjust_colours(int r, int g, int b)
 {
-	const time_of_day& tod = status_.get_time_of_day();
+	const time_of_day& tod = tod_manager_.get_time_of_day();
 	image::set_colour_adjustment(tod.red+r,tod.green+g,tod.blue+b);
 }
 
@@ -205,13 +220,13 @@ void game_display::select_hex(map_location hex)
 
 void game_display::highlight_hex(map_location hex)
 {
-	unit_map::const_iterator u = find_visible_unit(units_, hex, get_map(), teams_, teams_[viewing_team()], !viewpoint_);
-	if (u != units_.end()) {
+	const unit *u = get_visible_unit(units_, hex, teams_[viewing_team()], !viewpoint_);
+	if (u) {
 		displayedUnitHex_ = hex;
 		invalidate_unit();
 	} else {
-		u = find_visible_unit(units_, mouseoverHex_, get_map(), teams_, teams_[viewing_team()], !viewpoint_);
-		if (u != units_.end()) {
+		u = get_visible_unit(units_, mouseoverHex_, teams_[viewing_team()], !viewpoint_);
+		if (u) {
 			// mouse moved from unit hex to non-unit hex
 			if (units_.count(selectedHex_)) {
 				displayedUnitHex_ = selectedHex_;
@@ -227,8 +242,8 @@ void game_display::highlight_hex(map_location hex)
 
 void game_display::display_unit_hex(map_location hex)
 {
-	unit_map::const_iterator u = find_visible_unit(units_, hex, get_map(), teams_, teams_[viewing_team()], !viewpoint_);
-	if (u != units_.end()) {
+	const unit *u = get_visible_unit(units_, hex, teams_[viewing_team()], !viewpoint_);
+	if (u) {
 		displayedUnitHex_ = hex;
 		invalidate_unit();
 	}
@@ -244,7 +259,7 @@ void game_display::invalidate_unit_after_move(const map_location& src, const map
 
 void game_display::scroll_to_leader(unit_map& units, int side, SCROLL_TYPE scroll_type,bool force)
 {
-	const unit_map::iterator leader = find_leader(units,side);
+	unit_map::const_iterator leader = units.find_leader(side);
 
 	if(leader != units_.end()) {
 		// YogiHH: I can't see why we need another key_handler here,
@@ -268,7 +283,9 @@ void game_display::pre_draw() {
 std::vector<map_location> game_display::get_invalidated_unit_locations() {
 	std::vector<map_location> unit_locations;
 	foreach (const map_location& loc, invalidated_) {
-		if ((temp_unit_ && temp_unit_loc_ == loc) || units_.find(loc) != units_.end()) {
+		if ((temp_unit_ && temp_unit_->get_location() == loc) ||
+		    units_.find(loc) != units_.end())
+		{
 			unit_locations.push_back(loc);
 		}
 	}
@@ -283,9 +300,8 @@ image::TYPE game_display::get_image_type(const map_location& loc) {
 		if (loc == mouseoverHex_ || loc == attack_indicator_src_) {
 			return image::BRIGHTENED;
 		} else if (loc == selectedHex_) {
-			unit_map::iterator un = find_visible_unit(units_, loc, get_map(),
-					teams_, teams_[currentTeam_], !viewpoint_);
-			if (un != units_.end() && !un->second.get_hidden()) {
+			const unit *un = get_visible_unit(units_, loc, teams_[currentTeam_], !viewpoint_);
+			if (un && !un->get_hidden()) {
 				return image::BRIGHTENED;
 			}
 		}
@@ -338,11 +354,11 @@ void game_display::draw_hex(const map_location& loc)
 
 	// Draw the time-of-day mask on top of the terrain in the hex.
 	// tod may differ from tod if hex is illuminated.
-	std::string tod_hex_mask = timeofday_at(status_,units_,loc,get_map()).image_mask;
+	const std::string& tod_hex_mask = tod_manager_.get_time_of_day(0, loc).image_mask;
 	if(tod_hex_mask1 != NULL || tod_hex_mask2 != NULL) {
 		drawing_buffer_add(LAYER_TERRAIN_FG, loc, tblit(xpos, ypos, tod_hex_mask1));
 		drawing_buffer_add(LAYER_TERRAIN_FG, loc, tblit(xpos, ypos, tod_hex_mask2));
-	} else if(tod_hex_mask != "") {
+	} else if(!tod_hex_mask.empty()) {
 		drawing_buffer_add(LAYER_TERRAIN_FG, loc, tblit(xpos, ypos,
 			image::get_image(tod_hex_mask,image::UNMASKED)));
 	}
@@ -358,11 +374,6 @@ void game_display::draw_hex(const map_location& loc)
 
 	// Footsteps indicating a movement path
 	drawing_buffer_add(LAYER_TERRAIN_TMP_BG, loc, tblit(xpos, ypos, footsteps_images(loc)));
-	// Draw cross images for debug highlights
-	if(game_config::debug && debugHighlights_.count(loc)) {
-		drawing_buffer_add(LAYER_TERRAIN_TMP_BG, loc, tblit(xpos, ypos,
-			image::get_image(game_config::cross_image, image::UNMASKED)));
-	}
 	// Draw the attack direction indicator
 	if(on_map && loc == attack_indicator_src_) {
 		drawing_buffer_add(LAYER_ATTACK_INDICATOR, loc, tblit(xpos, ypos,
@@ -384,12 +395,20 @@ void game_display::draw_hex(const map_location& loc)
 	if(!is_shrouded && on_map) {
 		draw_movement_info(loc);
 	}
+
+	if(game_config::debug) {
+		int debugH = debugHighlights_[loc];
+		if (debugH) {
+			std::string txt = lexical_cast<std::string>(debugH);
+			draw_text_in_hex(loc, LAYER_MOVE_INFO, txt, 18, font::BAD_COLOUR);
+		}
+	}
 	//simulate_delay += 1;
 }
 
 void game_display::update_time_of_day()
 {
-	tod_ = status_.get_time_of_day();
+	tod_ = tod_manager_.get_time_of_day();
 }
 
 void game_display::redraw_units(const std::vector<map_location>& invalidated_unit_locations)
@@ -399,11 +418,11 @@ void game_display::redraw_units(const std::vector<map_location>& invalidated_uni
 	foreach (map_location loc, invalidated_unit_locations) {
 		unit_map::iterator u_it = units_.find(loc);
 		if (u_it != units_.end()) {
-			u_it->second.redraw_unit(*this, loc);
+			u_it->second.redraw_unit();
 			//simulate_delay += 1;
 		}
-		if (temp_unit_ && temp_unit_loc_ == loc) {
-			temp_unit_->redraw_unit(*this, temp_unit_loc_);
+		if (temp_unit_ && temp_unit_->get_location() == loc) {
+			temp_unit_->redraw_unit();
 			//simulate_delay += 1;
 		}
 	}
@@ -411,31 +430,17 @@ void game_display::redraw_units(const std::vector<map_location>& invalidated_uni
 
 void game_display::draw_report(reports::TYPE report_num)
 {
-	bool brighten;
-
 	if(!team_valid()) {
 		return;
 	}
 
-	reports::report report = reports::generate_report(report_num,report_,get_map(),
-							  units_, teams_,
+	reports::report report = reports::generate_report(report_num, report_,
 							  teams_[viewing_team()],
 							  size_t(currentTeam_+1),size_t(activeTeam_+1),
 							  selectedHex_, mouseoverHex_, displayedUnitHex_,
-							  status_, observers_, level_, !viewpoint_);
+		observers_, level_, !viewpoint_);
 
-	brighten = false;
-	if(report_num == reports::TIME_OF_DAY) {
-		time_of_day tod = timeofday_at(status_,units_,mouseoverHex_,get_map());
-		// Don't show illuminated time on fogged/shrouded tiles
-		if (fogged(mouseoverHex_) || shrouded(mouseoverHex_)) {
-
-			tod = status_.get_time_of_day(false,mouseoverHex_);
-		}
-		brighten = (tod.bonus_modified > 0);
-	}
-
-	refresh_report(report_num, report, brighten);
+	refresh_report(report_num, report);
 }
 
 void game_display::draw_game_status()
@@ -562,7 +567,7 @@ void game_display::draw_bar(const std::string& image, int xpos, int ypos,
 	drawing_buffer_add(LAYER_UNIT_BAR, loc, tblit(xpos, ypos, surf, top));
 	drawing_buffer_add(LAYER_UNIT_BAR, loc, tblit(xpos, ypos + top.h, surf, bot));
 
-	const size_t unfilled = static_cast<const size_t>(height*(1.0 - filled));
+	size_t unfilled = static_cast<size_t>(height * (1.0 - filled));
 
 	if(unfilled < height && alpha >= ftofxp(0.3)) {
 		const Uint8 r_alpha = std::min<unsigned>(unsigned(fxpmult(alpha,255)),255);
@@ -583,11 +588,11 @@ void game_display::set_game_mode(const tgame_mode game_mode)
 
 void game_display::draw_movement_info(const map_location& loc)
 {
-	// Search if there is a waypoint here
-	std::map<map_location, paths::route::waypoint>::iterator w = route_.waypoints.find(loc);
+	// Search if there is a mark here
+	pathfind::marked_route::mark_map::iterator w = route_.marks.find(loc);
 
 	// Don't use empty route or the first step (the unit will be there)
-	if(w != route_.waypoints.end()
+	if(w != route_.marks.end()
 				&& !route_.steps.empty() && route_.steps.front() != loc) {
 		const unit_map::const_iterator un = units_.find(route_.steps.front());
 		if(un != units_.end()) {
@@ -596,10 +601,11 @@ void game_display::draw_movement_info(const map_location& loc)
 			std::stringstream def_text;
 			def_text << def << "%";
 
-			// With 11 colors, the last one will be used only for def=100
-			int val = (game_config::defense_color_scale.size()-1) * def/100;
-			SDL_Color color = int_to_color(game_config::defense_color_scale[val]);
-			draw_text_in_hex(loc, LAYER_MOVE_INFO, def_text.str(), 18, color);
+			SDL_Color color = int_to_color(game_config::red_to_green(def, false));
+
+			// simple mark (no turn point) use smaller font
+			int def_font = w->second.turns > 0 ? 18 : 16;
+			draw_text_in_hex(loc, LAYER_MOVE_INFO, def_text.str(), def_font, color);
 
 			int xpos = get_location_x(loc);
 			int ypos = get_location_y(loc);
@@ -619,12 +625,18 @@ void game_display::draw_movement_info(const map_location& loc)
 					image::get_image("misc/capture.png", image::UNMASKED)));
 			}
 
+			if (w->second.pass_here) {
+				drawing_buffer_add(LAYER_MOVE_INFO, loc, tblit(xpos, ypos,
+					image::get_image("misc/waypoint.png", image::UNMASKED)));
+			}
+
 			//we display turn info only if different from a simple last "1"
-			if (w->second.turns > 1 || loc != route_.steps.back()) {
+			if (w->second.turns > 1 || (w->second.turns == 1 && loc != route_.steps.back())) {
 				std::stringstream turns_text;
 				turns_text << w->second.turns;
 				draw_text_in_hex(loc, LAYER_MOVE_INFO, turns_text.str(), 17, font::NORMAL_COLOUR, 0.5,0.8);
 			}
+
 			// The hex is full now, so skip the "show enemy moves"
 			return;
 		}
@@ -674,7 +686,7 @@ std::vector<surface> game_display::footsteps_images(const map_location& loc)
 	// and the second for the last step
 	const int second_half = (i+1 == route_.steps.end()) ? 0 : 1;
 
-	for (int h = first_half; h <= second_half; h++) {
+	for (int h = first_half; h <= second_half; ++h) {
 		const std::string sense( h==0 ? "-in" : "-out" );
 
 		if (!tiles_adjacent(*(i+(h-1)), *(i+h))) {
@@ -720,7 +732,7 @@ surface game_display::get_flag(const map_location& loc)
 		  (!fogged(loc) || !teams_[currentTeam_].is_enemy(i+1)))
 		{
 			flags_[i].update_last_draw_time();
-			image::locator image_flag = preferences::animate_map() ?
+			const image::locator& image_flag = preferences::animate_map() ?
 				flags_[i].get_current_frame() : flags_[i].get_first_frame();
 			return image::get_image(image_flag, image::SCALED_TO_HEX);
 		}
@@ -729,19 +741,17 @@ surface game_display::get_flag(const map_location& loc)
 	return surface(NULL);
 }
 
-void game_display::highlight_reach(const paths &paths_list)
+void game_display::highlight_reach(const pathfind::paths &paths_list)
 {
 	unhighlight_reach();
 	highlight_another_reach(paths_list);
 }
 
-void game_display::highlight_another_reach(const paths &paths_list)
+void game_display::highlight_another_reach(const pathfind::paths &paths_list)
 {
-	paths::routes_map::const_iterator r;
-
 	// Fold endpoints of routes into reachability map.
-	for (r = paths_list.routes.begin(); r != paths_list.routes.end(); ++r) {
-		reach_map_[r->first]++;
+	foreach (const pathfind::paths::step &dest, paths_list.destinations) {
+		reach_map_[dest.curr]++;
 	}
 	reach_map_changed_ = true;
 }
@@ -801,7 +811,7 @@ void game_display::invalidate_route()
 	}
 }
 
-void game_display::set_route(const paths::route* route)
+void game_display::set_route(const pathfind::marked_route *route)
 {
 	invalidate_route();
 
@@ -809,7 +819,7 @@ void game_display::set_route(const paths::route* route)
 		route_ = *route;
 	} else {
 		route_.steps.clear();
-		route_.waypoints.clear();
+		route_.marks.clear();
 	}
 
 	invalidate_route();
@@ -885,26 +895,20 @@ void game_display::invalidate_animations()
 {
 	new_animation_frame();
 	display::invalidate_animations();
-	for (unit_map::iterator u = units_.begin(),
-	     u_end = units_.end(); u != u_end; ++u)
-	{
-		if (!tile_nearly_on_screen(u->first)) continue;
-		u->second.refresh(*this, u->first);
-	}
-	if (temp_unit_ && tile_nearly_on_screen(temp_unit_loc_))
-		temp_unit_->refresh(*this, temp_unit_loc_);
+	unit_map::iterator unit;
+	for(unit=units_.begin() ; unit != units_.end() ; ++unit)
+		unit->second.refresh();
+	if (temp_unit_ )
+		temp_unit_->refresh();
 	bool new_inval = true;
 	while(new_inval) {
 		new_inval = false;
-		for (unit_map::iterator u = units_.begin(),
-		     u_end = units_.end(); u != u_end; ++u)
-		{
-			if (!tile_nearly_on_screen(u->first)) continue;
-			new_inval |= u->second.invalidate(u->first);
+		for(unit=units_.begin() ; unit != units_.end() ; ++unit) {
+			new_inval |= unit->second.invalidate(unit->second.get_location());
 		}
-		if (temp_unit_ && tile_nearly_on_screen(temp_unit_loc_)) {
+		if (temp_unit_ ) {
 			//new_inval |=invalidate(temp_unit_loc_);
-			new_inval |=temp_unit_->invalidate(temp_unit_loc_);
+			new_inval |=temp_unit_->invalidate(temp_unit_->get_location());
 		}
 	}
 }
@@ -915,18 +919,17 @@ void game_display::debug_highlight(const map_location& loc, fixed_t amount)
 	debugHighlights_[loc] += amount;
 }
 
-void game_display::place_temporary_unit(unit &u, const map_location& loc)
+void game_display::place_temporary_unit(unit &u)
 {
 	temp_unit_ = &u;
-	temp_unit_loc_ = loc;
-	invalidate(loc);
+	invalidate(u.get_location());
 }
 
 void game_display::remove_temporary_unit()
 {
 	if(!temp_unit_) return;
 
-	invalidate(temp_unit_loc_);
+	invalidate(temp_unit_->get_location());
 	// Redraw with no location to get rid of haloes
 	temp_unit_->clear_haloes();
 	temp_unit_ = NULL;
@@ -1006,13 +1009,15 @@ void game_display::write_overlays(config& cfg) const
 
 void game_display::parse_team_overlays()
 {
-	for (game_display::overlay_map::const_iterator overlay = overlays_.begin(); overlay != overlays_.end(); ++overlay) {
-
-		if (overlay->second.team_name != "" &&
-		bool(overlay->second.team_name.find(teams_[playing_team()].team_name())+1) !=
-		bool(overlay->second.team_name.find(teams_[playing_team()-1 < teams_.size() ? playing_team()-1 : teams_.size()-1].team_name())+1))
+	const team& curr_team = teams_[playing_team()];
+	const team& prev_team = teams_[playing_team()-1 < teams_.size() ? playing_team()-1 : teams_.size()-1];
+	foreach (const game_display::overlay_map::value_type i, overlays_) {
+		const overlay& ov = i.second;
+		if (!ov.team_name.empty() &&
+			((ov.team_name.find(curr_team.team_name()) + 1) != 0) !=
+			((ov.team_name.find(prev_team.team_name()) + 1) != 0))
 		{
-			invalidate(overlay->first);
+			invalidate(i.first);
 		}
 	}
 }
@@ -1026,6 +1031,200 @@ std::string game_display::current_team_name() const
 	return std::string();
 }
 
+#ifdef HAVE_LIBDBUS
+/** Use KDE 4 notifications. */
+static bool kde_style = false;
+
+struct wnotify
+{
+	uint32_t id;
+	std::string owner;
+	std::string message;
+};
+
+static std::list<wnotify> notifications;
+
+static DBusHandlerResult filter_dbus_signal(DBusConnection *, DBusMessage *buf, void *)
+{
+	if (!dbus_message_is_signal(buf, "org.freedesktop.Notifications", "NotificationClosed")) {
+		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	}
+
+	uint32_t id;
+	dbus_message_get_args(buf, NULL,
+		DBUS_TYPE_UINT32, &id,
+		DBUS_TYPE_INVALID);
+
+	std::list<wnotify>::iterator i = notifications.begin(),
+		i_end = notifications.end();
+	while (i != i_end && i->id != id) ++i;
+	if (i != i_end)
+		notifications.erase(i);
+
+	return DBUS_HANDLER_RESULT_HANDLED;
+}
+
+static DBusConnection *get_dbus_connection()
+{
+	static bool initted = false;
+	static DBusConnection *connection = NULL;
+	if (!initted)
+	{
+		initted = true;
+		if (utils::string_bool(preferences::get("disable_notifications"))) {
+			return NULL;
+		}
+		if (getenv("KDE_SESSION_VERSION")) {
+			// This variable is defined for KDE 4 only.
+			kde_style = true;
+		}
+		DBusError err;
+		dbus_error_init(&err);
+		connection = dbus_bus_get(DBUS_BUS_SESSION, &err);
+		if (!connection) {
+			ERR_DP << "Failed to open DBus session: " << err.message << '\n';
+			dbus_error_free(&err);
+			return NULL;
+		}
+		dbus_connection_add_filter(connection, filter_dbus_signal, NULL, NULL);
+	}
+	if (connection) {
+		dbus_connection_read_write(connection, 0);
+		while (dbus_connection_dispatch(connection) == DBUS_DISPATCH_DATA_REMAINS) {}
+	}
+	return connection;
+}
+
+static uint32_t send_dbus_notification(DBusConnection *connection, uint32_t replaces_id,
+	const std::string &owner, const std::string &message)
+{
+	DBusMessage *buf = dbus_message_new_method_call(
+		kde_style ? "org.kde.VisualNotifications" : "org.freedesktop.Notifications",
+		kde_style ? "/VisualNotifications" : "/org/freedesktop/Notifications",
+		kde_style ? "org.kde.VisualNotifications" : "org.freedesktop.Notifications",
+		"Notify");
+	const char *app_name = "Battle for Wesnoth";
+	dbus_message_append_args(buf,
+		DBUS_TYPE_STRING, &app_name,
+		DBUS_TYPE_UINT32, &replaces_id,
+		DBUS_TYPE_INVALID);
+	if (kde_style) {
+		const char *event_id = "";
+		dbus_message_append_args(buf,
+			DBUS_TYPE_STRING, &event_id,
+			DBUS_TYPE_INVALID);
+	}
+	std::string app_icon_ = game_config::path + "/images/wesnoth-icon-small.png";
+	const char *app_icon = app_icon_.c_str();
+	const char *summary = owner.c_str();
+	const char *body = message.c_str();
+	const char **actions = NULL;
+	dbus_message_append_args(buf,
+		DBUS_TYPE_STRING, &app_icon,
+		DBUS_TYPE_STRING, &summary,
+		DBUS_TYPE_STRING, &body,
+		DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &actions, 0,
+		DBUS_TYPE_INVALID);
+	DBusMessageIter iter, hints;
+	dbus_message_iter_init_append(buf, &iter);
+	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY, "{sv}", &hints);
+	dbus_message_iter_close_container(&iter, &hints);
+	int expire_timeout = kde_style ? 5000 : -1;
+	dbus_message_append_args(buf,
+		DBUS_TYPE_INT32, &expire_timeout,
+		DBUS_TYPE_INVALID);
+	DBusError err;
+	dbus_error_init(&err);
+	DBusMessage *ret = dbus_connection_send_with_reply_and_block(connection, buf, 1000, &err);
+	dbus_message_unref(buf);
+	if (!ret) {
+		ERR_DP << "Failed to send visual notification: " << err.message << '\n';
+		dbus_error_free(&err);
+		if (kde_style) {
+			ERR_DP << " Retrying with the freedesktop protocol.\n";
+			kde_style = false;
+			return send_dbus_notification(connection, replaces_id, owner, message);
+		}
+		return 0;
+	}
+	uint32_t id;
+	dbus_message_get_args(ret, NULL,
+		DBUS_TYPE_UINT32, &id,
+		DBUS_TYPE_INVALID);
+	dbus_message_unref(ret);
+	// TODO: remove once closing signals for KDE are handled in filter_dbus_signal.
+	if (kde_style) return 0;
+	return id;
+}
+#endif
+
+#if defined(HAVE_LIBDBUS) || defined(HAVE_GROWL)
+void game_display::send_notification(const std::string& owner, const std::string& message)
+#else
+void game_display::send_notification(const std::string& /*owner*/, const std::string& /*message*/)
+#endif
+{
+#if defined(HAVE_LIBDBUS) || defined(HAVE_GROWL)
+	Uint8 app_state = SDL_GetAppState();
+
+	// Do not show notifications when the window is visible...
+	if ((app_state & SDL_APPACTIVE) != 0)
+	{
+		// ... and it has a focus.
+		if ((app_state & (SDL_APPMOUSEFOCUS | SDL_APPINPUTFOCUS)) != 0) {
+			return;
+		}
+	}
+#endif
+
+#ifdef HAVE_LIBDBUS
+	DBusConnection *connection = get_dbus_connection();
+	if (!connection) return;
+
+	std::list<wnotify>::iterator i = notifications.begin(),
+		i_end = notifications.end();
+	while (i != i_end && i->owner != owner) ++i;
+
+	if (i != i_end) {
+		i->message += "\n";
+		i->message += message;
+		send_dbus_notification(connection, i->id, owner, i->message);
+		return;
+	}
+
+	uint32_t id = send_dbus_notification(connection, 0, owner, message);
+	if (!id) return;
+	wnotify visual;
+	visual.id = id;
+	visual.owner = owner;
+	visual.message = message;
+	notifications.push_back(visual);
+#endif
+
+#ifdef HAVE_GROWL
+	CFStringRef app_name = CFStringCreateWithCString(NULL, "Wesnoth", kCFStringEncodingUTF8);
+	CFStringRef cf_owner = CFStringCreateWithCString(NULL, owner.c_str(), kCFStringEncodingUTF8);
+	CFStringRef cf_message = CFStringCreateWithCString(NULL, message.c_str(), kCFStringEncodingUTF8);
+	//Should be changed as soon as there are more than 2 types of notifications
+	CFStringRef cf_note_name = CFStringCreateWithCString(NULL, owner == "Turn changed" ? "Turn changed" : "Chat message", kCFStringEncodingUTF8);
+
+	growl_obj.applicationName = app_name;
+	growl_obj.registrationDictionary = NULL;
+	growl_obj.applicationIconData = NULL;
+	growl_obj.growlIsReady = NULL;
+	growl_obj.growlNotificationWasClicked = NULL;
+	growl_obj.growlNotificationTimedOut = NULL;
+
+	Growl_SetDelegate(&growl_obj);
+	Growl_NotifyWithTitleDescriptionNameIconPriorityStickyClickContext(cf_owner, cf_message, cf_note_name, NULL, NULL, NULL, NULL);
+
+	CFRelease(app_name);
+	CFRelease(cf_owner);
+	CFRelease(cf_message);
+	CFRelease(cf_note_name);
+#endif
+}
+
 void game_display::set_team(size_t teamindex, bool show_everything)
 {
 	assert(teamindex < teams_.size());
@@ -1066,7 +1265,7 @@ namespace {
 }
 
 void game_display::add_chat_message(const time_t& time, const std::string& speaker,
-		int side, const std::string& message, game_display::MESSAGE_TYPE type,
+		int side, const std::string& message, events::chat_handler::MESSAGE_TYPE type,
 		bool bell)
 {
 	const bool whisper = speaker.find("whisper: ") == 0;
@@ -1074,13 +1273,13 @@ void game_display::add_chat_message(const time_t& time, const std::string& speak
 	if (whisper) {
 		sender.assign(speaker, 9, speaker.size());
 	}
-	if (!preferences::show_lobby_join(sender, message)) return;
+	if (!preferences::parse_should_show_lobby_join(sender, message)) return;
 	if (preferences::is_ignored(sender)) return;
 
-	preferences::admin_authentication(sender, message);
+	preferences::parse_admin_authentication(sender, message);
 
 	if (bell) {
-		if ((type == MESSAGE_PRIVATE && (!is_observer() || whisper))
+		if ((type == events::chat_handler::MESSAGE_PRIVATE && (!is_observer() || whisper))
 			|| utils::word_match(message, preferences::login())) {
 			sound::play_UI_sound(game_config::sounds::receive_message_highlight);
 		} else if (preferences::is_friend(sender)) {
@@ -1094,8 +1293,7 @@ void game_display::add_chat_message(const time_t& time, const std::string& speak
 
 	bool action = false;
 
-	// We also need to disable markup for the message itself
-	std::string msg(1, font::NULL_MARKUP);
+	std::string msg;
 
 	if (message.find("/me ") == 0) {
 		msg.assign(message, 4, message.size());
@@ -1109,13 +1307,14 @@ void game_display::add_chat_message(const time_t& time, const std::string& speak
 		// so now catch the exception and ignore the message.
 		msg = font::word_wrap_text(msg,font::SIZE_SMALL,map_outside_area().w*3/4);
 	} catch (utils::invalid_utf8_exception&) {
-		LOG_STREAM(err, engine) << "Invalid utf-8 found, chat message is ignored.\n";
+		ERR_NG << "Invalid utf-8 found, chat message is ignored.\n";
 		return;
 	}
 
 	int ypos = chat_message_x;
 	for(std::vector<chat_message>::const_iterator m = chat_messages_.begin(); m != chat_messages_.end(); ++m) {
-		ypos += font::get_floating_label_rect(m->handle).h;
+		ypos += std::max(font::get_floating_label_rect(m->handle).h,
+			font::get_floating_label_rect(m->speaker_handle).h);
 	}
 	SDL_Color speaker_colour = {255,255,255,255};
 	if(side >= 1) {
@@ -1126,7 +1325,7 @@ void game_display::add_chat_message(const time_t& time, const std::string& speak
 	std::stringstream str;
 	std::stringstream message_str;
 
-	if(type == MESSAGE_PUBLIC) {
+	if(type ==  events::chat_handler::MESSAGE_PUBLIC) {
 		if(action) {
 			str << "<" << speaker << " " << msg << ">";
 			message_colour = speaker_colour;
@@ -1148,20 +1347,24 @@ void game_display::add_chat_message(const time_t& time, const std::string& speak
 		}
 	}
 
-	// prepend message with timestamp and disable markup
+	// Prepend message with timestamp.
 	std::stringstream message_complete;
-	message_complete << font::NULL_MARKUP;
-	message_complete << preferences::get_chat_timestamp(time);
-	message_complete << str.str();
+	message_complete << preferences::get_chat_timestamp(time) << str.str();
 
 	const SDL_Rect rect = map_outside_area();
-	const int speaker_handle = font::add_floating_label(message_complete.str(),font::SIZE_SMALL,speaker_colour,
-		rect.x+chat_message_x,rect.y+ypos,
-		0,0,-1,rect,font::LEFT_ALIGN,&chat_message_bg,chat_message_border);
+	int speaker_handle = font::add_floating_label(message_complete.str(),
+		font::SIZE_SMALL, speaker_colour, rect.x + chat_message_x,
+		rect.y + ypos, 0, 0, -1, rect, font::LEFT_ALIGN, &chat_message_bg,
+		chat_message_border, font::ANCHOR_LABEL_SCREEN, false);
 
-	const int message_handle = font::add_floating_label(message_str.str(),font::SIZE_SMALL,message_colour,
-		rect.x + chat_message_x + font::get_floating_label_rect(speaker_handle).w,rect.y+ypos,
-		0,0,-1,rect,font::LEFT_ALIGN,&chat_message_bg,chat_message_border);
+	int message_handle = font::add_floating_label(message_str.str(),
+		font::SIZE_SMALL, message_colour,
+		rect.x + chat_message_x + font::get_floating_label_rect(speaker_handle).w,
+		rect.y + ypos, 0, 0, -1, rect, font::LEFT_ALIGN, &chat_message_bg,
+		chat_message_border, font::ANCHOR_LABEL_SCREEN, false);
+
+	// Send system notification if appropriate.
+	send_notification(speaker, message);
 
 	chat_messages_.push_back(chat_message(speaker_handle,message_handle));
 
@@ -1170,21 +1373,24 @@ void game_display::add_chat_message(const time_t& time, const std::string& speak
 
 void game_display::prune_chat_messages(bool remove_all)
 {
-	const unsigned int message_ttl = remove_all ? 0 : 1200000;
-	const unsigned int max_chat_messages = preferences::chat_lines();
-	if(chat_messages_.empty() == false && (chat_messages_.front().created_at+message_ttl < SDL_GetTicks() || chat_messages_.size() > max_chat_messages)) {
-		const int movement = font::get_floating_label_rect(chat_messages_.front().handle).h;
+	unsigned message_ttl = remove_all ? 0 : 1200000;
+	unsigned max_chat_messages = preferences::chat_lines();
+	int movement = 0;
 
-		font::remove_floating_label(chat_messages_.front().speaker_handle);
-		font::remove_floating_label(chat_messages_.front().handle);
+	while (!chat_messages_.empty() &&
+	       (chat_messages_.front().created_at + message_ttl < SDL_GetTicks() ||
+	        chat_messages_.size() > max_chat_messages))
+	{
+		const chat_message &old = chat_messages_.front();
+		movement += font::get_floating_label_rect(old.handle).h;
+		font::remove_floating_label(old.speaker_handle);
+		font::remove_floating_label(old.handle);
 		chat_messages_.erase(chat_messages_.begin());
+	}
 
-		for(std::vector<chat_message>::const_iterator i = chat_messages_.begin(); i != chat_messages_.end(); ++i) {
-			font::move_floating_label(i->speaker_handle,0,-movement);
-			font::move_floating_label(i->handle,0,-movement);
-		}
-
-		prune_chat_messages(remove_all);
+	foreach (const chat_message &cm, chat_messages_) {
+		font::move_floating_label(cm.speaker_handle, 0, - movement);
+		font::move_floating_label(cm.handle, 0, - movement);
 	}
 }
 
