@@ -1,6 +1,6 @@
-/* $Id: game_display.hpp 33266 2009-03-01 20:19:58Z boucman $ */
+/* $Id: game_display.hpp 40856 2010-01-25 15:37:35Z esr $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -18,15 +18,16 @@
 #define GAME_DISPLAY_H_INCLUDED
 
 class config;
-class gamestatus;
+class tod_manager;
 class team;
 class unit;
 class unit_map;
 
 #include "animated.hpp"
+#include "chat_events.hpp"
 #include "display.hpp"
 #include "image.hpp"
-#include "pathfind.hpp"
+#include "pathfind/pathfind.hpp"
 
 #include "SDL.h"
 
@@ -42,9 +43,9 @@ class game_display : public display
 {
 public:
 	game_display(unit_map& units, CVideo& video,
-			const gamemap& map, const gamestatus& status,
+			const gamemap& map, const tod_manager& tod_manager,
 			const std::vector<team>& t, const config& theme_cfg,
-			const config& cfg, const config& level);
+			const config& level);
 
 	static game_display* create_dummy_display(CVideo& video);
 
@@ -102,23 +103,23 @@ public:
 	 * for the unit to move along.
 	 * All other paths will be greyed out.
 	 */
-	void highlight_reach(const paths &paths_list);
+	void highlight_reach(const pathfind::paths &paths_list);
 
 	/**
 	 * Add more paths to highlight.  Print numbers where they overlap.
 	 * Used only by Show Enemy Moves.
 	 */
-	void highlight_another_reach(const paths &paths_list);
+  void highlight_another_reach(const pathfind::paths &paths_list);
 
 	/** Reset highlighting of paths. */
 	void unhighlight_reach();
 
 	/**
 	 * Sets the route along which footsteps are drawn to show movement of a
-	 * unit. If NULL, no route is displayed. route does not have to remain
+	 * unit. If NULL, no route is displayed. @a route does not have to remain
 	 * valid after being set.
 	 */
-	void set_route(const paths::route* route);
+  void set_route(const pathfind::marked_route *route);
 
 	/** Function to float a label above a tile */
 	void float_label(const map_location& loc, const std::string& text,
@@ -133,7 +134,6 @@ public:
 	/** Draws the movement info (turns available) for a given location. */
 	void draw_movement_info(const map_location& loc);
 
-	const gamestatus &get_game_status() { return status_; }
 	void draw_report(reports::TYPE report_num);
 
 	/** Function to invalidate that unit status displayed on the sidebar. */
@@ -186,7 +186,7 @@ protected:
 
 public:
 	/** Temporarily place a unit on map (moving: can overlap others). */
-	void place_temporary_unit(unit &u, const map_location& loc);
+	void place_temporary_unit(unit &u);
 	void remove_temporary_unit();
 
 	/** Set the attack direction indicator. */
@@ -229,6 +229,8 @@ public:
 	//void draw_terrain_palette(int x, int y, terrain_type::TERRAIN selected);
 	t_translation::t_terrain get_terrain_on(int palx, int paly, int x, int y);
 
+	void send_notification(const std::string& owner, const std::string& message);
+
 	/**
 	 * Sets the team controlled by the player using the computer.
 	 *
@@ -237,11 +239,11 @@ public:
 	 */
 	void set_team(size_t team, bool observe=false);
 	void set_playing_team(size_t team);
-	size_t get_playing_team() const {return activeTeam_;};
-	const std::vector<team>& get_teams() {return teams_;};
+	size_t get_playing_team() const {return activeTeam_;}
+	const std::vector<team>& get_teams() {return teams_;}
 
-	unit_map& get_units() {return units_;};
-	const unit_map& get_const_units() const {return units_;};
+	unit_map& get_units() {return units_;}
+	const unit_map& get_const_units() const {return units_;}
 
 	/**
 	 * Draws a cross on a tile to emphasize something there.
@@ -253,6 +255,7 @@ public:
 
 	/** The viewing team is the team currently viewing the game. */
 	size_t viewing_team() const { return currentTeam_; }
+	int viewing_side() const { return currentTeam_ + 1; }
 
 	/** The playing team is the team whose turn it is. */
 	size_t playing_team() const { return activeTeam_; }
@@ -264,9 +267,8 @@ public:
 	void remove_observer(const std::string& name) { observers_.erase(name); }
 	const std::set<std::string>& observers() const { return observers_; }
 
-	enum MESSAGE_TYPE { MESSAGE_PUBLIC, MESSAGE_PRIVATE };
 	void add_chat_message(const time_t& time, const std::string& speaker,
-			int side, const std::string& msg, MESSAGE_TYPE type, bool bell);
+		int side, const std::string& msg, events::chat_handler::MESSAGE_TYPE type, bool bell);
 	void clear_chat_messages() { prune_chat_messages(true); }
 
 	void begin_game();
@@ -305,7 +307,6 @@ private:
 	unit_map& units_;
 
 	unit *temp_unit_;
-	map_location temp_unit_loc_;
 
 	// Locations of the attack direction indicator's parts
 	map_location attack_indicator_src_;
@@ -319,9 +320,9 @@ private:
 	const SDL_Rect& calculate_energy_bar(surface surf);
 	std::map<surface,SDL_Rect> energy_bar_rects_;
 
-	paths::route route_;
+	pathfind::marked_route route_;
 
-	const gamestatus& status_;
+	const tod_manager& tod_manager_;
 
 	const std::vector<team>& teams_;
 
