@@ -1,6 +1,6 @@
-/* $Id: game_preferences.cpp 28552 2008-08-13 14:57:04Z ivanovic $ */
+/* $Id: game_preferences.cpp 42333 2010-04-30 23:22:12Z loonycyborg $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -16,27 +16,20 @@
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
-#include "config.hpp"
-#include "filesystem.hpp"
+#include "foreach.hpp"
+#include "game_display.hpp"
 #include "game_preferences.hpp"
 #include "gamestatus.hpp"
 #include "gettext.hpp"
-#include "hotkeys.hpp"
 #include "log.hpp"
+#include "map.hpp"
 #include "network.hpp" // ping_timeout
-#include "sound.hpp"
-#include "settings.hpp"
-#include "util.hpp"
-#include "video.hpp" // non_interactive()
-#include "wesconfig.h"
-#include "wml_exception.hpp"
-#include "serialization/parser.hpp"
 #include "serialization/string_utils.hpp"
+#include "settings.hpp"
+#include "unit.hpp"
+#include "unit_map.hpp"
+#include "wml_exception.hpp"
 
-#include <cstdlib>
-#include <iostream>
-#include <iterator>
-#include <sstream>
 
 namespace {
 
@@ -47,25 +40,22 @@ bool haloes = true;
 std::set<std::string> encountered_units_set;
 std::set<t_translation::t_terrain> encountered_terrains_set;
 
-//! Add a nick to the specified relation setting.
-void add_relation(const std::string nick, const std::string relation) {
-	std::vector<std::string> r = utils::split(preferences::get(relation));
-	r.push_back(nick);
-	preferences::set(relation, utils::join(r));
-}
+std::map<std::string, std::vector<std::string> > history_map;
+const unsigned max_history_saved = 50;
 
-//! Remove a nick from the specified relation setting.
-void remove_relation(const std::string nick, const std::string relation) {
-	std::vector<std::string> r = utils::split(preferences::get(relation));
-	r.erase(std::remove(r.begin(), r.end(), nick), r.end());
-	preferences::set(relation, utils::join(r));
-}
+std::set<std::string> friends;
+std::set<std::string> ignores;
 
+bool friends_initialized = false;
+bool ignores_initialized = false;
+
+bool authenticated = false;
 } // anon namespace
 
 namespace preferences {
 
-manager::manager()
+manager::manager() :
+	base()
 {
 	set_music_volume(music_volume());
 	set_sound_volume(sound_volume());
@@ -87,6 +77,23 @@ manager::manager()
 	std::copy(terrain.begin(), terrain.end(),
 			std::inserter(encountered_terrains_set, encountered_terrains_set.begin()));
 
+	if (const config &history = preferences::get_child("history"))
+	{
+/* Structure of the history
+	[history]
+		[history_id]
+			[line]
+				message = foobar
+			[/line]
+*/
+		foreach (const config::any_child &h, history.all_children_range())
+		{
+			foreach (const config &l, h.cfg.child_range("line")) {
+				history_map[h.key].push_back(l["message"]);
+			}
+		}
+	}
+
 	network::ping_timeout = get_ping_timeout();
 }
 
@@ -100,76 +107,160 @@ manager::~manager()
 			  std::back_inserter(terrain));
 	preferences::set("encountered_terrain_list", t_translation::write_list(terrain));
 
+/* Structure of the history
+	[history]
+		[history_id]
+			[line]
+				message = foobar
+			[/line]
+*/
+	config history;
+	typedef std::pair<std::string, std::vector<std::string> > hack;
+	foreach(const hack& history_id, history_map) {
+
+		config history_id_cfg; // [history_id]
+		foreach(const std::string& line, history_id.second) {
+			config cfg; // [line]
+
+			cfg["message"] = line;
+			history_id_cfg.add_child("line", cfg);
+		}
+
+		history.add_child(history_id.first, history_id_cfg);
+	}
+	preferences::set_child("history", history);
+
+	history_map.clear();
 	encountered_units_set.clear();
 	encountered_terrains_set.clear();
-	set_ping_timeout(network::ping_timeout);
 }
 
-std::string get_friends() {
-	return preferences::get("friends");
+bool is_authenticated() {
+	return authenticated;
 }
 
-std::string get_ignores() {
-	return preferences::get("ignores");
+void parse_admin_authentication(const std::string& sender, const std::string& message) {
+	if(sender != "server") return;
+	if(message.find("You are now recognized as an administrator.") == 0) {
+		authenticated = true;
+	} else if(message.find("You are no longer recognized as an administrator.") == 0) {
+		authenticated = false;
+	}
 }
 
-bool add_friend(const std::string nick) {
-	if (!utils::isvalid_username(nick)) return false;
-	add_relation(nick, "friends");
+static void initialize_friends() {
+	if(!friends_initialized) {
+		std::vector<std::string> names = utils::split(preferences::get("friends"));
+		std::set<std::string> tmp(names.begin(), names.end());
+		friends.swap(tmp);
+
+		friends_initialized = true;
+	}
+}
+
+const std::set<std::string> & get_friends() {
+	initialize_friends();
+	return friends;
+}
+
+static void initialize_ignores() {
+	if(!ignores_initialized) {
+		std::vector<std::string> names = utils::split(preferences::get("ignores"));
+		std::set<std::string> tmp(names.begin(), names.end());
+		ignores.swap(tmp);
+
+		ignores_initialized = true;
+	}
+}
+
+const std::set<std::string> & get_ignores() {
+	return ignores;
+}
+
+bool add_friend(const std::string& nick) {
+	if (!utils::isvalid_wildcard(nick)) return false;
+	friends.insert(nick);
+	preferences::set("friends", utils::join(friends));
 	return true;
 }
 
-bool add_ignore(const std::string nick) {
-	if (!utils::isvalid_username(nick)) return false;
-	add_relation(nick, "ignores");
+bool add_ignore(const std::string& nick) {
+	if (!utils::isvalid_wildcard(nick)) return false;
+	ignores.insert(nick);
+	preferences::set("ignores", utils::join(ignores));
 	return true;
 }
 
-void remove_friend(const std::string nick) {
-	remove_relation(nick, "friends");
+void remove_friend(const std::string& nick) {
+	std::set<std::string>::iterator i = friends.find(nick);
+	if(i != friends.end()) {
+		friends.erase(i);
+		preferences::set("friends", utils::join(friends));
+	}
 }
 
-void remove_ignore(const std::string nick) {
-	remove_relation(nick, "ignores");
+void remove_ignore(const std::string& nick) {
+	std::set<std::string>::iterator i = ignores.find(nick);
+	if(i != ignores.end()) {
+		ignores.erase(i);
+		preferences::set("ignores", utils::join(ignores));
+	}
 }
 
 void clear_friends() {
-	preferences::set("friends", "");	
+	preferences::set("friends", "");
 }
 
 void clear_ignores() {
-	preferences::set("ignores", "");	
+	preferences::set("ignores", "");
 }
 
-bool is_friend(const std::string nick) {
-	const std::vector<std::string>& friends = utils::split(get_friends());
-	return (std::find(friends.begin(), friends.end(), nick) != friends.end());
+bool is_friend(const std::string& nick) {
+	initialize_friends();
+	return friends.find(nick) != friends.end();
 }
 
-bool is_ignored(const std::string nick) {
-	const std::vector<std::string>& ignores = utils::split(get_ignores());
-	return (std::find(ignores.begin(), ignores.end(), nick) != ignores.end());
+bool is_ignored(const std::string& nick) {
+	initialize_ignores();
+	return ignores.find(nick) != ignores.end();
 }
 
-bool show_lobby_join(const std::string& sender, const std::string& message) {
+void add_completed_campaign(const std::string& campaign_id) {
+	std::vector<std::string> completed = utils::split(preferences::get("completed_campaigns"));
+
+	if(std::find(completed.begin(), completed.end(), campaign_id) != completed.end())
+		return;
+
+	completed.push_back(campaign_id);
+	preferences::set("completed_campaigns", utils::join(completed));
+}
+
+bool is_campaign_completed(const std::string& campaign_id) {
+	std::vector<std::string> completed = utils::split(preferences::get("completed_campaigns"));
+
+	return std::find(completed.begin(), completed.end(), campaign_id) != completed.end();
+}
+
+bool parse_should_show_lobby_join(const std::string &sender, const std::string &message)
+{
 	// If it's actually not a lobby join message return true (show it).
-	if (sender != "server" || message.find("has logged into the lobby") == std::string::npos) return true;
-	if (lobby_joins() == SHOW_NONE) return false;
-	if (lobby_joins() == SHOW_ALL) return true;
-	const std::string::const_iterator i =
-			std::find(message.begin(), message.end(), ' ');
-	const std::string joiner(message.begin(), i);
-	if (lobby_joins() == SHOW_FRIENDS && is_friend(joiner)) return true;
-	return false;
+	if (sender != "server") return true;
+	std::string::size_type pos = message.find(" has logged into the lobby");
+	if (pos == std::string::npos) return true;
+	int lj = lobby_joins();
+	if (lj == SHOW_NONE) return false;
+	if (lj == SHOW_ALL) return true;
+	return is_friend(message.substr(0, pos));
 }
 
 int lobby_joins()
 {
-    if(preferences::get("lobby_joins") == "friends") {
+	std::string pref = preferences::get("lobby_joins");
+	if (pref == "friends") {
 		return SHOW_FRIENDS;
-	} else if(preferences::get("lobby_joins") == "all") {
+	} else if (pref == "all") {
 		return SHOW_ALL;
-	} else if(preferences::get("lobby_joins") == "none") {
+	} else if (pref == "none") {
 		return SHOW_NONE;
 	} else {
 		return SHOW_FRIENDS;
@@ -188,47 +279,38 @@ void _set_lobby_joins(int show)
 	}
 }
 
-bool sort_list()
+bool new_lobby()
 {
-	return utils::string_bool(preferences::get("sort_list"), true);
+	std::cerr << get("new_lobby") << "\n";
+	return utils::string_bool(get("new_lobby"), false);
 }
 
-
-void _set_sort_list(bool sort)
+void set_new_lobby(bool value)
 {
-	preferences::set("sort_list", sort ? "yes" : "no");
+	std::cerr << get("new_lobby") << value;
+	preferences::set("new_lobby", (value ? "yes" : "no"));
+	std::cerr << get("new_lobby") << "\n";
 }
 
-bool iconize_list()
-{
-	return utils::string_bool(preferences::get("iconize_list"), true);
-}
-
-void _set_iconize_list(bool sort)
-{
-	preferences::set("iconize_list", sort ? "yes" : "no");
-}
 
 const std::vector<game_config::server_info>& server_list()
 {
 	static std::vector<game_config::server_info> pref_servers;
 	if(pref_servers.empty()) {
 		std::vector<game_config::server_info> &game_servers = game_config::server_list;
-		VALIDATE(game_servers.size() > 0, _("No server has been defined."));
+		VALIDATE(!game_servers.empty(), _("No server has been defined."));
 		pref_servers.insert(pref_servers.begin(), game_servers.begin(), game_servers.end());
-		const std::vector<config *> &user_servers = get_prefs()->get_children("server");
-		std::vector<config *>::const_iterator server;
-		for(server = user_servers.begin(); server != user_servers.end(); ++server) {
+		foreach (const config &server, get_prefs()->child_range("server")) {
 			game_config::server_info sinf;
-			sinf.name = (**server)["name"];
-			sinf.address = (**server)["address"];
+			sinf.name = server["name"];
+			sinf.address = server["address"];
 			pref_servers.push_back(sinf);
 		}
 	}
 	return pref_servers;
 }
 
-const std::string network_host()
+std::string network_host()
 {
 	const std::string res = preferences::get("host");
 	if(res.empty()) {
@@ -243,7 +325,7 @@ void set_network_host(const std::string& host)
 	preferences::set("host", host);
 }
 
-const unsigned int get_ping_timeout()
+unsigned int get_ping_timeout()
 {
 	return lexical_cast_default<unsigned>(preferences::get("ping_timeout"), 0);
 }
@@ -254,7 +336,7 @@ void set_ping_timeout(unsigned int timeout)
 	preferences::set("ping_timeout", lexical_cast<std::string>(timeout));
 }
 
-const std::string campaign_server()
+std::string campaign_server()
 {
 	if(!preferences::get("campaign_server").empty()) {
 		return preferences::get("campaign_server");
@@ -268,7 +350,7 @@ void set_campaign_server(const std::string& host)
 	preferences::set("campaign_server", host);
 }
 
-const std::string login()
+std::string login()
 {
 	const std::string res = preferences::get("login");
 	if(res.empty()) {
@@ -290,6 +372,38 @@ void set_login(const std::string& username)
 	preferences::set("login", username);
 }
 
+namespace prv {
+	std::string password;
+}
+
+std::string password()
+{
+	if(remember_password()) {
+		return preferences::get("password");
+	} else {
+		return prv::password;
+	}
+}
+
+void set_password(const std::string& password)
+{
+	prv::password = password;
+	if(remember_password()) {
+		preferences::set("password", password);
+	}
+}
+
+bool remember_password()
+{
+	return utils::string_bool(preferences::get("remember_password"), false);
+}
+
+void set_remember_password(bool remember)
+{
+	preferences::set("remember_password", remember ? "yes" : "no");
+	preferences::set("password", remember ? prv::password : "");
+}
+
 bool turn_dialog()
 {
 	return utils::string_bool(preferences::get("turn_dialog"), false);
@@ -325,6 +439,33 @@ void set_use_map_settings(bool value)
 	preferences::set("mp_use_map_settings", value ? "yes" : "no");
 }
 
+int mp_server_warning_disabled()
+{
+	return lexical_cast_default<int>(preferences::get("mp_server_warning_disabled"), 0);
+}
+
+void set_mp_server_warning_disabled(int value)
+{
+	preferences::set("mp_server_warning_disabled", lexical_cast<std::string>(value));
+}
+
+void set_mp_server_program_name(const std::string& path)
+{
+	if (path.empty())
+	{
+		preferences::clear("mp_server_program_name");
+	}
+	else
+	{
+		preferences::set("mp_server_program_name", path);
+	}
+}
+
+std::string get_mp_server_program_name()
+{
+	return preferences::get("mp_server_program_name");
+}
+
 bool random_start_time()
 {
 	return settings::use_random_start_time(preferences::get("mp_random_start_time"));
@@ -410,7 +551,7 @@ void set_countdown_reservoir_time(int value)
 int countdown_turn_bonus()
 {
 	return lexical_cast_in_range<int>(
-		preferences::get("mp_countdown_turn_bonus"), 40, 0, 300);
+		preferences::get("mp_countdown_turn_bonus"), 60, 0, 300);
 }
 
 void set_countdown_turn_bonus(int value)
@@ -519,6 +660,16 @@ bool ask_delete_saves()
 	return utils::string_bool(preferences::get("ask_delete"), true);
 }
 
+void set_interrupt_when_ally_sighted(bool value)
+{
+	preferences::set("ally_sighted_interrupts", value ? "yes" : "no");
+}
+
+bool interrupt_when_ally_sighted()
+{
+	return utils::string_bool(preferences::get("ally_sighted_interrupts"), true);
+}
+
 int autosavemax()
 {
 	return lexical_cast_default<int>(preferences::get("auto_save_max"), 10);
@@ -543,7 +694,7 @@ std::string clock_format()
 	return "%H:%M";
 }
 
-const std::string theme()
+std::string theme()
 {
 	if(non_interactive()) {
 		static const std::string null_theme = "null";
@@ -585,11 +736,6 @@ void set_message_private(bool value)
 	message_private_on = value;
 }
 
-bool show_tip_of_day()
-{
-	return utils::string_bool(preferences::get("tip_of_day"), true);
-}
-
 bool show_haloes()
 {
 	return haloes;
@@ -611,34 +757,16 @@ void set_flip_time(bool value)
 	preferences::set("flip_time", value ? "yes" : "no");
 }
 
-bool upload_log()
-{
-	return utils::string_bool(preferences::get("upload_log"), false);
-}
-
-void set_upload_log(bool value)
-{
-	preferences::set("upload_log", value ? "yes" : "no");
-}
-
-const std::string upload_id()
-{
-	// We create a unique id for each person, *when asked for* to increase
-	// randomness.
-	if (preferences::get("upload_id") == "") {
-		srand(time(NULL));
-		preferences::set("upload_id",
-			lexical_cast<std::string>(rand())
-				 + lexical_cast<std::string>(SDL_GetTicks()));
-	}
-	return preferences::get("upload_id");
-}
-
 bool compress_saves()
 {
 	return utils::string_bool(preferences::get("compress_saves"), true);
 }
 
+bool startup_effect()
+{
+	return utils::string_bool(preferences::get("startup_effect"), true);
+}
+
 std::string get_chat_timestamp(const time_t& t) {
 	if (chat_timestamping()) {
 		return lg::get_timestamp(t, clock_format()) + " ";
@@ -672,6 +800,24 @@ std::set<t_translation::t_terrain> &encountered_terrains() {
 	return encountered_terrains_set;
 }
 
+std::string custom_command() {
+	return preferences::get("custom_command");
+}
+
+void set_custom_command(const std::string& command) {
+	preferences::set("custom_command", command);
+}
+
+/**
+ * Returns a pointer to the history vector associated with given id
+ * making a new one if it doesn't exist.
+ *
+ * @todo FIXME only used for gui2. Could be used for the above histories.
+ */
+std::vector<std::string>* get_history(const std::string& id) {
+	return &history_map[id];
+}
+
 bool green_confirm()
 {
 	std::string confirmation = preferences::get("confirm_end_turn");
@@ -696,7 +842,7 @@ bool confirm_no_moves()
 
 void encounter_recruitable_units(std::vector<team>& teams){
 	for (std::vector<team>::iterator help_team_it = teams.begin();
-		help_team_it != teams.end(); help_team_it++) {
+		help_team_it != teams.end(); ++help_team_it) {
 		help_team_it->log_recruitable();
 		std::copy(help_team_it->recruits().begin(), help_team_it->recruits().end(),
 				  std::inserter(encountered_units_set, encountered_units_set.begin()));
@@ -705,27 +851,28 @@ void encounter_recruitable_units(std::vector<team>& teams){
 
 void encounter_start_units(unit_map& units){
 	for (unit_map::const_iterator help_unit_it = units.begin();
-		 help_unit_it != units.end(); help_unit_it++) {
-		const std::string name = help_unit_it->second.id();
+		 help_unit_it != units.end(); ++help_unit_it) {
+		const std::string name = help_unit_it->second.type_id();
 		encountered_units_set.insert(name);
 	}
 }
 
-void encounter_recallable_units(game_state& gamestate){
-	for(std::map<std::string, player_info>::iterator pi = gamestate.players.begin(); pi!=gamestate.players.end(); ++pi) {
-		for(std::vector<unit>::iterator help_recall_it = pi->second.available_units.begin(); help_recall_it != pi->second.available_units.end(); help_recall_it++) {
-			encountered_units_set.insert(help_recall_it->id());
+void encounter_recallable_units(std::vector<team>& teams){
+	for (std::vector<team>::iterator help_team_it = teams.begin();
+		help_team_it != teams.end(); ++help_team_it) {
+		for(std::vector<unit>::iterator help_recall_it = help_team_it->recall_list().begin(); help_recall_it != help_team_it->recall_list().end(); ++help_recall_it) {
+			encountered_units_set.insert(help_recall_it->type_id());
 		}
 	}
 }
 
 void encounter_map_terrain(gamemap& map){
-	for (int map_x = 0; map_x < map.w(); map_x++) {
-		for (int map_y = 0; map_y < map.h(); map_y++) {
-			const t_translation::t_terrain t = map.get_terrain(gamemap::location(map_x, map_y));
+	for (int map_x = 0; map_x < map.w(); ++map_x) {
+		for (int map_y = 0; map_y < map.h(); ++map_y) {
+			const t_translation::t_terrain t = map.get_terrain(map_location(map_x, map_y));
 			preferences::encountered_terrains().insert(t);
-			const t_translation::t_list& underlaying_list = map.underlying_union_terrain(gamemap::location(map_x, map_y));
-			for (std::vector<t_translation::t_terrain>::const_iterator ut = underlaying_list.begin(); ut != underlaying_list.end(); ut++) {
+			const t_translation::t_list& underlaying_list = map.underlying_union_terrain(map_location(map_x, map_y));
+			for (std::vector<t_translation::t_terrain>::const_iterator ut = underlaying_list.begin(); ut != underlaying_list.end(); ++ut) {
 				preferences::encountered_terrains().insert(*ut);
 			};
 		}
