@@ -1,6 +1,6 @@
-/* $Id: game_preferences.hpp 33092 2009-02-25 01:45:48Z soliton $ */
+/* $Id: game_preferences.hpp 42333 2010-04-30 23:22:12Z loonycyborg $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -38,29 +38,29 @@ namespace preferences {
 	};
 
 	bool is_authenticated();
-	void admin_authentication(const std::string& sender, const std::string& message);
+	void parse_admin_authentication(const std::string& sender, const std::string& message);
 
-	bool show_lobby_join(const std::string& sender, const std::string& message);
+	bool parse_should_show_lobby_join(const std::string& sender, const std::string& message);
 	int lobby_joins();
 	void _set_lobby_joins(int show);
 	enum { SHOW_NONE, SHOW_FRIENDS, SHOW_ALL };
 
+	bool new_lobby();
+	void set_new_lobby(bool value);
+
+
 	const std::set<std::string> & get_friends();
 	const std::set<std::string> & get_ignores();
 	bool add_friend(const std::string& nick);
 	bool add_ignore(const std::string& nick);
+	void add_completed_campaign(const std::string& campaign_id);
 	void remove_friend(const std::string& nick);
 	void remove_ignore(const std::string& nick);
 	void clear_friends();
 	void clear_ignores();
 	bool is_friend(const std::string& nick);
 	bool is_ignored(const std::string& nick);
-
-	bool sort_list();
-	void _set_sort_list(bool show);
-
-	bool iconize_list();
-	void _set_iconize_list(bool show);
+	bool is_campaign_completed(const std::string& campaign_id);
 
 	const std::vector<game_config::server_info>& server_list();
 
@@ -164,6 +164,9 @@ namespace preferences {
 	void set_ask_delete_saves(bool value);
 	bool ask_delete_saves();
 
+    void set_interrupt_when_ally_sighted(bool value);
+    bool interrupt_when_ally_sighted();
+
 	void set_autosavemax(int value);
 	int autosavemax();
 
@@ -175,8 +178,6 @@ namespace preferences {
 	bool message_private();
 	void set_message_private(bool value);
 
-	bool show_tip_of_day();
-
 	bool show_haloes();
 	void set_show_haloes(bool value);
 
@@ -184,11 +185,6 @@ namespace preferences {
 	bool flip_time();
 	void set_flip_time(bool value);
 
-	bool has_upload_log();
-	bool upload_log();
-	void set_upload_log(bool value);
-	std::string upload_id();
-
 	// Multiplayer functions
 	std::string get_chat_timestamp(const time_t& t);
 	bool chat_timestamping();
@@ -231,9 +227,10 @@ namespace preferences {
 	// system.
 	void encounter_start_units(unit_map& units);
 	// Add all units that are recallable as encountred units.
-	void encounter_recallable_units(game_state& gamestate);
+	void encounter_recallable_units(std::vector<team>& teams);
 	// Add all terrains on the map as encountered terrains.
 	void encounter_map_terrain(gamemap& map);
+
 }
 
 #endif
