@@ -1,6 +1,6 @@
-/* $Id: game_preferences_display.cpp 36370 2009-06-23 19:40:55Z mordante $ */
+/* $Id: game_preferences_display.cpp 42239 2010-04-24 08:49:09Z ivanovic $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,10 @@
 #include "display.hpp"
 #include "filesystem.hpp"
 #include "filechooser.hpp"
+#include "foreach.hpp"
 #include "gettext.hpp"
+#include "gui/dialogs/transient_message.hpp"
+#include "lobby_preferences.hpp"
 #include "preferences_display.hpp"
 #include "wml_separators.hpp"
 #include "widgets/slider.hpp"
@@ -93,13 +96,15 @@ private:
 	// change
 	gui::slider music_slider_, sound_slider_, UI_sound_slider_, bell_slider_,
 	            scroll_slider_, chat_lines_slider_,
-	  buffer_size_slider_, idle_anim_slider_, autosavemax_slider_;
+	  buffer_size_slider_, idle_anim_slider_, autosavemax_slider_, advanced_slider_;
 	gui::list_slider<double> turbo_slider_;
 	gui::button fullscreen_button_, scroll_to_action_button_,turbo_button_, show_ai_moves_button_,
+			interrupt_when_ally_sighted_button_,
 			show_grid_button_, save_replays_button_, delete_saves_button_,
 			show_lobby_joins_button1_,
 			show_lobby_joins_button2_,
 			show_lobby_joins_button3_,
+			new_lobby_button_,
 			sort_list_by_group_button_, iconize_list_button_,
 			remember_pw_button_, mp_server_search_button_,
 			friends_list_button_, friends_back_button_,
@@ -118,7 +123,8 @@ private:
 	gui::label music_label_, sound_label_, UI_sound_label_, bell_label_,
 	           scroll_label_, chat_lines_label_,
 	           turbo_slider_label_, sample_rate_label_, buffer_size_label_,
-	  idle_anim_slider_label_, autosavemax_slider_label_;
+			   idle_anim_slider_label_, autosavemax_slider_label_,
+			   advanced_slider_label_;
 	gui::textbox sample_rate_input_, friends_input_;
 
 	unsigned slider_label_width_;
@@ -145,19 +151,22 @@ preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 	  scroll_slider_(disp.video()),
 	  chat_lines_slider_(disp.video()), buffer_size_slider_(disp.video()),
 	  idle_anim_slider_(disp.video()), autosavemax_slider_(disp.video()),
+	  advanced_slider_(disp.video()),
 	  turbo_slider_(disp.video()),
 
 
-	  fullscreen_button_(disp.video(), _("Toggle Full Screen"), gui::button::TYPE_CHECK),
+	  fullscreen_button_(disp.video(), _("Full Screen"), gui::button::TYPE_CHECK),
 	  scroll_to_action_button_(disp.video(), _("Enable scroll tracking of unit actions"), gui::button::TYPE_CHECK),
 	  turbo_button_(disp.video(), _("Accelerated Speed"), gui::button::TYPE_CHECK),
 	  show_ai_moves_button_(disp.video(), _("Skip AI Moves"), gui::button::TYPE_CHECK),
+	  interrupt_when_ally_sighted_button_(disp.video(), _("Interrupt move when an ally is sighted"), gui::button::TYPE_CHECK),
 	  show_grid_button_(disp.video(), _("Show Grid"), gui::button::TYPE_CHECK),
 	  save_replays_button_(disp.video(), _("Save Replay on SP/MP Victory or MP Defeat"), gui::button::TYPE_CHECK),
 	  delete_saves_button_(disp.video(), _("Delete Auto-Saves on SP/MP Victory or MP Defeat"), gui::button::TYPE_CHECK),
 	  show_lobby_joins_button1_(disp.video(), _("Do Not Show Lobby Joins"), gui::button::TYPE_CHECK),
 	  show_lobby_joins_button2_(disp.video(), _("Show Lobby Joins Of Friends Only"), gui::button::TYPE_CHECK),
 	  show_lobby_joins_button3_(disp.video(), _("Show All Lobby Joins"), gui::button::TYPE_CHECK),
+	  new_lobby_button_(disp.video(), _("Use new lobby interface"), gui::button::TYPE_CHECK),
 	  sort_list_by_group_button_(disp.video(), _("Sort Lobby List"), gui::button::TYPE_CHECK),
 	  iconize_list_button_(disp.video(), _("Iconize Lobby List"), gui::button::TYPE_CHECK),
 	  remember_pw_button_(disp.video(), _("Save password to preferences (clear text)"), gui::button::TYPE_CHECK),
@@ -198,6 +207,7 @@ preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 	  sample_rate_label_(disp.video(), _("Sample Rate (Hz):")), buffer_size_label_(disp.video(), ""),
 	  idle_anim_slider_label_(disp.video(), _("Frequency:")),
 	  autosavemax_slider_label_(disp.video(), "", font::SIZE_SMALL),
+	  advanced_slider_label_(disp.video(), "", font::SIZE_SMALL),
 
 	  sample_rate_input_(disp.video(), 70),
 	  friends_input_(disp.video(), 170),
@@ -215,7 +225,7 @@ preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 #ifdef USE_TINY_GUI
 	set_measurements(180, 180);		  // FIXME: should compute this, but using what data ?
 #else
-	set_measurements(440, 405);
+	set_measurements(465, 425);
 #endif
 
 
@@ -329,6 +339,9 @@ preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 	show_ai_moves_button_.set_check(!show_ai_moves());
 	show_ai_moves_button_.set_help_string(_("Do not animate AI units moving"));
 
+	interrupt_when_ally_sighted_button_.set_check(interrupt_when_ally_sighted());
+	interrupt_when_ally_sighted_button_.set_help_string(_("Sighting an allied unit interrupts your unit's movement"));
+
 	save_replays_button_.set_check(save_replays());
 	save_replays_button_.set_help_string(_("Save Replay on SP/MP Victory or MP Defeat"));
 
@@ -346,13 +359,17 @@ preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 	remember_pw_button_.set_check(remember_password());
 	remember_pw_button_.set_help_string(_("Uncheck to delete the saved password (on exit)."));
 
-	show_lobby_joins_button1_.set_check(lobby_joins() == SHOW_NONE);
+	int lj = lobby_joins();
+	show_lobby_joins_button1_.set_check(lj == SHOW_NONE);
 	show_lobby_joins_button1_.set_help_string(_("Do not show messages about players joining the multiplayer lobby"));
-	show_lobby_joins_button2_.set_check(lobby_joins() == SHOW_FRIENDS);
+	show_lobby_joins_button2_.set_check(lj == SHOW_FRIENDS);
 	show_lobby_joins_button2_.set_help_string(_("Show messages about your friends joining the multiplayer lobby"));
-	show_lobby_joins_button3_.set_check(lobby_joins() == SHOW_ALL);
+	show_lobby_joins_button3_.set_check(lj == SHOW_ALL);
 	show_lobby_joins_button3_.set_help_string(_("Show messages about all players joining the multiplayer lobby"));
 
+	new_lobby_button_.set_check(new_lobby());
+	new_lobby_button_.set_help_string(_("mouseover^Use new lobby interface (only works after a manual reconnect to the server)"));
+
 	mp_server_search_button_.set_help_string(_("Find and set path to MP server to host LAN games."));
 	friends_list_button_.set_help_string(_("View and edit your friends and ignores list"));
 	friends_back_button_.set_help_string(_("Back to the multiplayer options"));
@@ -406,11 +423,13 @@ handler_vector preferences_dialog::handler_members()
 	h.push_back(&idle_anim_slider_);
 	h.push_back(&autosavemax_slider_);
 	h.push_back(&buffer_size_slider_);
+	h.push_back(&advanced_slider_);
 	h.push_back(&fullscreen_button_);
 	h.push_back(&scroll_to_action_button_);
 	h.push_back(&turbo_button_);
 	h.push_back(&idle_anim_button_);
 	h.push_back(&show_ai_moves_button_);
+	h.push_back(&interrupt_when_ally_sighted_button_);
 	h.push_back(&save_replays_button_);
 	h.push_back(&delete_saves_button_);
 	h.push_back(&show_grid_button_);
@@ -420,6 +439,7 @@ handler_vector preferences_dialog::handler_members()
 	h.push_back(&show_lobby_joins_button1_);
 	h.push_back(&show_lobby_joins_button2_);
 	h.push_back(&show_lobby_joins_button3_);
+	h.push_back(&new_lobby_button_);
 	h.push_back(&mp_server_search_button_);
 	h.push_back(&friends_list_button_);
 	h.push_back(&friends_back_button_);
@@ -456,6 +476,7 @@ handler_vector preferences_dialog::handler_members()
 	h.push_back(&turbo_slider_label_);
 	h.push_back(&idle_anim_slider_label_);
 	h.push_back(&autosavemax_slider_label_);
+	h.push_back(&advanced_slider_label_);
 	h.push_back(&chat_lines_label_);
 	h.push_back(&sample_rate_label_);
 	h.push_back(&buffer_size_label_);
@@ -500,6 +521,7 @@ void preferences_dialog::update_location(SDL_Rect const &rect)
 	turbo_slider_.set_location(turbo_rect);
 	ypos += item_interline; show_ai_moves_button_.set_location(rect.x, ypos);
 	ypos += short_interline; turn_dialog_button_.set_location(rect.x, ypos);
+	ypos += short_interline; interrupt_when_ally_sighted_button_.set_location(rect.x, ypos);
 	ypos += item_interline; show_team_colours_button_.set_location(rect.x, ypos);
 	ypos += short_interline; show_grid_button_.set_location(rect.x, ypos);
 	ypos += item_interline; save_replays_button_.set_location(rect.x, ypos);
@@ -609,6 +631,8 @@ void preferences_dialog::update_location(SDL_Rect const &rect)
 	ypos += short_interline; show_lobby_joins_button2_.set_location(rect.x, ypos);
 	ypos += short_interline; show_lobby_joins_button3_.set_location(rect.x, ypos);
 
+	ypos += item_interline; new_lobby_button_.set_location(rect.x, ypos);
+
 	friends_list_button_.set_location(rect.x, bottom_row_y - friends_list_button_.height());
 
 	mp_server_search_button_.set_location(rect.x + horizontal_padding + friends_list_button_.width(), bottom_row_y - mp_server_search_button_.height());
@@ -642,6 +666,10 @@ void preferences_dialog::update_location(SDL_Rect const &rect)
 	ypos += advanced_.height() + font::relative_size(14);
 
 	advanced_button_.set_location(rect.x,ypos);
+	advanced_slider_label_.set_location(rect.x,ypos);
+	const SDL_Rect advanced_slider_rect = { rect.x, ypos+short_interline,
+				rect.w - right_border, 0};
+	advanced_slider_.set_location(advanced_slider_rect);
 
 	set_selection(tab_);
 }
@@ -656,6 +684,8 @@ void preferences_dialog::process_event()
 		}
 		if (show_ai_moves_button_.pressed())
 			set_show_ai_moves(!show_ai_moves_button_.checked());
+		if (interrupt_when_ally_sighted_button_.pressed())
+			set_interrupt_when_ally_sighted(interrupt_when_ally_sighted_button_.checked());
 		if (show_grid_button_.pressed())
 			set_grid(show_grid_button_.checked());
 		if (save_replays_button_.pressed())
@@ -835,6 +865,8 @@ void preferences_dialog::process_event()
 			show_lobby_joins_button2_.set_check(false);
 			show_lobby_joins_button3_.set_check(true);
 		}
+		if (new_lobby_button_.pressed())
+			set_new_lobby(new_lobby_button_.checked());
 		if (sort_list_by_group_button_.pressed())
 			set_sort_list(sort_list_by_group_button_.checked());
 		if (iconize_list_button_.pressed())
@@ -915,19 +947,36 @@ void preferences_dialog::process_event()
 			const config* const adv = get_advanced_pref();
 			if(adv != NULL) {
 				const config& pref = *adv;
-				advanced_button_.set_width(0);
-				advanced_button_.set_label(pref["name"]);
+				const std::string description = pref["description"];
 				std::string value = preferences::get(pref["field"]);
+				advanced_button_.hide(pref["type"] != "boolean");
+				const bool hide_int = pref["type"] != "int";
+				advanced_slider_.hide(hide_int);
+				advanced_slider_label_.hide(hide_int);
 				if(value.empty()) {
 					value = pref["default"];
 				}
-
-				advanced_button_.set_check(value == "yes");
+				if (pref["type"] == "boolean") {
+					advanced_button_.set_width(0);
+					advanced_button_.set_label(pref["name"]);
+					advanced_button_.set_check(value == "yes");
+					advanced_button_.set_help_string(description);
+				} else if (pref["type"] == "int") {
+					std::stringstream ss;
+					ss << pref["name"] << ": " << value;
+					advanced_slider_label_.set_text(ss.str());
+					advanced_slider_label_.set_help_string(description);
+					advanced_slider_.set_min(lexical_cast<int>(pref["min"]));
+					advanced_slider_.set_max(lexical_cast<int>(pref["max"]));
+					advanced_slider_.set_increment(lexical_cast_default<int>(pref["step"], 1));
+					advanced_slider_.set_value(lexical_cast<int>(value));
+					advanced_slider_.set_help_string(description);
+				}
 			}
 		}
 
+		const config* const adv = get_advanced_pref();
 		if(advanced_button_.pressed()) {
-			const config* const adv = get_advanced_pref();
 			if(adv != NULL) {
 				const config& pref = *adv;
 				preferences::set(pref["field"],
@@ -936,15 +985,29 @@ void preferences_dialog::process_event()
 			}
 		}
 
+		if(advanced_slider_.value_change()) {
+			if(adv != NULL) {
+				const config& pref = *adv;
+				preferences::set(pref["field"],
+						str_cast(advanced_slider_.value()));
+				set_advanced_menu();
+				std::stringstream ss;
+				ss << pref["name"] << ": " << advanced_slider_.value();
+				advanced_slider_label_.set_text(ss.str());
+			}
+		}
+
 		return;
 	}
 }
 
 const config* preferences_dialog::get_advanced_pref() const
 {
-	const config::child_list& adv = game_cfg_.get_children("advanced_preference");
-	if(advanced_selection_ >= 0 && advanced_selection_ < int(adv.size())) {
-		return adv[advanced_selection_];
+	config::const_child_itors itors = game_cfg_.child_range("advanced_preference");
+	int nb_prefs = std::distance(itors.first, itors.second);
+	if (advanced_selection_ >= 0 && advanced_selection_ < nb_prefs) {
+		std::advance(itors.first, advanced_selection_);
+		return &*itors.first;
 	} else {
 		return NULL;
 	}
@@ -953,12 +1016,12 @@ const config* preferences_dialog::get_advanced_pref() const
 void preferences_dialog::set_advanced_menu()
 {
 	std::vector<std::string> advanced_items;
-	const config::child_list& adv = game_cfg_.get_children("advanced_preference");
-	for(config::child_list::const_iterator i = adv.begin(); i != adv.end(); ++i) {
+	foreach (const config &adv, game_cfg_.child_range("advanced_preference"))
+	{
 		std::ostringstream str;
-		std::string field = preferences::get((**i)["field"]);
+		std::string field = preferences::get(adv["field"]);
 		if(field.empty()) {
-			field = (**i)["default"];
+			field = adv["default"];
 		}
 
 		if(field == "yes") {
@@ -967,7 +1030,7 @@ void preferences_dialog::set_advanced_menu()
 			field = _("no");
 		}
 
-		str << (**i)["name"] << COLUMN_SEPARATOR << field;
+		str << adv["name"] << COLUMN_SEPARATOR << field;
 		advanced_items.push_back(str.str());
 	}
 
@@ -1020,6 +1083,7 @@ void preferences_dialog::set_selection(int index)
 	turbo_slider_.enable(turbo());
 	show_ai_moves_button_.hide(hide_general);
 	turn_dialog_button_.hide(hide_general);
+	interrupt_when_ally_sighted_button_.hide(hide_general);
 	hotkeys_button_.hide(hide_general);
 	show_team_colours_button_.hide(hide_general);
 	show_grid_button_.hide(hide_general);
@@ -1089,6 +1153,7 @@ void preferences_dialog::set_selection(int index)
 	show_lobby_joins_button1_.hide(hide_multiplayer);
 	show_lobby_joins_button2_.hide(hide_multiplayer);
 	show_lobby_joins_button3_.hide(hide_multiplayer);
+	new_lobby_button_.hide(hide_multiplayer);
 	friends_list_button_.hide(hide_multiplayer);
 	mp_server_search_button_.hide(hide_multiplayer);
 
@@ -1102,7 +1167,12 @@ void preferences_dialog::set_selection(int index)
 
 	const bool hide_advanced = tab_ != ADVANCED_TAB;
 	advanced_.hide(hide_advanced);
-	advanced_button_.hide(hide_advanced);
+	const std::string adv_type = get_advanced_pref() != NULL ? (*get_advanced_pref())["type"] : "";
+	const bool hide_advanced_bool = hide_advanced || adv_type != "boolean";
+	const bool hide_advanced_int = hide_advanced || adv_type != "int";
+	advanced_button_.hide(hide_advanced_bool);
+	advanced_slider_label_.hide(hide_advanced_int);
+	advanced_slider_.hide(hide_advanced_int);
 }
 
 }
@@ -1151,18 +1221,18 @@ bool show_theme_dialog(display& disp)
 {
 	int action = 0;
 	std::vector<std::string> options = disp.get_theme().get_known_themes();
-	if(options.size()){
+	if(!options.empty()){
 		std::string current_theme=_("Saved Theme Preference: ")+preferences::theme();
 		action = gui::show_dialog(disp,NULL,"",current_theme,gui::OK_CANCEL,&options);
 		if(action >= 0){
 		preferences::set_theme(options[action]);
 		//it would be preferable for the new theme to take effect
 		//immediately, however, this will have to do for now.
-		gui::message_dialog(disp,"",_("New theme will take effect on next new or loaded game.")).show();
+		gui2::show_transient_message(disp.video(),"",_("New theme will take effect on next new or loaded game."));
 		return(1);
 		}
 	}else{
-		gui::message_dialog(disp,"",_("No known themes. Try changing from within an existing game.")).show();
+		gui2::show_transient_message(disp.video(),"",_("No known themes. Try changing from within an existing game."));
 	}
 	return(0);
 }
