@@ -1,6 +1,6 @@
-/* $Id: generic_event.cpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: generic_event.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -42,7 +42,7 @@ bool generic_event::attach_handler(observer* obs){
 				handler_attached = true;
 			}
 		}
-		catch (std::exception&){
+		catch (...){
 			change_handler_ = false;
 			throw;
 		}
@@ -68,7 +68,7 @@ bool generic_event::detach_handler(observer* obs){
 				handler_detached = true;
 			}
 		}
-		catch (std::exception&){
+		catch (...){
 			change_handler_ = false;
 			throw;
 		}
@@ -83,11 +83,11 @@ void generic_event::notify_observers(){
 		notify_active_ = true;
 		try{
 			for (std::vector<observer*>::const_iterator it = observers_.begin();
-				it != observers_.end(); it++){
+				it != observers_.end(); ++it){
 				(*it)->handle_generic_event(name_);
 			}
 		}
-		catch (std::exception&){
+		catch (...){
 			//reset the flag if event handlers throw exceptions and don't catch them
 			notify_active_ = false;
 			throw;
