@@ -1,15 +1,15 @@
-/* $Id: image.hpp 37939 2009-08-18 19:47:08Z mordante $ */
+/* $Id: image.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
    or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
 #ifndef GUI_WIDGETS_IMAGE_HPP_INCLUDED
@@ -29,6 +29,32 @@ public:
 	{
 	}
 
+	/**
+	 * Wrapper for set_label.
+	 *
+	 * Some people considered this function missing and confusing so added
+	 * this forward version.
+	 *
+	 * @param label               The filename image to show.
+	 */
+	void set_image(const t_string& label)
+	{
+		set_label(label);
+	}
+
+	/**
+	 * Wrapper for label.
+	 *
+	 * Some people considered this function missing and confusing so added
+	 * this forward version.
+	 *
+	 * @returns                   The filename of the image shown.
+	 */
+	t_string get_image() const
+	{
+		return label();
+	}
+
 	/***** ***** ***** ***** layout functions ***** ***** ***** *****/
 
 private:
@@ -48,7 +74,7 @@ public:
 	unsigned get_state() const { return ENABLED; }
 
 	/** Inherited from tcontrol. */
-	bool does_block_easy_close() const { return false; }
+	bool disable_click_dismiss() const { return false; }
 
 private:
 
