@@ -1,15 +1,15 @@
-/* $Id: label.hpp 37939 2009-08-18 19:47:08Z mordante $ */
+/* $Id: label.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
    or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
 #ifndef GUI_WIDGETS_LABEL_HPP_INCLUDED
@@ -34,9 +34,6 @@ public:
 	/** Inherited from twidget. */
 	bool can_wrap() const { return can_wrap_; }
 
-	/** Inherited from twidget. */
-	bool can_shrink_width() const { return true; }
-
 	/** Inherited from tcontrol. */
 	void set_active(const bool active)
 		{ if(get_active() != active) set_state(active ? ENABLED : DISABLED); }
@@ -48,7 +45,7 @@ public:
 	unsigned get_state() const { return state_; }
 
 	/** Inherited from tcontrol. */
-	bool does_block_easy_close() const { return false; }
+	bool disable_click_dismiss() const { return false; }
 
 	/***** ***** ***** setters / getters for members ***** ****** *****/
 
