@@ -1,23 +1,28 @@
-/* $Id: listbox.cpp 37939 2009-08-18 19:47:08Z mordante $ */
+/* $Id: listbox.cpp 42002 2010-04-05 09:55:41Z mordante $ */
 /*
-   copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
    or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "gui/widgets/listbox.hpp"
 
+#include "gui/auxiliary/layout_exception.hpp"
+#include "gui/auxiliary/log.hpp"
 #include "gui/widgets/window.hpp"
 
+#define LOG_SCOPE_HEADER get_control_type() + " [" + id() + "] " + __func__
+#define LOG_HEADER LOG_SCOPE_HEADER + ':'
+
 namespace gui2 {
 
 namespace {
@@ -35,62 +40,58 @@ tlistbox::tlistbox(const bool has_minimum, const bool has_maximum,
 	, generator_(NULL)
 	, list_builder_(NULL)
 	, callback_value_changed_(NULL)
+	, need_layout_(false)
 {
 	generator_ = tgenerator_::build(
 			has_minimum, has_maximum, placement, select);
 }
 
-void tlistbox::add_row(const string_map& item)
+void tlistbox::add_row(const string_map& item, const int index)
 {
 	assert(generator_);
 	generator_->create_item(
-			-1, list_builder_, item, callback_list_item_clicked);
-
-	tgrid& grid = generator_->get_item(get_item_count() - 1);
-	twindow* window = get_window();
-	assert(window);
-
-	if(get_item_count() == 1) {
-		init_linked_size_widets(*window, grid.begin(), grid.end());
-
-		/**
-		 * @todo add footer and make one function to do this part since the
-		 * code is duplicated at two places.
-		 */
-		tgrid* header_grid = dynamic_cast<tgrid*>(
-				content_grid()->find_widget("_header_grid", false));
-
-		if(header_grid) {
-			add_linked_size_widgets(*window, header_grid->begin(),
-					header_grid->end());
-		}
-	}
-	add_linked_size_widgets(*window, grid.begin(), grid.end());
+			index, list_builder_, item, callback_list_item_clicked);
 }
 
 void tlistbox::add_row(
-		const std::map<std::string /* widget id */, string_map>& data)
+		  const std::map<std::string /* widget id */, string_map>& data
+		, const int index)
 {
 	assert(generator_);
 	generator_->create_item(
-			-1, list_builder_, data, callback_list_item_clicked);
+			index, list_builder_, data, callback_list_item_clicked);
+}
 
-	tgrid& grid = generator_->get_item(get_item_count() - 1);
-	twindow* window = get_window();
-	assert(window);
+void tlistbox::remove_row(const unsigned row, unsigned count)
+{
+	assert(generator_);
 
-	if(get_item_count() == 1) {
-		init_linked_size_widets(*window, grid.begin(), grid.end());
-
-		tgrid* header_grid = dynamic_cast<tgrid*>(
-				content_grid()->find_widget("_header_grid", false));
-
-		if(header_grid) {
-			add_linked_size_widgets(*window, header_grid->begin(),
-					header_grid->end());
-		}
+	if(row >= get_item_count()) {
+		return;
 	}
-	add_linked_size_widgets(*window, grid.begin(), grid.end());
+
+	if(!count || count > get_item_count()) {
+		count = get_item_count();
+	}
+
+	unsigned height_reduced = 0;
+	for(; count; --count) {
+		if(generator_->item(row).get_visible() != INVISIBLE) {
+			height_reduced += generator_->item(row).get_height();
+		}
+		generator_->delete_item(row);
+	}
+
+	if(height_reduced != 0) {
+		resize_content(0, -height_reduced);
+	}
+}
+
+void tlistbox::clear()
+{
+	// Due to the removing from the linked group, don't use
+	// generator_->clear() directly.
+	remove_row(0, 0);
 }
 
 unsigned tlistbox::get_item_count() const
@@ -102,14 +103,85 @@ unsigned tlistbox::get_item_count() const
 void tlistbox::set_row_active(const unsigned row, const bool active)
 {
 	assert(generator_);
-	generator_->get_item(row).set_active(active);
+	generator_->item(row).set_active(active);
+}
+
+void tlistbox::set_row_shown(const unsigned row, const bool shown)
+{
+	assert(generator_);
+
+	twindow *window = get_window();
+	assert(window);
+
+	const int selected_row = get_selected_row();
+
+	bool resize_needed;
+	{
+		twindow::tinvalidate_layout_blocker invalidate_layout_blocker(*window);
+
+		generator_->set_item_shown(row, shown);
+		generator_->place(generator_->get_origin()
+				, generator_->calculate_best_size());
+		resize_needed = !content_resize_request();
+	}
+
+	if(resize_needed) {
+		window->invalidate_layout();
+	} else {
+		content_grid_->set_visible_area(content_visible_area());
+		set_dirty();
+	}
+
+	if(selected_row != get_selected_row() && callback_value_changed_) {
+		callback_value_changed_(this);
+	}
+}
+
+void tlistbox::set_row_shown(const std::vector<bool>& shown)
+{
+	assert(generator_);
+	assert(shown.size() == get_item_count());
+
+	twindow *window = get_window();
+	assert(window);
+
+	const int selected_row = get_selected_row();
+
+	bool resize_needed;
+	{
+		twindow::tinvalidate_layout_blocker invalidate_layout_blocker(*window);
+
+		for(size_t i = 0; i < shown.size(); ++i) {
+			generator_->set_item_shown(i, shown[i]);
+		}
+		generator_->place(generator_->get_origin()
+				, generator_->calculate_best_size());
+		resize_needed = !content_resize_request();
+	}
+
+	if(resize_needed) {
+		window->invalidate_layout();
+	} else {
+		content_grid_->set_visible_area(content_visible_area());
+		set_dirty();
+	}
+
+	if(selected_row != get_selected_row() && callback_value_changed_) {
+		callback_value_changed_(this);
+	}
 }
 
 const tgrid* tlistbox::get_row_grid(const unsigned row) const
 {
 	assert(generator_);
 	// rename this function and can we return a reference??
-	return &generator_->get_item(row);
+	return &generator_->item(row);
+}
+
+tgrid* tlistbox::get_row_grid(const unsigned row)
+{
+	assert(generator_);
+	return &generator_->item(row);
 }
 
 bool tlistbox::select_row(const unsigned row, const bool select)
@@ -138,7 +210,7 @@ void tlistbox::list_item_clicked(twidget* caller)
 
 	for(size_t i = 0; i < generator_->get_item_count(); ++i) {
 
-		if(generator_->get_item(i).has_widget(caller)) {
+		if(generator_->item(i).has_widget(caller)) {
 			generator_->toggle_item(i);
 			if(callback_value_changed_) {
 				callback_value_changed_(this);
@@ -149,6 +221,86 @@ void tlistbox::list_item_clicked(twidget* caller)
 	assert(false);
 }
 
+bool tlistbox::update_content_size()
+{
+	if(get_visible() == twidget::INVISIBLE) {
+		return true;
+	}
+
+	if(get_size() == tpoint(0, 0)) {
+		return false;
+	}
+
+	if(content_resize_request(true)) {
+		content_grid_->set_visible_area(content_visible_area());
+		set_dirty();
+		return true;
+	}
+
+	return false;
+}
+
+void tlistbox::place(const tpoint& origin, const tpoint& size)
+{
+	// Inherited.
+	tscrollbar_container::place(origin, size);
+
+	/**
+	 * @todo Work-around to set the selected item visible again.
+	 *
+	 * At the moment the listboxes and dialogs in general are resized a lot as
+	 * work-around for sizing. So this function makes the selected item in view
+	 * again. It doesn't work great in all cases but the proper fix is to avoid
+	 * resizing dialogs a lot. Need more work later on.
+	 */
+	const int selected_item = generator_->get_selected_item();
+	if(selected_item != -1) {
+		const SDL_Rect& visible = content_visible_area();
+		SDL_Rect rect = generator_->item(selected_item).get_rect();
+
+		rect.x = visible.x;
+		rect.w = visible.w;
+
+		show_content_rect(rect);
+	}
+}
+
+void tlistbox::resize_content(
+		  const int width_modification
+		, const int height_modification)
+{
+	DBG_GUI_L << LOG_HEADER << " current size " << content_grid()->get_size()
+			<< " width_modification " << width_modification
+			<< " height_modification " << height_modification
+			<< ".\n";
+
+	if(content_resize_request(width_modification, height_modification)) {
+
+		// Calculate new size.
+		tpoint size = content_grid()->get_size();
+		size.x += width_modification;
+		size.y += height_modification;
+
+		// Set new size.
+		content_grid()->set_size(size);
+
+		// Set status.
+		need_layout_ = true;
+		// If the content grows assume it "overwrites" the old content.
+		if(width_modification < 0 || height_modification < 0) {
+			set_dirty();
+		}
+		DBG_GUI_L << LOG_HEADER << " succeeded.\n";
+	} else {
+		DBG_GUI_L << LOG_HEADER << " failed.\n";
+	}
+}
+
+void tlistbox::layout_children()
+{
+	layout_children(false);
+}
+
 void tlistbox::child_populate_dirty_list(twindow& caller,
 		const std::vector<twidget*>& call_stack)
 {
@@ -170,7 +322,7 @@ void tlistbox::handle_key_up_arrow(SDLMod modifier, bool& handled)
 		// When scrolling make sure the new items is visible but leave the
 		// horizontal scrollbar position.
 		const SDL_Rect& visible = content_visible_area();
-		SDL_Rect rect = generator_->get_item(
+		SDL_Rect rect = generator_->item(
 				generator_->get_selected_item()).get_rect();
 
 		rect.x = visible.x;
@@ -197,7 +349,7 @@ void tlistbox::handle_key_down_arrow(SDLMod modifier, bool& handled)
 		// When scrolling make sure the new items is visible but leave the
 		// horizontal scrollbar position.
 		const SDL_Rect& visible = content_visible_area();
-		SDL_Rect rect = generator_->get_item(
+		SDL_Rect rect = generator_->item(
 				generator_->get_selected_item()).get_rect();
 
 		rect.x = visible.x;
@@ -221,7 +373,22 @@ void tlistbox::handle_key_left_arrow(SDLMod modifier, bool& handled)
 	generator_->handle_key_left_arrow(modifier, handled);
 
 	// Inherited.
-	if(!handled) {
+	if(handled) {
+		// When scrolling make sure the new items is visible but leave the
+		// vertical scrollbar position.
+		const SDL_Rect& visible = content_visible_area();
+		SDL_Rect rect = generator_->item(
+				generator_->get_selected_item()).get_rect();
+
+		rect.y = visible.y;
+		rect.h = visible.h;
+
+		show_content_rect(rect);
+
+		if(callback_value_changed_) {
+			callback_value_changed_(this);
+		}
+	} else {
 		tscrollbar_container::handle_key_left_arrow(modifier, handled);
 	}
 }
@@ -233,50 +400,23 @@ void tlistbox::handle_key_right_arrow(SDLMod modifier, bool& handled)
 	generator_->handle_key_right_arrow(modifier, handled);
 
 	// Inherited.
-	if(!handled) {
-		tscrollbar_container::handle_key_right_arrow(modifier, handled);
-	}
-}
+	if(handled) {
+		// When scrolling make sure the new items is visible but leave the
+		// vertical scrollbar position.
+		const SDL_Rect& visible = content_visible_area();
+		SDL_Rect rect = generator_->item(
+				generator_->get_selected_item()).get_rect();
 
-void tlistbox::init_linked_size_widets(twindow& window,
-		const tgrid::iterator& begin, const tgrid::iterator& end)
-{
-	for(tgrid::iterator itor = begin; itor != end; ++itor) {
+		rect.y = visible.y;
+		rect.h = visible.h;
 
-		assert(*itor);
+		show_content_rect(rect);
 
-		// Add to list.
-		if(!itor->id().empty()) {
-			window.init_linked_size_group(itor->id(), true, false);
-		}
-
-		// Recurse though the children.
-		tcontainer_* container = dynamic_cast<tcontainer_*>(*itor);
-		if(container) {
-			init_linked_size_widets(window,
-					container->begin(), container->end());
-		}
-	}
-}
-
-void tlistbox::add_linked_size_widgets(twindow& window,
-		const tgrid::iterator& begin, const tgrid::iterator& end)
-{
-	for(tgrid::iterator itor = begin; itor != end; ++itor) {
-
-		assert(*itor);
-
-		// Add to list.
-		if(!itor->id().empty()) {
-			window.add_linked_widget(itor->id(), *itor);
-		}
-
-		// Recurse though the children.
-		tcontainer_* container = dynamic_cast<tcontainer_*>(*itor);
-		if(container) {
-			add_linked_size_widgets(window,
-					container->begin(), container->end());
+		if(callback_value_changed_) {
+			callback_value_changed_(this);
 		}
+	} else {
+		tscrollbar_container::handle_key_left_arrow(modifier, handled);
 	}
 }
 
@@ -296,10 +436,10 @@ void swap_grid(tgrid* grid,
 	// Get the container containing the wanted widget.
 	tgrid* parent_grid = NULL;
 	if(grid) {
-		parent_grid = grid->find_widget<tgrid>(id, false, false);
+		parent_grid = find_widget<tgrid>(grid, id, false, false);
 	}
 	if(!parent_grid) {
-		parent_grid = content_grid->find_widget<tgrid>(id, true, false);
+		parent_grid = find_widget<tgrid>(content_grid, id, true, false);
 	}
 	parent_grid = dynamic_cast<tgrid*>(parent_grid->parent());
 	assert(parent_grid);
@@ -337,6 +477,41 @@ void tlistbox::finalize(
 
 }
 
+void tlistbox::set_content_size(const tpoint& origin, const tpoint& size)
+{
+	/** @todo This function needs more testing. */
+	assert(content_grid());
+
+	const int best_height = content_grid()->get_best_size().y;
+	const tpoint s(size.x, size.y < best_height ? size.y : best_height);
+	try {
+		content_grid()->place(origin, s);
+	} catch(tlayout_placement_failed&) {
+		ERR_GUI_L << LOG_HEADER << " placement of the content has failed, "
+				" hope the window can save us.\n";
+		twindow *window = get_window();
+		assert(window);
+
+		window->invalidate_layout();
+	}
+}
+
+void tlistbox::layout_children(const bool force)
+{
+	assert(content_grid());
+
+	if(need_layout_ || force) {
+		content_grid()->place(
+				  content_grid()->get_origin()
+				, content_grid()->get_size());
+
+		content_grid()->set_visible_area(content_visible_area_);
+
+		need_layout_ = false;
+		set_dirty();
+	}
+}
+
 const std::string& tlistbox::get_control_type() const
 {
 	static const std::string type = "listbox";
