@@ -1,15 +1,15 @@
-/* $Id: menubar.hpp 37939 2009-08-18 19:47:08Z mordante $ */
+/* $Id: menubar.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
    or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
 #ifndef GUI_WIDGETS_MENUBAR_HPP_INCLUDED
@@ -20,7 +20,9 @@
 namespace gui2 {
 
 class tselectable_;
-
+namespace implementation {
+	struct tbuilder_menubar;
+}
 /**
  * A menu bar.
  *
@@ -30,7 +32,7 @@ class tselectable_;
  */
 class tmenubar : public tcontainer_
 {
-	friend struct tbuilder_menubar;
+	friend struct implementation::tbuilder_menubar;
 public:
 	/** The direction is which the items are next to eachother. */
 	enum tdirection { HORIZONTAL, VERTICAL };
@@ -56,9 +58,6 @@ public:
 	/** Inherited from tcontrol. */
 	unsigned get_state() const { return state_; }
 
-	/** Inherited from tcontainer_. */
-	bool has_vertical_scrollbar() const { return false; }
-
 	/**
 	 * Update our state when a widget selected or deselected an item.
 	 *
