@@ -1,7 +1,7 @@
-/* $Id: password_box.cpp 35608 2009-05-12 19:58:37Z mordante $ */
+/* $Id: password_box.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2009 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
-   Copyright (C) 2009 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2009 - 2010 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
+   Copyright (C) 2009 - 2010 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -16,8 +16,13 @@
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "gui/widgets/password_box.hpp"
+
+#include "gui/auxiliary/log.hpp"
 #include "serialization/string_utils.hpp"
 
+#define LOG_SCOPE_HEADER get_control_type() + " [" + id() + "] " + __func__
+#define LOG_HEADER LOG_SCOPE_HEADER + ':'
+
 namespace gui2 {
 
 namespace {
@@ -51,7 +56,7 @@ void tpassword_box::handle_key_backspace(SDLMod /*modifier*/, bool& handled) {
 	pre_function();
 
 	// Copy & paste from ttext_::handle_key_backspace()
-	DBG_G_E << "Text: key press: backspace.\n";
+	DBG_GUI_E << LOG_SCOPE_HEADER << '\n';
 
 	handled = true;
 	if(get_selection_length() != 0) {
@@ -67,7 +72,7 @@ void tpassword_box::handle_key_delete(SDLMod /*modifier*/, bool& handled) {
 	pre_function();
 
 	// Copy & paste from ttext_::handle_key_delete()
-	DBG_G_E << "Text: key press: delete.\n";
+	DBG_GUI_E << LOG_SCOPE_HEADER << '\n';
 
 	handled = true;
 	if(get_selection_length() != 0) {
@@ -118,5 +123,11 @@ void tpassword_box::post_function() {
 	set_dirty();
 }
 
+const std::string& tpassword_box::get_control_type() const
+{
+	static const std::string type = "password_box";
+	return type;
+}
+
 } //namespace gui2
 
