@@ -1,27 +1,32 @@
-/* $Id: scrollbar_container.hpp 37939 2009-08-18 19:47:08Z mordante $ */
+/* $Id: scrollbar_container.hpp 41319 2010-02-21 10:43:59Z mordante $ */
 /*
-   copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
    or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
 #ifndef GUI_WIDGETS_SCROLLBAR_CONTAINER_HPP_INCLUDED
 #define GUI_WIDGETS_SCROLLBAR_CONTAINER_HPP_INCLUDED
 
 #include "gui/widgets/container.hpp"
+#include "gui/widgets/scrollbar.hpp"
 
 namespace gui2 {
 
-class tscrollbar_;
 class tspacer;
 
+namespace implementation {
+	struct tbuilder_scroll_label;
+	struct tbuilder_scrollbar_panel;
+}
+
 /**
  * Base class for creating containers with one or two scrollbar(s).
  *
@@ -35,8 +40,11 @@ class tscrollbar_container
 {
 	friend class tdebug_layout_graph;
 
-	friend struct tbuilder_scroll_label;
+	friend struct implementation::tbuilder_scroll_label;
+	friend struct implementation::tbuilder_scrollbar_panel;
 	friend class tlistbox;
+	friend class ttree_view;
+	friend struct tscrollbar_container_implementation;
 
 public:
 
@@ -46,29 +54,41 @@ public:
 
 	/** The way to handle the showing or hiding of the scrollbar. */
 	enum tscrollbar_mode {
-		SHOW,                     /**<
-								   * The scrollbar is always shown, whether
-								   * needed or not.
-								   */
-		HIDE,                     /**<
-								   * The scrollbar is never shown even not when
-								   * needed. There's also no space reserved for
-								   * the scrollbar.
-								   */
-		SHOW_WHEN_NEEDED          /**<
-								   * The scrollbar is shown when the number of
-								   * items is larger as the visible items. The
-								   * space for the scrollbar is always
-								   * reserved, just in case it's needed after
-								   * the initial sizing (due to adding items).
-								   */
+		always_visible,           /**<
+		                           * The scrollbar is always shown, whether
+		                           * needed or not.
+		                           */
+		always_invisible,         /**<
+		                           * The scrollbar is never shown even not
+		                           * when needed. There's also no space
+		                           * reserved for the scrollbar.
+		                           */
+		auto_visible,             /**<
+		                           * The scrollbar is shown when the number of
+		                           * items is larger as the visible items. The
+		                           * space for the scrollbar is always
+		                           * reserved, just in case it's needed after
+		                           * the initial sizing (due to adding items).
+		                           */
+		auto_visible_first_run    /**<
+		                           * Like auto_visible, but when not needed
+		                           * upon the initial layout phase, the bars
+		                           * are not shown and no space is reserved
+		                           * for them. (The algorithm hides them by
+		                           * default.
+		                           */
 	};
 
 	/***** ***** ***** ***** layout functions ***** ***** ***** *****/
 
 	/** Inherited from tcontainer_. */
-	void layout_init();
-	void layout_init2(const bool full_initialization);
+	void layout_init(const bool full_initialization);
+
+	/** Inherited from twidget. */
+	void request_reduce_height(const unsigned maximum_height);
+
+	/** Inherited from tcontrol. */
+	void request_reduce_width(const unsigned maximum_width);
 
 	/** Inherited from tcontainer_. */
 	bool can_wrap() const
@@ -79,32 +99,13 @@ public:
 			: false;
 	}
 
-	/** Inherited from twidget (not tcontainer_). */
-	void layout_wrap(const unsigned maximum_width);
-
-	/** Inherited from twidget (not tcontainer_). */
-	bool has_vertical_scrollbar() const;
-
-	/** Inherited from tcontainer_. */
-	void layout_use_vertical_scrollbar(const unsigned maximum_height);
-
-	/** Inherited from twidget (not tcontainer_). */
-	bool has_horizontal_scrollbar() const;
-
-	/** Inherited from tcontainer_. */
-	void layout_use_horizontal_scrollbar(const unsigned maximum_width);
-
-	/** Inherited from tcontainer. */
-	void layout_fit_width(const unsigned maximum_width,
-			const tfit_flags flags);
-
 private:
 	/** Inherited from tcontainer_. */
 	tpoint calculate_best_size() const;
 public:
 
 	/** Inherited from tcontainer_. */
-	void set_size(const tpoint& origin, const tpoint& size);
+	void place(const tpoint& origin, const tpoint& size);
 
 	/** Inherited from tcontainer_. */
 	void set_origin(const tpoint& origin);
@@ -114,13 +115,6 @@ public:
 
 	/***** ***** ***** inherited ****** *****/
 
-	/** Inherited from tevent_executor. */
-	void key_press(tevent_handler& event,
-		bool& handled, SDLKey key, SDLMod modifier, Uint16 unicode);
-
-	/** Inherted from tevent_executor. */
-	void focus(tevent_handler&);
-
 	/** Inherited from tcontainer_. */
 	bool get_active() const { return state_ != DISABLED; }
 
@@ -128,24 +122,29 @@ public:
 	unsigned get_state() const { return state_; }
 
 	/** Inherited from tcontainer_. */
-	twidget* find_widget(const tpoint& coordinate, const bool must_be_active);
+	twidget* find_at(const tpoint& coordinate, const bool must_be_active);
 
 	/** Inherited from tcontainer_. */
-	const twidget* find_widget(const tpoint& coordinate,
+	const twidget* find_at(const tpoint& coordinate,
 			const bool must_be_active) const;
 
-	/** Import overloaded versions. */
-	using tcontainer_::find_widget;
+	/** Inherited from tcontainer_. */
+	twidget* find(const std::string& id, const bool must_be_active);
+
+	/** Inherited from tcontrol.*/
+	const twidget* find(const std::string& id, const bool must_be_active) const;
 
 	/** Inherited from tcontainer_. */
-	bool does_block_easy_close() const;
+	bool disable_click_dismiss() const;
 
 	/***** ***** ***** setters / getters for members ***** ****** *****/
 
+	/** @note shouldn't be called after being shown in a dialog. */
 	void set_vertical_scrollbar_mode(const tscrollbar_mode scrollbar_mode);
 	tscrollbar_mode get_vertical_scrollbar_mode() const
 		{ return vertical_scrollbar_mode_; }
 
+	/** @note shouldn't be called after being shown in a dialog. */
 	void set_horizontal_scrollbar_mode(const tscrollbar_mode scrollbar_mode);
 	tscrollbar_mode get_horizontal_scrollbar_mode() const
 		{ return horizontal_scrollbar_mode_; }
@@ -159,6 +158,22 @@ public:
 	const SDL_Rect& content_visible_area() const
 		{ return content_visible_area_; }
 
+	/***** ***** ***** scrollbar helpers ***** ****** *****/
+
+	/**
+	 * Scrolls the vertical scrollbar.
+	 *
+	 * @param scroll              The position to scroll to.
+	 */
+	void scroll_vertical_scrollbar(const tscrollbar_::tscroll scroll);
+
+	/**
+	 * Scrolls the horizontal scrollbar.
+	 *
+	 * @param scroll              The position to scroll to.
+	 */
+	void scroll_horizontal_scrollbar(const tscrollbar_::tscroll scroll);
+
 	/**
 	 * Callback when the scrollbar moves (NOTE maybe only one callback needed).
 	 * Maybe also make protected or private and add a friend.
@@ -204,19 +219,77 @@ protected:
 	 */
 	void set_scrollbar_button_status();
 
-	/***** ***** ***** ***** mouse wheel functions ***** ***** ***** *****/
+	/**
+	 * Notification if the content of a child needs a resize.
+	 *
+	 * When a resize is required the container first can try to handle it
+	 * itself. If it can't honour the request the function will call @ref
+	 * twindow::invalidate_layout().
+	 *
+	 * @note Calling this function on a widget with size == (0, 0) results
+	 * false but doesn't call invalidate_layout, the engine expects to be in
+	 * build up phase with the layout already invalidated.
+	 *
+	 * @param force_sizing        If the contents fit do we want to force a
+	 *                            resize? This is needed in the MP lobby since
+	 *                            items might not be properly placed yet.
+	 *                            (The listboxes with the player info need it.)
+	 *
+	 * @returns                   True if the resize is handled, false
+	 *                            otherwise.
+	 */
+	bool content_resize_request(const bool force_sizing = false);
 
-	/** Inherited from tevent_executor. */
-	void mouse_wheel_up(tevent_handler& event_handler, bool& handled);
+	/**
+	 * Request from the content to modify the size of the container.
+	 *
+	 * When the wanted resize fails the function will call @ref
+	 * twindow::invalidate_layout().
+	 *
+	 *
+	 * @note Calling this function on a widget with size == (0, 0) results
+	 * false but doesn't call invalidate_layout, the engine expects to be in
+	 * build up phase with the layout already invalidated.
+	 *
+	 * @note If @ref twindow::get_need_layout() is true the function returns
+	 * false and doesn't try to fit the contents since a layout phase will be
+	 * triggered anyway.
+	 *
+	 * @note This function might replace the @ref content_resize_request above.
+	 *
+	 * @param width_modification  The wanted modification to the width:
+	 *                            * negative values reduce width.
+	 *                            * zero leave width as is.
+	 *                            * positive values increase width.
+	 * @param height_modification The wanted modification to the height:
+	 *                            * negative values reduce height.
+	 *                            * zero leave height as is.
+	 *                            * positive values increase height.
+	 *
+	 * @returns                   True is wanted modification is accepted false
+	 *                            otherwise.
+	 */
+	bool content_resize_request(
+			  const int width_modification
+			, const int height_modification);
 
-	/** Inherited from tevent_executor. */
-	void mouse_wheel_down(tevent_handler& event_handler, bool& handled);
+private:
 
-	/** Inherited from tevent_executor. */
-	void mouse_wheel_left(tevent_handler& event_handler, bool& handled);
+	/**
+	 * Helper for @ref content_resize_request.
+	 *
+	 * Handle the width modification.
+	 */
+	bool content_resize_width(const int width_modification);
 
-	/** Inherited from tevent_executor. */
-	void mouse_wheel_right(tevent_handler& event_handler, bool& handled);
+	/**
+	 * Helper for @ref content_resize_request.
+	 *
+	 * Handle the height modification.
+	 */
+	bool content_resize_height(const int height_modification);
+
+protected:
 
 	/***** ***** ***** ***** keyboard functions ***** ***** ***** *****/
 
@@ -343,16 +416,6 @@ private:
 		vertical_scrollbar_mode_,
 		horizontal_scrollbar_mode_;
 
-	/**
-	 * The initial mode of how to show the scrollbar.
-	 *
-	 * If layout_init is called a second time the scrollbar_mode should be
-	 * reset to its 'initial' mode. This is the last mode the user set.
-	 */
-	tscrollbar_mode
-		initial_vertical_scrollbar_mode_,
-		initial_horizontal_scrollbar_mode_;
-
 	/** These are valid after finalize_setup(). */
 	tgrid
 		*vertical_scrollbar_grid_,
@@ -377,7 +440,7 @@ private:
 	SDL_Rect content_visible_area_;
 
 	/** The builder needs to call us so we do our setup. */
-	void finalize_setup();
+	void finalize_setup(); // FIXME make protected
 
 	/**
 	 * Function for the subclasses to do their setup.
@@ -386,6 +449,9 @@ private:
 	 */
 	virtual void finalize_subclass() {}
 
+	/** Inherited from tcontainer_. */
+	void layout_children();
+
 	/** Inherited from tcontainer_. */
 	void impl_draw_children(surface& frame_buffer);
 
@@ -393,17 +459,38 @@ private:
 	void child_populate_dirty_list(twindow& caller,
 		const std::vector<twidget*>& call_stack);
 
-	/** Sets the visible state of the vertical scrollbar. */
-	void show_vertical_scrollbar();
-
-	/** Sets the visible state of the horizontal scrollbar. */
-	void show_horizontal_scrollbar();
+	/**
+	 * Sets the size of the content grid.
+	 *
+	 * This function normally just updates the content grid but can be
+	 * overridden by a subclass.
+	 *
+	 * @param origin              The origin for the content.
+	 * @param size                The size of the content.
+	 */
+	virtual void set_content_size(const tpoint& origin, const tpoint& size);
 
 	/** Helper function which needs to be called after the scollbar moved. */
 	void scrollbar_moved();
 
 	/** Inherited from tcontrol. */
 	const std::string& get_control_type() const;
+
+	/***** ***** ***** signal handlers ***** ****** *****/
+
+	void signal_handler_sdl_key_down(const event::tevent event
+			, bool& handled
+			, const SDLKey key
+			, SDLMod modifier);
+
+	void signal_handler_sdl_wheel_up(const event::tevent event
+			, bool& handled);
+	void signal_handler_sdl_wheel_down(const event::tevent event
+			, bool& handled);
+	void signal_handler_sdl_wheel_left(const event::tevent event
+			, bool& handled);
+	void signal_handler_sdl_wheel_right(const event::tevent event
+			, bool& handled);
 };
 
 } // namespace gui2
