@@ -1,20 +1,22 @@
-/* $Id: selectable.hpp 33524 2009-03-10 14:55:11Z silene $ */
+/* $Id: selectable.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   copyright (C) 2007 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2007 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
    or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
 #ifndef GUI_WIDGETS_SELECTABLE_HPP_INCLUDED
 #define GUI_WIDGETS_SELECTABLE_HPP_INCLUDED
 
+#include <boost/function.hpp>
+
 namespace gui2 {
 
 class twidget;
@@ -40,7 +42,7 @@ public:
 	 * When the user does something to change the widget state this event is
 	 * fired. Most of the time it will be a left click on the widget.
 	 */
-	virtual void set_callback_state_change(void (*callback) (twidget*)) = 0;
+	virtual void set_callback_state_change(boost::function<void (twidget*)> callback) = 0;
 };
 
 } // namespace gui2
