@@ -1,15 +1,15 @@
-/* $Id: toggle_button.cpp 37939 2009-08-18 19:47:08Z mordante $ */
+/* $Id: toggle_button.cpp 40674 2010-01-11 22:11:12Z mordante $ */
 /*
-   copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
    or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
@@ -17,12 +17,37 @@
 #include "gui/widgets/toggle_button.hpp"
 
 #include "foreach.hpp"
+#include "gui/auxiliary/log.hpp"
+#include "gui/widgets/settings.hpp"
 #include "gui/widgets/window.hpp"
 #include "sound.hpp"
 
+#include <boost/bind.hpp>
+
+#define LOG_SCOPE_HEADER get_control_type() + " [" + id() + "] " + __func__
+#define LOG_HEADER LOG_SCOPE_HEADER + ':'
 
 namespace gui2 {
 
+ttoggle_button::ttoggle_button()
+	: tcontrol(COUNT)
+	, state_(ENABLED)
+	, retval_(0)
+	, callback_state_change_()
+	, icon_name_()
+{
+	connect_signal<event::MOUSE_ENTER>(boost::bind(
+				&ttoggle_button::signal_handler_mouse_enter, this, _2, _3));
+	connect_signal<event::MOUSE_LEAVE>(boost::bind(
+				&ttoggle_button::signal_handler_mouse_leave, this, _2, _3));
+
+	connect_signal<event::LEFT_BUTTON_CLICK>(boost::bind(
+				&ttoggle_button::signal_handler_left_button_click
+					, this, _2, _3));
+	connect_signal<event::LEFT_BUTTON_DOUBLE_CLICK>(boost::bind(
+				&ttoggle_button::signal_handler_left_button_double_click
+					, this, _2, _3));
+}
 
 void ttoggle_button::set_members(const string_map& data)
 {
@@ -35,57 +60,6 @@ void ttoggle_button::set_members(const string_map& data)
 	}
 }
 
-void ttoggle_button::mouse_enter(tevent_handler&)
-{
-	DBG_G_E << "Toggle button: mouse enter.\n";
-
-	if(get_value()) {
-		set_state(FOCUSSED_SELECTED);
-	} else {
-		set_state(FOCUSSED);
-	}
-}
-
-void ttoggle_button::mouse_leave(tevent_handler&)
-{
-	DBG_G_E << "Toggle button: mouse leave.\n";
-
-	if(get_value()) {
-		set_state(ENABLED_SELECTED);
-	} else {
-		set_state(ENABLED);
-	}
-}
-
-void ttoggle_button::mouse_left_button_click(tevent_handler&)
-{
-	DBG_G_E << "Toggle button: left mouse button click.\n";
-
-	sound::play_UI_sound(settings::sound_toggle_button_click);
-
-	if(get_value()) {
-		set_state(ENABLED);
-	} else {
-		set_state(ENABLED_SELECTED);
-	}
-
-	if(callback_state_change_) {
-		callback_state_change_(this);
-	}
-}
-
-void ttoggle_button::mouse_left_button_double_click(tevent_handler&)
-{
-	DBG_G_E << "Toggle button: left mouse button double click.\n";
-
-	assert(retval_ != 0);
-
-	twindow* window = get_window();
-	assert(window);
-
-	window->set_retval(retval_);
-}
-
 void ttoggle_button::set_active(const bool active)
 {
 	if(active) {
@@ -144,7 +118,6 @@ void ttoggle_button::set_state(const tstate state)
 {
 	if(state != state_) {
 		state_ = state;
-		set_block_easy_close(get_visible() && get_active());
 		set_dirty(true);
 	}
 }
@@ -155,5 +128,66 @@ const std::string& ttoggle_button::get_control_type() const
 	return type;
 }
 
+void ttoggle_button::signal_handler_mouse_enter(
+		const event::tevent event, bool& handled)
+{
+	DBG_GUI_E << LOG_HEADER << ' ' << event << ".\n";
+
+	if(get_value()) {
+		set_state(FOCUSSED_SELECTED);
+	} else {
+		set_state(FOCUSSED);
+	}
+	handled = true;
+}
+
+void ttoggle_button::signal_handler_mouse_leave(
+		const event::tevent event, bool& handled)
+{
+	DBG_GUI_E << LOG_HEADER << ' ' << event << ".\n";
+
+	if(get_value()) {
+		set_state(ENABLED_SELECTED);
+	} else {
+		set_state(ENABLED);
+	}
+	handled = true;
+}
+
+void ttoggle_button::signal_handler_left_button_click(
+		const event::tevent event, bool& handled)
+{
+	DBG_GUI_E << LOG_HEADER << ' ' << event << ".\n";
+
+	sound::play_UI_sound(settings::sound_toggle_button_click);
+
+	if(get_value()) {
+		set_state(ENABLED);
+	} else {
+		set_state(ENABLED_SELECTED);
+	}
+
+	if(callback_state_change_) {
+		callback_state_change_(this);
+	}
+	handled = true;
+}
+
+void ttoggle_button::signal_handler_left_button_double_click(
+		const event::tevent event, bool& handled)
+{
+	DBG_GUI_E << LOG_HEADER << ' ' << event << ".\n";
+
+	if(retval_ == 0) {
+		return;
+	}
+
+	twindow* window = get_window();
+	assert(window);
+
+	window->set_retval(retval_);
+
+	handled = true;
+}
 } // namespace gui2
 
