@@ -1,15 +1,15 @@
-/* $Id: toggle_button.hpp 37939 2009-08-18 19:47:08Z mordante $ */
+/* $Id: toggle_button.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
    or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
 #ifndef GUI_WIDGETS_TOGGLE_BUTTON_HPP_INCLUDED
@@ -29,29 +29,10 @@ namespace gui2 {
 class ttoggle_button : public tcontrol, public tselectable_
 {
 public:
-	ttoggle_button() :
-		tcontrol(COUNT),
-		state_(ENABLED),
-		retval_(0),
-		callback_state_change_(0),
-		icon_name_()
-	{
-	}
+	ttoggle_button();
 
 	/***** ***** ***** ***** Inherited ***** ***** ***** *****/
 
-	/** Inherted from tevent_executor. */
-	void mouse_enter(tevent_handler&);
-
-	/** Inherted from tevent_executor. */
-	void mouse_leave(tevent_handler&);
-
-	/** Inherted from tevent_executor. */
-	void mouse_left_button_click(tevent_handler&);
-
-	/** Inherted from tevent_executor. */
-	void mouse_left_button_double_click(tevent_handler&);
-
 	/**
 	 * Inherited from tcontrol.
 	 *
@@ -70,9 +51,6 @@ public:
 	/** Inherited from tcontrol. */
 	unsigned get_state() const { return state_; }
 
-	/** Inherited from tcontrol. */
-	bool does_block_easy_close() const { return true; }
-
 	/** Inherited from tcontrol. */
 	void update_canvas();
 
@@ -87,7 +65,7 @@ public:
 	void set_retval(const int retval);
 
 	/** Inherited from tselectable_. */
-	void set_callback_state_change(void (*callback) (twidget*))
+	void set_callback_state_change(boost::function<void (twidget*)> callback)
 		{ callback_state_change_ = callback; }
 
 	void set_icon_name(const std::string& icon_name)
@@ -127,7 +105,7 @@ private:
 	int retval_;
 
 	/** See tselectable_::set_callback_state_change. */
-	void (*callback_state_change_) (twidget*);
+	boost::function<void (twidget*)> callback_state_change_;
 
 	/**
 	 * The toggle button can contain an icon next to the text.
@@ -138,6 +116,17 @@ private:
 	/** Inherited from tcontrol. */
 	const std::string& get_control_type() const;
 
+	/***** ***** ***** signal handlers ***** ****** *****/
+
+	void signal_handler_mouse_enter(const event::tevent event, bool& handled);
+
+	void signal_handler_mouse_leave(const event::tevent event, bool& handled);
+
+	void signal_handler_left_button_click(
+			const event::tevent event, bool& handled);
+
+	void signal_handler_left_button_double_click(
+			const event::tevent event, bool& handled);
 };
 
 } // namespace gui2
