@@ -1,15 +1,15 @@
-/* $Id: toggle_panel.hpp 37939 2009-08-18 19:47:08Z mordante $ */
+/* $Id: toggle_panel.hpp 41499 2010-03-07 18:43:38Z mordante $ */
 /*
-   copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
    or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
 #ifndef GUI_WIDGETS_TOGGLE_PANEL_HPP_INCLUDED
@@ -30,13 +30,7 @@ namespace gui2 {
 class ttoggle_panel : public tpanel, public tselectable_
 {
 public:
-	ttoggle_panel() :
-		tpanel(COUNT),
-		state_(ENABLED),
-		retval_(0),
-		callback_state_change_(0)
-	{
-	}
+	ttoggle_panel();
 
 	/**
 	 * Sets the members of the child controls.
@@ -50,20 +44,8 @@ public:
 
 	/***** ***** ***** ***** Inherited ***** ***** ***** *****/
 
-	/** Inherted from tevent_executor. */
-	void mouse_enter(tevent_handler&);
-
-	/** Inherted from tevent_executor. */
-	void mouse_leave(tevent_handler&);
-
-	/** Inherted from tevent_executor. */
-	void mouse_left_button_click(tevent_handler&);
-
-	/** Inherted from tevent_executor. */
-	void mouse_left_button_double_click(tevent_handler&);
-
 	/** Inherited from tcontainer_ */
-	twidget* find_widget(const tpoint& coordinate, const bool must_be_active)
+	twidget* find_at(const tpoint& coordinate, const bool must_be_active)
 	{
 		/**
 		 * @todo since there is no mouse event nesting (or event nesting at all)
@@ -72,22 +54,20 @@ public:
 		 * intended button on the addon panel. So we need to chain mouse events
 		 * as well and also add a handled flag for them.
 		 */
-//		twidget* result = tcontainer_::find_widget(coordinate, must_be_active);
-		return /*result ? result :*/ tcontrol::find_widget(coordinate, must_be_active);
+
+		twidget* result = tcontainer_::find_at(coordinate, must_be_active);
+		return result ? result : tcontrol::find_at(coordinate, must_be_active);
 	}
 
 	/** Inherited from tcontainer_ */
-	const twidget* find_widget(const tpoint& coordinate, const bool must_be_active) const
+	const twidget* find_at(
+			const tpoint& coordinate, const bool must_be_active) const
 	{
-//		const twidget* result = tcontainer_::find_widget(coordinate, must_be_active);
-		return /*result ? result :*/ tcontrol::find_widget(coordinate, must_be_active);
+		const twidget* result =
+				tcontainer_::find_at(coordinate, must_be_active);
+		return result ? result : tcontrol::find_at(coordinate, must_be_active);
 	}
 
-	// Needed to import the find_widget(const tpoint&, const bool) and it's const version
-	// inheriting from panel eventhought they are the same as tcontainer_ but it might be
-	// panel reimplements it.
-	using tpanel::find_widget;
-
 	/** Inherited from tpanel. */
 	void set_active(const bool active);
 
@@ -127,9 +107,11 @@ public:
 	void set_retval(const int retval);
 
 	/** Inherited from tselectable_. */
-	void set_callback_state_change(void (*callback) (twidget*))
+	void set_callback_state_change(boost::function<void (twidget*)> callback)
 		{ callback_state_change_ = callback; }
 
+	void set_callback_mouse_left_double_click(boost::function<void (twidget*)> callback)
+		{ callback_mouse_left_double_click_ = callback; }
 private:
 
 	/**
@@ -155,7 +137,7 @@ private:
 	 */
 	tstate state_;
 
- 	/**
+	/**
 	 * The return value of the button.
 	 *
 	 * If this value is not 0 and the button is double clicked it sets the
@@ -163,8 +145,11 @@ private:
 	 */
 	int retval_;
 
- 	/** See tselectable_::set_callback_state_change. */
-	void (*callback_state_change_) (twidget*);
+	/** See tselectable_::set_callback_state_change. */
+	boost::function<void (twidget*)> callback_state_change_;
+
+	/** Mouse left double click callback */
+	boost::function<void (twidget*)> callback_mouse_left_double_click_;
 
 	/** Inherited from tpanel. */
 	void impl_draw_background(surface& frame_buffer)
@@ -186,6 +171,17 @@ private:
 	/** Inherited from tpanel. */
 	const std::string& get_control_type() const;
 
+	/***** ***** ***** signal handlers ***** ****** *****/
+
+	void signal_handler_mouse_enter(const event::tevent event, bool& handled);
+
+	void signal_handler_mouse_leave(const event::tevent event, bool& handled);
+
+	void signal_handler_left_button_click(
+			const event::tevent event, bool& handled);
+
+	void signal_handler_left_button_double_click(
+			const event::tevent event, bool& handled);
 };
 
 } // namespace gui2
