@@ -1,6 +1,6 @@
-/* $Id: window.hpp 37939 2009-08-18 19:47:08Z mordante $ */
+/* $Id: window.hpp 40722 2010-01-17 11:08:02Z mordante $ */
 /*
-   Copyright (C) 2007 - 2009 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2007 - 2010 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -22,17 +22,16 @@
 #define GUI_WIDGETS_WINDOW_HPP_INCLUDED
 
 #include "cursor.hpp"
-#include "gui/widgets/event_handler.hpp"
-#include "gui/widgets/formula.hpp"
+#include "gui/auxiliary/formula.hpp"
 #include "gui/widgets/helper.hpp"
 #include "gui/widgets/panel.hpp"
-#include "gui/widgets/settings.hpp"
 #include "gui/widgets/tooltip.hpp"
 
 #include "events.hpp"
 #include "SDL.h"
 
 #include <string>
+#include <boost/function.hpp>
 
 class CVideo;
 
@@ -41,13 +40,9 @@ namespace gui2{
 class tdialog;
 class tdebug_layout_graph;
 
-/**
- * Dummy struct for the layout engine.
- *
- * When an object changes it's width in the height phase of the layout engine
- * things might need to change so rerun the layout phase in that case.
- */
-struct trelayout_exception {};
+namespace event {
+	class tdistributor;
+} // namespace event
 
 /**
  * base class of top level items, the only item
@@ -55,13 +50,12 @@ struct trelayout_exception {};
  */
 class twindow
 	: public tpanel
-	, public tevent_handler
 	, public cursor::setter
 {
 	friend class tdebug_layout_graph;
-
-	// Wants to use layout().
-	friend class tmessage;
+	friend twindow* build(CVideo&, const std::string&);
+	friend struct twindow_implementation;
+	friend class tinvalidate_layout_blocker;
 
 public:
 
@@ -73,8 +67,12 @@ public:
 		const bool automatic_placement,
 		const unsigned horizontal_placement,
 		const unsigned vertical_placement,
+		const unsigned maximum_width,
+		const unsigned maximum_height,
 		const std::string& definition);
 
+	~twindow();
+
 	/**
 	 * Update the size of the screen variables in settings.
 	 *
@@ -85,6 +83,15 @@ public:
 	 */
 	static void update_screen_size();
 
+	/**
+	 * Returns the intance of a window.
+	 *
+	 * @param handle              The instance id of the window.
+	 *
+	 * @returns                   The window or NULL.
+	 */
+	static twindow* window_instance(const unsigned handle);
+
 	/**
 	 * Default return values.
 	 *
@@ -103,10 +110,15 @@ public:
 										* message popup can do it.
 										*/
 		OK = -1,                       /**< Dialog is closed with ok button. */
-		CANCEL = -2                    /**<
+		CANCEL = -2,                   /**<
 										* Dialog is closed with the cancel
 										* button.
 										*/
+		AUTO_CLOSE = -3                /**<
+		                                * The dialog is closed automatically
+		                                * since it's timeout has been
+		                                * triggered.
+		                                */
 		};
 
 	/** Gets the retval for the default buttons. */
@@ -117,16 +129,17 @@ public:
 	 *
 	 * @param restore             Restore the screenarea the window was on
 	 *                            after closing it?
-	 * @param flip_function       If not NULL this function is used to do the
-	 *                            flipping of the video buffer, otherwise
-	 *                            twindow::flip() is used. @todo Evaluate
-	 *                            whether this is still needed and if so
-	 *                            implement.
+	 * @param auto_close_timeout  The time in ms after which the window will
+	 *                            automatically close, if 0 it doesn't close.
+	 *                            @note the timeout is a minimum time and
+	 *                            there's no quarantee about how fast it closes
+	 *                            after the minimum.
 	 *
 	 * @returns                   The close code of the window, predefined
 	 *                            values are listed in tretval.
 	 */
-	int show(const bool restore = true, void* flip_function = NULL);
+	int show(const bool restore = true,
+			const unsigned auto_close_timeout = 0);
 
 	/**
 	 * Draws the window.
@@ -169,14 +182,23 @@ public:
 	void close() { status_ = REQUEST_CLOSE; }
 
 	/**
-	 * Resize event for the window.
+	 * Helper class to block invalidate_layout.
 	 *
-	 * @param event_handler       The handler sending the event.
-	 * @param new_width           The new width for the window.
-	 * @param new_height          The new height for the window.
+	 * Some widgets can handling certain layout aspects without help. For
+	 * example a listbox can handle hiding and showing rows without help but
+	 * setting the visibility calls invalidate_layout(). When this blocker is
+	 * instanciated the call to invalidate_layout() becomes a nop.
+	 *
+	 * @note The class can't be used recursively.
 	 */
-	void window_resize(tevent_handler& event_handler,
-		const unsigned new_width, const unsigned new_height);
+	class tinvalidate_layout_blocker
+	{
+	public:
+		tinvalidate_layout_blocker(twindow& window);
+		~tinvalidate_layout_blocker();
+	private:
+		twindow& window_;
+	};
 
 	/**
 	 * Updates the size of the window.
@@ -185,11 +207,7 @@ public:
 	 * window. To be used after creation and after modification or items which
 	 * can have different sizes eg listboxes.
 	 */
-	void invalidate_layout() { need_layout_ = true; }
-
-	/** Inherited from tevent_executor. */
-	void key_press(tevent_handler& event_handler, bool& handled,
-		SDLKey key, SDLMod modifier, Uint16 unicode);
+	void invalidate_layout();
 
 	/** Inherited from tevent_handler. */
 	twindow& get_window() { return *this; }
@@ -198,52 +216,70 @@ public:
 	const twindow& get_window() const { return *this; }
 
 	/** Inherited from tevent_handler. */
-	twidget* find_widget(const tpoint& coordinate, const bool must_be_active)
-		{ return tpanel::find_widget(coordinate, must_be_active); }
+	twidget* find_at(const tpoint& coordinate, const bool must_be_active)
+		{ return tpanel::find_at(coordinate, must_be_active); }
 
 	/** Inherited from tevent_handler. */
-	const twidget* find_widget(const tpoint& coordinate,
+	const twidget* find_at(const tpoint& coordinate,
 			const bool must_be_active) const
-		{ return tpanel::find_widget(coordinate, must_be_active); }
+		{ return tpanel::find_at(coordinate, must_be_active); }
 
 	/** Inherited from twidget. */
 	tdialog* dialog() { return owner_; }
 
 	/** Inherited from tcontainer_. */
-	twidget* find_widget(const std::string& id, const bool must_be_active)
-		{ return tcontainer_::find_widget(id, must_be_active); }
+	twidget* find(const std::string& id, const bool must_be_active)
+		{ return tcontainer_::find(id, must_be_active); }
 
 	/** Inherited from tcontainer_. */
-	const twidget* find_widget(const std::string& id,
+	const twidget* find(const std::string& id,
 			const bool must_be_active) const
-		{ return tcontainer_::find_widget(id, must_be_active); }
-
+		{ return tcontainer_::find(id, must_be_active); }
+#if 0
+	/** @todo Implement these functions. */
 	/**
 	 * Register a widget that prevents easy closing.
 	 *
-	 * Duplicate registration are ignored. See easy_close_ for more info.
+	 * Duplicate registration are ignored. See click_dismiss_ for more info.
 	 *
 	 * @param id                  The id of the widget to register.
 	 */
-	void add_easy_close_blocker(const std::string& id);
+	void add_click_dismiss_blocker(const std::string& id);
 
 	/**
 	 * Unregister a widget the prevents easy closing.
 	 *
 	 * Removing a non registered id is allowed but will do nothing. See
-	 * easy_close_ for more info.
+	 * click_dismiss_ for more info.
 	 *
 	 * @param id                  The id of the widget to register.
 	 */
-	void remove_easy_close_blocker(const std::string& id);
+	void remove_click_dismiss_blocker(const std::string& id);
+#endif
 
 	/**
 	 * Does the window close easily?
 	 *
+	 * The behaviour can change at run-time, but that might cause oddities
+	 * with the easy close button (when one is needed).
+	 *
 	 * @returns                   Whether or not the window closes easily.
 	 */
-	bool does_easy_close() const
-		{ return easy_close_ && easy_close_blocker_.empty(); }
+	bool does_click_dismiss() const
+	{
+		return click_dismiss_ && !disable_click_dismiss();
+	}
+
+	/**
+	 * Disable the enter key.
+	 *
+	 * This is added to block dialogs from being closed automatically.
+	 *
+	 * @todo this function should be merged with the hotkey support once
+	 * that has been added.
+	 */
+	void set_enter_disabled(const bool enter_disabled)
+		{ enter_disabled_ = enter_disabled; }
 
 	/**
 	 * Disable the escape key.
@@ -268,12 +304,20 @@ public:
 	void init_linked_size_group(const std::string& id,
 			const bool fixed_width, const bool fixed_height);
 
+	/**
+	 * Is the linked size group defined for this window?
+	 *
+	 * @param id                  The id of the group.
+	 *
+	 * @returns                   True if defined, false otherwise.
+	 */
+	bool has_linked_size_group(const std::string& id);
+
 	/**
 	 * Adds a widget to a linked size group.
 	 *
 	 * The group needs to exist, which is done by calling
 	 * init_linked_size_group. A widget may only be member of one group.
-	 * @todo There's no way to remove a widget from the list.
 	 * @todo Untested if a new widget is added after showing the widgets.
 	 *
 	 * @param id                  The id of the group.
@@ -281,23 +325,43 @@ public:
 	 */
 	void add_linked_widget(const std::string& id, twidget* widget);
 
+	/**
+	 * Removes a widget from a linked size group.
+	 *
+	 * The group needs to exist, which is done by calling
+	 * init_linked_size_group. If the widget is no member of the group the
+	 * function does nothing.
+	 *
+	 * @param id                  The id of the group.
+	 * @param widget              The widget to remove from the group.
+	 */
+	void remove_linked_widget(const std::string& id, const twidget* widget);
+
 	/***** ***** ***** setters / getters for members ***** ****** *****/
 
+	CVideo& video() { return video_; }
+
 	/**
 	 * Sets there return value of the window.
 	 *
 	 * @param retval              The return value for the window.
-	 * @param close               Close the window after setting the value.
+	 * @param close_window        Close the window after setting the value.
 	 */
 	void set_retval(const int retval, const bool close_window = true)
 		{ retval_ = retval; if(close_window) close(); }
 
 	void set_owner(tdialog* owner) { owner_ = owner; }
 
-	void set_easy_close(const bool easy_close) { easy_close_ = easy_close; }
+	void set_click_dismiss(const bool click_dismiss)
+	{
+		click_dismiss_ = click_dismiss;
+	}
 
 	static void set_sunset(const unsigned interval)
 		{ sunset_ = interval ? interval : 5; }
+
+	bool get_need_layout() const { return need_layout_; }
+
 private:
 
 	/** Needed so we can change what's drawn on the screen. */
@@ -320,17 +384,12 @@ private:
 	 */
 	bool need_layout_;
 
+	/** Is invalidate layout blocked see tinvalidate_layout_blocker. */
+	bool invalidate_layout_blocked_;
+
 	/** Avoid drawing the window.  */
 	bool suspend_drawing_;
 
-	/**
-	 * The first window shown is the toplevel window.
-	 *
-	 * The toplevel window is the one that starts and stops the drawing timer.
-	 * It's set when the timer is 0 when the window is shown.
-	 */
-	bool top_level_;
-
 	/** When the window closes this surface is used to undraw the window. */
 	surface restorer_;
 
@@ -367,6 +426,12 @@ private:
 	 */
 	const unsigned vertical_placement_;
 
+	/** The maximum width if automatic_placement_ is true. */
+	unsigned maximum_width_;
+
+	/** The maximum height if automatic_placement_ is true. */
+	unsigned maximum_height_;
+
 	/** The formula to calulate the x value of the dialog. */
 	tformula<unsigned>x_;
 
@@ -382,16 +447,24 @@ private:
 	/**
 	 * Do we want to have easy close behaviour?
 	 *
-	 * Easy closing means that whenever a mouse click is done the dialog will be
-	 * closed. The widgets in the window may override this behaviour by
-	 * registering themselves as blockers. These items will be stored in
-	 * easy_close_blocker_. So in order to do an easy close the boolean needs to
-	 * be true and the vector empty.
+	 * Easy closing means that whenever a mouse click is done the dialog will
+	 * be closed. The widgets in the window may override this behaviour by
+	 * registering themselves as blockers. This is tested by the function
+	 * disable_click_dismiss().
+	 *
+	 * The handling of easy close is done in the window, in order to do so a
+	 * window either needs a click_dismiss or an ok button. Both will be hidden
+	 * when not needed and when needed first the ok is tried and then the
+	 * click_dismiss button. this allows adding a click_dismiss button to the
+	 * window definition and use the ok from the window instance.
+	 *
+	 * @todo After testing the click dismiss feature it should be documented in
+	 * the wiki.
 	 */
-	bool easy_close_;
+	bool click_dismiss_;
 
-	/** The list with items which prevent the easy close behaviour. */
-	std::vector<std::string> easy_close_blocker_;
+	/** Disable the enter key see our setter for more info. */
+	bool enter_disabled_;
 
 	/** Disable the escape key see our setter for more info. */
 	bool escape_disabled_;
@@ -434,25 +507,30 @@ private:
 	/** List of the widgets, whose size are linked together. */
 	std::map<std::string, tlinked_size> linked_size_;
 
-	/** Layouts the window. */
-	void layout();
-
 	/**
 	 * Layouts the window.
 	 *
-	 * This is a new routine which will be used as fallback when the normal
-	 * layout engine fails. In 1.7 this will become the real layout engine.
+	 * This part does the pre and post processing for the actual layout
+	 * algorithm.
 	 *
-	 * @param maximum_width       The maximum width of the window.
-	 * @param maximum_height      The maximum height of the window.
+	 * @see layout_algorihm for more information.
 	 */
-	void layout2(const unsigned maximum_width, const unsigned maximum_height);
+	void layout();
 
+	/**
+	 * Layouts the linked widgets.
+	 *
+	 * @see layout_algorihm for more information.
+	 */
+	void layout_linked_widgets();
+
+public:
 	/** Inherited from tevent_handler. */
 	void do_show_tooltip(const tpoint& location, const t_string& tooltip);
 
 	/** Inherited from tevent_handler. */
 	void do_remove_tooltip();
+private:
 
 	/** Inherited from tevent_handler. */
 	void do_show_help_popup(const tpoint& location, const t_string& help_popup);
@@ -462,7 +540,7 @@ private:
 		{ help_popup_.set_visible(twidget::HIDDEN); }
 
 	/** Inherited from tevent_handler. */
-	bool easy_close();
+	bool click_dismiss();
 
 	/** Inherited from tcontrol. */
 	const std::string& get_control_type() const;
@@ -483,12 +561,17 @@ private:
 	 */
 	std::vector<std::vector<twidget*> > dirty_list_;
 
+	/**
+	 * Finishes the initialization of the grid.
+	 *
+	 * @param content_grid        The new contents for the content grid.
+	 */
+	void finalize(const boost::intrusive_ptr<tbuilder_grid>& content_grid);
+
 #ifdef DEBUG_WINDOW_LAYOUT_GRAPHS
 	tdebug_layout_graph* debug_layout_;
 
 public:
-	// The destructor only needs to delete the debug_layout_ so declared here.
-	~twindow();
 
 	/** wrapper for tdebug_layout_graph::generate_dot_file. */
 	void generate_dot_file(
@@ -497,6 +580,46 @@ public:
 	void generate_dot_file(const std::string&,
 			const unsigned) {}
 #endif
+
+	event::tdistributor* event_distributor_;
+
+public:
+	// mouse and keyboard_capture should be renamed and stored in the
+	// dispatcher. Chaining probably should remain exclusive to windows.
+	void mouse_capture(const bool capture = true);
+	void keyboard_capture(twidget* widget);
+
+	/**
+	 * Adds the widget to the keyboard chain.
+	 *
+	 * @todo rename to keyboard_add_to_chain.
+	 * @param widget              The widget to add to the chain. The widget
+	 *                            should be valid widget, which hasn't been
+	 *                            added to the chain yet.
+	 */
+	void add_to_keyboard_chain(twidget* widget);
+
+	/**
+	 * Remove the widget from the keyborad chain.
+	 *
+	 * @todo rename to keyboard_remove_from_chain.
+	 *
+	 * @param widget              The widget to be removed from the chain.
+	 */
+	void remove_from_keyboard_chain(twidget* widget);
+
+private:
+
+	/***** ***** ***** signal handlers ***** ****** *****/
+
+	void signal_handler_sdl_video_resize(
+			const event::tevent event, bool& handled, const tpoint& new_size);
+
+	void signal_handler_click_dismiss(
+			const event::tevent event, bool& handled, bool& halt);
+
+	void signal_handler_sdl_key_down(
+			const event::tevent event, bool& handled, const SDLKey key);
 };
 
 } // namespace gui2
