@@ -1,6 +1,6 @@
-/* $Id: halo.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: halo.cpp 41250 2010-02-16 15:19:40Z ai0867 $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,25 +12,19 @@
    See the COPYING file for more details.
 */
 
-//! @file halo.cpp 
-//! Maintain halo-effects for units and items.
-//! Examples: white mage, lighthouse.
+/**
+ * @file halo.cpp
+ * Maintain halo-effects for units and items.
+ * Examples: white mage, lighthouse.
+ */
 
 #include "global.hpp"
-
+#include "animated.hpp"
 #include "display.hpp"
 #include "game_preferences.hpp"
 #include "halo.hpp"
-#include "image.hpp"
-#include "sdl_utils.hpp"
-#include "util.hpp"
-#include "video.hpp"
 #include "serialization/string_utils.hpp"
 
-#include <algorithm>
-#include <cassert>
-#include <map>
-
 namespace halo
 {
 
@@ -40,8 +34,8 @@ display* disp = NULL;
 class effect
 {
 public:
-	effect(int xpos, int ypos, const animated<std::string>::anim_description& img,
-			const gamemap::location& loc, ORIENTATION, bool infinite);
+	effect(int xpos, int ypos, const animated<image::locator>::anim_description& img,
+			const map_location& loc, ORIENTATION, bool infinite);
 
 	void set_location(int x, int y);
 
@@ -51,14 +45,14 @@ public:
 	bool expired()     const { return !images_.cycles() && images_.animation_finished(); }
 	bool need_update() const { return images_.need_update(); }
 	bool does_change() const { return !images_.does_not_change(); }
-	bool on_location(const std::set<gamemap::location>& locations) const;
+	bool on_location(const std::set<map_location>& locations) const;
 
-	void add_overlay_location(std::set<gamemap::location>& locations);
+	void add_overlay_location(std::set<map_location>& locations);
 private:
 
-	const std::string& current_image() { return images_.get_current_frame(); }
+	const image::locator& current_image() { return images_.get_current_frame(); }
 
-	animated<std::string> images_;
+	animated<image::locator> images_;
 
 	ORIENTATION orientation_;
 
@@ -66,40 +60,54 @@ private:
 	surface surf_, buffer_;
 	SDL_Rect rect_;
 
-	//! The location of the center of the halo
-	gamemap::location loc_;
+	/** The location of the center of the halo. */
+	map_location loc_;
 
-	//! All locations over which the halo lies
-	std::vector<gamemap::location> overlayed_hexes_;
+	/** All locations over which the halo lies. */
+	std::vector<map_location> overlayed_hexes_;
 };
 
 std::map<int, effect> haloes;
 int halo_id = 1;
 
-//! Upon unrendering, an invalidation list is send. All haloes in 
-//! that area and the other invalidated haloes are stored in this set. 
-//! Then there'll be tested which haloes overlap 
-//! and they're also stored in this set.
+/**
+ * Upon unrendering, an invalidation list is send. All haloes in that area and
+ * the other invalidated haloes are stored in this set. Then there'll be
+ * tested which haloes overlap and they're also stored in this set.
+ */
 std::set<int> invalidated_haloes;
 
-//! A newly added halo will be added to this list, these haloes don't need to be 
-//! unrendered but do not to be rendered regardless which tiles are invalidated.
-//! These haloes will stay in this set until there're really rendered
-//! (rendering won't happen if for example the halo is offscreen).
+/**
+ * A newly added halo will be added to this list, these haloes don't need to be
+ * unrendered but do not to be rendered regardless which tiles are invalidated.
+ * These haloes will stay in this set until there're really rendered (rendering
+ * won't happen if for example the halo is offscreen).
+ */
 std::set<int> new_haloes;
 
-//! Upon deleting, a halo isn't deleted but added to this set, 
-//! upon unrendering the image is unrendered and deleted.
+/**
+ * Upon deleting, a halo isn't deleted but added to this set, upon unrendering
+ * the image is unrendered and deleted.
+ */
 std::set<int> deleted_haloes;
 
-//! Haloes that have an animation or expiration time 
-//! need to be checked every frame and are stored in this set.
+/**
+ * Haloes that have an animation or expiration time need to be checked every
+ * frame and are stored in this set.
+ */
 std::set<int> changing_haloes;
 
-effect::effect(int xpos, int ypos, const animated<std::string>::anim_description& img,
-	const gamemap::location& loc, ORIENTATION orientation, bool infinite) :
-		images_(img), orientation_(orientation), x_(xpos), y_(ypos),
-		surf_(NULL), buffer_(NULL), rect_(empty_rect), loc_(loc)
+effect::effect(int xpos, int ypos, const animated<image::locator>::anim_description& img,
+		const map_location& loc, ORIENTATION orientation, bool infinite) :
+	images_(img),
+	orientation_(orientation),
+	x_(xpos),
+	y_(ypos),
+	surf_(NULL),
+	buffer_(NULL),
+	rect_(empty_rect),
+	loc_(loc),
+	overlayed_hexes_()
 {
 	assert(disp != NULL);
 
@@ -111,7 +119,7 @@ effect::effect(int xpos, int ypos, const animated<std::string>::anim_description
 
 void effect::set_location(int x, int y)
 {
-	const gamemap::location zero_loc(0,0);
+	const map_location zero_loc(0,0);
 	int new_x = x - disp->get_location_x(zero_loc);
 	int new_y = y - disp->get_location_y(zero_loc);
 	if (new_x != x_ || new_y != y_) {
@@ -132,16 +140,16 @@ bool effect::render()
 		if(disp->shrouded(loc_)) {
 			return false;
 		} else {
-			// The location of a halo is an x,y value and not a map location. 
-			// This means when a map is zoomed, the halo's won't move, 
-			// This glitch is most visible on [item] haloes. 
-			// This workaround always recalculates the location of the halo 
-			// (item haloes have a location parameter to hide them under the shroud) 
+			// The location of a halo is an x,y value and not a map location.
+			// This means when a map is zoomed, the halo's won't move,
+			// This glitch is most visible on [item] haloes.
+			// This workaround always recalculates the location of the halo
+			// (item haloes have a location parameter to hide them under the shroud)
 			// and reapplies that location.
 			// It might be optimized by storing and comparing the zoom value.
 			set_location(
-				disp->get_location_x(loc_) + disp->hex_size() / 2, 
-				disp->get_location_y(loc_) + disp->hex_size() / 2);			
+				disp->get_location_x(loc_) + disp->hex_size() / 2,
+				disp->get_location_y(loc_) + disp->hex_size() / 2);
 		}
 	}
 
@@ -157,7 +165,7 @@ bool effect::render()
 		surf_.assign(flop_surface(surf_));
 	}
 
-	const gamemap::location zero_loc(0,0);
+	const map_location zero_loc(0,0);
 	const int screenx = disp->get_location_x(zero_loc);
 	const int screeny = disp->get_location_y(zero_loc);
 
@@ -168,15 +176,13 @@ bool effect::render()
 	rect_ = rect;
 	SDL_Rect clip_rect = disp->map_outside_area();
 
-	// If rendered the first time, need to determine the area affected. 
+	// If rendered the first time, need to determine the area affected.
 	// If a halo changes size, it is not updated.
 	if(overlayed_hexes_.empty()) {
-		gamemap::location topleft, bottomright;
-		disp->get_rect_hex_bounds(rect, topleft, bottomright);
-		for (int x = topleft.x; x <= bottomright.x; ++x) {
-			for (int y = topleft.y; y <= bottomright.y; ++y) {
-				overlayed_hexes_.push_back(gamemap::location(x, y));
-			}
+		display::rect_of_hexes hexes = disp->hexes_under_rect(rect);
+		display::rect_of_hexes::iterator i = hexes.begin(), end = hexes.end();
+		for (;i != end; ++i) {
+			overlayed_hexes_.push_back(*i);
 		}
 	}
 
@@ -185,7 +191,7 @@ bool effect::render()
 		return false;
 	}
 
-	surface const screen = disp->video().getSurface();
+	surface const screen = disp->get_screen_surface();
 
 	const clip_rect_setter clip_setter(screen,clip_rect);
 	if(buffer_ == NULL || buffer_->w != rect.w || buffer_->h != rect.h) {
@@ -205,18 +211,18 @@ bool effect::render()
 
 void effect::unrender()
 {
-	if(buffer_ == NULL) {
+	if (!surf_ || !buffer_) {
 		return;
 	}
 
-	surface const screen = disp->video().getSurface();
+	surface const screen = disp->get_screen_surface();
 
 	SDL_Rect clip_rect = disp->map_outside_area();
 	const clip_rect_setter clip_setter(screen,clip_rect);
 
-	// Due to scrolling, the location of the rendered halo 
+	// Due to scrolling, the location of the rendered halo
 	// might have changed; recalculate
-	const gamemap::location zero_loc(0,0);
+	const map_location zero_loc(0,0);
 	const int screenx = disp->get_location_x(zero_loc);
 	const int screeny = disp->get_location_y(zero_loc);
 
@@ -228,9 +234,9 @@ void effect::unrender()
 	update_rect(rect_);
 }
 
-bool effect::on_location(const std::set<gamemap::location>& locations) const
+bool effect::on_location(const std::set<map_location>& locations) const
 {
-	for(std::vector<gamemap::location>::const_iterator itor = overlayed_hexes_.begin();
+	for(std::vector<map_location>::const_iterator itor = overlayed_hexes_.begin();
 			itor != overlayed_hexes_.end(); ++itor) {
 		if(locations.find(*itor) != locations.end()) {
 			return true;
@@ -239,9 +245,9 @@ bool effect::on_location(const std::set<gamemap::location>& locations) const
 	return false;
 }
 
-void effect::add_overlay_location(std::set<gamemap::location>& locations)
+void effect::add_overlay_location(std::set<map_location>& locations)
 {
-	for(std::vector<gamemap::location>::const_iterator itor = overlayed_hexes_.begin();
+	for(std::vector<map_location>::const_iterator itor = overlayed_hexes_.begin();
 			itor != overlayed_hexes_.end(); ++itor) {
 
 		locations.insert(*itor);
@@ -266,12 +272,12 @@ manager::~manager()
 	disp = old;
 }
 
-int add(int x, int y, const std::string& image, const gamemap::location& loc,
+int add(int x, int y, const std::string& image, const map_location& loc,
 		ORIENTATION orientation, bool infinite)
 {
 	const int id = halo_id++;
-	animated<std::string>::anim_description image_vector;
-	std::vector<std::string> items = utils::split(image);
+	animated<image::locator>::anim_description image_vector;
+	std::vector<std::string> items = utils::paranthetical_split(image, ',');
 	std::vector<std::string>::const_iterator itor = items.begin();
 	for(; itor != items.end(); ++itor) {
 		const std::vector<std::string>& items = utils::split(*itor, ':');
@@ -285,7 +291,7 @@ int add(int x, int y, const std::string& image, const gamemap::location& loc,
 			str = *itor;
 			time = 100;
 		}
-		image_vector.push_back(animated<std::string>::frame_description(time,std::string(str)));
+		image_vector.push_back(animated<image::locator>::frame_description(time,image::locator(str)));
 
 	}
 	haloes.insert(std::pair<int,effect>(id,effect(x,y,image_vector,loc,orientation,infinite)));
@@ -306,7 +312,7 @@ void set_location(int handle, int x, int y)
 
 void remove(int handle)
 {
-	// Silently ignore invalid haloes. 
+	// Silently ignore invalid haloes.
 	// This happens when Wesnoth is being terminated as well.
 	if(handle == NO_HALO || haloes.find(handle) == haloes.end())  {
 		return;
@@ -315,12 +321,12 @@ void remove(int handle)
 	deleted_haloes.insert(handle);
 }
 
-void unrender(std::set<gamemap::location> invalidated_locations)
+void unrender(std::set<map_location> invalidated_locations)
 {
-	assert(invalidated_haloes.size() == 0);
 	if(preferences::show_haloes() == false || haloes.size() == 0) {
 		return;
 	}
+	assert(invalidated_haloes.size() == 0);
 
 	// Remove expired haloes
 	std::map<int, effect>::iterator itor = haloes.begin();
@@ -349,12 +355,12 @@ void unrender(std::set<gamemap::location> invalidated_locations)
 	// Find all halo's in a the invalidated area
 	size_t halo_count;
 
-	// Repeat until set of haloes in the invalidated area didn't change 
+	// Repeat until set of haloes in the invalidated area didn't change
 	// (including none found) or all exisiting haloes are found.
 	do {
 		halo_count = invalidated_haloes.size();
 		for(itor = haloes.begin(); itor != haloes.end(); ++itor) {
-			// Test all haloes not yet in the set 
+			// Test all haloes not yet in the set
 			// which match one of the locations
 			if(invalidated_haloes.find(itor->first) == invalidated_haloes.end() &&
 					itor->second.on_location(invalidated_locations)) {
@@ -399,11 +405,11 @@ void render()
 		return;
 	}
 
-	// Keep track of not rendered new images they have to be kept scheduled 
+	// Keep track of not rendered new images they have to be kept scheduled
 	// for rendering otherwise the invalidation area is never properly set
 	std::set<int> unrendered_new_haloes;
 
-	// Render the haloes: 
+	// Render the haloes:
 	// iterate through all the haloes and draw if in either set
 	for(std::map<int, effect>::iterator itor = haloes.begin();
 			itor != haloes.end(); ++itor) {
