@@ -1,6 +1,6 @@
-/* $Id: hotkeys.cpp 34027 2009-03-22 12:41:54Z ilor $ */
+/* $Id: hotkeys.cpp 41678 2010-03-22 16:04:08Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,7 @@
 
 #include "construct_dialog.hpp"
 #include "display.hpp"
+#include "foreach.hpp"
 #include "hotkeys.hpp"
 #include "game_end_exceptions.hpp"
 #include "gettext.hpp"
@@ -27,11 +28,11 @@
 #include "wesconfig.h"
 #include "wml_separators.hpp"
 
-
-#define ERR_G LOG_STREAM(err, general)
-#define LOG_G LOG_STREAM(info, general)
-#define DBG_G LOG_STREAM(debug, general)
-#define ERR_CONFIG LOG_STREAM(err, config)
+static lg::log_domain log_config("config");
+#define ERR_G LOG_STREAM(err, lg::general)
+#define LOG_G LOG_STREAM(info, lg::general)
+#define DBG_G LOG_STREAM(debug, lg::general)
+#define ERR_CF LOG_STREAM(err, log_config)
 
 namespace {
 
@@ -44,6 +45,7 @@ const struct {
 } hotkey_list_[] = {
 	{ hotkey::HOTKEY_CYCLE_UNITS, "cycle", N_("Next Unit"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_CYCLE_BACK_UNITS, "cycleback", N_("Previous Unit"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_ADD_WAYPOINT, "addwaypoint", N_("Add waypoint"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_UNIT_HOLD_POSITION, "holdposition", N_("Hold Position"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_END_UNIT_TURN, "endunitturn", N_("End Unit Turn"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_LEADER, "leader", N_("Leader"), false, hotkey::SCOPE_GAME },
@@ -60,19 +62,20 @@ const struct {
 	{ hotkey::HOTKEY_RENAME_UNIT, "renameunit", N_("Rename Unit"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_SAVE_GAME, "save", N_("Save Game"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_SAVE_REPLAY, "savereplay", N_("Save Replay"), false, hotkey::SCOPE_GAME },
-	{ hotkey::HOTKEY_SAVE_MAP, "savemap", N_("Save The Map"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_SAVE_MAP, "savemap", N_("Save Map"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_LOAD_GAME, "load", N_("Load Game"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_RECRUIT, "recruit", N_("Recruit"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_REPEAT_RECRUIT, "repeatrecruit", N_("Repeat Recruit"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_RECALL, "recall", N_("Recall"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_ENDTURN, "endturn", N_("End Turn"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_TOGGLE_ELLIPSES, "toggleellipses", N_("Toggle Ellipses"), false, hotkey::SCOPE_GENERAL },
 	{ hotkey::HOTKEY_TOGGLE_GRID, "togglegrid", N_("Toggle Grid"), false, hotkey::SCOPE_GENERAL },
 	{ hotkey::HOTKEY_MOUSE_SCROLL, "mousescroll", N_("Mouse Scrolling"), false, hotkey::SCOPE_GENERAL },
 	{ hotkey::HOTKEY_STATUS_TABLE, "statustable", N_("Status Table"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_MUTE, "mute", N_("Mute"), false, hotkey::SCOPE_GENERAL },
 	{ hotkey::HOTKEY_SPEAK, "speak", N_("Speak"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_CREATE_UNIT, "createunit", N_("Create Unit (Debug!)"), false, hotkey::SCOPE_GAME },
-	{ hotkey::HOTKEY_CHANGE_UNIT_SIDE, "changeside", N_("Change Unit Side (Debug!)"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_CHANGE_SIDE, "changeside", N_("Change Side (Debug!)"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_PREFERENCES, "preferences", N_("Preferences"), false, hotkey::SCOPE_GENERAL },
 	{ hotkey::HOTKEY_OBJECTIVES, "objectives", N_("Scenario Objectives"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_UNIT_LIST, "unitlist", N_("Unit List"), false, hotkey::SCOPE_GAME },
@@ -98,7 +101,7 @@ const struct {
 	  N_("Team 1"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_REPLAY_SKIP_ANIMATION, "replayskipanimation", N_("Skip animation"), false, hotkey::SCOPE_GAME },
 
-#ifndef DISABLE_EDITOR2
+#ifndef DISABLE_EDITOR
 	{ hotkey::HOTKEY_EDITOR_QUIT_TO_DESKTOP, "editor-quit-to-desktop", N_("Quit to Desktop"), false, hotkey::SCOPE_EDITOR },
 	{ hotkey::HOTKEY_EDITOR_CLOSE_MAP, "editor-close-map", N_("Close Map"), false, hotkey::SCOPE_EDITOR },
 	{ hotkey::HOTKEY_EDITOR_SWITCH_MAP, "editor-switch-map", N_("Switch Map"), false, hotkey::SCOPE_EDITOR },
@@ -200,8 +203,6 @@ hotkey::hotkey_item null_hotkey_;
 
 std::string hotkey_tag_name = "hotkey";
 
-const std::string scope_strings_[] = {"general", "game", "editor"};
-const std::string scope_labels_[] = {"Common", "Game", "Editor"};
 std::vector<bool> scope_active_(hotkey::SCOPE_COUNT, false);
 }
 
@@ -225,16 +226,6 @@ bool is_scope_active(scope s)
 	return scope_active_[s];
 }
 
-const std::string& get_scope_string(scope s)
-{
-	return scope_strings_[s];
-}
-
-const std::string& get_scope_label(scope s)
-{
-	return scope_labels_[s];
-}
-
 static void key_event_execute(display& disp, const SDL_KeyboardEvent& event, command_executor* executor);
 
 const std::string CLEARED_TEXT = "__none__";
@@ -273,43 +264,46 @@ void hotkey_item::load_from_config(const config& cfg)
 	ctrl_ = utils::string_bool(cfg["ctrl"]);
 	shift_ = utils::string_bool(cfg["shift"]);
 
-	if (!key.empty()) {
-		// They may really want a specific key on the keyboard: we assume
-		// that any single character keyname is a character.
-		if (key == CLEARED_TEXT)
-		{
-			type_ = hotkey_item::CLEARED;
-		}
-		else if (key.size() > 1) {
-			type_ = BY_KEYCODE;
+	if (key.empty()) return;
 
-			keycode_ = sdl_keysym_from_name(key);
-			if (keycode_ == SDLK_UNKNOWN) {
-				if (tolower(key[0]) != 'f') {
-					ERR_CONFIG << "hotkey key '" << key << "' invalid\n";
-				} else {
-					int num = lexical_cast_default<int>
-						(std::string(key.begin()+1,	key.end()), 1);
-					keycode_ = num + SDLK_F1 - 1;
-				}
+	if (key == CLEARED_TEXT)
+	{
+		type_ = hotkey_item::CLEARED;
+		return;
+	}
+
+	wide_string wkey = utils::string_to_wstring(key);
+
+	// They may really want a specific key on the keyboard: we assume
+	// that any single character keyname is a character.
+	if (wkey.size() > 1) {
+		type_ = BY_KEYCODE;
+
+		keycode_ = sdl_keysym_from_name(key);
+		if (keycode_ == SDLK_UNKNOWN) {
+			if (tolower(key[0]) != 'f') {
+				ERR_CF << "hotkey key '" << key << "' invalid\n";
+			} else {
+				int num = lexical_cast_default<int>(key.c_str() + 1);
+				keycode_ = num + SDLK_F1 - 1;
 			}
-		} else if (key == " " || shift_
-#ifdef __APPLE__
-			   || alt_
-#endif
-			   ) {
-			// Space must be treated as a key because shift-space
-			// isn't a different character from space, and control key
-			// makes it go weird.  shift=yes should never be specified
-			// on single characters (eg. key=m, shift=yes would be
-			// key=M), but we don't want to break old preferences
-			// files.
-			type_ = BY_KEYCODE;
-			keycode_ = key[0];
-		} else {
-			type_ = BY_CHARACTER;
-			character_ = key[0];
 		}
+	} else if (key == " " || shift_
+#ifdef __APPLE__
+		|| alt_
+#endif
+		) {
+		// Space must be treated as a key because shift-space
+		// isn't a different character from space, and control key
+		// makes it go weird.  shift=yes should never be specified
+		// on single characters (eg. key=m, shift=yes would be
+		// key=M), but we don't want to break old preferences
+		// files.
+		type_ = BY_KEYCODE;
+		keycode_ = wkey[0];
+	} else {
+		type_ = BY_CHARACTER;
+		character_ = wkey[0];
 	}
 }
 
@@ -453,11 +447,11 @@ void set_hotkey_tag_name(const std::string& name)
 
 void load_hotkeys(const config& cfg)
 {
-	const config::child_list& children = cfg.get_children(hotkey_tag_name);
-	for(config::child_list::const_iterator i = children.begin(); i != children.end(); ++i) {
-		hotkey_item& h = get_hotkey((**i)["command"]);
+	foreach (const config &hk, cfg.child_range(hotkey_tag_name))
+	{
+		hotkey_item& h = get_hotkey(hk["command"]);
 		if(h.get_id() != HOTKEY_NULL) {
-			h.load_from_config(**i);
+			h.load_from_config(hk);
 		}
 	}
 }
@@ -482,7 +476,7 @@ void save_hotkeys(config& cfg)
 			item["key"] = SDL_GetKeyName(SDLKey(i->get_keycode()));
 			item["shift"] = i->get_shift() ? "yes" : "no";
 		} else if (i->get_type() == hotkey_item::BY_CHARACTER) {
-			item["key"] = std::string(1, static_cast<char>(i->get_character()));
+			item["key"] = utils::wchar_to_string(i->get_character());
 		}
 		item["alt"] = i->get_alt() ? "yes" : "no";
 		item["ctrl"] = i->get_ctrl() ? "yes" : "no";
@@ -534,7 +528,7 @@ hotkey_item& get_hotkey(int character, int keycode, bool shift, bool ctrl,
 		   << "\n";
 
 	// Sometimes control modifies by -64, ie ^A == 1.
-	if (character < 64 && ctrl) {
+	if (0 < character && character < 64 && ctrl) {
 		if (shift)
 			character += 64;
 		else
@@ -648,7 +642,7 @@ void basic_handler::handle_event(const SDL_Event& event)
 void key_event(display& disp, const SDL_KeyboardEvent& event, command_executor* executor)
 {
 	if(event.keysym.sym == SDLK_ESCAPE && disp.in_game()) {
-		ERR_G << "escape pressed..showing quit\n";
+		LOG_G << "escape pressed..showing quit\n";
 		const int res = gui::dialog(disp,_("Quit"),_("Do you really want to quit?"),gui::YES_NO).show();
 		if(res == 0) {
 			throw end_level_exception(QUIT);
@@ -690,6 +684,9 @@ bool command_executor::execute_command(HOTKEY_COMMAND command, int /*index*/)
 		case HOTKEY_ENDTURN:
 			end_turn();
 			break;
+		case HOTKEY_ADD_WAYPOINT:
+			add_waypoint();
+			break;
 		case HOTKEY_UNIT_HOLD_POSITION:
 			unit_hold_position();
 			break;
@@ -723,6 +720,9 @@ bool command_executor::execute_command(HOTKEY_COMMAND command, int /*index*/)
 		case HOTKEY_LOAD_GAME:
 			load_game();
 			break;
+		case HOTKEY_TOGGLE_ELLIPSES:
+			toggle_ellipses();
+			break;
 		case HOTKEY_TOGGLE_GRID:
 			toggle_grid();
 			break;
@@ -750,8 +750,8 @@ bool command_executor::execute_command(HOTKEY_COMMAND command, int /*index*/)
 		case HOTKEY_CREATE_UNIT:
 			create_unit();
 			break;
-		case HOTKEY_CHANGE_UNIT_SIDE:
-			change_unit_side();
+		case HOTKEY_CHANGE_SIDE:
+			change_side();
 			break;
 		case HOTKEY_PREFERENCES:
 			preferences();
@@ -810,9 +810,7 @@ bool command_executor::execute_command(HOTKEY_COMMAND command, int /*index*/)
 		case HOTKEY_CUSTOM_CMD:
 			custom_command();
 			break;
-//%%
 		case HOTKEY_AI_FORMULA:
-			std::cerr <<" run ai formula\n";
 			ai_formula();
 			break;
 		case HOTKEY_CLEAR_MSG:
@@ -960,7 +958,7 @@ void command_executor::show_menu(const std::vector<std::string>& items_arg, int
 			return;
 		}
 
-		std::vector<std::string> menu = get_menu_images(items);
+		std::vector<std::string> menu = get_menu_images(gui, items);
 
 		int res = 0;
 		{
@@ -985,7 +983,7 @@ std::string command_executor::get_menu_image(hotkey::HOTKEY_COMMAND command, int
 	}
 }
 
-std::vector<std::string> command_executor::get_menu_images(const std::vector<std::string>& items){
+std::vector<std::string> command_executor::get_menu_images(display &disp, const std::vector<std::string>& items){
 	std::vector<std::string> result;
 	bool has_image = false;
 
@@ -1004,7 +1002,13 @@ std::vector<std::string> command_executor::get_menu_images(const std::vector<std
 		if (hk.get_id() == hotkey::HOTKEY_NULL) {
 			str << item.substr(0, item.find_last_not_of(' ') + 1) << COLUMN_SEPARATOR;
 		} else {
-			str << hk.get_description() << COLUMN_SEPARATOR << hk.get_name();
+			std::string desc = hk.get_description();
+			if (hk.get_id() == HOTKEY_ENDTURN) {
+				const theme::menu *b = disp.get_theme().get_menu_item("button-endturn");
+				assert(b);
+				desc = b->title();
+			}
+			str << desc << COLUMN_SEPARATOR << hk.get_name();
 		}
 
 		result.push_back(str.str());
