@@ -1,6 +1,6 @@
-/* $Id: hotkeys.hpp 34027 2009-03-22 12:41:54Z ilor $ */
+/* $Id: hotkeys.hpp 41353 2010-02-22 20:08:38Z boucman $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,6 @@
 
 #include "events.hpp"
 #include "tstring.hpp"
-#include "SDL.h"
 
 #include <string>
 #include <vector>
@@ -41,7 +40,8 @@ enum scope {
 
 
 enum HOTKEY_COMMAND {
-	HOTKEY_CYCLE_UNITS,HOTKEY_CYCLE_BACK_UNITS, HOTKEY_UNIT_HOLD_POSITION,
+	HOTKEY_CYCLE_UNITS,HOTKEY_CYCLE_BACK_UNITS,
+	HOTKEY_ADD_WAYPOINT, HOTKEY_UNIT_HOLD_POSITION,
 	HOTKEY_END_UNIT_TURN, HOTKEY_LEADER,
 	HOTKEY_UNDO, HOTKEY_REDO,
 	HOTKEY_ZOOM_IN, HOTKEY_ZOOM_OUT, HOTKEY_ZOOM_DEFAULT,
@@ -49,8 +49,8 @@ enum HOTKEY_COMMAND {
 	HOTKEY_UNIT_DESCRIPTION, HOTKEY_RENAME_UNIT,
 	HOTKEY_SAVE_GAME, HOTKEY_SAVE_REPLAY, HOTKEY_SAVE_MAP, HOTKEY_LOAD_GAME,
 	HOTKEY_RECRUIT, HOTKEY_REPEAT_RECRUIT, HOTKEY_RECALL, HOTKEY_ENDTURN,
-	HOTKEY_TOGGLE_GRID, HOTKEY_STATUS_TABLE, HOTKEY_MUTE, HOTKEY_MOUSE_SCROLL,
-	HOTKEY_SPEAK, HOTKEY_CREATE_UNIT, HOTKEY_CHANGE_UNIT_SIDE, HOTKEY_PREFERENCES,
+	HOTKEY_TOGGLE_ELLIPSES, HOTKEY_TOGGLE_GRID, HOTKEY_STATUS_TABLE, HOTKEY_MUTE, HOTKEY_MOUSE_SCROLL,
+	HOTKEY_SPEAK, HOTKEY_CREATE_UNIT, HOTKEY_CHANGE_SIDE, HOTKEY_PREFERENCES,
 	HOTKEY_OBJECTIVES, HOTKEY_UNIT_LIST, HOTKEY_STATISTICS, HOTKEY_STOP_NETWORK, HOTKEY_START_NETWORK, HOTKEY_QUIT_GAME,
 	HOTKEY_LABEL_TEAM_TERRAIN, HOTKEY_LABEL_TERRAIN, HOTKEY_CLEAR_LABELS,HOTKEY_SHOW_ENEMY_MOVES, HOTKEY_BEST_ENEMY_MOVES,
 	HOTKEY_DELAY_SHROUD, HOTKEY_UPDATE_SHROUD, HOTKEY_CONTINUE_MOVE,
@@ -61,7 +61,7 @@ enum HOTKEY_COMMAND {
 	HOTKEY_REPLAY_SHOW_EACH, HOTKEY_REPLAY_SHOW_TEAM1,
 	HOTKEY_REPLAY_SKIP_ANIMATION,
 
-#ifndef DISABLE_EDITOR2
+#ifndef DISABLE_EDITOR
 	HOTKEY_EDITOR_QUIT_TO_DESKTOP,
 	HOTKEY_EDITOR_CLOSE_MAP, HOTKEY_EDITOR_SWITCH_MAP,
 	HOTKEY_EDITOR_SETTINGS,
@@ -105,8 +105,6 @@ enum HOTKEY_COMMAND {
 void deactivate_all_scopes();
 void set_scope_active(scope s, bool set = true);
 bool is_scope_active(scope s);
-const std::string& get_scope_string(scope s);
-const std::string& get_scope_label(scope s);
 
 class hotkey_item {
 public:
@@ -243,6 +241,7 @@ public:
 	virtual void cycle_back_units() {}
 	virtual void end_turn() {}
 	virtual void goto_leader() {}
+	virtual void add_waypoint() {}
 	virtual void unit_hold_position() {}
 	virtual void end_unit_turn() {}
 	virtual void undo() {}
@@ -253,6 +252,7 @@ public:
 	virtual void save_replay() {}
 	virtual void save_map() {}
 	virtual void load_game() {}
+	virtual void toggle_ellipses() {}
 	virtual void toggle_grid() {}
 	virtual void status_table() {}
 	virtual void recall() {}
@@ -262,7 +262,7 @@ public:
 	virtual void whisper() {}
 	virtual void shout() {}
 	virtual void create_unit() {}
-	virtual void change_unit_side() {}
+	virtual void change_side() {}
 	virtual void preferences() {}
 	virtual void objectives() {}
 	virtual void unit_list() {}
@@ -304,7 +304,7 @@ public:
 	//Returns the appropriate menu image. Checkable items will get a checked/unchecked image.
 	std::string get_menu_image(hotkey::HOTKEY_COMMAND command, int index=-1) const;
 	//Returns a vector of images for a given menu
-	std::vector<std::string> get_menu_images(const std::vector<std::string>& items_arg);
+	std::vector<std::string> get_menu_images(display &, const std::vector<std::string>& items_arg);
 
 	void show_menu(const std::vector<std::string>& items_arg, int xloc, int yloc, bool context_menu, display& gui);
 
