@@ -1,6 +1,6 @@
-/* $Id: image.hpp 35477 2009-05-08 00:06:52Z alink $ */
+/* $Id: image.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -17,15 +17,16 @@
 
 #include "map_location.hpp"
 #include "sdl_utils.hpp"
+#include "terrain_translation.hpp"
 
 #include "SDL.h"
 #include "cassert"
+
+#include <map>
 #include <string>
 #include <vector>
 #include <list>
 
-#include <boost/tuple/tuple.hpp>
-
 ///this module manages the cache of images. With an image name, you can get
 ///the surface corresponding to that image.
 //
@@ -97,22 +98,19 @@ namespace image {
 	//a generic image locator. Abstracts the location of an image.
 	class locator
 	{
+	public:
+		enum type { NONE, FILE, SUB_FILE };
 	private:
 		// Called by each constructor after actual construction to
 		// initialize the index_ field
 		void init_index();
 		void parse_arguments();
-	public:
-		enum type { NONE, FILE, SUB_FILE };
-
 		struct value {
 			value();
 			value(const value &a);
 			value(const char *filename);
-			value(const char *filename, const std::string& modifications);
 			value(const std::string& filename);
 			value(const std::string& filename, const std::string& modifications);
-			value(const std::string& filename, const map_location& loc, const std::string& modifications);
 			value(const std::string& filename, const map_location& loc, int center_x, int center_y, const std::string& modifications);
 
 			bool operator==(const value& a) const;
@@ -126,6 +124,18 @@ namespace image {
 			int center_y_;
 		};
 
+		friend size_t hash_value(const value&);
+
+	public:
+
+		/**
+		 * @todo replace this with std::unordered_map<value, int> or boost::unordered_map<value, int>
+		 *       boost::unordered_map can almost just be dropped in as boost::hash<T>(T val) will return hash_value(val), but it
+		 *       requires boost 1.35 (preferably 1.36 or later)
+		 *
+		 **/
+		typedef std::map<size_t, std::map<value, int> > locator_finder_t;
+
 		// Constructing locators is somewhat slow, accessing image
 		// through locators is fast. The idea is that calling functions
 		// should store locators, and not strings to construct locators
@@ -133,10 +143,8 @@ namespace image {
 	        locator();
 		locator(const locator &a, const std::string &mods ="");
 		locator(const char *filename);
-		locator(const char *filename, const std::string& modifications);
 		locator(const std::string& filename);
 		locator(const std::string& filename, const std::string& modifications);
-		locator(const std::string& filename, const map_location& loc, const std::string& modifications="");
 		locator(const std::string& filename, const map_location& loc, int center_x, int center_y, const std::string& modifications="");
 
 		locator& operator=(const locator &a);
@@ -169,26 +177,16 @@ namespace image {
 		// loads the image it is pointing to from the disk
 		surface load_from_disk() const;
 
-		bool in_cache(cache_type<surface>& cache) const
+		template <typename T>
+		bool in_cache(cache_type<T>& cache) const
 			{ return index_ == -1 ? false : cache.get_element(index_).loaded; }
-		surface locate_in_cache(cache_type<surface>& cache) const
-			{ return index_ == -1 ? surface() : cache.get_element(index_).item; }
-		void add_to_cache(cache_type<surface>& cache, const surface &image) const
-			{ if(index_ != -1 ) cache.get_element(index_) = cache_item<surface>(image); cache.on_load(index_); }
+		template <typename T>
+		T locate_in_cache(cache_type<T>& cache) const
+			{ return index_ == -1 ? T() : cache.get_element(index_).item; }
+		template <typename T>
+		void add_to_cache(cache_type<T>& cache, const T &data) const
+			{ if(index_ != -1 ) cache.get_element(index_) = cache_item<T>(data); cache.on_load(index_); }
 
-		bool in_cache(cache_type<bool>& cache) const
-			{ return index_ == -1 ? false : cache.get_element(index_).loaded; }
-		bool locate_in_cache(cache_type<bool>& cache) const
-			{ return index_ == -1 ? false : cache.get_element(index_).item; }
-		void add_to_cache(cache_type<bool>& cache, bool st) const
-			{ if(index_ != -1 ) cache.get_element(index_) = cache_item<bool>(st); cache.on_load(index_); }
-
-		bool in_cache(cache_type<locator>& cache) const
-			{ return index_ == -1 ? false : cache.get_element(index_).loaded; cache.on_load(index_); }
-		locator locate_in_cache(cache_type<locator>& cache) const
-			{ return index_ == -1 ? locator() : cache.get_element(index_).item; }
-		void add_to_cache(cache_type<locator>& cache, const locator &image) const
-			{ if(index_ != -1) cache.get_element(index_) = cache_item<locator>(image); }
 	protected:
 		static int last_index_;
 	private:
@@ -200,9 +198,10 @@ namespace image {
 		value val_;
 	};
 
+	size_t hash_value(const locator::value&);
+
 
 	typedef cache_type<surface> image_cache;
-	typedef cache_type<locator> locator_cache;
 	typedef cache_type<bool> bool_cache;
 	typedef std::map<t_translation::t_terrain, surface> mini_terrain_cache_map;
 	extern mini_terrain_cache_map mini_terrain_cache;
@@ -229,8 +228,6 @@ namespace image {
 	///for representing day/night. Invalidates all scaled images.
 	void set_colour_adjustment(int r, int g, int b);
 
-	boost::tuple<int, int, int> get_colour_adjustment();
-
 	class colour_adjustment_resetter
 	{
 		public:
