@@ -1,6 +1,6 @@
-/* $Id: image_function.hpp 34314 2009-03-30 23:15:23Z shadowmaster $ */
+/* $Id: image_function.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2009 by Ignacio R. Morelle <shadowm2006@gmail.com>
+   Copyright (C) 2009 - 2010 by Ignacio R. Morelle <shadowm2006@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -36,10 +36,10 @@ public:
 };
 
 /**
- * Recolor (RC) function.
- * It is used both for palette switches (e.g. "~RC(pal1=pal2)")
- * color-range-based recoloring (also known as TC, e.g. "~RC(magenta>teal)")
- * and team-based recoloring (e.g. "~TC(1,magenta)").
+ * Recolor (RC/TC/PAL) function.
+ * It is used not only for color-range-based recoloring ("~RC(magenta>teal)")
+ * but also for team-color-based color range selection and recoloring
+ * ("~TC(3,magenta)") and palette switches ("~PAL(000000,005000 > FFFFFF,FF00FF)").
  */
 class rc_function : public function_base
 {
@@ -188,6 +188,14 @@ private:
 	int depth_;
 };
 
+/**
+ * Overlay with ToD brightening (BRIGHTEN).
+ */
+struct brighten_function : function_base
+{
+	virtual surface operator()(const surface &src) const;
+};
+
 } /* end namespace image */
 
 #endif /* !defined(IMAGE_FUNCTION_HPP_INCLUDED) */
