@@ -1,6 +1,6 @@
-/* $Id: key.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: key.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,20 +15,7 @@
 #include "global.hpp"
 #include "key.hpp"
 
-CKey::CKey() : 
-	key_list(0),
-	is_enabled(true)
+CKey::CKey() :
+	key_list(SDL_GetKeyState(NULL))
 {
-	static int num_keys = 300;
-	key_list = SDL_GetKeyState( &num_keys );
-}
-
-int CKey::operator[]( int code ) const
-{
-	return (code == SDLK_ESCAPE || is_enabled) && int(key_list[code]);
-}
-
-void CKey::SetEnabled( bool enable )
-{
-	is_enabled = enable;
 }
