@@ -1,6 +1,6 @@
-/* $Id: language.cpp 36421 2009-06-27 08:40:00Z silene $ */
+/* $Id: language.cpp 42004 2010-04-05 10:01:48Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -38,43 +38,10 @@ extern "C" int _putenv(const char*);
 #include <cerrno>
 #endif
 
-#define DBG_FS LOG_STREAM(debug, filesystem)
-#define DBG_GENERAL LOG_STREAM(debug, general)
-#define LOG_GENERAL LOG_STREAM(debug, general)
-#define WRN_GENERAL LOG_STREAM(debug, general)
-
-
-/** Tests one locale to be available. */
-static bool has_locale(const char* s) {
-	try {
-		// The way to find out whether a locale is available is to set it and
-		// hope not runtime error gets thrown.
-		std::locale dummy(s);
-		return true;
-	} catch (std::runtime_error&) {
-		return false;
-	}
-}
-
-/** Test the locale for a language and it's utf-8 variations. */
-static bool has_language(const std::string& language)
-{
-	if(has_locale(language.c_str())) {
-		return true;
-	}
-
-	std::string utf = language + ".utf-8";
-	if(has_locale(utf.c_str())) {
-		return true;
-	}
-
-	utf = language + ".UTF-8";
-	if(has_locale(utf.c_str())) {
-		return true;
-	}
-
-	return false;
-}
+#define DBG_G LOG_STREAM(debug, lg::general)
+#define LOG_G LOG_STREAM(info, lg::general)
+#define WRN_G LOG_STREAM(warn, lg::general)
+#define ERR_G LOG_STREAM(err, lg::general)
 
 namespace {
 	language_def current_language;
@@ -83,11 +50,6 @@ namespace {
 
 static language_list known_languages;
 
-std::string languagedef_name (const language_def& def)
-{
-	return def.language;
-}
-
 bool current_language_rtl()
 {
 	return get_language().rtl;
@@ -98,33 +60,6 @@ bool language_def::operator== (const language_def& a) const
 	return ((language == a.language) /* && (localename == a.localename) */ );
 }
 
-bool language_def::available() const
-{
-#if defined(_WIN32) || defined(__APPLE__)
-	// Under windows and OSX all locales are available and testing for it seems
-	// to fail so just return true.
-	return true;
-#endif
-
-	if (game_config::use_dummylocales)
-	{
-		// Dummy has every language available.
-		return true;
-	} else {
-		if(has_language(localename)) {
-			return true;
-		} else {
-			foreach(const std::string& lang, alternates) {
-				if(has_language(lang)) {
-					return true;
-				}
-			}
-		}
-
-		return false;
-	}
-}
-
 symbol_table string_table;
 
 const t_string& symbol_table::operator[](const std::string& key) const
@@ -162,11 +97,11 @@ bool load_language_list()
 	known_languages.push_back(
 		language_def("", t_string(N_("System default language"), "wesnoth"), "ltr", "", "A"));
 
-	config::const_child_itors langs = cfg.child_range("locale");
-	for(;langs.first != langs.second; ++langs.first) {
+	foreach (const config &lang, cfg.child_range("locale"))
+	{
 		known_languages.push_back(
-			language_def((**langs.first)["locale"], (**langs.first)["name"], (**langs.first)["dir"],
-				(**langs.first)["alternates"], (**langs.first)["sort_name"]));
+			language_def(lang["locale"], lang["name"], lang["dir"],
+			             lang["alternates"], lang["sort_name"]));
 	}
 
 	return true;
@@ -180,11 +115,10 @@ language_list get_languages()
 	return known_languages;
 }
 
-static void wesnoth_setlocale(int category, std::string slocale,
+static void wesnoth_setlocale(int category, std::string const &slocale,
 	std::vector<std::string> const *alternates)
 {
-	const char *locale = slocale.c_str();
-	std::string extra;
+	std::string locale = slocale;
 	// FIXME: ideally we should check LANGUAGE and on first invocation
 	// use that value, so someone with es would get the game in Spanish
 	// instead of en_US the first time round
@@ -197,98 +131,69 @@ static void wesnoth_setlocale(int category, std::string slocale,
 #endif
 
 #if defined(__BEOS__) || defined(__APPLE__)
-	if (category == LC_MESSAGES && setenv("LANG", locale, 1) == -1)
-		std::cerr << "setenv LANG failed: " << strerror(errno);
+	if (category == LC_MESSAGES && setenv("LANG", locale.c_str(), 1) == -1)
+		ERR_G << "setenv LANG failed: " << strerror(errno);
 #endif
 
 #ifdef _WIN32
-	std::string win_locale = locale;
-	win_locale = win_locale.substr(0,2);
+	std::string win_locale(locale, 0, 2);
 	#include "language_win32.ii"
 	if(category == LC_MESSAGES) {
-		std::string env = "LANG=" + slocale;
+		std::string env = "LANG=" + locale;
 		_putenv(env.c_str());
 		SetEnvironmentVariable("LANG", win_locale.c_str());
 		return;
 	}
-	locale = win_locale.c_str();
-#endif
-
-#ifdef USE_DUMMYLOCALES
-	if (game_config::use_dummylocales)
-	{
-		static enum { UNINIT, NONE, PRESENT } status = UNINIT;
-		static std::string locpath;
-		if (status == UNINIT) {
-			if (char const *p = getenv("LOCPATH")) {
-				locpath = p;
-				status = PRESENT;
-			} else status = NONE;
-		}
-		if (slocale.empty())
-			if (status == NONE)
-				unsetenv("LOCPATH");
-			else
-				setenv("LOCPATH", locpath.c_str(), 1);
-		else {
-			setenv("LOCPATH", (game_config::path + "/locales").c_str(), 1);
-			DBG_GENERAL << "LOCPATH set to '" << (game_config::path + "/locales") << "'\n";
-		}
-		std::string xlocale;
-		if (!slocale.empty()) {
-			// dummy suffix to prevent locale aliasing from kicking in
-			extra = "@wesnoth";
-			xlocale = slocale + "@wesnoth";
-			locale = xlocale.c_str();
-		}
-	}
+	locale = win_locale;
 #endif
 
 	char *res = NULL;
-	std::string orig_locale;
-	orig_locale.assign(locale);
-
-	typedef boost::scoped_array<char> char_array;
-	size_t length = orig_locale.length()+1;
-	char_array try_loc(new char[length]);
-	orig_locale.copy(try_loc.get(), orig_locale.length());
-	try_loc[orig_locale.length()] = 0;
 	std::vector<std::string>::const_iterator i;
 	if (alternates) i = alternates->begin();
-	while (true) {
-		res = std::setlocale(category, try_loc.get());
-		if (res) break;
 
-		std::string utf8 = orig_locale + std::string(".utf-8");
-		res = std::setlocale(category, utf8.c_str());
-		if (res) break;
-
-		utf8 = orig_locale + std::string(".UTF-8");
-		res = std::setlocale(category, utf8.c_str());
-		if (res) break;
-
-		if (!alternates) break;
-		if (i == alternates->end()) break;
-		orig_locale = *i + extra;
-		if (length < orig_locale.length()+1)
-		{
-			length = orig_locale.length()+1;
-			try_loc.reset(new char[length]);
+	for (;;)
+	{
+		std::string lang = locale, extra;
+		std::string::size_type pos = locale.find('@');
+		if (pos != std::string::npos) {
+			lang.erase(pos);
+			extra = locale.substr(pos);
 		}
-		orig_locale.copy(try_loc.get(), orig_locale.length());
-		try_loc[orig_locale.length()] = 0;
-		i++;
+
+		/*
+		 * The "" is the last item to work-around a problem in glibc picking
+		 * the non utf8 locale instead an utf8 version if available.
+		 */
+		char const *encoding[] = { ".utf-8", ".UTF-8", "" };
+		for (int j = 0; j != 3; ++j)
+		{
+			locale = lang + encoding[j] + extra;
+			res = std::setlocale(category, locale.c_str());
+			if (res) {
+				LOG_G << "Set locale to '" << locale << "' result: '" << res << "'.\n";
+				goto done;
+			}
+		}
+
+		if (!alternates || i == alternates->end()) break;
+		locale = *i;
+		++i;
 	}
 
-	if (res == NULL)
-		WRN_GENERAL << "WARNING: setlocale() failed for '"
-			  << locale << "'.\n";
-	else
-		LOG_GENERAL << "set locale to '" << (try_loc.get()) << "' result: '" << res <<"'\n";
-
-	DBG_GENERAL << "Numeric locale: " << std::setlocale(LC_NUMERIC, NULL) << '\n';
-	DBG_GENERAL << "Full locale: " << std::setlocale(LC_ALL, NULL) << '\n';
+	WRN_G << "setlocale() failed for '" << slocale << "'.\n";
+#ifndef _WIN32
+#ifndef __AMIGAOS4__
+		if(category == LC_MESSAGES) {
+			WRN_G << "Setting LANGUAGE to '" << slocale << "'.\n";
+			setenv("LANGUAGE", slocale.c_str(), 1);
+			std::setlocale(LC_MESSAGES, "");
+		}
+#endif
+#endif
 
+	done:
+	DBG_G << "Numeric locale: " << std::setlocale(LC_NUMERIC, NULL) << '\n';
+	DBG_G << "Full locale: " << std::setlocale(LC_ALL, NULL) << '\n';
 }
 
 bool set_language(const language_def& locale)
@@ -302,9 +207,9 @@ bool set_language(const language_def& locale)
 	config cfg;
 
 	current_language = locale;
-	wesnoth_setlocale(LC_MESSAGES, locale.localename, &locale.alternates);
 	wesnoth_setlocale(LC_COLLATE, locale.localename, &locale.alternates);
 	wesnoth_setlocale(LC_TIME, locale.localename, &locale.alternates);
+	wesnoth_setlocale(LC_MESSAGES, locale.localename, &locale.alternates);
 
 	// fill string_table (should be moved somwhere else some day)
 	try {
@@ -315,24 +220,17 @@ bool set_language(const language_def& locale)
 		throw e;
 	}
 
-	config* langp = cfg.child("language");
-	if (langp == NULL) {
+	config &langp = cfg.child("language");
+	if (!langp) {
 		std::cerr << "No [language] block found in english.cfg\n";
 		return false;
 	}
 
-	for(string_map::const_iterator j = langp->values.begin(); j != langp->values.end(); ++j) {
-		strings_[j->first] = j->second;
+	foreach (const config::attribute &j, langp.attribute_range()) {
+		strings_[j.first] = j.second;
 	}
 	// end of string_table fill
 
-	// Reset translations for the name of current languages
-	for (language_list::iterator itor = known_languages.begin();
-			itor != known_languages.end(); ++itor) {
-
-		itor->language.reset_translation();
-	}
-
 	return true;
 }
 
@@ -352,7 +250,7 @@ const language_def& get_locale()
 			if (prefs_locale == i->localename)
 				return *i;
 		}
-		LOG_STREAM(info, general) << "'" << prefs_locale << "' locale not found in known array; defaulting to system locale\n";
+		LOG_G << "'" << prefs_locale << "' locale not found in known array; defaulting to system locale\n";
 		return known_languages[0];
 	}
 
@@ -374,17 +272,16 @@ const language_def& get_locale()
 	}
 #endif
 
-	LOG_STREAM(info, general) << "locale could not be determined; defaulting to system locale\n";
+	LOG_G << "locale could not be determined; defaulting to system locale\n";
 	return known_languages[0];
 }
 
 void init_textdomains(const config& cfg)
 {
-	config::const_child_itors t = cfg.child_range("textdomain");
-
-	for(;t.first != t.second; ++t.first) {
-		const std::string name = (**t.first)["name"];
-		const std::string path = (**t.first)["path"];
+	foreach (const config &t, cfg.child_range("textdomain"))
+	{
+		const std::string &name = t["name"];
+		const std::string &path = t["path"];
 
 		if(path.empty()) {
 			t_string::add_textdomain(name, get_intl_dir());
@@ -394,8 +291,7 @@ void init_textdomains(const config& cfg)
 			if (location.empty()) {
 				//if location is empty, this causes a crash on Windows, so we
 				//disallow adding empty domains
-				LOG_STREAM(err, general) << "no location found for '" << path
-					<< "', not adding textdomain\n";
+				ERR_G << "no location found for '" << path << "', skipping textdomain\n";
 			} else {
 				t_string::add_textdomain(name, location);
 			}
