@@ -1,6 +1,6 @@
-/* $Id: language.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: language.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -55,19 +55,10 @@ struct language_def
 	bool operator== (const language_def&) const;
 	bool operator< (const language_def& a) const
 		{ return sort_name < a.sort_name; }
-
-	/**
-	 * Is the locale available on the system?
-	 *
-	 * If the dummy locales are selected we always return true.
-	 */
-	bool available() const;
 };
 
 typedef std::vector<language_def> language_list;
 
-std::string languagedef_name (const language_def& def);
-
 struct symbol_table
 {
 	const t_string& operator[](const std::string& key) const;
