@@ -1,6 +1,6 @@
-/* $Id: leader_list.hpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: leader_list.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2007 - 2009
+   Copyright (C) 2007 - 2010
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -22,13 +22,14 @@ namespace gui { class combo; }
 
 #include "unit_types.hpp"
 #include <string>
+#include <vector>
 
 class leader_list_manager
 {
 public:
 	static const std::string random_enemy_picture;
 
-	leader_list_manager(const config::child_list& side_list,
+	leader_list_manager(const std::vector<const config *> &side_list,
 			gui::combo* leader_combo = NULL, gui::combo* gender_combo = NULL);
 
 	void set_leader_combo(gui::combo* combo);
@@ -47,7 +48,7 @@ private:
 	std::vector<std::string> leaders_;
 	std::vector<std::string> genders_;
 	std::vector<std::string> gender_ids_;
-	config::child_list side_list_;
+	std::vector<const config *> side_list_;
 	gui::combo* leader_combo_;
 	gui::combo* gender_combo_;
 	int colour_;
