@@ -1,6 +1,6 @@
-/* $Id: map_create.hpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: map_create.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@ class map_generator;
 
 #include <string>
 
-std::string random_generate_map(const std::string& parms, const config* cfg);
-config random_generate_scenario(const std::string& parms, const config* cfg);
+std::string random_generate_map(const std::string& parms, const config &cfg);
+config random_generate_scenario(const std::string& parms, const config &cfg);
 
-map_generator* create_map_generator(const std::string& name, const config* cfg);
+map_generator* create_map_generator(const std::string& name, const config &cfg);
 
 #endif
