@@ -1,6 +1,6 @@
-/* $Id: map_label.cpp 32329 2009-01-27 16:50:09Z zookeeper $ */
+/* $Id: map_label.cpp 41483 2010-03-06 23:30:25Z fendrin $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,10 @@
 #include "global.hpp"
 
 #include "display.hpp"
+#include "foreach.hpp"
+#include "gamestatus.hpp"
 #include "map_label.hpp"
+#include "resources.hpp"
 #include "formula_string_utils.hpp"
 
 
@@ -32,27 +35,11 @@ static bool is_shrouded(const display& disp, const map_location& loc)
 	return disp.shrouded(loc) || disp.shrouded(map_location(loc.x,loc.y+1));
 }
 
-map_labels::map_labels(const display& disp,
-					   const team* team) :
-		disp_(disp),
-		team_(team),
-		labels_(),
-		label_cache_()
+map_labels::map_labels(const display &disp, const team *team) :
+	disp_(disp), team_(team), labels_()
 {
 }
 
-map_labels::map_labels(const display& disp,
-		       const config& cfg,
-		       const team* team,
-		       const variable_set *variables) :
-		disp_(disp),
-		team_(team),
-		labels_(),
-		label_cache_()
-{
-	read(cfg, variables);
-}
-
 map_labels::~map_labels()
 {
 	clear_all();
@@ -72,14 +59,14 @@ void map_labels::write(config& res) const
 	}
 }
 
-void map_labels::read(const config& cfg, const variable_set *variables)
+void map_labels::read(const config &cfg)
 {
 	clear_all();
 
-	const config::child_list& items = cfg.get_children("label");
-	for(config::child_list::const_iterator i = items.begin(); i != items.end(); ++i) {
-		const map_location loc(**i, variables);
-		terrain_label* label = new terrain_label(*this, **i, variables);
+	foreach (const config &i, cfg.child_range("label"))
+	{
+		const map_location loc(i, resources::state_of_game);
+		terrain_label *label = new terrain_label(*this, i);
 		add_label(loc, label);
 	}
 	recalculate_labels();
@@ -140,13 +127,14 @@ void map_labels::set_team(const team* team)
 
 const terrain_label* map_labels::set_label(const map_location& loc,
 					   const std::string& text,
-					   const std::string team_name,
+					   const std::string& team_name,
 					   const SDL_Color colour,
-					   const bool visible_in_fog)
+					   const bool visible_in_fog,
+					   const bool visible_in_shroud)
 {
 	terrain_label* res = 0;
-	const team_label_map::const_iterator current_label_map = labels_.find(team_name);
-	label_map::const_iterator current_label;
+	team_label_map::iterator current_label_map = labels_.find(team_name);
+	label_map::iterator current_label;
 
 	if ( current_label_map != labels_.end()
 			&& (current_label = current_label_map->second.find(loc)) != current_label_map->second.end() )
@@ -154,10 +142,10 @@ const terrain_label* map_labels::set_label(const map_location& loc,
 		// Found old checking if need to erase it
 		if(text.empty())
 		{
-			const_cast<terrain_label*>(current_label->second)->set_text("");
-			res = new terrain_label("",team_name,loc,*this,colour,visible_in_fog);
+			current_label->second->set_text("");
+			res = new terrain_label("",team_name,loc,*this,colour,visible_in_fog,visible_in_shroud);
 			delete current_label->second;
-			const_cast<label_map&>(current_label_map->second).erase(loc);
+			current_label_map->second.erase(loc);
 
 			team_label_map::iterator global_label_map = labels_.find("");
 			label_map::iterator itor;
@@ -168,16 +156,14 @@ const terrain_label* map_labels::set_label(const map_location& loc,
 			}
 			if (update)
 			{
-				const_cast<terrain_label*>(itor->second)->recalculate();
+				itor->second->recalculate();
 			}
 
 		}
 		else
 		{
-			const_cast<terrain_label*>(current_label->second)->update_info(text,
-																		   team_name,
-																		    colour);
-			res = const_cast<terrain_label*>(current_label->second);
+			current_label->second->update_info(text, team_name, colour);
+			res = current_label->second;
 		}
 	}
 	else if(!text.empty())
@@ -195,33 +181,24 @@ const terrain_label* map_labels::set_label(const map_location& loc,
 				loc,
 				*this,
 				colour,
-				visible_in_fog);
+				visible_in_fog,
+				visible_in_shroud);
 		add_label(loc,label);
 
 		res = label;
 
 		if (update)
 		{
-			const_cast<terrain_label*>(itor->second)->recalculate();
+			itor->second->recalculate();
 		}
 
 	}
 	return res;
 }
 
-void map_labels::add_label(const map_location& loc,
-	const terrain_label* new_label)
+void map_labels::add_label(const map_location &loc, terrain_label *new_label)
 {
-	team_label_map::const_iterator labs = labels_.find(new_label->team_name());
-	if(labs == labels_.end())
-	{
-		labels_.insert(std::pair<std::string,label_map>(new_label->team_name(), label_map()));
-		labs = labels_.find(new_label->team_name());
-
-		assert(labs != labels_.end());
-	}
-
-	const_cast<label_map&>(labs->second).insert(std::pair<map_location,const terrain_label*>(loc,new_label));
+	labels_[new_label->team_name()][loc] = new_label;
 }
 
 void map_labels::clear(const std::string& team_name)
@@ -239,20 +216,20 @@ void map_labels::clear(const std::string& team_name)
 	}
 }
 
-void map_labels::clear_map(const label_map& m)
+void map_labels::clear_map(label_map &m)
 {
-	for (label_map::const_iterator j = m.begin(); j != m.end(); ++j)
+	foreach (label_map::value_type &v, m)
 	{
-		delete j->second;
+		delete v.second;
 	}
-	const_cast<label_map&>(m).clear();
+	m.clear();
 }
 
 void map_labels::clear_all()
 {
-	for(team_label_map::const_iterator i = labels_.begin(); i != labels_.end(); ++i)
+	foreach (team_label_map::value_type &m, labels_)
 	{
-		clear_map(i->second);
+		clear_map(m.second);
 	}
 	labels_.clear();
 }
@@ -270,11 +247,11 @@ void map_labels::scroll(double xmove, double ymove)
 
 void map_labels::recalculate_labels()
 {
-	for(team_label_map::const_iterator i = labels_.begin(); i != labels_.end(); ++i)
+	foreach (team_label_map::value_type &m, labels_)
 	{
-		for (label_map::const_iterator j = i->second.begin(); j != i->second.end(); ++j)
+		foreach (label_map::value_type &l, m.second)
 		{
-			const_cast<terrain_label*>(j->second)->recalculate();
+			l.second->recalculate();
 		}
 	}
 }
@@ -288,11 +265,11 @@ bool map_labels::visible_global_label(const map_location& loc) const
 
 void map_labels::recalculate_shroud()
 {
-	for(team_label_map::const_iterator i = labels_.begin(); i != labels_.end(); ++i)
+	foreach (team_label_map::value_type &m, labels_)
 	{
-		for (label_map::const_iterator j = i->second.begin(); j != i->second.end(); ++j)
+		foreach (label_map::value_type &l, m.second)
 		{
-			const_cast<terrain_label*>(j->second)->calculate_shroud();
+			l.second->calculate_shroud();
 		}
 	}
 }
@@ -304,11 +281,13 @@ terrain_label::terrain_label(const std::string& text,
 							 const map_location& loc,
 							 const map_labels& parent,
 							 const SDL_Color colour,
-							 const bool visible_in_fog)  :
+							 const bool visible_in_fog,
+							 const bool visible_in_shroud)  :
 		handle_(0),
 		text_(text),
 		team_name_(team_name),
 		visible_in_fog_(visible_in_fog),
+		visible_in_shroud_(visible_in_shroud),
 		colour_(colour),
 		parent_(&parent),
 		loc_(loc)
@@ -317,31 +296,18 @@ terrain_label::terrain_label(const std::string& text,
 	draw();
 }
 
-terrain_label::terrain_label(const map_labels& parent)  :
-		handle_(0),
-		text_(),
-		team_name_(),
-		visible_in_fog_(true),
-		colour_(),
-		parent_(&parent),
-		loc_()
-{
-}
-
-
 /// Load label from config
-terrain_label::terrain_label(const map_labels& parent,
-			     const config& cfg,
-			     const variable_set *variables) :
+terrain_label::terrain_label(const map_labels &parent, const config &cfg) :
 		handle_(0),
 		text_(),
 		team_name_(),
 		visible_in_fog_(true),
+		visible_in_shroud_(false),
 		colour_(),
 		parent_(&parent),
 		loc_()
 {
-	read(cfg, variables);
+	read(cfg);
 	check_text_length();
 }
 
@@ -351,25 +317,21 @@ terrain_label::~terrain_label()
 	clear();
 }
 
-void terrain_label::read(const config& cfg, const variable_set *variables)
+void terrain_label::read(const config &cfg)
 {
-	loc_ = map_location(cfg, variables);
+	const variable_set &vs = *resources::state_of_game;
+	loc_ = map_location(cfg, &vs);
 	SDL_Color colour = font::LABEL_COLOUR;
 	std::string tmp_colour = cfg["colour"];
 
 	text_      = cfg["text"];
 	team_name_ = cfg["team_name"];
 	visible_in_fog_ = utils::string_bool(cfg["visible_in_fog"],true);
+	visible_in_shroud_ = utils::string_bool(cfg["visible_in_shroud"],false);
 
-	if (variables)
-	{
-		text_ = utils::interpolate_variables_into_string(
-				text_, *variables);
-		team_name_ = utils::interpolate_variables_into_string(
-					team_name_, *variables);
-		tmp_colour = utils::interpolate_variables_into_string(
-				  tmp_colour, *variables);
-	}
+	text_ = utils::interpolate_variables_into_string(text_, vs);
+	team_name_ = utils::interpolate_variables_into_string(team_name_, vs);
+	tmp_colour = utils::interpolate_variables_into_string(tmp_colour, vs);
 
 	if(!tmp_colour.empty()) {
 		std::vector<Uint32> temp_rgb;
@@ -392,6 +354,7 @@ void terrain_label::write(config& cfg) const
 	cfg["team_name"] = (this->team_name());
 	cfg["colour"] = cfg_colour();
 	cfg["visible_in_fog"] = visible_in_fog() ? "yes" : "no";
+	cfg["visible_in_shroud"] = visible_in_shroud() ? "yes" : "no";
 }
 
 const std::string& terrain_label::text() const
@@ -409,6 +372,11 @@ bool terrain_label::visible_in_fog() const
 	return visible_in_fog_;
 }
 
+bool terrain_label::visible_in_shroud() const
+{
+	return visible_in_shroud_;
+}
+
 const map_location& terrain_label::location() const
 {
 	return loc_;
@@ -470,9 +438,8 @@ void terrain_label::calculate_shroud() const
 
 	if (handle_)
 	{
-		font::show_floating_label(handle_,
-								  !is_shrouded(parent_->disp(),
-											   loc_));
+        bool shrouded = visible_in_shroud_ || !is_shrouded(parent_->disp(), loc_);
+        font::show_floating_label(handle_, shrouded);
 	}
 }
 
@@ -506,14 +473,13 @@ void terrain_label::draw()
 
 bool terrain_label::visible() const
 {
-	if (!visible_in_fog_)
-	{
-		if (parent_->disp().fogged(loc_))
-		{
-			return false;
-		}
+	if ((!visible_in_fog_ && parent_->disp().fogged(loc_))
+        || (!visible_in_shroud_ && parent_->disp().shrouded(loc_))) {
+            return false;
 	}
-	return  (parent_->team_name() == team_name_
+
+
+	return (parent_->team_name() == team_name_
 			|| (team_name_.empty() && parent_->visible_global_label(loc_)));
 }
 
