@@ -1,6 +1,6 @@
-/* $Id: mapgen.cpp 24154 2008-02-29 18:17:30Z mordante $ */
+/* $Id: mapgen.cpp 40859 2010-01-25 18:00:22Z esr $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,37 +12,29 @@
    See the COPYING file for more details.
 */
 
-//! @file mapgen.cpp
-//! Map-generator, with standalone testprogram.
+/**
+ * @file mapgen.cpp
+ * Map-generator, with standalone testprogram.
+ */
 
 #include "global.hpp"
 
+#include "foreach.hpp"
 #include "gettext.hpp"
 #include "language.hpp"
 #include "log.hpp"
+#include "map.hpp"
 #include "mapgen.hpp"
-#include "pathfind.hpp"
+#include "pathfind/pathfind.hpp"
+#include "pathutils.hpp"
 #include "race.hpp"
-#include "scoped_resource.hpp"
-#include "serialization/string_utils.hpp"
-#include "util.hpp"
 #include "wml_exception.hpp"
+#include "formula_string_utils.hpp"
 
-#include <algorithm>
-#include <cassert>
-#include <cctype>
-#include <cmath>
-#include <cstdlib>
-#include <iostream>
-#include <set>
-#include <sstream>
-#include <string>
-#include <ctime>
-#include <vector>
 
-#define ERR_CF LOG_STREAM(err, config)
-#define ERR_NG LOG_STREAM(err, engine)
-#define LOG_NG LOG_STREAM(info, engine)
+static lg::log_domain log_engine("engine");
+#define ERR_NG LOG_STREAM(err, log_engine)
+#define LOG_NG LOG_STREAM(info, log_engine)
 
 config map_generator::create_scenario(const std::vector<std::string>& args)
 {
@@ -54,21 +46,21 @@ config map_generator::create_scenario(const std::vector<std::string>& args)
 typedef std::vector<std::vector<int> > height_map;
 typedef t_translation::t_map terrain_map;
 
-typedef gamemap::location location;
+typedef map_location location;
 
-//! Generate a height-map.
-//! Basically we generate alot of hills,
-//! each hill being centered at a certain point,
-//! with a certain radius - being a half sphere.
-//! Hills are combined additively to form a bumpy surface.
-//! The size of each hill varies randomly from 1-hill_size.
-//! We generate 'iterations' hills in total.
-//! The range of heights is normalized to 0-1000.
-//! 'island_size' controls whether or not the map should tend toward an island shape,
-//! and if so, how large the island should be.
-//! Hills with centers that are more than 'island_size' away from
-//! the center of the map will be inverted (i.e. be valleys).
-//! 'island_size' as 0 indicates no island.
+/**
+ * Generate a height-map.
+ *
+ * Basically we generate alot of hills, each hill being centered at a certain
+ * point, with a certain radius - being a half sphere.  Hills are combined
+ * additively to form a bumpy surface.  The size of each hill varies randomly
+ * from 1-hill_size.  We generate 'iterations' hills in total.  The range of
+ * heights is normalized to 0-1000.  'island_size' controls whether or not the
+ * map should tend toward an island shape, and if so, how large the island
+ * should be.  Hills with centers that are more than 'island_size' away from
+ * the center of the map will be inverted (i.e. be valleys).  'island_size' as
+ * 0 indicates no island.
+ */
 static height_map generate_height_map(size_t width, size_t height,
                                size_t iterations, size_t hill_size,
 							   size_t island_size, size_t island_off_center)
@@ -186,12 +178,15 @@ static height_map generate_height_map(size_t width, size_t height,
 	return res;
 }
 
-//! Generate a lake.
-//! It will create water at (x,y), and then have 'lake_fall_off' % chance
-//! to make another water tile in each of the directions n,s,e,w.
-//! In each of the directions it does make another water tile,
-//! it will have 'lake_fall_off'/2 % chance to make another water tile
-//! in each of the directions. This will continue recursively.
+/**
+ * Generate a lake.
+ *
+ * It will create water at (x,y), and then have 'lake_fall_off' % chance to
+ * make another water tile in each of the directions n,s,e,w.  In each of the
+ * directions it does make another water tile, it will have 'lake_fall_off'/2 %
+ * chance to make another water tile in each of the directions. This will
+ * continue recursively.
+ */
 static bool generate_lake(terrain_map& terrain, int x, int y, int lake_fall_off, std::set<location>& locs_touched)
 {
 	if(x < 0 || y < 0 || size_t(x) >= terrain.size() || size_t(y) >= terrain.front().size()) {
@@ -220,21 +215,23 @@ static bool generate_lake(terrain_map& terrain, int x, int y, int lake_fall_off,
 	return true;
 }
 
-//! River generation.
-//! Rivers have a source, and then keep on flowing
-//! until they meet another body of water, which they flow into,
-//! or until they reach the edge of the map.
-//! Rivers will always flow downhill, except that they can flow
-//! a maximum of 'river_uphill' uphill.
-//! This is to represent the water eroding the higher ground lower.
-//!
-//! Every possible path for a river will be attempted, in random order,
-//! and the first river path that can be found that makes the river flow
-//! into another body of water or off the map will be used.
-//!
-//! If no path can be found, then the river's generation will be aborted,
-//! and false will be returned.
-//! true is returned if the river is generated successfully.
+/**
+ * River generation.
+ *
+ * Rivers have a source, and then keep on flowing until they meet another body
+ * of water, which they flow into, or until they reach the edge of the map.
+ * Rivers will always flow downhill, except that they can flow a maximum of
+ * 'river_uphill' uphill.  This is to represent the water eroding the higher
+ * ground lower.
+ *
+ * Every possible path for a river will be attempted, in random order, and the
+ * first river path that can be found that makes the river flow into another
+ * body of water or off the map will be used.
+ *
+ * If no path can be found, then the river's generation will be aborted, and
+ * false will be returned.  true is returned if the river is generated
+ * successfully.
+ */
 static bool generate_river_internal(const height_map& heights,
 	terrain_map& terrain, int x, int y, std::vector<location>& river,
 	std::set<location>& seen_locations, int river_uphill)
@@ -303,8 +300,10 @@ static std::vector<location> generate_river(const height_map& heights, terrain_m
 	return river;
 }
 
-//! Return a random tile at one of the borders of a map
-//! that is of the given dimensions.
+/**
+ * Returns a random tile at one of the borders of a map that is of the given
+ * dimensions.
+ */
 static location random_point_at_side(size_t width, size_t height)
 {
 	const int side = rand()%4;
@@ -319,7 +318,7 @@ static location random_point_at_side(size_t width, size_t height)
 	}
 }
 
-//! Function which, given the map will output it in a valid format.
+/** Function which, given the map will output it in a valid format. */
 static std::string output_map(const terrain_map& terrain,
 		std::map<int, t_translation::coordinate> starting_positions)
 {
@@ -356,17 +355,24 @@ static std::string output_map(const terrain_map& terrain,
 
 namespace {
 
-//! Calculate the cost of building a road over terrain.
-//! For use in the a_star_search algorithm.
-struct road_path_calculator : cost_calculator
+/**
+ * Calculates the cost of building a road over terrain.  For use in the
+ * a_star_search algorithm.
+ */
+struct road_path_calculator : pathfind::cost_calculator
 {
-	road_path_calculator(const terrain_map& terrain, const config& cfg)
-		        : calls(0), map_(terrain), cfg_(cfg),
+	road_path_calculator(const terrain_map& terrain, const config& cfg) :
+    	calls(0),
+		map_(terrain),
+		cfg_(cfg),
+		// Find out how windy roads should be.
+		windiness_(std::max<int>(1,atoi(cfg["road_windiness"].c_str()))),
+		seed_(rand()),
+		cache_()
+	{
+	}
 
-				  // Find out how windy roads should be.
-				  windiness_(maximum<int>(1,atoi(cfg["road_windiness"].c_str()))),
-				  seed_(rand()) {}
-	virtual double cost(const location& src, const location& loc, const double so_far, const bool isDst) const;
+	virtual double cost(const location& loc, const double so_far) const;
 
 	mutable int calls;
 private:
@@ -377,14 +383,14 @@ private:
 	mutable std::map<t_translation::t_terrain, double> cache_;
 };
 
-double road_path_calculator::cost(const location& /*src*/, const location& loc,
-	const double /*so_far*/, const bool /*isDst*/) const
+double road_path_calculator::cost(const location& loc,
+	const double /*so_far*/) const
 {
 	++calls;
 	if (loc.x < 0 || loc.y < 0 || loc.x >= static_cast<long>(map_.size()) ||
 			loc.y >= static_cast<long>(map_.front().size())) {
 
-		return (getNoPathValue());
+		return (pathfind::cost_calculator::getNoPathValue());
 	}
 
 	// We multiply the cost by a random amount,
@@ -408,7 +414,7 @@ double road_path_calculator::cost(const location& /*src*/, const location& loc,
 		int noise = random % (windiness_ * 137) / 137;
 		windiness += noise;
 	}
-	
+
 	const t_translation::t_terrain c = map_[loc.x][loc.y];
 	const std::map<t_translation::t_terrain, double>::const_iterator itor = cache_.find(c);
 	if(itor != cache_.end()) {
@@ -417,17 +423,15 @@ double road_path_calculator::cost(const location& /*src*/, const location& loc,
 
 	static std::string terrain;
 	terrain = t_translation::write_terrain_code(c);
-	const config* const child = cfg_.find_child("road_cost","terrain",terrain);
 	double res = getNoPathValue();
-	if(child != NULL) {
-		res = double(atof((*child)["cost"].c_str()));
+	if (const config &child = cfg_.find_child("road_cost", "terrain", terrain)) {
+		res = atof(child["cost"].c_str());
 	}
 
 	cache_.insert(std::pair<t_translation::t_terrain, double>(c,res));
 	return windiness*res;
 }
 
-//!
 struct is_valid_terrain
 {
 	is_valid_terrain(const t_translation::t_map& map,
@@ -456,15 +460,14 @@ bool is_valid_terrain::operator()(int x, int y) const
 
 }
 
-//!
 static int rank_castle_location(int x, int y, const is_valid_terrain& valid_terrain, int min_x, int max_x, int min_y, int max_y,
-						 size_t min_distance, const std::vector<gamemap::location>& other_castles, int highest_ranking)
+						 size_t min_distance, const std::vector<map_location>& other_castles, int highest_ranking)
 {
-	const gamemap::location loc(x,y);
+	const map_location loc(x,y);
 
 	size_t avg_distance = 0, lowest_distance = 1000;
 
-	for(std::vector<gamemap::location>::const_iterator c = other_castles.begin(); c != other_castles.end(); ++c) {
+	for(std::vector<map_location>::const_iterator c = other_castles.begin(); c != other_castles.end(); ++c) {
 		const size_t distance = distance_between(loc,*c);
 		if(distance < 6) {
 			return 0;
@@ -494,10 +497,10 @@ static int rank_castle_location(int x, int y, const is_valid_terrain& valid_terr
 		}
 	}
 
-	const int x_from_border = minimum<int>(x - min_x,max_x - x);
-	const int y_from_border = minimum<int>(y - min_y,max_y - y);
+	const int x_from_border = std::min<int>(x - min_x,max_x - x);
+	const int y_from_border = std::min<int>(y - min_y,max_y - y);
 
-	const int border_ranking = min_distance - minimum<int>(x_from_border,y_from_border) +
+	const int border_ranking = min_distance - std::min<int>(x_from_border,y_from_border) +
 	                           min_distance - x_from_border - y_from_border;
 
 	int current_ranking = border_ranking*2 + avg_distance*10 + lowest_distance*10;
@@ -524,17 +527,16 @@ static int rank_castle_location(int x, int y, const is_valid_terrain& valid_terr
 
 typedef std::map<t_translation::t_terrain, t_translation::t_list> tcode_list_cache;
 
-//!
-static gamemap::location place_village(const t_translation::t_map& map,
+static map_location place_village(const t_translation::t_map& map,
 	const size_t x, const size_t y, const size_t radius, const config& cfg,
 	tcode_list_cache &adj_liked_cache)
 {
-	const gamemap::location loc(x,y);
-	std::set<gamemap::location> locs;
+	const map_location loc(x,y);
+	std::set<map_location> locs;
 	get_tiles_radius(loc,radius,locs);
-	gamemap::location best_loc;
+	map_location best_loc;
 	size_t best_rating = 0;
-	for(std::set<gamemap::location>::const_iterator i = locs.begin();
+	for(std::set<map_location>::const_iterator i = locs.begin();
 			i != locs.end(); ++i) {
 
 		if(i->x < 0 || i->y < 0 || i->x >= static_cast<long>(map.size()) ||
@@ -545,20 +547,19 @@ static gamemap::location place_village(const t_translation::t_map& map,
 
 		const t_translation::t_terrain t = map[i->x][i->y];
 		const std::string str = t_translation::write_terrain_code(t);
-		const config* const child = cfg.find_child("village","terrain",str);
-		if(child != NULL) {
+		if (const config &child = cfg.find_child("village", "terrain", str)) {
 			tcode_list_cache::iterator l = adj_liked_cache.find(t);
 			t_translation::t_list *adjacent_liked;
 			if (l != adj_liked_cache.end()) {
 				adjacent_liked = &(l->second);
 			} else {
-				adj_liked_cache[t] = t_translation::read_list((*child)["adjacent_liked"]);
+				adj_liked_cache[t] = t_translation::read_list(child["adjacent_liked"]);
 				adjacent_liked = &(adj_liked_cache[t]);
 			}
 
-			size_t rating = atoi((*child)["rating"].c_str());
-			gamemap::location adj[6];
-			get_adjacent_tiles(gamemap::location(i->x,i->y),adj);
+			size_t rating = atoi(child["rating"].c_str());
+			map_location adj[6];
+			get_adjacent_tiles(map_location(i->x,i->y),adj);
 			for(size_t n = 0; n != 6; ++n) {
 				if(adj[n].x < 0 || adj[n].y < 0 ||
 						adj[n].x >= static_cast<long>(map.size()) ||
@@ -572,7 +573,7 @@ static gamemap::location place_village(const t_translation::t_map& map,
 			}
 
 			if(rating > best_rating) {
-				best_loc = gamemap::location(i->x,i->y);
+				best_loc = map_location(i->x,i->y);
 				best_rating = rating;
 			}
 		}
@@ -581,7 +582,6 @@ static gamemap::location place_village(const t_translation::t_map& map,
 	return best_loc;
 }
 
-//!
 static std::string generate_name(const unit_race& name_generator, const std::string& id,
 		std::string* base_name=NULL,
 		utils::string_map* additional_symbols=NULL)
@@ -612,6 +612,36 @@ static std::string generate_name(const unit_race& name_generator, const std::str
 	return "";
 }
 
+// "flood fill" a tile name to adjacent tiles of certain terrain
+static void flood_name(const map_location& start, const std::string& name, std::map<map_location,std::string>& tile_names,
+	const t_translation::t_match& tile_types, const terrain_map& terrain,
+	unsigned width, unsigned height,
+	size_t label_count, std::map<map_location,std::string>* labels, const std::string& full_name) {
+	map_location adj[6];
+	get_adjacent_tiles(start,adj);
+	size_t n;
+	//if adjacent tiles are tiles and unnamed, name them
+	for (n = 0; n < 6; n++) {
+		//we do not care for tiles outside the middle part
+		//cast to unsigned to skip x < 0 || y < 0 as well.
+		if (unsigned(adj[n].x) >= width / 3 || unsigned(adj[n].y) >= height / 3) {
+			continue;
+		}
+
+		const t_translation::t_terrain terr = terrain[adj[n].x + (width / 3)][adj[n].y + (height / 3)];
+		const location loc(adj[n].x, adj[n].y);
+		if((t_translation::terrain_matches(terr, tile_types)) && (tile_names.find(loc) == tile_names.end())) {
+			tile_names.insert(std::pair<location, std::string>(loc, name));
+			//labeling decision: this is result of trial and error on what looks best in game
+			if (label_count % 6 == 0) { //ensure that labels do not occur more often than every 6 recursions
+				labels->insert(std::pair<map_location, std::string>(loc, full_name));
+				label_count++; //ensure that no adjacent tiles get labeled
+			}
+			flood_name(adj[n], name, tile_names, tile_types, terrain, width, height, label_count++, labels, full_name);
+		}
+	}
+}
+
 namespace {
 
 // the configuration file should contain a number of [height] tags:
@@ -699,11 +729,10 @@ t_translation::t_terrain terrain_converter::convert_to() const
 
 } // end anon namespace
 
-//! Generate the map.
 std::string default_generate_map(size_t width, size_t height, size_t island_size, size_t island_off_center,
                                  size_t iterations, size_t hill_size,
 						         size_t max_lakes, size_t nvillages, size_t castle_size, size_t nplayers, bool roads_between_castles,
-								 std::map<gamemap::location,std::string>* labels, const config& cfg)
+								 std::map<map_location,std::string>* labels, const config& cfg)
 {
 	log_scope("map generation");
 
@@ -735,20 +764,18 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	LOG_NG << "done generating height map...\n";
 	LOG_NG << (SDL_GetTicks() - ticks) << "\n"; ticks = SDL_GetTicks();
 
-	const config* const names_info = cfg.child("naming");
-	config naming;
-	if(names_info != NULL) {
-		naming = *names_info;
-	}
+	config naming = cfg.child_or_empty("naming");
+	// HACK: dummy names to satisfy unit_race requirements
+	naming["id"] = "village_naming";
+	naming["plural_name"] = "villages";
 
 	// Make a dummy race for generating names
-	unit_race name_generator(naming);
+	const unit_race name_generator(naming);
 
 	std::vector<terrain_height_mapper> height_conversion;
 
-	const config::child_list& height_cfg = cfg.get_children("height");
-	for(config::child_list::const_iterator h = height_cfg.begin(); h != height_cfg.end(); ++h) {
-		height_conversion.push_back(terrain_height_mapper(**h));
+	foreach (const config &h, cfg.child_range("height")) {
+		height_conversion.push_back(terrain_height_mapper(h));
 	}
 
 	terrain_map terrain(width, t_translation::t_list(height, grassland));
@@ -765,6 +792,8 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 		}
 	}
 
+	std::map<int, t_translation::coordinate> starting_positions;
+	LOG_NG << output_map(terrain, starting_positions);
 	LOG_NG << "placed land forms\n";
 	LOG_NG << (SDL_GetTicks() - ticks) << "\n"; ticks = SDL_GetTicks();
 
@@ -783,7 +812,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	// We will also attempt to source a river from each lake.
 	std::set<location> lake_locs;
 
-	std::map<location,std::string> river_names, lake_names;
+	std::map<location, std::string> river_names, lake_names, road_names, bridge_names, mountain_names, forest_names, swamp_names;
 
 	const size_t nlakes = max_lakes > 0 ? (rand()%max_lakes) : 0;
 	for(size_t lake = 0; lake != nlakes; ++lake) {
@@ -801,10 +830,10 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 					size_t name_frequency = 20;
 					for(std::vector<location>::const_iterator r = river.begin(); r != river.end(); ++r) {
 
-						const gamemap::location loc(r->x-width/3,r->y-height/3);
+						const map_location loc(r->x-width/3,r->y-height/3);
 
 						if(((r - river.begin())%name_frequency) == name_frequency/2) {
-							labels->insert(std::pair<gamemap::location,std::string>(loc,name));
+							labels->insert(std::pair<map_location,std::string>(loc,name));
 						}
 
 						river_names.insert(std::pair<location,std::string>(loc,base_name));
@@ -842,14 +871,14 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 					}
 
 					if(!touches_other_lake) {
-						const gamemap::location loc(x-width/3,y-height/3);
+						const map_location loc(x-width/3,y-height/3);
 						labels->erase(loc);
-						labels->insert(std::pair<gamemap::location,std::string>(loc,name));
+						labels->insert(std::pair<map_location,std::string>(loc,name));
 					}
 
 					for(i = locs.begin(); i != locs.end(); ++i) {
 						const location loc(i->x-width/3,i->y-height/3);
-						lake_names.insert(std::pair<location,std::string>(*i,base_name));
+						lake_names.insert(std::pair<location, std::string>(loc, base_name));
 					}
 				}
 
@@ -863,11 +892,14 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 
 	const size_t default_dimensions = 40*40*9;
 
-	//! Convert grassland terrain to other types of flat terrain.
-	//! We generate a 'temperature map' which uses the height generation algorithm
-	//! to generate the temperature levels all over the map.
-	//! Then we can use a combination of height and terrain
-	//! to divide terrain up into more interesting types than the default.
+	/*
+	 * Convert grassland terrain to other types of flat terrain.
+	 *
+	 * We generate a 'temperature map' which uses the height generation
+	 * algorithm to generate the temperature levels all over the map.  Then we
+	 * can use a combination of height and terrain to divide terrain up into
+	 * more interesting types than the default.
+	 */
 	const height_map temperature_map = generate_height_map(width,height,
 	                                                       (atoi(cfg["temperature_iterations"].c_str())*width*height)/default_dimensions,
 														   atoi(cfg["temperature_size"].c_str()),0,0);
@@ -876,9 +908,8 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	LOG_NG << (SDL_GetTicks() - ticks) << "\n"; ticks = SDL_GetTicks();
 
 	std::vector<terrain_converter> converters;
-	const config::child_list& converter_items = cfg.get_children("convert");
-	for(config::child_list::const_iterator cv = converter_items.begin(); cv != converter_items.end(); ++cv) {
-		converters.push_back(terrain_converter(**cv));
+	foreach (const config &cv, cfg.child_range("convert")) {
+		converters.push_back(terrain_converter(cv));
 	}
 
 	LOG_NG << "created terrain converters\n";
@@ -901,34 +932,40 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	LOG_NG << "placing villages...\n";
 	LOG_NG << (SDL_GetTicks() - ticks) << "\n"; ticks = SDL_GetTicks();
 
-	//! Place villages in a 'grid', to make placing fair,
-	//! but with villages displaced from their position
-	//! according to terrain and randomness, to add some variety.
+	/*
+	 * Place villages in a 'grid', to make placing fair, but with villages
+	 * displaced from their position according to terrain and randomness, to
+	 * add some variety.
+	 */
 	std::set<location> villages;
 
 	LOG_NG << "placing castles...\n";
 
-	//! Try to find configuration for castles.
-	const config* const castle_config = cfg.child("castle");
-	if(castle_config == NULL) {
+	/** Try to find configuration for castles. */
+	const config &castle_config = cfg.child("castle");
+	if (!castle_config) {
 		LOG_NG << "Could not find castle configuration\n";
-		return "";
+		return std::string();
 	}
 
-	//! Castle configuration tag contains a 'valid_terrain' attribute
-	//! which is a list of terrains that the castle may appear on.
+	/*
+	 * Castle configuration tag contains a 'valid_terrain' attribute which is a
+	 * list of terrains that the castle may appear on.
+	 */
 	const t_translation::t_list list =
-		t_translation::read_list((*castle_config)["valid_terrain"]);
+		t_translation::read_list(castle_config["valid_terrain"]);
 
 	const is_valid_terrain terrain_tester(terrain, list);
 
-	//! Attempt to place castles at random.
-	//! Once we have placed castles, we run a sanity check
-	//! to make sure that the castles are well-placed.
-	//! If the castles are not well-placed, we try again.
-	//! Definition of 'well-placed' is if no two castles
-	//! are closer than 'min_distance' hexes from each other,
-	//! and the castles appear on a terrain listed in 'valid_terrain'.
+	/*
+	 * Attempt to place castles at random.
+	 *
+	 * Once we have placed castles, we run a sanity check to make sure that the
+	 * castles are well-placed.  If the castles are not well-placed, we try
+	 * again.  Definition of 'well-placed' is if no two castles are closer than
+	 * 'min_distance' hexes from each other, and the castles appear on a
+	 * terrain listed in 'valid_terrain'.
+	 */
 	std::vector<location> castles;
 	std::set<location> failed_locs;
 
@@ -939,7 +976,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 		const int min_y = height/3 + 3;
 		const int max_x = (width/3)*2 - 4;
 		const int max_y = (height/3)*2 - 4;
-		const size_t min_distance = atoi((*castle_config)["min_distance"].c_str());
+		const size_t min_distance = atoi(castle_config["min_distance"].c_str());
 
 		location best_loc;
 		int best_ranking = 0;
@@ -962,8 +999,10 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 			}
 		}
 		if(best_ranking == 0) {
+			//FIXME: Make this error message translatable (not possible atm due to string freeze)
 			ERR_NG << "No castle location found, aborting.\n";
-			return "";
+			std::string error = "No valid castle location found. Too many or too few mountain hexes? (please check the 'max hill size' parameter)";
+			throw mapgen_exception(error);
 		}
 		assert(std::find(castles.begin(), castles.end(), best_loc) == castles.end());
 		castles.push_back(best_loc);
@@ -988,9 +1027,11 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	for(size_t road = 0; road != nroads; ++road) {
 		log_scope("creating road");
 
-		//! We want the locations to be on the portion of the map
-		//! we're actually going to use, since roads on other parts of the map
-		//! won't have any influence, and doing it like this will be quicker.
+		/*
+		 * We want the locations to be on the portion of the map we're actually
+		 * going to use, since roads on other parts of the map won't have any
+		 * influence, and doing it like this will be quicker.
+		 */
 		location src = random_point_at_side(width/3 + 2,height/3 + 2);
 		location dst = random_point_at_side(width/3 + 2,height/3 + 2);
 
@@ -1015,14 +1056,15 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 			continue;
 		}
 
-		if (calc.cost(src,src, 0.0, false) >= 1000.0 || calc.cost(src,dst, 0.0, true) >= 1000.0) {
+		if (calc.cost(src, 0.0) >= 1000.0 || calc.cost(dst, 0.0) >= 1000.0) {
 			continue;
 		}
 
 		// Search a path out for the road
-		const paths::route rt = a_star_search(src, dst, 10000.0, &calc, width, height);
+		pathfind::plain_route rt = pathfind::a_star_search(src, dst, 10000.0, &calc, width, height);
 
-		const std::string& name = generate_name(name_generator,"road_name");
+		std::string road_base_name;
+		const std::string& name = generate_name(name_generator, "road_name", &road_base_name);
 		const int name_frequency = 20;
 		int name_count = 0;
 
@@ -1044,9 +1086,9 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 
 			// Find the configuration which tells us
 			// what to convert this tile to, to make it into a road.
-			const config* const child = cfg.find_child("road_cost", "terrain",
-					t_translation::write_terrain_code(terrain[x][y]));
-			if(child != NULL) {
+			if (const config &child = cfg.find_child("road_cost", "terrain",
+					t_translation::write_terrain_code(terrain[x][y])))
+			{
 				// Convert to bridge means that we want to convert
 				// depending upon the direction the road is going.
 				// Typically it will be in a format like,
@@ -1056,7 +1098,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 				// '\' will be used if the road is going south east-north west
 				// The terrain will be left unchanged otherwise
 				// (if there is no clear direction).
-				const std::string& convert_to_bridge = (*child)["convert_to_bridge"];
+				const std::string &convert_to_bridge = child["convert_to_bridge"];
 				if(convert_to_bridge.empty() == false) {
 					if(step == rt.steps.begin() || step+1 == rt.steps.end())
 						continue;
@@ -1070,25 +1112,27 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 					int direction = -1;
 
 					// If we are going north-south
-					if(last == adj[0] && next == adj[3] || last == adj[3] && next == adj[0]) {
+					if((last == adj[0] && next == adj[3]) || (last == adj[3] && next == adj[0])) {
 						direction = 0;
 					}
 
 					// If we are going south west-north east
-					else if(last == adj[1] && next == adj[4] || last == adj[4] && next == adj[1]) {
+					else if((last == adj[1] && next == adj[4]) || (last == adj[4] && next == adj[1])) {
 						direction = 1;
 					}
 
 					// If we are going south east-north west
-					else if(last == adj[2] && next == adj[5] || last == adj[5] && next == adj[2]) {
+					else if((last == adj[2] && next == adj[5]) || (last == adj[5] && next == adj[2])) {
 						direction = 2;
 					}
 
 					if(labels != NULL && on_bridge == false) {
 						on_bridge = true;
-						const std::string& name = generate_name(name_generator,"bridge_name");
-						const location loc(x-width/3,y-height/3);
-						labels->insert(std::pair<gamemap::location,std::string>(loc,name));
+						std::string bridge_base_name;
+						const std::string& name = generate_name(name_generator, "bridge_name", &bridge_base_name);
+						const location loc(x - width / 3, y-height/3);
+						labels->insert(std::pair<map_location,std::string>(loc,name));
+						bridge_names.insert(std::pair<location,std::string>(loc, bridge_base_name)); //add to use for village naming
 						bridges.insert(loc);
 					}
 
@@ -1105,16 +1149,18 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 				}
 
 				// Just a plain terrain substitution for a road
-				const std::string& convert_to = (*child)["convert_to"];
+				const std::string &convert_to = child["convert_to"];
 				if(convert_to.empty() == false) {
 					const t_translation::t_terrain letter =
 						t_translation::read_terrain_code(convert_to);
 					if(labels != NULL && terrain[x][y] != letter && name_count++ == name_frequency && on_bridge == false) {
-						labels->insert(std::pair<gamemap::location,std::string>(gamemap::location(x-width/3,y-height/3),name));
+						labels->insert(std::pair<map_location,std::string>(map_location(x-width/3,y-height/3),name));
 						name_count = 0;
 					}
 
 					terrain[x][y] = letter;
+					const location loc(x - width / 3, y - height / 3); //add to use for village naming
+					road_names.insert(std::pair<location,std::string>(loc, road_base_name));
 				}
 			}
 		}
@@ -1124,7 +1170,6 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 
 
 	// Now that road drawing is done, we can plonk down the castles.
-	std::map<int, t_translation::coordinate> starting_positions;
 	for(std::vector<location>::const_iterator c = castles.begin(); c != castles.end(); ++c) {
 		if(c->valid() == false) {
 			continue;
@@ -1161,12 +1206,61 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	LOG_NG << "placed castles\n";
 	LOG_NG << (SDL_GetTicks() - ticks) << "\n"; ticks = SDL_GetTicks();
 
-	if(nvillages > 0) {
-		const config* const naming = cfg.child("village_naming");
-		config naming_cfg;
-		if(naming != NULL) {
-			naming_cfg = *naming;
+	/*Random naming for landforms: mountains, forests, swamps, hills
+	 *we name these now that everything else is placed (as e.g., placing
+	 * roads could split a forest)
+	 */
+	for (x = width / 3; x < (width / 3)*2; x++) {
+		for (y = height / 3; y < (height / 3) * 2;y++) {
+		//check the terrain of the tile
+		const location loc(x - width / 3, y - height / 3);
+		const t_translation::t_terrain terr = terrain[x][y];
+		std::string name, base_name;
+		std::set<std::string> used_names;
+		if (t_translation::terrain_matches(terr, t_translation::ALL_MOUNTAINS)) {
+			//name every 15th mountain
+			if ((rand()%15) == 0) {
+				for(size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
+					name = generate_name(name_generator, "mountain_name", &base_name);
+				}
+				labels->insert(std::pair<map_location, std::string>(loc, name));
+				mountain_names.insert(std::pair<location, std::string>(loc, base_name));
+			}
 		}
+		else if (t_translation::terrain_matches(terr, t_translation::ALL_FORESTS)) {
+			//if the forest tile is not named yet, name it
+			const std::map<location, std::string>::const_iterator forest_name = forest_names.find(loc);
+			if(forest_name == forest_names.end()) {
+				for(size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
+					name = generate_name(name_generator, "forest_name", &base_name);
+				}
+				forest_names.insert(std::pair<location, std::string>(loc, base_name));
+				// name all connected forest tiles accordingly
+				flood_name(loc, base_name, forest_names, t_translation::ALL_FORESTS, terrain, width, height, 0, labels, name);
+			}
+		}
+		else if (t_translation::terrain_matches(terr, t_translation::ALL_SWAMPS)) {
+			//if the swamp tile is not named yet, name it
+			const std::map<location, std::string>::const_iterator swamp_name = swamp_names.find(loc);
+			if(swamp_name == swamp_names.end()) {
+				for(size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
+					name = generate_name(name_generator, "swamp_name", &base_name);
+				}
+				swamp_names.insert(std::pair<location, std::string>(loc, base_name));
+				// name all connected swamp tiles accordingly
+				flood_name(loc, base_name, swamp_names, t_translation::ALL_SWAMPS, terrain, width, height, 0, labels, name);
+			}
+		}
+
+		}
+	}
+
+	if (nvillages > 0)
+	{
+		config naming_cfg = cfg.child_or_empty("village_naming");
+		// HACK: dummy names to satisfy unit_race requirements
+		naming_cfg["id"] = "village_naming";
+		naming_cfg["plural_name"] = "villages";
 
 		const unit_race village_names_generator(naming_cfg);
 
@@ -1197,7 +1291,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 				const size_t x = (vx + add_x) - 1;
 				const size_t y = (vy + add_y) - 1;
 
-				const gamemap::location res = place_village(terrain,x,y,2,cfg,adj_liked_cache);
+				const map_location res = place_village(terrain,x,y,2,cfg,adj_liked_cache);
 
 				if(res.x >= static_cast<long>(width) / 3 &&
 						res.x  < static_cast<long>(width * 2) / 3 &&
@@ -1206,10 +1300,9 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 
 					const std::string str =
 						t_translation::write_terrain_code(terrain[res.x][res.y]);
-					const config* const child =
-						cfg.find_child("village","terrain",str);
-					if(child != NULL) {
-						const std::string& convert_to = (*child)["convert_to"];
+					if (const config &child = cfg.find_child("village", "terrain", str))
+					{
+						const std::string &convert_to = child["convert_to"];
 						if(convert_to != "") {
 							terrain[res.x][res.y] =
 								t_translation::read_terrain_code(convert_to);
@@ -1217,9 +1310,9 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 							villages.insert(res);
 
 							if(labels != NULL && naming_cfg.empty() == false) {
-								const gamemap::location loc(res.x-width/3,res.y-height/3);
+								const map_location loc(res.x-width/3,res.y-height/3);
 
-								gamemap::location adj[6];
+								map_location adj[6];
 								get_adjacent_tiles(loc,adj);
 
 								std::string name_type = "village_name";
@@ -1235,18 +1328,35 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 
 								size_t n;
 								for(n = 0; n != 6; ++n) {
+									const std::map<location,std::string>::const_iterator road_name = road_names.find(adj[n]);
+									if(road_name != road_names.end()) {
+										symbols["road"] = road_name->second;
+										name_type = "village_name_road";
+										break;
+									}
+
 									const std::map<location,std::string>::const_iterator river_name = river_names.find(adj[n]);
 									if(river_name != river_names.end()) {
 										symbols["river"] = river_name->second;
 										name_type = "village_name_river";
 
-										if(bridges.count(loc)) {
-											name_type = "village_name_river_bridge";
+										const std::map<location,std::string>::const_iterator bridge_name = bridge_names.find(adj[n]);
+										if(bridge_name != bridge_names.end()) {
+										//we should always end up here, since if there is an adjacent bridge, there has to be an adjacent river too
+										symbols["bridge"] = bridge_name->second;
+										name_type = "village_name_river_bridge";
 										}
 
 										break;
 									}
 
+									const std::map<location,std::string>::const_iterator forest_name = forest_names.find(adj[n]);
+									if(forest_name != forest_names.end()) {
+										symbols["forest"] = forest_name->second;
+										name_type = "village_name_forest";
+										break;
+									}
+
 									const std::map<location,std::string>::const_iterator lake_name = lake_names.find(adj[n]);
 									if(lake_name != lake_names.end()) {
 										symbols["lake"] = lake_name->second;
@@ -1254,6 +1364,20 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 										break;
 									}
 
+									const std::map<location,std::string>::const_iterator mountain_name = mountain_names.find(adj[n]);
+									if(mountain_name != mountain_names.end()) {
+										symbols["mountain"] = mountain_name->second;
+										name_type = "village_name_mountain";
+										break;
+									}
+
+									const std::map<location,std::string>::const_iterator swamp_name = swamp_names.find(adj[n]);
+									if(swamp_name != swamp_names.end()) {
+										symbols["swamp"] = swamp_name->second;
+										name_type = "village_name_swamp";
+										break;
+									}
+
 									const t_translation::t_terrain terr =
 										terrain[adj[n].x+width/3][adj[n].y+height/3];
 
@@ -1274,7 +1398,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 									} else if(forest_count >= 2) {
 										name_type = "village_name_forest";
 									} else if(mountain_count >= 1) {
-										name_type = "village_name_mountain";
+										name_type = "village_name_mountain_anonymous";
 									} else if(hill_count >= 2) {
 										name_type = "village_name_hill";
 									}
@@ -1286,7 +1410,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 								}
 
 								used_names.insert(name);
-								labels->insert(std::pair<gamemap::location,std::string>(loc,name));
+								labels->insert(std::pair<map_location,std::string>(loc,name));
 							}
 						}
 					}
@@ -1310,7 +1434,7 @@ generator_map generators;
 
 #ifdef TEST_MAPGEN
 
-//! Standalone testprogram for the mapgenerator.
+/** Standalone testprogram for the mapgenerator. */
 int main(int argc, char** argv)
 {
 	int x = 50, y = 50, iterations = 50,
