@@ -1,6 +1,6 @@
-/* $Id: mapgen_dialog.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: mapgen_dialog.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
 class default_map_generator : public map_generator
 {
 public:
-	default_map_generator(const config* game_config);
+	default_map_generator(const config &game_config);
 
 	bool allow_user_config() const;
 	void user_config(display& disp);
