@@ -1,6 +1,6 @@
-/* $Id: marked-up_text.cpp 37533 2009-08-07 02:24:48Z soliton $ */
+/* $Id: marked-up_text.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #include "font.hpp"
 #include "gettext.hpp"
 #include "marked-up_text.hpp"
+#include "serialization/string_utils.hpp"
 #include "video.hpp"
 #include "wml_exception.hpp"
 
@@ -38,6 +39,17 @@ const char LARGE_TEXT='*', SMALL_TEXT='`',
            GREEN_TEXT='@', RED_TEXT='#',
            COLOR_TEXT='<', IMAGE='&';
 
+// some colors often used in UI
+const std::string weapon = "<245,230,193>",
+		weapon_details = "<166,146,117>",
+		unit_type = "<245,230,193>",
+		race = "<166,146,117>";
+const SDL_Color
+	weapon_color = { 245, 230, 193, 0 },
+	weapon_details_color = { 166, 146, 117, 0 },
+	unit_type_color = { 245, 230, 193, 0 },
+	race_color = { 166, 146, 117, 0 };
+
 std::string::const_iterator parse_markup(std::string::const_iterator i1,
 												std::string::const_iterator i2,
 												int* font_size,
@@ -140,16 +152,8 @@ std::string del_tags(const std::string& text){
 	return utils::join(lines, '\n');
 }
 
-std::string nullify_markup(const std::string& text) {
-	std::vector<std::string> lines = utils::split(text, '\n', 0);
-	std::vector<std::string>::iterator line;
-	for(line = lines.begin(); line != lines.end(); ++line) {
-		*line = std::string() + NULL_MARKUP + *line;
-	}
-	return utils::join(lines, '\n');
-}
-
-std::string color2markup(const SDL_Color color) {
+std::string color2markup(const SDL_Color &color)
+{
 	std::stringstream markup;
 	// The RGB of SDL_Color are Uint8, we need to cast them to int.
 	// Without cast, it gives their char equivalent.
@@ -160,6 +164,13 @@ std::string color2markup(const SDL_Color color) {
 	return markup.str();
 }
 
+std::string color2hexa(const SDL_Color &color)
+{
+	char buf[7];
+	sprintf(buf, "%02x%02x%02x", color.r, color.g, color.b);
+	return buf;
+}
+
 SDL_Rect text_area(const std::string& text, int size, int style)
 {
 	const SDL_Rect area = {0,0,10000,10000};
@@ -191,9 +202,7 @@ SDL_Rect draw_text(surface dst, const SDL_Rect& area, int size,
 		i1 = parse_markup(i1,i2,&sz,&col,&text_style);
 
 		if(i1 != i2) {
-			std::string new_string(i1,i2);
-
-			utils::unescape(new_string);
+			std::string new_string = utils::unescape(std::string(i1, i2));
 
 			const SDL_Rect rect = draw_text_line(dst, area, sz, col, new_string, x, y, use_tooltips, text_style);
 			if(rect.w > res.w) {
@@ -240,6 +249,43 @@ bool is_format_char(char c)
 	}
 }
 
+bool is_cjk_char(const wchar_t ch)
+{
+	/**
+	 * You can check these range at http://unicode.org/charts/
+	 * see the "East Asian Scripts" part.
+	 * Notice that not all characters in that part is still in use today, so don't list them all here.
+	 * Below are characters that I guess may be used in wesnoth translations.
+	 */
+
+	//FIXME add range from Japanese-specific and Korean-specific section if you know the characters are used today.
+	return
+		//Han Ideographs: all except Supplement
+		(ch >= 0x4e00 && ch < 0x9fcf) ||
+		(ch >= 0x3400 && ch < 0x4dbf) ||
+		(ch >= 0x20000 && ch < 0x2a6df) ||
+		(ch >= 0xf900 && ch < 0xfaff) ||
+		(ch >= 0x3190 && ch < 0x319f) ||
+
+		//Radicals: all except Ideographic Description
+		(ch >= 0x2e80 && ch < 0x2eff) ||
+		(ch >= 0x2f00 && ch < 0x2fdf) ||
+		(ch >= 0x31c0 && ch < 0x31ef) ||
+
+		//Chinese-specific: Bopomofo
+		(ch >= 0x3000 && ch < 0x303f) ||
+
+		//Japanese-specific: Halfwidth Katakana
+		(ch >= 0xff00 && ch < 0xffef) ||
+
+		//Japanese-specific: Hiragana, Katakana
+		(ch >= 0x3040 && ch <= 0x309f) ||
+		(ch >= 0x30a0 && ch <= 0x30ff) ||
+
+		//Korean-specific: Hangul Syllables, Halfwidth Jamo
+		(ch >= 0xac00 && ch < 0xd7af) ||
+		(ch >= 0xff00 && ch < 0xffef);
+}
 static void cut_word(std::string& line, std::string& word, int font_size, int style, int max_width)
 {
 	std::string tmp = line;
@@ -274,19 +320,41 @@ namespace {
  *
  * * the following characters cannot end a line (so we will never break after them):
  * （〔［｛〈《「『【‘“
+ *
+ * Unicode range that concerns word wrap for Chinese:
+ *   全角ASCII、全角中英文标点 (Fullwidth Character for ASCII, English punctuations and part of Chinese punctuations)
+ *   http://www.unicode.org/charts/PDF/UFF00.pdf
+ *   CJK 标点符号 (CJK punctuations)
+ *   http://www.unicode.org/charts/PDF/U3000.pdf
  */
-inline bool no_break_after(wchar_t ch)
+inline bool no_break_after(const wchar_t ch)
 {
 	return
+		/**
+		 * don't break after these Japanese characters
+		 */
 		ch == 0x2018 || ch == 0x201c || ch == 0x3008 || ch == 0x300a || ch == 0x300c ||
 		ch == 0x300e || ch == 0x3010 || ch == 0x3014 || ch == 0xff08 || ch == 0xff3b ||
-		ch == 0xff5b;
+		ch == 0xff5b ||
 
+		/**
+		 * FIXME don't break after these Korean characters
+		 */
+
+		/**
+		 * don't break after these Chinese characters
+		 * contains left side of different kinds of brackets and quotes
+		 */
+		ch == 0x3014 || ch == 0x3016 || ch == 0x3008 || ch == 0x301a || ch == 0x3008 ||
+		ch == 0x300a || ch == 0x300c || ch == 0x300e || ch == 0x3010 || ch == 0x301d;
 }
 
-inline bool no_break_before(wchar_t ch)
+inline bool no_break_before(const wchar_t ch)
 {
 	return
+		/**
+		 * don't break before these Japanese characters
+		 */
 		ch == 0x2019 || ch == 0x201d || ch == 0x2026 || ch == 0x3001 || ch == 0x3002 ||
 		ch == 0x3005 || ch == 0x3009 || ch == 0x300b || ch == 0x300d || ch == 0x300f ||
 		ch == 0x3011 || ch == 0x3015 || ch == 0x3041 || ch == 0x3043 || ch == 0x3045 ||
@@ -296,36 +364,45 @@ inline bool no_break_before(wchar_t ch)
 		ch == 0x30e3 || ch == 0x30e5 || ch == 0x30e7 || ch == 0x30ee || ch == 0x30f5 ||
 		ch == 0x30f6 || ch == 0x30fb || ch == 0x30fc || ch == 0x30fd || ch == 0x30fe ||
 		ch == 0xff01 || ch == 0xff09 || ch == 0xff0c || ch == 0xff0e || ch == 0xff1a ||
-		ch == 0xff1b || ch == 0xff1f || ch == 0xff3d || ch == 0xff5d;
+		ch == 0xff1b || ch == 0xff1f || ch == 0xff3d || ch == 0xff5d ||
+
+		/**
+		 * FIXME don't break before these Korean characters
+		 */
+
+		/**
+		 * don't break before these Chinese characters
+		 * contains
+		 *   many Chinese punctuations that should not start a line
+		 *   and right side of different kinds of brackets, quotes
+		 */
+		ch == 0x3001 || ch == 0x3002 || ch == 0x301c || ch == 0xff01 || ch == 0xff0c ||
+		ch == 0xff0d || ch == 0xff0e || ch == 0xff1a || ch == 0xff1b || ch == 0xff1f ||
+		ch == 0xff64 || ch == 0xff65 || ch == 0x3015 || ch == 0x3017 || ch == 0x3009 ||
+		ch == 0x301b || ch == 0x3009 || ch == 0x300b || ch == 0x300d || ch == 0x300f ||
+		ch == 0x3011 || ch == 0x301e;
 }
 
-inline bool break_before(wchar_t ch)
+inline bool break_before(const wchar_t ch)
 {
 	if(no_break_before(ch))
 		return false;
 
-	return ch == ' ' ||
-		// CKJ characters
-		(ch >= 0x3000 && ch < 0xa000) ||
-		(ch >= 0xf900 && ch < 0xfb00) ||
-		(ch >= 0xff00 && ch < 0xfff0);
+	return is_cjk_char(ch);
 }
 
-inline bool break_after(wchar_t ch)
+inline bool break_after(const wchar_t ch)
 {
 	if(no_break_after(ch))
 		return false;
 
-	return ch == ' ' ||
-		// CKJ characters
-		(ch >= 0x3000 && ch < 0xa000) ||
-		(ch >= 0xf900 && ch < 0xfb00) ||
-		(ch >= 0xff00 && ch < 0xfff0);
+	return is_cjk_char(ch);
 }
 
 } // end of anon namespace
 
-std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width, int max_height, int max_lines)
+std::string word_wrap_text(const std::string& unwrapped_text, int font_size,
+	int max_width, int max_height, int max_lines, bool partial_line)
 {
 	VALIDATE(max_width > 0, _("The maximum text width is less than 1."));
 
@@ -403,7 +480,7 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 			line_width += word_width;
 
 			if(static_cast<long>(line_width) > max_width) {
-				if(static_cast<long>(word_width) > max_width) {
+				if (!partial_line && static_cast<long>(word_width) > max_width) {
 					cut_word(current_line,
 						current_word, font_sz, style, max_width);
 				}
@@ -449,52 +526,5 @@ SDL_Rect draw_wrapped_text(CVideo* gui, const SDL_Rect& area, int font_size,
 	return font::draw_text(gui, area, font_size, colour, wrapped_text, x, y, false);
 }
 
-size_t text_to_lines(std::string& message, size_t max_length)
-{
-	std::string starting_markup;
-	bool at_start = true;
-
-	size_t cur_line = 0, longest_line = 0;
-	for(std::string::iterator i = message.begin(); i != message.end(); ++i) {
-		if(at_start) {
-			if(font::is_format_char(*i)) {
-				starting_markup.push_back(*i);
-			} else {
-				at_start = false;
-			}
-		}
-
-		if(*i == '\n') {
-			at_start = true;
-			starting_markup = "";
-		}
-
-		if(*i == ' ' && cur_line > max_length) {
-			*i = '\n';
-			const size_t index = i - message.begin();
-			message.insert(index+1,starting_markup);
-			i = message.begin() + index + starting_markup.size();
-
-			if(cur_line > longest_line)
-				longest_line = cur_line;
-
-			cur_line = 0;
-		}
-
-		if(*i == '\n' || i+1 == message.end()) {
-			if(cur_line > longest_line)
-				longest_line = cur_line;
-
-			cur_line = 0;
-
-		} else {
-			++cur_line;
-		}
-	}
-
-	return longest_line;
-}
-
-
 } // end namespace font
 
