@@ -1,6 +1,6 @@
-/* $Id: marked-up_text.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: marked-up_text.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,10 @@ namespace font {
 extern const char LARGE_TEXT, SMALL_TEXT, BOLD_TEXT, NORMAL_TEXT, NULL_MARKUP, BLACK_TEXT, GRAY_TEXT,
                   GOOD_TEXT, BAD_TEXT, GREEN_TEXT, RED_TEXT, COLOR_TEXT, YELLOW_TEXT, IMAGE;
 
+extern const std::string weapon, weapon_details, unit_type, race;
+
+extern const SDL_Color weapon_color, weapon_details_color, unit_type_color, race_color;
+
 /** Parses the markup-tags at the front of a string. */
 std::string::const_iterator parse_markup(std::string::const_iterator i1,
 												std::string::const_iterator i2,
@@ -71,9 +75,6 @@ SDL_Rect text_area(const std::string& text, int size, int style=0);
 /** Copy string, but without tags at the beginning */
 std::string del_tags(const std::string& text);
 
-/** Copy string, but with NULL MARKUP tag at the beginning of each line */
-std::string nullify_markup(const std::string& text);
-
 /**
  * Determine if char is one of the special chars used as markup.
  *
@@ -82,8 +83,19 @@ std::string nullify_markup(const std::string& text);
  */
 bool is_format_char(char c);
 
+/**
+ * Determine if a wchar_t is a CJK character
+ *
+ * @retval true                   Input-char is a CJK char
+ * @retval false                  Input-char is a not CJK char.
+ */
+bool is_cjk_char(const wchar_t ch);
+
 /** Create string of color-markup, such as "<255,255,0>" for yellow. */
-std::string color2markup(const SDL_Color color);
+std::string color2markup(const SDL_Color &color);
+
+/** Creates the hexadecimal string of a color, such as "#ffff00" for yellow. */
+std::string color2hexa(const SDL_Color &color);
 
 /**
  * Wrap text.
@@ -92,7 +104,8 @@ std::string color2markup(const SDL_Color color);
  * - If this is not possible, e.g. the word is too big to fit, wrap it on a
  * - char basis.
  */
-std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width, int max_height=-1, int max_lines=-1);
+std::string word_wrap_text(const std::string& unwrapped_text, int font_size,
+	int max_width, int max_height = -1, int max_lines = -1, bool partial_line = false);
 
 /**
  * Draw text on the screen, fit text to maximum width, no markup, no tooltips.
@@ -109,9 +122,6 @@ SDL_Rect draw_wrapped_text(CVideo* gui, const SDL_Rect& area, int font_size,
 			     const SDL_Color& colour, const std::string& text,
 			     int x, int y, int max_width);
 
-/** Chop up one long string of text into lines. */
-size_t text_to_lines(std::string& text, size_t max_length);
-
 } // end namespace font
 
 #endif // MARKED_UP_TEXT_HPP_INCLUDED
