@@ -1,6 +1,6 @@
-/* $Id: menu_events.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: menu_events.hpp 41353 2010-02-22 20:08:38Z boucman $ */
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -18,13 +18,12 @@
 
 #include "global.hpp"
 #include "actions.hpp"
+#include "chat_events.hpp"
 #include "show_dialog.hpp"
-#include "display.hpp"
 #include "floating_textbox.hpp"
-#include "widgets/textbox.hpp"
 
 class game_state;
-class gamestatus;
+class tod_manager;
 
 namespace events {
 	class mouse_handler;
@@ -32,31 +31,13 @@ namespace events {
 
 struct fallback_ai_to_human_exception {};
 
-namespace events{
-
-class chat_handler {
-public:
-	chat_handler();
-	virtual ~chat_handler();
-
-protected:
-	void do_speak(const std::string& message, bool allies_only=false);
-
-	//called from do_speak
-	virtual void add_chat_message(const time_t& time,
-			const std::string& speaker, int side, const std::string& message,
-			game_display::MESSAGE_TYPE type=game_display::MESSAGE_PRIVATE)=0;
-	virtual void send_chat_message(const std::string& message, bool allies_only=false)=0;
-	void send_command(const std::string& cmd, const std::string& args="");
-	void change_logging(const std::string& data);
-	friend class chat_command_handler;
-};
+namespace events {
 
 class menu_handler : private chat_handler {
 public:
 	menu_handler(game_display* gui, unit_map& units, std::vector<team>& teams,
 		const config& level, const gamemap& map,
-		const config& game_config, const gamestatus& status, game_state& gamestate,
+		const config& game_config, const tod_manager& tod_mng, game_state& gamestate,
 		undo_list& undo_stack, undo_list& redo_stack);
 	virtual ~menu_handler();
 
@@ -64,43 +45,44 @@ public:
 	gui::floating_textbox& get_textbox();
 	void set_gui(game_display* gui) { gui_ = gui; }
 
-	std::string get_title_suffix(int team_num);
-	void objectives(const unsigned int team_num);
-	void show_statistics(const unsigned int team_num);
+	std::string get_title_suffix(int side_num);
+	void objectives(int side_num);
+	void show_statistics(int side_num);
 	void unit_list();
 	void status_table(int selected=0);
-	void save_game(const std::string& message, gui::DIALOG_TYPE dialog_type, const bool has_exit_button=false, const bool replay=false);
 	void save_map();
-	void load_game();
 	void preferences();
 	void show_chat_log();
 	void show_help();
 	void speak();
 	void whisper();
 	void shout();
-	void recruit(const bool browse, const unsigned int team_num, const map_location& last_hex);
-	void repeat_recruit(const unsigned int team_num, const map_location& last_hex);
-	void recall(const unsigned int team_num, const map_location& last_hex);
-	void undo(const unsigned int team_num);
-	void redo(const unsigned int team_num);
-	void show_enemy_moves(bool ignore_units, const unsigned int team_num);
-	void toggle_shroud_updates(const unsigned int team_num);
-	void update_shroud_now(const unsigned int team_num);
-	bool end_turn(const unsigned int team_num);
-	void goto_leader(const unsigned int team_num);
+	void recruit(bool browse, int side_num, const map_location &last_hex);
+	void repeat_recruit(int side_num, const map_location &last_hex);
+	void recall(int side_num, const map_location& last_hex);
+	void undo(int side_num);
+	void redo(int side_num);
+	void show_enemy_moves(bool ignore_units, int side_num);
+	void toggle_shroud_updates(int side_num);
+	void update_shroud_now(int side_num);
+	bool end_turn(int side_num);
+	void goto_leader(int side_num);
 	void unit_description(mouse_handler& mousehandler);
 	void rename_unit(mouse_handler& mousehandler);
 	void create_unit(mouse_handler& mousehandler);
-	void change_unit_side(mouse_handler& mousehandler);
+	void create_unit_2(mouse_handler& mousehandler); // TODO: replace create_unit when complete
+	void change_side(mouse_handler& mousehandler);
 	void label_terrain(mouse_handler& mousehandler, bool team_only);
 	void clear_labels();
-	void continue_move(mouse_handler& mousehandler, const unsigned int team_num);
+	void continue_move(mouse_handler &mousehandler, int side_num);
+	void execute_gotos(mouse_handler &mousehandler, int side_num);
+	void toggle_ellipses();
 	void toggle_grid();
-	void unit_hold_position(mouse_handler& mousehandler, const unsigned int team_num);
-	void end_unit_turn(mouse_handler& mousehandler, const unsigned int team_num);
+	void unit_hold_position(mouse_handler &mousehandler, int side_num);
+	void end_unit_turn(mouse_handler &mousehandler, int side_num);
 	void search();
 	void user_command();
-	void custom_command(mouse_handler& mousehandler, const unsigned int team_num);
+	void custom_command(mouse_handler& mousehandler, int side_num);
 	void ai_formula();
 	void clear_messages();
 #ifdef USRCMD2
@@ -109,20 +91,19 @@ public:
 #endif
 
 	unit_map::iterator current_unit(mouse_handler& mousehandler);
-	unit_map::const_iterator current_unit(const mouse_handler& mousehandler) const;
+	unit_map::const_iterator current_unit(const mouse_handler &mousehandler) const
+	{ return const_cast<menu_handler *>(this)->current_unit(const_cast<mouse_handler &>(mousehandler)); }
 	void move_unit_to_loc(const unit_map::const_iterator& ui, const map_location& target,
-		bool continue_move, const unsigned int team_num, mouse_handler& mousehandler);
+		bool continue_move, int side_num, mouse_handler &mousehandler);
 	void do_speak();
 	void do_search(const std::string& new_search);
-	void do_command(const std::string& str, const unsigned int team_num, mouse_handler& mousehandler);
-	void do_ai_formula(const std::string& str, const unsigned int team_num, mouse_handler& mousehandler);
-	void clear_undo_stack(const unsigned int team_num);
-	void autosave(const std::string &label, unsigned turn, const config &starting_pos) const;
-	bool has_team() const;
+	void do_command(const std::string &str, int side_num, mouse_handler &mousehandler);
+	void do_ai_formula(const std::string &str, int side_num, mouse_handler &mousehandler);
+	void clear_undo_stack(int side_num);
 protected:
 	void add_chat_message(const time_t& time, const std::string& speaker,
 			int side, const std::string& message,
-			game_display::MESSAGE_TYPE type=game_display::MESSAGE_PRIVATE);
+			events::chat_handler::MESSAGE_TYPE type=events::chat_handler::MESSAGE_PRIVATE);
 	void send_chat_message(const std::string& message, bool allies_only=false);
 private:
 	//console_handler is basically a sliced out part of menu_handler
@@ -130,11 +111,10 @@ private:
 	friend class console_handler;
 
 	//void do_speak(const std::string& message, bool allies_only);
-	void do_recruit(const std::string& name, const unsigned int team_num, const map_location& last_hex);
+	void do_recruit(const std::string& name, int side_num, const map_location& last_hex);
 //	std::vector<std::string> create_unit_table(const statistics::stats::str_int_map& m,unsigned int team);
-	void write_game_snapshot(config& start) const;
 	bool has_friends() const;
-	bool clear_shroud(const unsigned int team_num);
+	bool clear_shroud(int side_num);
 	static void change_controller(const std::string& side, const std::string& controller);
 	static void change_side_controller(const std::string& side, const std::string& player, bool own_side=false);
 	void scenario_settings_table(int selected=0);
@@ -145,7 +125,7 @@ private:
 	const config& level_;
 	const gamemap& map_;
 	const config& game_config_;
-	const gamestatus& status_;
+	const tod_manager& tod_manager_;
 	game_state& gamestate_;
 
 	undo_list& undo_stack_;
