@@ -1,6 +1,6 @@
-/* $Id: minimap.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: minimap.hpp 41179 2010-02-13 10:18:45Z silene $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -14,16 +14,16 @@
 #ifndef MINIMAP_HPP_INCLUDED
 #define MINIMAP_HPP_INCLUDED
 
-#include "map.hpp"
-#include "sdl_utils.hpp"
-
+#include <cstddef>
 
+class gamemap;
+class surface;
 class team;
 
 namespace image {
 	///function to create the minimap for a given map
 	///the surface returned must be freed by the user
-	surface getMinimap(int w, int h, const gamemap& map_, const viewpoint* vm=NULL);
+	surface getMinimap(int w, int h, const gamemap &map_, const team *vm = NULL);
 }
 
 #endif
