@@ -1,6 +1,6 @@
-/* $Id: mouse_events.cpp 36315 2009-06-21 10:47:04Z silene $ */
+/* $Id: mouse_events.cpp 46670 2010-09-23 16:00:03Z alink $ */
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -13,10 +13,13 @@
    See the COPYING file for more details.
 */
 
+#include "global.hpp"
+
 #include "mouse_events.hpp"
 
 #include "attack_prediction_display.hpp"
 #include "dialogs.hpp"
+#include "foreach.hpp"
 #include "game_end_exceptions.hpp"
 #include "game_events.hpp"
 #include "gettext.hpp"
@@ -24,23 +27,32 @@
 #include "map.hpp"
 #include "marked-up_text.hpp"
 #include "menu_events.hpp"
+#include "play_controller.hpp"
 #include "sound.hpp"
 #include "replay.hpp"
+#include "resources.hpp"
+#include "rng.hpp"
+#include "tod_manager.hpp"
 #include "wml_separators.hpp"
 
+#include <boost/bind.hpp>
+
+static lg::log_domain log_engine("engine");
+#define ERR_NG LOG_STREAM(err, log_engine)
+#define LOG_NG LOG_STREAM(info, log_engine)
 
 namespace events{
 
 
 mouse_handler::mouse_handler(game_display* gui, std::vector<team>& teams,
-		unit_map& units, gamemap& map, gamestatus& status,
+		unit_map& units, gamemap& map, tod_manager& tod_mng,
 		undo_list& undo_stack, undo_list& redo_stack) :
 	mouse_handler_base(),
 	map_(map),
 	gui_(gui),
 	teams_(teams),
 	units_(units),
-	status_(status),
+	tod_manager_(tod_mng),
 	undo_stack_(undo_stack),
 	redo_stack_(redo_stack),
 	previous_hex_(),
@@ -48,11 +60,11 @@ mouse_handler::mouse_handler(game_display* gui, std::vector<team>& teams,
 	selected_hex_(),
 	next_unit_(),
 	current_route_(),
+	waypoints_(),
 	current_paths_(),
 	enemy_paths_(false),
 	path_turns_(0),
-	team_num_(1),
-	enemies_visible_(false),
+	side_num_(1),
 	undo_(false),
 	over_route_(false),
 	attackmove_(false),
@@ -64,12 +76,13 @@ mouse_handler::mouse_handler(game_display* gui, std::vector<team>& teams,
 
 mouse_handler::~mouse_handler()
 {
+	rand_rng::clear_new_seed_callback();
 	singleton_ = NULL;
 }
 
-void mouse_handler::set_team(const int team_number)
+void mouse_handler::set_side(int side_number)
 {
-	team_num_ = team_number;
+	side_num_ = side_number;
 }
 
 int mouse_handler::drag_threshold() const
@@ -111,7 +124,7 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update)
 	if (update) {
 		if (reachmap_invalid_) {
 			reachmap_invalid_ = false;
-			if (!current_paths_.routes.empty() && !show_partial_move_) {
+			if (!current_paths_.destinations.empty() && !show_partial_move_) {
 				unit_map::iterator u = find_unit(selected_hex_);
 				if(selected_hex_.valid() && u != units_.end() ) {
 					// reselect the unit without firing events (updates current_paths_)
@@ -130,7 +143,7 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update)
 
 		if(enemy_paths_) {
 			enemy_paths_ = false;
-			current_paths_ = paths();
+			current_paths_ = pathfind::paths();
 			gui().unhighlight_reach();
 		} else if(over_route_) {
 			over_route_ = false;
@@ -151,11 +164,16 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update)
 		//If the cursor is on WAIT, we don't change it and let the setter
 		//of this state end it
 		if (cursor::get() != cursor::WAIT) {
-			if(selected_unit != units_.end() && selected_unit->second.side() == team_num_
-			   && !selected_unit->second.incapacitated() && !browse) {
+			if (selected_unit != units_.end() &&
+			    selected_unit->second.side() == side_num_ &&
+			    !selected_unit->second.incapacitated() && !browse)
+			{
 				if (attack_from.valid()) {
 					cursor::set(dragging_started_ ? cursor::ATTACK_DRAG : cursor::ATTACK);
-				} else if (mouseover_unit==units_.end() && current_paths_.routes.count(new_hex)) {
+				}
+				else if (mouseover_unit==units_.end() &&
+				         current_paths_.destinations.contains(new_hex))
+				{
 					cursor::set(dragging_started_ ? cursor::MOVE_DRAG : cursor::MOVE);
 				} else {
 					// selecte unit can't attack or move there
@@ -189,14 +207,16 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update)
 		if(dest == selected_hex_ || dest_un != units_.end()) {
 			current_route_.steps.clear();
 			gui().set_route(NULL);
-		} else if(!current_paths_.routes.empty() && map_.on_board(selected_hex_) &&
-		   map_.on_board(new_hex)) {
-
+		}
+		else if (!current_paths_.destinations.empty() &&
+		         map_.on_board(selected_hex_) && map_.on_board(new_hex))
+		{
 			if(selected_unit != units_.end() && !selected_unit->second.incapacitated()) {
+				// Show the route from selected unit to mouseover hex
 				// the movement_reset is active only if it's not the unit's turn
 				unit_movement_resetter move_reset(selected_unit->second,
-						selected_unit->second.side() != team_num_);
-				current_route_ = get_route(selected_unit, dest, viewing_team());
+						selected_unit->second.side() != side_num_);
+				current_route_ = get_route(selected_unit, dest, waypoints_, viewing_team());
 				if(!browse) {
 					gui().set_route(&current_route_);
 				}
@@ -205,13 +225,15 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update)
 
 		unit_map::iterator un = mouseover_unit;
 
-		if(un != units_.end() && current_paths_.routes.empty() && !gui().fogged(un->first)) {
-			if (un->second.side() != team_num_) {
+		if (un != units_.end() && current_paths_.destinations.empty() &&
+		    !gui().fogged(un->first))
+		{
+			if (un->second.side() != side_num_) {
 				//unit under cursor is not on our team, highlight reach
 				unit_movement_resetter move_reset(un->second);
 
-				const bool teleport = un->second.get_ability_bool("teleport",un->first);
-				current_paths_ = paths(map_,units_,new_hex,teams_,
+				bool teleport = un->second.get_ability_bool("teleport");
+				current_paths_ = pathfind::paths(map_,units_,new_hex,teams_,
 									false,teleport,viewing_team(),path_turns_);
 				gui().highlight_reach(current_paths_);
 				enemy_paths_ = true;
@@ -219,7 +241,7 @@ void mouse_handler::mouse_motion(int x, int y, const bool browse, bool update)
 				//unit is on our team, show path if the unit has one
 				const map_location go_to = un->second.get_goto();
 				if(map_.on_board(go_to)) {
-					paths::route route = get_route(un, go_to, current_team());
+					pathfind::marked_route route = get_route(un, go_to, un->second.waypoints(), current_team());
 					gui().set_route(&route);
 				}
 				over_route_ = true;
@@ -240,18 +262,18 @@ unit_map::iterator mouse_handler::selected_unit()
 
 unit_map::iterator mouse_handler::find_unit(const map_location& hex)
 {
-	return find_visible_unit(units_,hex,map_,teams_,viewing_team());
+	return find_visible_unit(units_, hex, viewing_team());
 }
 
 unit_map::const_iterator mouse_handler::find_unit(const map_location& hex) const
 {
-	return find_visible_unit(units_,hex,map_,teams_,viewing_team());
+	return find_visible_unit(units_, hex, viewing_team());
 }
 
 map_location mouse_handler::current_unit_attacks_from(const map_location& loc)
 {
 	const unit_map::const_iterator current = find_unit(selected_hex_);
-	if(current == units_.end() || current->second.side() != team_num_
+	if(current == units_.end() || current->second.side() != side_num_
 		|| current->second.attacks_left()==0) {
 		return map_location();
 	}
@@ -280,7 +302,8 @@ map_location mouse_handler::current_unit_attacks_from(const map_location& loc)
 			continue;
 		}
 
-		if(current_paths_.routes.count(adj[n])) {
+		if (current_paths_.destinations.contains(adj[n]))
+		{
 			static const size_t NDIRECTIONS = map_location::NDIRECTIONS;
 			unsigned int difference = abs(int(preferred - n));
 			if(difference > NDIRECTIONS/2) {
@@ -301,30 +324,64 @@ map_location mouse_handler::current_unit_attacks_from(const map_location& loc)
 	return res;
 }
 
-paths::route mouse_handler::get_route(unit_map::const_iterator un, map_location go_to, team &team)
+void mouse_handler::add_waypoint(const map_location& loc) {
+	std::vector<map_location>::iterator w = std::find(waypoints_.begin(), waypoints_.end(), loc);
+	//toggle between add a new one and remove an old one
+	if(w != waypoints_.end()){
+		waypoints_.erase(w);
+	} else {
+		waypoints_.push_back(loc);
+	}
+
+	// we need to update the route, simulate a mouse move for the moment
+	// (browse is supposed false here, 0,0 are dummy values)
+	mouse_motion(0,0, false, true);
+}
+
+pathfind::marked_route mouse_handler::get_route(unit_map::const_iterator un, map_location go_to, const std::vector<map_location>& waypoints, team &team)
 {
 	// The pathfinder will check unit visibility (fogged/stealthy).
-	const shortest_path_calculator calc(un->second,team,units_,teams_,map_);
+	const pathfind::shortest_path_calculator calc(un->second,team,units_,teams_,map_);
 
-	std::set<map_location> allowed_teleports;
+	std::set<map_location> allowed_teleports = pathfind::get_teleport_locations(
+		un->second, units_, viewing_team());
 
-	if(un->second.get_ability_bool("teleport",un->first)) {
-		// search all known empty friendly villages
-		for(std::set<map_location>::const_iterator i = team.villages().begin();
-				i != team.villages().end(); ++i) {
-			if (viewing_team().is_enemy(un->second.side()) && viewing_team().fogged(*i))
-				continue;
+	pathfind::plain_route route;
 
-			unit_map::const_iterator occupant = find_unit(*i);
-			if (occupant != units_.end() && occupant != un)
-				continue;
+	if (waypoints.empty()) {
+		// standard shortest path
+		route = pathfind::a_star_search(un->first, go_to, 10000.0, &calc, map_.w(), map_.h(), &allowed_teleports);
+	} else {
+		// initialize the main route with the first step
+		route.steps.push_back(un->first);
+		route.move_cost = 0;
 
-			allowed_teleports.insert(*i);
+		//copy waypoints and add first source and last destination
+		//TODO: don't copy but use vector index trick
+		std::vector<map_location> waypts;
+		waypts.push_back(un->first);
+		waypts.insert(waypts.end(), waypoints.begin(), waypoints.end());
+		waypts.push_back(go_to);
+
+		std::vector<map_location>::iterator src = waypts.begin(),
+			dst = ++waypts.begin();
+		for(; dst != waypts.end(); ++src,++dst){
+			if (*src == *dst) continue;
+			pathfind::plain_route inter_route = pathfind::a_star_search(*src, *dst, 10000.0, &calc, map_.w(), map_.h(), &allowed_teleports);
+			if(inter_route.steps.size()>=1) {
+				// add to the main route but skip the head (already in)
+				route.steps.insert(route.steps.end(),
+					inter_route.steps.begin()+1,inter_route.steps.end());
+				route.move_cost+=inter_route.move_cost;
+			} else {
+				// we can't reach dst, stop the route at the last src
+				// as the normal case do
+				break;
+			}
 		}
 	}
-	paths::route route = a_star_search(un->first, go_to, 10000.0, &calc, map_.w(), map_.h(), &allowed_teleports);
-	route_turns_to_complete(un->second, route, viewing_team(), units_,teams_,map_);
-	return route;
+
+	return mark_route(route, waypoints, un->second, viewing_team(), units_,teams_,map_);
 }
 
 void mouse_handler::mouse_press(const SDL_MouseButtonEvent& event, const bool browse)
@@ -332,7 +389,7 @@ void mouse_handler::mouse_press(const SDL_MouseButtonEvent& event, const bool br
 	mouse_handler_base::mouse_press(event, browse);
 }
 
-bool mouse_handler::right_click_show_menu(int /*x*/, int /*y*/, const bool browse)
+bool mouse_handler::right_click_show_menu(int x, int y, const bool browse)
 {
 	// The first right-click cancel the selection if any,
 	// the second open the context menu
@@ -340,7 +397,7 @@ bool mouse_handler::right_click_show_menu(int /*x*/, int /*y*/, const bool brows
 		select_hex(map_location(), browse);
 		return false;
 	} else {
-		return true;
+		return point_in_rect(x, y, gui().map_area());
 	}
 }
 
@@ -349,7 +406,7 @@ bool mouse_handler::left_click(int x, int y, const bool browse)
 	undo_ = false;
 	if (mouse_handler_base::left_click(x, y, browse)) return false;
 
-	bool check_shroud = teams_[team_num_ - 1].auto_shroud_updates();
+	bool check_shroud = current_team().auto_shroud_updates();
 
 	//we use the last registered highlighted hex
 	//since it's what update our global state
@@ -358,65 +415,96 @@ bool mouse_handler::left_click(int x, int y, const bool browse)
 	unit_map::iterator u = find_unit(selected_hex_);
 
 	//if the unit is selected and then itself clicked on,
-	//any goto command is cancelled
-	if(u != units_.end() && !browse && selected_hex_ == hex && u->second.side() == team_num_) {
+	//any goto command and waypoints are cancelled
+	if(u != units_.end() && !browse && selected_hex_ == hex && u->second.side() == side_num_) {
 		u->second.set_goto(map_location());
+		u->second.waypoints().clear();
+		waypoints_.clear();
 	}
 
 	unit_map::iterator clicked_u = find_unit(hex);
 
 	const map_location src = selected_hex_;
-	paths orig_paths = current_paths_;
+	pathfind::paths orig_paths = current_paths_;
 	const map_location& attack_from = current_unit_attacks_from(hex);
 
 	//see if we're trying to do a attack or move-and-attack
 	if(!browse && !commands_disabled && attack_from.valid()) {
 		if (attack_from == selected_hex_) { //no move needed
-			attack_enemy(u, clicked_u);
+			int choice = show_attack_dialog(attack_from, clicked_u->first);
+			if (choice >=0 ) {
+				attack_enemy(u, clicked_u, choice);
+			}
 			return false;
 		}
-		else if (move_unit_along_current_route(false, true)) {//move the unit without updating shroud
+		else {
+			// we will now temporary move next to the enemy
+			pathfind::paths::dest_vect::const_iterator itor =
+					current_paths_.destinations.find(attack_from);
+			if(itor == current_paths_.destinations.end()) {
+				// can't reach the attacking location
+				// not supposed to happen, so abort
+				return false;
+			}
+			// update movement_left as if we did the move
+			int move_left_dst = itor->move_left;
+			int move_left_src = u->second.movement_left();
+			u->second.set_movement(move_left_dst);
+
+			int choice = -1;
+			// block where we temporary move the unit
+			{
+				temporary_unit_mover temp_mover(units_, src, attack_from);
+				choice = show_attack_dialog(attack_from, clicked_u->first);
+			}
+			// restore unit as before
+			u = units_.find(src);
+			u->second.set_movement(move_left_src);
+			u->second.set_standing();
+
+			if (choice < 0) {
+				// user hit cancel, don't start move+attack
+				return false;
+			}
+
+			//register the mouse-UI waypoints into the unit's waypoints
+			u->second.waypoints() = waypoints_;
+
+			// move the unit without clearing fog (to avoid interruption)
+			//TODO: clear fog and interrupt+resume move
+			if(!move_unit_along_current_route(false, true)) {
+				// interrupted move
+				// we assume that move_unit() did the cleaning
+				// (update shroud/fog, clear undo if needed)
+				return false;
+			}
+
 			// a WML event could have invalidated both attacker and defender
 			// so make sure they're valid before attacking
 			u = find_unit(attack_from);
 			unit_map::iterator enemy = find_unit(hex);
-			if(u != units_.end() && u->second.side() == team_num_ &&
+			if(u != units_.end() && u->second.side() == side_num_ &&
 				enemy != units_.end() && current_team().is_enemy(enemy->second.side()) && !enemy->second.incapacitated()
 				&& !commands_disabled) {
 
-				// reselect the unit to make the attacker's stats appear during the attack dialog
-				gui().select_hex(attack_from);
-
-				if(attack_enemy(u,enemy)) { // Fight !!
-					return false;
-				} else { //canceled attack, undo the move
-					undo_ = true;
-					selected_hex_ = src;
-					gui().select_hex(src);
-					current_paths_ = orig_paths;
-					gui().highlight_reach(current_paths_);
-					return false;
-				}
-			}
-			else {  // the attack is not valid anymore, abort
+				attack_enemy(u, enemy, choice); // Fight !!
 				return false;
 			}
 
 		}
-		else { // interrupted move
-			// we assume that move_unit() did the cleaning
-			// (update shroud/fog, clear undo if needed)
-			return false;
-		}
 	}
 
 	//otherwise we're trying to move to a hex
 	else if(!commands_disabled && !browse && selected_hex_.valid() && selected_hex_ != hex &&
-		     u != units_.end() && u->second.side() == team_num_ &&
+		     u != units_.end() && u->second.side() == side_num_ &&
 		     clicked_u == units_.end() && !current_route_.steps.empty() &&
 		     current_route_.steps.front() == selected_hex_) {
 
 		gui().unhighlight_reach();
+
+		//register the mouse-UI waypoints into the unit's waypoints
+		u->second.waypoints() = waypoints_;
+
 		move_unit_along_current_route(check_shroud);
 		// during the move, we may have selected another unit
 		// (but without triggering a select event (command was disabled)
@@ -439,6 +527,7 @@ void mouse_handler::select_hex(const map_location& hex, const bool browse) {
 	gui().select_hex(hex);
 	gui().clear_attack_indicator();
 	gui().set_route(NULL);
+	waypoints_.clear();
 	show_partial_move_ = false;
 
 	unit_map::iterator u = find_unit(hex);
@@ -448,9 +537,9 @@ void mouse_handler::select_hex(const map_location& hex, const bool browse) {
 		{
 			// if it's not the unit's turn, we reset its moves
 			// and we restore them before the "select" event is raised
-			unit_movement_resetter move_reset(u->second, u->second.side() != team_num_);
-			bool teleport = u->second.get_ability_bool("teleport", u->first);
-			current_paths_ = paths(map_, units_, hex, teams_,
+			unit_movement_resetter move_reset(u->second, u->second.side() != side_num_);
+			bool teleport = u->second.get_ability_bool("teleport");
+			current_paths_ = pathfind::paths(map_, units_, hex, teams_,
 				false, teleport, viewing_team(), path_turns_);
 		}
 		show_attack_options(u);
@@ -461,15 +550,15 @@ void mouse_handler::select_hex(const map_location& hex, const bool browse) {
 		gui().set_route(NULL);
 
 		// selection have impact only if we are not observing and it's our unit
-		if (!browse && !commands_disabled && u->second.side() == gui().viewing_team()+1) {
+		if (!browse && !commands_disabled && u->second.side() == gui().viewing_side()) {
 			sound::play_UI_sound("select-unit.wav");
-			u->second.set_selecting(gui(), u->first);
+			u->second.set_selecting();
 			game_events::fire("select", hex);
 		}
 
 	} else {
 		gui().unhighlight_reach();
-		current_paths_ = paths();
+		current_paths_ = pathfind::paths();
 		current_route_.steps.clear();
 	}
 }
@@ -480,7 +569,7 @@ void mouse_handler::deselect_hex() {
 
 void mouse_handler::clear_undo_stack()
 {
-	apply_shroud_changes(undo_stack_,&gui(),map_,units_,teams_,team_num_-1);
+	apply_shroud_changes(undo_stack_, side_num_);
 	undo_stack_.clear();
 }
 
@@ -499,14 +588,13 @@ bool mouse_handler::move_unit_along_current_route(bool check_shroud, bool attack
 	gui().select_hex(map_location());
 
 	// will be invalid after the move
-	current_paths_ = paths();
+	current_paths_ = pathfind::paths();
 	current_route_.steps.clear();
 
 	attackmove_ = attackmove;
 	size_t moves = 0;
 	try{
-		moves = ::move_unit(&gui(),map_,units_,teams_,
-	                   steps,&recorder,&undo_stack_,&next_unit_,false,check_shroud);
+		moves = ::move_unit(NULL, steps, &recorder, &undo_stack_, true, &next_unit_, false, check_shroud);
 	} catch(end_turn_exception&) {
 		attackmove_ = false;
 		cursor::set(cursor::NORMAL);
@@ -545,40 +633,49 @@ bool mouse_handler::move_unit_along_current_route(bool check_shroud, bool attack
 	return moves == steps.size();
 }
 
-bool mouse_handler::attack_enemy(unit_map::iterator attacker, unit_map::iterator defender)
+
+int mouse_handler::fill_weapon_choices(std::vector<battle_context>& bc_vector, unit_map::iterator attacker, unit_map::iterator defender)
 {
-	try {
-		return attack_enemy_(attacker, defender);
-	} catch(std::bad_alloc) {
-		lg::wml_error << "Memory exhausted a unit has either a lot hitpoints or a negative amount.\n";
-		return false;
-	}
-
-}
-
-bool mouse_handler::attack_enemy_(unit_map::iterator attacker, unit_map::iterator defender)
-{
-	//we must get locations by value instead of by references, because the iterators
-	//may become invalidated later
-	const map_location attacker_loc = attacker->first;
-	const map_location defender_loc = defender->first;
-
-	std::vector<std::string> items;
-
-	std::vector<battle_context> bc_vector;
-	unsigned int i, best = 0;
-	for (i = 0; i < attacker->second.attacks().size(); i++) {
+	int best = 0;
+	for (unsigned int i = 0; i < attacker->second.attacks().size(); i++) {
 		// skip weapons with attack_weight=0
 		if (attacker->second.attacks()[i].attack_weight() > 0) {
-			battle_context bc(map_, teams_, units_, status_, attacker->first, defender->first, i);
+			battle_context bc(units_, attacker->first, defender->first, i);
 			bc_vector.push_back(bc);
 			if (bc.better_attack(bc_vector[best], 0.5)) {
 				best = i;
 			}
 		}
 	}
+	return best;
+}
 
-	for (i = 0; i < bc_vector.size(); i++) {
+int mouse_handler::show_attack_dialog(const map_location& attacker_loc, const map_location& defender_loc)
+{
+	unit_map::iterator attacker = units_.find(attacker_loc);
+	unit_map::iterator defender = units_.find(defender_loc);
+
+	std::vector<battle_context> bc_vector;
+	int best = fill_weapon_choices(bc_vector, attacker, defender);
+
+	if (bc_vector.empty())
+	{
+		dialogs::units_list_preview_pane attacker_preview(attacker->second, dialogs::unit_preview_pane::SHOW_BASIC, true);
+		dialogs::units_list_preview_pane defender_preview(defender->second, dialogs::unit_preview_pane::SHOW_BASIC, false);
+		std::vector<gui::preview_pane*> preview_panes;
+		preview_panes.push_back(&attacker_preview);
+		preview_panes.push_back(&defender_preview);
+
+		gui::show_dialog(gui(), NULL, _("Attack Enemy"),
+			_("No usable weapon"), gui::CANCEL_ONLY, NULL,
+			&preview_panes, "", NULL, -1, NULL, -1, -1, NULL, NULL);
+		return -1;
+	}
+
+
+	std::vector<std::string> items;
+
+	for (unsigned int i = 0; i < bc_vector.size(); i++) {
 		const battle_context::unit_stats& att = bc_vector[i].get_attacker_stats();
 		const battle_context::unit_stats& def = bc_vector[i].get_defender_stats();
 		config tmp_config;
@@ -589,14 +686,17 @@ bool mouse_handler::attack_enemy_(unit_map::iterator attacker, unit_map::iterato
 		attw.set_specials_context(attacker->first, defender->first, attacker->second, true);
 		defw.set_specials_context(attacker->first, defender->first, attacker->second, false);
 
-		//if there is an attack special or defend special, we output a single space for the other unit, to make sure
-		//that the attacks line up nicely.
-		std::string special_pad = "";
-		if (!attw.weapon_specials().empty() || !defw.weapon_specials().empty())
-			special_pad = " ";
+		// if missing, add dummy special, to be sure to have
+		// big enough mimimum width (weapon's name can be very short)
+		std::string att_weapon_special = attw.weapon_specials();
+		if (att_weapon_special.empty())
+			att_weapon_special += "       ";
+		std::string def_weapon_special = defw.weapon_specials();
+		if (def_weapon_special.empty())
+			def_weapon_special += "       ";
 
 		std::stringstream atts;
-		if (i == best) {
+		if (static_cast<int>(i) == best) {
 			atts << DEFAULT_ITEM;
 		}
 
@@ -604,34 +704,44 @@ bool mouse_handler::attack_enemy_(unit_map::iterator attacker, unit_map::iterato
 		if (!range.empty()) {
 			range = gettext(range.c_str());
 		}
+
+		// add dummy names if missing, to keep stats aligned
+		std::string attw_name = attw.name();
+		if(attw_name.empty())
+			attw_name = " ";
+		std::string defw_name = defw.name();
+		if(defw_name.empty())
+			defw_name = " ";
+
+		// color CtH in red-yellow-green
+		SDL_Color att_cth_color =
+				int_to_color( game_config::red_to_green(att.chance_to_hit) );
+		SDL_Color def_cth_color =
+				int_to_color( game_config::red_to_green(def.chance_to_hit) );
+
 		atts << IMAGE_PREFIX << attw.icon() << COLUMN_SEPARATOR
-			 << font::BOLD_TEXT << attw.name() << "\n" << att.damage << "-"
-			 << att.num_blows << " "  << " (" << att.chance_to_hit << "%)\n"
-			 << attw.weapon_specials() << special_pad
-			 << COLUMN_SEPARATOR << "<245,230,193>" << "- " << range << " -" << COLUMN_SEPARATOR
-			 << font::BOLD_TEXT << defw.name() << "\n" << def.damage << "-"
-			 << def.num_blows << " "  << " (" << def.chance_to_hit << "%)\n"
-			 << defw.weapon_specials() << special_pad << COLUMN_SEPARATOR
-			 << IMAGE_PREFIX << defw.icon();
+			 << font::BOLD_TEXT << attw_name  << "\n"
+			 << att.damage << "-" << att.num_blows
+			 << "  " << att_weapon_special << "\n"
+			 << font::color2markup(att_cth_color) << att.chance_to_hit << "%"
+			 << COLUMN_SEPARATOR << font::weapon_details << "- " << range << " -" << COLUMN_SEPARATOR
+			 << font::BOLD_TEXT << defw_name  << "\n"
+			 << def.damage << "-" << def.num_blows
+			 << "  " << def_weapon_special << "\n"
+			 << font::color2markup(def_cth_color) << def.chance_to_hit << "%"
+			 << COLUMN_SEPARATOR << IMAGE_PREFIX << defw.icon();
 
 		items.push_back(atts.str());
 	}
 
-	//make it so that when we attack an enemy, the attacking unit
-	//is again shown in the status bar, so that we can easily
-	//compare between the attacking and defending unit
-	gui().highlight_hex(map_location());
-	gui().draw(true,true);
-
-	attack_prediction_displayer ap_displayer(gui(), bc_vector, map_, teams_, units_, status_, attacker_loc, defender_loc);
+	attack_prediction_displayer ap_displayer(bc_vector, attacker_loc, defender_loc);
 	std::vector<gui::dialog_button_info> buttons;
 	buttons.push_back(gui::dialog_button_info(&ap_displayer, _("Damage Calculations")));
 
 	int res = 0;
-
 	{
-		dialogs::units_list_preview_pane attacker_preview(gui(),&map_,attacker->second,dialogs::unit_preview_pane::SHOW_BASIC,true);
-		dialogs::units_list_preview_pane defender_preview(gui(),&map_,defender->second,dialogs::unit_preview_pane::SHOW_BASIC,false);
+		dialogs::units_list_preview_pane attacker_preview(attacker->second, dialogs::unit_preview_pane::SHOW_BASIC, true);
+		dialogs::units_list_preview_pane defender_preview(defender->second, dialogs::unit_preview_pane::SHOW_BASIC, false);
 		std::vector<gui::preview_pane*> preview_panes;
 		preview_panes.push_back(&attacker_preview);
 		preview_panes.push_back(&defender_preview);
@@ -641,70 +751,136 @@ bool mouse_handler::attack_enemy_(unit_map::iterator attacker, unit_map::iterato
 				gui::OK_CANCEL,&items,&preview_panes,"",NULL,-1,NULL,-1,-1,
 				NULL,&buttons);
 	}
-
 	cursor::set(cursor::NORMAL);
-	if(size_t(res) < bc_vector.size()) {
-		const battle_context::unit_stats &att = bc_vector[res].get_attacker_stats();
-		const battle_context::unit_stats &def = bc_vector[res].get_defender_stats();
 
-		attacker->second.set_goto(map_location());
-		clear_undo_stack();
-		redo_stack_.clear();
+	return res;
+}
 
-		current_paths_ = paths();
-		gui().clear_attack_indicator();
-		gui().unhighlight_reach();
-
-		gui().draw();
-
-		recorder.add_attack(attacker_loc,defender_loc,att.attack_num,def.attack_num);
-
-		//MP_COUNTDOWN grant time bonus for attacking
-		current_team().set_action_bonus_count(1 + current_team().action_bonus_count());
-
-		try {
-			attack(gui(),map_,teams_,attacker_loc,defender_loc,att.attack_num,def.attack_num,units_,status_);
-		} catch(end_level_exception&) {
-			//if the level ends due to a unit being killed, still see if
-			//either the attacker or defender should advance
-			dialogs::advance_unit(map_,units_,attacker_loc,gui());
-			unit_map::const_iterator defu = units_.find(defender_loc);
-			if (defu != units_.end()) {
-				bool defender_human = teams_[defu->second.side()-1].is_human();
-				dialogs::advance_unit(map_,units_,defender_loc,gui(),!defender_human);
-			}
-			throw;
-		}
-
-		dialogs::advance_unit(map_,units_,attacker_loc,gui());
-		unit_map::const_iterator defu = units_.find(defender_loc);
-		if (defu != units_.end()) {
-			bool defender_human = teams_[defu->second.side()-1].is_human();
-			dialogs::advance_unit(map_,units_,defender_loc,gui(),!defender_human);
-		}
-
-		check_victory(units_, teams_, gui());
-
-		gui().draw();
-
-		return true;
-	} else {
-		return false;
+void mouse_handler::attack_enemy(unit_map::iterator attacker, unit_map::iterator defender, int choice)
+{
+	try {
+		attack_enemy_(attacker, defender, choice);
+	} catch(std::bad_alloc) {
+		lg::wml_error << "Memory exhausted a unit has either a lot hitpoints or a negative amount.\n";
 	}
 }
 
-void mouse_handler::show_attack_options(unit_map::const_iterator u)
+void mouse_handler::attack_enemy_(unit_map::iterator attacker, unit_map::iterator defender, int choice)
 {
-	team& current_team = teams_[team_num_-1];
+	//we must get locations by value instead of by references, because the iterators
+	//may become invalidated later
+	const map_location attacker_loc = attacker->first;
+	const map_location defender_loc = defender->first;
 
-	if(u == units_.end() || u->second.attacks_left() == 0)
+	commands_disabled++;
+
+	attacker->second.set_goto(map_location());
+	//This triggers a shroud update which could fire a sighted event
+	//invalidating our iterators (or do more changes)
+	clear_undo_stack();
+	redo_stack_.clear();
+
+	// refresh iterators
+	attacker = units_.find(attacker_loc);
+	defender = units_.find(defender_loc);
+
+	if(attacker == units_.end() || attacker->second.incapacitated()
+			|| attacker->second.side() != side_num_) {
+		return;
+	}
+	if(defender == units_.end() || defender->second.incapacitated()
+			|| current_team().is_enemy(defender->second.side()) == false) {
+		return;
+	}
+
+	std::vector<battle_context> bc_vector;
+	fill_weapon_choices(bc_vector, attacker, defender);
+
+	if(size_t(choice) >= bc_vector.size()) {
+		return;
+	}
+
+	const battle_context::unit_stats &att = bc_vector[choice].get_attacker_stats();
+	const battle_context::unit_stats &def = bc_vector[choice].get_defender_stats();
+
+	current_paths_ = pathfind::paths();
+	// make the attacker's stats appear during the attack
+	gui().display_unit_hex(attacker_loc);
+	// remove highlighted hexes etc..
+	gui().select_hex(map_location());
+	gui().highlight_hex(map_location());
+	gui().clear_attack_indicator();
+	gui().unhighlight_reach();
+	gui().draw();
+
+	//@TODO: change ToD to be location specific for the defender
+	recorder.add_attack(attacker_loc, defender_loc, att.attack_num, def.attack_num,
+		attacker->second.type_id(), defender->second.type_id(), att.level,
+		def.level, resources::tod_manager->turn(), resources::tod_manager->get_time_of_day());
+	rand_rng::invalidate_seed();
+	if (rand_rng::has_valid_seed()) { //means SRNG is disabled
+		perform_attack(attacker_loc, defender_loc, att.attack_num, def.attack_num, rand_rng::get_last_seed());
+	} else {
+		rand_rng::set_new_seed_callback(boost::bind(&mouse_handler::perform_attack,
+			this, attacker_loc, defender_loc, att.attack_num, def.attack_num, _1));
+	}
+}
+
+void mouse_handler::perform_attack(
+	map_location attacker_loc, map_location defender_loc,
+	int attacker_weapon, int defender_weapon, rand_rng::seed_t seed)
+{
+	// this function gets it's arguments by value because the calling function
+	// object might get deleted in the clear callback call below, invalidating
+	// const ref arguments
+	rand_rng::clear_new_seed_callback();
+	LOG_NG << "Performing attack with seed " << seed << "\n";
+	recorder.add_seed("attack", seed);
+	//MP_COUNTDOWN grant time bonus for attacking
+	current_team().set_action_bonus_count(1 + current_team().action_bonus_count());
+
+	try {
+		events::command_disabler disabler; // Rather than decrementing for every possible exception, use RAII
+		commands_disabled--;
+		attack_unit(attacker_loc, defender_loc, attacker_weapon, defender_weapon);
+	} catch(end_level_exception&) {
+		//if the level ends due to a unit being killed, still see if
+		//either the attacker or defender should advance
+		dialogs::advance_unit(attacker_loc);
+		unit_map::const_iterator defu = units_.find(defender_loc);
+		if (defu != units_.end()) {
+			bool defender_human = teams_[defu->second.side()-1].is_human();
+			dialogs::advance_unit(defender_loc, !defender_human);
+		}
+		throw;
+	}
+
+	dialogs::advance_unit(attacker_loc);
+	unit_map::const_iterator defu = units_.find(defender_loc);
+	if (defu != units_.end()) {
+		bool defender_human = teams_[defu->second.side()-1].is_human();
+		dialogs::advance_unit(defender_loc, !defender_human);
+	}
+
+	resources::controller->check_victory();
+	gui().draw();
+}
+
+void mouse_handler::show_attack_options(const unit_map::const_iterator &u)
+{
+	if (u == units_.end() || u->second.attacks_left() == 0)
 		return;
 
-	for(unit_map::const_iterator target = units_.begin(); target != units_.end(); ++target) {
-		if(current_team.is_enemy(target->second.side()) &&
-			distance_between(target->first,u->first) == 1 && !target->second.incapacitated()) {
-			current_paths_.routes[target->first] = paths::route();
-		}
+	map_location adj[6];
+	get_adjacent_tiles(u->first, adj);
+	foreach (const map_location &loc, adj)
+	{
+		if (!map_.on_board(loc)) continue;
+		unit_map::const_iterator i = units_.find(loc);
+		if (i == units_.end()) continue;
+		const unit &target = i->second;
+		if (current_team().is_enemy(target.side()) && !target.incapacitated())
+			current_paths_.destinations.insert(loc);
 	}
 }
 
@@ -713,8 +889,8 @@ bool mouse_handler::unit_in_cycle(unit_map::const_iterator it)
 	if (it == units_.end())
 		return false;
 
-	if(it->second.side() != team_num_ || it->second.user_end_turn()
-			|| gui().fogged(it->first) || !unit_can_move(it->first,it->second,units_,map_,teams_))
+	if (it->second.side() != side_num_ || it->second.user_end_turn()
+	    || gui().fogged(it->first) || !unit_can_move(it->second))
 		return false;
 
 	if (current_team().is_enemy(int(gui().viewing_team()+1)) &&
@@ -759,7 +935,7 @@ void mouse_handler::cycle_units(const bool browse, const bool reverse)
 	}
 }
 
-void mouse_handler::set_current_paths(paths new_paths) {
+void mouse_handler::set_current_paths(pathfind::paths new_paths) {
 	gui().unhighlight_reach();
 	current_paths_ = new_paths;
 	current_route_.steps.clear();
