@@ -1,6 +1,6 @@
-/* $Id: mouse_handler_base.cpp 34574 2009-04-05 23:37:21Z alink $ */
+/* $Id: mouse_handler_base.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -16,9 +16,12 @@
 #include "mouse_handler_base.hpp"
 
 #include "cursor.hpp"
+#include "display.hpp"
 #include "log.hpp"
 #include "preferences.hpp"
 
+static lg::log_domain log_display("display");
+#define WRN_DP LOG_STREAM(warn, log_display)
 
 namespace events {
 
@@ -107,7 +110,8 @@ bool mouse_handler_base::mouse_motion_default(int x, int y, bool /*update*/)
 	if (is_dragging() && !dragging_started_) {
 		if ((dragging_left_ && (SDL_GetMouseState(&mx,&my) & SDL_BUTTON_LEFT) != 0)
 		|| (dragging_right_ && (SDL_GetMouseState(&mx,&my) & SDL_BUTTON_RIGHT) != 0)) {
-			const double drag_distance = std::pow((double) (drag_from_x_- mx), 2) + std::pow((double) (drag_from_y_- my), 2);
+			const double drag_distance = std::pow(static_cast<double>(drag_from_x_- mx), 2)
+					+ std::pow(static_cast<double>(drag_from_y_- my), 2);
 			if (drag_distance > drag_threshold()*drag_threshold()) {
 				dragging_started_ = true;
 				cursor::set_dragging(true);
@@ -251,7 +255,7 @@ bool mouse_handler_base::right_click(int x, int y, const bool browse)
 		if (m != NULL) {
 			show_menu_ = true;
 		} else {
-			LOG_STREAM(warn, display) << "no context menu found...\n";
+			WRN_DP << "no context menu found...\n";
 		}
 		return true;
 	}
@@ -260,6 +264,8 @@ bool mouse_handler_base::right_click(int x, int y, const bool browse)
 
 void mouse_handler_base::right_drag_end(int /*x*/, int /*y*/, const bool /*browse*/)
 {
+	//FIXME: This is called when we select an option in context-menu,
+	//       which is bad because that was not a real dragging
 }
 
 void mouse_handler_base::right_mouse_up(int /*x*/, int /*y*/, const bool /*browse*/)
