@@ -1,6 +1,6 @@
-/* $Id: multiplayer.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: multiplayer.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2005 - 2008 Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2010 Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,6 @@
 
 class config;
 class game_display;
-class game_data;
 
 namespace mp {
 
@@ -29,26 +28,24 @@ const size_t max_login_size = 18;
  * This is the main entry points of multiplayer mode.
  */
 
-/** Starts a multiplayer game in server mode, or in single-user mode.
+/** Starts a multiplayer game in single-user mode.
  *
  * @param disp        The global display
  * @param game_config The global, top-level WML configuration for the game
- * @param data        The global game data (unit types, etc)
  * @param default_controller The default controller type
- * @param is_server   Whether to open a port to the outside, or not.
  */
-void start_server(game_display& disp, const config& game_config, game_data& data,
-		mp::controller default_controller, bool is_server);
+void start_local_game(game_display& disp, const config& game_config,
+		mp::controller default_controller);
 
 /** Starts a multiplayer game in client mode.
  *
  * @param disp        The global display
  * @param game_config The global, top-level WML configuration for the game
- * @param data        The global game data (unit types, etc)
  * @param host        The host to connect to.
  */
-void start_client(game_display& disp, const config& game_config, game_data& data,
-		const std::string host);
+void start_client(game_display& disp, const config& game_config,
+		const std::string& host);
+
 
 }
 #endif
