@@ -1,6 +1,6 @@
-/* $Id: multiplayer_connect.hpp 32245 2009-01-21 00:30:01Z shadowmaster $ */
+/* $Id: multiplayer_connect.hpp 41481 2010-03-06 21:36:42Z mordante $ */
 /*
-   Copyright (C) 2007 - 2009
+   Copyright (C) 2007 - 2010
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -21,13 +21,17 @@
 #include "gamestatus.hpp"
 #include "leader_list.hpp"
 #include "multiplayer_ui.hpp"
-#include "multiplayer_create.hpp"
 #include "network.hpp"
 #include "widgets/scrollpane.hpp"
+#include "widgets/slider.hpp"
 #include "widgets/combo_drag.hpp"
 
 #include <string>
 
+namespace ai {
+	struct description;
+}
+
 namespace mp {
 
 class connect : public mp::ui
@@ -76,6 +80,9 @@ public:
 		 */
 		bool available(const std::string& name = "") const;
 
+		/** Returns true, if the player has chosen his leader and this side is ready for the game to start */
+		bool ready_for_start() const;
+
 		/** Return true if players are allowed to take this side. */
 		bool allow_player() const;
 
@@ -90,11 +97,13 @@ public:
 		const std::string& get_current_player() const
 			{ return current_player_; }
 
-		const std::string& get_id() const;
-		bool is_owned_by(const std::string& name) const;
+		const std::string& get_player_id() const;
 
 		/** Sets the username of this side. */
-		void set_id(const std::string& id);
+		void set_player_id(const std::string& player_id);
+
+		/** Sets if the joining player has chosen his leader. */
+		void set_ready_for_start(bool ready_for_start);
 
 		const std::string& get_save_id() const;
 
@@ -136,6 +145,7 @@ public:
 		// Configurable variables
 		int index_;
 		std::string id_;
+		std::string player_id_;
 		std::string save_id_;
 		std::string current_player_;
 		mp::controller controller_;
@@ -147,7 +157,13 @@ public:
 		std::string leader_;
 		std::string gender_;
 		std::string ai_algorithm_;
-		//bool taken_;
+		bool ready_for_start_;
+
+		// Flags for controlling the dialog widgets of the game lobby
+		bool gold_lock_;
+		bool income_lock_;
+		bool team_lock_;
+		bool colour_lock_;
 
 		// Widgets for this side
 		gui::label player_number_;
@@ -165,6 +181,7 @@ public:
 		gui::label label_income_;
 
 		bool allow_player_;
+		bool allow_changes_;
 		bool enabled_;
 		bool changed_;
 		leader_list_manager llm_;
@@ -175,8 +192,8 @@ public:
 	typedef std::vector<side> side_list;
 
 
-	connect(game_display& disp, const config& game_config,
-			chat& c, config& gamelist, const create::parameters& params,
+	connect(game_display& disp, const config& game_config, chat& c,
+			config& gamelist, const mp_game_settings& params, const int num_turns,
 			mp::controller default_controller, bool local_players_only = false);
 
 	virtual void process_event();
@@ -226,7 +243,17 @@ private:
 	void update_and_send_diff(bool update_time_of_day = false);
 
 	/** Returns true if there still are sides available for this game. */
-	bool sides_available();
+	bool sides_available() const;
+
+	/** Returns true if all sides are ready to start the game. */
+	bool sides_ready() const;
+
+	/**
+	 * Validates whether the game can be started.
+	 *
+	 * returns                       Can the game be started?
+	 */
+	bool can_start_game() const;
 
 	/**
 	 * Updates the state of the player list, the launch button and of the start
@@ -243,9 +270,6 @@ private:
 	/** Adds a player. */
 	void update_user_combos();
 
-	/** Removes a player and kicks it from the game. */
-	void kick_player(const std::string& name);
-
 	bool local_only_;
 
 	config level_;
@@ -253,17 +277,18 @@ private:
 	/** This is the "game state" object which is created by this dialog. */
 	game_state state_;
 
-	create::parameters params_;
+	mp_game_settings params_;
+	int num_turns_;
 
 	/** The list of available sides for the current era. */
-	config::child_list era_sides_;
+	std::vector<const config *> era_sides_;
 
 	// Lists used for combos
 	std::vector<std::string> player_types_;
 	std::vector<std::string> player_factions_;
 	std::vector<std::string> player_teams_;
 	std::vector<std::string> player_colours_;
-	std::vector<std::string> ai_algorithms_;
+	std::vector<ai::description*> ai_algorithms_;
 
 	// team_name list and "Team" prefix
 	std::vector<std::string> team_names_;
