@@ -1,6 +1,6 @@
-/* $Id: multiplayer_create.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: multiplayer_create.hpp 40817 2010-01-23 19:43:28Z mordante $ */
 /*
-   Copyright (C) 2007 - 2008
+   Copyright (C) 2007 - 2010
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -12,12 +12,12 @@
    See the COPYING file for more details.
 */
 
-//! @file multiplayer_create.hpp
-//!
+/** @file multiplayer_create.hpp */
 
 #ifndef MULTIPLAYER_CREATE_HPP_INCLUDED
 #define MULTIPLAYER_CREATE_HPP_INCLUDED
 
+#include "mp_game_settings.hpp"
 #include "multiplayer_ui.hpp"
 #include "widgets/slider.hpp"
 #include "widgets/label.hpp"
@@ -30,81 +30,11 @@ namespace mp {
 class create : public mp::ui
 {
 public:
-	struct parameters
-	{
-		parameters() :
-			name(),
-			era(),
-			num_turns(0),
-			village_gold(0),
-			xp_modifier(0),
-			mp_countdown_init_time(0),
-			mp_countdown_reservoir_time(0),
-			mp_countdown_turn_bonus(0),
-			mp_countdown_action_bonus(0),
-			mp_countdown(false),
-			use_map_settings(false),
-			random_start_time(false),
-			fog_game(false),
-			shroud_game(false),
-			allow_observers(false),
-			share_view(false),
-			share_maps(false),
-			saved_game(false),
-			scenario_data()
-
-		{ reset(); };
-
-		void reset() {
-			name = "";
-			era = "";
-			password = "";
-			num_turns = 0;
-			village_gold = 0;
-			xp_modifier = 0;
-			mp_countdown_init_time=0;
-			mp_countdown_reservoir_time=0;
-			mp_countdown_turn_bonus=0;
-			mp_countdown_action_bonus=0;
-			mp_countdown=false;
-			use_map_settings = random_start_time = fog_game = shroud_game = allow_observers = share_view = share_maps = false;
-
-			scenario_data.clear();
-		}
-
-		// The items returned while configuring the game
-
-		std::string name;
-		std::string era;
-		std::string password;
-
-		int num_turns;
-		int village_gold;
-		int xp_modifier;
-		int mp_countdown_init_time;
-		int mp_countdown_reservoir_time;
-		int mp_countdown_turn_bonus;
-		int mp_countdown_action_bonus;
-		bool mp_countdown;
-		bool use_map_settings;
-		bool random_start_time;
-		bool fog_game;
-		bool shroud_game;
-		bool allow_observers;
-		bool share_view;
-		bool share_maps;
-
-		bool saved_game;
-
-		//! If the game is to be randomly generated, the map generator
-		//! will create the scenario data in this variable
-		config scenario_data;
-	};
-
 	create(game_display& dist, const config& game_config, chat& c, config& gamelist);
 	~create();
 
-	parameters& get_parameters();
+	mp_game_settings& get_parameters();
+	int num_turns() const { return num_turns_; }
 
 protected:
 	virtual void layout_children(const SDL_Rect& rect);
@@ -112,7 +42,7 @@ protected:
 	virtual void hide_children(bool hide=true);
 
 private:
-	void update_minimap(void);
+	void update_minimap();
 
 	tooltips::manager tooltip_manager_;
 	int map_selection_;
@@ -122,8 +52,11 @@ private:
 
 	std::vector<std::string> user_maps_;
 	std::vector<std::string> map_options_;
-	//! Due to maps not available the index of the selected map and 
-	//! mp scenarios is not 1:1 so we use a lookup table.
+
+	/**
+	 * Due to maps not available the index of the selected map and mp scenarios
+	 * is not 1:1 so we use a lookup table.
+	 */
 	std::vector<size_t> map_index_;
 
 	gui::menu maps_menu_;
@@ -169,7 +102,8 @@ private:
 
 	util::scoped_ptr<map_generator> generator_;
 
-	parameters parameters_;
+	int num_turns_;
+	mp_game_settings parameters_;
 };
 
 } // end namespace mp
