@@ -1,6 +1,6 @@
-/* $Id: multiplayer_lobby.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id$ */
 /*
-   Copyright (C) 2007 - 2008
+   Copyright (C) 2007 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -12,8 +12,7 @@
    See the COPYING file for more details.
 */
 
-//! @file multiplayer_lobby.hpp
-//!
+/** @file multiplayer_lobby.hpp */
 
 #ifndef MULTIPLAYER_LOBBY_HPP_INCLUDED
 #define MULTIPLAYER_LOBBY_HPP_INCLUDED
@@ -26,13 +25,43 @@ class config;
 class video;
 class game_display;
 
-//! This module controls the multiplayer lobby. 
-//! A section on the server which allows players to chat, 
-//! create games, and join games.
+/**
+ * This module controls the multiplayer lobby.
+ *
+ * A section on the server which allows players to chat, create games, and join
+ * games.
+ */
 namespace mp {
 class gamebrowser : public gui::menu {
 public:
 	struct game_item {
+
+		game_item() :
+			mini_map(),
+			id(),
+			map_data(),
+			name(),
+			map_info(),
+			map_info_size(),
+			gold(),
+			xp(),
+			vision(),
+			status(),
+			time_limit(),
+			vacant_slots(0),
+			current_turn(0),
+			reloaded(false),
+			started(false),
+			fog(false),
+			shroud(false),
+			observers(false),
+			use_map_settings(false),
+			verified(false),
+			password_required(false),
+			have_era(false)
+		{
+		}
+
 		surface mini_map;
 		std::string id;
 		std::string map_data;
@@ -54,8 +83,9 @@ public:
 		bool use_map_settings;
 		bool verified;
 		bool password_required;
+		bool have_era;
 	};
-	gamebrowser(CVideo& video,const config* map_hashes);
+	gamebrowser(CVideo& video, const config &map_hashes);
 	void scroll(unsigned int pos);
 	void handle_event(const SDL_Event& event);
 	void set_inner_location(const SDL_Rect& rect);
@@ -67,12 +97,15 @@ public:
 	SDL_Rect get_item_rect(size_t index) const;
 	bool empty() const { return games_.empty(); }
 	bool selection_is_joinable() const
-	{ return empty() ? false : (games_[selected_].vacant_slots > 0 && !games_[selected_].started); }
-	bool selection_is_observable() const { return empty() ? false : games_[selected_].observers; }
+	{ return empty() ? false : (games_[selected_].vacant_slots > 0 && !games_[selected_].started && games_[selected_].have_era); }
+	// Moderators may observe any game.
+	bool selection_is_observable() const { return empty() ? false : (games_[selected_].observers && games_[selected_].have_era) || preferences::is_authenticated(); }
 	bool selected() const { return double_clicked_ && !empty(); }
 	void reset_selection() { double_clicked_ = false; }
 	int selection() const { return selected_; }
 	game_item selected_game() { return games_[selected_]; }
+	void select_game(const std::string& id);
+	bool game_matches_filter(const game_item& i, const config& cfg);
 protected:
 	unsigned int row_height() const { return item_height_ + (2 * style_->get_thickness()); }
 private:
@@ -83,7 +116,7 @@ private:
 	image::locator observer_icon_locator_;
 	image::locator no_observer_icon_locator_;
 
-	const config* map_hashes_;
+	const config &map_hashes_;
 
 	unsigned int item_height_;
 	int margin_;
@@ -139,12 +172,23 @@ private:
 #endif
 	gui::button quit_game_;
 
+#ifndef USE_TINY_GUI
+	gui::button apply_filter_;
+	gui::button invert_filter_;
+	gui::button vacant_slots_;
+	gui::button friends_in_game_;
+	gui::label filter_label_;
+	gui::textbox filter_text_;
+#endif
+
 	int last_selected_game_;
 
 	lobby_sorter sorter_;
 	gamebrowser games_menu_;
 
 	std::map<std::string,std::string> minimaps_;
+
+	std::string search_string_;
 };
 
 } // end namespace mp
