@@ -1,6 +1,6 @@
-/* $Id: multiplayer_ui.cpp 37533 2009-08-07 02:24:48Z soliton $ */
+/* $Id: multiplayer_ui.cpp 42339 2010-05-01 09:40:42Z ilor $ */
 /*
-   Copyright (C) 2005 - 2009
+   Copyright (C) 2005 - 2010
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -15,27 +15,37 @@
 #include "global.hpp"
 
 #include "construct_dialog.hpp"
+#include "foreach.hpp"
+#include "gamestatus.hpp"
+#include "game_display.hpp"
 #include "gettext.hpp"
 #include "gui/dialogs/mp_cmd_wrapper.hpp"
+#include "lobby_preferences.hpp"
 #include "log.hpp"
 #include "marked-up_text.hpp"
+#include "menu_events.hpp"
 #include "multiplayer.hpp"
 #include "multiplayer_ui.hpp"
 #include "sound.hpp"
 #include "replay.hpp"
 #include "wml_separators.hpp"
 
-#define LOG_NG LOG_STREAM(info, engine)
-#define ERR_NG LOG_STREAM(err, engine)
-#define ERR_CF LOG_STREAM(err, config)
-#define DBG_NW LOG_STREAM(debug, network)
-#define LOG_NW LOG_STREAM(info, network)
-#define ERR_NW LOG_STREAM(err, network)
+static lg::log_domain log_engine("engine");
+#define LOG_NG LOG_STREAM(info, log_engine)
+#define ERR_NG LOG_STREAM(err, log_engine)
 
-#define MAX_MESSAGES 256
+static lg::log_domain log_config("config");
+#define ERR_CF LOG_STREAM(err, log_config)
+
+static lg::log_domain log_network("network");
+#define LOG_NW LOG_STREAM(info, log_network)
+#define ERR_NW LOG_STREAM(err, log_network)
 
 namespace {
 
+	/** The maximum number of messages in the chat history. */
+	const size_t max_messages = 256;
+
 	class user_menu_style : public gui::menu::imgsel_style {
 	public:
 		user_menu_style() : gui::menu::imgsel_style("misc/selection", false,
@@ -70,9 +80,8 @@ void check_response(network::connection res, const config& data)
 		throw network::error(_("Connection timed out"));
 	}
 
-	const config* err = data.child("error");
-	if(err != NULL) {
-		throw network::error((*err)["message"]);
+	if (const config &err = data.child("error")) {
+		throw network::error(err["message"]);
 	}
 }
 
@@ -80,12 +89,12 @@ void level_to_gamestate(config& level, game_state& state)
 {
 	//any replay data is only temporary and should be removed from
 	//the level data in case we want to save the game later
-	config * const replay_data = level.child("replay");
+	const config &replay_data = level.child("replay");
 	config replay_data_store;
-	if(replay_data != NULL) {
-		replay_data_store = *replay_data;
+	if (replay_data) {
+		replay_data_store = replay_data;
 		LOG_NW << "setting replay\n";
-		state.replay_data = *replay_data;
+		state.replay_data = replay_data;
 		recorder = replay(replay_data_store);
 		if(!recorder.empty()) {
 			recorder.set_skip(false);
@@ -104,27 +113,29 @@ void level_to_gamestate(config& level, game_state& state)
 	}
 
 	//adds the starting pos to the level
-	if(level.child("replay_start") == NULL){
+	if (!level.child("replay_start")) {
 		level.add_child("replay_start", level);
+		level.child("replay_start").remove_child("multiplayer", 0);
 	}
 	//this is important, if it does not happen, the starting position is missing and
 	//will be drawn from the snapshot instead (which is not what we want since we have
 	//all needed information here already)
-	state.starting_pos = *(level.child("replay_start"));
+	state.starting_pos = level.child("replay_start");
 
 	level["campaign_type"] = "multiplayer";
-	state.campaign_type = "multiplayer";
-	state.completion = level["completion"];
-	state.version = level["version"];
+	state.classification().campaign_type = "multiplayer";
+	state.classification().completion = level["completion"];
+	state.classification().version = level["version"];
 
-	const config* const vars = level.child("variables");
-	if(vars != NULL) {
-		state.set_variables(*vars);
+	if (const config &vars = level.child("variables")) {
+		state.set_variables(vars);
 	}
-	state.set_menu_items(level.get_children("menu_item"));
+	state.set_menu_items(level.child_range("menu_item"));
+	state.mp_settings().set_from_config(level);
 
 	//Check whether it is a save-game by looking for snapshot data
-	const bool saved_game = (level.child("snapshot") && level.child("snapshot")->child("side"));
+	const config &snapshot = level.child("snapshot");
+	const bool saved_game = snapshot && snapshot.child("side");
 
 	//It might be a MP campaign start-of-scenario save
 	//In this case, it's not entirely a new game, but not a save, either
@@ -133,38 +144,34 @@ void level_to_gamestate(config& level, game_state& state)
 
 	//If we start a fresh game, there won't be any snapshot information. If however this
 	//is a savegame, we got a valid snapshot here.
-	if (saved_game){
-		state.snapshot = *(level.child("snapshot"));
-		if (state.snapshot.child("variables") != NULL){
-			state.set_variables(*state.snapshot.child("variables"));
+	if (saved_game) {
+		state.snapshot = snapshot;
+		if (const config &v = snapshot.child("variables")) {
+			state.set_variables(v);
 		}
-		state.set_menu_items(state.snapshot.get_children("menu_item"));
-	}
-
-	//If it is a start-of-scenario save, we need to load the player information from
-	//the [player] tags
-	if(start_of_scenario){
-		state.load_recall_list(state.starting_pos.get_children("player"));
+		state.set_menu_items(snapshot.child_range("menu_item"));
 	}
 
 	//In any type of reload(normal save or start-of-scenario) the players could have
 	//changed and need to be replaced
 	if(saved_game || start_of_scenario){
-		const config::child_list& saved_sides = (saved_game ?
-			state.snapshot.get_children("side") :
-			state.starting_pos.get_children("side"));
+		config::child_itors saved_sides = saved_game ?
+			state.snapshot.child_range("side") :
+			state.starting_pos.child_range("side");
+		config::const_child_itors level_sides = level.child_range("side");
 
-		const config::child_list& level_sides = level.get_children("side");
-		for(config::child_list::const_iterator side = saved_sides.begin(); side != saved_sides.end(); ++side) {
-			for(config::child_list::const_iterator lside = level_sides.begin(); lside != level_sides.end(); ++lside) {
-				if ( ((**side)["side"] == (**lside)["side"]) && (
-					 ((**side)["current_player"] != (**lside)["current_player"]) ||
-					 ((**side)["controller"] != (**lside)["controller"]) )){
-
-					(**side)["current_player"] = (**lside)["current_player"];
-					(**side)["id"] = (**lside)["id"];
-					(**side)["save_id"] = (**lside)["save_id"];
-					(**side)["controller"] = (**lside)["controller"];
+		foreach (config &side, saved_sides)
+		{
+			foreach (const config &lside, level_sides)
+			{
+				if (side["side"] == lside["side"] &&
+						(side["current_player"] != lside["current_player"] ||
+						 side["controller"] != lside["controller"]))
+				{
+					side["current_player"] = lside["current_player"];
+					side["id"] = lside["id"];
+					side["save_id"] = lside["save_id"];
+					side["controller"] = lside["controller"];
 					break;
 				}
 			}
@@ -181,9 +188,7 @@ void level_to_gamestate(config& level, game_state& state)
 std::string get_colour_string(int id)
 {
 	std::string prefix = team::get_side_highlight(id);
-	std::stringstream side_id;
-	side_id << (id + 1);
-	std::map<std::string, t_string>::iterator name = game_config::team_rgb_name.find(side_id.str());
+	std::map<std::string, t_string>::iterator name = game_config::team_rgb_name.find(str_cast(id + 1));
 	if(name != game_config::team_rgb_name.end()){
 		return prefix + name->second;
 	}else{
@@ -202,7 +207,7 @@ void chat::add_message(const time_t& time, const std::string& user,
 {
 	message_history_.push_back(msg(time, user, message));
 
-	while (message_history_.size() > MAX_MESSAGES) {
+	while (message_history_.size() > max_messages) {
 		message_history_.pop_front();
 
 		if (last_update_ > 0)
@@ -465,7 +470,7 @@ void ui::handle_event(const SDL_Event& event)
 	}
 }
 
-void ui::add_chat_message(const time_t& time, const std::string& speaker, int /*side*/, const std::string& message, game_display::MESSAGE_TYPE /*type*/)
+void ui::add_chat_message(const time_t& time, const std::string& speaker, int /*side*/, const std::string& message, events::chat_handler::MESSAGE_TYPE /*type*/)
 {
 	chat_.add_message(time, speaker, message);
 	chat_.update_textbox(chat_textbox_);
@@ -483,9 +488,13 @@ void ui::send_chat_message(const std::string& message, bool /*allies_only*/)
 }
 
 
+#ifdef USE_TINY_GUI
+void ui::handle_key_event(const SDL_KeyboardEvent& /*event*/)
+{
+}
+#else
 void ui::handle_key_event(const SDL_KeyboardEvent& event)
 {
-#ifndef USE_TINY_GUI
 	//On enter, adds the current chat message to the chat textbox.
 	if((event.keysym.sym == SDLK_RETURN || event.keysym.sym == SDLK_KP_ENTER) && !entry_textbox_.text().empty()) {
 
@@ -511,16 +520,17 @@ void ui::handle_key_event(const SDL_KeyboardEvent& event)
 		}
 		entry_textbox_.set_text(text);
 	}
-#endif
 }
+#endif
 
 void ui::process_message(const config& msg, const bool whisper) {
 	const std::string& sender = msg["sender"];
 	const std::string& message = msg["message"];
-	if (!preferences::show_lobby_join(sender, message)) return;
+	std::string room = msg["room"];
+	if (!preferences::parse_should_show_lobby_join(sender, message)) return;
 	if (preferences::is_ignored(sender)) return;
 
-	preferences::admin_authentication(sender, message);
+	preferences::parse_admin_authentication(sender, message);
 
 	if (whisper || utils::word_match(message, preferences::login())) {
 		sound::play_UI_sound(game_config::sounds::receive_message_highlight);
@@ -532,19 +542,21 @@ void ui::process_message(const config& msg, const bool whisper) {
 		// too annoying and probably not any helpful
 		//sound::play_UI_sound(game_config::sounds::receive_message);
 	}
-	chat_.add_message(time(NULL), (whisper ? "whisper: " : "") + msg["sender"],
+	if (room == "lobby") room = "";
+	if (!room.empty()) room = room + ": ";
+	chat_.add_message(time(NULL), room + (whisper ? "whisper: " : "") + msg["sender"],
 			msg["message"]);
 	chat_.update_textbox(chat_textbox_);
 }
 
 void ui::process_network_data(const config& data, const network::connection /*sock*/)
 {
-	if(data.child("error")) {
-		throw network::error((*data.child("error"))["message"]);
-	} else if (data.child("message")) {
-		process_message(*data.child("message"));
-	} else if(data.child("whisper")){
-		process_message(*data.child("whisper"), true);
+	if (const config &c = data.child("error")) {
+		throw network::error(c["message"]);
+	} else if (const config &c = data.child("message")) {
+		process_message(c);
+	} else if (const config &c = data.child("whisper")) {
+		process_message(c, true);
 	} else if(data.child("gamelist")) {
 		const cursor::setter cursor_setter(cursor::WAIT);
 		gamelist_initialized_ = true;
@@ -552,10 +564,10 @@ void ui::process_network_data(const config& data, const network::connection /*so
 		gamelist_updated(false);
 		gamelist_refresh_ = false;
 		lobby_clock_ = SDL_GetTicks();
-	} else if(data.child("gamelist_diff")) {
+	} else if (const config &c = data.child("gamelist_diff")) {
 		if(gamelist_initialized_) {
 			try {
-				gamelist_.apply_diff(*data.child("gamelist_diff"));
+				gamelist_.apply_diff(c);
 			} catch(config::error& e) {
 				ERR_CF << "Error while applying the gamelist diff: '"
 					<< e.message << "' Getting a new gamelist.\n";
@@ -563,6 +575,43 @@ void ui::process_network_data(const config& data, const network::connection /*so
 			}
 			gamelist_refresh_ = true;
 		}
+	} else if (const config &c = data.child("room_join")) {
+		if (c["player"] == preferences::login()) {
+			chat_.add_message(time(NULL), "server",
+				"You have joined the room '" + c["room"] + "'");
+		} else {
+			chat_.add_message(time(NULL), "server",
+				c["player"] + " has joined the room '" + c["room"] + "'");
+		}
+		chat_.update_textbox(chat_textbox_);
+	} else if (const config &c = data.child("room_part")) {
+		if (c["player"] == preferences::login()) {
+			chat_.add_message(time(NULL), "server",
+				"You have left the room '" + c["room"] + "'");
+		} else {
+			chat_.add_message(time(NULL), "server",
+				c["player"] + " has left the room '" + c["room"] + "'");
+		}
+		chat_.update_textbox(chat_textbox_);
+	} else if (const config &c = data.child("room_query_response")) {
+		if (const config &ms = c.child("members")) {
+			std::stringstream ss;
+			ss << "Room " << c["room"] << " members: ";
+			foreach (const config& m, ms.child_range("member")) {
+				ss << m["name"] << " ";
+			}
+			chat_.add_message(time(NULL), "server", ss.str());
+			chat_.update_textbox(chat_textbox_);
+		}
+		if (const config &rs = c.child("rooms")) {
+			std::stringstream ss;
+			ss << "Rooms: ";
+			foreach (const config& r, rs.child_range("room")) {
+				ss << r["name"] << "(" << r["size"] << ") ";
+			}
+			chat_.add_message(time(NULL), "server", ss.str());
+			chat_.update_textbox(chat_textbox_);
+		}
 	}
 }
 
@@ -661,25 +710,18 @@ bool ui::user_info::operator> (const user_info& b) const {
 
 void ui::gamelist_updated(bool silent)
 {
-	config::child_list users = gamelist_.get_children("user");
-	config::child_iterator user;
 	std::list<user_info> u_list;
 
-	for (user = users.begin(); user != users.end(); ++user) {
+	foreach (const config &user, gamelist_.child_range("user"))
+	{
 		user_info u_elem;
-		u_elem.name = (**user)["name"];
-		u_elem.game_id = "";
-		u_elem.location = "";
-		u_elem.state = (**user)["available"] == "no" ? GAME : LOBBY;
-		u_elem.registered = utils::string_bool((**user)["registered"]);
-		if(!(**user)["game_id"].empty()) {
-			u_elem.game_id = (**user)["game_id"];
-			if (u_elem.game_id == selected_game_) {
-				u_elem.state = SEL_GAME;
-			}
-		}
-		if(!(**user)["location"].empty()) {
-			u_elem.location = (**user)["location"];
+		u_elem.name = user["name"];
+		u_elem.state = user["available"] == "no" ? GAME : LOBBY;
+		u_elem.registered = utils::string_bool(user["registered"]);
+		u_elem.game_id = user["game_id"];
+		u_elem.location = user["location"];
+		if (!u_elem.game_id.empty() && u_elem.game_id == selected_game_) {
+			u_elem.state = SEL_GAME;
 		}
 		if (u_elem.name == preferences::login()) {
 			u_elem.relation = ME;
@@ -741,7 +783,7 @@ void ui::gamelist_updated(bool silent)
 	set_user_menu_items(menu_strings);
 }
 
-void ui::set_selected_game(const std::string game_id)
+void ui::set_selected_game(const std::string& game_id)
 {
 	// reposition the player list to show the players in the selected game
 	if (preferences::sort_list() && (selected_game_ != game_id)) {
@@ -775,17 +817,11 @@ void ui::set_user_list(const std::vector<std::string>& list, bool silent)
 	user_list_ = list;
 }
 
-std::string ui::get_selected_user_game() {
-	config::child_list users = gamelist_.get_children("user");
-	config::child_iterator user;
-
-	for (user = users.begin(); user != users.end(); ++user) {
-		if((**user)["name"] == selected_user_) {
-			return (**user)["game_id"];
-		}
-	}
-
-	return "";
+std::string ui::get_selected_user_game()
+{
+	const config &u = gamelist_.find_child("user", "name", selected_user_);
+	if (u) return u["game_id"];
+	return std::string();
 }
 
 void ui::append_to_title(const std::string& text) {
