@@ -1,6 +1,6 @@
-/* $Id: multiplayer_wait.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: multiplayer_wait.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2007 - 2008
+   Copyright (C) 2007 - 2010
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 #include "widgets/menu.hpp"
 #include "widgets/combo.hpp"
 
+#include "leader_list.hpp"
 #include "gamestatus.hpp"
 #include "multiplayer_ui.hpp"
 
@@ -28,7 +29,7 @@ namespace mp {
 class wait : public ui
 {
 public:
-	wait(game_display& disp, const config& cfg, const game_data& data, chat& c, config& gamelist);
+	wait(game_display& disp, const config& cfg, chat& c, config& gamelist);
 	virtual void process_event();
 
 	void join_game(bool observe);
@@ -37,9 +38,6 @@ public:
 
 	void start_game();
 
-	// Gets current game snapshot if observer wants to skip the replay
-	game_state& request_snapshot();
-
 protected:
 	virtual void layout_children(const SDL_Rect& rect);
 	virtual void hide_children(bool hide=true);
@@ -48,8 +46,8 @@ private:
 	class leader_preview_pane : public gui::preview_pane
 	{
 	public:
-		leader_preview_pane(game_display& disp, const game_data* data,
-				const config::child_list& side_list);
+		leader_preview_pane(game_display& disp,
+			const std::vector<const config *> &side_list, int color);
 
 		bool show_above() const;
 		bool left_side() const;
@@ -62,12 +60,12 @@ private:
 		virtual void draw_contents();
 		virtual void process_event();
 
-		const config::child_list side_list_;
+		std::vector<const config *> side_list_;
+		const int color_;
 		gui::combo leader_combo_; // Must appear before the leader_list_manager
 		gui::combo gender_combo_; // Must appear before the leader_list_manager
 		leader_list_manager leaders_;
 		size_t  selection_;
-		const game_data* data_;
 	};
 
 	void generate_menu();
@@ -77,7 +75,6 @@ private:
 	gui::menu game_menu_;
 
 	// int team_;
-	const game_data& game_data_;
 
 	config level_;
 	game_state state_;
