@@ -1,6 +1,6 @@
-/* $Id: network.cpp 37795 2009-08-15 15:57:24Z mordante $ */
+/* $Id: network.cpp 42116 2010-04-12 02:46:12Z ai0867 $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,9 @@
 #include "gettext.hpp"
 #include "log.hpp"
 #include "network_worker.hpp"
+#include "serialization/string_utils.hpp"
 #include "thread.hpp"
+#include "util.hpp"
 
 #include "filesystem.hpp"
 
@@ -52,10 +54,11 @@
 #define SOCKET int
 #endif
 
-#define DBG_NW LOG_STREAM(debug, network)
-#define LOG_NW LOG_STREAM(info, network)
-#define WRN_NW LOG_STREAM(warn, network)
-#define ERR_NW LOG_STREAM(err, network)
+static lg::log_domain log_network("network");
+#define DBG_NW LOG_STREAM(debug, log_network)
+#define LOG_NW LOG_STREAM(info, log_network)
+#define WRN_NW LOG_STREAM(warn, log_network)
+#define ERR_NW LOG_STREAM(err, log_network)
 // Only warnings and not errors to avoid DoS by log flooding
 
 namespace {
@@ -157,7 +160,7 @@ static void check_timeout()
 			<< "' Time since last ping: " << now - last_ping << "s\n";
 	// Reset last_ping if we didn't check for the last 10s.
 	if (last_ping_check + 10 <= now) last_ping = now;
-	if (static_cast<time_t>(last_ping + network::ping_timeout) <= now) {
+	if (static_cast<time_t>(last_ping + network::ping_interval + network::ping_timeout) <= now) {
 
 		time_t timeout = now - last_ping;
 		ERR_NW << "No server ping since " << timeout
@@ -244,12 +247,6 @@ manager::manager(size_t min_threads, size_t max_threads) : free_(true)
 		return;
 	}
 
-	// On Unix-based systems, set sigpipe to be ignored
-#if !(defined(_WIN32) || defined(__APPLE__) || defined(__AMIGAOS4__))
-	WRN_NW << "ignoring SIGPIPE\n";
-	signal(SIGPIPE,SIG_IGN);
-#endif
-
 	if(SDLNet_Init() == -1) {
 		ERR_NW << "could not initialize SDLNet; throwing error...\n";
 		throw error(SDL_GetError());
@@ -369,14 +366,15 @@ void connect_operation::run()
 	char* const hostname = host_.empty() ? NULL : const_cast<char*>(host_.c_str());
 	IPaddress ip;
 	if(SDLNet_ResolveHost(&ip,hostname,port_) == -1) {
-		error_ = "Could not connect to host";
+		error_ = N_("Could not connect to host.");
 		return;
 	}
 
 	TCPsocket sock = SDLNet_TCP_Open(&ip);
 	if(!sock) {
-		error_ = hostname == NULL ? "Could not bind to port" :
-		                            "Could not connect to host";
+		error_ = hostname == NULL
+				? "Could not bind to port"
+				: N_("Could not connect to host.");
 		return;
 	}
 
@@ -386,7 +384,8 @@ void connect_operation::run()
 	{
 	_TCPsocket* raw_sock = reinterpret_cast<_TCPsocket*>(sock);
 	int no = 0;
-	setsockopt(raw_sock->channel, IPPROTO_TCP, TCP_NODELAY, (char*)&no, sizeof(no));
+	setsockopt(raw_sock->channel,
+			IPPROTO_TCP, TCP_NODELAY, reinterpret_cast<char*>(&no), sizeof(no));
 	}
 #endif
 
@@ -488,6 +487,66 @@ connection connect(const std::string& host, int port, threading::waiter& waiter)
 	return static_cast<connect_operation*>(op.get())->result();
 }
 
+namespace {
+
+connection accept_connection_pending(std::vector<TCPsocket>& pending_sockets,
+	SDLNet_SocketSet& pending_socket_set)
+{
+	DBG_NW << "pending socket activity...\n";
+
+	std::vector<TCPsocket>::iterator i = pending_sockets.begin();
+	while (i != pending_sockets.end() && !SDLNet_SocketReady(*i)) ++i;
+
+	if (i == pending_sockets.end()) return 0;
+
+	// Receive the 4 bytes telling us if they're a new connection
+	// or trying to recover a connection
+	char buf[4] ALIGN_4;
+
+	const TCPsocket psock = *i;
+	SDLNet_TCP_DelSocket(pending_socket_set,psock);
+	pending_sockets.erase(i);
+
+	DBG_NW << "receiving data from pending socket...\n";
+
+	const int len = SDLNet_TCP_Recv(psock,buf,4);
+	if(len != 4) {
+		WRN_NW << "pending socket disconnected\n";
+		SDLNet_TCP_Close(psock);
+		return 0;
+	}
+
+	const int handle = SDLNet_Read32(reinterpret_cast<void*>(buf));
+
+	DBG_NW << "received handshake from client: '" << handle << "'\n";
+
+	const int res = SDLNet_TCP_AddSocket(socket_set,psock);
+	if(res == -1) {
+		ERR_NW << "SDLNet_GetError() is " << SDLNet_GetError() << "\n";
+		SDLNet_TCP_Close(psock);
+
+		throw network::error(_("Could not add socket to socket set"));
+	}
+
+	const connection connect = create_connection(psock,"",0);
+
+	// Send back their connection number
+	SDLNet_Write32(connect, reinterpret_cast<void*>(buf));
+	const int nbytes = SDLNet_TCP_Send(psock,buf,4);
+	if(nbytes != 4) {
+		SDLNet_TCP_DelSocket(socket_set,psock);
+		SDLNet_TCP_Close(psock);
+		remove_connection(connect);
+		throw network::error(_("Could not send initial handshake"));
+	}
+
+	waiting_sockets.insert(connect);
+	sockets.push_back(connect);
+	return connect;
+}
+
+} //anon namespace
+
 connection accept_connection()
 {
 	if(!server_socket) {
@@ -508,7 +567,7 @@ connection accept_connection()
 	const TCPsocket sock = SDLNet_TCP_Accept(server_socket);
 	if(sock) {
 		DBG_NW << "received connection. Pending handshake...\n";
-		pending_sockets.push_back(sock);
+
 		if(pending_socket_set == 0) {
 			pending_socket_set = SDLNet_AllocSocketSet(32);
 		}
@@ -516,11 +575,16 @@ connection accept_connection()
 		if(pending_socket_set != 0) {
 			int res = SDLNet_TCP_AddSocket(pending_socket_set,sock);
 
-			if (res == -1)
-			{
+			if (res != -1) {
+				pending_sockets.push_back(sock);
+			} else {
 				ERR_NW << "Pending socket set is full! Disconnecting " << sock << " connection\n";
+				ERR_NW << "SDLNet_GetError() is " << SDLNet_GetError() << "\n";
+
 				SDLNet_TCP_Close(sock);
 			}
+		} else {
+			ERR_NW << "Error in SDLNet_AllocSocketSet\n";
 		}
 	}
 
@@ -533,60 +597,7 @@ connection accept_connection()
 		return 0;
 	}
 
-	DBG_NW << "pending socket activity...\n";
-
-	for(std::vector<TCPsocket>::iterator i = pending_sockets.begin(); i != pending_sockets.end(); ++i) {
-		if(!SDLNet_SocketReady(*i)) {
-			continue;
-		}
-
-		// Receive the 4 bytes telling us if they're a new connection
-		// or trying to recover a connection
-		char buf[4] ALIGN_4;
-
-		const TCPsocket sock = *i;
-		SDLNet_TCP_DelSocket(pending_socket_set,sock);
-		pending_sockets.erase(i);
-
-		DBG_NW << "receiving data from pending socket...\n";
-
-		const int len = SDLNet_TCP_Recv(sock,buf,4);
-		if(len != 4) {
-			WRN_NW << "pending socket disconnected\n";
-			SDLNet_TCP_Close(sock);
-			return 0;
-		}
-
-		const int handle = SDLNet_Read32(reinterpret_cast<void*>(buf));
-
-		DBG_NW << "received handshake from client: '" << handle << "'\n";
-
-		const int res = SDLNet_TCP_AddSocket(socket_set,sock);
-		if(res == -1) {
-			SDLNet_TCP_Close(sock);
-
-			throw network::error(_("Could not add socket to socket set"));
-		}
-
-
-		const connection connect = create_connection(sock,"",0);
-
-		// Send back their connection number
-		SDLNet_Write32(connect, reinterpret_cast<void*>(buf));
-		const int nbytes = SDLNet_TCP_Send(sock,buf,4);
-		if(nbytes != 4) {
-			SDLNet_TCP_DelSocket(socket_set,sock);
-			SDLNet_TCP_Close(sock);
-			remove_connection(connect);
-			throw network::error(_("Could not send initial handshake"));
-		}
-
-		waiting_sockets.insert(connect);
-		sockets.push_back(connect);
-		return connect;
-	}
-
-	return 0;
+	return accept_connection_pending(pending_sockets, pending_socket_set);
 }
 
 bool disconnect(connection s)
@@ -649,7 +660,8 @@ connection receive_data(config& cfg, connection connection_num, unsigned int tim
 {
 	unsigned int start_ticks = SDL_GetTicks();
 	while(true) {
-		const connection res = receive_data(cfg,connection_num,(bool*)0 , bandwidth_in);
+		const connection res = receive_data(
+				cfg,connection_num, static_cast<bool*>(0) , bandwidth_in);
 		if(res != 0) {
 			return res;
 		}
@@ -766,8 +778,7 @@ connection receive_data(config& cfg, connection connection_num, bool* gzipped, b
 	waiting_sockets.insert(result);
 	if(!is_server()) {
 		const time_t& now = time(NULL);
-		const string_map::const_iterator ping = cfg.values.find("ping");
-		if (ping != cfg.values.end()) {
+		if (cfg.has_attribute("ping")) {
 			LOG_NW << "Lag: " << (now - lexical_cast<time_t>(cfg["ping"])) << "\n";
 			last_ping = now;
 		} else if (last_ping != 0) {
