@@ -1,6 +1,6 @@
-/* $Id: network.hpp 32989 2009-02-22 01:27:50Z dave $ */
+/* $Id: network.hpp 42116 2010-04-12 02:46:12Z ai0867 $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -258,6 +258,8 @@ statistics get_receive_stats(connection handle);
 
 /** Amount of seconds after the last server ping when we assume to have timed out. */
 extern unsigned int ping_timeout;
+/** Minimum interval between pings. */
+const int ping_interval = 30;
 } // network namespace
 
 
