@@ -1,6 +1,6 @@
-/* $Id: network_worker.cpp 37795 2009-08-15 15:57:24Z mordante $ */
+/* $Id: network_worker.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,7 @@
 
 #include <cerrno>
 #include <deque>
+#include <sstream>
 
 #ifdef HAVE_SENDFILE
 #include <sys/sendfile.h>
@@ -86,9 +87,10 @@ typedef int socklen_t;
 #  endif
 #endif
 
-#define DBG_NW LOG_STREAM(debug, network)
-#define LOG_NW LOG_STREAM(info, network)
-#define ERR_NW LOG_STREAM(err, network)
+static lg::log_domain log_network("network");
+#define DBG_NW LOG_STREAM(debug, log_network)
+#define LOG_NW LOG_STREAM(info, log_network)
+#define ERR_NW LOG_STREAM(err, log_network)
 
 namespace {
 struct _TCPsocket {
@@ -511,7 +513,7 @@ static SOCKET_STATE send_file(buffer* buf)
 		buffer.resize(4);
 		SDLNet_Write32(filesize,&buffer[0]);
 		int socket = reinterpret_cast<_TCPsocket*>(buf->sock)->channel;
-		const scoped_fd in_file(open(buf->config_error.c_str(),O_NOATIME | O_RDONLY));
+		const scoped_fd in_file(open(buf->config_error.c_str(), O_RDONLY));
 		cork_setter set_socket_cork(socket);
 		int poll_res;
 		struct pollfd fd = {socket, POLLOUT, 0 };
@@ -602,9 +604,9 @@ static SOCKET_STATE send_file(buffer* buf)
 
 	}
 	if (upto != filesize && !file_stream->good()) {
-		ERR_NW << "send_file failed because stream not good from file "
-			<< buf->config_error << " upto: " << upto
-			<< " size: " << filesize << "\n";
+		ERR_NW << "send_file failed because the stream from file '"
+			<< buf->config_error << "' is not good. Sent up to: " << upto
+			<< " of file size: " << filesize << "\n";
 	}
 	return result;
 }
