@@ -1,6 +1,6 @@
-/* $Id: play_controller.cpp 33687 2009-03-15 16:50:42Z mordante $ */
+/* $Id: play_controller.cpp 42178 2010-04-19 05:30:57Z silene $ */
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -20,38 +20,68 @@
 
 #include "play_controller.hpp"
 #include "dialogs.hpp"
-#include "config_adapter.hpp"
+#include "foreach.hpp"
+#include "game_events.hpp"
 #include "gettext.hpp"
+#include "halo.hpp"
 #include "loadscreen.hpp"
 #include "log.hpp"
+#include "resources.hpp"
+#include "savegame.hpp"
 #include "sound.hpp"
 #include "unit_id.hpp"
 #include "terrain_filter.hpp"
+#include "save_blocker.hpp"
+#include "preferences_display.hpp"
+#include "replay.hpp"
+#include "soundsource.hpp"
+#include "tooltips.hpp"
+#include "game_preferences.hpp"
+#include "wml_exception.hpp"
+#include "formula_string_utils.hpp"
+#include "ai/manager.hpp"
+#include "ai/testing.hpp"
 
+static lg::log_domain log_engine("engine");
+#define LOG_NG LOG_STREAM(info, log_engine)
+#define DBG_NG LOG_STREAM(debug, log_engine)
 
-#define LOG_NG LOG_STREAM(info, engine)
+static lg::log_domain log_display("display");
+#define ERR_DP LOG_STREAM(err, log_display)
+
+static void clear_resources()
+{
+	resources::game_map = NULL;
+	resources::units = NULL;
+	resources::teams = NULL;
+	resources::state_of_game = NULL;
+	resources::controller = NULL;
+	resources::screen = NULL;
+	resources::soundsources = NULL;
+	resources::tod_manager = NULL;
+}
 
 play_controller::play_controller(const config& level, game_state& state_of_game,
 		int ticks, int num_turns, const config& game_config, CVideo& video,
 		bool skip_replay) :
 	controller_base(ticks, game_config, video),
-	verify_manager_(units_),
-	team_manager_(teams_),
+	observer(),
+	savegame_config(),
 	prefs_disp_manager_(),
 	tooltips_manager_(),
 	events_manager_(),
 	halo_manager_(),
 	labels_manager_(),
 	help_manager_(&game_config, &map_),
-	mouse_handler_(NULL, teams_, units_, map_, status_, undo_stack_, redo_stack_),
-	menu_handler_(NULL, units_, teams_, level, map_, game_config, status_, state_of_game, undo_stack_, redo_stack_),
+	mouse_handler_(NULL, teams_, units_, map_, tod_manager_, undo_stack_, redo_stack_),
+	menu_handler_(NULL, units_, teams_, level, map_, game_config, tod_manager_, state_of_game, undo_stack_, redo_stack_),
 	soundsources_manager_(),
+	tod_manager_(level, num_turns, &state_of_game),
 	gui_(),
 	statistics_context_(level["name"]),
 	level_(level),
 	teams_(),
 	gamestate_(state_of_game),
-	status_(level, num_turns, &state_of_game),
 	map_(game_config, level["map_data"]),
 	units_(),
 	undo_stack_(),
@@ -61,29 +91,44 @@ play_controller::play_controller(const config& level, game_state& state_of_game,
 	first_human_team_(-1),
 	player_number_(1),
 	first_player_(lexical_cast_default<unsigned int, std::string>(level_["playing_team"], 0) + 1),
-	start_turn_(status_.turn()),
+	start_turn_(turn()),
 	is_host_(true),
 	skip_replay_(skip_replay),
 	linger_(false),
-	first_turn_(true),
+	previous_turn_(0),
 	savenames_(),
 	wml_commands_(),
+	victory_when_enemies_defeated_(true),
+	end_level_data_(),
 	victory_music_(),
 	defeat_music_()
 {
+	resources::game_map = &map_;
+	resources::units = &units_;
+	resources::teams = &teams_;
+	resources::state_of_game = &gamestate_;
+	resources::controller = this;
+	resources::tod_manager = &tod_manager_;
+
 	// Setup victory and defeat music
 	set_victory_music_list(level_["victory_music"]);
 	set_defeat_music_list(level_["defeat_music"]);
 
-	status_.teams = &teams_;
 	game_config::add_color_info(level);
 	hotkey::deactivate_all_scopes();
 	hotkey::set_scope_active(hotkey::SCOPE_GENERAL);
 	hotkey::set_scope_active(hotkey::SCOPE_GAME);
-	init(video);
+	try {
+		init(video);
+	} catch (...) {
+		clear_resources();
+		throw;
+	}
 }
 
-play_controller::~play_controller(){
+play_controller::~play_controller()
+{
+	clear_resources();
 }
 
 void play_controller::init(CVideo& video){
@@ -105,57 +150,72 @@ void play_controller::init(CVideo& video){
 	}
 	recorder.set_skip(false);
 
-	const config::child_list& unit_cfg = level_.get_children("side");
 	const bool snapshot = utils::string_bool(level_["snapshot"]);
 
 	if(utils::string_bool(level_["modify_placing"])) {
 		LOG_NG << "modifying placing...\n";
-		place_sides_in_preferred_locations(map_,unit_cfg);
+		place_sides_in_preferred_locations();
 	}
 
-
+	foreach (const config &t, level_.child_range("time_area")) {
+		add_time_area(t);
+	}
 
 	LOG_NG << "initialized teams... "    << (SDL_GetTicks() - ticks_) << "\n";
 	loadscreen::global_loadscreen->set_progress(60, _("Initializing teams"));
 
-
 	// This *needs* to be created before the show_intro and show_map_scene
 	// as that functions use the manager state_of_game
 	// Has to be done before registering any events!
-	events_manager_.reset(new game_events::manager(level_,map_,
-                                                   units_,teams_, gamestate_,status_));
+	events_manager_.reset(new game_events::manager(level_));
 
 	std::set<std::string> seen_save_ids;
 
-	for(config::child_list::const_iterator ui = unit_cfg.begin(); ui != unit_cfg.end(); ++ui) {
-		std::string save_id = get_unique_saveid(**ui, seen_save_ids);
+	int team_num = 0;
+	foreach (const config &side, level_.child_range("side"))
+	{
+		std::string save_id = get_unique_saveid(side, seen_save_ids);
 		seen_save_ids.insert(save_id);
-		if (first_human_team_ == -1){
-			first_human_team_ = get_first_human_team(ui, unit_cfg);
+		if (first_human_team_ == -1) {
+			const std::string &controller = side["controller"];
+			if (controller == preferences::client_type() &&
+			    side["id"] == preferences::login()) {
+				first_human_team_ = team_num;
+			} else if (controller == "human") {
+				first_human_team_ = team_num;
+			}
 		}
-		get_player_info(**ui, gamestate_, save_id, teams_, level_, map_, units_, status_, snapshot);
+		gamestate_.build_team(side, save_id, teams_, level_, map_
+				, units_, snapshot);
+		++team_num;
 	}
 
+	// mouse_handler expects at least one team for linger mode to work.
+	if (teams_.empty()) end_level_data_.linger_mode = false;
+
 	LOG_NG << "loading units..." << (SDL_GetTicks() - ticks_) << "\n";
 	loadscreen::global_loadscreen->set_progress(70, _("Loading units"));
 	preferences::encounter_recruitable_units(teams_);
 	preferences::encounter_start_units(units_);
-	preferences::encounter_recallable_units(gamestate_);
+	preferences::encounter_recallable_units(teams_);
 	preferences::encounter_map_terrain(map_);
 
 
 	loadscreen::global_loadscreen->set_progress(80, _("Initializing display"));
 	LOG_NG << "initializing display... " << (SDL_GetTicks() - ticks_) << "\n";
 
-	const config* theme_cfg = get_theme(game_config_, level_["theme"]);
-	if (theme_cfg)
-		gui_.reset(new game_display(units_,video,map_,status_,teams_,*theme_cfg, game_config_, level_));
-	else
-		gui_.reset(new game_display(units_,video,map_,status_,teams_,config(), game_config_, level_));
+	const config &theme_cfg = get_theme(game_config_, level_["theme"]);
+	gui_.reset(new game_display(units_, video, map_, tod_manager_, teams_, theme_cfg, level_));
+	if (!gui_->video().faked()) {
+		if (gamestate_.mp_settings().mp_countdown)
+			gui_->get_theme().modify_label("time-icon", _ ("time left for current turn"));
+		else
+			gui_->get_theme().modify_label("time-icon", _ ("current local time"));
+	}
 	loadscreen::global_loadscreen->set_progress(90, _("Initializing display"));
 	mouse_handler_.set_gui(gui_.get());
 	menu_handler_.set_gui(gui_.get());
-	events_manager_->set_gui(*gui_);
+	resources::screen = gui_.get();
 	theme::set_known_themes(&game_config_);
 
 	LOG_NG << "done initializing display... " << (SDL_GetTicks() - ticks_) << "\n";
@@ -163,7 +223,7 @@ void play_controller::init(CVideo& video){
 	if(first_human_team_ != -1) {
 		gui_->set_team(first_human_team_);
 	}
-	else if (team_manager_.is_observer())
+	else if (is_observer())
 	{
 		// Find first team that is allowed to be observered.
 		// If not set here observer would be without fog untill
@@ -182,9 +242,8 @@ void play_controller::init(CVideo& video){
 
 	init_managers();
 	// add era events for MP game
-	const config* era_cfg = level_.child("era");
-	if (era_cfg != NULL) {
-		game_events::add_events(era_cfg->get_children("event"),"era_events");
+	if (const config &era_cfg = level_.child("era")) {
+		game_events::add_events(era_cfg.child_range("event"), "era_events");
 	}
 
 
@@ -199,7 +258,7 @@ void play_controller::init_managers(){
 	tooltips_manager_.reset(new tooltips::manager(gui_->video()));
 	soundsources_manager_.reset(new soundsource::manager(*gui_));
 
-	events_manager_->set_soundsource(*soundsources_manager_);
+	resources::soundsources = soundsources_manager_.get();
 
 	halo_manager_.reset(new halo::manager(*gui_));
 	LOG_NG << "done initializing managers... " << (SDL_GetTicks() - ticks_) << "\n";
@@ -240,41 +299,43 @@ struct placing_info {
 
 static bool operator<(const placing_info& a, const placing_info& b) { return a.score > b.score; }
 
-void play_controller::place_sides_in_preferred_locations(gamemap& map, const config::child_list& sides)
+void play_controller::place_sides_in_preferred_locations()
 {
 	std::vector<placing_info> placings;
 
-	const int num_pos = map.num_valid_starting_positions();
+	int num_pos = map_.num_valid_starting_positions();
 
-	for(config::child_list::const_iterator s = sides.begin(); s != sides.end(); ++s) {
-		const int side_num = s - sides.begin() + 1;
+	int side_num = 1;
+	foreach (const config &side, level_.child_range("side"))
+	{
 		for(int p = 1; p <= num_pos; ++p) {
-			const map_location& pos = map.starting_position(p);
-			const int score = placing_score(**s,map,pos);
+			const map_location& pos = map_.starting_position(p);
+			int score = placing_score(side, map_, pos);
 			placing_info obj;
 			obj.side = side_num;
 			obj.score = score;
 			obj.pos = pos;
 			placings.push_back(obj);
 		}
+		++side_num;
 	}
 
 	std::sort(placings.begin(),placings.end());
 	std::set<int> placed;
 	std::set<map_location> positions_taken;
 
-	for(std::vector<placing_info>::const_iterator i = placings.begin(); i != placings.end() && placed.size() != sides.size(); ++i) {
+	for (std::vector<placing_info>::const_iterator i = placings.begin(); i != placings.end() && int(placed.size()) != side_num - 1; ++i) {
 		if(placed.count(i->side) == 0 && positions_taken.count(i->pos) == 0) {
 			placed.insert(i->side);
 			positions_taken.insert(i->pos);
-			map.set_starting_position(i->side,i->pos);
+			map_.set_starting_position(i->side,i->pos);
 			LOG_NG << "placing side " << i->side << " at " << i->pos << '\n';
 		}
 	}
 }
 
 void play_controller::objectives(){
-	menu_handler_.objectives(player_number_);
+	menu_handler_.objectives(gui_->viewing_team()+1);
 }
 
 void play_controller::show_statistics(){
@@ -290,19 +351,37 @@ void play_controller::status_table(){
 }
 
 void play_controller::save_game(){
-	menu_handler_.save_game("",gui::OK_CANCEL);
+	if(save_blocker::try_block()) {
+		save_blocker::save_unblocker unblocker;
+		savegame::game_savegame save(gamestate_, *gui_, to_config(), preferences::compress_saves());
+		save.save_game_interactive(gui_->video(), "", gui::OK_CANCEL);
+	} else {
+		save_blocker::on_unblock(this,&play_controller::save_game);
+	}
 }
 
 void play_controller::save_replay(){
-	menu_handler_.save_game("", gui::OK_CANCEL, false, true);
+	if(save_blocker::try_block()) {
+		save_blocker::save_unblocker unblocker;
+		savegame::replay_savegame save(gamestate_, preferences::compress_saves());
+		save.save_game_interactive(gui_->video(), "", gui::OK_CANCEL);
+	} else {
+		save_blocker::on_unblock(this,&play_controller::save_replay);
+	}
 }
 
 void play_controller::save_map(){
-	menu_handler_.save_map();
+	if(save_blocker::try_block()) {
+		save_blocker::save_unblocker unblocker;
+		menu_handler_.save_map();
+	} else {
+		save_blocker::on_unblock(this,&play_controller::save_map);
+	}
 }
 
 void play_controller::load_game(){
-	menu_handler_.load_game();
+	savegame::loadgame load(*gui_, game_config_, gamestate_);
+	load.load_game();
 }
 
 void play_controller::preferences(){
@@ -349,6 +428,10 @@ void play_controller::unit_description(){
 	menu_handler_.unit_description(mouse_handler_);
 }
 
+void play_controller::toggle_ellipses(){
+	menu_handler_.toggle_ellipses();
+}
+
 void play_controller::toggle_grid(){
 	menu_handler_.toggle_grid();
 }
@@ -359,25 +442,29 @@ void play_controller::search(){
 
 
 void play_controller::fire_prestart(bool execute){
+	// Run initialization scripts, even if loading from a snapshot.
+	game_events::fire("preload");
+
 	// pre-start events must be executed before any GUI operation,
 	// as those may cause the display to be refreshed.
 	if (execute){
 		update_locker lock_display(gui_->video());
 		game_events::fire("prestart");
+		check_end_level();
 		// prestart event may modify start turn with WML, reflect any changes.
-		start_turn_ = status_.turn();
+		start_turn_ = turn();
 	}
 }
 
 void play_controller::fire_start(bool execute){
 	if(execute) {
 		game_events::fire("start");
+		check_end_level();
 		// start event may modify start turn with WML, reflect any changes.
-		start_turn_ = status_.turn();
+		start_turn_ = turn();
 		gamestate_.set_variable("turn_number", str_cast<size_t>(start_turn_));
-		first_turn_ = true;
 	} else {
-		first_turn_ = false;
+		previous_turn_ = turn();
 	}
 }
 
@@ -386,59 +473,69 @@ void play_controller::init_gui(){
 	gui_->adjust_colours(0,0,0);
 
 	for(std::vector<team>::iterator t = teams_.begin(); t != teams_.end(); ++t) {
-		::clear_shroud(*gui_,map_,units_,teams_,(t-teams_.begin()));
+		::clear_shroud(t - teams_.begin() + 1);
 	}
 }
 
-void play_controller::init_side(const unsigned int team_index, bool /*is_replay*/){
+void play_controller::init_side(const unsigned int team_index, bool is_replay){
 	log_scope("player turn");
 	team& current_team = teams_[team_index];
 
-	mouse_handler_.set_team(team_index+1);
+	mouse_handler_.set_side(team_index + 1);
 
 	// If we are observers we move to watch next team if it is allowed
-	if (team_manager_.is_observer()
+	if (is_observer()
 		&& !current_team.get_disallow_observers()) {
 		gui_->set_team(size_t(team_index));
 	}
-		gui_->set_playing_team(size_t(team_index));
+	gui_->set_playing_team(size_t(team_index));
 
-	std::stringstream player_number_str;
-	player_number_str << player_number_;
-	gamestate_.set_variable("side_number",player_number_str.str());
+	gamestate_.set_variable("side_number", str_cast(player_number_));
 	gamestate_.last_selected = map_location::null_location;
 
-	/*
-		Normally, events must not be actively fired through replays, because
-		they have been recorded previously and therefore will get executed anyway.
-		Firing them in the normal code would lead to double execution.
-		However, the following events are different in that they need to be executed _now_
-		(before calculation of income and healing) or we will risk OOS errors if we manipulate
-		these informations inside the events and in the replay have a different order of execution.
-	*/
-	bool real_side_change = true;
-	if(first_turn_) {
-		first_turn_ = false;
-		game_events::fire("turn " + str_cast<size_t>(start_turn_));
-		game_events::fire("new turn");
+	/**
+	 * We do this only for local side when we are not replaying.
+	 * For all other sides it is recorded in replay and replay handler has to handle calling do_init_side()
+	 * functions.
+	 **/
+	if (!current_team.is_local()
+		|| is_replay)
+		return;
+	if (!loading_game_) recorder.init_side();
+	do_init_side(team_index);
+}
+
+/**
+ * Called by replay handler or init_side() to do actual work for turn change.
+ */
+void play_controller::do_init_side(const unsigned int team_index){
+	log_scope("player turn");
+	team& current_team = teams_[team_index];
+
+	// If this is right after loading a game we don't need to fire events and such. It was already done before saving.
+	if (!loading_game_) {
+		std::string turn_num = str_cast(turn());
+		std::string side_num = str_cast(team_index + 1);
+
+		if (turn() != previous_turn_)
+		{
+			game_events::fire("turn " + turn_num);
+			game_events::fire("new turn");
+			previous_turn_ = turn();
+		}
+
 		game_events::fire("side turn");
-	} else if (team_index != (first_player_ - 1) || status_.turn() > start_turn_) {
-		// Fire side turn event only if real side change occurs,
-		// not counting changes from void to a side
-		game_events::fire("side turn");
-	} else {
-		real_side_change = false;
+		game_events::fire("side " + side_num + " turn");
+		game_events::fire("side " + side_num + " turn " + turn_num);
 	}
 
 	// We want to work out if units for this player should get healed,
 	// and the player should get income now.
 	// Healing/income happen if it's not the first turn of processing,
-	// or if we are loading a game, and this is not the player it started with.
-	const bool turn_refresh = status_.turn() > start_turn_ || (loading_game_ && team_index != (first_player_ - 1));
-
-	if(turn_refresh) {
+	// or if we are loading a game.
+	if (!loading_game_ && turn() > 1) {
 		for(unit_map::iterator i = units_.begin(); i != units_.end(); ++i) {
-			if(i->second.side() == static_cast<size_t>(player_number_)) {
+			if (i->second.side() == player_number_) {
 				i->second.new_turn();
 			}
 		}
@@ -447,33 +544,89 @@ void play_controller::init_side(const unsigned int team_index, bool /*is_replay*
 
 		// If the expense is less than the number of villages owned,
 		// then we don't have to pay anything at all
-		const int expense = team_upkeep(units_,player_number_) -
+		int expense = side_upkeep(units_, player_number_) -
 								current_team.villages().size();
 		if(expense > 0) {
 			current_team.spend_gold(expense);
 		}
 
-		calculate_healing((*gui_),map_,units_,player_number_,teams_, !skip_replay_);
+		calculate_healing(player_number_, !skip_replay_);
 		reset_resting(units_, player_number_);
 	}
-	if(turn_refresh || real_side_change) {
+
+	if (!loading_game_) {
 		game_events::fire("turn refresh");
 	}
 
-	const time_of_day &tod = status_.get_time_of_day();
-	current_team.set_time_of_day(int(status_.turn()), tod);
+	const time_of_day &tod = tod_manager_.get_time_of_day();
 
-	if(team_index == first_player_ - 1)
+	if (int(team_index) + 1 == first_player_)
 		sound::play_sound(tod.sounds, sound::SOUND_SOURCES);
 
 	if (!recorder.is_skipping()){
-		::clear_shroud(*gui_,map_,units_,teams_,team_index);
+		::clear_shroud(team_index + 1);
 		gui_->invalidate_all();
 	}
 
 	if (!recorder.is_skipping() && !skip_replay_){
 		gui_->scroll_to_leader(units_, player_number_,game_display::ONSCREEN,false);
 	}
+	loading_game_ = false;
+}
+
+//builds the snapshot config from its members and their configs respectively
+config play_controller::to_config() const
+{
+	config cfg;
+
+	cfg.merge_attributes(level_);
+
+	std::stringstream buf;
+
+	for(std::vector<team>::const_iterator t = teams_.begin(); t != teams_.end(); ++t) {
+		int side_num = t - teams_.begin() + 1;
+
+		config& side = cfg.add_child("side");
+		t->write(side);
+		side["no_leader"] = "yes";
+		buf.str(std::string());
+		buf << side_num;
+		side["side"] = buf.str();
+
+		if (!linger_){
+			//current visible units
+			for(unit_map::const_iterator i = units_.begin(); i != units_.end(); ++i) {
+				if(i->second.side() == side_num) {
+					config& u = side.add_child("unit");
+					i->first.write(u);
+					i->second.write(u);
+				}
+			}
+		}
+		//recall list
+		{
+			for(std::vector<unit>::const_iterator j = t->recall_list().begin();
+				j != t->recall_list().end(); ++j) {
+					config& u = side.add_child("unit");
+					j->write(u);
+			}
+		}
+	}
+
+	cfg.merge_with(tod_manager_.to_config());
+
+	// Write terrain_graphics data in snapshot, too
+	const config::child_list& terrains = level_.get_children("terrain_graphics");
+	for(config::child_list::const_iterator tg = terrains.begin();
+			tg != terrains.end(); ++tg) {
+
+		cfg.add_child("terrain_graphics", **tg);
+	}
+
+	//write out the current state of the map
+	cfg["map_data"] = map_.write();
+
+	return cfg;
 }
 
 void play_controller::finish_side_turn(){
@@ -494,18 +647,11 @@ void play_controller::finish_side_turn(){
 	game_events::pump();
 }
 
-void play_controller::finish_turn(){
-	std::stringstream event_stream;
-	event_stream << status_.turn();
-
-	{
-		LOG_NG << "turn event..." << (recorder.is_skipping() ? "skipping" : "no skip") << "\n";
-		update_locker lock_display(gui_->video(),recorder.is_skipping());
-		const std::string turn_num = event_stream.str();
-		gamestate_.set_variable("turn_number",turn_num);
-		game_events::fire("turn " + turn_num);
-		game_events::fire("new turn");
-	}
+void play_controller::finish_turn()
+{
+	LOG_NG << "turn event..." << (recorder.is_skipping() ? "skipping" : "no skip") << '\n';
+	update_locker lock_display(gui_->video(),recorder.is_skipping());
+	gamestate_.set_variable("turn_number", str_cast(turn()));
 }
 
 bool play_controller::enemies_visible() const
@@ -538,8 +684,7 @@ bool play_controller::execute_command(hotkey::HOTKEY_COMMAND command, int index)
 			recorder.add_event(wml_commands_[i]->name, menu_hex);
 			if(game_events::fire(wml_commands_[i]->name, menu_hex)) {
 				// The event has mutated the gamestate
-				apply_shroud_changes(undo_stack_, gui_.get(), map_,
-					units_, teams_, (player_number_ - 1));
+				apply_shroud_changes(undo_stack_, player_number_);
 				undo_stack_.clear();
 			}
 			return true;
@@ -571,6 +716,7 @@ bool play_controller::can_execute_command(hotkey::HOTKEY_COMMAND command, int in
 	case hotkey::HOTKEY_MAP_SCREENSHOT:
 	case hotkey::HOTKEY_ACCELERATED:
 	case hotkey::HOTKEY_SAVE_MAP:
+	case hotkey::HOTKEY_TOGGLE_ELLIPSES:
 	case hotkey::HOTKEY_TOGGLE_GRID:
 	case hotkey::HOTKEY_MOUSE_SCROLL:
 	case hotkey::HOTKEY_STATUS_TABLE:
@@ -620,7 +766,7 @@ bool play_controller::can_execute_command(hotkey::HOTKEY_COMMAND command, int in
 		return !events::commands_disabled &&
 			menu_handler_.current_unit(mouse_handler_) != units_.end() &&
 			!(menu_handler_.current_unit(mouse_handler_)->second.unrenamable()) &&
-			menu_handler_.current_unit(mouse_handler_)->second.side() == gui_->viewing_team()+1 &&
+			menu_handler_.current_unit(mouse_handler_)->second.side() == gui_->viewing_side() &&
 			teams_[menu_handler_.current_unit(mouse_handler_)->second.side() - 1].is_human();
 
 	default:
@@ -657,20 +803,40 @@ void play_controller::enter_textbox()
 		break;
 	default:
 		menu_handler_.get_textbox().close(*gui_);
-		LOG_STREAM(err, display) << "unknown textbox mode\n";
+		ERR_DP << "unknown textbox mode\n";
 	}
 
 }
 
+std::string play_controller::get_unique_saveid(const config& cfg, std::set<std::string>& seen_save_ids)
+{
+	std::string save_id = cfg["save_id"];
+
+	if(save_id.empty()) {
+		save_id=cfg["id"];
+	}
+
+	if(save_id.empty()) {
+		save_id="Unknown";
+	}
+
+	// Make sure the 'save_id' is unique
+	while(seen_save_ids.count(save_id)) {
+		save_id += "_";
+	}
+
+	return save_id;
+}
+
 team& play_controller::current_team()
 {
-	assert(player_number_ > 0 && player_number_ <= teams_.size());
+	assert(player_number_ > 0 && player_number_ <= int(teams_.size()));
 	return teams_[player_number_-1];
 }
 
 const team& play_controller::current_team() const
 {
-	assert(player_number_ > 0 && player_number_ <= teams_.size());
+	assert(player_number_ > 0 && player_number_ <= int(teams_.size()));
 	return teams_[player_number_-1];
 }
 
@@ -702,9 +868,10 @@ void play_controller::slice_before_scroll() {
 }
 
 void play_controller::slice_end() {
-	if(!browse_ && current_team().objectives_changed()) {
-		dialogs::show_objectives(*gui_, level_, current_team().objectives());
-		current_team().reset_objectives_changed();
+	if(!browse_ && teams_[gui_->viewing_team()].objectives_changed()) {
+		check_end_level();
+		dialogs::show_objectives(level_, teams_[gui_->viewing_team()].objectives());
+		teams_[gui_->viewing_team()].reset_objectives_changed();
 	}
 }
 
@@ -744,15 +911,16 @@ void play_controller::process_keyup_event(const SDL_Event& event) {
 			const unit_map::iterator u = mouse_handler_.selected_unit();
 
 			if(u != units_.end()) {
-				const bool teleport = u->second.get_ability_bool("teleport",u->first);
+				bool teleport = u->second.get_ability_bool("teleport");
 
 				// if it's not the unit's turn, we reset its moves
 				unit_movement_resetter move_reset(u->second, u->second.side() != player_number_);
 
-				mouse_handler_.set_current_paths(paths(map_,units_,u->first,
+				mouse_handler_.set_current_paths(pathfind::paths(map_,units_,u->first,
 				                       teams_,false,teleport, teams_[gui_->viewing_team()],
 				                       mouse_handler_.get_path_turns()));
-				gui_->highlight_reach(mouse_handler_.get_current_paths());
+
+				gui_->highlight_reach(mouse_handler_.current_paths());
 			}
 		}
 	}
@@ -786,9 +954,9 @@ void play_controller::expand_autosaves(std::vector<std::string>& items)
 			items.erase(items.begin() + i);
 			std::vector<std::string> newitems;
 			std::vector<std::string> newsaves;
-			for (unsigned int turn = status_.turn(); turn != 0; turn--) {
-				std::string name = gamestate_.label + "-" + _("Auto-Save") + lexical_cast<std::string>(turn);
-				if (save_game_exists(name)) {
+			for (unsigned int turn = this->turn(); turn != 0; turn--) {
+				std::string name = gamestate_.classification().label + "-" + _("Auto-Save") + lexical_cast<std::string>(turn);
+				if (savegame::manager::save_game_exists(name, preferences::compress_saves())) {
 					if(preferences::compress_saves()) {
 						newsaves.push_back(name + ".gz");
 					} else {
@@ -836,9 +1004,9 @@ void play_controller::expand_wml_commands(std::vector<std::string>& items)
 				config& show_if = itor->second->show_if;
 				config filter_location = itor->second->filter_location;
 				if ((show_if.empty()
-					|| game_events::conditional_passed(&units_, &show_if))
+					|| game_events::conditional_passed(&units_, vconfig(show_if)))
 				&& (filter_location.empty()
-					|| terrain_filter(&filter_location, map_, status_, units_)(hex))
+					|| terrain_filter(vconfig(filter_location), units_)(hex))
 				&& (!itor->second->needs_select
 					|| gamestate_.last_selected.valid()))
 				{
@@ -904,12 +1072,14 @@ bool play_controller::in_context_menu(hotkey::HOTKEY_COMMAND command) const
 	case hotkey::HOTKEY_RECALL: {
 		// last_hex_ is set by mouse_events::mouse_motion
 		// Enable recruit/recall on castle/keep tiles
-		const unit_map::const_iterator leader = team_leader(player_number_,units_);
-		if (leader != units_.end()) {
-			return can_recruit_on(map_, leader->first, mouse_handler_.get_last_hex());
-		} else {
-			return false;
+		for(unit_map::const_iterator leader = units_.begin();
+				leader != units_.end();++leader) {
+			if (leader->second.can_recruit() &&
+				static_cast<int>(leader->second.side()) == player_number_ &&
+				can_recruit_on(map_, leader->first, mouse_handler_.get_last_hex()))
+				return true;
 		}
+		return false;
 	}
 	default:
 		return true;
@@ -931,7 +1101,7 @@ hotkey::ACTION_STATE play_controller::get_action_state(hotkey::HOTKEY_COMMAND co
 {
 	switch(command) {
 	case hotkey::HOTKEY_DELAY_SHROUD:
-		return current_team().auto_shroud_updates() ? hotkey::ACTION_OFF : hotkey::ACTION_ON;
+		return teams_[gui_->viewing_team()].auto_shroud_updates() ? hotkey::ACTION_OFF : hotkey::ACTION_ON;
 	default:
 		return hotkey::ACTION_STATELESS;
 	}
@@ -946,7 +1116,7 @@ const std::string& play_controller::select_victory_music() const
 	if(victory_music_.empty())
 		return empty_str;
 
-	const size_t p = gamestate_.rng().get_random() % victory_music_.size();
+	const size_t p = gamestate_.rng().get_next_random() % victory_music_.size();
 	assert(p < victory_music_.size());
 	return victory_music_[p];
 }
@@ -956,7 +1126,7 @@ const std::string& play_controller::select_defeat_music() const
 	if(defeat_music_.empty())
 		return empty_str;
 
-	const size_t p = gamestate_.rng().get_random() % defeat_music_.size();
+	const size_t p = gamestate_.rng().get_next_random() % defeat_music_.size();
 	assert(p < defeat_music_.size());
 	return defeat_music_[p];
 }
@@ -976,3 +1146,83 @@ void play_controller::set_defeat_music_list(const std::string& list)
 		defeat_music_ = utils::split(game_config::default_defeat_music);
 }
 
+void play_controller::check_victory()
+{
+	check_end_level();
+
+	std::vector<unsigned> seen_leaders;
+	for (unit_map::const_iterator i = units_.begin(),
+	     i_end = units_.end(); i != i_end; ++i)
+	{
+		if (i->second.can_recruit()) {
+			DBG_NG << "seen leader for side " << i->second.side() << "\n";
+			seen_leaders.push_back(i->second.side());
+		}
+	}
+
+	// Clear villages for teams that have no leader
+	for (std::vector<team>::iterator tm_beg = teams_.begin(), tm = tm_beg,
+	     tm_end = teams_.end(); tm != tm_end; ++tm)
+	{
+		if (std::find(seen_leaders.begin(), seen_leaders.end(), tm - tm_beg + 1) == seen_leaders.end()) {
+			tm->clear_villages();
+			// invalidate_all() is overkill and expensive but this code is
+			// run rarely so do it the expensive way.
+			gui_->invalidate_all();
+		}
+	}
+
+	bool found_player = false;
+
+	for (size_t n = 0; n != seen_leaders.size(); ++n) {
+		size_t side = seen_leaders[n] - 1;
+
+		for (size_t m = n +1 ; m != seen_leaders.size(); ++m) {
+			if (teams_[side].is_enemy(seen_leaders[m])) {
+				return;
+			}
+		}
+
+		if (teams_[side].is_human()) {
+			found_player = true;
+		}
+	}
+
+	if (found_player) {
+		game_events::fire("enemies defeated");
+		check_end_level();
+	}
+
+	if (!victory_when_enemies_defeated_ && (found_player || is_observer())) {
+		// This level has asked not to be ended by this condition.
+		return;
+	}
+
+	if (non_interactive()) {
+		std::cout << "winner: ";
+		foreach (unsigned l, seen_leaders) {
+			std::string ai = ai::manager::get_active_ai_identifier_for_side(l);
+			if (ai.empty()) ai = "default ai";
+			std::cout << l << " (using " << ai << ") ";
+		}
+		std::cout << '\n';
+		ai_testing::log_victory(seen_leaders);
+	}
+
+	DBG_NG << "throwing end level exception...\n";
+	throw end_level_exception(found_player ? VICTORY : DEFEAT);
+}
+
+void play_controller::process_oos(const std::string& msg) const
+{
+	if (game_config::ignore_replay_errors) return;
+
+	/** @todo FIXME: activate translation support after string freeze */
+	std::stringstream message;
+	message << "The game is out of sync. It might not make much sense to continue. Do you want to save your game?";
+	message << "\n\nError details:\n\n" << msg;
+
+	savegame::oos_savegame save(to_config());
+	save.save_game_interactive(resources::screen->video(), message.str(), gui::YES_NO); // can throw end_level_exception
+}
+
