@@ -1,7 +1,7 @@
-/* $Id: playcampaign.cpp 32891 2009-02-17 22:53:25Z jhinrichs $ */
+/* $Id: playcampaign.cpp 44384 2010-07-19 17:54:40Z silene $ */
 /*
    Copyright (C) 2003-2005 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2009 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2010 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,10 @@
 
 #include "global.hpp"
 
+#include "foreach.hpp"
+#include "game_preferences.hpp"
+#include "gui/dialogs/message.hpp"
+#include "gui/dialogs/transient_message.hpp"
 #include "playcampaign.hpp"
 #include "map_create.hpp"
 #include "playmp_controller.hpp"
@@ -28,13 +32,15 @@
 #include "map_exception.hpp"
 #include "dialogs.hpp"
 #include "gettext.hpp"
+#include "savegame.hpp"
 #include "sound.hpp"
 #include "wml_exception.hpp"
 #include "formula_string_utils.hpp"
 
+#define LOG_G LOG_STREAM(info, lg::general)
 
-#define LOG_G LOG_STREAM(info, general)
-#define LOG_NG LOG_STREAM(info, engine)
+static lg::log_domain log_engine("engine");
+#define LOG_NG LOG_STREAM(info, log_engine)
 
 namespace {
 
@@ -58,92 +64,113 @@ typedef std::map<std::string, player_controller> controller_map;
 
 } // end anon namespace
 
+/** Marks all the toplevel [lua] tag as preload events. */
+static void preload_lua_tags(const config &game_config, config &target)
+{
+	foreach (const config &cfg, game_config.child_range("lua"))
+	{
+		config &ev = target.add_child("event");
+		ev["name"] = "preload";
+		ev["first_time_only"] = "no";
+		ev.add_child("lua", cfg);
+	}
+}
+
 void play_replay(display& disp, game_state& gamestate, const config& game_config,
 		CVideo& video)
 {
-	std::string type = gamestate.campaign_type;
+	std::string type = gamestate.classification().campaign_type;
 	if(type.empty())
 		type = "scenario";
 
-	config const* scenario = NULL;
-
 	// 'starting_pos' will contain the position we start the game from.
 	config starting_pos;
 
 	if (gamestate.starting_pos.empty()){
 		// Backwards compatibility code for 1.2 and 1.2.1
-		scenario = game_config.find_child(type,"id",gamestate.scenario);
-		assert(scenario != NULL);
-		gamestate.starting_pos = *scenario;
+		const config &scenario = game_config.find_child(type,"id",gamestate.classification().scenario);
+		assert(scenario);
+		gamestate.starting_pos = scenario;
 	}
-	recorder.set_save_info(gamestate);
+
+	// Detect replays stored by the server, since they are missing core lua tags.
+	if (const config &core = game_config.child("lua"))
+	{
+		bool found_core = false;
+		foreach (const config &cfg, gamestate.starting_pos.child_range("lua")) {
+			if (cfg["code"] == core["code"]) {
+				found_core = true;
+				break;
+			}
+		}
+		if (!found_core)
+			preload_lua_tags(game_config, gamestate.starting_pos);
+	}
+
 	starting_pos = gamestate.starting_pos;
-	scenario = &starting_pos;
+
+	//for replays, use the variables specified in starting_pos
+	if (const config &vars = starting_pos.child("variables")) {
+		gamestate.set_variables(vars);
+	}
 
 	try {
 		// Preserve old label eg. replay
-		if (gamestate.label.empty())
-			gamestate.label = (*scenario)["name"];
+		if (gamestate.classification().label.empty())
+			gamestate.classification().label = starting_pos["name"];
 		//if (gamestate.abbrev.empty())
 		//	gamestate.abbrev = (*scenario)["abbrev"];
 
-		play_replay_level(game_config,scenario,video,gamestate);
+		play_replay_level(game_config, &starting_pos, video, gamestate);
 
 		gamestate.snapshot = config();
 		recorder.clear();
 		gamestate.replay_data.clear();
 
 	} catch(game::load_game_failed& e) {
-		gui::show_error_message(disp, _("The game could not be loaded: ") + e.message);
+		gui2::show_error_message(disp.video(), _("The game could not be loaded: ") + e.message);
 	} catch(game::game_error& e) {
-		gui::show_error_message(disp, _("Error while playing the game: ") + e.message);
+		gui2::show_error_message(disp.video(), _("Error while playing the game: ") + e.message);
 	} catch(incorrect_map_format_exception& e) {
-		gui::show_error_message(disp, std::string(_("The game map could not be loaded: ")) + e.msg_);
+		gui2::show_error_message(disp.video(), std::string(_("The game map could not be loaded: ")) + e.msg_);
 	} catch(twml_exception& e) {
 		e.show(disp);
 	}
 }
 
-static void clean_saves(const std::string &label)
-{
-	std::vector<save_info> games = get_saves_list();
-	std::string prefix = label + "-" + _("Auto-Save");
-	std::cerr << "Cleaning saves with prefix '" << prefix << "'\n";
-	for (std::vector<save_info>::iterator i = games.begin(); i != games.end(); i++) {
-		if (i->name.compare(0, prefix.length(), prefix) == 0) {
-			std::cerr << "Deleting savegame '" << i->name << "'\n";
-			delete_game(i->name);
-		}
-	}
-}
-
 static LEVEL_RESULT playsingle_scenario(const config& game_config,
 		const config* level, display& disp, game_state& state_of_game,
-		const std::vector<config*>& story, upload_log& log, bool skip_replay, end_level_exception *end_level)
+		const config::const_child_itors &story, 
+		bool skip_replay, end_level_data &end_level)
 {
 	const int ticks = SDL_GetTicks();
 	const int num_turns = atoi((*level)["turns"].c_str());
+
 	LOG_NG << "creating objects... " << (SDL_GetTicks() - ticks) << "\n";
 	playsingle_controller playcontroller(*level, state_of_game, ticks, num_turns, game_config, disp.video(), skip_replay);
 	LOG_NG << "created objects... " << (SDL_GetTicks() - playcontroller.get_ticks()) << "\n";
 
-	const LEVEL_RESULT res = playcontroller.play_scenario(story, log, skip_replay, end_level);
+	LEVEL_RESULT res = playcontroller.play_scenario(story, skip_replay);
+
+	end_level = playcontroller.get_end_level_data();
 
 	if (res == DEFEAT) {
-		gui::message_dialog(disp,
+		gui2::show_transient_message(disp.video(),
 				    _("Defeat"),
 				    _("You have been defeated!")
-				    ).show();
+				    );
 	}
 
-	if (!disp.video().faked() && res != QUIT && end_level->linger_mode)
+	if (!disp.video().faked() && res != QUIT && end_level.linger_mode)
+	{
 		try {
-			playcontroller.linger(log);
+			playcontroller.linger();
 		} catch(end_level_exception& e) {
 			if (e.result == QUIT) {
 				return QUIT;
 			}
 		}
+	}
 
 	return res;
 }
@@ -151,28 +178,29 @@ static LEVEL_RESULT playsingle_scenario(const config& game_config,
 
 static LEVEL_RESULT playmp_scenario(const config& game_config,
 		config const* level, display& disp, game_state& state_of_game,
-		const config::child_list& story, upload_log& log, bool skip_replay,
-		io_type_t& io_type, end_level_exception *end_level)
+		const config::const_child_itors &story, bool skip_replay,
+		io_type_t& io_type, end_level_data &end_level)
 {
 	const int ticks = SDL_GetTicks();
 	const int num_turns = atoi((*level)["turns"].c_str());
 	playmp_controller playcontroller(*level, state_of_game, ticks, num_turns,
 		game_config, disp.video(), skip_replay, io_type == IO_SERVER);
-	const LEVEL_RESULT res = playcontroller.play_scenario(story, log, skip_replay, end_level);
+	LEVEL_RESULT res = playcontroller.play_scenario(story, skip_replay);
 
+	end_level = playcontroller.get_end_level_data();
 	//Check if the player started as mp client and changed to host
 	if (io_type == IO_CLIENT && playcontroller.is_host())
 		io_type = IO_SERVER;
 
 	if (res == DEFEAT) {
-		gui::message_dialog(disp,
+		gui2::show_transient_message(disp.video(),
 				    _("Defeat"),
 				    _("You have been defeated!")
-				    ).show();
+				    );
 	}
 
 	if (!disp.video().faked() && res != QUIT) {
-		if(!end_level->linger_mode) {
+		if (!end_level.linger_mode) {
 			if(!playcontroller.is_host()) {
 				// If we continue without lingering we need to
 				// make sure the host uploads the next scenario
@@ -181,7 +209,7 @@ static LEVEL_RESULT playmp_scenario(const config& game_config,
 			}
 		} else {
 			try {
-				playcontroller.linger(log);
+				playcontroller.linger();
 			} catch(end_level_exception& e) {
 				if (e.result == QUIT) {
 					return QUIT;
@@ -194,9 +222,9 @@ static LEVEL_RESULT playmp_scenario(const config& game_config,
 }
 
 LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_config,
-		upload_log &log, io_type_t io_type, bool skip_replay)
+		               io_type_t io_type, bool skip_replay)
 {
-	std::string type = gamestate.campaign_type;
+	std::string type = gamestate.classification().campaign_type;
 	if(type.empty())
 		type = "scenario";
 
@@ -205,29 +233,33 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 	// 'starting_pos' will contain the position we start the game from.
 	config starting_pos;
 
-	recorder.set_save_info(gamestate);
-
 	// Do we have any snapshot data?
 	// yes => this must be a savegame
 	// no  => we are starting a fresh scenario
 	if(gamestate.snapshot.child("side") == NULL || !recorder.at_end()) {
-		gamestate.completion = "running";
-		recorder.set_save_info_completion(gamestate.completion);
+		gamestate.classification().completion = "running";
 		// Campaign or Multiplayer?
 		// If the gamestate already contains a starting_pos,
 		// then we are starting a fresh multiplayer game.
 		// Otherwise this is the start of a campaign scenario.
-		if(gamestate.starting_pos.empty() == false) {
+		if(gamestate.starting_pos["id"].empty() == false) {
 			LOG_G << "loading starting position...\n";
+			preload_lua_tags(game_config, gamestate.starting_pos);
 			starting_pos = gamestate.starting_pos;
 			scenario = &starting_pos;
 		} else {
-			LOG_G << "loading scenario: '" << gamestate.scenario << "'\n";
-			scenario = game_config.find_child(type,"id",gamestate.scenario);
-			if(scenario) {
+			//reload of the scenario, as starting_pos contains carryover information only
+			LOG_G << "loading scenario: '" << gamestate.classification().scenario << "'\n";
+			scenario = &game_config.find_child(type, "id", gamestate.classification().scenario);
+			if (*scenario) {
 				starting_pos = *scenario;
-				gamestate.starting_pos = *scenario;
-			}
+				preload_lua_tags(game_config, starting_pos);
+				config temp(starting_pos);
+				write_players(gamestate, temp, false, true);
+				gamestate.starting_pos = temp;
+				scenario = &starting_pos;
+			} else
+				scenario = NULL;
 			LOG_G << "scenario found: " << (scenario != NULL ? "yes" : "no") << "\n";
 		}
 	} else {
@@ -237,59 +269,38 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 		scenario = &gamestate.snapshot;
 		// When starting wesnoth --multiplayer there might be
 		// no variables which leads to a segfault
-		if(gamestate.snapshot.child("variables") != NULL) {
-			gamestate.set_variables(*gamestate.snapshot.child("variables"));
+		if (const config &vars = gamestate.snapshot.child("variables")) {
+			gamestate.set_variables(vars);
 		}
-		gamestate.set_menu_items(gamestate.snapshot.get_children("menu_item"));
+		gamestate.set_menu_items(gamestate.snapshot.child_range("menu_item"));
 		// Replace game label with that from snapshot
 		if (!gamestate.snapshot["label"].empty()){
-			gamestate.label = gamestate.snapshot["label"];
+			gamestate.classification().label = gamestate.snapshot["label"];
 		}
-		{
-			// Get the current gold values of players, so they don't start
-			// with the amount they had at the start of the scenario
-			const std::vector<config*>& player_cfg = gamestate.snapshot.get_children("player");
-			for (std::vector<config*>::const_iterator p = player_cfg.begin(); p != player_cfg.end(); p++){
-				std::string save_id = (**p)["save_id"];
-				player_info* player = gamestate.get_player(save_id);
-				if (player != NULL){
-					player->gold = lexical_cast <int> ((**p)["gold"]);
-				}
-			}
-		}
-		{
-			// Also get the recruitment list if there are some specialties in this scenario
-			const std::vector<config*>& player_cfg = gamestate.snapshot.get_children("side");
-			for (std::vector<config*>::const_iterator p = player_cfg.begin(); p != player_cfg.end(); p++){
-				std::string save_id = (**p)["save_id"];
-				player_info* player = gamestate.get_player(save_id);
-				if (player != NULL){
-					const std::string& can_recruit_str = (**p)["recruit"];
-					if(can_recruit_str != "") {
-						player->can_recruit.clear();
-						const std::vector<std::string> can_recruit = utils::split(can_recruit_str);
-						std::copy(can_recruit.begin(),can_recruit.end(),std::inserter(player->can_recruit,player->can_recruit.end()));
-					}
-				}
-			}
+		// Helper for transitioning middle-of-scenario savefiles from 1.6 to 1.8.
+		// To be removed for 1.10.
+		if (!scenario->find_child("event", "name", "preload")) {
+			LOG_NG << "Adding dummy preload event.\n";
+			config &ev = gamestate.snapshot.add_child("event");
+			ev["name"] = "preload";
+			ev["first_time_only"] = "no";
+			config &lua = ev.add_child("lua");
+			lua["code"] = "wesnoth.dofile 'lua/wml-tags.lua'";
 		}
 	}
 
 	controller_map controllers;
 
 	if(io_type == IO_SERVER) {
-		const config::child_list& sides_list = scenario->get_children("side");
-		for(config::child_list::const_iterator side = sides_list.begin();
-				side != sides_list.end(); ++side) {
-			if ((**side)["current_player"] == preferences::login())
-			{
-				(**side)["controller"] = preferences::client_type();
+		foreach (config &side, const_cast<config *>(scenario)->child_range("side"))
+		{
+			if (side["current_player"] == preferences::login()) {
+				side["controller"] = preferences::client_type();
 			}
-			std::string id = (**side)["save_id"];
+			std::string id = side["save_id"];
 			if(id.empty())
 				continue;
-			controllers[id] = player_controller((**side)["controller"],
-					(**side)["id"]);
+			controllers[id] = player_controller(side["controller"], side["id"]);
 		}
 	}
 
@@ -301,36 +312,35 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 				scenario = &starting_pos;
 			}
 
-			const config::child_list& sides_list = starting_pos.get_children("side");
-			for(config::child_list::const_iterator side = sides_list.begin();
-					side != sides_list.end(); ++side) {
-				if((**side)["current_player"] == preferences::login()) {
-					(**side)["controller"] = preferences::client_type();
-				} else if((**side)["controller"] != "null") {
+			foreach (config &side, starting_pos.child_range("side"))
+			{
+				if (side["current_player"] == preferences::login()) {
+					side["controller"] = preferences::client_type();
+				} else if (side["controller"] != "null") {
 					// @todo: Fix logic to use network_ai controller
 					// if it is networked ai
-					(**side)["controller"] = "network";
+					side["controller"] = "network";
 				}
 			}
 		}
 
-		const config::child_list& story = scenario->get_children("story");
-		gamestate.next_scenario = (*scenario)["next_scenario"];
+		config::const_child_itors story = scenario->child_range("story");
+		gamestate.classification().next_scenario = (*scenario)["next_scenario"];
 
 		bool save_game_after_scenario = true;
 
 		LEVEL_RESULT res = VICTORY;
-		end_level_exception *end_level = new end_level_exception(VICTORY);
+		end_level_data end_level;
 
 		try {
 			// Preserve old label eg. replay
-			if (gamestate.label.empty()) {
-				if (gamestate.abbrev.empty())
-					gamestate.label = (*scenario)["name"];
+			if (gamestate.classification().label.empty()) {
+				if (gamestate.classification().abbrev.empty())
+					gamestate.classification().label = (*scenario)["name"];
 				else {
-					gamestate.label = std::string(gamestate.abbrev);
-					gamestate.label.append("-");
-					gamestate.label.append((*scenario)["name"]);
+					gamestate.classification().label = std::string(gamestate.classification().abbrev);
+					gamestate.classification().label.append("-");
+					gamestate.classification().label.append((*scenario)["name"]);
 				}
 			}
 
@@ -342,8 +352,11 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 				static config scenario2;
 				scenario2 = random_generate_scenario((*scenario)["scenario_generation"], scenario->child("generator"));
 				//level_ = scenario;
-
-				gamestate.starting_pos = scenario2;
+				preload_lua_tags(game_config, scenario2);
+				//merge carryover information into the newly generated scenario
+				config temp(scenario2);
+				write_players(gamestate, temp, false, true);
+				gamestate.starting_pos = temp;
 				scenario = &scenario2;
 			}
 			std::string map_data = (*scenario)["map_data"];
@@ -361,76 +374,66 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 				// it will not ask for the map to be generated again on reload
 				static config new_level;
 				new_level = *scenario;
-				new_level.values["map_data"] = map_data;
+				new_level["map_data"] = map_data;
 				scenario = &new_level;
 
-				gamestate.starting_pos = new_level;
+				//merge carryover information into the scenario
+				config temp(new_level);
+				write_players(gamestate, temp, false, true);
+				gamestate.starting_pos = temp;
 				LOG_G << "generated map\n";
 			}
 
 			sound::empty_playlist();
 
+			//add the variables to the starting_pos unless they are already there
+			const config &wmlvars = gamestate.starting_pos.child("variables");
+			if (!wmlvars || wmlvars.empty()){
+				gamestate.starting_pos.clear_children("variables");
+				gamestate.starting_pos.add_child("variables", gamestate.get_variables());
+			}
+
 			switch (io_type){
 			case IO_NONE:
-				res = playsingle_scenario(game_config,scenario,disp,gamestate,story,log, skip_replay, end_level);
+				res = playsingle_scenario(game_config, scenario, disp, gamestate, story, skip_replay, end_level);
 				break;
 			case IO_SERVER:
 			case IO_CLIENT:
-				res = playmp_scenario(game_config,scenario,disp,gamestate,story,log, skip_replay, io_type, end_level);
+				res = playmp_scenario(game_config, scenario, disp, gamestate, story, skip_replay, io_type, end_level);
 				break;
 			}
 		} catch(game::load_game_failed& e) {
-			gui::show_error_message(disp, _("The game could not be loaded: ") + e.message);
+			gui2::show_error_message(disp.video(), _("The game could not be loaded: ") + e.message);
 			return QUIT;
 		} catch(game::game_error& e) {
-			gui::show_error_message(disp, _("Error while playing the game: ") + e.message);
+			gui2::show_error_message(disp.video(), _("Error while playing the game: ") + e.message);
 			return QUIT;
 		} catch(incorrect_map_format_exception& e) {
-			gui::show_error_message(disp, std::string(_("The game map could not be loaded: ")) + e.msg_);
+			gui2::show_error_message(disp.video(), std::string(_("The game map could not be loaded: ")) + e.msg_);
 			return QUIT;
 		} catch(config::error& e) {
 			std::cerr << "caught config::error...\n";
-			gui::show_error_message(disp, _("Error while reading the WML: ") + e.message);
+			gui2::show_error_message(disp.video(), _("Error while reading the WML: ") + e.message);
 			return QUIT;
 		} catch(twml_exception& e) {
 			e.show(disp);
 			return QUIT;
 		}
 
-		gamestate.snapshot = config();
 
-		// Save-nagement ioptions fire on game end.
+		// Save-management options fire on game end.
 		// This means: (a) we have a victory, or
 		// or (b) we're multiplayer live, in which
 		// case defeat is also game end.  Someday,
 		// if MP campaigns ever work again, we might
 		// need to change this test.
-
-		// OBSERVER_END probably can be removed here if the observer disconnect
-		// bug (#10077) is fixed so that when the host ends the game observers
-		// again get asked if they want to save a replay of the game
-		if (res == VICTORY || (io_type != IO_NONE && (res == DEFEAT || res == OBSERVER_END))) {
+		if (res == VICTORY || (io_type != IO_NONE && res == DEFEAT)) {
 			if (preferences::delete_saves())
-				clean_saves(gamestate.label);
+				savegame::manager::clean_saves(gamestate.classification().label);
 
 			if (preferences::save_replays()) {
-				std::string label = gamestate.label + _(" replay");
-				if (dialogs::get_save_name(disp, "", _("Name: "), &label,
-					gui::OK_CANCEL, _("Save Replay"), false, false) == 0) {
-				try {
-						config snapshot;
-						//If the starting position contains player information, use this for
-						//the replay savegame (this originally comes from the gamestate constructor,
-						//where the player stuff is added to the starting position to be used here.
-						config::child_list player_list = gamestate.starting_pos.get_children("player");
-						if (player_list.size() > 0) {
-							recorder.set_save_info(gamestate, player_list);
-						}
-						recorder.save_game(label, snapshot, gamestate.starting_pos);
-					} catch(game::save_game_failed&) {
-						gui::show_error_message(disp, _("The replay could not be saved"));
-					}
-				}
+				savegame::replay_savegame save(gamestate, preferences::compress_saves());
+				save.save_game_automatic(disp.video(), true);
 			}
 		}
 
@@ -440,27 +443,34 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 		// On DEFEAT, QUIT, or OBSERVER_END, we're done now
 		if (res != VICTORY)
 		{
-			if (res != OBSERVER_END || gamestate.next_scenario.empty())
+			if (res != OBSERVER_END || gamestate.classification().next_scenario.empty()) {
+				gamestate.snapshot = config();
 				return res;
+			}
 
 			const int dlg_res = gui::dialog(disp,"Game Over",
 				_("This scenario has ended. Do you want to continue the campaign?"),
 				gui::YES_NO).show();
-			if (dlg_res != 0)
+			if (dlg_res != 0) {
+				gamestate.snapshot = config();
 				return res;
+			}
 		}
 
 		// Continue without saving is like a victory,
 		// but the save game dialog isn't displayed
-		if(!end_level->save)
+		if (!end_level.prescenario_save)
 			save_game_after_scenario = false;
 
 		// Switch to the next scenario.
-		gamestate.scenario = gamestate.next_scenario;
+		gamestate.classification().scenario = gamestate.classification().next_scenario;
 		gamestate.rng().rotate_random();
 
 		if(io_type == IO_CLIENT) {
-			if (gamestate.next_scenario.empty()) return res;
+			if (gamestate.classification().next_scenario.empty()) {
+				gamestate.snapshot = config();
+				return res;
+			}
 
 			// Ask for the next scenario data.
 			network::send_data(config("load_next_scenario"), 0, true);
@@ -470,44 +480,54 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 				cfg.clear();
 				network::connection data_res = dialogs::network_receive_dialog(disp,
 						msg, cfg);
-				if(!data_res) return QUIT;
-			} while(cfg.child("next_scenario") == NULL);
+				if(!data_res) {
+					gamestate.snapshot = config();
+					return QUIT;
+				}
+			} while (!cfg.child("next_scenario"));
 
-			if(cfg.child("next_scenario")) {
-				starting_pos = (*cfg.child("next_scenario"));
+			if (const config &c = cfg.child("next_scenario")) {
+				starting_pos = c;
 				scenario = &starting_pos;
 				gamestate = game_state(starting_pos);
 			} else {
+				gamestate.snapshot = config();
 				return QUIT;
 			}
 		} else {
-			scenario = game_config.find_child(type,"id",gamestate.scenario);
+			scenario = &game_config.find_child(type, "id", gamestate.classification().scenario);
+			if (!*scenario)
+				scenario = NULL;
+			else
+			{
+				starting_pos = *scenario;
+				preload_lua_tags(game_config, starting_pos);
+				scenario = &starting_pos;
+			}
 
 			if(io_type == IO_SERVER && scenario != NULL) {
-				starting_pos = *scenario;
-				scenario = &starting_pos;
-
 				// Tweaks sides to adapt controllers and descriptions.
-				const config::child_list& sides_list = starting_pos.get_children("side");
-				for(config::child_list::const_iterator side = sides_list.begin();
-						side != sides_list.end(); ++side) {
-					std::string id = (**side)["save_id"];
+				foreach (config &side, starting_pos.child_range("side"))
+				{
+					std::string id = side["save_id"];
 					if(id.empty()) {
-						continue;
+						id = side["id"];
 					}
-
-					/* Update side info to match current_player info
-					 * to allow it taking the side in next scenario
-					 * and to be set in the players list on side server
-					 */
-					controller_map::const_iterator ctr = controllers.find(id);
-					if(ctr != controllers.end()) {
-						player_info *player = gamestate.get_player(id);
-						if (player) {
-							(**side)["current_player"] = player->name;
+					if(!id.empty()) {
+						/* Update side info to match current_player info
+						 * to allow it taking the side in next scenario
+						 * and to be set in the players list on side server
+						 */
+						controller_map::const_iterator ctr = controllers.find(id);
+						if(ctr != controllers.end()) {
+							if (const config& c = gamestate.snapshot.find_child("side", "save_id", id)) {
+								side["current_player"] = c["current_player"];
+							}
+							side["controller"] = ctr->second.controller;
 						}
-						(**side)["controller"] = ctr->second.controller;
 					}
+					if (side["controller"].empty())
+						side["controller"] = "ai";
 				}
 
 				// If the entire scenario should be randomly generated
@@ -518,7 +538,7 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 					static config scenario2;
 					scenario2 = random_generate_scenario((*scenario)["scenario_generation"], scenario->child("generator"));
 					//level_ = scenario;
-
+					preload_lua_tags(game_config, scenario2);
 					gamestate.starting_pos = scenario2;
 					scenario = &scenario2;
 				}
@@ -537,7 +557,7 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 					// it will not ask for the map to be generated again on reload
 					static config new_level;
 					new_level = *scenario;
-					new_level.values["map_data"] = map_data;
+					new_level["map_data"] = map_data;
 					scenario = &new_level;
 
 					gamestate.starting_pos = new_level;
@@ -546,27 +566,37 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 
 				// Sends scenario data
 				config cfg;
-				cfg.add_child("store_next_scenario", *scenario);
-				//Add the player section to the starting position so we can get the correct recall list
-				//when loading the replay later on
-				write_players(gamestate, gamestate.starting_pos);
+				config& next_cfg = cfg.add_child("store_next_scenario", *scenario);
+
 				// Adds player information, and other state
 				// information, to the configuration object
-				assert(cfg.child("store_next_scenario") != NULL);
-				write_game(gamestate, *cfg.child("store_next_scenario"), WRITE_SNAPSHOT_ONLY);
+				gamestate.write_snapshot(next_cfg);
+				next_cfg["next_scenario"] = (*scenario)["next_scenario"];
+				next_cfg.add_child("snapshot");
+				//move the player information into the hosts gamestate
+				write_players(gamestate, starting_pos, true, true);
+				gamestate.starting_pos = *scenario;
+
+				next_cfg.add_child("multiplayer", gamestate.mp_settings().to_config());
+				next_cfg.add_child("replay_start", gamestate.starting_pos);
+				//move side information from gamestate into the config that is sent to the other clients
+				next_cfg.clear_children("side");
+				foreach (config& side, gamestate.starting_pos.child_range("side"))
+					next_cfg.add_child("side", side);
+
 				network::send_data(cfg, 0, true);
 			}
 		}
 
 		if(scenario != NULL) {
 			// Update the label
-			std::string oldlabel = gamestate.label;
-			if (gamestate.abbrev.empty())
-				gamestate.label = (*scenario)["name"];
+			std::string oldlabel = gamestate.classification().label;
+			if (gamestate.classification().abbrev.empty())
+				gamestate.classification().label = (*scenario)["name"];
 			else {
-				gamestate.label = std::string(gamestate.abbrev);
-				gamestate.label.append("-");
-				gamestate.label.append((*scenario)["name"]);
+				gamestate.classification().label = std::string(gamestate.classification().abbrev);
+				gamestate.classification().label.append("-");
+				gamestate.classification().label.append((*scenario)["name"]);
 			}
 
 			// If this isn't the last scenario, then save the game
@@ -578,56 +608,51 @@ LEVEL_RESULT play_game(display& disp, game_state& gamestate, const config& game_
 				// start-of-scenario save and the
 				// starting position needs to be empty,
 				// to force a reload of the scenario config.
-				if (gamestate.campaign_type == "multiplayer"){
-					gamestate.starting_pos = *scenario;
-				}
-				else{
+				if (gamestate.classification().campaign_type != "multiplayer"){
 					gamestate.starting_pos = config();
 				}
 
+				//add the variables to the starting position
+				gamestate.starting_pos.add_child("variables", gamestate.get_variables());
+
+				savegame::scenariostart_savegame save(gamestate, preferences::compress_saves());
+
+#ifdef USE_TINY_GUI
 				bool retry = true;
 
 				while(retry) {
 					retry = false;
-
-#ifdef TINY_GUI
-					const int should_save = dialogs::get_save_name(disp,
-						_("Do you want to save your game?"),
-						_("Name:"),
-						&gamestate.label);
-					if(should_save == 0)
-#endif /* TINY_GUI */
-					{
-						try {
-							save_game(gamestate);
-						} catch(game::save_game_failed&) {
-							gui::show_error_message(disp, _("The game could not be saved"));
-							retry = true;
-						}
-					}
+					retry = !save.save_game_interactive(disp.video()
+									, _("Do you want to save your game?")
+									, gui::YES_NO);
 				}
+#else
+				save.save_game_automatic(disp.video());
+#endif /* USE_TINY_GUI */
 			}
 
-			if (gamestate.campaign_type != "multiplayer"){
+			if (gamestate.classification().campaign_type != "multiplayer"){
 				gamestate.starting_pos = *scenario;
+				//add the variables to the starting position
+				gamestate.starting_pos.add_child("variables", gamestate.get_variables());
+				write_players(gamestate, gamestate.starting_pos, true, true);
 			}
 		}
-
-		recorder.set_save_info(gamestate);
+		gamestate.snapshot = config();
 	}
 
-	if (!gamestate.scenario.empty() && gamestate.scenario != "null") {
+	if (!gamestate.classification().scenario.empty() && gamestate.classification().scenario != "null") {
 		std::string message = _("Unknown scenario: '$scenario|'");
 		utils::string_map symbols;
-		symbols["scenario"] = gamestate.scenario;
+		symbols["scenario"] = gamestate.classification().scenario;
 		message = utils::interpolate_variables_into_string(message, &symbols);
-		gui::show_error_message(disp, message);
+		gui2::show_error_message(disp.video(), message);
 		return QUIT;
 	}
 
-	if (gamestate.campaign_type == "scenario"){
+	if (gamestate.classification().campaign_type == "scenario"){
 		if (preferences::delete_saves())
-			clean_saves(gamestate.label);
+			savegame::manager::clean_saves(gamestate.classification().label);
 	}
 	return VICTORY;
 }
