@@ -1,7 +1,7 @@
-/* $Id: playcampaign.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: playcampaign.hpp 42333 2010-04-30 23:22:12Z loonycyborg $ */
 /*
    Copyright (C) 2003-2005 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2009 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2010 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,6 @@ class display;
 class game_state;
 class config;
 class CVideo;
-struct upload_log;
 
 enum io_type_t {
 	IO_NONE,
@@ -34,7 +33,6 @@ enum io_type_t {
 
 LEVEL_RESULT play_game(display& disp, game_state& state,
 		const config& game_config,
-		upload_log &log,
 		io_type_t io_type=IO_NONE,
 		bool skip_replay = false);
 
