@@ -1,6 +1,6 @@
-/* $Id: playmp_controller.hpp 32690 2009-02-08 22:15:06Z jhinrichs $ */
+/* $Id: playmp_controller.hpp 42333 2010-04-30 23:22:12Z loonycyborg $ */
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -33,7 +33,7 @@ public:
 		bool skip_replay, bool is_host);
 	virtual ~playmp_controller();
 
-	bool is_host() { return is_host_; }
+	bool is_host() const { return is_host_; }
 
 	static unsigned int replay_last_turn() { return replay_last_turn_; }
 	static void set_replay_last_turn(unsigned int turn);
@@ -42,9 +42,10 @@ public:
 	void reset_countdown();
 	void think_about_countdown(int ticks);
 	void process(events::pump_info &info);
-	void linger(upload_log& log);
+	void linger();
 	/** Wait for the host to upload the next scenario. */
 	void wait_for_upload();
+	void process_oos(const std::string& err_msg) const;
 
 protected:
 	virtual void handle_generic_event(const std::string& name);
@@ -69,7 +70,6 @@ protected:
 	int beep_warning_time_;
 	mutable bool network_processing_stopped_;
 private:
-	void process_oos(const std::string& err_msg);
 	void set_end_scenario_button();
 	void reset_end_scenario_button();
 	static unsigned int replay_last_turn_;
