@@ -1,6 +1,6 @@
-/* $Id: playsingle_controller.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: playsingle_controller.hpp 42333 2010-04-30 23:22:12Z loonycyborg $ */
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -19,10 +19,8 @@
 #include "global.hpp"
 
 #include "cursor.hpp"
-#include "game_end_exceptions.hpp"
 #include "play_controller.hpp"
-
-struct upload_log;
+#include "replay.hpp"
 
 #include <vector>
 
@@ -31,9 +29,10 @@ class playsingle_controller : public play_controller
 public:
 	playsingle_controller(const config& level, game_state& state_of_game,
 		const int ticks, const int num_turns, const config& game_config, CVideo& video, bool skip_replay);
-        virtual ~playsingle_controller() ;
+	virtual ~playsingle_controller();
 
-	LEVEL_RESULT play_scenario(const std::vector<config*>& story, upload_log& log, bool skip_replay, end_level_exception* end_level = NULL);
+	LEVEL_RESULT play_scenario(const config::const_child_itors &story,
+		bool skip_replay);
 
 	virtual void handle_generic_event(const std::string& name);
 
@@ -44,11 +43,13 @@ public:
 	virtual void toggle_shroud_updates();
 	virtual void update_shroud_now();
 	virtual void end_turn();
+	virtual void force_end_turn();
 	virtual void rename_unit();
 	virtual void create_unit();
-	virtual void change_unit_side();
+	virtual void change_side();
 	virtual void label_terrain(bool);
 	virtual void continue_move();
+	virtual void add_waypoint();
 	virtual void unit_hold_position();
 	virtual void end_unit_turn();
 	virtual void user_command();
@@ -59,7 +60,11 @@ public:
 	virtual void user_command_2();
 	virtual void user_command_3();
 #endif
-	void linger(upload_log& log);
+	void linger();
+
+	virtual void force_end_level(LEVEL_RESULT res)
+	{ level_result_ = res; }
+	virtual void check_end_level();
 
 protected:
 	virtual void play_turn(bool no_save);
@@ -74,6 +79,8 @@ protected:
 	void play_ai_turn();
 	virtual void init_gui();
 	void check_time_over();
+	void store_recalls();
+	void store_gold(bool obs = false);
 
 	const cursor::setter cursor_setter;
 	std::deque<config> data_backlog_;
@@ -85,15 +92,11 @@ protected:
 	bool replaying_;
 	bool turn_over_;
 	bool skip_next_turn_;
+	LEVEL_RESULT level_result_;
 private:
-	void report_victory(std::stringstream& report,
-		    end_level_exception& end_level,
-		    int player_gold,
-		    int remaining_gold, int finishing_bonus_per_turn,
-		    int turns_left, int finishing_bonus);
-
-	std::vector<std::string> victory_music_;
-	std::vector<std::string> defeat_music_;
+	void report_victory(std::ostringstream &report, int player_gold,
+		int remaining_gold, int finishing_bonus_per_turn,
+		int turns_left, int finishing_bonus);
 };
 
 
