@@ -1,6 +1,6 @@
-/* $Id: playturn.cpp 34964 2009-04-16 10:22:19Z suokko $ */
+/* $Id: playturn.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,23 +15,34 @@
 #include "playturn.hpp"
 
 #include "construct_dialog.hpp"
+#include "foreach.hpp"
+#include "game_display.hpp"
 #include "game_end_exceptions.hpp"
 #include "game_preferences.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
 #include "replay.hpp"
+#include "resources.hpp"
+#include "rng.hpp"
 #include "formula_string_utils.hpp"
+#include "play_controller.hpp"
 
-turn_info::turn_info(game_state& state_of_game,
-                     const gamestatus& status, game_display& gui, gamemap& map,
-		     std::vector<team>& teams, unsigned int team_num, unit_map& units,
-			 replay_network_sender& replay_sender, undo_list& undo_stack)
-  : state_of_game_(state_of_game), status_(status),
-    gui_(gui), map_(map), teams_(teams), team_num_(team_num),
-    units_(units), undo_stack_(undo_stack),
-	replay_sender_(replay_sender), replay_error_("network_replay_error"),
-	host_transfer_("host_transfer")
-{}
+#include <ctime>
+
+static lg::log_domain log_network("network");
+#define ERR_NW LOG_STREAM(err, log_network)
+
+turn_info::turn_info(unsigned team_num, replay_network_sender &replay_sender, undo_list &undo_stack) :
+	team_num_(team_num), undo_stack_(undo_stack),
+	replay_sender_(replay_sender),
+	host_transfer_("host_transfer"), replay_()
+{
+	/**
+	 * We do network sync so [init_side] is transfered to network hosts
+	 */
+	if(network::nconnections() > 0)
+		send_data();
+}
 
 turn_info::~turn_info(){
 	undo_stack_.clear();
@@ -67,32 +78,33 @@ void turn_info::send_data()
 turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg,
 		network::connection from, std::deque<config>& backlog, bool skip_replay)
 {
-	if (cfg.child("message")) {
-		const config& cmessage = *cfg.child("message");
-		const int side = lexical_cast_default<int>(cmessage["side"],0);
-
-		gui_.add_chat_message(time(NULL), cmessage["sender"], side,
-				cmessage["message"], game_display::MESSAGE_PUBLIC,
-				preferences::message_bell());
-	}
-	if (cfg.child("whisper") != NULL /*&& is_observer()*/) {
-		const config& cwhisper = *cfg.child("whisper");
-		gui_.add_chat_message(time(NULL), "whisper: " + cwhisper["sender"], 0,
-				cwhisper["message"], game_display::MESSAGE_PRIVATE,
-				preferences::message_bell());
-	}
-	if(cfg.child("observer") != NULL) {
-		const config::child_list& observers = cfg.get_children("observer");
-		for(config::child_list::const_iterator ob = observers.begin(); ob != observers.end(); ++ob) {
-			gui_.add_observer((**ob)["name"]);
-		}
+	if (const config &rnd_seed = cfg.child("random_seed")) {
+		rand_rng::set_seed(lexical_cast<rand_rng::seed_t>(rnd_seed["seed"]));
+		//may call a callback function, see rand_rng::set_seed_callback
 	}
 
-	if(cfg.child("observer_quit") != NULL) {
-		const config::child_list& observers = cfg.get_children("observer_quit");
-		for(config::child_list::const_iterator ob = observers.begin(); ob != observers.end(); ++ob) {
-			gui_.remove_observer((**ob)["name"]);
-		}
+	if (const config &msg = cfg.child("message"))
+	{
+		const int side = lexical_cast_default<int>(msg["side"],0);
+
+		resources::screen->add_chat_message(time(NULL), msg["sender"], side,
+				msg["message"], events::chat_handler::MESSAGE_PUBLIC,
+				preferences::message_bell());
+	}
+
+	if (const config &msg = cfg.child("whisper") /*&& is_observer()*/)
+	{
+		resources::screen->add_chat_message(time(NULL), "whisper: " + msg["sender"], 0,
+				msg["message"], events::chat_handler::MESSAGE_PRIVATE,
+				preferences::message_bell());
+	}
+
+	foreach (const config &ob, cfg.child_range("observer")) {
+		resources::screen->add_observer(ob["name"]);
+	}
+
+	foreach (const config &ob, cfg.child_range("observer_quit")) {
+		resources::screen->remove_observer(ob["name"]);
 	}
 
 	if(cfg.child("leave_game") != NULL) {
@@ -101,76 +113,63 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 
 	bool turn_end = false;
 
-	const config::child_list& turns = cfg.get_children("turn");
-	if(turns.empty() == false && from != network::null_connection) {
+	config::const_child_itors turns = cfg.child_range("turn");
+	if (turns.first != turns.second && from != network::null_connection) {
 		//forward the data to other peers
 		network::send_data_all_except(cfg, from, true);
 	}
 
-	for(config::child_list::const_iterator t = turns.begin(); t != turns.end(); ++t) {
-
+	foreach (const config &t, turns)
+	{
 		if(turn_end == false) {
 			/** @todo FIXME: Check what commands we execute when it's our turn! */
-			replay replay_obj(**t);
-			replay_obj.set_skip(skip_replay);
-			replay_obj.start_replay();
+			replay_.append(t);
+			replay_.set_skip(skip_replay);
 
-			try{
-				turn_end = do_replay(gui_, map_, units_, teams_,
-						team_num_, status_, state_of_game_, &replay_obj);
-			}
-			catch (replay::error& e){
-				//notify remote hosts of out of sync error
-				config cfg;
-				config& info = cfg.add_child("info");
-				info["type"] = "termination";
-				info["condition"] = "out of sync";
-				network::send_data(cfg, 0, true);
-
-				replay::last_replay_error = e.message; //FIXME: some better way to pass this?
-				replay_error_.notify_observers();
-			}
-			recorder.add_config(**t,replay::MARK_AS_SENT);
+			turn_end = do_replay(team_num_, &replay_);
+			recorder.add_config(t, replay::MARK_AS_SENT);
 		} else {
 
 			//this turn has finished, so push the remaining moves
 			//into the backlog
 			backlog.push_back(config());
-			backlog.back().add_child("turn",**t);
+			backlog.back().add_child("turn", t);
 		}
 	}
 
-	if(const config* change= cfg.child("change_controller")) {
+	if (const config &change= cfg.child("change_controller"))
+	{
 		//don't use lexical_cast_default it's "safer" to end on error
-		const int side = lexical_cast<int>((*change)["side"]);
+		const int side = lexical_cast<int>(change["side"]);
 		const size_t index = static_cast<size_t>(side-1);
 
-		const std::string& controller = (*change)["controller"];
-		const std::string& player = (*change)["player"];
+		const std::string &controller = change["controller"];
+		const std::string &player = change["player"];
 
-		if(index < teams_.size()) {
+		if(index < resources::teams->size()) {
+			team &tm = (*resources::teams)[index];
 			if (!player.empty())
-				teams_[index].set_current_player(player);
-			const unit_map::iterator leader = find_leader(units_, side);
-			bool restart = gui_.get_playing_team() == index;
-			if(!player.empty() && leader != units_.end())
+				tm.set_current_player(player);
+			unit_map::iterator leader = resources::units->find_leader(side);
+			bool restart = resources::screen->get_playing_team() == index;
+			if (!player.empty() && leader.valid())
 				leader->second.rename(player);
 
 
-			if ( (controller == "human") && (!teams_[index].is_human()) ) {
-				if (!teams_[gui_.get_playing_team()].is_human())
+			if (controller == "human" && !tm.is_human()) {
+				if (!(*resources::teams)[resources::screen->get_playing_team()].is_human())
 				{
-					gui_.set_team(index);
+					resources::screen->set_team(index);
 				}
-				teams_[index].make_human();
-			} else if ( (controller == "human_ai" ) && (!teams_[index].is_human_ai() )) {
-				teams_[index].make_human_ai();
-			} else if ( (controller == "network") && (!teams_[index].is_network_human()) ){
-				teams_[index].make_network();
-			} else if ( (controller == "network_ai") && (!teams_[index].is_network_ai()) ){
-				teams_[index].make_network_ai();
-			} else if ( (controller == "ai") && (!teams_[index].is_ai()) ) {
-				teams_[index].make_ai();
+				tm.make_human();
+			} else if (controller == "human_ai" && !tm.is_human_ai()) {
+				tm.make_human_ai();
+			} else if (controller == "network" && !tm.is_network_human()) {
+				tm.make_network();
+			} else if (controller == "network_ai" && !tm.is_network_ai()) {
+				tm.make_network_ai();
+			} else if (controller == "ai" && !tm.is_ai()) {
+				tm.make_ai();
 			}
 			else
 			{
@@ -188,19 +187,20 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 		const size_t side = atoi(side_str.c_str());
 		const size_t side_index = side-1;
 
-		bool restart = side_index == gui_.get_playing_team();
+		bool restart = side_index == resources::screen->get_playing_team();
 
-		if(side_index >= teams_.size()) {
-			LOG_STREAM(err, network) << "unknown side " << side_index << " is dropping game\n";
+		if (side_index >= resources::teams->size()) {
+			ERR_NW << "unknown side " << side_index << " is dropping game\n";
 			throw network::error("");
 		}
 
-		const unit_map::iterator leader = find_leader(units_,side);
-		const bool have_leader = (leader != units_.end());
+		team &tm = (*resources::teams)[side_index];
+		unit_map::iterator leader = resources::units->find_leader(side);
+		const bool have_leader = leader.valid();
 
 		if (controller == "ai"){
-			teams_[side_index].make_ai();
-			teams_[side_index].set_current_player("ai"+side_str);
+			tm.make_ai();
+			tm.set_current_player("ai" + side_str);
 			if(have_leader) leader->second.rename("ai"+side_str);
 
 
@@ -221,28 +221,31 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 			options.push_back(_("Abort game"));
 
 			//get all observers in as options to transfer control
-			for(std::set<std::string>::const_iterator ob = gui_.observers().begin(); ob != gui_.observers().end(); ++ob) {
-				t_vars["player"] = *ob;
+			foreach (const std::string &ob, resources::screen->observers())
+			{
+				t_vars["player"] = ob;
 				options.push_back(vgettext("Replace with $player", t_vars));
-				observers.push_back(*ob);
+				observers.push_back(ob);
 			}
 
 			//get all allies in as options to transfer control
-			for (std::vector<team>::iterator team = teams_.begin(); team != teams_.end(); team++){
-				if ( (!team->is_enemy(side)) && (!team->is_human()) && (!team->is_ai()) && (!team->is_empty())
-					&& (team->current_player() != teams_[side_index].current_player()) ){
+			foreach (team &t, *resources::teams)
+			{
+				if (!t.is_enemy(side) && !t.is_human() && !t.is_ai() && !t.is_empty()
+					&& t.current_player() != tm.current_player())
+				{
 					//if this is an ally of the dropping side and it is not us (choose local player
 					//if you want that) and not ai or empty and if it is not the dropping side itself,
 					//get this team in as well
-					t_vars["player"] = team->current_player();
+					t_vars["player"] = t.current_player();
 					options.push_back(vgettext("Replace with $player", t_vars));
-					allies.push_back(&(*team));
+					allies.push_back(&t);
 				}
 			}
 
-			t_vars["player"] = teams_[side_index].current_player();
+			t_vars["player"] = tm.current_player();
 			const std::string msg =  vgettext("$player has left the game. What do you want to do?", t_vars);
-			gui::dialog dlg(gui_, "", msg, gui::OK_ONLY);
+			gui::dialog dlg(*resources::screen, "", msg, gui::OK_ONLY);
 			dlg.set_menu(options);
 			action = dlg.show();
 		}
@@ -252,8 +255,8 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 		//an AI.
 		switch(action) {
 			case 0:
-				teams_[side_index].make_human_ai();
-				teams_[side_index].set_current_player("ai"+side_str);
+				tm.make_human_ai();
+				tm.set_current_player("ai" + side_str);
 				if(have_leader) leader->second.rename("ai"+side_str);
 				change_controller(side_str, "human_ai");
 
@@ -261,8 +264,8 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 				return restart?PROCESS_RESTART_TURN:PROCESS_CONTINUE;
 
 			case 1:
-				teams_[side_index].make_human();
-				teams_[side_index].set_current_player("human"+side_str);
+				tm.make_human();
+				tm.set_current_player("human" + side_str);
 				if(have_leader) leader->second.rename("human"+side_str);
 
 
@@ -273,11 +276,11 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 				throw end_level_exception(QUIT);
 			default:
 				if (action > 2) {
-					
+
 					{
 						// Server thinks this side is ours now so in case of error transfering side we have to make local state to same as what server thinks it is.
-						teams_[side_index].make_human();
-						teams_[side_index].set_current_player("human"+side_str);
+						tm.make_human();
+						tm.set_current_player("human"+side_str);
 						if(have_leader) leader->second.rename("human"+side_str);
 					}
 
@@ -288,8 +291,8 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 						size_t i = index - observers.size();
 						change_side_controller(side_str, allies[i]->current_player(), false /*not our own side*/);
 					} else {
-						teams_[side_index].make_human_ai();
-						teams_[side_index].set_current_player("ai"+side_str);
+						tm.make_human_ai();
+						tm.set_current_player("ai"+side_str);
 						if(have_leader) leader->second.rename("ai"+side_str);
 						change_controller(side_str, "human_ai");
 					}
@@ -303,7 +306,7 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 	// The host has ended linger mode in a campaign -> enable the "End scenario" button
 	// and tell we did get the notification.
 	if (cfg.child("notify_next_scenario")) {
-		gui::button* btn_end = gui_.find_button("button-endturn");
+		gui::button* btn_end = resources::screen->find_button("button-endturn");
 		if(btn_end) {
 			btn_end->enable(true);
 		}
@@ -311,8 +314,8 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 	}
 
 	//If this client becomes the new host, notify the play_controller object about it
-	if (const config* cfg_host_transfer = cfg.child("host_transfer")){
-		if ( (*cfg_host_transfer)["value"] == "1"){
+	if (const config &cfg_host_transfer = cfg.child("host_transfer")){
+		if (cfg_host_transfer["value"] == "1") {
 			host_transfer_.notify_observers();
 		}
 	}
@@ -349,9 +352,9 @@ void turn_info::change_side_controller(const std::string& side, const std::strin
 void turn_info::take_side(const std::string& side, const std::string& controller)
 {
 	config cfg;
-	cfg.values["side"] = side;
-	cfg.values["controller"] = controller;
-	cfg.values["name"] = controller+side;
+	cfg["side"] = side;
+	cfg["controller"] = controller;
+	cfg["name"] = controller+side;
 	network::send_data(cfg, 0, true);
 }
 #endif
