@@ -1,6 +1,6 @@
-/* $Id: preferences.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: preferences.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,8 +12,7 @@
    See the COPYING file for more details.
 */
 
-//! @file preferences.hpp 
-//!
+/** @file preferences.hpp */
 
 #ifndef PREFERENCES_HPP_INCLUDED
 #define PREFERENCES_HPP_INCLUDED
@@ -29,18 +28,6 @@ class display;
 #include <utility>
 #include <set>
 
-// Only there temporary
-#ifdef USE_TINY_GUI
-const int min_allowed_width = 320;
-const int min_allowed_height = 240;
-#elif defined USE_SMALL_GUI
-const int min_allowed_width = 800;
-const int min_allowed_height = 480;
-#else
-const int min_allowed_width = 800;
-const int min_allowed_height = 600;
-#endif
-
 namespace preferences {
 
 	struct base_manager
@@ -52,9 +39,12 @@ namespace preferences {
 	void write_preferences();
 
 	// Low-level, should be seen only by preferences_display ?
-	void set(const std::string key, std::string value);
-	const std::string get(const std::string key);
-	void erase(const std::string key);
+	void set(const std::string& key, std::string value);
+	void clear(const std::string& key);
+	void set_child(const std::string& key, const config& val);
+	const config &get_child(const std::string &key);
+	std::string get(const std::string& key);
+	void erase(const std::string& key);
 
 	void disable_preferences_save();
 
@@ -63,6 +53,12 @@ namespace preferences {
 	bool fullscreen();
 	void _set_fullscreen(bool ison);
 
+	bool scroll_to_action();
+	void _set_scroll_to_action(bool ison);
+
+	int min_allowed_width();
+	int min_allowed_height();
+
 	std::pair<int,int> resolution();
 	void _set_resolution(const std::pair<int,int>& res);
 
@@ -119,12 +115,27 @@ namespace preferences {
 	void load_hotkeys();
 	void save_hotkeys();
 
+	void add_alias(const std::string& alias, const std::string& command);
+	const config &get_alias();
+
 	bool use_colour_cursors();
 	void _set_colour_cursors(bool value);
 
 	int scroll_speed();
 	void set_scroll_speed(const int scroll);
 
+	bool middle_click_scrolls();
+	bool mouse_scroll_enabled();
+	void enable_mouse_scroll(bool value);
+
+	/**
+	 * Gets the threshold for when to scroll.
+	 *
+	 * This scrolling happens when the mouse is in the application and near
+	 * the border.
+	 */
+	int mouse_scroll_threshold();
+
 	int draw_delay();
 	void set_draw_delay(int value);
 
@@ -134,6 +145,9 @@ namespace preferences {
 	bool show_fps();
 	void set_show_fps(bool value);
 
+	bool ellipses();
+	void _set_ellipses(bool ison);
+
 	bool grid();
 	void _set_grid(bool ison);
 } // end namespace preferences
