@@ -1,6 +1,6 @@
-/* $Id: preferences_display.cpp 33589 2009-03-13 02:07:45Z alink $ */
+/* $Id: preferences_display.cpp 41601 2010-03-15 11:34:55Z ai0867 $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,7 @@
 
 #include "display.hpp"
 #include "gettext.hpp"
+#include "gui/dialogs/transient_message.hpp"
 #include "hotkeys.hpp"
 #include "log.hpp"
 #include "marked-up_text.hpp"
@@ -54,28 +55,90 @@ display_manager::~display_manager()
 	disp = NULL;
 }
 
+bool detect_video_settings(CVideo& video, std::pair<int,int>& resolution, int& bpp, int& video_flags)
+{
+	video_flags = fullscreen() ? FULL_SCREEN : 0;
+	resolution = preferences::resolution();
+
+	int DefaultBPP = 24;
+	const SDL_VideoInfo* const video_info = SDL_GetVideoInfo();
+	if(video_info != NULL && video_info->vfmt != NULL) {
+		DefaultBPP = video_info->vfmt->BitsPerPixel;
+	}
+
+	std::cerr << "Checking video mode: " << resolution.first << 'x'
+		<< resolution.second << 'x' << DefaultBPP << "...\n";
+
+	typedef std::pair<int, int> res_t;
+	std::vector<res_t> res_list;
+	res_list.push_back(res_t(1024, 768));
+	res_list.push_back(res_t(1024, 600));
+	res_list.push_back(res_t(800, 600));
+	if (game_config::small_gui)
+		res_list.push_back(res_t(800, 480));
+#ifdef USE_TINY_GUI
+	res_list.push_back(res_t(640, 480));
+	res_list.push_back(res_t(320, 240));
+#endif
+
+	bpp = video.modePossible(resolution.first, resolution.second,
+		DefaultBPP, video_flags);
+
+	foreach (const res_t &res, res_list)
+	{
+		if (bpp != 0) break;
+		std::cerr << "Video mode " << resolution.first << 'x'
+			<< resolution.second << 'x' << DefaultBPP
+			<< " is not supported; attempting " << res.first
+			<< 'x' << res.second << 'x' << DefaultBPP << "...\n";
+		resolution = res;
+		bpp = video.modePossible(resolution.first, resolution.second,
+			DefaultBPP, video_flags);
+	}
+
+	return bpp != 0;
+}
+
+void set_fullscreen(CVideo& video, const bool ison)
+{
+	_set_fullscreen(ison);
+
+	const std::pair<int,int>& res = resolution();
+	if(video.isFullScreen() != ison) {
+		const int flags = ison ? FULL_SCREEN : 0;
+		int bpp = video.modePossible(res.first,res.second,32,flags);
+		if (bpp <= 0) {
+			bpp = video.modePossible(res.first,res.second,16,flags);
+		}
+
+		if(bpp > 0) {
+			video.setMode(res.first,res.second,bpp,flags);
+			if(disp) {
+				disp->redraw_everything();
+			}
+		} else {
+			int tmp_flags = flags;
+			std::pair<int,int> tmp_res;
+			if(detect_video_settings(video, tmp_res, bpp, tmp_flags)) {
+				set_resolution(video, tmp_res.first, tmp_res.second);
+			// TODO: see if below line is actually needed, possibly for displays that only support 16 bbp
+			} else if(video.modePossible(1024,768,16,flags)) {
+				set_resolution(video, 1024, 768);
+			} else {
+				gui2::show_transient_message(video,"",_("The video mode could not be changed. Your window manager must be set to 16 bits per pixel to run the game in windowed mode. Your display must support 1024x768x16 to run the game full screen."));
+			}
+			// We reinit color cursors, because SDL on Mac seems to forget the SDL_Cursor
+			set_colour_cursors(utils::string_bool(preferences::get("colour_cursors")));
+		}
+	}
+}
+
 void set_fullscreen(bool ison)
 {
 	_set_fullscreen(ison);
 
 	if(disp != NULL) {
-		const std::pair<int,int>& res = resolution();
-		CVideo& video = disp->video();
-		if(video.isFullScreen() != ison) {
-			const int flags = ison ? FULL_SCREEN : 0;
-			int bpp = video.modePossible(res.first,res.second,32,flags);
-			if (bpp <= 0) bpp = video.modePossible(res.first,res.second,16,flags);
-			if(bpp > 0) {
-				video.setMode(res.first,res.second,bpp,flags);
-				disp->redraw_everything();
-			} else if(video.modePossible(1024,768,16,flags)) {
-				set_resolution(std::pair<int,int>(1024,768));
-			} else {
-				gui::message_dialog(*disp,"",_("The video mode could not be changed. Your window manager must be set to 16 bits per pixel to run the game in windowed mode. Your display must support 1024x768x16 to run the game full screen.")).show();
-			}
-			// We reinit color cursors, because SDL on Mac seems to forget the SDL_Cursor
-			set_colour_cursors(utils::string_bool(preferences::get("colour_cursors")));
-		}
+		set_fullscreen(disp->video(), ison);
 	}
 }
 
@@ -85,8 +148,21 @@ void set_scroll_to_action(bool ison)
 }
 void set_resolution(const std::pair<int,int>& resolution)
 {
-	std::pair<int,int> res = resolution;
+	if(disp) {
+		set_resolution(disp->video(), resolution.first, resolution.second);
+	} else {
+		/* This part is needed when wesnoth is started with the -r parameter. */
+		const std::string postfix = fullscreen() ? "resolution" : "windowsize";
+		preferences::set(
+				'x' + postfix, lexical_cast<std::string>(resolution.first));
+		preferences::set(
+				'y' + postfix, lexical_cast<std::string>(resolution.second));
+	}
+}
 
+bool set_resolution(CVideo& video
+		, const unsigned width, const unsigned height)
+{
 	// - Ayin: disabled the following code. Why would one want to enforce that?
 	// Some 16:9, or laptop screens, may have resolutions which do not
 	// comply to this rule (see bug 10630).
@@ -96,28 +172,34 @@ void set_resolution(const std::pair<int,int>& resolution)
 	//res.first &= ~3;
 	//res.second &= ~3;
 
-	bool write_resolution = true;
+	SDL_Rect rect;
+	SDL_GetClipRect(video.getSurface(), &rect);
+	if(rect.w == width && rect.h == height) {
+		return true;
+	}
 
-	if(disp != NULL) {
-		CVideo& video = disp->video();
-		const int flags = fullscreen() ? FULL_SCREEN : 0;
-		int bpp = video.modePossible(res.first,res.second,32,flags);
-		if (bpp == 0) bpp = video.modePossible(res.first,res.second, 16, flags);
-		if(bpp != 0) {
-			video.setMode(res.first,res.second,bpp,flags);
+	const int flags = fullscreen() ? FULL_SCREEN : 0;
+	int bpp = video.modePossible(width, height, 32, flags);
+	if(bpp == 0) {
+		bpp = video.modePossible(width, height, 16, flags);
+	}
+	if(bpp != 0) {
+		video.setMode(width, height, bpp, flags);
+
+		if(disp) {
 			disp->redraw_everything();
-
-		} else {
-			write_resolution = false;
-			gui::message_dialog(*disp,"",_("The video mode could not be changed. Your window manager must be set to 16 bits per pixel to run the game in windowed mode. Your display must support 1024x768x16 to run the game full screen.")).show();
 		}
+
+	} else {
+		gui2::show_transient_message(video,"",_("The video mode could not be changed. Your window manager must be set to 16 bits per pixel to run the game in windowed mode. Your display must support 1024x768x16 to run the game full screen."));
+		return false;
 	}
 
-	if(write_resolution) {
-		const std::string postfix = fullscreen() ? "resolution" : "windowsize";
-		preferences::set('x' + postfix, lexical_cast<std::string>(res.first));
-		preferences::set('y' + postfix, lexical_cast<std::string>(res.second));
-	}
+	const std::string postfix = fullscreen() ? "resolution" : "windowsize";
+	preferences::set('x' + postfix, lexical_cast<std::string>(width));
+	preferences::set('y' + postfix, lexical_cast<std::string>(height));
+
+	return true;
 }
 
 void set_turbo(bool ison)
@@ -138,6 +220,11 @@ void set_turbo_speed(double speed)
 	}
 }
 
+void set_ellipses(bool ison)
+{
+	_set_ellipses(ison);
+}
+
 void set_grid(bool ison)
 {
 	_set_grid(ison);
@@ -172,7 +259,7 @@ namespace {
 class escape_handler : public events::handler {
 public:
 	escape_handler() : escape_pressed_(false) {}
-	bool escape_pressed() { return escape_pressed_; }
+	bool escape_pressed() const { return escape_pressed_; }
 	void handle_event(const SDL_Event &event) { escape_pressed_ |= (event.type == SDL_KEYDOWN)
 		&& (reinterpret_cast<const SDL_KeyboardEvent&>(event).keysym.sym == SDLK_ESCAPE); }
 private:
@@ -311,7 +398,7 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 				if(oldhk.get_id() != newhk.get_id() && !oldhk.null()) {
 					std::stringstream msg;
 					msg << "   " << oldhk.get_description() << " : " << oldhk.get_name();
-					gui::message_dialog(disp,_("This Hotkey is already in use."),msg.str()).show();
+					gui2::show_transient_message(disp.video(),_("This Hotkey is already in use."),msg.str());
 				} else {
 
 					newhk.set_key(character, keycode, (mod & KMOD_SHIFT) != 0,
@@ -322,7 +409,7 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 					if ((newhk.get_id() == hotkey::HOTKEY_SCREENSHOT
 							|| newhk.get_id() == hotkey::HOTKEY_MAP_SCREENSHOT)
 							 && (mod & any_mod) == 0) {
-						gui::message_dialog(disp,"", _("Warning: screenshot hotkeys not combined with Control, Alt or Meta keys.")).show();
+						gui2::show_transient_message(disp.video(),"", _("Warning: screenshot hotkeys not combined with Control, Alt or Meta keys."));
 					}
 				}
 			}
@@ -383,15 +470,18 @@ bool show_video_mode_dialog(display& disp)
 			{ 0, 0,  640, 480  },
 			{ 0, 0,  800, 480  },	// small-gui (EeePC resolution)
 			{ 0, 0,  800, 600  },
+			{ 0, 0, 1024, 600  },	// used on many netbooks
 			{ 0, 0, 1024, 768  },
 			{ 0, 0, 1280, 960  },
 			{ 0, 0, 1280, 1024 },
+			{ 0, 0, 1366, 768  },	// 16:9 notebooks
 			{ 0, 0, 1440, 900  },
 			{ 0, 0, 1440, 1200 },
 			{ 0, 0, 1600, 1200 },
 			{ 0, 0, 1680, 1050 },
 			{ 0, 0, 1920, 1080 },
 			{ 0, 0, 1920, 1200 },
+			{ 0, 0, 2560, 1600 }
 		};
 		static const SDL_Rect * const scr_modes_list[] = {
 			&scr_modes[0],
@@ -407,13 +497,16 @@ bool show_video_mode_dialog(display& disp)
 			&scr_modes[10],
 			&scr_modes[11],
 			&scr_modes[12],
+			&scr_modes[13],
+			&scr_modes[14],
+			&scr_modes[15],
 			NULL
 		};
 
 		modes = scr_modes_list;
 	} else if(modes == NULL) {
 		std::cerr << "No modes supported\n";
-		gui::message_dialog(disp,"",_("There are no alternative video modes available")).show();
+		gui2::show_transient_message(disp.video(),"",_("There are no alternative video modes available"));
 		return false;
 	}
 
