@@ -1,6 +1,6 @@
-/* $Id: race.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: race.cpp 42010 2010-04-05 17:24:59Z crab $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,22 +12,22 @@
    See the COPYING file for more details.
 */
 
-//! @file race.cpp
-//! Generate race-specific unit-names.
+/**
+ *  @file race.cpp
+ *  Generate race-specific unit-names.
+ */
 
 #include "global.hpp"
 
-#include "gamestatus.hpp"
 #include "race.hpp"
+
+#include "log.hpp"
 #include "random.hpp"
-#include "serialization/string_utils.hpp"
-
-#include <cstdlib>
-
-namespace {
-
-config::child_list empty_traits;
+#include "simple_rng.hpp"
 
+static const config &empty_traits() {
+		static config cfg;
+		return cfg;
 }
 
 static void add_prefixes(const wide_string& str, size_t length, markov_prefix_map& res)
@@ -51,8 +51,8 @@ static markov_prefix_map markov_prefixes(const std::vector<std::string>& items,
 	return res;
 }
 
-static wide_string markov_generate_name(const markov_prefix_map& prefixes, 
-	size_t chain_size, size_t max_len, game_state* state)
+static wide_string markov_generate_name(const markov_prefix_map& prefixes,
+	size_t chain_size, size_t max_len, rand_rng::simple_rng* rng)
 {
 	if(chain_size == 0)
 		return wide_string();
@@ -72,7 +72,7 @@ static wide_string markov_generate_name(const markov_prefix_map& prefixes,
 	std::vector<int> random(max_len);
 	size_t j = 0;
 	for(; j < max_len; ++j) {
-		random[j] = state ? state->get_random() : get_random();
+		random[j] = rng ? rng->get_next_random() : get_random_nocheck();
 	}
 
 	j = 0;
@@ -107,7 +107,7 @@ static wide_string markov_generate_name(const markov_prefix_map& prefixes,
 	// originally generated name.
 	wide_string originalRes = res;
 	int prefixLen;
-	while (res.size() > 0) {
+	while(!res.empty()) {
 		prefixLen = chain_size < res.size() ? chain_size : res.size();
 		prefix = wide_string(res.end() - prefixLen, res.end());
 
@@ -131,39 +131,34 @@ static wide_string markov_generate_name(const markov_prefix_map& prefixes,
 	return originalRes;
 }
 
-unit_race::unit_race() : 
+unit_race::unit_race() :
 		id_(),
 		plural_name_(),
 		description_(),
-		ntraits_(0), 
-		chain_size_(0), 
-		traits_(&empty_traits), 
+		ntraits_(0),
+		chain_size_(0),
+		traits_(empty_traits().child_range("trait")),
 		global_traits_(true)
 {
 		name_[MALE] = "";
 		name_[FEMALE] = "";
 }
 
-unit_race::unit_race(const config& cfg) : 
+unit_race::unit_race(const config& cfg) :
 		id_(cfg["id"]),
-		plural_name_(cfg["plural_name"]), 
+		plural_name_(cfg["plural_name"]),
 		description_(cfg["description"]),
 		ntraits_(atoi(cfg["num_traits"].c_str())),
 		chain_size_(atoi(cfg["markov_chain_size"].c_str())),
-		traits_(&cfg.get_children("trait")), 
+		traits_(cfg.child_range("trait")),
 		global_traits_(!utils::string_bool(cfg["ignore_global_traits"]))
 
 {
-	//! @todo FIXME remove support after branching 1.4.
-	//! 2 versions with lg::wml_error
-	//! VALIDATE after that
-	if(id_.empty()) {
-		// This code is only for compatibility with old race defs.
-		id_ = (cfg["name"]);
+	if (id_.empty()) {
+		lg::wml_error << "[race] '" << cfg["name"] << "' is missing an id field.";
 	}
-	if(plural_name_.empty()) {
-		// This code is only for compatibility with old race defs.
-		plural_name_ = (cfg["name"]);
+	if (plural_name_.empty()) {
+		lg::wml_error << "[race] '" << cfg["name"] << "' is missing a plural_name field.";
 	}
 	// use "name" if "male_name" or "female_name" aren't available
 	name_[MALE] = cfg["male_name"];
@@ -184,10 +179,10 @@ unit_race::unit_race(const config& cfg) :
 }
 
 std::string unit_race::generate_name(
-		unit_race::GENDER gender, game_state* state) const
+		unit_race::GENDER gender, rand_rng::simple_rng* rng) const
 {
 	return utils::wstring_to_string(
-		markov_generate_name(next_[gender], chain_size_, 12, state));
+		markov_generate_name(next_[gender], chain_size_, 12, rng));
 }
 
 bool unit_race::uses_global_traits() const
@@ -195,9 +190,9 @@ bool unit_race::uses_global_traits() const
 	return global_traits_;
 }
 
-const config::child_list& unit_race::additional_traits() const
+const config::const_child_itors &unit_race::additional_traits() const
 {
-	return *traits_;
+	return traits_;
 }
 
 unsigned int unit_race::num_traits() const { return ntraits_; }
