@@ -1,6 +1,6 @@
-/* $Id: race.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: race.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -19,8 +19,8 @@
 #include <string>
 #include <vector>
 
-#include "serialization/string_utils.hpp"
 #include "config.hpp"
+#include "serialization/string_utils.hpp"
 
 namespace rand_rng {
 	class simple_rng;
@@ -46,7 +46,7 @@ public:
 
 	bool uses_global_traits() const;
 
-	const config::child_list& additional_traits() const;
+	const config::const_child_itors &additional_traits() const;
 	unsigned int num_traits() const;
 
 private:
@@ -58,7 +58,7 @@ private:
 	markov_prefix_map next_[NUM_GENDERS];
 	int chain_size_;
 
-	const config::child_list* traits_;
+	config::const_child_itors traits_;
 	bool global_traits_;
 };
 
