@@ -1,7 +1,7 @@
-/* $Id: random.cpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: random.cpp 41185 2010-02-13 13:40:11Z ilor $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2009 by Yann Dirson <ydirson@altern.org>
+   Copyright (C) 2005 - 2010 by Yann Dirson <ydirson@altern.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -40,11 +40,26 @@
 #include "global.hpp"
 
 #include "config.hpp"
+#include "log.hpp"
+#include "network.hpp"
 #include "random.hpp"
+#include "rng.hpp"
+#include "serialization/string_utils.hpp"
+#include "simple_rng.hpp"
+#include "util.hpp"
 
+static lg::log_domain log_random("random");
+#define DBG_RND LOG_STREAM(debug, log_random)
+#define LOG_RND LOG_STREAM(info, log_random)
+#define WRN_RND LOG_STREAM(warn, log_random)
+#define ERR_RND LOG_STREAM(err, log_random)
 
 namespace {
   rand_rng::rng *random_generator = NULL ;
+  rand_rng::seed_t last_seed;
+  bool seed_valid = false;
+  boost::function<void (rand_rng::seed_t)> new_seed_callback;
+  bool srng_disabled = false;
 }
 
 
@@ -55,6 +70,12 @@ int get_random()
   return r ;
 }
 
+int get_random_nocheck()
+{
+  assert(random_generator!=NULL);
+  int r = random_generator->get_random_nocheck();
+  return r ;
+}
 
 const config* get_random_results()
 {
@@ -73,22 +94,102 @@ void set_random_results(const config& cfg)
 namespace rand_rng
 {
 
-rng::rng() : random_(NULL), random_child_(0)
-{}
+void set_seed(seed_t seed)
+{
+	LOG_RND << "set_seed with " << seed << "\n";
+	assert(random_generator!=NULL);
+	last_seed = seed;
+	seed_valid = true;
+	random_generator->set_seed(seed);
+	if (new_seed_callback) {
+		LOG_RND << "set_seed calling new_seed_callback\n";
+		new_seed_callback(seed);
+	}
+}
+
+void invalidate_seed()
+{
+	LOG_RND << "invalidate_seed\n";
+	assert(random_generator!=NULL);
+	last_seed = rand();
+	if (has_valid_seed()) { //aka SRNG is disabled
+		random_generator->set_seed(last_seed);
+	}
+	seed_valid = false;
+}
+
+bool has_valid_seed()
+{
+	//if the SRNG is disabled or we're in a SP game the seed is always valid
+	return srng_disabled || (network::nconnections() == 0) || seed_valid;
+}
+
+seed_t get_last_seed()
+{
+	return last_seed;
+}
+
+void set_new_seed_callback(boost::function<void (seed_t)> f)
+{
+	DBG_RND << "set_new_seed_callback\n";
+	new_seed_callback = f;
+}
+
+void clear_new_seed_callback()
+{
+	DBG_RND << "clear_new_seed_callback\n";
+	new_seed_callback = NULL;
+}
+
+void disable_server_rng()
+{
+	srng_disabled = true;
+}
+
+
+
+rng::rng() : random_(NULL), random_child_(0), generator_()
+{
+}
 
 int rng::get_random()
 {
-	if (!random_)
-		return rand() & 0x7FFFFFFF;
+	return get_random_private(true);
+}
+
+int rng::get_random_nocheck()
+{
+	return get_random_private(false);
+}
+
+int rng::get_random_private(bool check)
+{
+	if (!random_) {
+		int r = generator_.get_next_random();
+		LOG_RND << "get_random() returning " << r << " (random_ is null)\n";
+		return r;
+	}
 
 	const config::child_list random(random_->get_children("random"));
 	if (random_child_ >= random.size()) {
 		random_child_ = random.size() + 1;
-		int res = rand() & 0x7FFFFFFF;
+		int res = generator_.get_next_random() & 0x7FFFFFFF;
 		(random_->add_child("random"))["value"] = lexical_cast<std::string>(res);
+		LOG_RND << "get_random() returning " << res << " (added to random_)\n";
 		return res;
 	} else {
-		return lexical_cast_default<int>((*random[random_child_++])["value"], 0);
+		int mine = generator_.get_next_random();
+		int stored = lexical_cast_default<int>((*random[random_child_++])["value"], 0);
+		if (mine != stored) {
+			if (check) {
+				ERR_RND << "Random number mismatch, mine " << mine << " vs " << stored << "\n";
+				//OOS here?
+			} else {
+				LOG_RND << "Random number mismatch (nocheck), mine " << mine << " vs " << stored << "\n";
+			}
+		}
+		LOG_RND << "get_random() returning " << stored << "\n";
+		return stored;
 	}
 }
 
@@ -98,7 +199,8 @@ const config* rng::get_random_results()
 
 	const config::child_list random(random_->get_children("random"));
 	if (random_child_ <= 0 ||random_child_ > random.size()) return NULL;
-	return random[random_child_-1]->child("results");
+	const config &res = random[random_child_-1]->child("results");
+	return res ? &res : NULL;
 }
 
 void rng::set_random_results(const config& cfg)
@@ -123,6 +225,12 @@ void rng::set_random(config* random)
 	return;
 }
 
+void rng::set_seed(seed_t seed)
+{
+	LOG_RND << "Set random seed to " << seed << "\n";
+	generator_.seed_random(seed, 0);
+}
+
 
 set_random_generator::set_random_generator(rng* r) : old_(random_generator)
 {
@@ -138,7 +246,8 @@ simple_rng::simple_rng() :
     random_seed_(rand()),
     random_pool_(random_seed_),
     random_calls_(0)
-{ }
+{
+}
 
 simple_rng::simple_rng(const config& cfg) :
     /**
@@ -148,14 +257,16 @@ simple_rng::simple_rng(const config& cfg) :
     random_seed_(lexical_cast_default<int>(cfg["random_seed"], 42)),
     random_pool_(random_seed_),
     random_calls_(0)
-{}
+{
+}
 
-int simple_rng::get_random()
+int simple_rng::get_next_random()
 {
 	random_next();
 	++random_calls_;
-	//DBG_NG << "pulled user random " << random_pool_
-	//	<< " for call " << random_calls_ << '\n';
+	DBG_RND << "pulled user random " << random_pool_
+		<< " for call " << random_calls_
+		<< " with seed " << random_seed_ << '\n';
 
 	return (static_cast<unsigned>(random_pool_ / 65536) % 32768);
 }
@@ -172,9 +283,9 @@ void simple_rng::seed_random(const int seed, const unsigned call_count)
 	for(random_calls_ = 0; random_calls_ < call_count; ++random_calls_) {
 		random_next();
 	}
-	//DBG_NG << "Seeded random with " << random_seed_ << " with "
-	//	<< random_calls_ << " calls, pool is now at "
-	//	<< random_pool_ << '\n';
+	DBG_RND << "Seeded random with " << random_seed_ << " with "
+		<< random_calls_ << " calls, pool is now at "
+		<< random_pool_ << '\n';
 }
 
 void simple_rng::random_next()
