@@ -1,7 +1,7 @@
-/* $Id: random.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: random.hpp 41185 2010-02-13 13:40:11Z ilor $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2008 by Yann Dirson <ydirson@altern.org>
+   Copyright (C) 2005 - 2010 by Yann Dirson <ydirson@altern.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,44 +13,32 @@
    See the COPYING file for more details.
 */
 
-//! @file random.hpp
-//!
+/** @file random.hpp */
 
-#ifndef RANDOM_H_INCLUDED
-#define RANDOM_H_INCLUDED
-
-#include "SDL_types.h"
+#ifndef RANDOM_HPP_INCLUDED
+#define RANDOM_HPP_INCLUDED
 
 class config;
 
 int get_random();
+int get_random_nocheck();
 const config* get_random_results();
 void set_random_results(const config& cfg);
 
-class rng
+namespace rand_rng
 {
-public:
-	rng();
-	int get_random();
 
-	const config* get_random_results();
-	void set_random_results(const config& cfg);
+typedef unsigned int seed_t;
 
-protected:
-	config* random();
-	void set_random(config*);
-
-private:
-	config* random_;
-	size_t random_child_;
-};
+class rng;
 
 struct set_random_generator {
 	set_random_generator(rng* r);
 	~set_random_generator();
-
 private:
 	rng* old_;
 };
 
+} // ends rand_rng namespace
+
 #endif
