@@ -1,6 +1,6 @@
-/* $Id: replay_controller.hpp 33524 2009-03-10 14:55:11Z silene $ */
+/* $Id: replay_controller.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -17,6 +17,7 @@
 #define REPLAY_CONTROLLER_H_INCLUDED
 
 #include "game_end_exceptions.hpp"
+#include "gamestatus.hpp"
 #include "play_controller.hpp"
 
 #include <vector>
@@ -40,18 +41,22 @@ public:
 	void stop_replay();
 	void replay_next_turn();
 	void replay_next_side();
+	void process_oos(const std::string& msg) const;
 	void replay_show_everything();
 	void replay_show_each();
 	void replay_show_team1();
 	void replay_skip_animation();
 
+	virtual void force_end_turn() {}
+	virtual void force_end_level(LEVEL_RESULT) {}
+	virtual void check_end_level() {}
+
 	std::vector<team> teams_start_;
 
 protected:
 	virtual void init_gui();
 
 private:
-	bool continue_replay();
 	void init();
 	virtual void play_turn();
 	virtual void play_side(const unsigned int team_index, bool save);
@@ -62,8 +67,8 @@ private:
 	void handle_generic_event(const std::string& /*name*/);
 
 	game_state gamestate_start_;
-	gamestatus status_start_;
 	unit_map units_start_;
+	tod_manager tod_manager_start_;
 
 	unsigned int current_turn_;
 	int delay_;
@@ -71,7 +76,6 @@ private:
 
 	bool show_everything_;
 	unsigned int show_team_;
-	bool skip_next_turn_;
 };
 
 
