@@ -1,6 +1,6 @@
-/* $Id: reports.cpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: reports.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -22,12 +22,12 @@ namespace {
 		"unit_name", "unit_type",
 		"unit_race", "unit_level", "unit_side", "unit_amla",
 		"unit_traits", "unit_status", "unit_alignment", "unit_abilities",
-		"unit_hp", "unit_xp", "unit_advancement_options", "unit_moves",
+		"unit_hp", "unit_xp", "unit_advancement_options", "unit_defense", "unit_moves",
 		"unit_weapons", "unit_image", "unit_profile", "time_of_day",
 		"turn", "gold", "villages", "num_units", "upkeep", "expenses",
 		"income", "terrain", "position", "side_playing", "observers",
 		"report_countdown", "report_clock",
-		"selected_terrain", "edit_left_button_function", "editor2_tool_hint"
+		"selected_terrain", "edit_left_button_function", "editor_tool_hint"
 	};
 }
 
@@ -41,24 +41,10 @@ const std::string& report_name(TYPE type)
 	return report_names[type];
 }
 
-void report::add_text(std::stringstream& text, std::stringstream& tooltip) {
-	add_text(text.str(), tooltip.str());
-	// Clear the streams
-	text.str("");
-	tooltip.str("");
-}
-
 void report::add_text(const std::string& text, const std::string& tooltip) {
 	this->push_back(element(text,"",tooltip));
 }
 
-void report::add_image(std::stringstream& image, std::stringstream& tooltip) {
-	add_image(image.str(), tooltip.str());
-	// Clear the streams
-	image.str("");
-	tooltip.str("");
-}
-
 void report::add_image(const std::string& image, const std::string& tooltip) {
 	this->push_back(element("",image,tooltip));
 }
