@@ -1,6 +1,6 @@
-/* $Id: sdl_utils.hpp 34753 2009-04-12 06:01:26Z boucman $ */
+/* $Id: sdl_utils.hpp 41180 2010-02-13 10:43:57Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 #include "SDL.h"
 
 #include <cstdlib>
-#include <iostream>
+#include <iosfwd>
 #include <map>
 #include <string>
 #include <vector>
@@ -63,10 +63,11 @@ SDL_Rect intersect_rects(SDL_Rect const &rect1, SDL_Rect const &rect2);
  * @param rect1                   The first rectangle.
  * @param rect2                   The second rectangle.
  *
- * @returns                       The union of rect1 and rect2.
+ * @returns                       The union of the two rectangles. If the
+ *                                rectangles don't intersect and empty
+ *                                rectangle is returned.
  */
 SDL_Rect get_rect_union(const SDL_Rect& rect1, const SDL_Rect& rect2);
-
 /**
  *  Creates an empty SDL_Rect.
  *
@@ -134,9 +135,9 @@ private:
 
 bool operator<(const surface& a, const surface& b);
 
-surface make_neutral_surface(surface const &surf);
+surface make_neutral_surface(const surface &surf);
 surface create_neutral_surface(int w, int h);
-surface create_optimized_surface(surface const &surf);
+surface create_optimized_surface(const surface &surf);
 
 /**
  *  Streches a surface in the horizontal direction.
@@ -186,7 +187,7 @@ surface stretch_surface_vertical(
  *  @retval surf             Returned if w == surf->w and h == surf->h
  *                           note this ignores the optimize flag.
  */
-surface scale_surface(surface const &surf, int w, int h, bool optimize=true);
+surface scale_surface(const surface &surf, int w, int h, bool optimize=true);
 
 /** Scale an opaque surface
  *  @param surf              The source surface.
@@ -199,18 +200,31 @@ surface scale_surface(surface const &surf, int w, int h, bool optimize=true);
  *  @retval surf             Returned if w == surf->w and h == surf->h
  *                           note this ignores the optimize_format flag.
  */
-surface scale_opaque_surface(surface const &surf, int w, int h, bool optimize_format=false);
+surface scale_opaque_surface(const surface &surf, int w, int h, bool optimize_format=false);
 
-surface scale_surface_blended(surface const &surf, int w, int h, bool optimize=true);
-surface adjust_surface_colour(surface const &surf, int r, int g, int b, bool optimize=true);
-surface greyscale_image(surface const &surf, bool optimize=true);
-surface darken_image(surface const &surf, bool optimize=true);
+surface scale_surface_blended(const surface &surf, int w, int h, bool optimize=true);
+surface adjust_surface_colour(const surface &surf, int r, int g, int b, bool optimize=true);
+surface greyscale_image(const surface &surf, bool optimize=true);
 /** create an heavy shadow of the image, by blurring, increasing alpha and darkening */
-surface shadow_image(surface const &surf, bool optimize=true);
+surface shadow_image(const surface &surf, bool optimize=true);
+
+/**
+ * Recolors a surface using a map with source and converted palette values.
+ * This is most often used for team-coloring.
+ *
+ * @param surf               The source surface.
+ * @param map_rgb            Map of color values, with the keys corresponding to the
+ *                           source palette, and the values to the recolored palette.
+ * @param optimize           Whether the new surface should be RLE encoded. Only
+ *                           useful when the source is not the screen and it is
+ *                           going to be used multiple times.
+ * @return                   A recolored surface, or a null surface if there are
+ *                           problems with the source.
+ */
 surface recolor_image(surface surf, const std::map<Uint32, Uint32>& map_rgb,
 	bool optimize=true);
 
-surface brighten_image(surface const &surf, fixed_t amount, bool optimize=true);
+surface brighten_image(const surface &surf, fixed_t amount, bool optimize=true);
 
 /** Get a portion of the screen.
  *  Send NULL if the portion is outside of the screen.
@@ -223,20 +237,30 @@ surface brighten_image(surface const &surf, fixed_t amount, bool optimize=true);
  *                           No RLE or Alpha bits are set.
  *  @retval 0                if error or the portion is outside of the surface.
  */
-surface get_surface_portion(surface const &surf, SDL_Rect &rect,
+surface get_surface_portion(const surface &surf, SDL_Rect &rect,
 	bool optimize_format=false);
 
-surface adjust_surface_alpha(surface const &surf, fixed_t amount, bool optimize=true);
-surface adjust_surface_alpha_add(surface const &surf, int amount, bool optimize=true);
+surface adjust_surface_alpha(const surface &surf, fixed_t amount, bool optimize=true);
+surface adjust_surface_alpha_add(const surface &surf, int amount, bool optimize=true);
 
 /** Applies a mask on a surface. */
-surface mask_surface(surface const &surf, surface const &mask);
+surface mask_surface(const surface &surf, const surface &mask);
 
 /** Check if a surface fit into a mask */
-bool in_mask_surface(surface const &surf, surface const &mask);
+bool in_mask_surface(const surface &surf, const surface &mask);
 
 /** Cross-fades a surface. */
-surface blur_surface(surface const &surf, int depth = 1, bool optimize=true);
+surface blur_surface(const surface &surf, int depth = 1, bool optimize=true);
+
+/**
+ * Cross-fades a surface in place.
+ *
+ * @param surf                    The surface to blur, must be not optimized
+ *                                and have 32 bits per pixel.
+ * @param rect                    The part of the surface to blur.
+ * @param depth                   The depth of the blurring.
+ */
+void blur_surface(surface& surf, SDL_Rect rect, unsigned depth = 1);
 
 /**
  * Cross-fades a surface with alpha channel.
@@ -244,16 +268,16 @@ surface blur_surface(surface const &surf, int depth = 1, bool optimize=true);
  * @todo FIXME: This is just an adapted copy-paste
  * of the normal blur but with blur alpha channel too
  */
-surface blur_alpha_surface(surface const &surf, int depth = 1, bool optimize=true);
+surface blur_alpha_surface(const surface &surf, int depth = 1, bool optimize=true);
 
 /** Cuts a rectangle from a surface. */
-surface cut_surface(surface const &surf, SDL_Rect const &r);
-surface blend_surface(surface const &surf, double amount, Uint32 colour, bool optimize=true);
-surface flip_surface(surface const &surf, bool optimize=true);
-surface flop_surface(surface const &surf, bool optimize=true);
-surface create_compatible_surface(surface const &surf, int width = -1, int height = -1);
+surface cut_surface(const surface &surf, SDL_Rect const &r);
+surface blend_surface(const surface &surf, double amount, Uint32 colour, bool optimize=true);
+surface flip_surface(const surface &surf, bool optimize=true);
+surface flop_surface(const surface &surf, bool optimize=true);
+surface create_compatible_surface(const surface &surf, int width = -1, int height = -1);
 
-/*
+/**
  *  Replacement for SDL_BlitSurface.
  *
  *  SDL_BlitSurface has problems with blitting partly transparent surfaces so
@@ -270,9 +294,9 @@ surface create_compatible_surface(surface const &surf, int width = -1, int heigh
 void blit_surface(const surface& src,
 	const SDL_Rect* srcrect, surface& dst, const SDL_Rect* dstrect);
 
-void fill_rect_alpha(SDL_Rect &rect, Uint32 colour, Uint8 alpha, surface const &target);
+void fill_rect_alpha(SDL_Rect &rect, Uint32 colour, Uint8 alpha, const surface &target);
 
-SDL_Rect get_non_transparent_portion(surface const &surf);
+SDL_Rect get_non_transparent_portion(const surface &surf);
 
 bool operator==(const SDL_Rect& a, const SDL_Rect& b);
 bool operator!=(const SDL_Rect& a, const SDL_Rect& b);
@@ -280,44 +304,10 @@ bool operator==(const SDL_Color& a, const SDL_Color& b);
 bool operator!=(const SDL_Color& a, const SDL_Color& b);
 SDL_Color inverse(const SDL_Color& colour);
 SDL_Color int_to_color(const Uint32 rgb);
-Uint32 color_to_int(const SDL_Color& rgb);
-
-class config; // no need to include config.hpp
-
-struct pixel_data
-{
-	pixel_data() : r(0), g(0), b(0)
-	{}
-
-	pixel_data(int red, int green, int blue) : r(red), g(green), b(blue)
-	{}
-
-	pixel_data(int pixel, SDL_PixelFormat* fmt) : r(0), g(0), b(0) {
-		unformat(pixel, fmt);
-	}
-
-	pixel_data(config& cfg) : r(0), g(0), b(0) {
-		read(cfg);
-	}
-
-	int format(SDL_PixelFormat* fmt) const {
-		return SDL_MapRGB(fmt,r,g,b);
-	}
-
-	void unformat(int pixel, SDL_PixelFormat* fmt) {
-		r = ((pixel&fmt->Rmask) >> fmt->Rshift);
-		g = ((pixel&fmt->Gmask) >> fmt->Gshift);
-		b = ((pixel&fmt->Bmask) >> fmt->Bshift);
-	}
-
-	void read(const config& cfg);
-
-	int r, g, b;
-};
 
 struct surface_lock
 {
-	surface_lock(surface const &surf) : surface_(surf), locked_(false)
+	surface_lock(const surface &surf) : surface_(surf), locked_(false)
 	{
 		if(SDL_MUSTLOCK(surface_)) {
 			const int res = SDL_LockSurface(surface_);
@@ -361,7 +351,7 @@ private:
 
 struct clip_rect_setter
 {
-	clip_rect_setter(surface const &surf, const SDL_Rect& r) : surface_(surf), rect()
+	clip_rect_setter(const surface &surf, const SDL_Rect& r) : surface_(surf), rect()
 	{
 		SDL_GetClipRect(surface_,&rect);
 		SDL_SetClipRect(surface_,&r);
