@@ -1,7 +1,7 @@
-/* $Id: binary_wml.cpp 33126 2009-02-27 16:32:20Z soliton $ */
+/* $Id: binary_wml.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2010 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -14,19 +14,20 @@
 */
 
 /**
- * @file serialization/binary_wml.cpp.
+ * @file serialization/binary_wml.cpp
  * Data compression, designed for network traffic.
  */
 
 #include "global.hpp"
 
 #include "config.hpp"
+#include "foreach.hpp"
 #include "loadscreen.hpp"
 #include "log.hpp"
 #include "serialization/binary_wml.hpp"
 
-
-#define ERR_CF LOG_STREAM(err, config)
+static lg::log_domain log_config("config");
+#define ERR_CF LOG_STREAM(err, log_config)
 
 // Data compression. Compression is designed for network traffic.
 // Assumptions the compression is based on:
@@ -130,24 +131,21 @@ static void write_compressed_internal(std::ostream &out, config const &cfg, comp
 	if (level > max_recursion_levels)
 		throw config::error("Too many recursion levels in compressed config write");
 
-	for (string_map::const_iterator i = cfg.values.begin(), i_end = cfg.values.end(); i != i_end; ++i) {
-		if (i->second.empty() == false) {
+	foreach (const config::attribute &i, cfg.attribute_range()) {
+		if (!i.second.empty()) {
 			// Output the name, using compression
-			compress_emit_word(out, i->first, schema);
+			compress_emit_word(out, i.first, schema);
 
 			// Output the value, with no compression
-			compress_output_literal_word(out, i->second.to_serialized());
+			compress_output_literal_word(out, i.second.to_serialized());
 		}
 	}
 
-	for (config::all_children_iterator j = cfg.ordered_begin(), j_end = cfg.ordered_end(); j != j_end; ++j) {
-		std::pair< std::string const *, config const * > const &item = *j;
-		std::string const &name = *item.first;
-		config const &cfg2 = *item.second;
-
+	foreach (const config::any_child &item, cfg.all_children_range())
+	{
 		out.put(compress_open_element);
-		compress_emit_word(out, name, schema);
-		write_compressed_internal(out, cfg2, schema, level + 1);
+		compress_emit_word(out, item.key, schema);
+		write_compressed_internal(out, item.cfg, schema, level + 1);
 		out.put(compress_close_element);
 	}
 }
@@ -202,7 +200,7 @@ static void read_compressed_internal(config &cfg, std::istream &in, compression_
 				// We have a name/value pair, the value is always a literal string
 				std::string value = compress_read_literal_word(in);
 				t_string t_value = t_string::from_serialized(value);
-				cfg.values.insert(std::make_pair(word, t_value));
+				cfg[word] = t_value;
 			}
 		}
 
