@@ -1,7 +1,7 @@
-/* $Id: parser.hpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: parser.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2010 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,16 +13,15 @@
    See the COPYING file for more details.
 */
 
-/** @file serialization/parser.hpp. */
+/** @file serialization/parser.hpp */
 
 #ifndef SERIALIZATION_PARSER_HPP_INCLUDED
 #define SERIALIZATION_PARSER_HPP_INCLUDED
 
 #include "global.hpp"
 
-#include <iostream>
 #include <iosfwd>
-#include <vector>
+#include <string>
 
 class config;
 class t_string;
