@@ -1,7 +1,7 @@
-/* $Id: preprocessor.cpp 35988 2009-06-01 11:41:05Z silene $ */
+/* $Id: preprocessor.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2010 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -26,10 +26,13 @@
 #include "log.hpp"
 #include "wesconfig.h"
 #include "serialization/binary_or_text.hpp"
+#include "serialization/string_utils.hpp"
+#include "util.hpp"
 
-#define ERR_CF LOG_STREAM(err, config)
-#define LOG_CF LOG_STREAM(info, config)
-#define DBG_CF LOG_STREAM(debug, config)
+static lg::log_domain log_config("config");
+#define ERR_CF LOG_STREAM(err, log_config)
+#define LOG_CF LOG_STREAM(info, log_config)
+#define DBG_CF LOG_STREAM(debug, log_config)
 
 using std::streambuf;
 
@@ -86,22 +89,20 @@ void preproc_define::read(const config& cfg)
 	linenum = lexical_cast<int>(cfg["linenum"]);
 	location = cfg["location"];
 
-	foreach (const config *arg, cfg.get_children("argument"))
-		read_argument(*arg);
+	foreach (const config &arg, cfg.child_range("argument"))
+		read_argument(arg);
 }
 
-preproc_map::value_type preproc_define::read_pair(const config* cfg)
+preproc_map::value_type preproc_define::read_pair(const config &cfg)
 {
-	std::string first = (*cfg)["name"];
-
 	preproc_define second;
-	second.read(*cfg);
-	return std::make_pair(first, second);
+	second.read(cfg);
+	return preproc_map::value_type(cfg["name"], second);
 }
 
 std::ostream& operator<<(std::ostream& stream, const preproc_define& def)
 {
-	return stream << std::string("value: ") << def.value << std::string(" arguments: ") << def.location;
+	return stream << "value: " << def.value << " arguments: " << def.location;
 }
 
 std::ostream& operator<<(std::ostream& stream, const preproc_map::value_type& def)
@@ -126,8 +127,7 @@ class preprocessor
 	int old_linenum_;
 protected:
 	preprocessor_streambuf &target_;
-	std::vector<std::string> *called_macros_;
-	preprocessor(preprocessor_streambuf &, std::vector<std::string> *);
+	preprocessor(preprocessor_streambuf &);
 public:
 	/**
 	 * Preprocesses and sends some text to the #target_ buffer.
@@ -289,13 +289,12 @@ void preprocessor_streambuf::error(const std::string& error_type, const std::str
  * Saves the current preprocessing context of #target_. It will be
  * automatically restored on destruction.
  */
-preprocessor::preprocessor(preprocessor_streambuf &t, std::vector<std::string> *callstack) :
+preprocessor::preprocessor(preprocessor_streambuf &t) :
 	old_preprocessor_(t.current_),
 	old_textdomain_(t.textdomain_),
 	old_location_(t.location_),
 	old_linenum_(t.linenum_),
-	target_(t),
-	called_macros_(callstack)
+	target_(t)
 {
 	++target_.depth_;
 	target_.current_ = this;
@@ -332,7 +331,7 @@ class preprocessor_file: preprocessor
 	std::vector< std::string > files_;
 	std::vector< std::string >::const_iterator pos_, end_;
 public:
-	preprocessor_file(preprocessor_streambuf &, std::vector<std::string> *, std::string const &);
+	preprocessor_file(preprocessor_streambuf &, std::string const &);
 	virtual bool get_chunk();
 };
 
@@ -353,6 +352,7 @@ class preprocessor_data: preprocessor
 		 * - 'I': skipping the "if" branch of a ifdef/ifndef (the "else" branch, if any, will be processed)
 		 * - 'J': skipping the "else" branch of a ifdef/ifndef
 		 * - '"': processing a string
+		 * - '<': processing a verbatim string
 		 * - '{': processing between chunks of a macro call (skip spaces)
 		 * - '[': processing inside a chunk of a macro call (stop on space or '(')
 		 * - '(': processing a parenthesized macro argument
@@ -366,9 +366,10 @@ class preprocessor_data: preprocessor
 	std::string directory_;
 	/** Buffer for delayed input processing. */
 	std::vector< std::string > strings_;
+	/** Mapping of macro arguments to their content. */
+	std::map<std::string, std::string> *local_defines_;
 	/** Stack of nested preprocessing chunks. */
 	std::vector< token_desc > tokens_;
-	bool is_macro;
 	/**
 	 * Set to true whenever input tokens cannot be directly sent to the target
 	 * buffer. For instance, this happens with macro arguments. In that case,
@@ -394,19 +395,17 @@ class preprocessor_data: preprocessor
 	= 0 */);
 public:
 	preprocessor_data(preprocessor_streambuf &,
-                      std::vector<std::string> *,
-                      std::istream *,
+	                  std::istream *,
 	                  std::string const &history,
 	                  std::string const &name, int line,
 	                  std::string const &dir, std::string const &domain,
-                      std::string* = NULL);
-    ~preprocessor_data();
+	                  std::map<std::string, std::string> *defines);
+	~preprocessor_data();
 	virtual bool get_chunk();
 };
 
-preprocessor_file::preprocessor_file(preprocessor_streambuf &t, std::vector<std::string> *callstack,
-		std::string const &name) :
-	preprocessor(t,callstack),
+preprocessor_file::preprocessor_file(preprocessor_streambuf &t, std::string const &name) :
+	preprocessor(t),
 	files_(),
 	pos_(),
 	end_()
@@ -420,7 +419,8 @@ preprocessor_file::preprocessor_file(preprocessor_streambuf &t, std::vector<std:
 			delete file_stream;
 		}
 		else
-			new preprocessor_data(t, called_macros_, file_stream, "", name, 1, directory_name(name), t.textdomain_);
+			new preprocessor_data(t, file_stream, "", get_short_wml_path(name),
+				1, directory_name(name), t.textdomain_, NULL);
 	}
 	pos_ = files_.begin();
 	end_ = files_.end();
@@ -440,37 +440,33 @@ bool preprocessor_file::get_chunk()
 		// Use reverse iterator to optimize testing
 		if (sz < 5 || !std::equal(name.rbegin(), name.rbegin() + 4, "gfc."))
 			continue;
-		new preprocessor_file(target_, called_macros_, name);
+		new preprocessor_file(target_, name);
 		return true;
 	}
 	return false;
 }
 
-preprocessor_data::preprocessor_data(preprocessor_streambuf &t, std::vector<std::string> *callstack,
-        std::istream *i, std::string const &history, std::string const &name, int linenum,
-		std::string const &directory, std::string const &domain, std::string *symbol) :
-	preprocessor(t,callstack),
+preprocessor_data::preprocessor_data(preprocessor_streambuf &t,
+	std::istream *i, std::string const &history, std::string const &name, int linenum,
+	std::string const &directory, std::string const &domain,
+	std::map<std::string, std::string> *defines) :
+	preprocessor(t),
 	in_(i),
 	directory_(directory),
 	strings_(),
+	local_defines_(defines),
 	tokens_(),
-	is_macro(symbol != NULL),
 	slowpath_(0),
 	skipping_(0),
 	linenum_(linenum)
 {
-    if(is_macro) {
-        called_macros_->push_back(*symbol);
-    }
-
 	std::ostringstream s;
 
 	s << history;
 	if (!name.empty()) {
-		std::string ename(name);
 		if (!history.empty())
 			s << ' ';
-		s << utils::escape(ename, " \\");
+		s << utils::escape(name, " \\");
 	}
 	if (!t.location_.empty())
 		s << ' ' << t.linenum_ << ' ' << t.location_;
@@ -486,10 +482,7 @@ preprocessor_data::preprocessor_data(preprocessor_streambuf &t, std::vector<std:
 
 preprocessor_data::~preprocessor_data()
 {
-    if(is_macro)
-    {
-        called_macros_->pop_back();
-    }
+	delete local_defines_;
 }
 
 void preprocessor_data::push_token(char t)
@@ -637,6 +630,22 @@ bool preprocessor_data::get_chunk()
 		buffer += '\n';
 		// line_change = 1-1 = 0
 		put(buffer);
+	} else if (token.type == '<') {
+		put(c);
+		if (c == '>' && in_->peek() == '>') {
+			put(in_->get());
+			if (!skipping_ && slowpath_) {
+				std::string tmp = strings_.back();
+				pop_token();
+				strings_.back() += tmp;
+			} else
+				pop_token();
+		}
+	} else if (c == '<' && in_->peek() == '<') {
+		in_->get();
+		push_token('<');
+		put('<');
+		put('<');
 	} else if (c == '"') {
 		if (token.type == '"') {
 			target_.quoted_ = false;
@@ -776,7 +785,7 @@ bool preprocessor_data::get_chunk()
 			target_.defines_->erase(symbol);
 			LOG_CF << "undefine macro " << symbol << " (location " << target_.location_ << ")\n";
 		} else
-			comment = true;
+			comment = token.type != '{';
 		skip_eol();
 		if (comment)
 			put('\n');
@@ -801,7 +810,7 @@ bool preprocessor_data::get_chunk()
 			if (token.type == '{') {
 				if (!strings_.back().empty()) {
 					std::ostringstream error;
-	        			std::ostringstream location;
+					std::ostringstream location;
 					error << "Can't parse new macro parameter with a macro call scope open";
 					location<<linenum_<<' '<<target_.location_;
 					target_.error(error.str(), location.str());
@@ -815,132 +824,62 @@ bool preprocessor_data::get_chunk()
 				std::string::iterator b = symbol.begin(); // invalidated at each iteration
 				symbol.erase(b + pos, b + symbol.find('\n', pos + 1) + 1);
 			}
+			std::map<std::string, std::string>::const_iterator arg;
+			preproc_map::const_iterator macro;
 			// If this is a known pre-processing symbol, then we insert it,
 			// otherwise we assume it's a file name to load.
-			preproc_map::const_iterator macro = target_.defines_->find(symbol);
-			if(macro != target_.defines_->end()) {
-
-/**
- * @todo it seems some addons use other names instead of include so disable the
- * code for now. Once 1.4 has been released we can try to fix it again or make
- * it mandatory to use INCLUDE for this purpose.
- *
- * ESR says: No, it was a fundamentally mistaken idea to do this here; wmllint
- * could do the same direct-recursion check without imposing runtime overhead
- * or causing the problems this code did.  Leaving this code in for
- * documentation purposes only, don't re-enable it.
- */
-#if 0
-				// INCLUDE is special and is allowed to be used recusively.
-				if(symbol != "INCLUDE") {
-				    for(std::vector<std::string>::iterator
-							iter=called_macros_->begin();
-							iter!=called_macros_->end(); ++iter) {
-						if(*iter==symbol) {
-							std::ostringstream error;
-							error << "symbol '" << symbol << "' will cause a recursive macro call";
-							std::ostringstream location;
-							location<<linenum_<<' '<<target_.location_;
-							target_.error(error.str(), location.str());
-						}
-					}
+			if (local_defines_ &&
+			    (arg = local_defines_->find(symbol)) != local_defines_->end())
+			{
+				if (strings_.size() - token.stack_pos != 1)
+				{
+					std::ostringstream error;
+					error << "macro argument '" << symbol
+					      << "' does not expect any argument";
+					std::ostringstream location;
+					location << linenum_ << ' ' << target_.location_;
+					target_.error(error.str(), location.str());
 				}
-#endif // 0
+				std::ostringstream v;
+				v << arg->second << "\376line " << linenum_ << ' ' << target_.location_
+				  << "\n\376textdomain " << target_.textdomain_ << '\n';
+				pop_token();
+				put(v.str());
+			}
+			else if ((macro = target_.defines_->find(symbol)) != target_.defines_->end())
+			{
 				preproc_define const &val = macro->second;
 				size_t nb_arg = strings_.size() - token.stack_pos - 1;
-				if (nb_arg != val.arguments.size()) {
-                    std::ostringstream error;
+				if (nb_arg != val.arguments.size())
+				{
+					std::ostringstream error;
 					error << "preprocessor symbol '" << symbol << "' expects "
 					      << val.arguments.size() << " arguments, but has "
 					      << nb_arg << " arguments";
-			        std::ostringstream location;
-					location<<linenum_<<' '<<target_.location_;
+					std::ostringstream location;
+					location << linenum_ << ' ' << target_.location_;
 					target_.error(error.str(), location.str());
 				}
-
-				std::stringstream *buffer = new std::stringstream;
-				std::string::const_iterator i_bra = val.value.end();
-				int macro_num = val.linenum;
-				std::string macro_textdomain = val.textdomain;
-				bool quoting = false;
-				for(std::string::const_iterator i = val.value.begin(),
-				    i_end = val.value.end(); i != i_end; ++i) {
-					char c = *i;
-					if (c == '\n') {
-						++macro_num;
-					} else if (c == '"') {
-						quoting = !quoting;
-					}
-
-					if (c == '{') {
-						if (i_bra != i_end)
-							buffer->write(&*i_bra - 1, i - i_bra + 1);
-						i_bra = i + 1;
-					} else if (i_bra == i_end) {
-						if (c == '#' && !quoting) {
-							// Keep track of textdomain changes in the body of the
-							// macro, so they can be restored after each substitution
-							// of a macro argument.
-							std::string::const_iterator i_beg = i + 1;
-							if (i_end - i_beg >= 13 &&
-							    std::equal(i_beg, i_beg + 10, "textdomain")) {
-								i_beg += 10;
-								i = std::find(i_beg, i_end, '\n');
-								if (i_beg != i)
-									++i_beg;
-								macro_textdomain = std::string(i_beg, i);
-								*buffer << "#textdomain " << macro_textdomain;
-								++macro_num;
-								c = '\n';
-							} else if((i_end - i_beg < 6 || (!std::equal(i_beg, i_beg + 6, "define")
-							&& !std::equal(i_beg, i_beg + 6, "ifndef")))
-							&& (i_end - i_beg < 5 || (!std::equal(i_beg, i_beg + 5, "ifdef")
-							&& !std::equal(i_beg, i_beg + 5, "endif") && !std::equal(i_beg, i_beg + 5, "undef")))
-							&& (i_end - i_beg < 4 || !std::equal(i_beg, i_beg + 4, "else"))) {
-								// Check for define, ifdef, ifndef, endif, undef, else.
-								// Otherwise, this is a comment and should be skipped.
-								i = std::find(i_beg, i_end, '\n');
-								++macro_num;
-								c = '\n';
-							}
-						}
-						buffer->put(c);
-					} else if (c == '}') {
-						size_t sz = i - i_bra;
-						for(size_t n = 0; n < nb_arg; ++n) {
-							std::string const &arg = val.arguments[n];
-							if (arg.size() != sz ||
-							    !std::equal(i_bra, i, arg.begin()))
-								continue;
-							*buffer << strings_[token.stack_pos + n + 1]
-							        << "\376line " << macro_num
-							        << ' ' << val.location << "\n\376textdomain "
-							        << macro_textdomain << '\n';
-							i_bra = i_end;
-							break;
-						}
-						if (i_bra != i_end) {
-							// The bracketed text was no macro argument
-							buffer->write(&*i_bra - 1, sz + 2);
-							i_bra = i_end;
-						}
-					}
+				std::istringstream *buffer = new std::istringstream(val.value);
+				std::map<std::string, std::string> *defines =
+					new std::map<std::string, std::string>;
+				for (size_t i = 0; i < nb_arg; ++i) {
+					(*defines)[val.arguments[i]] = strings_[token.stack_pos + i + 1];
 				}
-
 				pop_token();
 				std::string const &dir = directory_name(val.location.substr(0, val.location.find(' ')));
 				if (!slowpath_) {
 					DBG_CF << "substituting macro " << symbol << '\n';
-					new preprocessor_data(target_, called_macros_, buffer, val.location, "",
-					                      val.linenum, dir, val.textdomain, &symbol);
+					new preprocessor_data(target_, buffer, val.location, "",
+					                      val.linenum, dir, val.textdomain, defines);
 				} else {
 					DBG_CF << "substituting (slow) macro " << symbol << '\n';
 					std::ostringstream res;
 					preprocessor_streambuf *buf =
 						new preprocessor_streambuf(target_);
 					{	std::istream in(buf);
-						new preprocessor_data(*buf, called_macros_, buffer, val.location, "",
-						                      val.linenum, dir, val.textdomain, &symbol);
+						new preprocessor_data(*buf, buffer, val.location, "",
+						                      val.linenum, dir, val.textdomain, defines);
 						res << in.rdbuf(); }
 					delete buf;
 					strings_.back() += res.str();
@@ -953,13 +892,13 @@ bool preprocessor_data::get_chunk()
 				if (!nfname.empty())
 				{
 					if (!slowpath_)
-						new preprocessor_file(target_, called_macros_, nfname);
+						new preprocessor_file(target_, nfname);
 					else {
 						std::ostringstream res;
 						preprocessor_streambuf *buf =
 							new preprocessor_streambuf(target_);
 						{	std::istream in(buf);
-							new preprocessor_file(*buf, called_macros_, nfname);
+							new preprocessor_file(*buf, nfname);
 							res << in.rdbuf(); }
 						delete buf;
 						strings_.back() += res.str();
@@ -992,16 +931,15 @@ struct preprocessor_deleter: std::basic_istream<char>
 {
 	preprocessor_streambuf *buf_;
 	preproc_map *defines_;
-	std::vector<std::string> *callstack_;
 	std::string *error_log;
-	preprocessor_deleter(preprocessor_streambuf *buf, preproc_map *defines, std::vector<std::string>*);
+	preprocessor_deleter(preprocessor_streambuf *buf, preproc_map *defines);
 	~preprocessor_deleter();
 };
 
 preprocessor_deleter::preprocessor_deleter(preprocessor_streambuf *buf,
-		preproc_map *defines,
-		std::vector<std::string> *callstack)
-	: std::basic_istream<char>(buf), buf_(buf), defines_(defines), callstack_(callstack), error_log(buf->error_log)
+		preproc_map *defines)
+	: std::basic_istream<char>(buf), buf_(buf), defines_(defines)
+	, error_log(buf->error_log)
 {
 }
 
@@ -1010,7 +948,6 @@ preprocessor_deleter::~preprocessor_deleter()
 	rdbuf(NULL);
 	delete buf_;
 	delete defines_;
-	delete callstack_;
 }
 
 
@@ -1028,10 +965,9 @@ std::istream *preprocess_file(std::string const &fname,
 		defines = owned_defines;
 	}
 	preprocessor_streambuf *buf = new preprocessor_streambuf(defines, error_log);
-	std::vector<std::string> *callstack=new std::vector<std::string>;
-	new preprocessor_file(*buf, callstack, fname);
-	if(error_log!=NULL&&error_log->empty()==false)
+	new preprocessor_file(*buf, fname);
+	if (error_log && !error_log->empty())
 		throw preproc_config::error("Error preprocessing files.");
-	return new preprocessor_deleter(buf, owned_defines,callstack);
+	return new preprocessor_deleter(buf, owned_defines);
 }
 
