@@ -1,8 +1,8 @@
-/* $Id: string_utils.cpp 32608 2009-02-04 21:48:16Z grzywacz $ */
+/* $Id: string_utils.cpp 40554 2010-01-03 18:44:55Z mordante $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Copyright (C) 2005 - 2009 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2010 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -24,9 +24,11 @@
 #include "gettext.hpp"
 #include "log.hpp"
 #include "serialization/string_utils.hpp"
+#include "../util.hpp"
 
-#define ERR_GENERAL LOG_STREAM(err, general)
-#define ERR_NG LOG_STREAM(err, engine)
+static lg::log_domain log_engine("engine");
+#define ERR_GENERAL LOG_STREAM(err, lg::general)
+#define ERR_NG LOG_STREAM(err, log_engine)
 
 variable_set::~variable_set()
 {
@@ -70,14 +72,6 @@ std::string &strip(std::string &str)
 	return str;
 }
 
-std::string& strip_char(std::string &str, const char c) {
-	if (*str.begin() == c)
-		str.erase(str.begin(), str.begin() + 1);
-	if (*(str.end() - 1) == c)
-		str.erase(str.end() - 1, str.end());
-	return str;
-}
-
 std::vector< std::string > split(std::string const &val, char c, int flags)
 {
 	std::vector< std::string > res;
@@ -147,9 +141,9 @@ std::vector< std::string > paranthetical_split(std::string const &val,
 			i1=i2;
 			continue;
 		}
-		if(part.size() && *i2 == part.back()){
+		if(!part.empty() && *i2 == part.back()){
 			part.pop_back();
-			if(!separator && part.size() == 0){
+			if(!separator && part.empty()){
 				std::string new_val(i1, i2);
 				if (flags & STRIP_SPACES)
 					strip(new_val);
@@ -157,7 +151,7 @@ std::vector< std::string > paranthetical_split(std::string const &val,
 				++i2;
 				i1=i2;
 			}else{
-				if (part.size() == 0)
+				if (part.empty())
 					in_paranthesis = false;
 				++i2;
 			}
@@ -166,7 +160,7 @@ std::vector< std::string > paranthetical_split(std::string const &val,
 		bool found=false;
 		for(size_t i=0; i < lp.size(); i++){
 			if (*i2 == lp[i]){
-				if (!separator && part.size()==0){
+				if (!separator && part.empty()){
 					std::string new_val(i1, i2);
 					if (flags & STRIP_SPACES)
 						strip(new_val);
@@ -193,7 +187,7 @@ std::vector< std::string > paranthetical_split(std::string const &val,
 	if (!(flags & REMOVE_EMPTY) || !new_val.empty())
 		res.push_back(new_val);
 
-	if(part.size()){
+	if(!part.empty()){
 			ERR_GENERAL << "Mismatched paranthesis:\n"<<val<<"\n";;
 	}
 
@@ -213,35 +207,33 @@ int apply_modifier( const int number, const std::string &amount, const int minim
 	return value;
 }
 
-std::string &escape(std::string &str, const std::string& special_chars)
+std::string escape(const std::string &str, const char *special_chars)
 {
-	std::string::size_type pos = 0;
+	std::string::size_type pos = str.find_first_of(special_chars);
+	if (pos == std::string::npos) {
+		// Fast path, possibly involving only reference counting.
+		return str;
+	}
+	std::string res = str;
 	do {
-		pos = str.find_first_of(special_chars, pos);
-		if (pos == std::string::npos)
-			break;
-		str.insert(pos, 1, '\\');
-		pos += 2;
-	} while (pos < str.size());
-	return str;
+		res.insert(pos, 1, '\\');
+		pos = res.find_first_of(special_chars, pos + 2);
+	} while (pos != std::string::npos);
+	return res;
 }
 
-std::string& escape(std::string& str)
+std::string unescape(const std::string &str)
 {
-	static const std::string special_chars("#@{}+-,\\*=");
-	return escape(str, special_chars);
-}
-
-std::string &unescape(std::string &str)
-{
-	std::string::size_type pos = 0;
+	std::string::size_type pos = str.find('\\');
+	if (pos == std::string::npos) {
+		// Fast path, possibly involving only reference counting.
+		return str;
+	}
+	std::string res = str;
 	do {
-		pos = str.find('\\', pos);
-		if (pos == std::string::npos)
-			break;
-		str.erase(pos, 1);
-		++pos;
-	} while (pos < str.size());
+		res.erase(pos, 1);
+		pos = res.find('\\', pos + 1);
+	} while (pos != std::string::npos);
 	return str;
 }
 
@@ -563,23 +555,19 @@ void utf8_iterator::update()
 
 std::string wstring_to_string(const wide_string &src)
 {
-	wchar_t ch;
-	wide_string::const_iterator i;
-	Uint32 bitmask;
 	std::string ret;
 
 	try {
-
-		for(i = src.begin(); i != src.end(); ++i) {
+		for(wide_string::const_iterator i = src.begin(); i != src.end(); ++i) {
 			unsigned int count;
-			ch = *i;
+			wchar_t ch = *i;
 
 			// Determine the bytes required
 			count = 1;
 			if(ch >= 0x80)
 				count++;
 
-			bitmask = 0x800;
+			Uint32 bitmask = 0x800;
 			for(unsigned int j = 0; j < 5; ++j) {
 				if(static_cast<Uint32>(ch) >= bitmask) {
 					count++;
@@ -644,48 +632,6 @@ wide_string string_to_wstring(const std::string &src)
 	return res;
 }
 
-utf8_string capitalize(const utf8_string& s)
-{
-	if(s.size() > 0) {
-		utf8_iterator itor(s);
-#if defined(__APPLE__) || defined(__AMIGAOS4__)
-		/** @todo FIXME: Should we support towupper on recent OSX platforms? */
-		wchar_t uchar = *itor;
-		if(uchar >= 0 && uchar < 0x100)
-			uchar = toupper(uchar);
-		std::string res = utils::wchar_to_string(uchar);
-#else
-		std::string res = utils::wchar_to_string(towupper(*itor));
-#endif
-		res.append(itor.substr().second, s.end());
-		return res;
-	}
-	return s;
-}
-
-utf8_string uppercase(const utf8_string& s)
-{
-	if(s.size() > 0) {
-		utf8_iterator itor(s);
-		std::string res;
-
-		for(;itor != utf8_iterator::end(s); ++itor) {
-#if defined(__APPLE__) || defined(__AMIGAOS4__)
-			/** @todo FIXME: Should we support towupper on recent OSX platforms? */
-			wchar_t uchar = *itor;
-			if(uchar >= 0 && uchar < 0x100)
-				uchar = toupper(uchar);
-			res += utils::wchar_to_string(uchar);
-#else
-			res += utils::wchar_to_string(towupper(*itor));
-#endif
-		}
-
-		return res;
-	}
-	return s;
-}
-
 utf8_string lowercase(const utf8_string& s)
 {
 	if(s.size() > 0) {
