@@ -1,6 +1,6 @@
-/* $Id: ban.cpp 33097 2009-02-25 13:37:02Z soliton $ */
+/* $Id: ban.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2008 - 2009 by Pauli Nieminen <paniemin@cc.hut.fi>
+   Copyright (C) 2008 - 2010 by Pauli Nieminen <paniemin@cc.hut.fi>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,10 +13,13 @@
 */
 
 #include "config.hpp"
+#include "foreach.hpp"
 #include "log.hpp"
 #include "filesystem.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/binary_or_text.hpp"
+#include "serialization/string_utils.hpp"
+#include "util.hpp"
 
 #include "ban.hpp"
 
@@ -25,16 +28,18 @@
 
 namespace wesnothd {
 
-#define ERR_SERVER LOG_STREAM(err, mp_server)
-#define LOG_SERVER LOG_STREAM(info, mp_server)
-#define DBG_SERVER LOG_STREAM(debug, mp_server)
+
+static lg::log_domain log_server("server");
+#define ERR_SERVER LOG_STREAM(err, log_server)
+#define LOG_SERVER LOG_STREAM(info, log_server)
+#define DBG_SERVER LOG_STREAM(debug, log_server)
 
 	std::ostream& operator<<(std::ostream& o, const banned& n)
 	{
 	   return o << "IP: " << n.get_ip() <<
 					(n.get_nick().empty() ? "" : "  nick: " + n.get_nick()) <<
 					"  reason: '" << n.get_reason() << "'"
-					"\nstart_time: " << n.get_human_start_time() <<
+					"  start_time: " << n.get_human_start_time() <<
 					"  end_time: " << n.get_human_end_time() <<
 					"  issuer: " <<  n.get_who_banned();
 	}
@@ -75,15 +80,6 @@ namespace wesnothd {
 		return a->get_mask_ip(b->mask()) < b->get_mask_ip(a->mask());
 	}
 
-	subnet_compare_setter::subnet_compare_setter()
-	{
-		banned_compare_subnet::set_use_subnet_mask(true);
-	}
-
-	subnet_compare_setter::~subnet_compare_setter()
-	{
-		banned_compare_subnet::set_use_subnet_mask(false);
-	}
 	const std::string banned::who_banned_default_ = "system";
 
 	banned_ptr banned::create_dummy(const std::string& ip)
@@ -143,7 +139,7 @@ namespace wesnothd {
 		read(cfg);
 	}
 
-	banned::ip_mask banned::parse_ip(const std::string& ip) const
+	ip_mask parse_ip(const std::string& ip)
 	{
 		// We use bit operations to construct the integer
 		// ip_mask is a pair: first is ip and second is mask
@@ -151,9 +147,11 @@ namespace wesnothd {
 		ret.first = 0;
 		ret.second = 0;
 		std::vector<std::string> split_ip = utils::split(ip, '.');
+		if (split_ip.size() > 4) throw banned::error("Malformed ip address: " + ip);
+
 		unsigned int shift = 4*8; // start shifting from the highest byte
 		unsigned int mask = 0xFF000000;
-		const unsigned int complite_part_mask = 0xFF;
+		const unsigned int complete_part_mask = 0xFF;
 		std::vector<std::string>::const_iterator part = split_ip.begin();
 		bool wildcard = false;
 		do {
@@ -162,7 +160,7 @@ namespace wesnothd {
 			if (part == split_ip.end())
 			{
 				if (!wildcard)
-					throw banned::error("Malformed ip address given for ban: " + ip);
+					throw banned::error("Malformed ip address: '" + ip + "'");
 				// Adding 0 to ip and mask is nop
 				// we can then break out of loop
 				break;
@@ -172,12 +170,12 @@ namespace wesnothd {
 					wildcard = true;
 					// Adding 0 to ip and mask is nop
 				} else {
-					wildcard = false;
-					unsigned int part_ip = lexical_cast_default<unsigned int>(*part, complite_part_mask + 1);
-					if (part_ip > complite_part_mask)
-						throw banned::error("Malformed ip address given for ban: " + ip);
+					//wildcard = false;
+					unsigned int part_ip = lexical_cast_default<unsigned int>(*part, complete_part_mask + 1);
+					if (part_ip > complete_part_mask)
+						throw banned::error("Malformed ip address: '" + ip + "'");
 					ret.first |= (part_ip << shift);
-					ret.second |= (complite_part_mask << shift);
+					ret.second |= (complete_part_mask << shift);
 				}
 			}
 			++part;
@@ -196,10 +194,10 @@ namespace wesnothd {
 		}
 		nick_ = cfg["nick"];
 		if (cfg.has_attribute("end_time"))
-			end_time_ 	= lexical_cast_default<time_t>(cfg["end_time"], 0);
+			end_time_ = lexical_cast_default<time_t>(cfg["end_time"], 0);
 		if (cfg.has_attribute("start_time"))
-			start_time_ 	= lexical_cast_default<time_t>(cfg["start_time"], 0);
-		reason_	  	= cfg["reason"];
+			start_time_ = lexical_cast_default<time_t>(cfg["start_time"], 0);
+		reason_ = cfg["reason"];
 
 		// only overwrite defaults if exists
 		if (cfg.has_attribute("who_banned"))
@@ -210,7 +208,7 @@ namespace wesnothd {
 
 	void banned::write(config& cfg) const
 	{
-		cfg["ip"]		= get_ip();
+		cfg["ip"] = get_ip();
 		cfg["nick"] = get_nick();
 		if (end_time_ > 0)
 		{
@@ -225,7 +223,7 @@ namespace wesnothd {
 			cfg["start_time"] = ss.str();
 		}
 
-		cfg["reason"]	= reason_;
+		cfg["reason"] = reason_;
 		if (who_banned_ != who_banned_default_)
 		{
 			cfg["who_banned"] = who_banned_;
@@ -240,17 +238,7 @@ namespace wesnothd {
 	{
 		if (start_time_ == 0)
 			return "unknown";
-		return banned::get_human_time(start_time_);
-	}
-
-	std::string banned::get_human_time(const time_t& time)
-	{
-		char buf[30];
-		struct tm* local;
-		local = localtime(&time);
-		strftime(buf,30,"%H:%M:%S %d.%m.%Y", local );
-		return std::string(buf);
-
+		return lg::get_timestamp(start_time_, "%H:%M:%S %d.%m.%Y");
 	}
 
 	std::string banned::get_human_end_time() const
@@ -259,7 +247,7 @@ namespace wesnothd {
 		{
 			return "permanent";
 		}
-		return banned::get_human_time(end_time_);
+		return lg::get_timestamp(end_time_, "%H:%M:%S %d.%m.%Y");
 	}
 
 	bool banned::operator>(const banned& b) const
@@ -272,13 +260,13 @@ namespace wesnothd {
 		return ip_ & mask & mask_;
 	}
 
-	bool banned::match_ip(const std::string& ip) const {
-		try {
-			ip_mask pair = parse_ip(ip);
-			return (ip_ & mask_) == (pair.first & mask_);
-		} catch (banned::error& e) {
-			return false;
-		}
+	bool banned::match_ip(const ip_mask& pair) const {
+		return (ip_ & mask_) == (pair.first & mask_);
+	}
+
+	// Unlike match_ip this function takes both masks into account.
+	bool banned::match_ipmask(const ip_mask& pair) const {
+		return (ip_ & mask_ & pair.second) == (pair.first & pair.second & mask_);
 	}
 
 	void ban_manager::read()
@@ -290,12 +278,10 @@ namespace wesnothd {
 		scoped_istream ban_file = istream_file(filename_);
 		read_gz(cfg, *ban_file);
 
-		const config::child_list& bans = cfg.get_children("ban");
-		for (config::child_list::const_iterator itor = bans.begin();
-				itor != bans.end(); ++itor)
+		foreach (const config &b, cfg.child_range("ban"))
 		{
 			try {
-				banned_ptr new_ban(new banned(**itor));
+				banned_ptr new_ban(new banned(b));
 				assert(bans_.insert(new_ban).second);
 
 				if (new_ban->get_end_time() != 0)
@@ -306,15 +292,12 @@ namespace wesnothd {
 		}
 
 		// load deleted too
-		if (cfg.child("deleted"))
+		if (const config &cfg_del = cfg.child("deleted"))
 		{
-			const config& cfg_del = *cfg.child("deleted");
-			const config::child_list& del_bans = cfg_del.get_children("ban");
-			for (config::child_list::const_iterator itor = del_bans.begin();
-					itor != del_bans.end(); ++itor)
+			foreach (const config &b, cfg_del.child_range("ban"))
 			{
 				try {
-					banned_ptr new_ban(new banned(**itor));
+					banned_ptr new_ban(new banned(b));
 					deleted_bans_.push_back(new_ban);
 				} catch (banned::error& e) {
 					ERR_SERVER << e.message << " while reading deleted bans\n";
@@ -351,19 +334,17 @@ namespace wesnothd {
 		writer.write(cfg);
 	}
 
-	time_t ban_manager::parse_time(std::string time_in) const
+	bool ban_manager::parse_time(const std::string& duration, time_t* time) const
 	{
-		time_t ret;
-		ret = time(NULL);
-		if (time_in.substr(0,4) == "TIME")
-		{
+		if (!time) return false;
+
+		if (duration.substr(0,4) == "TIME") {
 			struct tm* loc;
-			loc = localtime(&ret);
+			loc = localtime(time);
 
-			std::string::iterator i = time_in.begin() + 4;
 			size_t number = 0;
-			for (; i != time_in.end(); ++i)
-			{
+			for (std::string::const_iterator i = duration.begin() + 4;
+					i != duration.end(); ++i) {
 				if (is_digit(*i))
 				{
 					number = number * 10 + to_digit(*i);
@@ -397,64 +378,115 @@ namespace wesnothd {
 					number = 0;
 				}
 			}
-			return mktime(loc);
+			*time = mktime(loc);
+			return true;
 		}
-		default_ban_times::const_iterator time_itor = ban_times_.find(time_in);
-		if (time_itor != ban_times_.end())
-			ret += time_itor->second;
-		else
-		{
-			const size_t default_multipler = 60; // default to minutes
-			size_t multipler = default_multipler;
-			std::string::iterator i = time_in.begin();
-			size_t number = 0;
-			for (; i != time_in.end(); ++i)
-			{
+		default_ban_times::const_iterator time_itor = ban_times_.find(duration);
+		if (utils::lowercase(duration) == "permanent" || duration == "0") {
+			*time = 0;
+		} else if (ban_times_.find(duration) != ban_times_.end()) {
+			*time += time_itor->second;
+		} else {
+			std::string::const_iterator i = duration.begin();
+			int number = -1;
+			for (std::string::const_iterator d_end = duration.end(); i != d_end; ++i) {
 				if (is_digit(*i))
 				{
+					if (number == -1) number = 0;
 					number = number * 10 + to_digit(*i);
 				} else {
+					if (number == -1) number = 1;
 					switch(*i)
 					{
 						case 'Y':
-							multipler = 365*24*60*60; // a year;
+						case 'y':
+							if (++i != d_end && tolower(*i) == 'e'
+							&&  ++i != d_end && tolower(*i) == 'a'
+							&&  ++i != d_end && tolower(*i) == 'r'
+							&&  ++i != d_end && tolower(*i) == 's') {
+							} else --i;
+							*time += number * 365*24*60*60; // a year;
 							break;
 						case 'M':
-							multipler = 30*24*60*60; // 30 days
+							if (++i != d_end && tolower(*i) == 'i') {
+								if (++i != d_end && tolower(*i) == 'n'
+								&&  ++i != d_end && tolower(*i) == 'u'
+								&&  ++i != d_end && tolower(*i) == 't'
+								&&  ++i != d_end && tolower(*i) == 'e'
+								&&  ++i != d_end && tolower(*i) == 's') {
+								} else --i;
+								*time += number * 60;
+								break;
+							}
+							--i;
+							if (++i != d_end && tolower(*i) == 'o'
+							&&  ++i != d_end && tolower(*i) == 'n'
+							&&  ++i != d_end && tolower(*i) == 't'
+							&&  ++i != d_end && tolower(*i) == 'h'
+							&&  ++i != d_end && tolower(*i) == 's') {
+							} else --i;
+							*time += number * 30*24*60*60; // 30 days
 							break;
 						case 'D':
-							multipler = 24*60*60;
+						case 'd':
+							if (++i != d_end && tolower(*i) == 'a'
+							&&  ++i != d_end && tolower(*i) == 'y'
+							&&  ++i != d_end && tolower(*i) == 's') {
+							} else --i;
+							*time += number * 24*60*60;
 							break;
+						case 'H':
 						case 'h':
-							multipler = 60*60;
+							if (++i != d_end && tolower(*i) == 'o'
+							&&  ++i != d_end && tolower(*i) == 'u'
+							&&  ++i != d_end && tolower(*i) == 'r'
+							&&  ++i != d_end && tolower(*i) == 's') {
+							} else --i;
+							*time += number * 60*60;
 							break;
 						case 'm':
-							multipler = 60;
+							if (++i != d_end && tolower(*i) == 'o') {
+								if (++i != d_end && tolower(*i) == 'n'
+								&&  ++i != d_end && tolower(*i) == 't'
+								&&  ++i != d_end && tolower(*i) == 'h'
+								&&  ++i != d_end && tolower(*i) == 's') {
+								} else --i;
+								*time += number * 30*24*60*60; // 30 days
+								break;
+							}
+							--i;
+							if (++i != d_end && tolower(*i) == 'i'
+							&&  ++i != d_end && tolower(*i) == 'n'
+							&&  ++i != d_end && tolower(*i) == 'u'
+							&&  ++i != d_end && tolower(*i) == 't'
+							&&  ++i != d_end && tolower(*i) == 'e'
+							&&  ++i != d_end && tolower(*i) == 's') {
+							} else --i;
+							*time += number * 60;
 							break;
+						case 'S':
 						case 's':
-							multipler = 1;
+							if (++i != d_end && tolower(*i) == 'e'
+							&&  ++i != d_end && tolower(*i) == 'c'
+							&&  ++i != d_end && tolower(*i) == 'o'
+							&&  ++i != d_end && tolower(*i) == 'n'
+							&&  ++i != d_end && tolower(*i) == 'd'
+							&&  ++i != d_end && tolower(*i) == 's') {
+							} else --i;
+							*time += number;
 							break;
 						default:
-							DBG_SERVER << "Invalid time modifier given: '" << *i << "'. Assuming this is the begin of the reason.\n";
-							ret = number = multipler = 0;
+							return false;
 							break;
 					}
-					if (multipler == 0)
-						break;
-					if (number == 0)
-						number = 1;
-					ret += number * multipler;
-					multipler = default_multipler;
-					number = 0;
+					number = -1;
 				}
 			}
-			--i;
-			if (is_digit(*i))
-			{
-					ret += number * multipler;
+			if (is_digit(*--i)) {
+					*time += number * 60; // default to minutes
 			}
 		}
-		return ret;
+		return true;
 	}
 
 	std::string ban_manager::ban(const std::string& ip,
@@ -464,19 +496,19 @@ namespace wesnothd {
 								 const std::string& group,
 								 const std::string& nick)
 	{
+		std::ostringstream ret;
 		try {
 			ban_set::iterator ban;
 			if ((ban = bans_.find(banned::create_dummy(ip))) != bans_.end())
 			{
 				// Already exsiting ban for ip. We have to first remove it
-				LOG_SERVER << "Overwriting ban: " << (**ban) << "\n";
+				ret << "Overwriting ban: " << (**ban) << "\n";
 				bans_.erase(ban);
 			}
 		} catch (banned::error& e) {
 			ERR_SERVER << e.message << " while creating dummy ban for finding existing ban\n";
 			return e.message;
 		}
-		std::ostringstream ret;
 		try {
 			banned_ptr new_ban(new banned(ip, end_time, reason,who_banned, group, nick));
 			bans_.insert(new_ban);
@@ -551,7 +583,7 @@ namespace wesnothd {
 		write();
 	}
 
-	void ban_manager::list_deleted_bans(std::ostringstream& out) const
+	void ban_manager::list_deleted_bans(std::ostringstream& out, const std::string& mask) const
 	{
 		if (deleted_bans_.empty())
 		{
@@ -559,27 +591,43 @@ namespace wesnothd {
 			return;
 		}
 
+		ip_mask pair;
+		try {
+			pair = parse_ip(mask);
+		} catch (banned::error& e) {
+			out << "parse error: " << e.message;
+			return;
+		}
+
+		out << "DELETED BANS LIST";
 		for (deleted_ban_list::const_iterator i = deleted_bans_.begin();
 				i != deleted_bans_.end();
 				++i)
 		{
-			if (i != deleted_bans_.begin())
-				out << "\n";
-			out << (**i);
+			if ((*i)->match_ipmask(pair)) out << "\n" << (**i);
 		}
 
 	}
 
 
 
-	void ban_manager::list_bans(std::ostringstream& out) const
+	void ban_manager::list_bans(std::ostringstream& out, const std::string& mask) const
 	{
 		if (bans_.empty())
 		{
 			out << "No bans set.";
 			return;
 		}
-		out << "BAN LIST\n";
+
+		ip_mask pair;
+		try {
+			pair = parse_ip(mask);
+		} catch (banned::error& e) {
+			out << "parse error: " << e.message;
+			return;
+		}
+
+		out << "BAN LIST";
 		std::set<std::string> groups;
 
 		for (ban_set::const_iterator i = bans_.begin();
@@ -587,17 +635,16 @@ namespace wesnothd {
 		{
 			if ((*i)->get_group().empty())
 			{
-				if (i != bans_.begin())
-					out << "\n";
-				out << (**i);
+				if ((*i)->match_ipmask(pair)) out << "\n" << (**i);
 			} else {
 				groups.insert((*i)->get_group());
 			}
 		}
 
-		if (!groups.empty())
+		// Don't list ban groups when looking for specific bans.
+		if (!groups.empty() && mask == "*")
 		{
-			out << "\nban groups:\n";
+			out << "\nban groups: ";
 
 			out << *groups.begin();
 			std::ostream& (*fn)(std::ostream&,const std::string&) = &std::operator<<;
@@ -609,16 +656,26 @@ namespace wesnothd {
 
 	std::string ban_manager::is_ip_banned(const std::string& ip) const
 	{
-		ban_set::const_iterator ban = std::find_if(bans_.begin(), bans_.end(), boost::bind(&banned::match_ip, boost::bind(&banned_ptr::get, _1), ip));
+		ip_mask pair;
+		try {
+			pair = parse_ip(ip);
+		} catch (banned::error&) {
+			return "";
+		}
+		ban_set::const_iterator ban = std::find_if(bans_.begin(), bans_.end(), boost::bind(&banned::match_ip, boost::bind(&banned_ptr::get, _1), pair));
 		if (ban == bans_.end()) return "";
-		return (*ban)->get_reason();
+		const std::string& nick = (*ban)->get_nick();
+		return (*ban)->get_reason() + (nick.empty() ? "" : " (" + nick + ")");
 	}
 
 	void ban_manager::init_ban_help()
 	{
-		ban_help_ = "ban <ip|nickmask> [<time>] <reason>\n"
-				"The time format is: %d[%s[%d[%s[...]]]] where %s is a time modifier: s (seconds), m (minutes), h (hours), D (days), M (months) or Y (years) and %d is a number.\n"
-				"If no time is given then the ban is permanent.\n";
+		ban_help_ = "ban <mask> <time> <reason>\n"
+				"The time format is: %d[%s[%d[%s[...]]]] where %s is a time"
+				" modifier: s or S (seconds), m (minutes), h or H (hours), d"
+				" or D (days), M (months) or y or Y (years) and %d is a number.\n"
+				"Permanent bans can be set with 'permanent' or '0' as the time"
+				" argument.\n";
 		default_ban_times::iterator itor = ban_times_.begin();
 		if (itor != ban_times_.end())
 		{
@@ -631,7 +688,7 @@ namespace wesnothd {
 		}
 		if (!ban_times_.empty())
 		{
-			ban_help_ += " for standard ban times.\n";
+			ban_help_ += " for standard ban times. (not combinable)\n";
 		}
 		ban_help_ += "ban 127.0.0.1 2h20m flooded lobby\n"
 				"kban suokko 5D flooded again\n"
@@ -641,12 +698,11 @@ namespace wesnothd {
 	void ban_manager::load_config(const config& cfg)
 	{
 		ban_times_.clear();
-		const config::child_list& times = cfg.get_children("ban_time");
-		for (config::child_list::const_iterator itor = times.begin();
-				itor != times.end(); ++itor)
-		{
-			ban_times_.insert(default_ban_times::value_type((**itor)["name"],
-						parse_time((**itor)["time"])-time(NULL)));
+		foreach (const config &bt, cfg.child_range("ban_time")) {
+			time_t duration = 0;
+			if (parse_time(bt["time"], &duration)) {
+				ban_times_.insert(default_ban_times::value_type(bt["name"], duration));
+			}
 		}
 		init_ban_help();
 		if (filename_ != cfg["ban_save_file"])
