@@ -1,6 +1,6 @@
-/* $Id: ban.hpp 37602 2009-08-09 22:03:14Z soliton $ */
+/* $Id: ban.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2008 - 2009 by Pauli Nieminen <paniemin@cc.hut.fi>
+   Copyright (C) 2008 - 2010 by Pauli Nieminen <paniemin@cc.hut.fi>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -39,7 +39,6 @@ namespace wesnothd {
 		bool operator()(const banned_ptr& a, const banned_ptr& b) const;
 	};
 
-	struct subnet_compare_setter;
 	struct banned_compare_subnet {
 		bool operator()(const banned_ptr& a, const banned_ptr& b) const;
 		private:
@@ -48,12 +47,6 @@ namespace wesnothd {
 		bool less_with_subnet(const banned_ptr& a, const banned_ptr& b) const;
 		typedef bool (banned_compare_subnet::*compare_fn)(const banned_ptr& a, const banned_ptr& b) const;
 		static compare_fn active_;
-		friend struct subnet_compare_setter;
-	};
-
-	struct subnet_compare_setter {
-		subnet_compare_setter();
-		~subnet_compare_setter();
 	};
 
 	typedef std::set<banned_ptr,banned_compare_subnet > ban_set;
