@@ -1,6 +1,6 @@
-/* $Id: game.cpp 38325 2009-09-02 16:40:00Z soliton $ */
+/* $Id: game.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -20,43 +20,32 @@
 #include "../map.hpp" // gamemap::MAX_PLAYERS
 
 #include "game.hpp"
+#include "player_network.hpp"
+#include "serialization/string_utils.hpp"
+#include "util.hpp"
 
-#ifndef __func__
- #ifdef __FUNCTION__
-  #define __func__ __FUNCTION__
- #endif
-#endif
+#include <boost/bind.hpp>
 
-
-#define ERR_GAME LOG_STREAM(err, mp_server)
-#define WRN_GAME LOG_STREAM(warn, mp_server)
-#define LOG_GAME LOG_STREAM(info, mp_server)
-#define DBG_GAME LOG_STREAM(debug, mp_server)
-#define WRN_CONFIG LOG_STREAM(warn, config)
-
-namespace chat_message {
-
-const size_t max_message_length = 256;
-static void truncate_message(const simple_wml::string_span& str, simple_wml::node& message) {
-	// testing for msg.size() is not sufficient but we're not getting false negatives
-	// and it's cheaper than always converting to wstring.
-	if(str.size() > static_cast<int>(chat_message::max_message_length)) {
-		std::string tmp(str.begin(), str.end());
-		// The string can contain utf-8 characters so truncate as wide_string otherwise
-		// a corrupted utf-8 string can be returned.
-		utils::truncate_as_wstring(tmp, max_message_length);
-		message.set_attr_dup("message", tmp.c_str());
-	}
-}
-
-} // end chat_message namespace
+static lg::log_domain log_server("server");
+#define ERR_GAME LOG_STREAM(err, log_server)
+#define WRN_GAME LOG_STREAM(warn, log_server)
+#define LOG_GAME LOG_STREAM(info, log_server)
+#define DBG_GAME LOG_STREAM(debug, log_server)
+static lg::log_domain log_config("config");
+#define WRN_CONFIG LOG_STREAM(warn, log_config)
 
 namespace wesnothd {
 int game::id_num = 1;
 
+void game::missing_user(network::connection socket, const std::string& func) const
+{
+	WRN_GAME << func << "(): Could not find user (socket:\t" << socket
+		<< ") in player_info_ in game:\t\"" << name_ << "\" (" << id_ << ")\n";
+}
+
 game::game(player_map& players, const network::connection host,
-		const std::string name, bool save_replays,
-		const std::string replay_save_path) :
+		const std::string& name, bool save_replays,
+		const std::string& replay_save_path) :
 	player_info_(&players),
 	id_(id_num++),
 	name_(name),
@@ -79,24 +68,20 @@ game::game(player_map& players, const network::connection host,
 	save_replays_(save_replays),
 	replay_save_path_(replay_save_path)
 {
-	// Hack to handle the pseudo games lobby_ and not_logged_in_.
-	if (owner_ == 0) return;
+	assert(owner_);
 	players_.push_back(owner_);
 	const player_map::iterator pl = player_info_->find(owner_);
 	if (pl == player_info_->end()) {
-		ERR_GAME << "ERROR: Could not find host in player_info_. (socket: "
-			<< owner_ << ")\n";
+		missing_user(owner_, __func__);
 		return;
 	}
 	// Mark the host as unavailable in the lobby.
 	pl->second.mark_available(id_, name_);
+	pl->second.set_status(player::PLAYING);
 }
 
 game::~game()
 {
-	// Hack to handle the pseudo games lobby_ and not_logged_in_.
-	if (owner_ == 0) return;
-
 	save_replay();
 
 	user_vector users = all_game_users();
@@ -143,6 +128,30 @@ std::string describe_turns(int turn, const simple_wml::string_span& num_turns)
 
 }//anon namespace
 
+std::string game::username(const player_map::const_iterator pl) const
+{
+	if (pl != player_info_->end()) {
+		return pl->second.name();
+	}
+
+	return "(unknown)";
+}
+
+std::string game::list_users(user_vector users, const std::string& func) const
+{
+	std::string list;
+
+	for (user_vector::const_iterator user = users.begin(); user != users.end(); ++user) {
+		const player_map::const_iterator pl = player_info_->find(*user);
+		if (pl != player_info_->end()) {
+			if (!list.empty()) list += ", ";
+			list += pl->second.name();
+		} else missing_user(*user, func);
+	}
+
+	return list;
+}
+
 void game::start_game(const player_map::const_iterator starter) {
 	// If the game was already started we're actually advancing.
 	const bool advance = started_;
@@ -153,7 +162,7 @@ void game::start_game(const player_map::const_iterator starter) {
 	LOG_GAME << network::ip_address(starter->first) << "\t"
 		<< starter->second.name() << "\t" << (advance ? "advanced" : "started")
 		<< (save ? " reloaded" : "") << " game:\t\"" << name_ << "\" (" << id_
-		<< "). Settings: map: " << s["id"]
+		<< ") with: " << list_users(players_, __func__) << ". Settings: map: " << s["id"]
 		<< "\tera: "       << (s.child("era") ? (*s.child("era"))["id"] : "")
 		<< "\tXP: "        << s["experience_modifier"]
 		<< "\tGPV: "       << s["mp_village_gold"]
@@ -193,9 +202,9 @@ void game::start_game(const player_map::const_iterator starter) {
 	int turn = 1;
 	int side = 0;
 	// Savegames have a snapshot that tells us which side starts.
-	if (s.child("snapshot")) {
-		turn = lexical_cast_default<int>((*s.child("snapshot"))["turn_at"], 1);
-		side = lexical_cast_default<int>((*s.child("snapshot"))["playing_team"], 0);
+	if (const simple_wml::node* snapshot = s.child("snapshot")) {
+		turn = lexical_cast_default<int>((*snapshot)["turn_at"], 1);
+		side = lexical_cast_default<int>((*snapshot)["playing_team"], 0);
 		LOG_GAME << "Reload from turn: " << turn
 			<< ". Current side is: " << side + 1 << ".\n";
 	}
@@ -221,7 +230,7 @@ bool game::send_taken_side(simple_wml::document& cfg, const simple_wml::node::ch
 	cfg.root().set_attr_dup("side", (**side)["side"]);
 
 	// Tell the host which side the new player should take.
-	return send_to_one(cfg, owner_);
+	return wesnothd::send_to_one(cfg, owner_);
 }
 
 bool game::take_side(const player_map::const_iterator user)
@@ -271,20 +280,18 @@ void game::update_side_data() {
 	observers_.clear();
 
 	const simple_wml::node::child_list& level_sides = level_.root().children("side");
-	if (!lg::debug.dont_log(lg::mp_server)) {
+	if (!lg::debug.dont_log(log_server)) {
 		for (simple_wml::node::child_list::const_iterator side = level_sides.begin();
 				side != level_sides.end(); ++side)
-			DBG_GAME << "[side]\n" << (**side).output() << "[/side]\n";
+			DBG_GAME << "[side]\n" << simple_wml::node_to_string(**side) << "[/side]\n";
 	}
 	// For each user:
 	// * Find the username.
 	// * Find the side this username corresponds to.
 	for (user_vector::const_iterator user = users.begin(); user != users.end(); ++user) {
-		const player_map::const_iterator info = player_info_->find(*user);
+		player_map::iterator info = player_info_->find(*user);
 		if (info == player_info_->end()) {
-			ERR_GAME << "Game: " << id_
-				<< " ERROR: unable to find user info for connection: "
-				<< *user << "\n";
+			missing_user(*user, __func__);
 			continue;
 		}
 
@@ -314,8 +321,10 @@ void game::update_side_data() {
 		}
 		if (side_found) {
 			players_.push_back(*user);
+			info->second.set_status(player::PLAYING);
 		} else {
 			observers_.push_back(*user);
+			info->second.set_status(player::OBSERVING);
 		}
 	}
 	DBG_GAME << debug_player_info();
@@ -345,12 +354,9 @@ void game::transfer_side_control(const network::connection sock, const simple_wm
 
 	const simple_wml::string_span& newplayer_name = cfg["player"];
 	const network::connection old_player = sides_[side_num - 1];
-	const player_map::const_iterator oldplayer = player_info_->find(old_player);
-	if (oldplayer == player_info_->end()) {
-		WRN_GAME << "Could not find old player in player_info_. (socket: "
-			<< old_player << ")\n";
-	}
-	const std::string old_player_name(oldplayer != player_info_->end() ? oldplayer->second.name() : "(unknown)");
+	const player_map::iterator oldplayer = player_info_->find(old_player);
+	if (oldplayer == player_info_->end()) missing_user(old_player, __func__);
+	const std::string old_player_name = username(oldplayer);
 
 	// A player (un)droids his side.
 	if (newplayer_name.empty()) {
@@ -382,12 +388,8 @@ void game::transfer_side_control(const network::connection sock, const simple_wm
 		return;
 	}
 	//find the player that is passed control
-	player_map::const_iterator newplayer;
-	for (newplayer = player_info_->begin(); newplayer != player_info_->end(); newplayer++) {
-		if (newplayer_name == newplayer->second.name().c_str()) {
-			break;
-		}
-	}
+	player_map::iterator newplayer = find_user(newplayer_name);
+
 	// Is he in this game?
 	if (newplayer == player_info_->end() || !is_member(newplayer->first)) {
 		send_server_message((newplayer_name.to_string() + " is not in this game").c_str(), sock);
@@ -405,6 +407,7 @@ void game::transfer_side_control(const network::connection sock, const simple_wm
 	if (std::find(sides_.begin(), sides_.end(), old_player) == sides_.end()
 	&& is_player(old_player)) {
 		observers_.push_back(old_player);
+		oldplayer->second.set_status(player::OBSERVING);
 		players_.erase(std::remove(players_.begin(), players_.end(), old_player), players_.end());
 		// Tell others that the player becomes an observer.
 		send_and_record_server_message((old_player_name + " becomes an observer.").c_str());
@@ -418,6 +421,7 @@ void game::transfer_side_control(const network::connection sock, const simple_wm
 	// If we gave the new side to an observer add him to players_.
 	if (is_observer(newplayer->first)) {
 		players_.push_back(newplayer->first);
+		newplayer->second.set_status(player::PLAYING);
 		observers_.erase(std::remove(observers_.begin(), observers_.end(), newplayer->first), observers_.end());
 		// Send everyone but the new player the observer_quit message.
 		send_observerquit(newplayer);
@@ -462,7 +466,7 @@ void game::change_controller(const size_t side_num,
 	// side_drop already.)
 	if (!player_left) {
 		change.set_attr("controller", (side_controllers_[side_num] == "ai" ? "human_ai" : "human"));
-		send_to_one(response, sock);
+		wesnothd::send_to_one(response, sock);
 	}
 
 	// Update the level so observers who join get the new name. (The host handles level changes before game start.)
@@ -476,9 +480,7 @@ void game::change_controller(const size_t side_num,
 }
 
 void game::notify_new_host(){
-	const std::string owner_name =
-			(player_info_->find(owner_) != player_info_->end()
-			? player_info_->find(owner_)->second.name() : "");
+	const std::string owner_name = username(player_info_->find(owner_));
 	simple_wml::document cfg;
 	simple_wml::node& cfg_host_transfer = cfg.root().add_child("host_transfer");
 
@@ -486,7 +488,7 @@ void game::notify_new_host(){
 	cfg_host_transfer.set_attr("name", owner_name.c_str());
 	cfg_host_transfer.set_attr("value", "1");
 	std::string message = owner_name + " has been chosen as the new host.";
-	if (!send_to_one(cfg, owner_)) {
+	if (!wesnothd::send_to_one(cfg, owner_)) {
 		message += " But an internal error occured. You probably have to abandon this game.";
 	}
 	send_and_record_server_message(message.c_str());
@@ -533,38 +535,37 @@ void game::mute_all_observers() {
 	}
 }
 
-void game::mute_observer(const simple_wml::node& mute, const player_map::const_iterator muter) {
+void game::send_muted_observers(const player_map::const_iterator user) const
+{
+	if (all_observers_muted_) {
+		send_server_message("All observers are muted.", user->first);
+		return;
+	}
+	std::string muted_nicks = list_users(muted_observers_, __func__);
+
+	send_server_message(("Muted observers: " + muted_nicks).c_str(), user->first);
+}
+
+void game::mute_observer(const simple_wml::node& mute,
+		const player_map::const_iterator muter)
+{
 	if (muter->first != owner_) {
 		send_server_message("You cannot mute: not the game host.", muter->first);
 		return;
 	}
-	const simple_wml::string_span& name = mute["username"];
-	if (name.empty()) {
-		if (all_observers_muted_) {
-			send_server_message("All observers are muted.", muter->first);
-			return;
-		}
-		std::string muted_nicks = "";
-		for (user_vector::const_iterator muted_obs = muted_observers_.begin();
-			 muted_obs != muted_observers_.end(); ++muted_obs)
-		{
-			if (muted_nicks != "") {
-				muted_nicks += ", ";
-			}
-			muted_nicks += (player_info_->find(*muted_obs) != player_info_->end()
-					? player_info_->find(*muted_obs)->second.name() : "");
-		}
-
-		send_server_message(("Muted observers: " + muted_nicks).c_str(), muter->first);
+	const simple_wml::string_span& username = mute["username"];
+	if (username.empty()) {
+		send_muted_observers(muter);
 		return;
 	}
-	const player_map::const_iterator user = find_user(name);
+
+	const player_map::const_iterator user = find_user(username);
 	/**
-	 * @todo FIXME: Maybe rather save muted nicks as a vector of strings and
+	 * @todo FIXME: Maybe rather save muted nicks as a set of strings and
 	 * also allow muting of usernames not in the game.
 	 */
 	if (user == player_info_->end() || !is_observer(user->first)) {
-		send_server_message("Observer not found.", muter->first);
+		send_server_message(("Observer '" + username.to_string() + "' not found.").c_str(), muter->first);
 		return;
 	}
 
@@ -574,21 +575,55 @@ void game::mute_observer(const simple_wml::node& mute, const player_map::const_i
 		return;
 	}
 	if (is_muted_observer(user->first)) {
-		send_server_message((user->second.name() + " is already muted.").c_str(), muter->first);
+		send_server_message((username.to_string() + " is already muted.").c_str(), muter->first);
 		return;
 	}
-	muted_observers_.push_back(user->first);
 	LOG_GAME << network::ip_address(muter->first) << "\t"
-		<< muter->second.name() << " muted: " << name << " ("
+		<< muter->second.name() << " muted: " << username << " ("
 		<< network::ip_address(user->first) << ")\tin game:\t\""
 		<< name_ << "\" (" << id_ << ")\n";
-	send_and_record_server_message((user->second.name() + " has been muted.").c_str());
+	muted_observers_.push_back(user->first);
+	send_and_record_server_message((username.to_string() + " has been muted.").c_str());
+}
+
+void game::unmute_observer(const simple_wml::node& unmute,
+		const player_map::const_iterator unmuter)
+{
+	if (unmuter->first != owner_) {
+		send_server_message("You cannot unmute: not the game host.", unmuter->first);
+		return;
+	}
+	const simple_wml::string_span& username = unmute["username"];
+	if (username.empty()) {
+		muted_observers_.clear();
+		send_and_record_server_message("Everyone has been unmuted.");
+		return;
+	}
+
+	const player_map::const_iterator user = find_user(username);
+	if (user == player_info_->end() || !is_observer(user->first)) {
+		send_server_message(("Observer '" + username.to_string() + "' not found.").c_str(), unmuter->first);
+		return;
+	}
+
+	if (!is_muted_observer(user->first)) {
+		send_server_message((username.to_string() + " is not muted.").c_str(), unmuter->first);
+		return;
+	}
+
+	LOG_GAME << network::ip_address(unmuter->first) << "\t"
+		<< unmuter->second.name() << " unmuted: " << username << " ("
+		<< network::ip_address(user->first) << ")\tin game:\t\""
+		<< name_ << "\" (" << id_ << ")\n";
+	muted_observers_.erase(std::remove(muted_observers_.begin(),
+				muted_observers_.end(), user->first), muted_observers_.end());
+	send_and_record_server_message((username.to_string() + " has been unmuted.").c_str());
 }
 
 void game::send_leave_game(network::connection user) const
 {
 	static simple_wml::document leave_game("[leave_game]\n[/leave_game]\n", simple_wml::INIT_COMPRESSED);
-	send_to_one(leave_game, user);
+	wesnothd::send_to_one(leave_game, user);
 }
 
 network::connection game::kick_member(const simple_wml::node& kick,
@@ -598,10 +633,10 @@ network::connection game::kick_member(const simple_wml::node& kick,
 		send_server_message("You cannot kick: not the game host", kicker->first);
 		return 0;
 	}
-	const simple_wml::string_span& name = kick["username"];
-	const player_map::const_iterator user = find_user(name);
+	const simple_wml::string_span& username = kick["username"];
+	const player_map::const_iterator user = find_user(username);
 	if (user == player_info_->end() || !is_member(user->first)) {
-		send_server_message("Not a member of this game.", kicker->first);
+		send_server_message(("'" + username.to_string() + "' is not a member of this game.").c_str(), kicker->first);
 		return 0;
 	} else if (user->first == kicker->first) {
 		send_server_message("Don't kick yourself, silly.", kicker->first);
@@ -611,10 +646,10 @@ network::connection game::kick_member(const simple_wml::node& kick,
 		return 0;
 	}
 	LOG_GAME << network::ip_address(kicker->first) << "\t"
-		<< kicker->second.name() << "\tkicked: " << name << " ("
+		<< kicker->second.name() << "\tkicked: " << username << " ("
 		<< network::ip_address(user->first) << ")\tfrom game:\t\""
 		<< name_ << "\" (" << id_ << ")\n";
-	send_and_record_server_message((name.to_string() + " has been kicked.").c_str());
+	send_and_record_server_message((username.to_string() + " has been kicked.").c_str());
 
 	// Tell the user to leave the game.
 	send_leave_game(user->first);
@@ -629,27 +664,27 @@ network::connection game::ban_user(const simple_wml::node& ban,
 		send_server_message("You cannot ban: not the game host", banner->first);
 		return 0;
 	}
-	const simple_wml::string_span& name = ban["username"];
-	const player_map::const_iterator user = find_user(name);
+	const simple_wml::string_span& username = ban["username"];
+	const player_map::const_iterator user = find_user(username);
 	if (user == player_info_->end()) {
-		send_server_message("User not found", banner->first);
+		send_server_message(("User '" + username.to_string() + "' not found.").c_str(), banner->first);
 		return 0;
 	} else if (user->first == banner->first) {
 		send_server_message("Don't ban yourself, silly.", banner->first);
 		return 0;
 	} else if (player_is_banned(user->first)) {
-		send_server_message(("'" + name.to_string() + "' is already banned.").c_str(), banner->first);
+		send_server_message(("'" + username.to_string() + "' is already banned.").c_str(), banner->first);
 		return 0;
 	} else if (user->second.is_moderator()) {
 		send_server_message("You're not allowed to ban a moderator.", banner->first);
 		return 0;
 	}
 	LOG_GAME << network::ip_address(banner->first) << "\t"
-		<< banner->second.name() << "\tbanned: " << name << " ("
+		<< banner->second.name() << "\tbanned: " << username << " ("
 		<< network::ip_address(user->first) << ")\tfrom game:\t\""
 		<< name_ << "\" (" << id_ << ")\n";
 	bans_.push_back(network::ip_address(user->first));
-	send_and_record_server_message((name.to_string() + " has been banned.").c_str());
+	send_and_record_server_message((username.to_string() + " has been banned.").c_str());
 	if (is_member(user->first)) {
 		//tell the user to leave the game.
 		send_leave_game(user->first);
@@ -660,16 +695,34 @@ network::connection game::ban_user(const simple_wml::node& ban,
 	return 0;
 }
 
+void game::unban_user(const simple_wml::node& unban,
+		const player_map::const_iterator unbanner)
+{
+	if (unbanner->first != owner_) {
+		send_server_message("You cannot unban: not the game host.", unbanner->first);
+		return;
+	}
+	const simple_wml::string_span& username = unban["username"];
+	const player_map::const_iterator user = find_user(username);
+	if (user == player_info_->end()) {
+		send_server_message(("User '" + username.to_string() + "' not found.").c_str(), unbanner->first);
+		return;
+	}
+	if (!player_is_banned(user->first)) {
+		send_server_message(("'" + username.to_string() + "' is not banned.").c_str(), unbanner->first);
+		return;
+	}
+	LOG_GAME << network::ip_address(unbanner->first) << "\t"
+		<< unbanner->second.name() << "\tunbanned: " << username << " ("
+		<< network::ip_address(user->first) << ")\tfrom game:\t\""
+		<< name_ << "\" (" << id_ << ")\n";
+	bans_.erase(std::remove(bans_.begin(), bans_.end(), network::ip_address(user->first)), bans_.end());
+	send_and_record_server_message((username.to_string() + " has been unbanned.").c_str());
+}
+
 void game::process_message(simple_wml::document& data, const player_map::iterator user) {
-	// Hack to handle the pseudo game lobby_.
-	if (owner_ != 0) {
-	} else if (user->second.silenced()) {
-		return;
-	} else if (user->second.is_message_flooding()) {
-		send_server_message(
-				"Warning: you are sending too many messages too fast. "
-				"Your message has not been relayed.", user->first);
-		return;
+	if (owner_ == 0) {
+		ERR_GAME << "No owner in game::process_message\n";
 	}
 
 	simple_wml::node* const message = data.root().child("message");
@@ -679,19 +732,7 @@ void game::process_message(simple_wml::document& data, const player_map::iterato
 	const simple_wml::string_span& msg = (*message)["message"];
 	chat_message::truncate_message(msg, *message);
 
-	// Only log in the lobby_.
-	std::string game_prefix;
-	if (owner_ != 0) {
-		game_prefix = "game ";
-	} else if (msg.size() >= 3 && simple_wml::string_span(msg.begin(), 4) == "/me ") {
-		LOG_GAME << network::ip_address(user->first) << "\t<"
-			<< user->second.name() << simple_wml::string_span(msg.begin() + 3, msg.size() - 3) << ">\n";
-        } else {
-		LOG_GAME << network::ip_address(user->first) << "\t<"
-			<< user->second.name() << "> " << msg << "\n";
-	}
-
-	send_data(data, user->first, game_prefix + "message");
+	send_data(data, user->first, "game message");
 }
 
 bool game::is_legal_command(const simple_wml::node& command, bool is_player) {
@@ -729,14 +770,12 @@ bool game::process_turn(simple_wml::document& data, const player_map::const_iter
 		if (!is_current_player(user->first)
 		&& !is_legal_command(**command, player)) {
 			LOG_GAME << "ILLEGAL COMMAND in game: " << id_ << " ((("
-				<< (**command).output() << ")))\n";
+				<< simple_wml::node_to_string(**command) << ")))\n";
 			std::stringstream msg;
 			msg << "Removing illegal command '" << (**command).first_child().to_string()
 				<< "' from: " << user->second.name()
 				<< ". Current player is: "
-				<< (player_info_->find(current_player()) != player_info_->end()
-					? player_info_->find(current_player())->second.name()
-					: "(unfound)")
+				<< username(player_info_->find(current_player()))
 				<< " (" << end_turn_ + 1 << "/" << nsides_ << ").";
 			LOG_GAME << msg.str() << " (socket: " << current_player()
 				<< ") (game id: " << id_ << ")\n";
@@ -778,6 +817,16 @@ bool game::process_turn(simple_wml::document& data, const player_map::const_iter
 	for(std::vector<int>::const_iterator j = marked.begin(); j != marked.end(); ++j) {
 		turn->remove_child("command",*j);
 	}
+	for (command = commands.begin(); command != commands.end(); ++command) {
+		if (simple_wml::node* attack = (**command).child("attack")) {
+			int seed = rand();
+			attack->set_attr_int("seed", seed);
+			simple_wml::document doc;
+			simple_wml::node& rs = doc.root().add_child("random_seed");
+			rs.set_attr_int("seed", seed);
+			wesnothd::send_to_one(doc, user->first, "game replay");
+		}
+	}
 	if (turn->no_children()) {
 		return false;
 	}
@@ -792,7 +841,7 @@ bool game::process_turn(simple_wml::document& data, const player_map::const_iter
 			simple_wml::document* mdata = new simple_wml::document;
 			simple_wml::node& turn = mdata->root().add_child("turn");
 			(**command).copy_into(turn.add_child("command"));
-			send_data(*mdata, user->first,"game replay");
+			send_data(*mdata, user->first, "game replay");
 			record_data(mdata);
 			continue;
 		}
@@ -821,7 +870,7 @@ bool game::process_turn(simple_wml::document& data, const player_map::const_iter
 			send_data(*message, user->first, "game message");
 			record_data(message.release());
 		} else if (team_name == game_config::observer_team_name) {
-			send_data_observers(*message, user->first, "game message");
+			wesnothd::send_to_many(*message, observers_, user->first, "game message");
 			record_data(message.release());
 		} else {
 			send_data_team(*message, team_name, user->first, "game message");
@@ -863,15 +912,9 @@ bool game::add_player(const network::connection player, bool observer) {
 			<< player << ")\n";
 		return false;
 	}
-	// Hack to handle the pseudo games lobby_ and not_logged_in_.
-	if (owner_ == 0) {
-		observers_.push_back(player);
-		return true;
-	}
 	const player_map::iterator user = player_info_->find(player);
 	if (user == player_info_->end()) {
-		ERR_GAME << "ERROR: Could not find user in player_info_. (socket: "
-			<< owner_ << ")\n";
+		missing_user(player, __func__);
 		return false;
 	}
 	DBG_GAME << debug_player_info();
@@ -879,11 +922,15 @@ bool game::add_player(const network::connection player, bool observer) {
 	if (!started_ && !observer && take_side(user)) {
 		DBG_GAME << "adding player...\n";
 		players_.push_back(player);
+		user->second.set_status(player::PLAYING);
 		send_and_record_server_message((user->second.name() + " has joined the game.").c_str(), player);
 	} else if (!allow_observers() && !user->second.is_moderator()) {
 		return false;
 	} else {
-		if (!observer) became_observer = true;
+		if (!observer) {
+			became_observer = true;
+			observer = true;
+		}
 		DBG_GAME << "adding observer...\n";
 		observers_.push_back(player);
 		if (!allow_observers()) send_and_record_server_message((user->second.name() + " is now observing the game.").c_str(), player);
@@ -896,18 +943,18 @@ bool game::add_player(const network::connection player, bool observer) {
 	}
 	LOG_GAME << network::ip_address(player) << "\t" << user->second.name()
 		<< "\tjoined game:\t\"" << name_ << "\" (" << id_ << ")"
-		<< (observer || became_observer ? " as an observer" : "")
+		<< (observer ? " as an observer" : "")
 		<< ". (socket: " << player << ")\n";
 	user->second.mark_available(id_, name_);
+	user->second.set_status((observer) ? player::OBSERVING : player::PLAYING);
 	DBG_GAME << debug_player_info();
 	// Send the user the game data.
-	//std::cerr << "SENDING LEVEL {{{" << level_.output() << "}}}\n";
-	if (!send_to_one(level_, player)) return false;
+	if (!wesnothd::send_to_one(level_, player)) return false;
 
 	if(started_) {
 		//tell this player that the game has started
 		static simple_wml::document start_game_doc("[start_game]\n[/start_game]\n", simple_wml::INIT_COMPRESSED);
-		if (!send_to_one(start_game_doc, player)) return false;
+		if (!wesnothd::send_to_one(start_game_doc, player)) return false;
 		// Send observer join of all the observers in the game to the new player
 		// only once the game started. The client forgets about it anyway
 		// otherwise.
@@ -918,7 +965,7 @@ bool game::add_player(const network::connection player, bool observer) {
 		send_user_list();
 	}
 
-	const std::string clones = has_same_ip(player, observer || became_observer);
+	const std::string clones = has_same_ip(player, observer);
 	if (!clones.empty()) {
 		send_and_record_server_message((user->second.name() + " has the same IP as: " + clones).c_str());
 	}
@@ -936,18 +983,6 @@ bool game::remove_player(const network::connection player, const bool disconnect
 			<< player << ")\n";
 		return false;
 	}
-	// Hack to handle the pseudo games lobby_ and not_logged_in_.
-	if (owner_ == 0) {
-		const user_vector::iterator itor =
-			std::find(observers_.begin(), observers_.end(), player);
-		if (itor != observers_.end()) {
-			observers_.erase(itor);
-		} else {
-			ERR_GAME << "ERROR: Observer is not in this game. (socket: "
-				<< player << ")\n";
-		}
-		return false;
-	}
 	DBG_GAME << debug_player_info();
 	DBG_GAME << "removing player...\n";
 
@@ -958,8 +993,7 @@ bool game::remove_player(const network::connection player, const bool disconnect
 	const bool game_ended = players_.empty() || (host && !started_);
 	const player_map::iterator user = player_info_->find(player);
 	if (user == player_info_->end()) {
-		ERR_GAME << "ERROR: Could not find user in player_info_. (socket: "
-			<< player << ")\n";
+		missing_user(player, __func__);
 		return game_ended;
 	}
 	LOG_GAME << network::ip_address(user->first) << "\t" << user->second.name()
@@ -979,7 +1013,9 @@ bool game::remove_player(const network::connection player, const bool disconnect
 
 	// Don't mark_available() since the player got already removed from the
 	// games_and_users_list_.
-	if (!disconnect) user->second.mark_available();
+	if (!disconnect) {
+		user->second.mark_available();
+	}
 	if (observer) {
 		send_observerquit(user);
 	} else {
@@ -1000,12 +1036,12 @@ bool game::remove_player(const network::connection player, const bool disconnect
 		if (*side != player) continue;
 		if (side_controllers_[side_num] == "ai") ai_transfer = true;
 
-		player_map::const_iterator o = player_info_->find(owner_);
-		const std::string owner_name = (o != player_info_->end() ? o->second.name() : "(unknown)");
-		change_controller(side_num, owner_, owner_name);
+		player_map::iterator o = player_info_->find(owner_);
+		change_controller(side_num, owner_, username(o));
 		// Check whether the host is actually a player and make him one if not.
 		if (!is_player(owner_)) {
 			DBG_GAME << "making the owner a player...\n";
+			o->second.set_status(player::PLAYING);
 			observers_.erase(std::remove(observers_.begin(), observers_.end(), owner_), observers_.end());
 			players_.push_back(owner_);
 			send_observerquit(o);
@@ -1017,7 +1053,7 @@ bool game::remove_player(const network::connection player, const bool disconnect
 		drop.root().set_attr("side_drop", side_drop.c_str());
 		drop.root().set_attr("controller", side_controllers_[side_num].c_str());
 
-		send_to_one(drop, owner_);
+		wesnothd::send_to_one(drop, owner_);
 	}
 	if (ai_transfer) send_and_record_server_message("AI sides transferred to host.");
 
@@ -1050,42 +1086,18 @@ void game::load_next_scenario(const player_map::const_iterator user) const {
 	send_server_message_to_all((user->second.name() + " advances to the next scenario").c_str(), user->first);
 	simple_wml::document cfg_scenario;
 	level_.root().copy_into(cfg_scenario.root().add_child("next_scenario"));
-	if (!send_to_one(cfg_scenario, user->first)) return;
+	if (!wesnothd::send_to_one(cfg_scenario, user->first)) return;
 	// Send the player the history of the game to-date.
 	send_history(user->first);
 	// Send observer join of all the observers in the game to the user.
 	send_observerjoins(user->first);
 }
 
-void game::send_data(simple_wml::document& data, const network::connection exclude, std::string packet_type) const
+void game::send_data(simple_wml::document& data,
+						  const network::connection exclude,
+						  std::string packet_type) const
 {
-	if (packet_type.empty())
-		packet_type = data.root().first_child().to_string();
-	try {
-		simple_wml::string_span s = data.output_compressed();
-		const user_vector& users = all_game_users();
-		for(user_vector::const_iterator i = users.begin(); i != users.end(); ++i) {
-			if (*i != exclude) {
-				network::send_raw_data(s.begin(), s.size(), *i, packet_type);
-			}
-		}
-	} catch (simple_wml::error& e) {
-		WRN_CONFIG << __func__ << ": simple_wml error: " << e.message << std::endl;
-	}
-}
-
-bool game::send_to_one(simple_wml::document& data, const network::connection sock, std::string packet_type) const
-{
-	if (packet_type.empty())
-		packet_type = data.root().first_child().to_string();
-	try {
-		simple_wml::string_span s = data.output_compressed();
-		network::send_raw_data(s.begin(), s.size(), sock, packet_type);
-	} catch (simple_wml::error& e) {
-		WRN_CONFIG << __func__ << ": simple_wml error: " << e.message << std::endl;
-		return false;
-	}
-	return true;
+	wesnothd::send_to_many(data, all_game_users(), exclude, packet_type);
 }
 
 void game::send_data_team(simple_wml::document& data,
@@ -1094,34 +1106,11 @@ void game::send_data_team(simple_wml::document& data,
 						  std::string packet_type) const
 {
 	DBG_GAME << __func__ << "...\n";
-	if (packet_type.empty())
-		packet_type = data.root().first_child().to_string();
-	try {
-		simple_wml::string_span s = data.output_compressed();
-		for(user_vector::const_iterator i = players_.begin(); i != players_.end(); ++i) {
-			if(*i != exclude && is_on_team(team,*i)) {
-				network::send_raw_data(s.begin(), s.size(), *i, packet_type);
-			}
-		}
-	} catch (simple_wml::error& e) {
-		WRN_CONFIG << __func__ << ": simple_wml error: " << e.message << std::endl;
-	}
+	wesnothd::send_to_many(data, players_,
+		boost::bind(&game::is_on_team, this, boost::ref(team), _1),
+		exclude, packet_type);
 }
 
-void game::send_data_observers(simple_wml::document& data, const network::connection exclude, std::string packet_type) const {
-	if (packet_type.empty())
-		packet_type = data.root().first_child().to_string();
-	try {
-		simple_wml::string_span s = data.output_compressed();
-		for(user_vector::const_iterator i = observers_.begin(); i != observers_.end(); ++i) {
-			if (*i != exclude) {
-				network::send_raw_data(s.begin(), s.size(), *i, packet_type);
-			}
-		}
-	} catch (simple_wml::error& e) {
-		WRN_CONFIG << __func__ << ": simple_wml error: " << e.message << std::endl;
-	}
-}
 
 bool game::is_on_team(const simple_wml::string_span& team, const network::connection player) const {
 	const simple_wml::node::child_list& side_list = level_.root().children("side");
@@ -1163,9 +1152,7 @@ void game::send_observerjoins(const network::connection sock) const {
 		if (*ob == sock) continue;
 		const player_map::const_iterator obs = player_info_->find(*ob);
 		if (obs == player_info_->end()) {
-			ERR_GAME << "Game: " << id_
-				<< " ERROR: Can not find observer in player_info_. (socket: "
-				<< *ob << ")\n";
+			missing_user(*ob, __func__);
 			continue;
 		}
 
@@ -1176,7 +1163,7 @@ void game::send_observerjoins(const network::connection sock) const {
 			send_data(cfg, *ob);
 		} else {
 			// Send to the (new) user.
-			send_to_one(cfg, sock);
+			wesnothd::send_to_one(cfg, sock);
 		}
 	}
 }
@@ -1239,7 +1226,7 @@ void game::save_replay() {
 		const simple_wml::node::child_list& turn_list = (*i)->root().children("turn");
 		for (simple_wml::node::child_list::const_iterator turn = turn_list.begin();
 				turn != turn_list.end(); ++turn) {
-			replay_commands += (*turn)->output();
+			replay_commands += simple_wml::node_to_string(**turn);
 		}
 		delete *i;
 	}
@@ -1249,19 +1236,19 @@ void game::save_replay() {
 	name << level_["name"] << " Turn " << current_turn();
 
 	std::stringstream replay_data;
-	replay_data << "campaign_type=\"multiplayer\"\n"
-	<< "difficulty=\"NORMAL\"\n"
-	<< "label=\"" << name.str() << "\"\n"
-	<< "mp_game_title=\"" << name_ << "\"\n"
-	<< "random_seed=\"" << level_["random_seed"] << "\"\n"
-	<< "version=\"" << level_["version"] << "\"\n"
-	<< "[replay]\n" << replay_commands << "[/replay]\n"
-	<< "[replay_start]\n" << level_.output() << "[/replay_start]\n";
-
-	name << " (" << id_ << ").gz";
-
-	std::string replay_data_str = replay_data.str();
 	try {
+		replay_data << "campaign_type=\"multiplayer\"\n"
+		<< "difficulty=\"NORMAL\"\n"
+		<< "label=\"" << name.str() << "\"\n"
+		<< "mp_game_title=\"" << name_ << "\"\n"
+		<< "random_seed=\"" << level_["random_seed"] << "\"\n"
+		<< "version=\"" << level_["version"] << "\"\n"
+		<< "[replay]\n" << replay_commands << "[/replay]\n"
+		<< "[replay_start]\n" << level_.output() << "[/replay_start]\n";
+
+		name << " (" << id_ << ").gz";
+
+		std::string replay_data_str = replay_data.str();
 		simple_wml::document replay(replay_data_str.c_str(), simple_wml::INIT_STATIC);
 
 		std::string filename(name.str());
@@ -1310,16 +1297,6 @@ void game::set_termination_reason(const std::string& reason) {
 	if (termination_.empty()) { termination_ = reason; }
 }
 
-void game::add_players(const game& other_game, const bool observer) {
-	user_vector users = other_game.all_game_users();
-	if (observer){
-		observers_.insert(observers_.end(), users.begin(), users.end());
-	}
-	else{
-		players_.insert(players_.end(), users.begin(), users.end());
-	}
-}
-
 const user_vector game::all_game_users() const {
 	user_vector res;
 
@@ -1333,7 +1310,7 @@ std::string game::debug_player_info() const {
 	std::stringstream result;
 	result << "game id: " << id_ << "\n";
 //	result << "players_.size: " << players_.size() << "\n";
-	for (user_vector::const_iterator p = players_.begin(); p != players_.end(); p++){
+	for (user_vector::const_iterator p = players_.begin(); p != players_.end(); ++p){
 		const player_map::const_iterator user = player_info_->find(*p);
 		if (user != player_info_->end()){
 			result << "player: " << user->second.name().c_str() << "\n";
@@ -1343,7 +1320,7 @@ std::string game::debug_player_info() const {
 		}
 	}
 //	result << "observers_.size: " << observers_.size() << "\n";
-	for (user_vector::const_iterator o = observers_.begin(); o != observers_.end(); o++){
+	for (user_vector::const_iterator o = observers_.begin(); o != observers_.end(); ++o){
 		const player_map::const_iterator user = player_info_->find(*o);
 		if (user != player_info_->end()){
 			result << "observer: " << user->second.name().c_str() << "\n";
@@ -1360,14 +1337,19 @@ std::string game::debug_player_info() const {
 	return result.str();
 }
 
-player_map::const_iterator game::find_user(const simple_wml::string_span& name) const {
-	player_map::const_iterator pl;
-	for (pl = player_info_->begin(); pl != player_info_->end(); pl++) {
+player_map::iterator game::find_user(const simple_wml::string_span& name)
+{
+	player_map::iterator pl;
+	for (pl = player_info_->begin(); pl != player_info_->end(); ++pl) {
 		if (name == pl->second.name().c_str()) {
-			return pl;
+			break;
 		}
 	}
-	return player_info_->end();
+	return pl;
+}
+
+player_map::const_iterator game::find_user(const simple_wml::string_span& name) const {
+	return wesnothd::find_user(*player_info_, name);
 }
 
 void game::send_and_record_server_message(const char* message,
@@ -1407,7 +1389,7 @@ void game::send_server_message(const char* message, network::connection sock, si
 	}
 
 	if(sock) {
-		send_to_one(doc, sock, "message");
+		wesnothd::send_to_one(doc, sock, "message");
 	}
 }
-}
+} // namespace wesnothd
