@@ -1,6 +1,6 @@
-/* $Id: game.hpp 33094 2009-02-25 04:07:28Z soliton $ */
+/* $Id: game.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -34,7 +34,9 @@ typedef std::vector<network::connection> side_vector;
 class game
 {
 public:
-	game(player_map& players, const network::connection host=0, const std::string name="");
+	game(player_map& players, const network::connection host=0,
+			const std::string& name="", bool save_replays=false,
+			const std::string& replay_save_path="");
 	~game();
 
 	int id() const { return id_; }
@@ -64,6 +66,8 @@ public:
 	 */
 	void mute_observer(const simple_wml::node& mute, const player_map::const_iterator muter);
 
+	void unmute_observer(const simple_wml::node& unmute, const player_map::const_iterator unmuter);
+
 	/**
 	 * Kick a member by name.
 	 *
@@ -82,12 +86,14 @@ public:
 	 */
 	network::connection ban_user(const simple_wml::node& ban, const player_map::const_iterator banner);
 
+	void unban_user(const simple_wml::node& unban, const player_map::const_iterator unbanner);
+
 	/**
 	 * Add a user to the game.
 	 *
 	 * @return                    True iff the user successfully joined the game.
 	 */
-	bool add_player(const network::connection player, bool observer = false, bool admin = false);
+	bool add_player(const network::connection player, bool observer = false);
 
 	/**
 	 * Removes a user from the game.
@@ -96,19 +102,11 @@ public:
 	 *                            no more players or the host left on a not yet
 	 *                            started game.
 	 */
-	bool remove_player(const network::connection player, const bool disconnect=false);
+	bool remove_player(const network::connection player, const bool disconnect=false, const bool destruct=false);
 
 	/** Adds players and observers into one vector and returns that. */
 	const user_vector all_game_users() const;
 
-	/**
-	 * Adds players from one game to another. This is used to add players and
-	 * observers from a game to the lobby (which is also implemented as a game),
-	 * if that game ends. The second parameter controls, wether the players are
-	 * added to the players_ or observers_ vector (default observers_).
-	 */
-	void add_players(const game& other_game, const bool observer = true);
-
 	void start_game(const player_map::const_iterator starter);
 
 	/** A user (player only?) asks for the next scenario to advance to. */
@@ -147,9 +145,10 @@ public:
 	void send_and_record_server_message(const char* message,
 			const network::connection exclude=0);
 	void send_data(simple_wml::document& data, const network::connection exclude=0, std::string packet_type = "") const;
-	void send_to_one(simple_wml::document& data, const network::connection sock, std::string packet_type = "") const;
 
+	void clear_history();
 	void record_data(simple_wml::document* data);
+	void save_replay();
 
 	/** The full scenario data. */
 	simple_wml::document& level() { return level_; }
@@ -186,7 +185,9 @@ private:
 	{ return (current_player() == player); }
 	bool is_muted_observer(const network::connection player) const;
 	bool all_observers_muted() const { return all_observers_muted_; }
+	void send_muted_observers(const player_map::const_iterator user) const;
 
+	bool send_taken_side(simple_wml::document& cfg, const simple_wml::node::child_list::const_iterator side) const;
 	/**
 	 * Figures out which side to take and tells that side to the game owner.
 	 *
@@ -203,10 +204,12 @@ private:
 	 * or controller type (human or ai).
 	 */
 	void change_controller(const size_t side_num,
-			const player_map::const_iterator newplayer,
+			const network::connection sock,
+			const std::string& player_name,
 			const bool player_left = true,
-			const std::string controller = "");
-	void transfer_ai_sides();
+			const std::string& controller = "");
+	void transfer_ai_sides(const network::connection player);
+	void send_leave_game(network::connection user) const;
 	void send_data_team(simple_wml::document& data, const simple_wml::string_span& team,
 			const network::connection exclude=0, std::string packet_type = "") const;
 	void send_data_observers(simple_wml::document& data, const network::connection exclude=0, std::string packet_type = "") const;
@@ -222,14 +225,18 @@ private:
 	/** In case of a host transfer, notify the new host about its status. */
 	void notify_new_host();
 
-	/** Convenience function for finding a user by name. */
+	/** Shortcut to a convenience function for finding a user by name. */
+	player_map::iterator find_user(const simple_wml::string_span& name);
 	player_map::const_iterator find_user(const simple_wml::string_span& name) const;
 
 	bool observers_can_label() const { return false; }
 	bool observers_can_chat() const { return true; }
 	bool is_legal_command(const simple_wml::node& command, bool is_player);
 
-	/** Function which returns true iff 'player' is on 'team'. */
+	/**
+	 * Function which returns true iff 'player' is on 'team'.
+	 * AI controlled sides are not considered on a team.
+	 */
 	bool is_on_team(const simple_wml::string_span& team, const network::connection player) const;
 	/**
 	 * Checks whether a user has the same IP as members of this game.
@@ -253,6 +260,15 @@ private:
 	 */
 	void send_user_list(const network::connection exclude=0) const;
 
+	/** Returns the name of the user or "(unfound)". */
+	std::string username(const player_map::const_iterator pl) const;
+
+	/** Returns a comma seperated list of user names. */
+	std::string list_users(user_vector users, const std::string& func) const;
+
+	/** Function to log when we don't find a connection in player_info_. */
+	void missing_user(network::connection socket, const std::string& func) const;
+
 	/** Helps debugging player and observer lists. */
 	std::string debug_player_info() const;
 
@@ -306,6 +322,17 @@ private:
 	std::vector<std::string> bans_;
 
 	std::string termination_;
+
+	bool save_replays_;
+	std::string replay_save_path_;
+};
+
+struct game_is_member {
+	game_is_member(network::connection sock) : sock_(sock) {};
+	bool operator()(const game* g) const { return g->is_owner(sock_) || g->is_member(sock_); }
+
+private:
+	network::connection sock_;
 };
 
 struct game_id_matches {
