@@ -1,6 +1,6 @@
-/* $Id: input_stream.cpp 26477 2008-05-09 08:58:47Z suokko $ */
+/* $Id: input_stream.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -24,25 +24,29 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <cerrno>
 
 #endif
 
-input_stream::input_stream(const std::string& path) : fd_(-1), path_(path)
+input_stream::input_stream(const std::string& path) :
+	fd_(-1),
+	path_(path),
+	data_()
 {
 #ifndef _WIN32
 	if(path == "") {
 		return;
 	}
 
-	const int res = mkfifo(path.c_str(),0600);
+	const int res = mkfifo(path.c_str(),0660);
 	if(res != 0) {
-		std::cerr << "could not make fifo at '" << path << "'\n";
+		std::cerr << "could not make fifo at '" << path << "' (" << errno << ")\n";
 	}
 
 	fd_ = open(path.c_str(),O_RDONLY|O_NONBLOCK);
 
 	if(fd_ == -1) {
-		std::cerr << "failed to open fifo at '" << path << "'\n";
+		std::cerr << "failed to open fifo at '" << path << "' (" << errno << ")\n";
 	} else {
 		std::cerr << "opened fifo at '" << path << "'. Server commands may be written to this file.\n";
 	}
