@@ -1,6 +1,6 @@
-/* $Id: player.cpp 37530 2009-08-07 00:22:31Z soliton $ */
+/* $Id: player.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,8 +15,10 @@
 #include "../global.hpp"
 
 #include "player.hpp"
+#include "serialization/string_utils.hpp"
+#include "util.hpp"
 
-player::player(const std::string& n, simple_wml::node& cfg,
+wesnothd::player::player(const std::string& n, simple_wml::node& cfg,
                          bool registered, const size_t max_messages,
                          const size_t time_period, const bool sp,
                          const bool moderator)
@@ -28,6 +30,7 @@ player::player(const std::string& n, simple_wml::node& cfg,
   , messages_since_flood_start_(0)
   , MaxMessages(max_messages)
   , TimePeriod(time_period)
+  , status_(LOBBY)
   , moderator_(moderator)
 {
 	cfg_.set_attr_dup("name", n.c_str());
@@ -35,20 +38,48 @@ player::player(const std::string& n, simple_wml::node& cfg,
 	mark_available();
 }
 
-// keep 'available' and game name ('location') for backward compatibility
-void player::mark_available(const int game_id, const std::string location)
+
+void wesnothd::player::set_status(wesnothd::player::STATUS status)
 {
-	cfg_.set_attr("available", (game_id == 0) ? "yes" : "no");
+	status_ = status;
+	switch (status)
+	{
+		case wesnothd::player::LOBBY:
+			cfg_.set_attr("status", "lobby");
+			break;
+		case wesnothd::player::PLAYING:
+			cfg_.set_attr("status", "playing");
+			break;
+		case wesnothd::player::OBSERVING:
+			cfg_.set_attr("status", "observing");
+			break;
+		default:
+			cfg_.set_attr("status", "unknown");
+	}
+}
+
+// keep 'available' and game name ('location') for backward compatibility
+void wesnothd::player::mark_available(const int game_id,
+                                      const std::string& location)
+{
+	if (game_id == 0) {
+		cfg_.set_attr("available", "yes");
+		set_status(LOBBY);
+	} else {
+		cfg_.set_attr("available", "no");
+	}
 	cfg_.set_attr_dup("game_id", lexical_cast<std::string>(game_id).c_str());
 	cfg_.set_attr_dup("location", location.c_str());
 }
 
-void player::mark_registered(bool registered) {
+void wesnothd::player::mark_registered(bool registered)
+{
     cfg_.set_attr("registered", registered ? "yes" : "no");
     registered_ = registered;
 }
 
-bool player::is_message_flooding() {
+bool wesnothd::player::is_message_flooding()
+{
 	const time_t now = time(NULL);
 	if (flood_start_ == 0) {
 		flood_start_ = now;
@@ -63,6 +94,5 @@ bool player::is_message_flooding() {
 	} else if (messages_since_flood_start_ == MaxMessages) {
 		return true;
 	}
-
 	return false;
 }
