@@ -8,80 +8,114 @@
 
 #include "../log.hpp"
 
-#define ERR_SWML LOG_STREAM(err, config)
+static lg::log_domain log_config("config");
+#define ERR_SWML LOG_STREAM(err, log_config)
 
 namespace simple_wml {
 
 namespace {
 
 void debug_delete(node* n) {
-	if(n) {
-		delete n;
-	}
+	delete n;
 }
 
 char* uncompress_buffer(const string_span& input, string_span* span)
 {
-	std::istringstream stream(std::string(input.begin(), input.end()));
-	boost::iostreams::filtering_stream<boost::iostreams::input> filter;
-	filter.push(boost::iostreams::gzip_decompressor());
-	filter.push(stream);
+	int nalloc = input.size();
+	int state = 0;
+	try {
+		std::istringstream stream(std::string(input.begin(), input.end()));
+		state = 1;
+		boost::iostreams::filtering_stream<boost::iostreams::input> filter;
+		state = 2;
+		filter.push(boost::iostreams::gzip_decompressor());
+		filter.push(stream);
+		state = 3;
 
-	const size_t chunk_size = input.size() * 10;
-	std::vector<char> buf(chunk_size);
-	size_t len = 0;
-	size_t pos = 0;
-	while(filter.good() && (len = filter.read(&buf[pos], chunk_size).gcount()) == chunk_size) {
-		if(pos + chunk_size > 40000000) {
-			throw error("WML document exceeds 40MB limit");
+		const size_t chunk_size = input.size() * 10;
+		nalloc = chunk_size;
+		std::vector<char> buf(chunk_size);
+		state = 4;
+		size_t len = 0;
+		size_t pos = 0;
+		while(filter.good() && (len = filter.read(&buf[pos], chunk_size).gcount()) == chunk_size) {
+			if(pos + chunk_size > 40000000) {
+				throw error("WML document exceeds 40MB limit");
+			}
+
+			pos += len;
+			buf.resize(pos + chunk_size);
+			len = 0;
+		}
+
+		if(!filter.eof() && !filter.good()) {
+			throw error("failed to uncompress");
 		}
 
 		pos += len;
-		buf.resize(pos + chunk_size);
-		len = 0;
+		state = 5;
+		nalloc = pos;
+
+		buf.resize(pos);
+		state = 6;
+
+		char* small_out = new char[pos+1];
+		memcpy(small_out, &buf[0], pos);
+		state = 7;
+
+		small_out[pos] = 0;
+
+		*span = string_span(small_out, pos);
+		state = 8;
+		return small_out;
+	} catch (std::bad_alloc& e) {
+		ERR_SWML << "ERROR: bad_alloc caught in uncompress_buffer() state "
+		<< state << " alloc bytes " << nalloc << " with input: '"
+		<< input << "' " << e.what() << std::endl;
+		throw error("Bad allocation request in uncompress_buffer().");
 	}
-
-	if(!filter.eof() && !filter.good()) {
-		throw error("failed to uncompress");
-	}
-
-	pos += len;
-	buf.resize(pos);
-
-	char* small_out = new char[pos+1];
-	memcpy(small_out, &buf[0], pos);
-	small_out[pos] = 0;
-
-	*span = string_span(small_out, pos);
-	return small_out;
 }
 
 char* compress_buffer(const char* input, string_span* span)
 {
+	int nalloc = strlen(input);
+	int state = 0;
 	try {
 		std::string in(input);
+		state = 1;
 		std::istringstream stream(in);
+		state = 2;
 		boost::iostreams::filtering_stream<boost::iostreams::input> filter;
+		state = 3;
 		filter.push(boost::iostreams::gzip_compressor());
 		filter.push(stream);
+		state = 4;
+		nalloc = in.size()*2 + 80;
 
 		std::vector<char> buf(in.size()*2 + 80);
+		state = 5;
 		const int len = filter.read(&buf[0], buf.size()).gcount();
 		assert(len < 128*1024*1024);
 		if((!filter.eof() && !filter.good()) || len == static_cast<int>(buf.size())) {
 			throw error("failed to compress");
 		}
+		state = 6;
+		nalloc = len;
 
 		buf.resize(len);
+		state = 7;
 
 		char* small_out = new char[len];
 		memcpy(small_out, &buf[0], len);
+		state = 8;
 
 		*span = string_span(small_out, len);
 		assert(*small_out == 31);
+		state = 9;
 		return small_out;
 	} catch (std::bad_alloc& e) {
-		ERR_SWML << "ERROR: bad_alloc caught in compress_buffer() with input: '"
+		ERR_SWML << "ERROR: bad_alloc caught in compress_buffer() state "
+		<< state << " alloc bytes " << nalloc << " with input: '"
 		<< input << "' " << e.what() << std::endl;
 		throw error("Bad allocation request in compress_buffer().");
 	}
@@ -95,7 +129,10 @@ bool string_span::to_bool(bool default_value) const
 		return default_value;
 	}
 
-	return operator==("yes") || operator==("on") || operator==("true") || operator==("1");
+	if (operator==("no") || operator==("off") || operator==("false") || operator==("0") || operator==("0.0"))
+		return false;
+
+	return true;
 }
 
 int string_span::to_int() const
@@ -654,11 +691,13 @@ void node::shift_buffers(ptrdiff_t offset)
 	}
 }
 
-void node::output(char*& buf)
+void node::output(char*& buf, CACHE_STATUS cache_status)
 {
 	if(output_cache_.empty() == false) {
 		memcpy(buf, output_cache_.begin(), output_cache_.size());
-		shift_buffers(buf - output_cache_.begin());
+		if(cache_status == REFRESH_CACHE) {
+			shift_buffers(buf - output_cache_.begin());
+		}
 		buf += output_cache_.size();
 		return;
 	}
@@ -689,7 +728,7 @@ void node::output(char*& buf)
 		buf += attr.size();
 		*buf++ = ']';
 		*buf++ = '\n';
-		children_[i->child_map_index].second[i->child_list_index]->output(buf);
+		children_[i->child_map_index].second[i->child_list_index]->output(buf, cache_status);
 		*buf++ = '[';
 		*buf++ = '/';
 		memcpy(buf, attr.begin(), attr.size());
@@ -698,20 +737,21 @@ void node::output(char*& buf)
 		*buf++ = '\n';
 	}
 
-	output_cache_ = string_span(begin, buf - begin);
+	if(cache_status == REFRESH_CACHE) {
+		output_cache_ = string_span(begin, buf - begin);
+	}
 }
-const char* node::output() {
-	const char* out;
 
-	const int buf_size = output_size() + 1;
-	char* buf = new char[buf_size];
-	out = buf;
-
-	output(buf);
-	*buf++ = 0;
-	assert(buf == out + buf_size);
-
-	return out;
+std::string node_to_string(const node& n)
+{
+	//calling output with status=DO_NOT_MODIFY_CACHE really doesn't modify the
+	//node, so we can do it safely
+	node& mutable_node = const_cast<node&>(n);
+	std::vector<char> v(mutable_node.output_size());
+	char* ptr = &v[0];
+	mutable_node.output(ptr, node::DO_NOT_MODIFY_CACHE);
+	assert(ptr == &v[0] + v.size());
+	return std::string(v.begin(), v.end());
 }
 
 void node::copy_into(node& n) const
@@ -967,7 +1007,7 @@ const char* document::output()
 	buffers_.push_back(buf);
 	output_ = buf;
 
-	root_->output(buf);
+	root_->output(buf, node::REFRESH_CACHE);
 	*buf++ = 0;
 	assert(buf == output_ + buf_size);
 
@@ -1099,7 +1139,6 @@ std::string document::stats()
 	int text_size = 0;
 	int nbuffers = 0;
 	int nnodes = 0;
-	int nhas_nodes = 0;
 	int ndirty = 0;
 	int nattributes = 0;
 	for(document* d = head_doc; d != NULL; d = d->next_) {
@@ -1117,7 +1156,6 @@ std::string document::stats()
 		}
 
 		if(d->root_) {
-			++nhas_nodes;
 			nnodes += 1 + d->root_->nchildren();
 			nattributes += d->root_->nattributes_recursive();
 		}
