@@ -1,6 +1,6 @@
-/* $Id: user_handler.cpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: user_handler.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2008 - 2009 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
+   Copyright (C) 2008 - 2010 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,10 +13,13 @@
 */
 
 #include "user_handler.hpp"
+#include "../config.hpp"
+#include "serialization/string_utils.hpp"
 
+#include <ctime>
 
 bool user_handler::send_mail(const std::string& to_user,
-		const std::string& subject, const std::string& message) {
+		const std::string& /*subject*/, const std::string& /*message*/) {
 
 	//If this user is registerd at all
 	if(!user_exists(to_user)) {
@@ -28,27 +31,14 @@ bool user_handler::send_mail(const std::string& to_user,
 		throw error("Could not send email. The email address of the user '" + to_user + "' is empty.");
 	}
 
-	if(!mailer_) {
-		throw user_handler::error("This server is configured not to send email.");
-	}
-
-	if(!(mailer_->send_mail(get_mail(to_user), subject, message))) {
-		throw user_handler::error("There was an error sending the email.");
-	}
-
-	return true;
+	throw user_handler::error("This server is configured not to send email.");
 }
 
-void user_handler::init_mailer(config* c) {
-	if(c) {
-		mailer_ = new mailer(*c);
-	} else {
-		mailer_ = NULL;
-	}
+void user_handler::init_mailer(const config &) {
 }
 
 std::string user_handler::create_salt(int length) {
-	srand((unsigned)time(NULL));
+	srand(static_cast<unsigned>(time(NULL)));
 
 	std::stringstream ss;
 
