@@ -1,6 +1,6 @@
-/* $Id: show_dialog.cpp 32970 2009-02-21 15:48:02Z alink $ */
+/* $Id: show_dialog.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -18,15 +18,15 @@
 
 #include "display.hpp"
 #include "gettext.hpp"
+#include "gui/auxiliary/event/handler.hpp"
 #include "help.hpp"
 #include "log.hpp"
 #include "marked-up_text.hpp"
 
 
-
-#define ERR_DP LOG_STREAM(err, display)
-#define LOG_DP LOG_STREAM(info, display)
-#define ERR_G  LOG_STREAM(err, general)
+static lg::log_domain log_display("display");
+#define ERR_DP LOG_STREAM(err, log_display)
+#define ERR_G  LOG_STREAM(err, lg::general)
 
 namespace {
 bool is_in_dialog = false;
@@ -49,7 +49,10 @@ const int dialog_frame::title_border_h = 5;
 
 
 
-bool in_dialog() { return is_in_dialog; }
+bool in_dialog()
+{
+	return is_in_dialog || gui2::is_in_dialog();
+}
 
 dialog_manager::dialog_manager() : cursor::setter(cursor::NORMAL), reset_to(is_in_dialog)
 {
@@ -351,12 +354,6 @@ private:
 namespace gui
 {
 
-void show_error_message(display &disp, std::string const &message)
-{
-	ERR_G << message << std::endl;
-	dialog(disp, _("Error"), message, OK_ONLY).show();
-}
-
 int show_dialog(display& screen, surface image,
 				const std::string& caption, const std::string& message,
 				DIALOG_TYPE type,
@@ -373,7 +370,8 @@ int show_dialog(display& screen, surface image,
 				const menu::sorter* sorter,
 				menu::style* menu_style)
 {
-	const std::string& title = (image.null())? caption : "";
+	std::string title;
+	if (image.null()) title = caption;
 	const dialog::style& style = (dialog_style)? *dialog_style : dialog::default_style;
 	CVideo &disp = screen.video();
 
