@@ -1,6 +1,6 @@
-/* $Id: sound.hpp 32112 2009-01-13 10:26:54Z ilor $ */
+/* $Id: sound.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -40,16 +40,14 @@ void stop_UI_sound();
 void stop_bell();
 
 // Read config entry, alter track list accordingly.
-void play_music_config(const config &music);
+void play_music_config(const config &music_node);
 // Act on any track list changes from above.
 void commit_music_changes();
 
 // Play this particular music file over and over and over.
-void play_music_repeatedly(const std::string &name);
+void play_music_repeatedly(const std::string& id);
 // Play this particular music file once, then silence.
-void play_music_once(const std::string &name);
-// Empty the playlist (and stop playing)
-void play_no_music();
+void play_music_once(const std::string& id);
 // Empty the playlist
 void empty_playlist();
 // Start playing current music.
