@@ -1,6 +1,6 @@
-/* $Id: soundsource.cpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: soundsource.cpp 41650 2010-03-20 21:53:36Z ai0867 $ */
 /*
-   Copyright (C) 2006 - 2009 by Karol Nowak <grzywacz@sul.uni.lodz.pl>
+   Copyright (C) 2006 - 2010 by Karol Nowak <grzywacz@sul.uni.lodz.pl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
 #include "display.hpp"
 #include "foreach.hpp"
 #include "log.hpp"
-#include "pathutils.hpp"
+#include "serialization/string_utils.hpp"
 #include "sound.hpp"
 #include "soundsource.hpp"
 
@@ -32,6 +32,8 @@ const unsigned DEFAULT_FADE_RANGE       = 14;
 unsigned int positional_source::last_id = 0;
 
 manager::manager(const display &disp) :
+	observer(),
+	savegame_config(),
 	sources_(),
 	disp_(disp)
 {
@@ -96,16 +98,6 @@ void manager::update_positions()
 	}
 }
 
-void manager::add_location(const std::string &id, const map_location &loc)
-{
-	positional_source_iterator it = sources_.find(id);
-
-	if(it == sources_.end())
-		return;
-	else
-		(*it).second->add_location(loc);
-}
-
 void manager::write_sourcespecs(config& cfg) const
 {
 	for(positional_source_const_iterator i = sources_.begin(); i != sources_.end(); ++i) {
@@ -117,6 +109,13 @@ void manager::write_sourcespecs(config& cfg) const
 	}
 }
 
+config manager::to_config() const
+{
+	config cfg;
+	write_sourcespecs(cfg);
+	return cfg.child("sound_source");
+}
+
 positional_source::positional_source(const sourcespec &spec) :
 	last_played_(0),
 	min_delay_(spec.minimum_delay()),
@@ -126,6 +125,7 @@ positional_source::positional_source(const sourcespec &spec) :
 	range_(spec.full_range()),
 	faderange_(spec.fade_range()),
 	check_fogged_(spec.check_fogged()),
+	check_shrouded_(spec.check_shrouded()),
 	files_(spec.files()),
 	locations_(spec.get_locations())
 {
@@ -138,6 +138,11 @@ positional_source::~positional_source()
 	sound::reposition_sound(id_, DISTANCE_SILENT);
 }
 
+bool positional_source::is_global()
+{
+	return locations_.empty();
+}
+
 void positional_source::update(unsigned int time, const display &disp)
 {
 	if(time - last_played_ < min_delay_ || sound::is_sound_playing(id_))
@@ -150,7 +155,7 @@ void positional_source::update(unsigned int time, const display &disp)
 
 		// If no locations have been specified, treat the source as if
 		// it was present everywhere on the map
-		if(locations_.size() == 0) {
+		if(locations_.empty()) {
 			sound::play_sound_positioned(files_, id_, loops_, 0);	// max volume
 			return;
 		}
@@ -172,11 +177,12 @@ void positional_source::update(unsigned int time, const display &disp)
 
 void positional_source::update_positions(unsigned int time, const display &disp)
 {
+	if(is_global()) {
+		return;
+	}
+
 	int distance_volume = DISTANCE_SILENT;
 	for(std::vector<map_location>::iterator i = locations_.begin(); i != locations_.end(); ++i) {
-		if(disp.shrouded(*i) || (check_fogged_ && disp.fogged(*i)))
-			continue;
-
 		int v = calculate_volume(*i, disp);
 		if(v < distance_volume) {
 			distance_volume = v;
@@ -195,6 +201,9 @@ int positional_source::calculate_volume(const map_location &loc, const display &
 	assert(range_ > 0);
 	assert(faderange_ > 0);
 
+	if((check_shrouded_ && disp.shrouded(loc)) || (check_fogged_ && disp.fogged(loc)))
+		return DISTANCE_SILENT;
+
 	SDL_Rect area = disp.map_area();
 	map_location center = disp.hex_clicked_on(area.x + area.w / 2, area.y + area.h / 2);
 	size_t distance = distance_between(loc, center);
@@ -203,12 +212,8 @@ int positional_source::calculate_volume(const map_location &loc, const display &
 		return 0;
 	}
 
-	return static_cast<int>(( ( (distance - range_) / (double) faderange_) * DISTANCE_SILENT));
-}
-
-void positional_source::add_location(const map_location &loc)
-{
-	locations_.push_back(loc);
+	return static_cast<int>((((distance - range_)
+			/ static_cast<double>(faderange_)) * DISTANCE_SILENT));
 }
 
 void positional_source::write_config(config& cfg) const
@@ -217,6 +222,7 @@ void positional_source::write_config(config& cfg) const
 	cfg["delay"] = str_cast<unsigned int>(this->min_delay_);
 	cfg["chance"] = str_cast<unsigned int>(this->chance_);
 	cfg["check_fogged"] = this->check_fogged_ ? "yes" : "no";
+	cfg["check_shrouded"] = this->check_shrouded_ ? "yes" : "no";
 
 	cfg["x"] = cfg["y"] = "";
 	bool first_loc = true;
@@ -245,6 +251,7 @@ sourcespec::sourcespec(const config& cfg) :
 	range_(lexical_cast_default<int>(cfg["full_range"], 3)),
 	faderange_(lexical_cast_default<int>(cfg["fade_range"], 14)),
 	check_fogged_(utils::string_bool(cfg["check_fogged"], true)),
+	check_shrouded_(utils::string_bool(cfg["check_shrouded"], true)),
 	locations_()
 {
 	const std::vector<std::string>& vx = utils::split(cfg["x"]);
@@ -259,8 +266,12 @@ sourcespec::sourcespec(const config& cfg) :
 	}
 	else {
 		for(unsigned int i = 0; i < std::min(vx.size(), vy.size()); ++i) {
-			map_location loc(lexical_cast<int>(vx[i]), lexical_cast<int>(vy[i]));
-			locations_.push_back(loc);
+			try {
+				map_location loc(lexical_cast<int>(vx[i]), lexical_cast<int>(vy[i]));
+				locations_.push_back(loc);
+			} catch(bad_lexical_cast&) {
+				lg::wml_error << "non-numerical coordinates for soundsource (" << vx[i] << ',' << vy[i] << ')';
+			}
 		}
 	}
 }
