@@ -1,6 +1,6 @@
-/* $Id: statistics.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: statistics.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,8 +12,7 @@
    See the COPYING file for more details.
 */
 
-//! @file statistics.hpp
-//!
+/** @file statistics.hpp */
 
 #ifndef STATISTICS_HPP_INCLUDED
 #define STATISTICS_HPP_INCLUDED
@@ -41,36 +40,33 @@ namespace statistics
 		str_int_map recruits, recalls, advanced_to, deaths, killed;
 		int recruit_cost, recall_cost;
 
-		//! A type that will map a string of hit/miss
-		//! to the number of times that sequence has occurred.
+		/*
+		 *  A type that will map a string of hit/miss to the number of times
+		 *  that sequence has occurred.
+		 */
 		typedef str_int_map battle_sequence_frequency_map;
 
-		//! A type that will map different % chances to hit to different results.
+		/** A type that will map different % chances to hit to different results. */
 		typedef std::map<int,battle_sequence_frequency_map> battle_result_map;
 
 		battle_result_map attacks, defends;
 
-		int damage_inflicted, damage_taken;
-		int turn_damage_inflicted, turn_damage_taken;
+		long long damage_inflicted, damage_taken;
+		long long turn_damage_inflicted, turn_damage_taken;
 
-		// Expected value for damage inflicted/taken * 100, based on
+		static const int decimal_shift = 1000;
+
+		// Expected value for damage inflicted/taken * 1000, based on
 		// probability to hit,
 		// Use this long term to see how lucky a side is.
-		//! @todo FIXME: Since integers are used, rounding errors accumulate.
-		// Also, slow isn't accounted for properly.
-		// Rusty's simulator could be used obtain valid values.
-		int expected_damage_inflicted, expected_damage_taken;
-		int turn_expected_damage_inflicted, turn_expected_damage_taken;
+
+		long long expected_damage_inflicted, expected_damage_taken;
+		long long turn_expected_damage_inflicted, turn_expected_damage_taken;
+		std::string save_id;
 	};
 
 	int sum_str_int_map(const stats::str_int_map& m);
-
-	struct disabler
-	{
-		disabler();
-		~disabler();
-	};
-
+	int sum_cost_str_int_map(const stats::str_int_map &m);
 
 	struct scenario_context
 	{
@@ -83,15 +79,16 @@ namespace statistics
 		attack_context(const unit& a, const unit& d, int a_cth, int d_cth);
 		~attack_context();
 
-		enum ATTACK_RESULT { MISSES, HITS, KILLS };
+		enum hit_result { MISSES, HITS, KILLS };
 
-		void attack_result(ATTACK_RESULT res, int damage);
-		void defend_result(ATTACK_RESULT res, int damage);
+		void attack_expected_damage(double attacker_inflict, double defender_inflict);
+		void attack_result(hit_result res, int damage, int drain);
+		void defend_result(hit_result res, int damage, int drain);
 
 	private:
 
 		std::string attacker_type, defender_type;
-		int attacker_side, defender_side;
+		std::string attacker_side, defender_side;
 		int chance_to_hit_defender, chance_to_hit_attacker;
 		std::string attacker_res, defender_res;
 
@@ -112,8 +109,8 @@ namespace statistics
 	void fresh_stats();
 	void clear_current_scenario();
 
-	void reset_turn_stats(int side);
-	stats calculate_stats(int category, int side);
+	void reset_turn_stats(std::string save_id);
+	stats calculate_stats(int category, std::string save_id);
 } // end namespace statistics
 
 #endif
