@@ -1,6 +1,6 @@
-/* $Id: team.cpp 33693 2009-03-15 17:46:14Z jhinrichs $ */
+/* $Id: team.cpp 41650 2010-03-20 21:53:36Z ai0867 $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -19,90 +19,41 @@
 
 #include "global.hpp"
 
+#include "ai/manager.hpp"
 #include "foreach.hpp"
 #include "game_events.hpp"
 #include "gamestatus.hpp"
 #include "log.hpp"
 #include "map.hpp"
+#include "resources.hpp"
+#include "team.hpp"
+
+static lg::log_domain log_engine("engine");
+#define DBG_NG LOG_STREAM(debug, log_engine)
+#define LOG_NG LOG_STREAM(info, log_engine)
+#define WRN_NG LOG_STREAM(warn, log_engine)
 
 
-#define LOG_NG LOG_STREAM(info, engine)
-#define WRN_NG LOG_STREAM(warn, engine)
-
-//some default values for the AI parameters following the default values listed
-//in the wiki at http://www.wesnoth.org/wiki/AiWML
-
-//A number 0 or higher which determines how many scouts the AI recruits. If 0,
-//the AI doesn't recruit scouts to capture villages.
-const int ai_default_villages_per_scout = 4;
-//A number 0 or higher which determines how much the AI targets enemy leaders.
-const double ai_default_leader_value = 3.0;
-//A number 0 or higher which determines how much the AI tries to capture villages.
-const double ai_default_village_value = 1.0;
-//details see in the [AI] tag explaination in the wiki:
-//http://www.wesnoth.org/wiki/AiWML#the_.5Bai.5D_tag
-const double ai_default_aggression = 0.5;
-//details see in the [AI] tag explaination in the wiki:
-//http://www.wesnoth.org/wiki/AiWML#the_.5Bai.5D_tag
-const double ai_default_caution = 0.25;
-
-namespace {
-	std::vector<team>* teams = NULL;
-}
+static std::vector<team> *&teams = resources::teams;
 
 //static member initialization
 const int team::default_team_gold = 100;
 
-teams_manager::teams_manager(std::vector<team>& teams_list)
-{
-	assert(!teams);
-	teams = &teams_list;
-}
-
-teams_manager::~teams_manager()
-{
-	teams = NULL;
-}
-
 const std::vector<team>& teams_manager::get_teams()
 {
 	assert(teams);
 	return *teams;
 }
 
-bool teams_manager::is_observer()
-{
-	for(std::vector<team>::const_iterator i = teams->begin(); i != teams->end(); ++i) {
-		if(i->is_local()) {
-			return false;
-		}
-	}
-
-	return true;
-}
-
-team::target::target(const config& cfg)
-              : criteria(cfg), value(atof(cfg["value"].c_str()))
-{
-}
-
-void team::target::write(config& cfg) const
-{
-	cfg = criteria;
-}
-
 team::team_info::team_info(const config& cfg) :
 		name(cfg["name"]),
-		gold(cfg["gold"]),
-		start_gold(),
-		income(cfg["income"]),
-		income_per_village(),
+		gold(lexical_cast_default<int>(cfg["gold"])),
+		start_gold(0),
+		gold_add(false),
+		income(lexical_cast_default<int>(cfg["income"])),
+		income_per_village(0),
 		average_price(0),
-		number_of_possible_recruits_to_force_recruit(lexical_cast_default<float>(cfg["number_of_possible_recruits_to_force_recruit"],0.0)),
 		can_recruit(),
-		global_recruitment_pattern(utils::split(cfg["global_recruitment_pattern"])),
-		recruitment_pattern(utils::split(cfg["recruitment_pattern"])),
-		enemies(),
 		team_name(cfg["team_name"]),
 		user_team_name(cfg["user_team_name"]),
 		save_id(cfg["save_id"]),
@@ -115,15 +66,6 @@ team::team_info::team_info(const config& cfg) :
 		objectives(cfg["objectives"]),
 		objectives_changed(utils::string_bool(cfg["objectives_changed"])),
 		controller(),
-		ai_algorithm(cfg["ai_algorithm"]),
-		ai_params(),
-		ai_memory_(),
-		villages_per_scout(ai_default_villages_per_scout),
-		leader_value(ai_default_leader_value),
-		village_value(ai_default_village_value),
-		aggression_(ai_default_aggression),
-		caution_(ai_default_caution),
-		targets(),
 		share_maps(false),
 		share_view(false),
 		disallow_observers(utils::string_bool(cfg["disallow_observers"])),
@@ -131,34 +73,33 @@ team::team_info::team_info(const config& cfg) :
 		no_leader(utils::string_bool(cfg["no_leader"])),
 		hidden(utils::string_bool(cfg["hidden"])),
 		music(cfg["music"]),
-		colour(cfg["colour"].size() ? cfg["colour"] : cfg["side"])
+		colour(cfg["colour"].size() ? cfg["colour"] : cfg["side"]),
+		side(lexical_cast_default<int>(cfg["side"], 1)),
+		persistent(false)
 {
-	// If are starting new scenario overide settings from [ai] tags
+	// If arel starting new scenario overide settings from [ai] tags
 	if (!user_team_name.translatable())
 		user_team_name = user_team_name.from_serialized(user_team_name);
-	config global_ai_params;
-	const config::child_list& ai_parameters = cfg.get_children("ai");
-	for(config::child_list::const_iterator aiparam = ai_parameters.begin(); aiparam != ai_parameters.end(); ++aiparam) {
-		ai_params.push_back(**aiparam);
 
-		if((**aiparam)["turns"].empty() && (**aiparam)["time_of_day"].empty()) {
-			global_ai_params.append(**aiparam);
-		}
+	if(cfg.has_attribute("ai_config")) {
+		ai::manager::add_ai_for_side_from_file(side, cfg["ai_config"], true);
+	} else {
+		ai::manager::add_ai_for_side_from_config(side, cfg, true);
 	}
 
-	const config::child_list& ai_mems = cfg.get_children("ai_memory");
-	for(config::child_list::const_iterator aimem = ai_mems.begin(); aimem != ai_mems.end(); ++aimem) {
-	  ai_memory_.append(**aimem);
+	std::vector<std::string> recruits = utils::split(cfg["recruit"]);
+	for(std::vector<std::string>::const_iterator i = recruits.begin(); i != recruits.end(); ++i) {
+		can_recruit.insert(*i);
 	}
 
 	// at the start of a scenario "start_gold" is not set, we need to take the
 	// value from the gold setting (or fall back to the gold default)
 	if (!cfg["start_gold"].empty())
-		start_gold = cfg["start_gold"];
-	else if (!this->gold.empty())
-		start_gold = this->gold;
+		start_gold = lexical_cast_default<int>(cfg["start_gold"]);
+	else if (!cfg["gold"].empty())
+		start_gold = gold;
 	else
-		start_gold = str_cast(default_team_gold);
+		start_gold = default_team_gold;
 
 	if(team_name.empty()) {
 		team_name = cfg["side"];
@@ -171,8 +112,6 @@ team::team_info::team_info(const config& cfg) :
 		current_player = save_id;
 	}
 
-	int side = atoi(cfg["side"].c_str());
-
 	const std::string temp_rgb_str = cfg["team_rgb"];
 	std::map<std::string, color_range>::iterator global_rgb = game_config::team_rgb_range.find(cfg["side"]);
 
@@ -189,15 +128,9 @@ team::team_info::team_info(const config& cfg) :
 	else
 		income_per_village = lexical_cast_default<int>(village_income, game_config::village_income);
 
-	const std::string& enemies_list = cfg["enemy"];
-	if(!enemies_list.empty()) {
-		std::vector<std::string> venemies = utils::split(enemies_list);
-		for(std::vector<std::string>::const_iterator i = venemies.begin(); i != venemies.end(); ++i) {
-			enemies.push_back(atoi(i->c_str()));
-		}
-	}
-
 	std::string control = cfg["controller"];
+	//by default, persistence of a team is set depending on the controller
+	persistent = true;
 	if (control == "human")
 		controller = HUMAN;
 	else if (control == "human_ai")
@@ -210,122 +143,19 @@ team::team_info::team_info(const config& cfg) :
 	{
 		disallow_observers = utils::string_bool(cfg["disallow_observers"],true);
 		controller = EMPTY;
+		persistent = false;
 	}
 	else
+	{
 		controller = AI;
-
-	if(ai_algorithm.empty())
-	{
-		ai_algorithm = global_ai_params["ai_algorithm"];
+		persistent = false;
 	}
 
-	if (!cfg.has_attribute("number_of_possible_recruits_to_force_recruit"))
-	{
-		number_of_possible_recruits_to_force_recruit = lexical_cast_default<float>(global_ai_params["number_of_possible_recruits_to_force_recruit"],3.1);
-	}
+	//override persistence flag if it is explicitly defined in the config
+	persistent = utils::string_bool(cfg["persistent"], persistent);
 
-	std::string scouts_val = cfg["villages_per_scout"];
-
-	if(scouts_val.empty())
-	{
-		scouts_val = global_ai_params["villages_per_scout"];
-	}
-
-	if(scouts_val.empty()) {
-		villages_per_scout = ai_default_villages_per_scout;
-	} else {
-		villages_per_scout = atoi(scouts_val.c_str());
-	}
-
-	std::string leader_val = cfg["leader_value"];
-
-	if(leader_val.empty())
-	{
-		leader_val = global_ai_params["leader_value"];
-	}
-
-	if(leader_val.empty()) {
-		leader_value = ai_default_leader_value;
-	} else {
-		leader_value = atof(leader_val.c_str());
-	}
-
-	std::string village_val = cfg["village_value"];
-
-	if(village_val.empty())
-	{
-		village_val = global_ai_params["village_value"];
-	}
-
-	if(village_val.empty()) {
-		village_value = ai_default_village_value;
-	} else {
-		village_value = atof(village_val.c_str());
-	}
-
-	std::string aggression_val = cfg["aggression"];
-
-	if(aggression_val.empty())
-	{
-		aggression_val = global_ai_params["aggression"];
-	}
-
-	if(aggression_val.empty()) {
-		aggression_ = ai_default_aggression;
-	} else {
-		aggression_ = atof(aggression_val.c_str());
-	}
-
-	std::string caution_val = cfg["caution"];
-
-	if(caution_val.empty())
-	{
-		caution_val = global_ai_params["caution"];
-	}
-
-	if(caution_val.empty()) {
-		caution_ = ai_default_caution;
-	} else {
-		caution_ = atof(caution_val.c_str());
-	}
-
-
-	std::vector<std::string> recruits = utils::split(cfg["recruit"]);
-	for(std::vector<std::string>::const_iterator i = recruits.begin(); i != recruits.end(); ++i) {
-		can_recruit.insert(*i);
-	}
-
-	if(recruitment_pattern.empty())
-	{
-		recruitment_pattern =
-			utils::split(global_ai_params["recruitment_pattern"]);
-		LOG_NG << "Recruitment pattern: " << global_ai_params["recruitment_pattern"] << "\n";
-	} else {
-		LOG_NG << "Recruitment pattern from side: " << cfg["recruitment_pattern"] << "\n";
-	}
-
-
-/*	// Default recruitment pattern is to buy 2 fighters for every 1 archer
-	if(recruitment_pattern.empty()) {
-		recruitment_pattern.push_back("fighter");
-		recruitment_pattern.push_back("fighter");
-		recruitment_pattern.push_back("archer");
-	}
-*/
-	// Keep a copy of the initial recruitment_pattern,
-	// since it can be changed on a per-time-of-day
-	// or per-turn basis inside [ai] sections.
-	global_recruitment_pattern = recruitment_pattern;
-
-	// Additional targets
-	config::const_child_itors tgts;
-	for(tgts = cfg.child_range("target"); tgts.first != tgts.second; ++tgts.first) {
-		targets.push_back(target(**tgts.first));
-	}
-
-	for(tgts = global_ai_params.child_range("target"); tgts.first != tgts.second; ++tgts.first) {
-		targets.push_back(target(**tgts.first));
-	}
+	//========================================================
+	//END OF MESSY CODE
 
 	// Share_view and share_maps can't both be enabled,
 	// so share_view overrides share_maps.
@@ -338,15 +168,10 @@ team::team_info::team_info(const config& cfg) :
 
 void team::team_info::write(config& cfg) const
 {
-	for(std::vector<config>::const_iterator ai = ai_params.begin(); ai != ai_params.end(); ++ai) {
-		cfg.add_child("ai",*ai);
-	}
-	if(!ai_memory_.empty()) cfg.add_child("ai_memory",ai_memory_);
-	cfg["ai_algorithm"] = ai_algorithm;
-
-	cfg["gold"] = gold;
-	cfg["start_gold"] = start_gold;
-	cfg["income"] = income;
+	cfg["gold"] = str_cast(gold);
+	cfg["start_gold"] = str_cast(start_gold);
+	cfg["gold_add"] = gold_add ? "yes" : "no";
+	cfg["income"] = str_cast(income);
 	cfg["name"] = name;
 	cfg["team_name"] = team_name;
 	cfg["user_team_name"] = user_team_name;
@@ -364,16 +189,6 @@ void team::team_info::write(config& cfg) const
 	cfg["allow_player"] = allow_player ? "yes" : "no";
 	cfg["no_leader"] = no_leader ? "yes" : "no";
 	cfg["hidden"] = hidden ? "yes" : "no";
-	cfg["number_of_possible_recruits_to_force_recruit"] = lexical_cast<std::string>(number_of_possible_recruits_to_force_recruit);
-
-	std::stringstream enemies_str;
-	for(std::vector<int>::const_iterator en = enemies.begin(); en != enemies.end(); ++en) {
-		enemies_str << *en;
-		if(en+1 != enemies.end())
-			enemies_str << ",";
-	}
-
-	cfg["enemy"] = enemies_str.str();
 
 	switch(controller) {
 	case AI: cfg["controller"] = "ai"; break;
@@ -385,16 +200,6 @@ void team::team_info::write(config& cfg) const
 	default: assert(false); return;
 	}
 
-	cfg["villages_per_scout"] = str_cast(villages_per_scout);
-	cfg["leader_value"] = str_cast(leader_value);
-	cfg["village_value"] = str_cast(village_value);
-	cfg["aggression"] = str_cast(aggression_);
-	cfg["caution"] = str_cast(caution_);
-
-	for(std::vector<target>::const_iterator tg = targets.begin(); tg != targets.end(); ++tg) {
-		tg->write(cfg.add_child("target"));
-	}
-
 	std::stringstream can_recruit_str;
 	for(std::set<std::string>::const_iterator cr = can_recruit.begin(); cr != can_recruit.end(); ++cr) {
 		if(cr != can_recruit.begin())
@@ -405,28 +210,6 @@ void team::team_info::write(config& cfg) const
 
 	cfg["recruit"] = can_recruit_str.str();
 
-	std::stringstream global_recruit_pattern_str;
-	for(std::vector<std::string>::const_iterator p = global_recruitment_pattern.begin();
-		   	p != global_recruitment_pattern.end(); ++p) {
-		if(p != global_recruitment_pattern.begin())
-			global_recruit_pattern_str << ",";
-
-		global_recruit_pattern_str << *p;
-	}
-
-	cfg["global_recruitment_pattern"] = global_recruit_pattern_str.str();
-
-	std::stringstream recruit_pattern_str;
-	for(std::vector<std::string>::const_iterator p = recruitment_pattern.begin();
-		   	p != recruitment_pattern.end(); ++p) {
-		if(p != recruitment_pattern.begin())
-			recruit_pattern_str << ",";
-
-		recruit_pattern_str << *p;
-	}
-
-	cfg["recruitment_pattern"] = recruit_pattern_str.str();
-
 	cfg["share_maps"] = share_maps ? "yes" : "no";
 	cfg["share_view"] = share_view ? "yes" : "no";
 
@@ -434,6 +217,10 @@ void team::team_info::write(config& cfg) const
 		cfg["music"] = music;
 
 	cfg["colour"] = lexical_cast_default<std::string>(colour);
+
+	cfg["persistent"] = persistent ? "yes" : "no";
+
+	cfg.add_child("ai",ai::manager::to_config(side));
 }
 
 void team::merge_shroud_map_data(const std::string& shroud_data)
@@ -442,6 +229,7 @@ void team::merge_shroud_map_data(const std::string& shroud_data)
 }
 
 team::team(const config& cfg, const gamemap& map, int gold) :
+		savegame_config(),
 		gold_(gold),
 		villages_(),
 		shroud_(),
@@ -450,7 +238,7 @@ team::team(const config& cfg, const gamemap& map, int gold) :
 		info_(cfg),
 		countdown_time_(0),
 		action_bonus_count_(0),
-		aiparams_(),
+		recall_list_(),
 		enemies_(),
 		seen_(),
 		ally_shroud_(),
@@ -465,17 +253,17 @@ team::team(const config& cfg, const gamemap& map, int gold) :
 
 	// To ensure some mimimum starting gold,
 	// gold is the maximum of 'gold' and what is given in the config file
-	if(info_.gold.empty() == false)
-	{
-		gold_ = std::max(gold,::atoi(info_.gold.c_str()));
-		if (gold_ != ::atoi(info_.gold.c_str()))
-			info_.start_gold = str_cast(gold) + " (" + info_.start_gold + ")";
-	}
+	gold_ = std::max(gold, info_.gold);
+	if (gold_ != info_.gold)
+		info_.start_gold = gold;
+	// Old code was doing:
+	// info_.start_gold = str_cast(gold) + " (" + info_.start_gold + ")";
+	// Was it correct?
 
 	// Load in the villages the side controls at the start
-	const config::child_list& villages = cfg.get_children("village");
-	for(config::child_list::const_iterator v = villages.begin(); v != villages.end(); ++v) {
-		map_location loc(**v,game_events::get_state_of_game());
+	foreach (const config &v, cfg.child_range("village"))
+	{
+		map_location loc(v, resources::state_of_game);
 		if (map.is_village(loc)) {
 			villages_.insert(loc);
 		} else {
@@ -522,25 +310,26 @@ void team::remove_recruit(const std::string& recruit)
 {
 	info_.can_recruit.erase(recruit);
 	info_.average_price = 0;
+	ai::manager::raise_recruit_list_changed();
 }
 
 void team::set_recruits(const std::set<std::string>& recruits)
 {
 	info_.can_recruit = recruits;
 	info_.average_price = 0;
+	ai::manager::raise_recruit_list_changed();
 }
 
-void team::add_recruits(const std::set<std::string>& recruits)
+void team::add_recruit(const std::string &recruit)
 {
-	std::copy(recruits.begin(),recruits.end(),
-			std::inserter(info_.can_recruit, info_.can_recruit.end()));
+	info_.can_recruit.insert(recruit);
 	info_.average_price = 0;
+	ai::manager::raise_recruit_list_changed();
 }
 
-
 namespace {
 	struct count_average {
-		count_average(size_t& a) : a_(a), sum_(0), count_(0)
+		count_average(int a) : a_(a), sum_(0), count_(0)
 		{}
 		~count_average() {
 			// If no recruits disable leader from moving to keep
@@ -551,20 +340,19 @@ namespace {
 		}
 		void operator()(const std::string& recruit)
 		{
-			unit_type_data::unit_type_map::const_iterator i = unit_type_data::types().find_unit_type(recruit);
-			if (i == unit_type_data::types().end())
-				return;
+			const unit_type *ut = unit_types.find(recruit);
+			if (!ut) return;
 			++count_;
-			sum_ += i->second.cost();
+			sum_ += ut->cost();
 		}
 		private:
-		size_t& a_;
-		size_t sum_;
-		size_t count_;
+		int a_;
+		int sum_;
+		int count_;
 	};
 }
 
-size_t team::average_recruit_price()
+int team::average_recruit_price() const
 {
 	if (info_.average_price)
 		return info_.average_price;
@@ -578,49 +366,6 @@ size_t team::average_recruit_price()
 	return info_.average_price;
 }
 
-void team::set_time_of_day(int turn, const time_of_day& tod)
-{
-	aiparams_.clear();
-	for(std::vector<config>::const_iterator i = info_.ai_params.begin(); i != info_.ai_params.end(); ++i) {
-		const std::string& time_of_day = (*i)["time_of_day"];
-		if(time_of_day.empty() == false) {
-			const std::vector<std::string>& times = utils::split(time_of_day);
-			if(std::count(times.begin(),times.end(),tod.id) == 0) {
-				continue;
-			}
-		}
-
-		const std::string& turns = (*i)["turns"];
-		if(turns.empty() == false) {
-			bool matched = false;
-
-			const std::vector<std::string>& turns_list = utils::split(turns);
-			for(std::vector<std::string>::const_iterator j = turns_list.begin(); j != turns_list.end(); ++j) {
-				const std::pair<int,int> range = utils::parse_range(*j);
-				if(turn >= range.first && turn <= range.second) {
-					matched = true;
-					break;
-				}
-			}
-
-			if(!matched) {
-				continue;
-			}
-		}
-
-		aiparams_.append(*i);
-	}
-
-    // Get the recruitment pattern from the matching [ai] section,
-    // and fall back to the global recruitment pattern otherwise.
-	info_.recruitment_pattern = utils::split(aiparams_["recruitment_pattern"]);
-	if (info_.recruitment_pattern.empty())
-		info_.recruitment_pattern = info_.global_recruitment_pattern;
-	info_.aggression_ = lexical_cast_default<double>(aiparams_["aggression"],info_.aggression_);
-	info_.caution_ = lexical_cast_default<double>(aiparams_["caution"],info_.caution_);
-	info_.number_of_possible_recruits_to_force_recruit = lexical_cast_default<float>(aiparams_["number_of_possible_recruits_to_force_recruit"],info_.number_of_possible_recruits_to_force_recruit);
-}
-
 bool team::calculate_enemies(size_t index) const
 {
 	if(teams == NULL || index >= teams->size()) {
@@ -641,17 +386,25 @@ bool team::calculate_is_enemy(size_t index) const
 		return false;
 	}
 
-	// If we have a team name, we are friends
-	// with anyone who has the same team name
-	if(info_.team_name.empty() == false) {
-		return (*teams)[index].info_.team_name != info_.team_name;
+	// We are friends with anyone who we share a teamname with
+	std::vector<std::string> our_teams = utils::split(info_.team_name),
+						   their_teams = utils::split((*teams)[index].info_.team_name);
+	for(std::vector<std::string>::const_iterator t = our_teams.begin(); t != our_teams.end(); ++t) {
+		if(std::find(their_teams.begin(), their_teams.end(), *t) != their_teams.end())
+			return false;
 	}
+	return true;
+}
 
-	// If enemies aren't listed, then everyone is an enemy
-	if(info_.enemies.empty())
-		return true;
+void team::set_share_maps( bool share_maps ){
+	// Share_view and share_maps can't both be enabled,
+	// so share_view overrides share_maps.
+	// If you want to change them, be sure to change share_view FIRST
+	info_.share_maps = !info_.share_view && share_maps;
+}
 
-	return std::find(info_.enemies.begin(),info_.enemies.end(),int(index+1)) != info_.enemies.end();
+void team::set_share_view( bool share_view ){
+	info_.share_view = share_view;
 }
 
 void team::change_controller(const std::string& controller)
@@ -673,7 +426,7 @@ void team::change_controller(const std::string& controller)
 	info_.controller = cid;
 }
 
-void team::change_team(const std::string& name, const std::string& user_name)
+void team::change_team(const std::string &name, const t_string &user_name)
 {
 	info_.team_name = name;
 	if (!user_name.empty())
@@ -685,6 +438,10 @@ void team::change_team(const std::string& name, const std::string& user_name)
 		info_.user_team_name = name;
 	}
 
+	clear_caches();
+}
+
+void team::clear_caches(){
 	// Reset the cache of allies for all teams
 	if(teams != NULL) {
 		for(std::vector<team>::const_iterator i = teams->begin(); i != teams->end(); ++i) {
@@ -702,22 +459,6 @@ void team::set_objectives(const t_string& new_objectives, bool silently)
 		info_.objectives_changed = true;
 }
 
-void team::set_ai_memory(const config& ai_mem){
-  // It would perhaps be more efficient to allow writing to the memory directly,
-  // but this method comparmentalizes the functionality and protects against
-  // accidentally overwriting the memory
-  info_.ai_memory_=ai_mem;
-}
-
-void team::set_ai_parameters(const config::child_list& ai_parameters)
-{
-	info_.ai_params.clear();		// override
-	config::child_list::const_iterator i;
-	for (i = ai_parameters.begin(); i != ai_parameters.end(); ++i) {
-		info_.ai_params.push_back(**i);
-	}
-}
-
 bool team::shrouded(const map_location& loc) const
 {
 	if(!teams)
@@ -810,10 +551,9 @@ bool is_observer()
 		return true;
 	}
 
-	for(std::vector<team>::const_iterator i = teams->begin(); i != teams->end(); ++i) {
-		if(i->is_local()) {
+	foreach (const team &t, *teams) {
+		if (t.is_human())
 			return false;
-		}
 	}
 
 	return true;
@@ -830,15 +570,15 @@ void validate_side(int side)
 	}
 }
 
-bool team::shroud_map::clear(size_t x, size_t y)
+bool team::shroud_map::clear(int x, int y)
 {
-	if(enabled_ == false)
+	if(enabled_ == false || x < 0 || y < 0)
 		return false;
 
-	if(x >= data_.size())
+	if(x >= static_cast<int>(data_.size()))
 		data_.resize(x+1);
 
-	if(y >= data_[x].size())
+	if(y >= static_cast<int>(data_[x].size()))
 		data_[x].resize(y+1);
 
 	if(data_[x][y] == false) {
@@ -849,12 +589,18 @@ bool team::shroud_map::clear(size_t x, size_t y)
 	}
 }
 
-void team::shroud_map::place(size_t x, size_t y)
+void team::shroud_map::place(int x, int y)
 {
-	if(enabled_ == false)
+	if(enabled_ == false || x < 0 || y < 0)
 		return;
 
-	if(x < data_.size() && y < data_[x].size()) {
+	if (x >= static_cast<int>(data_.size())) {
+		DBG_NG << "Couldn't place shroud on invalid x coordinate: ("
+			<< x << ", " << y << ") - max x: " << data_.size() - 1 << "\n";
+	} else if (y >= static_cast<int>(data_[x].size())) {
+		DBG_NG << "Couldn't place shroud on invalid y coordinate: ("
+			<< x << ", " << y << ") - max y: " << data_[x].size() - 1 << "\n";
+	} else {
 		data_[x][y] = false;
 	}
 }
@@ -869,15 +615,15 @@ void team::shroud_map::reset()
 	}
 }
 
-bool team::shroud_map::value(size_t x, size_t y) const
+bool team::shroud_map::value(int x, int y) const
 {
-	if(enabled_ == false)
+	if(enabled_ == false || x < 0 || y < 0)
 		return false;
 
-	if(x >= data_.size())
+	if(x >= static_cast<int>(data_.size()))
 		return true;
 
-	if(y >= data_[x].size())
+	if(y >= static_cast<int>(data_[x].size()))
 		return true;
 
 	if(data_[x][y])
@@ -886,9 +632,9 @@ bool team::shroud_map::value(size_t x, size_t y) const
 		return true;
 }
 
-bool team::shroud_map::shared_value(const std::vector<const shroud_map*>& maps, size_t x, size_t y) const
+bool team::shroud_map::shared_value(const std::vector<const shroud_map*>& maps, int x, int y) const
 {
-	if(enabled_ == false)
+	if(enabled_ == false || x < 0 || y < 0)
 		return false;
 
 	for(std::vector<const shroud_map*>::const_iterator i = maps.begin(); i != maps.end(); ++i) {
@@ -998,9 +744,7 @@ std::string team::get_side_colour_index(int side)
 			return side_map;
 		}
 	}
-	std::stringstream id;
-	id<<side;
-	return id.str();
+	return str_cast(side);
 }
 
 std::string team::get_side_highlight(int side)
@@ -1011,12 +755,20 @@ std::string team::get_side_highlight(int side)
 void team::log_recruitable(){
 	LOG_NG << "Adding recruitable units: \n";
 	for (std::set<std::string>::const_iterator it = info_.can_recruit.begin();
-		 it != info_.can_recruit.end(); it++) {
+		 it != info_.can_recruit.end(); ++it) {
 		LOG_NG << *it << std::endl;
 	}
 	LOG_NG << "Added all recruitable units\n";
 }
 
+config team::to_config() const
+{
+	config cfg;
+	config& result = cfg.add_child("side");
+	write(result);
+	return result;
+}
+
 namespace player_teams {
 int village_owner(const map_location& loc)
 {
