@@ -1,6 +1,6 @@
-/* $Id: team.hpp 33039 2009-02-23 03:38:54Z sapient $ */
+/* $Id: team.hpp 41179 2010-02-13 10:18:45Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -18,10 +18,10 @@
 #include "color_range.hpp"
 #include "game_config.hpp"
 #include "map_location.hpp"
-#include "viewpoint.hpp"
+#include "savegame_config.hpp"
+#include "unit.hpp"
 
 class gamemap;
-struct time_of_day;
 
 #include <set>
 #include <string>
@@ -29,20 +29,22 @@ struct time_of_day;
 
 #include "SDL.h"
 
-//This class stores all the data for a single 'side' (in game nomenclature).
-//e.g. there is only one leader unit per team.
-class team : public viewpoint
+/**
+ * This class stores all the data for a single 'side' (in game nomenclature).
+ * E.g., there is only one leader unit per team.
+ */
+class team : public savegame::savegame_config
 {
 	class shroud_map {
 	public:
 		shroud_map() : enabled_(false), data_() {}
 
-		void place(size_t x, size_t y);
-		bool clear(size_t x, size_t y);
+		void place(int x, int y);
+		bool clear(int x, int y);
 		void reset();
 
-		bool value(size_t x, size_t y) const;
-		bool shared_value(const std::vector<const shroud_map*>& maps, size_t x, size_t y) const;
+		bool value(int x, int y) const;
+		bool shared_value(const std::vector<const shroud_map*>& maps, int x, int y) const;
 
 		bool copy_from(const std::vector<const shroud_map*>& maps);
 
@@ -58,28 +60,18 @@ class team : public viewpoint
 	};
 public:
 
-	struct target {
-		explicit target(const config& cfg);
-		void write(config& cfg) const;
-		config criteria;
-		double value;
-	};
-
 	struct team_info
 	{
 		team_info(const config& cfg);
 		void write(config& cfg) const;
 		std::string name;
-		std::string gold;
-		std::string start_gold;
-		std::string income;
+		int gold;
+		int start_gold;
+		bool gold_add;
+		int income;
 		int income_per_village;
-		size_t average_price;
-		float number_of_possible_recruits_to_force_recruit;
+		mutable int average_price;
 		std::set<std::string> can_recruit;
-		std::vector<std::string> global_recruitment_pattern;
-		std::vector<std::string> recruitment_pattern;
-		std::vector<int> enemies;
 		std::string team_name;
 		t_string user_team_name;
 		std::string save_id;
@@ -102,17 +94,6 @@ public:
 
 		enum CONTROLLER { HUMAN, HUMAN_AI, AI, NETWORK, NETWORK_AI, EMPTY };
 		CONTROLLER controller;
-		std::string ai_algorithm;
-
-		std::vector<config> ai_params;
-		config ai_memory_;
-
-		int villages_per_scout;
-		double leader_value, village_value;
-		//cached values for ai parameters
-		double aggression_, caution_;
-
-		std::vector<target> targets;
 
 		bool share_maps, share_view;
 		bool disallow_observers;
@@ -123,6 +104,9 @@ public:
 		std::string music;
 
 		std::string colour;
+
+		int side;
+		bool persistent;
 	};
 
 	static std::map<int, color_range> team_color_range_;
@@ -140,54 +124,44 @@ public:
 	bool owns_village(const map_location& loc) const
 		{ return villages_.count(loc) > 0; }
 
+	int side() const { return info_.side; }
 	int gold() const { return gold_; }
-	std::string start_gold() const { return info_.start_gold; }
-	int base_income() const
-		{ return atoi(info_.income.c_str()) + game_config::base_income; }
+	int start_gold() const { return info_.start_gold; }
+	bool gold_add() const { return info_.gold_add; }
+	int base_income() const { return info_.income + game_config::base_income; }
 	int village_gold() const { return info_.income_per_village; }
 	void set_village_gold(int income) { info_.income_per_village = income; }
-	int income() const
-		{ return atoi(info_.income.c_str()) + villages_.size()*info_.income_per_village+game_config::base_income; }
-	void new_turn() { gold_ += income(); }
-	void set_time_of_day(int turn, const struct time_of_day& tod);
+	int total_income() const { return base_income() + villages_.size() * info_.income_per_village; }
+	void new_turn() { gold_ += total_income(); }
 	void get_shared_maps();
+	void set_gold(int amount) { gold_ = amount; }
 	void spend_gold(const int amount) { gold_ -= amount; }
-	void set_income(const int amount)
-		{ info_.income = lexical_cast<std::string>(amount); }
+	void set_gold_add(bool b) {info_.gold_add = b; }
+	void set_base_income(int amount) { info_.income = amount - game_config::base_income; }
 	int countdown_time() const {  return countdown_time_; }
 	void set_countdown_time(const int amount)
 		{ countdown_time_ = amount; }
 	int action_bonus_count() const { return action_bonus_count_; }
 	void set_action_bonus_count(const int count) { action_bonus_count_ = count; }
-	void set_current_player(const std::string player)
+	std::vector<unit>& recall_list() {return recall_list_;}
+	const std::vector<unit>& recall_list() const {return recall_list_;}
+	void set_current_player(const std::string& player)
 		{ info_.current_player = player; }
 
-	size_t average_recruit_price();
-	float num_pos_recruits_to_force()
-	{ return info_.number_of_possible_recruits_to_force_recruit; }
+	int average_recruit_price() const;
+
 	const std::set<std::string>& recruits() const
 		{ return info_.can_recruit; }
-	void add_recruits(const std::set<std::string>& recruits);
+	void add_recruit(const std::string &);
 	void remove_recruit(const std::string& recruits);
 	void set_recruits(const std::set<std::string>& recruits);
-	const std::vector<std::string>& recruitment_pattern() const
-		{ return info_.recruitment_pattern; }
-	bool remove_recruitment_pattern_entry(const std::string& key)
-	{
-	   	std::vector<std::string>::iterator itor = std::find(info_.recruitment_pattern.begin(),
-				info_.recruitment_pattern.end(),
-				key);
-		if (itor == info_.recruitment_pattern.end())
-			return false;
-		info_.recruitment_pattern.erase(itor);
-		return true;
-	}
 	const std::string& name() const
 		{ return info_.name; }
 	const std::string& save_id() const { return info_.save_id; }
 	const std::string& current_player() const { return info_.current_player; }
 
 	void set_objectives(const t_string& new_objectives, bool silently=false);
+	void set_objectives_changed(bool c = true) { info_.objectives_changed = c; }
 	void reset_objectives_changed() { info_.objectives_changed = false; }
 
 	const t_string& objectives() const { return info_.objectives; }
@@ -217,10 +191,8 @@ public:
 		seen_[index] = true;
 	}
 
-	double aggression() const { return info_.aggression_; }
-	double caution() const { return info_.caution_; }
-
 	team_info::CONTROLLER controller() const { return info_.controller; }
+	const std::string& colour() const { return info_.colour; }
 	bool is_human() const { return info_.controller == team_info::HUMAN; }
 	bool is_human_ai() const { return info_.controller == team_info::HUMAN_AI; }
 	bool is_network_human() const { return info_.controller == team_info::NETWORK; }
@@ -228,7 +200,7 @@ public:
 	bool is_ai() const { return info_.controller == team_info::AI || is_human_ai(); }
 	bool is_empty() const { return info_.controller == team_info::EMPTY; }
 
-	bool is_local() const { return is_human() || is_human_ai(); }
+	bool is_local() const { return is_human() || is_ai(); }
 	bool is_network() const { return is_network_human() || is_network_ai(); }
 
 	void make_human() { info_.controller = team_info::HUMAN; }
@@ -242,28 +214,12 @@ public:
 	void change_controller(const std::string& controller);
 
 	const std::string& team_name() const { return info_.team_name; }
-	const std::string& user_team_name() const { return info_.user_team_name; }
-	void change_team(const std::string& name,
-					 const std::string& user_name);
+	const t_string &user_team_name() const { return info_.user_team_name; }
+	void change_team(const std::string &name, const t_string &user_name);
 
 	const std::string& flag() const { return info_.flag; }
 	const std::string& flag_icon() const { return info_.flag_icon; }
 
-	const std::string& ai_algorithm() const { return info_.ai_algorithm; }
-	const config& ai_parameters() const { return aiparams_; }
-	const config& ai_memory() const { return info_.ai_memory_; }
-	void set_ai_memory(const config& ai_mem);
-	void set_ai_parameters(const config::child_list& ai_parameters);
-
-	double leader_value() const { return info_.leader_value; }
-	void set_leader_value(double value) { info_.leader_value = value; }
-	double village_value() const { return info_.village_value; }
-	void set_village_value(double value) { info_.village_value = value; }
-
-	int villages_per_scout() const { return info_.villages_per_scout; }
-
-	std::vector<target>& targets() { return info_.targets; }
-
 	//Returns true if the hex is shrouded/fogged for this side, or
 	//any other ally with shared vision.
 	bool shrouded(const map_location& loc) const;
@@ -287,12 +243,13 @@ public:
 
 	bool auto_shroud_updates() const { return auto_shroud_updates_; }
 	void set_auto_shroud_updates(bool value) { auto_shroud_updates_ = value; }
-	bool get_disallow_observers() {return info_.disallow_observers; };
+	bool get_disallow_observers() const {return info_.disallow_observers; };
 	std::string map_colour_to() const { return info_.colour; };
 	bool no_leader() const { return info_.no_leader; }
 	void have_leader(bool value=true) { info_.no_leader = !value; }
 	bool hidden() const { return info_.hidden; }
 	void set_hidden(bool value) { info_.hidden=value; }
+	bool persistent() const {return info_.persistent;}
 
 	static int nteams();
 
@@ -307,6 +264,16 @@ public:
 
 	void log_recruitable();
 
+	/**set the share maps attribute */
+	void set_share_maps( bool share_maps );
+	/**set the share view attribute */
+	void set_share_view( bool share_view );
+
+	/** clear the shroud, fog, and enemies cache for all teams*/
+	static void clear_caches();
+
+	config to_config() const;
+
 private:
 	//Make these public if you need them, but look at knows_about_team(...) first.
 	bool share_maps() const { return info_.share_maps; }
@@ -327,8 +294,7 @@ private:
 	int countdown_time_;
 	int action_bonus_count_;
 
-	config aiparams_;
-
+	std::vector<unit> recall_list_;
 
 	bool calculate_enemies(size_t index) const;
 	bool calculate_is_enemy(size_t index) const;
@@ -339,13 +305,9 @@ private:
 	mutable std::vector<const shroud_map*> ally_shroud_, ally_fog_;
 };
 
-struct teams_manager {
-	teams_manager(std::vector<team>& teams);
-	~teams_manager();
-
-	bool is_observer();
-	static const std::vector<team>& get_teams();
-};
+namespace teams_manager {
+	const std::vector<team> &get_teams();
+}
 
 namespace player_teams {
 	int village_owner(const map_location& loc);
