@@ -1,6 +1,6 @@
-/* $Id: terrain.cpp 32518 2009-02-01 18:57:51Z soliton $ */
+/* $Id: terrain.cpp 41272 2010-02-17 18:49:50Z boucman $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -14,11 +14,20 @@
 
 #include "global.hpp"
 
-#include "config.hpp"
+#include "foreach.hpp"
 #include "gettext.hpp"
+#include "log.hpp"
+#include "serialization/string_utils.hpp"
 #include "terrain.hpp"
+#include "util.hpp"
 
+#include <set>
 
+static lg::log_domain log_config("config");
+#define ERR_CF LOG_STREAM(err, log_config)
+#define WRN_G LOG_STREAM(warn, lg::general)
+#define LOG_G LOG_STREAM(info, lg::general)
+#define DBG_G LOG_STREAM(debug, lg::general)
 
 terrain_type::terrain_type() :
 		minimap_image_("void"),
@@ -26,14 +35,17 @@ terrain_type::terrain_type() :
 		editor_image_("void"),
 		id_(),
 		name_(),
+		description_(),
 		number_(t_translation::VOID_TERRAIN),
 		mvt_type_(1, t_translation::VOID_TERRAIN),
 		def_type_(1, t_translation::VOID_TERRAIN),
 		union_type_(1, t_translation::VOID_TERRAIN),
-        height_adjust_(0),
+		height_adjust_(0),
+		height_adjust_set_(false),
 		submerge_(0.0),
+		submerge_set_(false),
 		light_modification_(0),
-        heals_(0),
+		heals_(0),
 		income_description_(),
 		income_description_ally_(),
 		income_description_enemy_(),
@@ -54,12 +66,15 @@ terrain_type::terrain_type(const config& cfg) :
 		editor_image_(cfg["editor_image"]),
 		id_(cfg["id"]),
 		name_(cfg["name"]),
+		description_(cfg["description"]),
 		number_(t_translation::read_terrain_code(cfg["string"])),
 		mvt_type_(),
 		def_type_(),
 		union_type_(),
 		height_adjust_(atoi(cfg["unit_height_adjust"].c_str())),
+		height_adjust_set_(!cfg["unit_height_adjust"].empty()),
 		submerge_(atof(cfg["submerge"].c_str())),
+		submerge_set_(!cfg["submerge"].empty()),
 		light_modification_(atoi(cfg["light"].c_str())),
 		heals_(lexical_cast_default<int>(cfg["heals"], 0)),
 		income_description_(),
@@ -79,16 +94,16 @@ terrain_type::terrain_type(const config& cfg) :
  *  @todo reenable these validations. The problem is that all MP
  *  scenarios/campaigns share the same namespace and one rogue scenario
  *  can avoid the player to create a MP game. So every scenario/campaign
- *  should get it's own namespace to be save.
+ *  should get it's own namespace to be safe.
  */
 #if 0
 	VALIDATE(number_ != t_translation::NONE_TERRAIN,
-		missing_mandatory_wml_key("terrain", "string"));
+		missing_mandatory_wml_key("terrain_type", "string"));
 	VALIDATE(!minimap_image_.empty(),
-		missing_mandatory_wml_key("terrain", "symbol_image", "string",
+		missing_mandatory_wml_key("terrain_type", "symbol_image", "string",
 		t_translation::write_terrain_code(number_)));
 	VALIDATE(!name_.empty(),
-		missing_mandatory_wml_key("terrain", "name", "string",
+		missing_mandatory_wml_key("terrain_type", "name", "string",
 		t_translation::write_terrain_code(number_)));
 #endif
 
@@ -162,12 +177,15 @@ terrain_type::terrain_type(const terrain_type& base, const terrain_type& overlay
 	editor_image_(overlay.editor_image_),
 	id_(base.id_+"^"+overlay.id_),
 	name_(overlay.name_),
+	description_(overlay.description_),
 	number_(t_translation::t_terrain(base.number_.base, overlay.number_.overlay)),
 	mvt_type_(overlay.mvt_type_),
 	def_type_(overlay.def_type_),
 	union_type_(),
-	height_adjust_(overlay.height_adjust_),
-	submerge_(overlay.submerge_),
+	height_adjust_(base.height_adjust_),
+	height_adjust_set_(base.height_adjust_set_),
+	submerge_(base.submerge_),
+	submerge_set_(base.submerge_set_),
 	light_modification_(base.light_modification_ + overlay.light_modification_),
 	heals_(std::max<int>(base.heals_, overlay.heals_)),
 	income_description_(),
@@ -184,6 +202,16 @@ terrain_type::terrain_type(const terrain_type& base, const terrain_type& overlay
 	hide_in_editor_(base.hide_in_editor_ || overlay.hide_in_editor_)
 {
 
+	if(overlay.height_adjust_set_) {
+		height_adjust_set_ = true;
+		height_adjust_ = overlay.height_adjust_;
+	}
+
+	if(overlay.submerge_set_) {
+		submerge_set_ = true;
+		submerge_ = overlay.submerge_;
+	}
+
 	merge_alias_lists(mvt_type_, base.mvt_type_);
 	merge_alias_lists(def_type_, base.def_type_);
 
@@ -226,16 +254,38 @@ t_translation::t_terrain terrain_type::terrain_with_default_base() const {
 	return number_;
 }
 
-void create_terrain_maps(const std::vector<config*>& cfgs,
+void create_terrain_maps(const config::const_child_itors &cfgs,
                          t_translation::t_list& terrain_list,
                          std::map<t_translation::t_terrain, terrain_type>& letter_to_terrain)
 {
-	for(std::vector<config*>::const_iterator i = cfgs.begin();
-	    i != cfgs.end(); ++i) {
-		terrain_type terrain(**i);
-		terrain_list.push_back(terrain.number());
-		letter_to_terrain.insert(std::pair<t_translation::t_terrain, terrain_type>(
-		                              terrain.number(),terrain));
+	foreach (const config &t, cfgs)
+	{
+		terrain_type terrain(t);
+		DBG_G << "create_terrain_maps: " << terrain.number() << " "
+			<< terrain.id() << " " << terrain.name() << " : " << terrain.editor_group() << "\n";
+
+		std::pair<std::map<t_translation::t_terrain, terrain_type>::iterator, bool> res;
+		res = letter_to_terrain.insert(std::make_pair(terrain.number(), terrain));
+		if (!res.second) {
+			terrain_type& curr = res.first->second;
+			WRN_G << "Duplicate terrain code definition found for " << terrain.number() << "\n";
+			WRN_G << "Trying to add terrain "
+				<< terrain.id() << " (" << terrain.name() << ") "
+				<< "[" << terrain.editor_group() << "]" << "\n";
+			WRN_G << "which conflicts with  "
+				<< curr.id() << " (" << curr.name() << ") "
+				<< "[" << curr.editor_group() << "]" << "\n";
+			std::vector<std::string> eg1 = utils::split(curr.editor_group());
+			std::vector<std::string> eg2 = utils::split(terrain.editor_group());
+			std::set<std::string> egs;
+			std::copy(eg1.begin(), eg1.end(), std::inserter(egs, egs.begin()));
+			std::copy(eg2.begin(), eg2.end(), std::inserter(egs, egs.begin()));
+			std::string joined = utils::join(egs);
+			curr.set_editor_group(joined);
+			WRN_G << "Editor groups merged to: " << joined << "\n";
+		} else {
+			terrain_list.push_back(terrain.number());
+		}
 	}
 }
 
@@ -246,7 +296,7 @@ void merge_alias_lists(t_translation::t_list& first, const t_translation::t_list
 	bool revert = (first.front() == t_translation::MINUS ? true : false);
 	t_translation::t_list::iterator i;
 
-	for(i = first.begin(); i != first.end(); i++) {
+	for(i = first.begin(); i != first.end(); ++i) {
 		if(*i == t_translation::PLUS) {
 			revert = false;
 			continue;
