@@ -1,6 +1,6 @@
-/* $Id: terrain.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: terrain.hpp 41272 2010-02-17 18:49:50Z boucman $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -14,13 +14,12 @@
 #ifndef TERRAIN_H_INCLUDED
 #define TERRAIN_H_INCLUDED
 
-class config;
+#include "config.hpp"
 #include "tstring.hpp"
 #include "terrain_translation.hpp"
 
 #include <map>
 #include <string>
-#include <vector>
 
 class terrain_type
 {
@@ -28,12 +27,17 @@ public:
 
 	terrain_type();
 	terrain_type(const config& cfg);
+	terrain_type(const terrain_type& base, const terrain_type& overlay);
 
 	const std::string& minimap_image() const { return minimap_image_; }
+	const std::string& minimap_image_overlay() const { return minimap_image_overlay_; }
 	const std::string& editor_image() const { return editor_image_; }
 	const t_string& name() const { return name_; }
+        const t_string& description() const { return description_.empty() ? name_ : description_; }
 	const std::string& id() const { return id_; }
 
+	bool hide_in_editor() const { return hide_in_editor_; }
+
 	//the character representing this terrain
 	t_translation::t_terrain number() const { return number_; }
 
@@ -62,29 +66,42 @@ public:
 	const t_string& income_description_own() const { return income_description_own_; }
 
 	const std::string& editor_group() const { return editor_group_; }
+	void set_editor_group(const std::string& str) { editor_group_ = str; }
+
+	bool is_overlay() const { return overlay_; }
+	bool is_combined() const { return combined_; }
+
+	t_translation::t_terrain default_base() const { return editor_default_base_; }
+	t_translation::t_terrain terrain_with_default_base() const;
 
 private:
-	//! The image used in the minimap
+	/** The image used in the minimap */
 	std::string minimap_image_;
+	std::string minimap_image_overlay_;
 
-	//! The image used in the editor pallete if not defined in WML it will
-	//! be initialized with the value of minimap_image_
+	/**
+	 *  The image used in the editor pallete if not defined in WML it will be
+	 *  initialized with the value of minimap_image_
+	 */
 	std::string editor_image_;
 	std::string id_;
 	t_string name_;
+	t_string description_;
 
 	//the 'number' is the number that represents this
 	//terrain type. The 'type' is a list of the 'underlying types'
 	//of the terrain. This may simply be the same as the number.
-	//This is the internal number used, WML still used characters
+	//This is the internal number used, WML still uses character strings.
 	t_translation::t_terrain number_;
 	t_translation::t_list mvt_type_;
 	t_translation::t_list def_type_;
 	t_translation::t_list union_type_;
 
 	int height_adjust_;
+	bool height_adjust_set_;
 
 	double submerge_;
+	bool submerge_set_;
 
 	int light_modification_, heals_;
 
@@ -96,9 +113,16 @@ private:
 	std::string editor_group_;
 
 	bool village_, castle_, keep_;
+
+	bool overlay_, combined_;
+	t_translation::t_terrain editor_default_base_;
+	bool hide_in_editor_;
 };
 
-void create_terrain_maps(const std::vector<config*>& cfgs,
+void create_terrain_maps(const config::const_child_itors &cfgs,
                          t_translation::t_list& terrain_list,
                          std::map<t_translation::t_terrain, terrain_type>& letter_to_terrain);
+
+void merge_alias_lists(t_translation::t_list& first, const t_translation::t_list& second);
+
 #endif
