@@ -1,6 +1,6 @@
-/* $Id: terrain_filter.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: terrain_filter.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -16,24 +16,26 @@
 #define TERRAIN_FILTER_H_INCLUDED
 
 #include "map_location.hpp"
-#include "pathfind.hpp"
+#include "pathutils.hpp"
+#include "variable.hpp"
 
 class config;
-class gamestatus;
 class unit;
-class vconfig;
 class unit_map;
+class team;
 
 //terrain_filter: a class that implements the Standard Location Filter
 class terrain_filter : public xy_pred {
 public:
+
 #ifdef _MSC_VER
 	// This constructor is required for MSVC 9 SP1 due to a bug there
 	// see http://social.msdn.microsoft.com/forums/en-US/vcgeneral/thread/34473b8c-0184-4750-a290-08558e4eda4e
 	// other compilers don't need it.
-	terrain_filter();
+	 terrain_filter();
 #endif
-	terrain_filter(const vconfig& cfg, const gamemap& map, const gamestatus& game_status,
+
+	terrain_filter(const vconfig& cfg,
 		const unit_map& units, const bool flat_tod=false, const size_t max_loop=MAX_MAP_AREA);
 	terrain_filter(const vconfig& cfg, const terrain_filter& original);
 	~terrain_filter() {};
@@ -42,12 +44,12 @@ public:
 	terrain_filter& operator=(const terrain_filter &other);
 
 	//match: returns true if and only if the given location matches this filter
-	bool match(const map_location& loc);
+	bool match(const map_location& loc) const;
 	virtual bool operator()(const map_location& loc) { return this->match(loc); }
 
 	//get_locations: gets all locations on the map that match this filter
 	// @param locs - out parameter containing the results
-	void get_locations(std::set<map_location>& locs);
+	void get_locations(std::set<map_location>& locs, bool with_border=false) const;
 
 	//restrict: limits the allowed radius size and also limits nesting
 	// The purpose to limit the time spent for WML handling
@@ -57,12 +59,11 @@ public:
 	//flatten: use base time of day -- ignore illumination ability
 	void flatten(const bool flat_tod=true) { flat_ = flat_tod; }
 
+	config to_config() const;
 private:
-	bool match_internal(const map_location& loc, const bool ignore_xy);
+	bool match_internal(const map_location& loc, const bool ignore_xy) const;
 
-	const vconfig& cfg_; //config contains WML for a Standard Location Filter
-	const gamemap& map_;
-	const gamestatus& status_;
+	const vconfig cfg_; //config contains WML for a Standard Location Filter
 	const unit_map& units_;
 
 	struct terrain_filter_cache {
@@ -88,7 +89,7 @@ private:
 		std::vector< std::pair<terrain_filter, std::map<map_location,bool> > > adjacent_match_cache;
 	};
 
-	terrain_filter_cache cache_;
+	mutable terrain_filter_cache cache_;
 	size_t max_loop_;
 	bool flat_;
 };
