@@ -1,6 +1,6 @@
-/* $Id: terrain_translation.cpp 34467 2009-04-04 16:04:47Z ilor $ */
+/* $Id: terrain_translation.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2006 - 2009 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2006 - 2010 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -24,11 +24,12 @@
 #include "log.hpp"
 #include "terrain_translation.hpp"
 #include "serialization/string_utils.hpp"
+#include "util.hpp"
 #include "wml_exception.hpp"
 
 
-#define ERR_G  LOG_STREAM(err, general)
-#define WRN_G  LOG_STREAM(warn, general)
+#define ERR_G LOG_STREAM(err, lg::general)
+#define WRN_G LOG_STREAM(warn, lg::general)
 
 namespace t_translation {
 
@@ -140,13 +141,14 @@ const t_terrain NOT = string_to_number_("!");
 const t_terrain STAR = string_to_number_("*");
 const t_terrain BASE = string_to_number_("_bas");
 
+const t_match ALL_FORESTS("F*,*^F*");
+const t_match ALL_HILLS("!,*^V*,!,H*");
+const t_match ALL_MOUNTAINS("!,*^V*,!,M*"); //excluding impassable mountains
+const t_match ALL_SWAMPS("!,*^V*,*^B*,!,S*"); //excluding swamp villages and bridges
+
 /***************************************************************************************/
 
-t_terrain::t_terrain(const std::string& b) :
-	base(string_to_layer_(b)), overlay(NO_LAYER)
-{}
-
-t_terrain::t_terrain(const std::string& b, const t_layer o) :
+t_terrain::t_terrain(const std::string& b, t_layer o) :
 	base(string_to_layer_(b)), overlay(o)
 {}
 
