@@ -1,6 +1,6 @@
-/* $Id: test_formula_ai.cpp 33677 2009-03-15 15:04:27Z dragonking $ */
+/* $Id: test_formula_ai.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2008 - 2009 by Pauli Nieminen <paniemin@cc.hut.fi>
+   Copyright (C) 2008 - 2010 by Pauli Nieminen <paniemin@cc.hut.fi>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
 
 #define GETTEXT_DOMAIN "wesnoth-test"
 
-#include "utils/test_support.hpp"
+#include <boost/test/unit_test.hpp>
 
 
 
@@ -32,8 +32,8 @@ BOOST_AUTO_TEST_CASE( test_move )
 	scenario.play();
 
 	// Remember that map_location() handles automaticaly transition to 1 based from 0 based locations
-	BOOST_CHECK_EQUAL(scenario.find_unit_loc("side_1_leader"), map_location(14,4));
+	BOOST_CHECK_EQUAL(scenario.get_unit_id(map_location(14, 4)), "side_1_leader");
 }
 
 
-BOOST_AUTO_TEST_SUITE_END();
+BOOST_AUTO_TEST_SUITE_END()
