@@ -1,6 +1,6 @@
-/* $Id: fake_display.hpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: fake_display.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2008 - 2009 by Pauli Nieminen <paniemin@cc.hut.fi>
+   Copyright (C) 2008 - 2010 by Pauli Nieminen <paniemin@cc.hut.fi>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,19 @@ class game_display;
 
 namespace test_utils {
 
-	game_display& get_fake_display();
+	/**
+	 * Gets a fake test display.
+	 *
+	 * The width and height parameter are ignored if either of them is less
+	 * than zero.
+	 *
+	 * @param width               The width of the display.
+	 * @param height              The height of the dislay.
+	 *
+	 * @returns                   The display.
+	 */
+	game_display& get_fake_display(
+			const int width, const int height);
 
 
 }
