@@ -1,6 +1,6 @@
-/* $Id: text.cpp 35983 2009-05-31 19:55:13Z mordante $ */
+/* $Id: text.cpp 42830 2010-05-24 14:41:15Z ai0867 $ */
 /*
-   Copyright (C) 2008 - 2009 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -16,15 +16,20 @@
 
 #include "text.hpp"
 
+#include "gettext.hpp"
 #include "gui/widgets/helper.hpp"
+#include "gui/auxiliary/log.hpp"
 #include "font.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <cassert>
 #include <cstring>
 
-namespace font {
+#ifndef PANGO_VERSION_CHECK
+#define PANGO_VERSION_CHECK(a,b,c) 0
+#endif
 
-namespace internal {
+namespace font {
 
 namespace {
 
@@ -59,8 +64,12 @@ const unsigned ttext::STYLE_ITALIC = TTF_STYLE_ITALIC;
 const unsigned ttext::STYLE_UNDERLINE = TTF_STYLE_UNDERLINE;
 
 ttext::ttext() :
+#if PANGO_VERSION_CHECK(1,22,0)
+	context_(pango_font_map_create_context(pango_cairo_font_map_get_default())),
+#else
 	context_(pango_cairo_font_map_create_context((
 		reinterpret_cast<PangoCairoFontMap*>(pango_cairo_font_map_get_default())))),
+#endif
 	layout_(pango_layout_new(context_)),
 	rect_(),
 	surface_(),
@@ -72,6 +81,7 @@ ttext::ttext() :
 	maximum_width_(-1),
 	maximum_height_(-1),
 	ellipse_mode_(PANGO_ELLIPSIZE_END),
+	alignment_(PANGO_ALIGN_LEFT),
 	maximum_length_(std::string::npos),
 	calculation_dirty_(true),
 	length_(0),
@@ -82,6 +92,7 @@ ttext::ttext() :
 	pango_cairo_context_set_resolution(context_, 72.0);
 
 	pango_layout_set_ellipsize(layout_, ellipse_mode_);
+	pango_layout_set_alignment(layout_, alignment_);
 
 	/*
 	 * Set the pango spacing a bit bigger since the default is deemed to small
@@ -137,12 +148,11 @@ bool ttext::is_truncated() const
 {
 	recalculate();
 
-#if defined(PANGO_VERSION_CHECK)
 #if PANGO_VERSION_CHECK(1,16,0)
-	return (pango_layout_is_ellipsized(layout_) == TRUE);
-#endif
-#endif
+	return (pango_layout_is_ellipsized(layout_) != 0);
+#else
 	return false;
+#endif
 }
 
 unsigned ttext::insert_text(const unsigned offset, const std::string& text)
@@ -239,7 +249,7 @@ gui2::tpoint ttext::get_column_line(const gui2::tpoint& position) const
 	// Now convert this offset to a column, this way is a bit hacky but haven't
 	// found a better solution yet.
 
-	/** 
+	/**
 	 * @todo There's still a bug left. When you select a text which is in the
 	 * ellipses on the right side the text gets reformatted with ellipses on
 	 * the left and the selected character is not the one under the cursor.
@@ -256,27 +266,31 @@ gui2::tpoint ttext::get_column_line(const gui2::tpoint& position) const
 	}
 }
 
-void ttext::clone()
-{
-	context_ = pango_cairo_font_map_create_context((
-		reinterpret_cast<PangoCairoFontMap*>(pango_cairo_font_map_get_default())));
-	// With 72 dpi the sizes are the same as with SDL_TTF so hardcoded.
-	pango_cairo_context_set_resolution(context_, 72.0);
-
-	layout_ = pango_layout_new(context_);
-	pango_layout_set_ellipsize(layout_, ellipse_mode_);
-
-	surface_dirty_ = true;
-	surface_buffer_ = 0;
-}
-
-ttext& ttext::set_text(const std::string& text, const bool markedup)
+bool ttext::set_text(const std::string& text, const bool markedup)
 {
 	if(markedup != markedup_text_ || text != text_) {
 		assert(layout_);
 
+		const wide_string wide = utils::string_to_wstring(text);
+		const std::string narrow = utils::wstring_to_string(wide);
+		if(text != narrow) {
+			ERR_GUI_L << "ttext::" << __func__
+					<< " text '" << text
+					<< "' contains invalid utf-8, trimmed the invalid parts.\n";
+		}
 		if(markedup) {
-			pango_layout_set_markup(layout_, text.c_str(), text.size());
+			if(!pango_parse_markup(narrow.c_str(), narrow.size()
+						, 0, NULL, NULL, NULL, NULL)) {
+
+				ERR_GUI_L << "ttext::" << __func__
+						<< " text '" << narrow
+						<< "' has broken markup, set to normal text.\n";
+				/** @todo Enable after 1.8. */
+//				set_text(_("The text contains invalid markup: ") + narrow, false);
+				set_text(narrow, false);
+				return false;
+			}
+			pango_layout_set_markup(layout_, narrow.c_str(), narrow.size());
 		} else {
 			/*
 			 * pango_layout_set_text after pango_layout_set_markup might
@@ -284,16 +298,16 @@ ttext& ttext::set_text(const std::string& text, const bool markedup)
 			 * clear it unconditionally.
 			 */
 			pango_layout_set_attributes(layout_, NULL);
-			pango_layout_set_text(layout_, text.c_str(), text.size());
+			pango_layout_set_text(layout_, narrow.c_str(), narrow.size());
 		}
-		text_ = text;
-		length_ = utils::string_to_wstring(text_).size();
+		text_ = narrow;
+		length_ = wide.size();
 		markedup_text_ = markedup;
 		calculation_dirty_ = true;
 		surface_dirty_ = true;
 	}
 
-	return *this;
+	return true;
 }
 
 ttext& ttext::set_font_size(const unsigned font_size)
@@ -336,8 +350,23 @@ ttext& ttext::set_maximum_width(int width)
 
 	if(width != maximum_width_) {
 		assert(context_);
-
-		pango_layout_set_width(layout_, width == -1 ? -1 : width * PANGO_SCALE);
+#if 0
+		/**
+		 * todo Adding 4 extra pixels feels a bit hacky.
+		 *
+		 * For some reason it's needed since the following scenario fails:
+		 * - pango_layout_set_width(value)
+		 * - pango_layout_get_pixel_extents() -> max_width_1
+		 * - pango_layout_set_width(max_width_1)
+		 * - pango_layout_get_pixel_extents() -> max_width_2
+		 *
+		 * Now it can happen max_width_2 < max_width_1. Adding the 4 seems to
+		 * "fix" the problem.
+		 */
+		pango_layout_set_width(layout_, width == -1
+				? -1
+				: (width + 4) * PANGO_SCALE);
+#endif
 		maximum_width_ = width;
 		calculation_dirty_ = true;
 		surface_dirty_ = true;
@@ -346,10 +375,11 @@ ttext& ttext::set_maximum_width(int width)
 	return *this;
 }
 
-ttext& ttext::set_maximum_height(int height)
+ttext& ttext::set_maximum_height(int height, bool multiline)
 {
 	if(height <= 0) {
 		height = -1;
+		multiline = false;
 	}
 
 	if(height != maximum_height_) {
@@ -359,10 +389,8 @@ ttext& ttext::set_maximum_height(int height)
  * @todo See whether we can make pango 1.20 mandatory before Wesnoth 1.6 is
  * released.
  */
-#if defined(PANGO_VERSION_CHECK)
 #if PANGO_VERSION_CHECK(1,20,0)
-		pango_layout_set_height(layout_, height == -1 ? -1 : height * PANGO_SCALE);
-#endif
+		pango_layout_set_height(layout_, !multiline ? -1 : height * PANGO_SCALE);
 #endif
 		maximum_height_ = height;
 		calculation_dirty_ = true;
@@ -386,6 +414,15 @@ ttext& ttext::set_ellipse_mode(const PangoEllipsizeMode ellipse_mode)
 	return *this;
 }
 
+ttext &ttext::set_alignment(const PangoAlignment alignment)
+{
+	if (alignment == alignment_) return *this;
+	pango_layout_set_alignment(layout_, alignment);
+	alignment_ = alignment;
+	surface_dirty_ = true;
+	return *this;
+}
+
 ttext& ttext::set_maximum_length(const size_t maximum_length)
 {
 	if(maximum_length != maximum_length_) {
@@ -455,25 +492,94 @@ void ttext::recalculate(const bool force) const
 		tfont font(get_font_families(), font_size_, font_style_);
 		pango_layout_set_font_description(layout_, font.get());
 
-		pango_layout_get_pixel_extents(layout_, NULL, &rect_);
+		/*
+		 * See set_maximum_width for some more background info as well.
+		 * In order to fix the problem first set a width which seems to render
+		 * correctly then lower it to fit. For the campaigns the 4 does "the
+		 * right thing" for the terrain labels it should use the value 0 to set
+		 * the ellipse properly. Need to see whether this is a bug in pango or
+		 * a bug in my understanding of the pango api.
+		 */
+		int hack = 4;
+		do {
+			pango_layout_set_width(layout_, maximum_width_ == -1
+					? -1
+					: (maximum_width_ + hack) * PANGO_SCALE);
+			pango_layout_get_pixel_extents(layout_, NULL, &rect_);
+
+			DBG_GUI_L << "ttext::" << __func__
+					<< " text '" << gui2::debug_truncate(text_)
+					<< "' maximum_width " << maximum_width_
+					<< " hack " << hack
+					<< " width " << rect_.x + rect_.width
+					<< ".\n";
+
+			--hack;
+		} while(maximum_width_ != -1
+				&& hack >= 0 && rect_.x + rect_.width > maximum_width_);
+
+		DBG_GUI_L << "ttext::" << __func__
+				<< " text '" << gui2::debug_truncate(text_)
+				<< "' font_size " << font_size_
+				<< " markedup_text " << markedup_text_
+				<< " font_style " << std::hex << font_style_ << std::dec
+				<< " maximum_width " << maximum_width_
+				<< " maximum_height " << maximum_height_
+				<< " result " <<  rect_
+				<< ".\n";
+		if(maximum_width_ != -1 && rect_.x + rect_.width > maximum_width_) {
+			DBG_GUI_L << "ttext::" << __func__
+					<< " text '" << gui2::debug_truncate(text_)
+					<< " ' width " << rect_.x + rect_.width
+					<< " greater as the wanted maximum of " << maximum_width_
+					<< ".\n";
+		}
 	}
 }
 
-/**
- * Helper function to decode pixels.
- *
- * @param alpha                   The value of the alpha channel, this value
- *                                must not be zero.
- * @param sub_pixel               The address of a sub pixel, either red,
- *                                green or blue.
- */
-static void decode_pixel(const unsigned char alpha, unsigned char *sub_pixel)
+struct decode_table
 {
-	unsigned colour = *sub_pixel;
-	// Sature at 255.
-	*sub_pixel = colour >= alpha
-			? 255
-			: static_cast<unsigned char>((colour << 8) / alpha);
+	// 1-based, from 1 to 255.
+	unsigned values[255];
+	decode_table()
+	{
+		for (int i = 1; i < 256; ++i) values[i - 1] = (255 * 256) / i;
+	}
+};
+
+static decode_table decode_table;
+
+/**
+ * Converts from premultiplied alpha to plain alpha.
+ * @param p pointer to a 4-byte endian-dependent color.
+ */
+static void decode_pixel(unsigned char *p)
+{
+// Assume everything not compiled with gcc to be on a little endian platform.
+#if defined(__GNUC__) && defined(__BIG_ENDIAN__)
+	int alpha = p[0];
+#else
+	int alpha = p[3];
+#endif
+	if (alpha == 0) return;
+
+	int div = decode_table.values[alpha - 1];
+
+#define DECODE(i) \
+	do { \
+		unsigned color = p[i]; \
+		color = color * div / 256; \
+		if (color > 255) color = 255; \
+		p[i] = color; \
+	} while (0)
+
+#if defined(__GNUC__) && defined(__BIG_ENDIAN__)
+	DECODE(3);
+#else
+	DECODE(0);
+#endif
+	DECODE(1);
+	DECODE(2);
 }
 
 void ttext::rerender(const bool force) const
@@ -484,8 +590,10 @@ void ttext::rerender(const bool force) const
 		recalculate(force);
 		surface_dirty_ = false;
 
-		const unsigned width = rect_.x + rect_.width;
-		const unsigned height = rect_.y + rect_.height;
+		int width = rect_.x + rect_.width;
+		int height = rect_.y + rect_.height;
+		if (maximum_width_  > 0 && width  > maximum_width_ ) width  = maximum_width_;
+		if (maximum_height_ > 0 && height > maximum_height_) height = maximum_height_;
 		const unsigned stride = width * 4;
 		create_surface_buffer(stride * height);
 
@@ -494,8 +602,6 @@ void ttext::rerender(const bool force) const
 				CAIRO_FORMAT_ARGB32, width, height, stride);
 		cairo_t *cr = cairo_create(cairo_surface);
 
-		pango_cairo_update_context (cr, context_); // Needed?
-
 		/* set colour (used for foreground). */
 		cairo_set_source_rgba(cr,
 			 (foreground_colour_ >> 24)         / 256.0,
@@ -510,29 +616,11 @@ void ttext::rerender(const bool force) const
 		// The cairo surface is in CAIRO_FORMAT_ARGB32 which uses
 		// pre-multiplied alpha. SDL doesn't use that so the pixels need to be
 		// decoded again.
-		for(size_t y = 0; y < height; ++y) {
-			for(size_t x = 0; x < width; ++x) {
-
+		for (int y = 0; y < height; ++y) {
+			for (int x = 0; x < width; ++x)
+			{
 				unsigned char *pixel = &surface_buffer_[(y * width + x) * 4];
-
-// Assume everything not compiled with gcc to be on a little endian platform.
-#if defined(__GNUC__) && defined(__BIG_ENDIAN__)
-				const unsigned char alpha = *(pixel + 0);
-#else
-				const unsigned char alpha = *(pixel + 3);
-#endif
-				if(alpha == 0) {
-					continue;
-				}
-
-// Assume everything not compiled with gcc to be on a little endian platform.
-#if defined(__GNUC__) && defined(__BIG_ENDIAN__)
-				decode_pixel(alpha, pixel + 3);
-#else
-				decode_pixel(alpha, pixel + 0);
-#endif
-				decode_pixel(alpha, pixel + 1);
-				decode_pixel(alpha, pixel + 2);
+				decode_pixel(pixel);
 			}
 		}
 #else
@@ -546,6 +634,7 @@ void ttext::rerender(const bool force) const
 			surface_buffer_, width, height, 32, stride,
 			0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000));
 		cairo_destroy(cr);
+		cairo_surface_destroy(cairo_surface);
 	}
 }
 
@@ -561,7 +650,5 @@ void ttext::create_surface_buffer(const size_t size) const
 	memset(surface_buffer_, 0, size);
 }
 
-} // namespace internal
-
 } // namespace font
 
