@@ -1,6 +1,6 @@
-/* $Id: theme.cpp 33185 2009-02-28 22:09:57Z ilor $ */
+/* $Id: theme.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -17,16 +17,19 @@
 #include "global.hpp"
 
 #include "font.hpp"
+#include "foreach.hpp"
 #include "gettext.hpp"
 #include "hotkeys.hpp"
 #include "log.hpp"
+#include "serialization/string_utils.hpp"
 #include "theme.hpp"
 #include "wml_exception.hpp"
 
 
-#define DBG_DP LOG_STREAM(debug, display)
-#define LOG_DP LOG_STREAM(info, display)
-#define ERR_DP LOG_STREAM(err, display)
+static lg::log_domain log_display("display");
+#define DBG_DP LOG_STREAM(debug, log_display)
+#define LOG_DP LOG_STREAM(info, log_display)
+#define ERR_DP LOG_STREAM(err, log_display)
 
 namespace {
 	const int XDim = 1024;
@@ -111,39 +114,34 @@ static std::string resolve_rect(const std::string& rect_str) {
 		return resolved.str();
 	}
 
-namespace {
-	config empty_config = config();
-}
+static config &find_ref(const std::string &id, config &cfg, bool remove = false)
+{
+	static config empty_config;
 
-static config& find_ref(const std::string& id, config& cfg, bool remove = false) {
-		for(config::child_map::const_iterator i = cfg.all_children().begin();
-		    i != cfg.all_children().end(); i++) {
-			for (config::child_list::const_iterator j = i->second.begin();
-			     j != i->second.end(); j++) {
-				if ((**j)["id"] == id) {
-					//DBG_DP << "Found a " << *(*i).first << "\n";
-					if (remove) {
-						const config* const res = cfg.find_child((*i).first,"id",id);
-						const size_t index = std::find((*i).second.begin(), (*i).second.end(),
-									       res) - (*i).second.begin();
-						cfg.remove_child((*i).first,index);
-						return empty_config;
-					} else {
-						return **j;
-					}
-				}
-
-				// recursively look in children
-				config& c = find_ref(id, **j, remove);
-				if (!c["id"].empty()) {
-					return c;
-				}
+	config::all_children_itors itors = cfg.all_children_range();
+	for (config::all_children_iterator i = itors.first; i != itors.second; ++i)
+	{
+		config &icfg = const_cast<config &>(i->cfg);
+		if (i->cfg["id"] == id) {
+			if (remove) {
+				cfg.erase(i);
+				return empty_config;
+			} else {
+				return icfg;
 			}
 		}
-		// not found
-		return empty_config;
+
+		// Recursively look in children.
+		config &c = find_ref(id, icfg, remove);
+		if (&c != &empty_config) {
+			return c;
+		}
 	}
 
+	// Not found.
+	return empty_config;
+}
+
 #ifdef DEBUG
 
 // to be called from gdb
@@ -169,9 +167,9 @@ static void expand_partialresolution(config& dst_cfg, const config& top_cfg)
 			std::vector<const config*> parent_stack(1, (*i));
 			const config* parent;
 			const t_string* parent_id = &((**i)["inherits"]);
-			while((parent = top_cfg.find_child("resolution", "id", (*parent_id))) == NULL) {
-				parent = top_cfg.find_child("partialresolution", "id", (*parent_id));
-				if(parent == NULL)
+			while (!*(parent = &top_cfg.find_child("resolution", "id", (*parent_id)))) {
+				parent = &top_cfg.find_child("partialresolution", "id", (*parent_id));
+				if (!*parent)
 					throw config::error("[partialresolution] refers to non-existant [resolution] " + (*parent_id).str());
 				parent_stack.push_back(parent);
 				parent_id = &((*parent)["inherits"]);
@@ -181,46 +179,32 @@ static void expand_partialresolution(config& dst_cfg, const config& top_cfg)
 			res_cfgs_.push_back(*parent);
 			while(!parent_stack.empty()) {
 				//override attributes
-				for(string_map::const_iterator j = parent_stack.back()->values.begin(); j != parent_stack.back()->values.end(); ++j) {
-					res_cfgs_.back().values[j->first] = j->second;
+				res_cfgs_.back().merge_attributes(*parent_stack.back());
+
+				foreach (const config &rm, parent_stack.back()->child_range("remove")) {
+					find_ref(rm["id"], res_cfgs_.back(), true);
 				}
 
+				foreach (const config &chg, parent_stack.back()->child_range("change"))
 				{
-					const config::child_list& c = parent_stack.back()->get_children("remove");
-					for(config::child_list::const_iterator j = c.begin(); j != c.end(); ++j) {
-						find_ref ((**j)["id"], res_cfgs_.back(), true);
-					}
+					config &target = find_ref(chg["id"], res_cfgs_.back());
+					target.merge_attributes(chg);
 				}
+
+				// cannot add [status] sub-elements, but who cares
+				if (const config &c = parent_stack.back()->child("add"))
 				{
-					const config::child_list& c = parent_stack.back()->get_children("change");
-					for(config::child_list::const_iterator j = c.begin(); j != c.end(); ++j) {
-						config& target = find_ref ((**j)["id"], res_cfgs_.back());
-						for(string_map::iterator k = (**j).values.begin();
-								k != (**j).values.end(); ++k) {
-							target.values[k->first] = k->second;
-						}
-					}
-				}
-				{
-					// cannot add [status] sub-elements, but who cares
-					const config* c = parent_stack.back()->child("add");
-					if (c != NULL) {
-						const config::child_map m = c->all_children();
-						for(config::child_map::const_iterator j = m.begin(); j != m.end(); ++j) {
-							for(config::child_list::const_iterator k = j->second.begin();
-									k != j->second.end(); ++k) {
-								res_cfgs_.back().add_child(j->first, **k);
-							}
-						}
+					foreach (const config::any_child &j, c.all_children_range()) {
+						res_cfgs_.back().add_child(j.key, j.cfg);
 					}
 				}
+
 				parent_stack.pop_back();
 			}
 		}
 		// Add all the resolutions
-		const config::child_list& res_list = top_cfg.get_children("resolution");
-		for(config::child_list::const_iterator j = res_list.begin(); j != res_list.end(); ++j) {
-			dst_cfg.add_child("resolution", (**j));
+		foreach (const config &res, top_cfg.child_range("resolution")) {
+			dst_cfg.add_child("resolution", res);
 		}
 		// Add all the resolved resolutions
 		for(std::vector<config>::const_iterator k = res_cfgs_.begin(); k != res_cfgs_.end(); ++k) {
@@ -232,16 +216,14 @@ static void expand_partialresolution(config& dst_cfg, const config& top_cfg)
 static void do_resolve_rects(const config& cfg, config& resolved_config, config* resol_cfg = NULL) {
 
 		// recursively resolve children
-		for(config::all_children_iterator i = cfg.ordered_begin(); i != cfg.ordered_end(); ++i) {
-			const std::pair<const std::string*,const config*>& value = *i;
-			config& childcfg = resolved_config.add_child(*value.first);
-			do_resolve_rects(*value.second, childcfg, (*value.first =="resolution") ? &childcfg : resol_cfg);
+		foreach (const config::any_child &value, cfg.all_children_range()) {
+			config &childcfg = resolved_config.add_child(value.key);
+			do_resolve_rects(value.cfg, childcfg,
+				value.key == "resolution" ? &childcfg : resol_cfg);
 		}
 
 		// copy all key/values
-		for(string_map::const_iterator j = cfg.values.begin(); j != cfg.values.end(); ++j) {
-			resolved_config.values[j->first] = j->second;
-		}
+		resolved_config.merge_attributes(cfg);
 
 		// override default reference rect with "ref" parameter if any
 		if (!cfg["ref"].empty()) {
@@ -263,7 +245,7 @@ static void do_resolve_rects(const config& cfg, config& resolved_config, config*
 		}
 		// resolve the rect value to absolute coordinates
 		if (!cfg["rect"].empty()) {
-			resolved_config.values["rect"] = resolve_rect(cfg["rect"]);
+			resolved_config["rect"] = resolve_rect(cfg["rect"]);
 		}
 	}
 
@@ -484,9 +466,8 @@ theme::status_item::status_item(const config& cfg) :
 	if(font_ == 0)
 		font_ = DefaultFontSize;
 
-	const config* const label_child = cfg.child("label");
-	if(label_child != NULL) {
-		label_ = label(*label_child);
+	if (const config &label_child = cfg.child("label")) {
+		label_ = label(label_child);
 	}
 
 	if(cfg["font_rgb"].size()){
@@ -588,7 +569,7 @@ bool theme::set_resolution(const SDL_Rect& screen)
 
 	if(current == resolutions.end()) {
 		if(!resolutions.empty()) {
-			LOG_STREAM(err, display) << "No valid resolution found\n";
+			ERR_DP << "No valid resolution found\n";
 		}
 		return false;
 	}
@@ -618,64 +599,56 @@ bool theme::set_resolution(const SDL_Rect& screen)
 	return result;
 }
 
-void theme::add_object(const config& cfg){
-
-	const config* const main_map_cfg = cfg.child("main_map");
-	if(main_map_cfg != NULL) {
-		main_map_ = object(*main_map_cfg);
+void theme::add_object(const config& cfg)
+{
+	if (const config &c = cfg.child("main_map")) {
+		main_map_ = object(c);
 	}
 
-	const config* const mini_map_cfg = cfg.child("mini_map");
-	if(mini_map_cfg != NULL) {
-		mini_map_ = object(*mini_map_cfg);
+	if (const config &c = cfg.child("mini_map")) {
+		mini_map_ = object(c);
 	}
 
-	const config* const status_cfg = cfg.child("status");
-	if(status_cfg != NULL) {
-		for(config::child_map::const_iterator i = status_cfg->all_children().begin(); i != status_cfg->all_children().end(); ++i) {
-			for(config::child_list::const_iterator j = i->second.begin(); j != i->second.end(); ++j) {
-				status_.insert(std::pair<std::string,status_item>(i->first,status_item(**j)));
-			}
+	if (const config &status_cfg = cfg.child("status"))
+	{
+		foreach (const config::any_child &i, status_cfg.all_children_range()) {
+			status_.insert(std::pair<std::string, status_item>(i.key, status_item(i.cfg)));
 		}
-		const config* const unit_image_cfg = status_cfg->child("unit_image");
-		if (unit_image_cfg != NULL) {
-			unit_image_ = object(*unit_image_cfg);
+		if (const config &unit_image_cfg = status_cfg.child("unit_image")) {
+			unit_image_ = object(unit_image_cfg);
 		} else {
 			unit_image_ = object();
 		}
 	}
 
-	const config::child_list& panel_list = cfg.get_children("panel");
-	for(config::child_list::const_iterator p = panel_list.begin(); p != panel_list.end(); ++p) {
-		panel new_panel(**p);
-		set_object_location(new_panel, (**p)["rect"], (**p)["ref"]);
+	foreach (const config &p, cfg.child_range("panel")) {
+		panel new_panel(p);
+		set_object_location(new_panel, p["rect"], p["ref"]);
 		panels_.push_back(new_panel);
 	}
 
-	const config::child_list& label_list = cfg.get_children("label");
-	for(config::child_list::const_iterator lb = label_list.begin(); lb != label_list.end(); ++lb) {
-		label new_label(**lb);
-		set_object_location(new_label, (**lb)["rect"], (**lb)["ref"]);
+	foreach (const config &lb, cfg.child_range("label")) {
+		label new_label(lb);
+		set_object_location(new_label, lb["rect"], lb["ref"]);
 		labels_.push_back(new_label);
 	}
 
-	const config::child_list& menu_list = cfg.get_children("menu");
-	for(config::child_list::const_iterator m = menu_list.begin(); m != menu_list.end(); ++m) {
-		menu new_menu(**m);
+	foreach (const config &m, cfg.child_range("menu"))
+	{
+		menu new_menu(m);
 		DBG_DP << "adding menu: " << (new_menu.is_context() ? "is context" : "not context") << "\n";
 		if(new_menu.is_context())
 			context_ = new_menu;
 		else{
-			set_object_location(new_menu, (**m)["rect"], (**m)["ref"]);
+			set_object_location(new_menu, m["rect"], m["ref"]);
 			menus_.push_back(new_menu);
 		}
 
 		DBG_DP << "done adding menu...\n";
 	}
 
-	const config* const border_cfg = cfg.child("main_map_border");
-	if (border_cfg != NULL) {
-		border_ = tborder(*border_cfg);
+	if (const config &c = cfg.child("main_map_border")) {
+		border_ = tborder(c);
 	} else {
 		border_ = tborder();
 	}
@@ -716,7 +689,8 @@ void theme::set_object_location(theme::object& element, std::string rect_str, st
 	}
 }
 
-void theme::modify(const config* cfg){
+void theme::modify(const config &cfg)
+{
 	std::map<std::string,std::string> title_stash;
 	std::vector<theme::menu>::iterator m;
 	for (m = menus_.begin(); m != menus_.end(); ++m) {
@@ -724,32 +698,26 @@ void theme::modify(const config* cfg){
 			title_stash[m->get_id()] = m->title();
 	}
 
+	// Change existing theme objects.
+	foreach (const config &c, cfg.child_range("change"))
 	{
-		// changes to existing theme objects
-		const config::child_list& c = cfg->get_children("change");
-		for(config::child_list::const_iterator j = c.begin(); j != c.end(); ++j) {
-			std::string id = (**j)["id"];
-			std::string ref_id = (**j)["ref"];
-			theme::object& element = find_element(id);
-			if (element.get_id() == id){
-				set_object_location(element, (**j)["rect"], ref_id);
-			}
-		}
+		std::string id = c["id"];
+		std::string ref_id = c["ref"];
+		theme::object &element = find_element(id);
+		if (element.get_id() == id)
+			set_object_location(element, c["rect"], ref_id);
 	}
-	// adding new theme objects
-	{
-		const config::child_list& c = cfg->get_children("add");
-		for(config::child_list::const_iterator j = c.begin(); j != c.end(); ++j) {
-			add_object(**j);
-		}
+
+	// Add new theme objects.
+	foreach (const config &c, cfg.child_range("add")) {
+		add_object(c);
 	}
-	// removing existent theme objects
-	{
-		const config::child_list& c = cfg->get_children("remove");
-		for(config::child_list::const_iterator j = c.begin(); j != c.end(); ++j) {
-			remove_object((**j)["id"]);
-		}
+
+	// Remove existent theme objects.
+	foreach (const config &c, cfg.child_range("remove")) {
+		remove_object(c["id"]);
 	}
+
 	for (m = menus_.begin(); m != menus_.end(); ++m) {
 		if (title_stash.find(m->get_id()) != title_stash.end())
 			m->set_title(title_stash[m->get_id()]);
@@ -784,31 +752,39 @@ const theme::status_item* theme::get_status_item(const std::string& key) const
 }
 
 std::map<std::string, config> theme::known_themes;
-void theme::set_known_themes(const config* cfg){
-        known_themes.clear();
-        if(cfg == NULL)
-	       return;
-	const config& v = *cfg;
-	const config::child_list& known_themes_cfg = v.get_children("theme");
+void theme::set_known_themes(const config* cfg)
+{
+	known_themes.clear();
+	if (!cfg)
+		return;
 
-	for(config::child_list::const_iterator thm = known_themes_cfg.begin(); thm != known_themes_cfg.end(); ++thm) {
-	       std::string thm_name=(**thm)["name"];
-	       if(thm_name!="null" && thm_name!="editor"){
-	              known_themes[thm_name]=(**thm);
-	       }
+	foreach (const config &thm, cfg->child_range("theme"))
+	{
+		std::string thm_name = thm["name"];
+		if (thm_name != "null" && thm_name != "editor")
+			known_themes[thm_name] = thm;
 	}
 }
 
 std::vector<std::string> theme::get_known_themes(){
-        std::vector<std::string> names;
+    std::vector<std::string> names;
 
 
-        for(std::map<std::string, config>::iterator p_thm=known_themes.begin();p_thm!=known_themes.end();p_thm++){
-	  names.push_back(p_thm->first);
-	}
-        return(names);
+    for(std::map<std::string, config>::iterator p_thm=known_themes.begin();p_thm!=known_themes.end();++p_thm){
+        names.push_back(p_thm->first);
+    }
+    return(names);
 }
 
+const theme::menu *theme::get_menu_item(const std::string &key) const
+{
+	foreach (const theme::menu &m, menus_) {
+		if (m.get_id() == key) return &m;
+	}
+	return NULL;
+}
+
+
 theme::menu* theme::refresh_title(const std::string& id, const std::string& new_title){
 	theme::menu* res = NULL;
 
@@ -823,7 +799,7 @@ theme::menu* theme::refresh_title(const std::string& id, const std::string& new_
 }
 
 theme::menu* theme::refresh_title2(const std::string& id, const std::string& title_tag){
-	std::string new_title = "";
+	std::string new_title;
 
 	config& cfg = find_ref(id, cfg_, false);
 	if (! cfg[title_tag].empty())
@@ -831,3 +807,13 @@ theme::menu* theme::refresh_title2(const std::string& id, const std::string& tit
 
 	return refresh_title(id, new_title);
 }
+
+void theme::modify_label(const std::string& id, const std::string& text)
+{
+	theme::label *label = dynamic_cast<theme::label *>(&find_element(id));
+	if (!label) {
+		ERR_DP << "Theme contains no label called '" << id << "'.\n";
+		return;
+	}
+	label->set_text(text);
+}
