@@ -1,6 +1,6 @@
-/* $Id: thread.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: thread.cpp 41760 2010-03-26 00:13:09Z ilor $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,34 +13,23 @@
 */
 
 #include "global.hpp"
+
+#include <vector>
+
 #include "log.hpp"
 #include "thread.hpp"
 
-#include <new>
-#include <iostream>
-#include <memory>
-#include <vector>
 
-#define ERR_G LOG_STREAM(err, general)
+#define ERR_G LOG_STREAM(err, lg::general)
+
 
 static int run_async_operation(void* data)
 {
-	threading::async_operation* const op = reinterpret_cast<threading::async_operation*>(data);
+	threading::async_operation_ptr op(*reinterpret_cast<threading::async_operation_ptr*>(data));
 	op->run();
 
-	bool should_delete = false;
-	{
-		const threading::lock l(op->get_mutex());
-		op->notify_finished(); //in case the operation didn't notify of finishing
-		if(op->is_aborted()) {
-			should_delete = true;
-		}
-	}
-
-	if(should_delete) {
-		delete op;
-	}
-
+	const threading::lock l(op->get_mutex());
+	op->notify_finished(); //in case the operation didn't notify of finishing
 
 	return 0;
 }
@@ -126,7 +115,7 @@ condition::WAIT_TIMEOUT_RESULT condition::wait_timeout(const mutex& m, unsigned
 	const int res = SDL_CondWaitTimeout(cond_,m.m_,timeout);
 	switch(res) {
 		case 0: return WAIT_OK;
-		case SDL_MUTEX_TIMEDOUT: return WAIT_TIMEOUT;
+		case SDL_MUTEX_TIMEDOUT: return WAIT_TIMED_OUT;
 		default:
 			ERR_G << "SDL_CondWaitTimeout: " << SDL_GetError() << "\n";
 			return WAIT_ERROR;
@@ -157,8 +146,9 @@ bool async_operation::notify_finished()
 	finishedVar_ = true;
 	return finished_.notify_one();
 }
+active_operation_list async_operation::active_;
 
-async_operation::RESULT async_operation::execute(waiter& wait)
+async_operation::RESULT async_operation::execute(async_operation_ptr this_ptr, waiter& wait)
 {
 	//the thread must be created after the lock, and also destroyed after it.
 	//this is because during the thread's execution, we must always hold the mutex
@@ -167,10 +157,10 @@ async_operation::RESULT async_operation::execute(waiter& wait)
 	//
 	//we cannot hold the mutex while waiting for the thread to join though, because
 	//the thread needs access to the mutex before it terminates
-	std::auto_ptr<thread> t(NULL);
 	{
 		const lock l(get_mutex());
-		t = std::auto_ptr<thread>(new thread(run_async_operation,this));
+		active_.push_back(this_ptr);
+		thread_.reset(new thread(run_async_operation,&this_ptr));
 
 		bool completed = false;
 		while(wait.process() == waiter::WAIT) {
@@ -188,7 +178,6 @@ async_operation::RESULT async_operation::execute(waiter& wait)
 
 		if(!completed) {
 			aborted_ = true;
-			t->detach();
 			return ABORTED;
 		}
 	}
