@@ -1,6 +1,6 @@
-/* $Id: thread.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: thread.hpp 41760 2010-03-26 00:13:09Z ilor $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -156,7 +156,7 @@ public:
 	//
 	bool wait(const mutex& m);
 
-	enum WAIT_TIMEOUT_RESULT { WAIT_OK, WAIT_TIMEOUT, WAIT_ERROR };
+	enum WAIT_TIMEOUT_RESULT { WAIT_OK, WAIT_TIMED_OUT, WAIT_ERROR };
 
 	// wait on the condition with a timeout. Basically the same as the
 	// wait() function, but if the lock is not aquired before the
