@@ -1,6 +1,6 @@
-/* $Id: titlescreen.cpp 33383 2009-03-07 14:53:43Z ilor $ */
+/* $Id: titlescreen.cpp 45866 2010-08-18 18:25:25Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -23,9 +23,13 @@
 
 #include "global.hpp"
 
+#include <algorithm>
+#include <vector>
+
 #include "config.hpp"
 #include "construct_dialog.hpp"
 #include "cursor.hpp"
+#include "foreach.hpp"
 #include "game_display.hpp"
 #include "game_preferences.hpp"
 #include "events.hpp"
@@ -39,21 +43,22 @@
 #include "preferences_display.hpp"
 #include "sdl_utils.hpp"
 #include "show_dialog.hpp"
+#include "text.hpp"
 #include "titlescreen.hpp"
 #include "video.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
-#include <algorithm>
-#include <vector>
 
-#include "SDL_ttf.h"
+static lg::log_domain log_engine("engine");
+#define ERR_NG LOG_STREAM(err, log_engine)
 
-/** Log info-messages to stdout during the game, mainly for debugging */
-#define LOG_DP LOG_STREAM(info, display)
-/** Log error-messages to stdout during the game, mainly for debugging */
-#define ERR_DP LOG_STREAM(err, display)
-#define LOG_CONFIG LOG_STREAM(info, config)
-#define ERR_CONFIG LOG_STREAM(err, config)
+static lg::log_domain log_display("display");
+#define LOG_DP LOG_STREAM(info, log_display)
+#define ERR_DP LOG_STREAM(err, log_display)
+
+static lg::log_domain log_config("config");
+#define LOG_CF LOG_STREAM(info, log_config)
+#define ERR_CF LOG_STREAM(err, log_config)
 
 /**
  *  Fade-in the wesnoth-logo.
@@ -62,25 +67,34 @@
  *  Used only once, after the game is started.
  *
  *  @param	screen	surface to operate on
- *  @param	xpos	x-position of logo
- *  @param	ypos	y-position of logo
+ *  @param	xcpos	x-position of center of the logo
+ *  @param	ycpos	y-position of center of the logo
  *
  *  @return		Result of running the routine
  *  @retval true	operation finished (successful or not)
  *  @retval false	operation failed (because modeChanged), need to retry
  */
-static bool fade_logo(game_display& screen, int xpos, int ypos)
+static bool fade_logo(game_display& screen, int xcpos, int ycpos)
 {
-	const surface logo(image::get_image(game_config::game_logo));
-	if(logo == NULL) {
+	surface const fb = screen.video().getSurface();
+	if (!fb) return true;
+
+	surface logo = image::get_image(game_config::game_logo);
+	if (!logo) {
 		ERR_DP << "Could not find game logo\n";
 		return true;
 	}
 
-	surface const fb = screen.video().getSurface();
+	int xpos = xcpos - logo->w / 2;
+	int ypos = ycpos - logo->h / 2;
 
-	if(fb == NULL || xpos < 0 || ypos < 0 || xpos + logo->w > fb->w || ypos + logo->h > fb->h) {
-		return true;
+	if (xpos < 0 || ypos < 0 || xpos + logo->w > fb->w || ypos + logo->h > fb->h) {
+		double scale = 2 * std::min(
+			std::min((double)xcpos / logo->w, (double)(fb->w - ycpos) / logo->w),
+			std::min((double)ycpos / logo->h, (double)(fb->h - ycpos) / logo->h));
+		logo = scale_surface(logo, logo->w * scale, logo->h * scale);
+		xpos = xcpos - logo->w / 2;
+		ypos = ycpos - logo->h / 2;
 	}
 
 	// Only once, when the game is first started, the logo fades in unless
@@ -131,77 +145,77 @@ static bool fade_logo(game_display& screen, int xpos, int ypos)
 
 
 /** Read the file with the tips-of-the-day. */
-static void read_tips_of_day(config& tips_of_day)
+void read_tips_of_day(config& tips_of_day)
 {
 	tips_of_day.clear();
-	LOG_CONFIG << "Loading tips of day\n";
+	LOG_CF << "Loading tips of day\n";
 	try {
 		scoped_istream stream = preprocess_file(get_wml_location("hardwired/tips.cfg"));
 		read(tips_of_day, *stream);
 	} catch(config::error&) {
-		ERR_CONFIG << "Could not read data/hardwired/tips.cfg\n";
+		ERR_CF << "Could not read data/hardwired/tips.cfg\n";
 	}
 
-	//we shuffle the tips after each initial loading. We only shuffle if
-	//the upload_log preference has been set. If it hasn't been set, it's the
-	//user's first time playing since this feature has been added, so we'll
-	//leave the tips in their default order, which will always contain a tip
-	//regarding the upload log first, so the user sees it.
-	config::child_itors tips = tips_of_day.child_range("tip");
-	if (tips.first != tips.second && preferences::has_upload_log()) {
-		std::random_shuffle(tips.first, tips.second);
-	}
-
-	//Make sure that the upload log preference is set, if it's not already, so
-	//that we know next time we've already seen the message about uploads.
-	if(!preferences::has_upload_log()) {
-		preferences::set_upload_log(preferences::upload_log());
+	//We shuffle the tips after each initial loading.
+	config::const_child_itors itors = tips_of_day.child_range("tip");
+	if (itors.first != itors.second ) {
+		std::vector<config> tips(itors.first, itors.second);
+		std::random_shuffle(tips.begin(), tips.end());
+		tips_of_day.clear();
+		foreach (const config &tip, tips) {
+			tips_of_day.add_child("tip", tip);
+		}
 	}
 }
 
 /** Go to the next tips-of-the-day */
-static void next_tip_of_day(config& tips_of_day, bool reverse = false)
+void next_tip_of_day(config& tips_of_day, bool reverse)
 {
 	// we just rotate the tip list, to avoid the need to keep track
 	// of the current one, and keep it valid, cycle it, etc...
-	config::child_itors tips = tips_of_day.child_range("tip");
-	if (tips.first != tips.second) {
-		config::child_iterator direction = reverse ? tips.first+1 : tips.second-1;
-		std::rotate(tips.first, direction, tips.second);
+	config::const_child_itors itors = tips_of_day.child_range("tip");
+	if (itors.first != itors.second) {
+		std::vector<config> tips(itors.first, itors.second);
+		std::vector<config>::iterator direction =
+			reverse ? tips.begin() + 1 : tips.end() - 1;
+		std::rotate(tips.begin(), direction, tips.end());
+		tips_of_day.clear();
+		foreach (const config &tip, tips) {
+			tips_of_day.add_child("tip", tip);
+		}
 	}
 }
 
-/** Return the text for one of the tips-of-the-day. */
-static const config* get_tip_of_day(config& tips_of_day)
+const config* get_tip_of_day(config& tips_of_day)
 {
 	if (tips_of_day.empty()) {
 		read_tips_of_day(tips_of_day);
 	}
 
-	const config::child_list& tips = tips_of_day.get_children("tip");
-
 	// next_tip_of_day rotate tips, so better stay iterator-safe
-	for (size_t t=0; t < tips.size(); t++, next_tip_of_day(tips_of_day)) {
-		const config* tip = tips.front();
-		if (tip == NULL) continue;
+	for (int nb_tips = tips_of_day.child_count("tip"); nb_tips > 0;
+	     --nb_tips, next_tip_of_day(tips_of_day))
+	{
+		const config &tip = tips_of_day.child("tip");
+		assert(tip);
 
-		const std::vector<std::string> needed_units = utils::split((*tip)["encountered_units"],',');
+		const std::vector<std::string> needed_units = utils::split(tip["encountered_units"], ',');
 		if (needed_units.empty()) {
-			return tip;
+			return &tip;
 		}
 		const std::set<std::string>& seen_units = preferences::encountered_units();
 
 		// test if one of the listed unit types is already encountered
 		// if if's a number, test if we have encountered more than this
 		for (std::vector<std::string>::const_iterator i = needed_units.begin();
-				i != needed_units.end(); i++) {
+				i != needed_units.end(); ++i) {
 			int needed_units_nb = lexical_cast_default<int>(*i,-1);
 			if (needed_units_nb !=-1) {
 				if (needed_units_nb <= static_cast<int>(seen_units.size())) {
-					return tip;
+					return &tip;
 				}
 			} else if (seen_units.find(*i) != seen_units.end()) {
-				return tip;
+				return &tip;
 			}
 		}
 	}
@@ -222,73 +236,67 @@ static void draw_tip_of_day(game_display& screen,
 							const SDL_Rect* const main_dialog_area,
 							surface_restorer& tip_of_day_restorer)
 {
-	if(preferences::show_tip_of_day() == false) {
-		return;
-	}
+	// Restore the previous tip of day area to its old state (section of the title image).
+	tip_of_day_restorer.restore();
 
-    // Restore the previous tip of day area to its old state (section of the title image).
-    tip_of_day_restorer.restore();
+	// Draw tip of the day
+	const config* tip = get_tip_of_day(tips_of_day);
+	if (!tip) return;
+	int tip_width = game_config::title_tip_width * screen.w() / 1024;
 
-    // Draw tip of the day
-    const config* tip = get_tip_of_day(tips_of_day);
-    if(tip != NULL) {
-    	int tip_width = game_config::title_tip_width * screen.w() / 1024;
+	font::ttext text, source;
+	text.set_text((*tip)["text"], false);
+	text.set_maximum_width(tip_width);
+	text.set_maximum_height(main_dialog_area->h);
+	source.set_text((*tip)["source"], false);
+	source.set_font_style(font::ttext::STYLE_ITALIC);
+	source.set_maximum_width(tip_width);
+	source.set_alignment(PANGO_ALIGN_RIGHT);
 
-		try {
-	        const std::string& text =
-				font::word_wrap_text((*tip)["text"], font::SIZE_NORMAL, tip_width);
-			const std::string& source =
-				font::word_wrap_text((*tip)["source"], font::SIZE_NORMAL, tip_width);
+	int pad = game_config::title_tip_padding;
 
-			const int pad = game_config::title_tip_padding;
+	SDL_Rect area, source_area;
+	area.w = tip_width;
+	area.h = text.get_height();
+	source_area.w = source.get_width();
+	source_area.h = source.get_height();
+	area.w = std::max<size_t>(area.w, source_area.w) + 2 * pad;
+	area.h += source_area.h + next_tip_button->location().h + 3 * pad;
 
-			SDL_Rect area = font::text_area(text,font::SIZE_NORMAL);
-			area.w = tip_width;
-			SDL_Rect source_area = font::text_area(source, font::SIZE_NORMAL, TTF_STYLE_ITALIC);
-			area.w = std::max<size_t>(area.w, source_area.w) + 2*pad;
-			area.h += source_area.h + next_tip_button->location().h + 3*pad;
+	area.x = main_dialog_area->x - (game_config::title_tip_x * screen.w() / 1024) - area.w;
+	area.y = main_dialog_area->y + main_dialog_area->h - area.h;
 
-			area.x = main_dialog_area->x - (game_config::title_tip_x * screen.w() / 1024) - area.w;
-			area.y = main_dialog_area->y + main_dialog_area->h - area.h;
+	// Note: The buttons' locations need to be set before the dialog frame is drawn.
+	// Otherwise, when the buttons restore their area, they
+	// draw parts of the old dialog frame at their old locations.
+	// This way, the buttons draw a part of the title image,
+	// because the call to restore above restored the area
+	// of the old tip of the day to its initial state (the title image).
+	int button_x = area.x + area.w - next_tip_button->location().w - pad;
+	int button_y = area.y + area.h - pad - next_tip_button->location().h;
+	next_tip_button->set_location(button_x, button_y);
+	next_tip_button->set_dirty(); //force redraw even if location did not change.
 
-			// Note: The buttons' locations need to be set before the dialog frame is drawn.
-			// Otherwise, when the buttons restore their area, they
-			// draw parts of the old dialog frame at their old locations.
-			// This way, the buttons draw a part of the title image,
-			// because the call to restore above restored the area
-			// of the old tip of the day to its initial state (the title image).
-			int button_x = area.x + area.w - next_tip_button->location().w - pad;
-			int button_y = area.y + area.h - pad - next_tip_button->location().h;
-			next_tip_button->set_location(button_x, button_y);
-			next_tip_button->set_dirty(); //force redraw even if location did not change.
+	button_x -= previous_tip_button->location().w + pad;
+	previous_tip_button->set_location(button_x, button_y);
+	previous_tip_button->set_dirty();
 
-			button_x -= previous_tip_button->location().w + pad;
-			previous_tip_button->set_location(button_x, button_y);
-			previous_tip_button->set_dirty();
+	button_x = area.x + pad;
+	help_tip_button->set_location(button_x, button_y);
+	help_tip_button->set_dirty();
 
-			button_x = area.x + pad;
-			help_tip_button->set_location(button_x, button_y);
-			help_tip_button->set_dirty();
+	gui::dialog_frame f(screen.video(), "", style, false);
+	tip_of_day_restorer = surface_restorer(&screen.video(), f.layout(area).exterior);
+	f.draw_background();
+	f.draw_border();
 
-			gui::dialog_frame f(screen.video(), "", style, false);
-			tip_of_day_restorer = surface_restorer(&screen.video(), f.layout(area).exterior);
-			f.draw_background();
-			f.draw_border();
+	surface text_s = text.render();
+	screen.video().blit_surface(area.x + pad, area.y + pad, text_s);
+	surface source_s = source.render();
+	screen.video().blit_surface(area.x + pad,
+		next_tip_button->location().y - source_area.h - pad, source_s);
 
-			font::draw_text(&screen.video(), area, font::SIZE_NORMAL, font::NORMAL_COLOUR,
-							 text, area.x + pad, area.y + pad);
-			// todo
-			font::draw_text(&screen.video(), area, font::SIZE_NORMAL, font::NORMAL_COLOUR,
-							 source, area.x + area.w - source_area.w - pad,
-							 next_tip_button->location().y - source_area.h - pad,
-							 false, TTF_STYLE_ITALIC);
-		} catch (utils::invalid_utf8_exception&) {
-			LOG_STREAM(err, engine) << "Invalid utf-8 found, tips of day aren't drawn.\n";
-			return;
-		}
-
-	    LOG_DP << "drew tip of day\n";
-	}
+	LOG_DP << "drew tip of day\n";
 }
 
 /**
@@ -307,7 +315,7 @@ static void draw_background(game_display& screen)
 			utils::split(game_config::game_title, ',', utils::STRIP_SPACES | utils::REMOVE_EMPTY);
 
 		if(game_title_list.empty()) {
-			ERR_CONFIG << "No title image defined\n";
+			ERR_CF << "No title image defined\n";
 		} else {
 			surface const title_surface(scale_opaque_surface(
 				image::get_image(game_title_list[rand()%game_title_list.size()]),
@@ -364,8 +372,7 @@ public:
 		}
 	}
 
-	bool get_esc_ignore() { return ignore_esc_; }
-	void set_esc_ignore(bool ignore) { ignore_esc_ = ignore; }
+	bool get_esc_ignore() const { return ignore_esc_; }
 
 	virtual void handle_event(const SDL_Event& event)
 	{
@@ -408,41 +415,42 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 	//- Texts for the menu-buttons.
 	//- Members of this array must correspond to the enumeration TITLE_RESULT
 	static const char* button_labels[] = {
-					       N_("TitleScreen button^Tutorial"),
-					       N_("TitleScreen button^Campaign"),
-					       N_("TitleScreen button^Multiplayer"),
-					       N_("TitleScreen button^Load"),
-					       N_("TitleScreen button^Add-ons"),
-#ifndef DISABLE_EDITOR2
-					       N_("TitleScreen button^Map Editor"),
+		N_("TitleScreen button^Tutorial"),
+		N_("TitleScreen button^Campaign"),
+		N_("TitleScreen button^Multiplayer"),
+		N_("TitleScreen button^Load"),
+		N_("TitleScreen button^Add-ons"),
+#ifndef DISABLE_EDITOR
+		N_("TitleScreen button^Map Editor"),
 #endif
-					       N_("TitleScreen button^Language"),
-					       N_("TitleScreen button^Preferences"),
-					       N_("TitleScreen button^Credits"),
-					       N_("TitleScreen button^Quit"),
-								// Only the above buttons go into the menu-frame
-								// Next 2 buttons go into frame for the tip-of-the-day:
-					       N_("TitleScreen button^Previous"),
-					       N_("TitleScreen button^Next"),
-					       N_("TitleScreen button^Help"),
-								// Next entry is no button, but shown as a mail-icon instead:
-					       N_("TitleScreen button^Help Wesnoth") };
+		N_("TitleScreen button^Language"),
+		N_("TitleScreen button^Preferences"),
+		N_("TitleScreen button^Credits"),
+		N_("TitleScreen button^Quit"),
+		// Only the above buttons go into the menu-frame
+		// Next 2 buttons go into frame for the tip-of-the-day:
+		N_("TitleScreen button^Previous"),
+		N_("TitleScreen button^Next"),
+		N_("TitleScreen button^Help"),
+	};
 	//- Texts for the tooltips of the menu-buttons
-	static const char* help_button_labels[] = { N_("Start a tutorial to familiarize yourself with the game"),
-						    N_("Start a new single player campaign"),
-						    N_("Play multiplayer (hotseat, LAN, or Internet), or a single scenario against the AI"),
-						    N_("Load a saved game"),
-						    N_("Download usermade campaigns, eras, or map packs"),
-#ifndef DISABLE_EDITOR2
-						    N_("Start the map editor"),
+	static const char* help_button_labels[] = {
+		N_("Start a tutorial to familiarize yourself with the game"),
+		N_("Start a new single player campaign"),
+		N_("Play multiplayer (hotseat, LAN, or Internet), or a single scenario against the AI"),
+		N_("Load a saved game"),
+		N_("Download usermade campaigns, eras, or map packs"),
+#ifndef DISABLE_EDITOR
+		N_("Start the map editor"),
 #endif
-						    N_("Change the language"),
-						    N_("Configure the game's settings"),
-						    N_("View the credits"),
-						    N_("Quit the game"),
-						    N_("Show next tip of the day"),
-						    N_("Show Battle for Wesnoth help"),
-						    N_("Upload statistics") };
+		N_("Change the language"),
+		N_("Configure the game's settings"),
+		N_("View the credits"),
+		N_("Quit the game"),
+		N_("Show previous tip of the day"),
+		N_("Show next tip of the day"),
+		N_("Show Battle for Wesnoth help")
+	};
 
 	static const size_t nbuttons = sizeof(button_labels)/sizeof(*button_labels);
 	const int menu_xbase = (game_config::title_buttons_x*screen.w())/1024;
@@ -461,13 +469,18 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 	std::vector<button> buttons;
 	size_t b, max_width = 0;
 	size_t n_menubuttons = 0;
-	for(b = 0; b != nbuttons; ++b) {
-		buttons.push_back(button(screen.video(),sgettext(button_labels[b])));
+	for (b = 0; b != nbuttons; ++b)
+	{
+		std::string label = sgettext(button_labels[b]);
+		if (b + TUTORIAL <= QUIT_GAME) {
+			buttons.push_back(button(screen.video(), label));
+			max_width = std::max<size_t>(max_width,buttons.back().width	());
+			n_menubuttons = b;
+		} else {
+			buttons.push_back(button(screen.video(), label,
+				button::TYPE_PRESS, "lite_small"));
+		}
 		buttons.back().set_help_string(sgettext(help_button_labels[b]));
-		max_width = std::max<size_t>(max_width,buttons.back().width());
-
-		n_menubuttons = b;
-		if(b + TUTORIAL == QUIT_GAME) break;	// Menu-frame ends at the quit-button
 	}
 
 	SDL_Rect main_dialog_area = {menu_xbase-padding, menu_ybase-padding, max_width+padding*2,
@@ -489,20 +502,9 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 		if(b + TUTORIAL == QUIT_GAME) break;
 	}
 
-	b = TIP_PREVIOUS - TUTORIAL;
-	gui::button previous_tip_button(screen.video(),sgettext(button_labels[b]),button::TYPE_PRESS,"lite_small");
-	previous_tip_button.set_help_string( sgettext(button_labels[b] ));
-
-	b = TIP_NEXT - TUTORIAL;
-	gui::button next_tip_button(screen.video(),sgettext(button_labels[b]),button::TYPE_PRESS,"lite_small");
-	next_tip_button.set_help_string( sgettext(button_labels[b] ));
-
-	b = SHOW_HELP - TUTORIAL;
-	gui::button help_tip_button(screen.video(),sgettext(button_labels[b]),button::TYPE_PRESS,"lite_small");
-	help_tip_button.set_help_string( sgettext(button_labels[b] ));
-
-	gui::button beg_button(screen.video(),_("Help Wesnoth"),button::TYPE_IMAGE,"menu-button",button::MINIMUM_SPACE);
-	beg_button.set_help_string(_("Help Wesnoth by sending us information"));
+	gui::button &previous_tip_button = buttons[TIP_PREVIOUS - TUTORIAL];
+	gui::button &next_tip_button = buttons[TIP_NEXT - TUTORIAL];
+	gui::button &help_tip_button = buttons[SHOW_HELP - TUTORIAL];
 
 	next_tip_of_day(tips_of_day);
 
@@ -511,9 +513,6 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 	draw_tip_of_day(screen, tips_of_day, gui::dialog_frame::titlescreen_style,
 					&previous_tip_button, &next_tip_button, &help_tip_button, &main_dialog_area, tip_of_day_restorer);
 
-	const int pad = game_config::title_tip_padding;
-	beg_button.set_location(screen.w() - pad - beg_button.location().w,
-		screen.h() - pad - beg_button.location().h);
 	events::raise_draw_event();
 
 	LOG_DP << "drew buttons dialog\n";
@@ -533,17 +532,18 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 	for(;;) {
 		events::pump();
 		for(size_t b = 0; b != buttons.size(); ++b) {
-			if(buttons[b].pressed()) {
+			if (b != TIP_PREVIOUS - TUTORIAL && b != TIP_NEXT - TUTORIAL &&
+			    buttons[b].pressed()) {
 				return static_cast<TITLE_RESULT>(b + TUTORIAL);
 			}
 		}
 
-		if(previous_tip_button.pressed()) {
+		if(previous_tip_button.pressed() || (key[SDLK_RETURN] && keyboard_button == TIP_PREVIOUS - TUTORIAL)) {
 			next_tip_of_day(tips_of_day, true);
 			draw_tip_of_day(screen, tips_of_day, gui::dialog_frame::titlescreen_style,
 						&previous_tip_button, &next_tip_button, &help_tip_button, &main_dialog_area, tip_of_day_restorer);
 		}
-		if(next_tip_button.pressed()) {
+		if(next_tip_button.pressed() || (key[SDLK_RETURN] && keyboard_button == TIP_NEXT - TUTORIAL)) {
 			next_tip_of_day(tips_of_day, false);
 			draw_tip_of_day(screen, tips_of_day, gui::dialog_frame::titlescreen_style,
 						&previous_tip_button, &next_tip_button, &help_tip_button, &main_dialog_area, tip_of_day_restorer);
@@ -552,27 +552,26 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 		if(help_tip_button.pressed()) {
 			return SHOW_HELP;
 		}
-		if(beg_button.pressed()) {
-			return BEG_FOR_UPLOAD;
-		}
 		if (key[SDLK_UP]) {
 			if (!key_processed) {
-				buttons[keyboard_button].set_active(false);
+				if (keyboard_button < nbuttons)
+					buttons[keyboard_button].set_active(false);
 				if (keyboard_button == 0) {
 					keyboard_button = nbuttons - 1;
 				} else {
-					keyboard_button--;
+					--keyboard_button;
 				}
 				key_processed = true;
 				buttons[keyboard_button].set_active(true);
 			}
 		} else if (key[SDLK_DOWN]) {
 			if (!key_processed) {
-				buttons[keyboard_button].set_active(false);
-				if (keyboard_button > nbuttons - 1) {
+				if (keyboard_button < nbuttons)
+					buttons[keyboard_button].set_active(false);
+				if (keyboard_button >= nbuttons - 1) {
 					keyboard_button = 0;
 				} else {
-					keyboard_button++;
+					++keyboard_button;
 				}
 				key_processed = true;
 				buttons[keyboard_button].set_active(true);
@@ -591,7 +590,9 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 		if (key[SDLK_F5])
 			return RELOAD_GAME_DATA;
 
-		if (key[SDLK_RETURN] && keyboard_button < nbuttons) {
+		if (key[SDLK_RETURN] && keyboard_button < nbuttons
+		&& keyboard_button != TIP_PREVIOUS - TUTORIAL
+		&& keyboard_button != TIP_NEXT - TUTORIAL) {
 			return static_cast<TITLE_RESULT>(keyboard_button + TUTORIAL);
 		}
 
