@@ -1,6 +1,6 @@
-/* $Id: exploder_cutter.cpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: exploder_cutter.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2004 - 2009 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2010 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -16,8 +16,11 @@
 #include "filesystem.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
+#include "serialization/string_utils.hpp"
 #include "SDL_image.h"
 
+#include <iostream>
+
 cutter::cutter()
 	: masks_()
 	, verbose_(false)
