@@ -1,6 +1,6 @@
-/* $Id: exploder_utils.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: exploder_utils.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2010 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -18,22 +18,24 @@
 #include <png.h>
 
 exploder_point::exploder_point(const std::string &s)
+	: x(0)
+	, y(0)
 {
 	std::vector<std::string> items = utils::split(s);
-	if(items.size() != 2) {
-		x = y = 0;
-	} else {
+	if(items.size() == 2) {
 		x = atoi(items[0].c_str());
 		y = atoi(items[1].c_str());
 	}
 }
 
 exploder_rect::exploder_rect(const std::string &s)
+	: x(0)
+	, y(0)
+	, w(0)
+	, h(0)
 {
 	std::vector<std::string> items = utils::split(s);
-	if(items.size() != 4) {
-		x = y = w = h = 0;
-	} else {
+	if(items.size() == 4) {
 		x = atoi(items[0].c_str());
 		y = atoi(items[1].c_str());
 		w = atoi(items[2].c_str());
@@ -95,7 +97,7 @@ void masked_overwrite_surface(surface dest, surface src, surface mask, int x, in
 		src_beg += (-y) * src->w;
 		mask_beg += (-y) * mask->w;
 		dest_beg += x;
-		if (src_height < (size_t)-y)
+		if (src_height < static_cast<size_t>(-y))
 			return;
 		src_height += y;
 		y = 0;
@@ -172,7 +174,7 @@ void save_image(surface surf, const std::string &filename)
 	//TODO: review whether providing NULL error handlers is something
 	//sensible
 	png_struct* png_ptr = png_create_write_struct
-		(PNG_LIBPNG_VER_STRING, (png_voidp)png_voidp_NULL,
+		(PNG_LIBPNG_VER_STRING, reinterpret_cast<png_voidp>(png_voidp_NULL),
 		 png_error_ptr_NULL, png_error_ptr_NULL);
 	if(!png_ptr)
 		throw exploder_failure("Unable to initialize the png write structure");
@@ -180,7 +182,7 @@ void save_image(surface surf, const std::string &filename)
 	png_info* info_ptr = png_create_info_struct(png_ptr);
 	if(!info_ptr) {
 		png_destroy_write_struct(&png_ptr,
-				(png_infopp)NULL);
+				static_cast<png_infopp>(NULL));
 		throw exploder_failure("Unable to initialize the png info structure");
 	}
 
@@ -208,7 +210,7 @@ void save_image(surface surf, const std::string &filename)
 	Uint32 *surf_data = lock.pixels();
 	int pos = 0;
 	for(int y = 0; y < surf->h; ++y) {
-		row_pointers[y] = (png_byte*) (rgba_data + pos);
+		row_pointers[y] = reinterpret_cast<png_byte*>(rgba_data + pos);
 		for(int x = 0; x < surf->w; ++x) {
 			Uint8 red, green, blue, alpha;
 			SDL_GetRGBA(*surf_data, surf->format, &red, &green, &blue, &alpha);
