@@ -1,6 +1,6 @@
-/* $Id: unit.hpp 33636 2009-03-14 14:43:04Z dragonking $ */
+/* $Id: unit.hpp 42880 2010-05-25 21:20:10Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -22,19 +22,18 @@
 #include "map_location.hpp"
 #include "portrait.hpp"
 #include "race.hpp"
-#include "team.hpp"
 #include "unit_types.hpp"
 #include "unit_map.hpp"
-#include "variable.hpp"
 
 class game_display;
-class gamestatus;
 class game_state;
-class config_writer;
+class vconfig;
+class team;
+class unit_type;
 
-#include <set>
 #include <string>
 #include <vector>
+#include <list>
 
 class unit_ability_list
 {
@@ -47,9 +46,8 @@ public:
 	bool empty() const;
 
 	std::pair<int,map_location> highest(const std::string& key, int def=0) const;
-	std::pair<int,map_location> lowest(const std::string& key, int def=100) const;
 
-	std::vector<std::pair<config*,map_location> > cfgs;
+	std::vector<std::pair<const config *, map_location> > cfgs;
 };
 
 
@@ -66,21 +64,22 @@ public:
 	// Copy constructor
 	unit(const unit& u);
 	/** Initilizes a unit from a config */
-	unit(const config& cfg, bool use_traits=false);
-	unit(unit_map* unitmap, const gamemap* map,
-		const gamestatus* game_status, const std::vector<team>* teams,
+	unit(unit_map* unitmap,
 		const config& cfg, bool use_traits=false, game_state* state = 0);
-	/** Initializes a unit from a unit type */
-	unit(const unit_type* t, int side, bool use_traits=false, bool dummy_unit=false, unit_race::GENDER gender=unit_race::MALE, std::string variation="");
-	unit(unit_map* unitmap, const gamemap* map, const gamestatus* game_status, const std::vector<team>* teams, const unit_type* t, int side, bool use_traits=false, bool dummy_unit=false, unit_race::GENDER gender=unit_race::MALE, std::string variation="", bool force_gender=false);
+	/**
+	  * Initializes a unit from a unit type
+	  * only real_unit may have random traits, name and gender
+	  * (to prevent OOS caused by RNG calls)
+	  */
+	unit(unit_map* unitmap, const unit_type* t, int side, bool real_unit, unit_race::GENDER gender=unit_race::NUM_GENDERS);
 	virtual ~unit();
 	unit& operator=(const unit&);
 
-	void set_game_context(unit_map* unitmap, const gamemap* map, const gamestatus* game_status, const std::vector<team>* teams);
+	void set_game_context(unit_map* unitmap);
 
 	/** Advances this unit to another type */
 	void advance_to(const unit_type* t, bool use_traits=false, game_state* state = 0);
-	const std::vector<std::string> advances_to() const { return advances_to_; }
+	const std::vector<std::string>& advances_to() const { return advances_to_; }
 
 	/** The type id of the unit */
 	const std::string& type_id() const { return type_; }
@@ -96,11 +95,14 @@ public:
 	const std::string& undead_variation() const {return undead_variation_;}
 
 	/** The unit name for display */
-	const std::string& name() const {return (name_);}
+	const t_string &name() const { return name_; }
+	void set_name(const t_string &name) { name_ = name; }
 	void rename(const std::string& name) {if (!unrenamable_) name_= name;}
 
 	/** The unit's profile */
 	const std::string& profile() const;
+	/** The unit's transparent profile */
+	std::string transparent() const;
 	/** Information about the unit -- a detailed description of it */
 	const std::string& unit_description() const { return cfg_["description"]; }
 
@@ -109,6 +111,8 @@ public:
 	int experience() const { return experience_; }
 	int max_experience() const { return max_experience_; }
 	int level() const { return level_; }
+	void remove_movement_ai();
+	void remove_attacks_ai();
 	/**
 	 * Adds 'xp' points to the units experience; returns true if advancement
 	 * should occur
@@ -120,17 +124,15 @@ public:
 	SDL_Colour xp_color() const;
 	/** Set to true for some scenario-specific units which should not be renamed */
 	bool unrenamable() const { return unrenamable_; }
-	unsigned int side() const { return side_; }
-	std::string side_id() const {return teams_manager::get_teams()[side()-1].save_id(); }
-	Uint32 team_rgb() const { return(team::get_side_rgb(side())); }
+	int side() const { return side_; }
+	std::string side_id() const;
 	const std::string& team_color() const { return flag_rgb_; }
 	unit_race::GENDER gender() const { return gender_; }
 	void set_side(unsigned int new_side) { side_ = new_side; }
 	fixed_t alpha() const { return alpha_; }
 
-	bool can_recruit() const { return utils::string_bool(cfg_["canrecruit"]); }
-	bool incapacitated() const { return utils::string_bool(get_state("stoned"),false); }
-	const std::vector<std::string>& recruits() const { return recruits_; }
+	bool can_recruit() const { return canrecruit_; }
+	bool incapacitated() const { return get_state(STATE_PETRIFIED); }
 	int total_movement() const { return max_movement_; }
 	int movement_left() const { return (movement_ == 0 || incapacitated()) ? 0 : movement_; }
 	void set_hold_position(bool value) { hold_position_ = value; }
@@ -145,49 +147,40 @@ public:
 	void end_turn();
 	void new_scenario();
 	/** Called on every draw */
-	void refresh(const game_display& disp,const map_location& loc) {
-		if (state_ == STATE_FORGET  && anim_ && anim_->animation_finished_potential()) {
-			set_standing( loc);
-			return;
-		}
-		if (state_ != STATE_STANDING || (get_current_animation_tick() < next_idling_) || incapacitated()) return;
-		if (get_current_animation_tick() > next_idling_ + 1000) { // prevent all units animating at the same
-			set_standing(loc);
-		} else {
-			set_idling(disp, loc);
-		}
-	}
+	void refresh();
 
+	void set_hitpoints(int hitpoints) { hit_points_ = hitpoints; }
 	bool take_hit(int damage) { hit_points_ -= damage; return hit_points_ <= 0; }
 	void heal(int amount);
 	void heal_all() { hit_points_ = max_hitpoints(); }
 	bool resting() const { return resting_; }
 	void set_resting(bool rest) { resting_ = rest; }
 
-	const std::map<std::string,std::string>& get_states() const;
-	std::string get_state(const std::string& state) const;
-	void set_state(const std::string& state, const std::string& value);
+	const std::map<std::string,std::string> get_states() const;
+	bool get_state(const std::string& state) const;
+	void set_state(const std::string &state, bool value);
+	enum state_t { STATE_SLOWED = 0, STATE_POISONED, STATE_PETRIFIED,
+		STATE_UNCOVERED, STATE_NOT_MOVED, STATE_UNKNOWN = -1 };
+	void set_state(state_t state, bool value);
+	bool get_state(state_t state) const;
+	static state_t get_known_boolean_state_id(const std::string &state);
+	static std::map<std::string, state_t> get_known_boolean_state_names();
 
 	bool has_moved() const { return movement_left() != total_movement(); }
 	bool has_goto() const { return get_goto().valid(); }
 	bool emits_zoc() const { return emit_zoc_ && !incapacitated();}
+	bool matches_id(const std::string& unit_id) const;
 	/* cfg: standard unit filter */
 	bool matches_filter(const vconfig& cfg,const map_location& loc,bool use_flat_tod=false) const;
 	void add_overlay(const std::string& overlay) { overlays_.push_back(overlay); }
 	void remove_overlay(const std::string& overlay) { overlays_.erase(std::remove(overlays_.begin(),overlays_.end(),overlay),overlays_.end()); }
 	const std::vector<std::string>& overlays() const { return overlays_; }
 
-	/**
-	 * Initialize this unit from a cfg object.
-	 *
-	 * @param cfg                 Configuration object from which to read the unit.
-	 * @param use_traits          ??
-	 * */
-	void read(const config& cfg, bool use_traits=true, game_state* state = 0);
 	void write(config& cfg) const;
-//	void write(config_writer& out) const;
 
-	void assign_role(const std::string& role) { role_ = role; }
+	void set_role(const std::string& role) { role_ = role; }
+	const std::string &get_role() const { return role_; }
+
 	void assign_ai_special(const std::string& s) { ai_special_ = s;}
             std::string get_ai_special() const { return(ai_special_); }
 	const std::vector<attack_type>& attacks() const { return attacks_; }
@@ -199,14 +192,14 @@ public:
 	const surface still_image(bool scaled = false) const;
 
 	/** draw a unit.  */
-	void redraw_unit(game_display& disp, const map_location& loc);
+	void redraw_unit();
 	/** Clear unit_halo_  */
 	void clear_haloes();
 
+	void set_standing(bool with_bars = true);
 
-	void set_standing(const map_location& loc, bool with_bars = true);
-	void set_idling(const game_display& disp,const map_location& loc);
-	void set_selecting(const game_display& disp,const map_location& loc);
+	void set_idling();
+	void set_selecting();
 	unit_animation* get_animation() {  return anim_;};
 	const unit_animation* get_animation() const {  return anim_;};
 	void set_facing(map_location::DIRECTION dir);
@@ -218,11 +211,17 @@ public:
 
 	int cost () const { return unit_value_; }
 
+	const map_location &get_location() const { return loc_; }
+	/** To be called by unit_map or for temporary units only. */
+	void set_location(const map_location &loc) { loc_ = loc; }
+
 	const map_location& get_goto() const { return goto_; }
 	void set_goto(const map_location& new_goto) { goto_ = new_goto; }
+	const std::vector<map_location>& waypoints() const { return waypoints_; }
+	std::vector<map_location>& waypoints() { return waypoints_; }
 
 	int upkeep() const;
-	bool loyal() const {return cfg_["upkeep"]=="loyal"; }
+	bool loyal() const;
 
 	void set_hidden(bool state);
 	bool get_hidden() const { return hidden_; }
@@ -230,7 +229,7 @@ public:
 	bool is_fearless() const { return is_fearless_; }
 	bool is_healthy() const { return is_healthy_; }
 	int movement_cost(const t_translation::t_terrain terrain) const;
-	int defense_modifier(t_translation::t_terrain terrain, int recurse_count=0) const;
+	int defense_modifier(t_translation::t_terrain terrain) const;
 	int resistance_against(const std::string& damage_name,bool attacker,const map_location& loc) const;
 	int resistance_against(const attack_type& damage_type,bool attacker,const map_location& loc) const
 		{return resistance_against(damage_type.type(), attacker, loc);};
@@ -245,8 +244,9 @@ public:
     std::map<std::string,std::string> advancement_icons() const;
     std::vector<std::pair<std::string,std::string> > amla_icons() const;
 
-	config::child_list get_modification_advances() const;
-	const config::child_list& modification_advancements() const { return cfg_.get_children("advancement"); }
+	std::vector<config> get_modification_advances() const;
+	config::const_child_itors modification_advancements() const
+	{ return cfg_.child_range("advancement"); }
 
 	size_t modification_count(const std::string& type, const std::string& id) const;
 
@@ -264,48 +264,53 @@ public:
 		STATE_STANDING,   /** anim must fit in a hex */
 		STATE_FORGET,     /** animation will be automaticaly replaced by a standing anim when finished */
 		STATE_ANIM};      /** normal anims */
-	void start_animation(const int start_time , const map_location &loc,const unit_animation* animation, bool with_bars,bool cycles=false,const std::string text = "", const Uint32 text_color =0,STATE state = STATE_ANIM);
+	void start_animation(int start_time, const unit_animation *animation,
+		bool with_bars, bool cycles = false, const std::string &text = "",
+		Uint32 text_color = 0, STATE state = STATE_ANIM);
 
 	/** The name of the file to game_display (used in menus). */
 	const std::string& absolute_image() const { return cfg_["image"]; }
 	const std::string& image_halo() const { return cfg_["halo"]; }
 
-	const std::string& get_hit_sound() const { return cfg_["get_hit_sound"]; }
 	const std::string& image_ellipse() const { return cfg_["ellipse"]; }
 
 	const std::string& usage() const { return cfg_["usage"]; }
 	unit_type::ALIGNMENT alignment() const { return alignment_; }
 	const unit_race* race() const { return race_; }
 
-	const unit_animation* choose_animation(const game_display& disp, const map_location& loc,const std::string& event,const int damage=0,const unit_animation::hit_type hit_type = unit_animation::INVALID,const attack_type* attack=NULL,const attack_type* second_attack = NULL, int swing_num =0) const;
+	const unit_animation* choose_animation(const game_display& disp,
+		       	const map_location& loc, const std::string& event,
+		       	const map_location& second_loc = map_location::null_location,
+			const int damage=0,
+			const unit_animation::hit_type hit_type = unit_animation::INVALID,
+			const attack_type* attack=NULL,const attack_type* second_attack = NULL,
+			int swing_num =0) const;
 
 	bool get_ability_bool(const std::string& ability, const map_location& loc) const;
-	unit_ability_list get_abilities(const std::string& ability, const map_location& loc) const;
-	std::vector<std::string> ability_tooltips(const map_location& loc) const;
-	std::vector<std::string> unit_ability_tooltips() const;
+	bool get_ability_bool(const std::string &ability) const
+	{ return get_ability_bool(ability, loc_); }
+	unit_ability_list get_abilities(const std::string &ability, const map_location& loc) const;
+	unit_ability_list get_abilities(const std::string &ability) const
+	{ return get_abilities(ability, loc_); }
+	std::vector<std::string> ability_tooltips(bool force_active = false) const;
 	std::vector<std::string> get_ability_list() const;
 	bool has_ability_type(const std::string& ability) const;
-	bool abilities_affects_adjacent() const;
 
 	const game_logic::map_formula_callable_ptr& formula_vars() const { return formula_vars_; }
 	void add_formula_var(std::string str, variant var);
 	bool has_formula() const { return !unit_formula_.empty(); }
 	bool has_loop_formula() const { return !unit_loop_formula_.empty(); }
 	bool has_priority_formula() const { return !unit_priority_formula_.empty(); }
-	bool has_on_fail_formula() const { return !unit_on_fail_formula_.empty(); }
 	const std::string& get_formula() const { return unit_formula_; }
 	const std::string& get_loop_formula() const { return unit_loop_formula_; }
 	const std::string& get_priority_formula() const { return unit_priority_formula_; }
-	const std::string& get_on_fail_formula() const { return unit_on_fail_formula_; }
 
-	void reset_modifications();
 	void backup_state();
 	void apply_modifications();
 	void remove_temporary_modifications();
 	void generate_traits(bool musthaveonly=false, game_state* state = 0);
 	void generate_traits_description();
-	std::string generate_name( rand_rng::simple_rng* rng = 0) const
-		{ return race_->generate_name(string_gender(cfg_["gender"]), rng); }
+	void generate_name(rand_rng::simple_rng *rng = 0);
 
 	// Only see_all=true use caching
 	bool invisible(const map_location& loc,
@@ -316,13 +321,7 @@ public:
 	 **/
 	unit& clone(bool is_temporary=true);
 
-	/**
-	 * Make sure that invisibility cache is revalidate
-	 * after ambush;
-	 **/
-	void ambush() const;
-
-	unit_race::GENDER generate_gender(const unit_type& type, bool gen, game_state* state = 0);
+	std::string TC_image_mods() const;
 	std::string image_mods() const;
 
 	/**
@@ -347,23 +346,21 @@ private:
 	bool ability_active(const std::string& ability,const config& cfg,const map_location& loc) const;
 	bool ability_affects_adjacent(const std::string& ability,const config& cfg,int dir,const map_location& loc) const;
 	bool ability_affects_self(const std::string& ability,const config& cfg,const map_location& loc) const;
-	bool resistance_filter_matches(const config& cfg,bool attacker,const std::string& damage_name) const;
-	bool resistance_filter_matches(const config& cfg,bool attacker,const attack_type& damage_type) const
-	{return resistance_filter_matches(cfg, attacker, damage_type.type()); };
+	bool resistance_filter_matches(const config& cfg,bool attacker,const std::string& damage_name, int res) const;
 
-	int movement_cost_internal(t_translation::t_terrain terrain, int recurse_count=0) const;
 	bool has_ability_by_id(const std::string& ability) const;
 	void remove_ability_by_id(const std::string& ability);
 
 	void set_underlying_id();
 
 	config cfg_;
+	map_location loc_;
 
 	std::vector<std::string> advances_to_;
 	std::string type_;
 	const unit_race* race_;
 	std::string id_;
-	std::string name_;
+	t_string name_;
 	size_t underlying_id_;
 	t_string type_name_;
 	std::string undead_variation_;
@@ -374,24 +371,22 @@ private:
 	int experience_;
 	int max_experience_;
 	int level_;
+	bool canrecruit_;
 	unit_type::ALIGNMENT alignment_;
 	std::string flag_rgb_;
 	std::string image_mods_;
 
 	bool unrenamable_;
-	unsigned int side_;
-	unit_race::GENDER gender_;
+	int side_;
+	const unit_race::GENDER gender_;
 
 	fixed_t alpha_;
 
 	std::string unit_formula_;
 	std::string unit_loop_formula_;
 	std::string unit_priority_formula_;
-	std::string unit_on_fail_formula_;
 	game_logic::map_formula_callable_ptr formula_vars_;
 
-	std::vector<std::string> recruits_;
-
 	int movement_;
 	int max_movement_;
 	mutable std::map<t_translation::t_terrain, int> movement_costs_; // movement cost cache
@@ -402,7 +397,9 @@ private:
 	int attacks_left_;
 	int max_attacks_;
 
-	std::map<std::string,std::string> states_;
+	std::set<std::string> states_;
+	std::vector<bool> known_boolean_states_;
+	static std::map<std::string, state_t> known_boolean_state_names_;
 	config variables_;
 	int emit_zoc_;
 	STATE state_;
@@ -417,6 +414,8 @@ private:
 	t_string traits_description_;
 	int unit_value_;
 	map_location goto_, interrupted_move_;
+	std::vector<map_location> waypoints_;
+
 	bool flying_, is_fearless_, is_healthy_;
 
 	string_map modification_descriptions_;
@@ -438,8 +437,6 @@ private:
 
 	friend void attack_type::set_specials_context(const map_location& loc, const map_location&, const unit& un, bool) const;
 	const unit_map* units_;
-	const gamemap* map_;
-	const gamestatus* gamestatus_;
 
 	/** Hold the visibility status cache for a unit, mutable since it's a cache. */
 	mutable std::map<map_location, bool> invisibility_cache_;
@@ -456,17 +453,8 @@ private:
 /** Object which temporarily resets a unit's movement */
 struct unit_movement_resetter
 {
-	unit_movement_resetter(unit& u, bool operate=true) : u_(u), moves_(u.movement_)
-	{
-		if(operate) {
-			u.movement_ = u.total_movement();
-		}
-	}
-
-	~unit_movement_resetter()
-	{
-		u_.movement_ = moves_;
-	}
+	unit_movement_resetter(unit& u, bool operate=true);
+	~unit_movement_resetter();
 
 private:
 	unit& u_;
@@ -475,20 +463,26 @@ private:
 
 void sort_units(std::vector< unit > &);
 
-/** Returns the number of units of the given side (team). */
-int team_units(const unit_map& units, unsigned int team_num);
-int team_upkeep(const unit_map& units, unsigned int team_num);
-unit_map::const_iterator team_leader(unsigned int side, const unit_map& units);
-unit_map::iterator find_visible_unit(unit_map& units,
-		const map_location loc,
-		const gamemap& map,
-		const std::vector<team>& teams, const team& current_team,
-		bool see_all=false);
-unit_map::const_iterator find_visible_unit(const unit_map& units,
-		const map_location loc,
-		const gamemap& map,
-		const std::vector<team>& teams, const team& current_team,
-		bool see_all=false);
+/** Returns the number of units of the side @a side_num. */
+int side_units(const unit_map& units, int side_num);
+
+/** Returns the total cost of units of side @a side_num. */
+int side_units_cost(const unit_map& units, int side_num);
+
+int side_upkeep(const unit_map& units, int side_num);
+
+unit_map::iterator find_visible_unit(unit_map &units, const map_location &loc,
+	const team &current_team, bool see_all = false);
+
+inline unit_map::const_iterator find_visible_unit(const unit_map &units,
+	const map_location &loc, const team &current_team, bool see_all = false)
+{
+	return find_visible_unit(const_cast<unit_map &>(units), loc,
+		current_team, see_all);
+}
+
+const unit *get_visible_unit(const unit_map &units, const map_location &loc,
+	const team &current_team, bool see_all = false);
 
 struct team_data
 {
@@ -525,6 +519,23 @@ private:
 	std::pair<map_location,unit> *temp_;
 };
 
+/**
+ * This object is used to temporary move a unit in the unit map, swapping out
+ * any unit that is already there.  On destruction, it restores the unit map to
+ * its original.
+ */
+struct temporary_unit_mover
+{
+	temporary_unit_mover(unit_map& m, const map_location& src,  const map_location& dst);
+	~temporary_unit_mover();
+
+private:
+	unit_map& m_;
+	const map_location& src_;
+	const map_location& dst_;
+	std::pair<map_location,unit> *temp_;
+};
+
 /**
  * Gets a checksum for a unit.
  *
