@@ -1,6 +1,6 @@
-/* $Id: unit_display.cpp 33742 2009-03-17 03:35:11Z alink $ */
+/* $Id: unit_display.cpp 45466 2010-08-10 15:29:30Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 #include "game_events.hpp"
 #include "log.hpp"
 #include "mouse_events.hpp"
+#include "resources.hpp"
 #include "terrain_filter.hpp"
 
 
@@ -29,21 +30,24 @@
 static void teleport_unit_between( const map_location& a, const map_location& b, unit& temp_unit)
 {
 	game_display* disp = game_display::get_singleton();
-	if(!disp || disp->video().update_locked() || (disp->fogged(a) && disp->fogged(b))) {
+	if(!disp || disp->video().update_locked() || disp->video().faked() || (disp->fogged(a) && disp->fogged(b))) {
 		return;
 	}
 	disp->scroll_to_tiles(a,b,game_display::ONSCREEN,true,0.0,false);
 
+	temp_unit.set_location(a);
 	if (!disp->fogged(a)) { // teleport
-		disp->place_temporary_unit(temp_unit,a);
+		disp->place_temporary_unit(temp_unit);
 		temp_unit.set_facing(a.get_relative_dir(b));
 		unit_animator animator;
 		animator.add_animation(&temp_unit,"pre_teleport",a);
 		animator.start_animations();
 		animator.wait_for_end();
 	}
+
+	temp_unit.set_location(b);
 	if (!disp->fogged(b)) { // teleport
-		disp->place_temporary_unit(temp_unit,b);
+		disp->place_temporary_unit(temp_unit);
 		temp_unit.set_facing(a.get_relative_dir(b));
 		disp->scroll_to_tiles(b,a,game_display::ONSCREEN,true,0.0,false);
 		unit_animator animator;
@@ -51,23 +55,25 @@ static void teleport_unit_between( const map_location& a, const map_location& b,
 		animator.start_animations();
 		animator.wait_for_end();
 	}
-	temp_unit.set_standing(b);
+
+	temp_unit.set_standing();
 	disp->update_display();
 	events::pump();
 }
 
-static void move_unit_between(const map_location& a, const map_location& b, unit& temp_unit)
+static void move_unit_between(const map_location& a, const map_location& b, unit& temp_unit,unsigned int step_num,unsigned int step_left)
 {
 	game_display* disp = game_display::get_singleton();
-	if(!disp || disp->video().update_locked() || (disp->fogged(a) && disp->fogged(b))) {
+	if(!disp || disp->video().update_locked() || disp->video().faked() || (disp->fogged(a) && disp->fogged(b))) {
 		return;
 	}
 
-
-	disp->place_temporary_unit(temp_unit,a);
+	temp_unit.set_location(a);
+	disp->place_temporary_unit(temp_unit);
 	temp_unit.set_facing(a.get_relative_dir(b));
 	unit_animator animator;
-	animator.replace_anim_if_invalid(&temp_unit,"movement",a);
+	animator.replace_anim_if_invalid(&temp_unit,"movement",a,b,step_num,
+			false,false,"",0,unit_animation::INVALID,NULL,NULL,step_left);
 	animator.start_animations();
         animator.pause_animation();
 	disp->scroll_to_tiles(a,b,game_display::ONSCREEN,true,0.0,false);
@@ -87,18 +93,18 @@ static void move_unit_between(const map_location& a, const map_location& b, unit
 	// This code causes backwards teleport because the time > 150 causes offset > 1.0
 	// which will not match with the following -1.0
 	// if(  target_time - animator.get_animation_time_potential() < 100 ) target_time +=150;
-	
+
 	animator.wait_until(target_time);
 		// debug code, see unit_frame::redraw()
 		// std::cout << "   end\n";
 	map_location arr[6];
 	get_adjacent_tiles(a, arr);
 	unsigned int i;
-	for (i = 0; i < 6; i++) {
+	for (i = 0; i < 6; ++i) {
 		disp->invalidate(arr[i]);
 	}
 	get_adjacent_tiles(b, arr);
-	for (i = 0; i < 6; i++) {
+	for (i = 0; i < 6; ++i) {
 		disp->invalidate(arr[i]);
 	}
 }
@@ -106,43 +112,40 @@ static void move_unit_between(const map_location& a, const map_location& b, unit
 namespace unit_display
 {
 
-bool unit_visible_on_path( const std::vector<map_location>& path, const unit& u, const unit_map& units, const std::vector<team>& teams)
-{
-	game_display* disp = game_display::get_singleton();
-	assert(disp);
-	for(size_t i = 0; i+1 < path.size(); ++i) {
-		const bool invisible = teams[u.side()-1].is_enemy(int(disp->viewing_team()+1)) &&
-	             u.invisible(path[i],units,teams) &&
-		         u.invisible(path[i+1],units,teams);
-		if(!invisible) {
-			return true;
-		}
-	}
-
-	return false;
-}
-
-void move_unit(const std::vector<map_location>& path, unit& u, const std::vector<team>& teams)
+void move_unit(const std::vector<map_location>& path, unit& u,
+		const std::vector<team>& teams, bool animate,
+		map_location::DIRECTION dir)
 {
 	game_display* disp = game_display::get_singleton();
 	assert(!path.empty());
 	assert(disp);
+	if(!disp || disp->video().update_locked() || disp->video().faked())
+		return;
 	// One hex path (strange), nothing to do
-	if (path.size()==1) return;
+	if(path.size() == 1)
+		return;
+	if(dir == map_location::NDIRECTIONS)
+		dir = path[path.size()-2].get_relative_dir(path.back());
+	// Don't animate, only set facing and redraw path ends
+	if(!animate) {
+		u.set_facing(dir);
+		disp->invalidate(path.front());
+		disp->invalidate(path.back());
+		return;
+	}
 
 	const unit_map& units = disp->get_units();
 
 	bool invisible = teams[u.side()-1].is_enemy(int(disp->viewing_team()+1)) &&
 		u.invisible(path[0],units,teams);
 
-
 	bool was_hidden = u.get_hidden();
 	// Original unit is usually hidden (but still on map, so count is correct)
 	unit temp_unit = u;
 	u.set_hidden(true);
-	temp_unit.set_standing(path[0],false);
+	temp_unit.set_standing(false);
 	temp_unit.set_hidden(false);
-	disp->place_temporary_unit(temp_unit,path[0]);
+	disp->place_temporary_unit(temp_unit);
 	if(!invisible) {
 		// Scroll to the path, but only if it fully fits on screen.
 		// If it does not fit we might be able to do a better scroll later.
@@ -154,7 +157,7 @@ void move_unit(const std::vector<map_location>& path, unit& u, const std::vector
 	// We use update=false because we don't need delay here (no time wasted)
 	// and no screen refresh (will be done by last 3rd draw() and it optimizes
 	// the double blitting done by these invalidations)
-    disp->draw(false);
+	disp->draw(false);
 	disp->draw(false);
 
 	// The last draw() was still slow, and its inital new_animation_frame() call
@@ -163,6 +166,15 @@ void move_unit(const std::vector<map_location>& path, unit& u, const std::vector
 	//       don't forget to change the previous draw(false) to true
 	disp->draw(true);
 
+	// extra immobile mvt anim for take-off
+	temp_unit.set_location(path[0]);
+	disp->place_temporary_unit(temp_unit);
+	temp_unit.set_facing(path[0].get_relative_dir(path[1]));
+	unit_animator animator;
+	animator.add_animation(&temp_unit,"pre_movement",path[0],path[1]);
+	animator.start_animations();
+	animator.wait_for_end();
+
 	for(size_t i = 0; i+1 < path.size(); ++i) {
 
 		invisible = teams[temp_unit.side()-1].is_enemy(int(disp->viewing_team()+1)) &&
@@ -170,12 +182,13 @@ void move_unit(const std::vector<map_location>& path, unit& u, const std::vector
 				temp_unit.invisible(path[i+1],units,teams);
 
 		if(!invisible) {
-			if (!disp->tile_on_screen(path[i]) || !disp->tile_on_screen(path[i+1])) {
+			if (!disp->tile_fully_on_screen(path[i]) || !disp->tile_fully_on_screen(path[i+1])) {
 				// prevent the unit from dissappearing if we scroll here with i == 0
-				disp->place_temporary_unit(temp_unit,path[i]);
+				temp_unit.set_location(path[i]);
+				disp->place_temporary_unit(temp_unit);
 				// scroll in as much of the remaining path as possible
 				std::vector<map_location> remaining_path;
-				for(size_t j = i; j < path.size(); j++) {
+				for(size_t j = i; j < path.size(); ++j) {
 					remaining_path.push_back(path[j]);
 				}
 				temp_unit.get_animation()->pause_animation();
@@ -185,7 +198,7 @@ void move_unit(const std::vector<map_location>& path, unit& u, const std::vector
 			}
 
 			if(tiles_adjacent(path[i], path[i+1])) {
-				move_unit_between(path[i],path[i+1],temp_unit);
+				move_unit_between(path[i],path[i+1],temp_unit,i,path.size()-2-i);
 			} else if (path[i] != path[i+1]) {
 				teleport_unit_between(path[i],path[i+1],temp_unit);
 			} else {
@@ -193,35 +206,91 @@ void move_unit(const std::vector<map_location>& path, unit& u, const std::vector
 			}
 		}
 	}
+	temp_unit.set_location(path[path.size() - 1]);
+	temp_unit.set_facing(path[path.size()-2].get_relative_dir(path[path.size()-1]));
+	animator.clear();
+	animator.add_animation(&temp_unit,"post_movement",path[path.size()-1],map_location::null_location);
+	animator.start_animations();
+	animator.wait_for_end();
 	disp->remove_temporary_unit();
-	u.set_facing(path[path.size()-2].get_relative_dir(path[path.size()-1]));
-	u.set_standing(path[path.size()-1]);
 
+	u.set_facing(dir);
 	u.set_hidden(was_hidden);
-	disp->invalidate_unit_after_move(path[0], path[path.size()-1]);
 
 	events::mouse_handler* mousehandler = events::mouse_handler::get_singleton();
 	if (mousehandler) {
 		mousehandler->invalidate_reachmap();
 	}
+
+	disp->invalidate(path.front());
+	disp->invalidate(path.back());
 }
 
-void unit_die(const map_location& loc, unit& loser,
-		const attack_type* attack,const attack_type* secondary_attack, unit* winner)
+void reset_helpers(const unit *attacker,const unit *defender);
+
+void unit_draw_weapon(const map_location& loc, unit& attacker,
+		const attack_type* attack,const attack_type* secondary_attack, const map_location& defender_loc,unit* defender)
 {
 	game_display* disp = game_display::get_singleton();
-	if(!disp ||disp->video().update_locked() || disp->fogged(loc) || preferences::show_combat() == false) {
+	if(!disp ||disp->video().update_locked() || disp->video().faked() || disp->fogged(loc) || preferences::show_combat() == false) {
+		return;
+	}
+	unit_animator animator;
+	animator.add_animation(&attacker,"draw_weapon",loc,defender_loc,0,false,false,"",0,unit_animation::HIT,attack,secondary_attack,0);
+	animator.add_animation(defender,"draw_weapon",defender_loc,loc,0,false,false,"",0,unit_animation::MISS,secondary_attack,attack,0);
+	animator.start_animations();
+	animator.wait_for_end();
+
+}
+
+
+void unit_sheath_weapon(const map_location& primary_loc, unit* primary_unit,
+		const attack_type* primary_attack,const attack_type* secondary_attack, const map_location& secondary_loc,unit* secondary_unit)
+{
+	game_display* disp = game_display::get_singleton();
+	if(!disp ||disp->video().update_locked() || disp->video().faked() || disp->fogged(primary_loc) || preferences::show_combat() == false) {
+		return;
+	}
+	unit_animator animator;
+	if(primary_unit) {
+		animator.add_animation(primary_unit,"sheath_weapon",primary_loc,secondary_loc,0,false,false,"",0,unit_animation::INVALID,primary_attack,secondary_attack,0);
+	}
+	if(secondary_unit) {
+		animator.add_animation(secondary_unit,"sheath_weapon",secondary_loc,primary_loc,0,false,false,"",0,unit_animation::INVALID,secondary_attack,primary_attack,0);
+	}
+
+	if(primary_unit || secondary_unit) {
+		animator.start_animations();
+		animator.wait_for_end();
+	}
+	if(primary_unit) {
+		primary_unit->set_standing();
+	}
+	if(secondary_unit) {
+		secondary_unit->set_standing();
+	}
+	reset_helpers(primary_unit,secondary_unit);
+
+}
+
+
+void unit_die(const map_location& loc, unit& loser,
+		const attack_type* attack,const attack_type* secondary_attack, const map_location& winner_loc,unit* winner)
+{
+	game_display* disp = game_display::get_singleton();
+	if(!disp ||disp->video().update_locked() || disp->video().faked() || disp->fogged(loc) || preferences::show_combat() == false) {
 		return;
 	}
 	unit_animator animator;
 	// hide the hp/xp bars of the loser (useless and prevent bars around an erased unit)
-	animator.add_animation(&loser,"death",loc,0,false,false,"",0,unit_animation::KILL,attack,secondary_attack,0);
+	animator.add_animation(&loser,"death",loc,winner_loc,0,false,false,"",0,unit_animation::KILL,attack,secondary_attack,0);
 	// but show the bars of the winner (avoid blinking and show its xp gain)
-	animator.add_animation(winner,"victory",loc.get_direction(loser.facing()),0,true,false,"",0,
+	animator.add_animation(winner,"victory",winner_loc,loc,0,true,false,"",0,
 			unit_animation::KILL,secondary_attack,attack,0);
 	animator.start_animations();
 	animator.wait_for_end();
 
+	reset_helpers(winner,&loser);
 	events::mouse_handler* mousehandler = events::mouse_handler::get_singleton();
 	if (mousehandler) {
 		mousehandler->invalidate_reachmap();
@@ -235,15 +304,15 @@ void unit_attack(
 		  int swing,std::string hit_text,bool drain,std::string att_text)
 {
 	game_display* disp = game_display::get_singleton();
-	if(!disp || preferences::show_combat() == false) return;
+	if(!disp ||disp->video().update_locked() || disp->video().faked() ||
+			(disp->fogged(a) && disp->fogged(b)) || preferences::show_combat() == false) {
+		return;
+	}
 	unit_map& units = disp->get_units();
 	disp->select_hex(map_location::null_location);
-	const bool hide = disp->video().update_locked() || (disp->fogged(a) && disp->fogged(b));
 
-	if(!hide) {
-		// scroll such that there is at least half a hex spacing around fighters
-		disp->scroll_to_tiles(a,b,game_display::ONSCREEN,true,0.5,false);
-	}
+	// scroll such that there is at least half a hex spacing around fighters
+	disp->scroll_to_tiles(a,b,game_display::ONSCREEN,true,0.5,false);
 
 	log_scope("unit_attack");
 
@@ -257,7 +326,7 @@ void unit_attack(
 	unit defender = def->second;
 	bool was_hidden = defender.get_hidden();
 	def->second.set_hidden(true);
-	disp->place_temporary_unit(defender,def->first);
+	disp->place_temporary_unit(defender);
 
 
 	att->second.set_facing(a.get_relative_dir(b));
@@ -266,54 +335,55 @@ void unit_attack(
 
 
 	unit_animator animator;
-	unit_ability_list leaders = attacker.get_abilities("leadership",a);
-	unit_ability_list helpers = defender.get_abilities("resistance",b);
-
-	{
-		std::string text ;
-		if(damage) text = lexical_cast<std::string>(damage);
-		if(!hit_text.empty()) {
-			text.insert(text.begin(),hit_text.size()/2,' ');
-			text = text + "\n" + hit_text;
-		}
-
-		std::string text_2 ;
-		if(drain && damage) text_2 = lexical_cast<std::string>(std::min<int>(damage,defender.hitpoints())/2);
-		if(!att_text.empty()) {
-			text_2.insert(text_2.begin(),att_text.size()/2,' ');
-			text_2 = text_2 + "\n" + att_text;
-		}
-
-		unit_animation::hit_type hit_type;
-		if(damage >= defender.hitpoints()) {
-			hit_type = unit_animation::KILL;
-		} else if(damage > 0) {
-			hit_type = unit_animation::HIT;
-		}else {
-			hit_type = unit_animation::MISS;
-		}
-		animator.add_animation(&attacker,"attack",att->first,damage,true,false,text_2,display::rgb(0,255,0),hit_type,&attack,secondary_attack,swing);
-		animator.add_animation(&defender,"defend",def->first,damage,true,false,text  ,display::rgb(255,0,0),hit_type,&attack,secondary_attack,swing);
-
-		for(std::vector<std::pair<config*,map_location> >::iterator itor = leaders.cfgs.begin(); itor != leaders.cfgs.end(); itor++) {
-			if(itor->second == a) continue;
-			if(itor->second == b) continue;
-			unit_map::iterator leader = units.find(itor->second);
-			assert(leader != units.end());
-			leader->second.set_facing(itor->second.get_relative_dir(a));
-			animator.add_animation(&leader->second,"leading",itor->second,damage,true,false,"",0,hit_type,&attack,secondary_attack,swing);
-		}
-		for(std::vector<std::pair<config*,map_location> >::iterator itor = helpers.cfgs.begin(); itor != helpers.cfgs.end(); itor++) {
-			if(itor->second == a) continue;
-			if(itor->second == b) continue;
-			unit_map::iterator helper = units.find(itor->second);
-			assert(helper != units.end());
-			helper->second.set_facing(itor->second.get_relative_dir(b));
-			animator.add_animation(&helper->second,"resistance",itor->second,damage,true,false,"",0,hit_type,&attack,secondary_attack,swing);
-		}
+	unit_ability_list leaders = attacker.get_abilities("leadership");
+	unit_ability_list helpers = defender.get_abilities("resistance");
 
+	std::string text ;
+	if(damage) text = lexical_cast<std::string>(damage);
+	if(!hit_text.empty()) {
+		text.insert(text.begin(),hit_text.size()/2,' ');
+		text = text + "\n" + hit_text;
 	}
 
+	std::string text_2 ;
+	if(drain && damage) text_2 = lexical_cast<std::string>(std::min<int>(damage,defender.hitpoints())/2);
+	if(!att_text.empty()) {
+		text_2.insert(text_2.begin(),att_text.size()/2,' ');
+		text_2 = text_2 + "\n" + att_text;
+	}
+
+	unit_animation::hit_type hit_type;
+	if(damage >= defender.hitpoints()) {
+		hit_type = unit_animation::KILL;
+	} else if(damage > 0) {
+		hit_type = unit_animation::HIT;
+	}else {
+		hit_type = unit_animation::MISS;
+	}
+	animator.add_animation(&attacker,"attack",att->first,def->first,damage,true,false,text_2,display::rgb(0,255,0),hit_type,&attack,secondary_attack,swing);
+
+	// note that we take an anim from the real unit, we'll use it later
+	const unit_animation * defender_anim = def->second.choose_animation(*disp,def->first,"defend",att->first,damage,hit_type,&attack,secondary_attack,swing);
+	animator.add_animation(&defender,defender_anim,def->first,true,false,text  ,display::rgb(255,0,0));;
+
+	for (std::vector<std::pair<const config *, map_location> >::iterator itor = leaders.cfgs.begin(); itor != leaders.cfgs.end(); ++itor) {
+		if(itor->second == a) continue;
+		if(itor->second == b) continue;
+		unit_map::iterator leader = units.find(itor->second);
+		assert(leader != units.end());
+		leader->second.set_facing(itor->second.get_relative_dir(a));
+		animator.add_animation(&leader->second,"leading",itor->second,att->first,damage,true,false,"",0,hit_type,&attack,secondary_attack,swing);
+	}
+	for (std::vector<std::pair<const config *, map_location> >::iterator itor = helpers.cfgs.begin(); itor != helpers.cfgs.end(); ++itor) {
+		if(itor->second == a) continue;
+		if(itor->second == b) continue;
+		unit_map::iterator helper = units.find(itor->second);
+		assert(helper != units.end());
+		helper->second.set_facing(itor->second.get_relative_dir(b));
+		animator.add_animation(&helper->second,"resistance",itor->second,def->first,damage,true,false,"",0,hit_type,&attack,secondary_attack,swing);
+	}
+
+
 	animator.start_animations();
 	animator.wait_until(0);
 	int damage_left = damage;
@@ -327,28 +397,60 @@ void unit_attack(
 		animator.wait_until(animator.get_animation_time_potential() +50);
 	}
 	animator.wait_for_end();
-	animator.set_all_standing();
+	// pass the animation back to the real unit
+	def->second.start_animation(animator.get_end_time(),defender_anim,true);
+	reset_helpers(&att->second,&def->second);
 	disp->remove_temporary_unit();
 	def->second.set_hidden(was_hidden);
-	def->second.set_standing(def->first);
-
 }
 
-
-void unit_recruited(map_location& loc)
+// private helper function, set all helpers to default position
+void reset_helpers(const unit *attacker,const unit *defender)
 {
 	game_display* disp = game_display::get_singleton();
-	if(!disp || disp->video().update_locked() ||disp->fogged(loc)) return;
+	unit_map& units = disp->get_units();
+	if(attacker) {
+		unit_ability_list leaders = attacker->get_abilities("leadership");
+		for (std::vector<std::pair<const config *, map_location> >::iterator itor = leaders.cfgs.begin(); itor != leaders.cfgs.end(); ++itor) {
+			unit_map::iterator leader = units.find(itor->second);
+			assert(leader != units.end());
+			leader->second.set_standing();
+		}
+	}
+
+	if(defender) {
+		unit_ability_list helpers = defender->get_abilities("resistance");
+		for (std::vector<std::pair<const config *, map_location> >::iterator itor = helpers.cfgs.begin(); itor != helpers.cfgs.end(); ++itor) {
+			unit_map::iterator helper = units.find(itor->second);
+			assert(helper != units.end());
+			helper->second.set_standing();
+		}
+	}
+}
+
+void unit_recruited(const map_location& loc,const map_location& leader_loc)
+{
+	game_display* disp = game_display::get_singleton();
+	if(!disp || disp->video().update_locked() || disp->video().faked() ||disp->fogged(loc)) return;
 	unit_map::iterator u = disp->get_units().find(loc);
 	if(u == disp->get_units().end()) return;
-
 	u->second.set_hidden(true);
-	disp->scroll_to_tile(loc,game_display::ONSCREEN,true,false);
+
+	unit_animator animator;
+	if(leader_loc != map_location::null_location) {
+		unit_map::iterator leader = disp->get_units().find(leader_loc);
+		if(leader == disp->get_units().end()) return;
+		disp->scroll_to_tiles(loc,leader_loc,game_display::ONSCREEN,true,0.0,false);
+		leader->second.set_facing(leader_loc.get_relative_dir(loc));
+		animator.add_animation(&leader->second,"recruiting",leader_loc,loc,0,true);
+	} else {
+		disp->scroll_to_tile(loc,game_display::ONSCREEN,true,false);
+	}
+
 	disp->draw();
 	u->second.set_hidden(false);
 	u->second.set_facing(static_cast<map_location::DIRECTION>(rand()%map_location::NDIRECTIONS));
-	unit_animator animator;
-	animator.add_animation(&u->second,"recruited",loc);
+	animator.add_animation(&u->second,"recruited",loc,leader_loc);
 	animator.start_animations();
 	animator.wait_for_end();
 	animator.set_all_standing();
@@ -358,7 +460,7 @@ void unit_recruited(map_location& loc)
 void unit_healing(unit& healed,map_location& healed_loc, std::vector<unit_map::iterator> healers, int healing)
 {
 	game_display* disp = game_display::get_singleton();
-	if(!disp || disp->video().update_locked() || disp->fogged(healed_loc)) return;
+	if(!disp || disp->video().update_locked() || disp->video().faked() || disp->fogged(healed_loc)) return;
 	if(healing==0) return;
 	// This is all the pretty stuff.
 	disp->scroll_to_tile(healed_loc, game_display::ONSCREEN,true,false);
@@ -367,12 +469,12 @@ void unit_healing(unit& healed,map_location& healed_loc, std::vector<unit_map::i
 
 	for(std::vector<unit_map::iterator>::iterator heal_anim_it = healers.begin(); heal_anim_it != healers.end(); ++heal_anim_it) {
 		(*heal_anim_it)->second.set_facing((*heal_anim_it)->first.get_relative_dir(healed_loc));
-		animator.add_animation(&(*heal_anim_it)->second,"healing",(*heal_anim_it)->first,healing);
+		animator.add_animation(&(*heal_anim_it)->second,"healing",(*heal_anim_it)->first,healed_loc,healing);
 	}
 	if (healing < 0) {
-		animator.add_animation(&healed,"poisoned",healed_loc,-healing,false,false,lexical_cast<std::string>(-healing), display::rgb(255,0,0));
+		animator.add_animation(&healed,"poisoned",healed_loc,map_location::null_location,-healing,false,false,lexical_cast<std::string>(-healing), display::rgb(255,0,0));
 	} else {
-		animator.add_animation(&healed,"healed",healed_loc,healing,false,false,lexical_cast<std::string>(healing), display::rgb(0,255,0));
+		animator.add_animation(&healed,"healed",healed_loc,map_location::null_location,healing,false,false,lexical_cast<std::string>(healing), display::rgb(0,255,0));
 	}
 	animator.start_animations();
 	animator.wait_for_end();
@@ -380,33 +482,36 @@ void unit_healing(unit& healed,map_location& healed_loc, std::vector<unit_map::i
 
 }
 
-void wml_animation_internal(unit_animator & animator,const vconfig &cfg, const gamemap& map, const gamestatus& game_status,unit_map & units,const map_location& default_location = map_location::null_location);
-void wml_animation(const vconfig &cfg,unit_map & units, const gamemap& map, const gamestatus& game_status,const map_location& default_location)
+void wml_animation_internal(unit_animator &animator, const vconfig &cfg, const map_location &default_location = map_location::null_location);
+
+void wml_animation(const vconfig &cfg, const map_location &default_location)
 {
-		unit_animator animator;
-		wml_animation_internal(animator,cfg,map,game_status,units,default_location);
-		animator.start_animations();
-		animator.wait_for_end();
-		animator.set_all_standing();
+	game_display &disp = *resources::screen;
+	if (disp.video().update_locked() || disp.video().faked()) return;
+	unit_animator animator;
+	wml_animation_internal(animator, cfg, default_location);
+	animator.start_animations();
+	animator.wait_for_end();
+	animator.set_all_standing();
 }
 
-void wml_animation_internal(unit_animator & animator,const vconfig &cfg, const gamemap& map, const gamestatus& game_status,unit_map & units,const map_location& default_location)
+void wml_animation_internal(unit_animator &animator, const vconfig &cfg, const map_location &default_location)
 {
-	unit_map::iterator u = units.find(default_location);
+	unit_map::iterator u = resources::units->find(default_location);
 
 	// Search for a valid unit filter,
 	// and if we have one, look for the matching unit
 	vconfig filter = cfg.child("filter");
 	if(!filter.null()) {
-		for(u = units.begin(); u != units.end(); ++u){
+		for (u = resources::units->begin(); u != resources::units->end(); ++u) {
 			if(game_events::unit_matches_filter(u, filter))
 				break;
 		}
 	}
 
 	// We have found a unit that matches the filter
-	game_display* disp = game_display::get_singleton();
-	if(u != units.end() && ! disp->fogged(u->first)) {
+	if (u.valid() && !resources::screen->fogged(u->first))
+	{
 		attack_type *primary = NULL;
 		attack_type *secondary = NULL;
 		Uint32 text_color;
@@ -448,23 +553,26 @@ void wml_animation_internal(unit_animator & animator,const vconfig &cfg, const g
 		} else {
 			text_color = display::rgb(atoi(cfg["red"].c_str()),atoi(cfg["green"].c_str()),atoi(cfg["blue"].c_str()));
 		}
-		disp->scroll_to_tile(u->first, game_display::ONSCREEN,true,false);
+		resources::screen->scroll_to_tile(u->first, game_display::ONSCREEN, true, false);
 		vconfig t_filter = cfg.child("facing");
+		map_location secondary_loc = map_location::null_location;
 		if(!t_filter.empty()) {
-			terrain_filter filter(t_filter,map,game_status,units);
+			terrain_filter filter(t_filter, *resources::units);
 			std::set<map_location> locs;
 			filter.get_locations(locs);
-			if(!locs.empty()) {
-				u->second.set_facing(u->first.get_relative_dir(*locs.begin()));
+			if (!locs.empty() && u->first != *locs.begin()) {
+				map_location::DIRECTION dir =u->first.get_relative_dir(*locs.begin());
+				u->second.set_facing(dir);
+				secondary_loc = u->first.get_direction(dir);
 			}
 		}
-		animator.add_animation(&u->second,cfg["flag"],u->first,lexical_cast_default<int>(cfg["value"]),utils::string_bool(cfg["with_bars"]),
-				false,cfg["text"],text_color, hits,primary,secondary,0);
+		animator.add_animation(&u->second,cfg["flag"],u->first,secondary_loc,lexical_cast_default<int>(cfg["value"]),utils::string_bool(cfg["with_bars"]),
+				false,cfg["text"],text_color, hits,primary,secondary,lexical_cast_default<int>(cfg["value_second"]));
 	}
 	const vconfig::child_list sub_anims = cfg.get_children("animate");
 	vconfig::child_list::const_iterator anim_itor;
-	for(anim_itor = sub_anims.begin(); anim_itor != sub_anims.end();anim_itor++) {
-		wml_animation_internal(animator,*anim_itor,map,game_status,units);
+	for(anim_itor = sub_anims.begin(); anim_itor != sub_anims.end();++anim_itor) {
+		wml_animation_internal(animator, *anim_itor);
 	}
 
 }
