@@ -1,6 +1,6 @@
-/* $Id: unit_frame.cpp 35578 2009-05-11 21:45:03Z boucman $ */
+/* $Id: unit_frame.cpp 44034 2010-07-09 09:05:49Z ivanovic $ */
 /*
-   Copyright (C) 2006 - 2009 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
+   Copyright (C) 2006 - 2010 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,12 +15,13 @@
 /** @file unit_frame.cpp */
 
 #include "global.hpp"
-#include "sound.hpp"
+
+#include "game_display.hpp"
 #include "halo.hpp"
-// NOTE: global.hpp must be first!
+#include "sound.hpp"
+#include "unit_frame.hpp"
 
 #define UNIT_FRAME_H_PART2
-#include "unit_frame.hpp"
 
 
 progressive_string::progressive_string(const std::string & data,int duration) :
@@ -31,7 +32,7 @@ progressive_string::progressive_string(const std::string & data,int duration) :
 		const int time_chunk = std::max<int>(duration / (first_pass.size()?first_pass.size():1),1);
 
 		std::vector<std::string>::const_iterator tmp;
-		for(tmp=first_pass.begin();tmp != first_pass.end() ; tmp++) {
+		for(tmp=first_pass.begin();tmp != first_pass.end() ; ++tmp) {
 			std::vector<std::string> second_pass = utils::split(*tmp,':');
 			if(second_pass.size() > 1) {
 				data_.push_back(std::pair<std::string,int>(second_pass[0],atoi(second_pass[1].c_str())));
@@ -44,7 +45,7 @@ int progressive_string::duration() const
 {
 	int total =0;
 	std::vector<std::pair<std::string,int> >::const_iterator cur_halo;
-	for(cur_halo = data_.begin() ; cur_halo != data_.end() ; cur_halo++) {
+	for(cur_halo = data_.begin() ; cur_halo != data_.end() ; ++cur_halo) {
 		total += cur_halo->second;
 	}
 	return total;
@@ -58,7 +59,7 @@ const std::string& progressive_string::get_current_element(int current_time)cons
 	if(data_.empty()) return empty_string;
 	while(time < current_time&& sub_halo < data_.size()) {
 		time += data_[sub_halo].second;
-		sub_halo++;
+		++sub_halo;
 
 	}
 	if(sub_halo) sub_halo--;
@@ -71,29 +72,21 @@ progressive_<T>::progressive_(const std::string &data, int duration) :
 	data_(),
 	input_(data)
 {
-	const std::vector<std::string> first_split = utils::split(data);
-	const int time_chunk = std::max<int>(duration / (first_split.size()?first_split.size():1),1);
+	int split_flag = utils::REMOVE_EMPTY; // useless to strip spaces
+	const std::vector<std::string> comma_split = utils::split(data,',',split_flag);
+	const int time_chunk = std::max<int>(1, duration / std::max<int>(comma_split.size(),1));
 
-	std::vector<std::string>::const_iterator tmp;
-	std::vector<std::pair<std::string,int> > first_pass;
-	for(tmp=first_split.begin();tmp != first_split.end() ; tmp++) {
-		std::vector<std::string> second_pass = utils::split(*tmp,':');
-		if(second_pass.size() > 1) {
-			first_pass.push_back(std::pair<std::string,int>(second_pass[0],atoi(second_pass[1].c_str())));
-		} else {
-			first_pass.push_back(std::pair<std::string,int>(second_pass[0],time_chunk));
-		}
-	}
-	std::vector<std::pair<std::string,int> >::const_iterator tmp2;
-	for(tmp2=first_pass.begin();tmp2 != first_pass.end() ; tmp2++) {
-		std::vector<std::string> range = utils::split(tmp2->first,'~');
-		data_.push_back(std::pair<std::pair<T, T>,int> (
-			std::pair<T, T>(
-				lexical_cast<T>(range[0].c_str()),
-				lexical_cast<T>(range.size() > 1 ? range[1].c_str() : range[0].c_str())),
-				tmp2->second));
-	}
+	std::vector<std::string>::const_iterator com_it = comma_split.begin();
+	for(; com_it != comma_split.end(); ++com_it) {
+		std::vector<std::string> colon_split = utils::split(*com_it,':',split_flag);
+		int time = (colon_split.size() > 1) ? atoi(colon_split[1].c_str()) : time_chunk;
 
+		std::vector<std::string> range = utils::split(colon_split[0],'~',split_flag);
+		T range0 = lexical_cast<T>(range[0]);
+		T range1 = (range.size() > 1) ? lexical_cast<T>(range[1]) : range0;
+		typedef std::pair<T,T> range_pair;
+		data_.push_back(std::pair<range_pair,int>(range_pair(range0, range1), time));
+	}
 }
 
 template <class T>
@@ -107,7 +100,7 @@ const T progressive_<T>::get_current_element(int current_time, T default_val) co
 	if(data_.empty()) return default_val;
 	while(time < searched_time&& sub_halo < data_.size()) {
 		time += data_[sub_halo].second;
-		sub_halo++;
+		++sub_halo;
 
 	}
 	if(sub_halo != 0) {
@@ -132,7 +125,7 @@ int progressive_<T>::duration() const
 {
 	int total = 0;
 	typename std::vector<std::pair<std::pair<T, T>, int> >::const_iterator cur_halo;
-	for(cur_halo = data_.begin() ; cur_halo != data_.end() ; cur_halo++) {
+	for(cur_halo = data_.begin() ; cur_halo != data_.end() ; ++cur_halo) {
 		total += cur_halo->second;
 	}
 	return total;
@@ -151,11 +144,33 @@ template class progressive_<int>;
 template class progressive_<double>;
 
 #undef UNIT_FRAME_H_PART2
-#include "unit_frame.hpp"
 
+frame_parameters::frame_parameters() :
+	duration(0),
+	image(),
+	image_diagonal(),
+	image_mod(""),
+	halo(""),
+	halo_x(0),
+	halo_y(0),
+	halo_mod(""),
+	sound(""),
+	text(""),
+	text_color(0),
+	blend_with(0),
+	blend_ratio(0.0),
+	highlight_ratio(1.0),
+	offset(0),
+	submerge(0.0),
+	x(0),
+	y(0),
+	drawing_layer(display::LAYER_UNIT_DEFAULT - display::LAYER_UNIT_FIRST),
+	in_hex(false),
+	diagonal_in_hex(false)
+{}
 
-
-frame_builder::frame_builder(const config& cfg,const std::string& frame_string) :
+frame_builder::frame_builder() :
+	duration_(1),
 	image_(),
 	image_diagonal_(),
 	image_mod_(""),
@@ -166,7 +181,28 @@ frame_builder::frame_builder(const config& cfg,const std::string& frame_string)
 	sound_(""),
 	text_(""),
 	text_color_(0),
+	blend_with_(0),
+	blend_ratio_(""),
+	highlight_ratio_(""),
+	offset_(""),
+	submerge_(""),
+	x_(""),
+	y_(""),
+	drawing_layer_(str_cast(display::LAYER_UNIT_DEFAULT - display::LAYER_UNIT_FIRST))
+{}
+
+frame_builder::frame_builder(const config& cfg,const std::string& frame_string) :
 	duration_(1),
+	image_(),
+	image_diagonal_(),
+	image_mod_(""),
+	halo_(""),
+	halo_x_(""),
+	halo_y_(""),
+	halo_mod_(""),
+	sound_(""),
+	text_(""),
+	text_color_(0),
 	blend_with_(0),
 	blend_ratio_(""),
 	highlight_ratio_(""),
@@ -205,10 +241,136 @@ frame_builder::frame_builder(const config& cfg,const std::string& frame_string)
 	x(cfg[frame_string+"x"]);
 	y(cfg[frame_string+"y"]);
 	drawing_layer(cfg[frame_string+"layer"]);
-
 }
 
-const frame_parameters frame_builder::parameters(int current_time) const
+frame_builder & frame_builder::image(const image::locator& image ,const std::string & image_mod)
+{
+	image_ = image;
+	image_mod_ = image_mod;
+	return *this;
+}
+frame_builder & frame_builder::image_diagonal(const image::locator& image_diagonal,const std::string& image_mod)
+{
+	image_diagonal_ = image_diagonal;
+	image_mod_ = image_mod;
+	return *this;
+}
+frame_builder & frame_builder::sound(const std::string& sound)
+{
+	sound_=sound;
+	return *this;
+}
+frame_builder & frame_builder::text(const std::string& text,const  Uint32 text_color)
+{
+	text_=text;
+	text_color_=text_color;
+	return *this;
+}
+frame_builder & frame_builder::halo(const std::string &halo, const std::string &halo_x, const std::string& halo_y,const std::string & halo_mod)
+{
+	halo_ = halo;
+	halo_x_ = halo_x;
+	halo_y_ = halo_y;
+	halo_mod_= halo_mod;
+	return *this;
+}
+frame_builder & frame_builder::duration(const int duration)
+{
+	duration_= duration;
+	return *this;
+}
+frame_builder & frame_builder::blend(const std::string& blend_ratio,const Uint32 blend_color)
+{
+	blend_with_=blend_color;
+	blend_ratio_=blend_ratio;
+	return *this;
+}
+frame_builder & frame_builder::highlight(const std::string& highlight)
+{
+	highlight_ratio_=highlight;
+	return *this;
+}
+frame_builder & frame_builder::offset(const std::string& offset)
+{
+	offset_=offset;
+	return *this;
+}
+frame_builder & frame_builder::submerge(const std::string& submerge)
+{
+	submerge_=submerge;
+	return *this;
+}
+frame_builder & frame_builder::x(const std::string& x)
+{
+	x_=x;
+	return *this;
+}
+frame_builder & frame_builder::y(const std::string& y)
+{
+	y_=y;
+	return *this;
+}
+frame_builder & frame_builder::drawing_layer(const std::string& drawing_layer)
+{
+	drawing_layer_=drawing_layer;
+	return *this;
+}
+
+
+frame_parsed_parameters::frame_parsed_parameters(const frame_builder & builder, int duration) :
+	duration_(duration ? duration :builder.duration_),
+	image_(builder.image_),
+	image_diagonal_(builder.image_diagonal_),
+	image_mod_(builder.image_mod_),
+	halo_(builder.halo_,duration_),
+	halo_x_(builder.halo_x_,duration_),
+	halo_y_(builder.halo_y_,duration_),
+	halo_mod_(builder.halo_mod_),
+	sound_(builder.sound_),
+	text_(builder.text_),
+	text_color_(builder.text_color_),
+	blend_with_(builder.blend_with_),
+	blend_ratio_(builder.blend_ratio_,duration_),
+	highlight_ratio_(builder.highlight_ratio_,duration_),
+	offset_(builder.offset_,duration_),
+	submerge_(builder.submerge_,duration_),
+	x_(builder.x_,duration_),
+	y_(builder.y_,duration_),
+	drawing_layer_(builder.drawing_layer_,duration_)
+{}
+
+
+bool frame_parsed_parameters::does_not_change() const
+{
+	return halo_.does_not_change() &&
+		halo_x_.does_not_change() &&
+		halo_y_.does_not_change() &&
+		blend_ratio_.does_not_change() &&
+		highlight_ratio_.does_not_change() &&
+		offset_.does_not_change() &&
+		submerge_.does_not_change() &&
+		x_.does_not_change() &&
+		y_.does_not_change() &&
+		drawing_layer_.does_not_change();
+}
+bool frame_parsed_parameters::need_update() const
+{
+	if(!halo_.does_not_change() ||
+			!halo_x_.does_not_change() ||
+			!halo_y_.does_not_change() ||
+			!blend_ratio_.does_not_change() ||
+			!highlight_ratio_.does_not_change() ||
+			!offset_.does_not_change() ||
+			!submerge_.does_not_change() ||
+			!x_.does_not_change() ||
+			!y_.does_not_change() ||
+			!drawing_layer_.does_not_change() ) {
+			return true;
+	}
+	return false;
+}
+
+const frame_parameters frame_parsed_parameters::parameters(int current_time) const
 {
 	frame_parameters result;
 	result.duration = duration_;
@@ -232,122 +394,49 @@ const frame_parameters frame_builder::parameters(int current_time) const
 	result.drawing_layer = drawing_layer_.get_current_element(current_time,display::LAYER_UNIT_DEFAULT-display::LAYER_UNIT_FIRST);
 	return result;
 }
-frame_builder & frame_builder::image(const image::locator image ,const std::string & image_mod)
+
+void frame_parsed_parameters::override( int duration
+		, const std::string& highlight
+		, const std::string& blend_ratio
+		, Uint32 blend_color
+		, const std::string& offset
+		, const std::string& layer)
 {
-	image_ = image;
-	image_mod_ = image_mod;
-	return *this;
-}
-frame_builder & frame_builder::image_diagonal(const image::locator image_diagonal,const std::string& image_mod)
-{
-	image_diagonal_ = image_diagonal;
-	image_mod_ = image_mod;
-	return *this;
-}
-frame_builder & frame_builder::sound(const std::string& sound)
-{
-	sound_=sound;
-	return *this;
-}
-frame_builder & frame_builder::text(const std::string& text,const  Uint32 text_color)
-{
-	text_=text;
-	text_color_=text_color;
-	return *this;
-}
-frame_builder & frame_builder::halo(const std::string &halo, const std::string &halo_x, const std::string& halo_y,const std::string & halo_mod)
-{
-	halo_ = progressive_string(halo,duration_);
-	halo_x_ = progressive_int(halo_x,duration_);
-	halo_y_ = progressive_int(halo_y,duration_);
-	halo_mod_= halo_mod;
-	return *this;
-}
-frame_builder & frame_builder::duration(const int duration)
-{
-	duration_= duration;
-	recalculate_duration();
-	return *this;
-}
-void frame_builder::recalculate_duration()
-{
-	halo_ = progressive_string(halo_.get_original(),duration_);
-	halo_x_ = progressive_int(halo_x_.get_original(),duration_);
-	halo_y_ = progressive_int(halo_y_.get_original(),duration_);
-	blend_ratio_=progressive_double(blend_ratio_.get_original(),duration_);
-	highlight_ratio_=progressive_double(highlight_ratio_.get_original(),duration_);
-	offset_=progressive_double(offset_.get_original(),duration_);
-	submerge_=progressive_double(submerge_.get_original(),duration_);
-	x_=progressive_int(x_.get_original(),duration_);
-	y_=progressive_int(y_.get_original(),duration_);
-	drawing_layer_=progressive_int(drawing_layer_.get_original(),duration_);
-}
-frame_builder & frame_builder::blend(const std::string& blend_ratio,const Uint32 blend_color)
-{
-	blend_with_=blend_color;
-	blend_ratio_=progressive_double(blend_ratio,duration_);
-	return *this;
-}
-frame_builder & frame_builder::highlight(const std::string& highlight)
-{
-	highlight_ratio_=progressive_double(highlight,duration_);
-	return *this;
-}
-frame_builder & frame_builder::offset(const std::string& offset)
-{
-	offset_=progressive_double(offset,duration_);
-	return *this;
-}
-frame_builder & frame_builder::submerge(const std::string& submerge)
-{
-	submerge_=progressive_double(submerge,duration_);
-	return *this;
-}
-frame_builder & frame_builder::x(const std::string& x)
-{
-	x_=progressive_int(x,duration_);
-	return *this;
-}
-frame_builder & frame_builder::y(const std::string& y)
-{
-	y_=progressive_int(y,duration_);
-	return *this;
-}
-frame_builder & frame_builder::drawing_layer(const std::string& drawing_layer)
-{
-	drawing_layer_=progressive_int(drawing_layer,duration_);
-	return *this;
-}
-bool frame_builder::does_not_change() const
-{
-	return halo_.does_not_change() &&
-		halo_x_.does_not_change() &&
-		halo_y_.does_not_change() &&
-		blend_ratio_.does_not_change() &&
-		highlight_ratio_.does_not_change() &&
-		offset_.does_not_change() &&
-		submerge_.does_not_change() &&
-		x_.does_not_change() &&
-		y_.does_not_change() &&
-		drawing_layer_.does_not_change();
-}
-bool frame_builder::need_update() const
-{
-	if(!halo_.does_not_change() ||
-			!halo_x_.does_not_change() ||
-			!halo_y_.does_not_change() ||
-			!blend_ratio_.does_not_change() ||
-			!highlight_ratio_.does_not_change() ||
-			!offset_.does_not_change() ||
-			!submerge_.does_not_change() ||
-			!x_.does_not_change() ||
-			!y_.does_not_change() ||
-			!drawing_layer_.does_not_change() ) {
-			return true;
+
+	if(!highlight.empty()) {
+		highlight_ratio_ = progressive_double(highlight,duration);
+	} else if(duration != duration_){
+		highlight_ratio_=progressive_double(highlight_ratio_.get_original(),duration);
+	}
+	if(!offset.empty()) {
+		offset_= progressive_double(offset,duration);
+	} else  if(duration != duration_){
+		offset_=progressive_double(offset_.get_original(),duration);
+	}
+	if(!blend_ratio.empty()) {
+		blend_ratio_ = progressive_double(blend_ratio,duration);
+		blend_with_  = blend_color;
+	} else  if(duration != duration_){
+		blend_ratio_=progressive_double(blend_ratio_.get_original(),duration);
+	}
+	if(!layer.empty()) {
+		drawing_layer_ = progressive_int(layer,duration);
+	} else  if(duration != duration_){
+		drawing_layer_=progressive_int(drawing_layer_.get_original(),duration);
+	}
+
+	if(duration != duration_) {
+		halo_ = progressive_string(halo_.get_original(),duration);
+		halo_x_ = progressive_int(halo_x_.get_original(),duration);
+		halo_y_ = progressive_int(halo_y_.get_original(),duration);
+		submerge_=progressive_double(submerge_.get_original(),duration);
+		x_=progressive_int(x_.get_original(),duration);
+		y_=progressive_int(y_.get_original(),duration);
+		duration_ = duration;
 	}
-	return false;
 }
 
+
 void unit_frame::redraw(const int frame_time,bool first_time,const map_location & src,const map_location & dst,int*halo_id,const frame_parameters & animation_val,const frame_parameters & engine_val,const bool primary)const
 {
 	const int xsrc = game_display::get_singleton()->get_location_x(src);
@@ -400,7 +489,7 @@ void unit_frame::redraw(const int frame_time,bool first_time,const map_location
 		if(primary) facing_north = true;
 		game_display::get_singleton()->render_unit_image(x + current_data.x- image->w/2,
 			       	y  + current_data.y- image->h/2,
-			       	(display::tdrawing_layer)(display::LAYER_UNIT_FIRST+current_data.drawing_layer),
+			       	static_cast<display::tdrawing_layer>(display::LAYER_UNIT_FIRST+current_data.drawing_layer),
 			       	src, image, facing_west, false,
 				ftofxp(current_data.highlight_ratio), current_data.blend_with,
 			       	current_data.blend_ratio,current_data.submerge,!facing_north);
@@ -453,7 +542,7 @@ void unit_frame::redraw(const int frame_time,bool first_time,const map_location
 		}
 	}
 }
-bool unit_frame::invalidate(const bool force,const int frame_time,const map_location & src,const map_location & dst,const frame_parameters & animation_val,const frame_parameters & engine_val,const bool primary) const
+std::set<map_location> unit_frame::get_overlaped_hex(const int frame_time,const map_location & src,const map_location & dst,const frame_parameters & animation_val,const frame_parameters & engine_val,const bool primary) const
 {
 	game_display* disp = game_display::get_singleton();
 	const int xsrc = disp->get_location_x(src);
@@ -465,58 +554,44 @@ bool unit_frame::invalidate(const bool force,const int frame_time,const map_loca
 	const frame_parameters current_data = merge_parameters(frame_time,animation_val,engine_val,primary);
 	double tmp_offset = current_data.offset;
 	int d2 = game_display::get_singleton()->hex_size() / 2;
-	bool image_fit_hex = false;
 
 	image::locator image_loc;
 	if(direction != map_location::NORTH && direction != map_location::SOUTH) {
 		image_loc = current_data.image_diagonal;
-		image_fit_hex = image::is_in_hex(image_loc);
 	}
 	if(image_loc.is_void() || image_loc.get_filename() == "") { // invalid diag image, or not diagonal
 		image_loc = current_data.image;
-		image_fit_hex = image::is_in_hex(image_loc);
 	}
 
-	surface image;
-	if(!image_loc.is_void() && image_loc.get_filename() != "") { // invalid diag image, or not diagonal
-		/**
-		 * @todo cache handling: here we will use the image again soon we
-		 * should cache it here and release it (if needed) after redrawn
-		 */
-		image=image::get_image(image_loc,
-				image::SCALED_TO_ZOOM
-				);
-	}
 	// we always invalidate our own hex because we need to be called at redraw time even
 	// if we don't draw anything in the hex itself
-	bool result = false;
-	if (image != NULL) {
-		const int x = static_cast<int>(tmp_offset * xdst + (1.0-tmp_offset) * xsrc)+current_data.x+d2-(image->w/2);
-		const int y = static_cast<int>(tmp_offset * ydst + (1.0-tmp_offset) * ysrc)+current_data.y+d2-(image->h/2);
-		const SDL_Rect r = {x,y,image->w,image->h};
-		// check if the unit fit in a hex
-		bool in_hex = image_fit_hex && r.x==xsrc && r.y==ysrc
-				&& r.w==disp->hex_size() && r.h==disp->hex_size();
-		// check if our underlying hexes are invalidated
-		bool rect_need_update = in_hex ?
-				disp->hex_need_update(src) : disp->rectangle_need_update(r);
-		// if we need to update ourselve because we changed, invalidate our hexes
-		// and return whether or not our hexs was invalidated
-		if(force || need_update() || rect_need_update) {
-			// invalidate ouself to be called at redraw time
-			result |= disp->invalidate(src);
-			if(in_hex == false) {
-				// invalidate all hexes we plan to overwrite
-				result |= disp->invalidate_visible_locations_in_rect(r);
-			}
-		}
+	std::set<map_location> result;
+	if(tmp_offset==0 && current_data.x == 0 && current_data.y == 0 && image::is_in_hex(image_loc)) {
+		result.insert(src);
 	} else {
-		// we have no "redraw surface" but we still need to invalidate our own hex
-		// in case we have a halo and/or sound that needs a redraw
-		if(force || need_update() ){
+		surface image;
+		if(!image_loc.is_void() && image_loc.get_filename() != "") { // invalid diag image, or not diagonal
+			image=image::get_image(image_loc,
+					image::SCALED_TO_ZOOM
+					);
+		}
+		if (image != NULL) {
+			const int x = static_cast<int>(tmp_offset * xdst + (1.0-tmp_offset) * xsrc)+current_data.x+d2-(image->w/2);
+			const int y = static_cast<int>(tmp_offset * ydst + (1.0-tmp_offset) * ysrc)+current_data.y+d2-(image->h/2);
+			const SDL_Rect r = {x,y,image->w,image->h};
+			// check if our underlying hexes are invalidated
+			// if we need to update ourselve because we changed, invalidate our hexes
+			// and return whether or not our hexs was invalidated
 			// invalidate ouself to be called at redraw time
-			result |= disp->invalidate(src);
-			result |= disp->invalidate(dst);
+			result.insert(src);
+			display::rect_of_hexes underlying_hex = disp->hexes_under_rect(r);
+			result.insert(underlying_hex.begin(),underlying_hex.end());
+		} else {
+			// we have no "redraw surface" but we still need to invalidate our own hex
+			// in case we have a halo and/or sound that needs a redraw
+			// invalidate ouself to be called at redraw time
+			result.insert(src);
+			result.insert(dst);
 		}
 	}
 	return result;
@@ -550,9 +625,15 @@ const frame_parameters unit_frame::merge_parameters(int current_time,const frame
 		result.image_diagonal = engine_val.image_diagonal;
 	}
 
-	/** engine provides a string for "stoned" and "team color" modifications */
+	/** engine provides a string for "petrified" and "team color" modifications
+          note that image_mod is the complete modification and halo_mod is only the TC part
+          see unit.cpp, we know that and use it*/
 		result.image_mod = current_val.image_mod +animation_val.image_mod;
-	if(primary)	result.image_mod += engine_val.image_mod;
+	if(primary) {
+                result.image_mod += engine_val.image_mod;
+        } else {
+                result.image_mod += engine_val.halo_mod;
+        }
 
 	assert(engine_val.halo.empty());
 	result.halo = current_val.halo.empty()?animation_val.halo:current_val.halo;
@@ -564,9 +645,8 @@ const frame_parameters unit_frame::merge_parameters(int current_time,const frame
 	result.halo_y = current_val.halo_y?current_val.halo_y:animation_val.halo_y;
 	result.halo_y += engine_val.halo_y;
 
-	/** engine provides hflip modifications */
-		result.halo_mod = current_val.halo_mod +animation_val.halo_mod;
-	if(primary)	result.halo_mod += engine_val.halo_mod;
+        result.halo_mod = current_val.halo_mod +animation_val.halo_mod;
+	result.halo_mod += engine_val.halo_mod;
 
 	assert(engine_val.duration == 0);
 	result.duration = current_val.duration;
@@ -582,11 +662,11 @@ const frame_parameters unit_frame::merge_parameters(int current_time,const frame
 
 	/** engine provide a blend colour for poisoned units */
 	result.blend_with = current_val.blend_with?current_val.blend_with:animation_val.blend_with;
-	if(primary&& engine_val.blend_with) result.blend_with = engine_val.blend_with;
+	if(primary&& engine_val.blend_with) result.blend_with = display::max_rgb(engine_val.blend_with,result.blend_with);
 
 	/** engine provide a blend colour for poisoned units */
 	result.blend_ratio = current_val.blend_ratio?current_val.blend_ratio:animation_val.blend_ratio;
-	if(primary && engine_val.blend_ratio) result.blend_ratio += engine_val.blend_ratio;
+	if(primary && engine_val.blend_ratio) result.blend_ratio = std::min(result.blend_ratio + engine_val.blend_ratio, (double) 1.0);
 
 	/** engine provide a highlight ratio for selected units and visible "invisible" units */
 	result.highlight_ratio = current_val.highlight_ratio!=1.0?current_val.highlight_ratio:animation_val.highlight_ratio;
