@@ -1,6 +1,6 @@
-/* $Id: unit_frame.hpp 35578 2009-05-11 21:45:03Z boucman $ */
+/* $Id: unit_frame.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2006 - 2009 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
+   Copyright (C) 2006 - 2010 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -24,12 +24,12 @@
 #include "util.hpp"
 #include "image.hpp"
 #include "serialization/string_utils.hpp"
-#include "game_display.hpp"
 
 class config;
 
 #include <string>
 #include <vector>
+#include <set>
 
 class progressive_string {
 	public:
@@ -66,28 +66,9 @@ typedef progressive_<double> progressive_double;
 /** All parameters from a frame at a given instant */
 class frame_parameters{
 	public:
-	frame_parameters():
-	image(),
-	image_diagonal(),
-	image_mod(""),
-	halo(""),
-	halo_x(0),
-	halo_y(0),
-	halo_mod(""),
-	sound(""),
-	text(""),
-	text_color(0),
-	duration(0),
-	blend_with(0),
-	blend_ratio(0.0),
-	highlight_ratio(1.0),
-	offset(0),
-	submerge(0.0),
-	x(0),
-	y(0),
-	drawing_layer(display::LAYER_UNIT_DEFAULT-display::LAYER_UNIT_FIRST)
-	{};
+	frame_parameters();
 
+	int duration;
 	image::locator image;
 	image::locator image_diagonal;
 	std::string image_mod;
@@ -98,7 +79,6 @@ class frame_parameters{
 	std::string sound;
 	std::string text;
 	Uint32 text_color;
-	int duration;
 	Uint32 blend_with;
 	double blend_ratio;
 	double highlight_ratio;
@@ -111,41 +91,20 @@ class frame_parameters{
 	bool diagonal_in_hex;
 } ;
 /**
- * keep most parameters in a separate class to simplify handling of large
- * number of parameters
+ * easily build frame parameters with the serialized constructors
  */
+class frame_parsed_parameters;
 class frame_builder {
 	public:
-		/** initial constructor */
-		frame_builder():
-		image_(image::locator()),
-		image_diagonal_(image::locator()),
-		image_mod_(""),
-		halo_(""),
-		halo_x_(""),
-		halo_y_(""),
-		halo_mod_(""),
-		sound_(""),
-		text_(""),
-		text_color_(0),
-		duration_(1),
-		blend_with_(0),
-		blend_ratio_(""),
-		highlight_ratio_(""),
-		offset_(""),
-		submerge_(""),
-		x_(""),
-		y_(""),
-		drawing_layer_(lexical_cast<std::string>(display::LAYER_UNIT_DEFAULT-display::LAYER_UNIT_FIRST))
-	{};
+		frame_builder();
 		frame_builder(const config& cfg,const std::string &frame_string = "");
 		/** allow easy chained modifications will raised assert if used after initialization */
-		frame_builder & image(const image::locator image ,const std::string & image_mod="");
-		frame_builder & image_diagonal(const image::locator image_diagonal,const std::string & image_mod="");
+		frame_builder & duration(const int duration);
+		frame_builder & image(const image::locator& image ,const std::string & image_mod="");
+		frame_builder & image_diagonal(const image::locator& image_diagonal,const std::string & image_mod="");
 		frame_builder & sound(const std::string& sound);
 		frame_builder & text(const std::string& text,const  Uint32 text_color);
 		frame_builder & halo(const std::string &halo, const std::string &halo_x, const std::string& halo_y,const std::string& halo_mod);
-		frame_builder & duration(const int duration);
 		frame_builder & blend(const std::string& blend_ratio,const Uint32 blend_color);
 		frame_builder & highlight(const std::string& highlight);
 		frame_builder & offset(const std::string& offset);
@@ -154,13 +113,50 @@ class frame_builder {
 		frame_builder & y(const std::string& y);
 		frame_builder & drawing_layer(const std::string& drawing_layer);
 		/** getters for the different parameters */
+	private:
+		friend class frame_parsed_parameters;
+		int duration_;
+		image::locator image_;
+		image::locator image_diagonal_;
+		std::string image_mod_;
+		std::string halo_;
+		std::string halo_x_;
+		std::string halo_y_;
+		std::string halo_mod_;
+		std::string sound_;
+		std::string text_;
+		Uint32 text_color_;
+		Uint32 blend_with_;
+		std::string blend_ratio_;
+		std::string highlight_ratio_;
+		std::string offset_;
+		std::string submerge_;
+		std::string x_;
+		std::string y_;
+		std::string drawing_layer_;
+};
+/**
+ * keep most parameters in a separate class to simplify handling of large
+ * number of parameters hanling is common for frame level and animation level
+ */
+class frame_parsed_parameters {
+	public:
+		frame_parsed_parameters(const frame_builder& builder=frame_builder(),int override_duration = 0);
+		/** allow easy chained modifications will raised assert if used after initialization */
+		void override( int duration
+				, const std::string& highlight = ""
+				, const std::string& blend_ratio =""
+				, Uint32 blend_color = 0
+				, const std::string& offset = ""
+				, const std::string& layer = "");
+		/** getters for the different parameters */
 		const frame_parameters parameters(int current_time) const ;
 
 		int duration() const{ return duration_;};
-		void recalculate_duration();
 		bool does_not_change() const;
 		bool need_update() const;
 	private:
+		int duration_;
 		image::locator image_;
 		image::locator image_diagonal_;
 		std::string image_mod_;
@@ -171,7 +167,6 @@ class frame_builder {
 		std::string sound_;
 		std::string text_;
 		Uint32 text_color_;
-		int duration_;
 		Uint32 blend_with_;
 		progressive_double blend_ratio_;
 		progressive_double highlight_ratio_;
@@ -181,7 +176,6 @@ class frame_builder {
 		progressive_int y_;
 		progressive_int drawing_layer_;
 };
-
 /** Describe a unit's animation sequence. */
 class unit_frame {
 	public:
@@ -194,9 +188,9 @@ class unit_frame {
 		int duration() const { return builder_.duration();};
 		bool does_not_change() const{ return builder_.does_not_change();};
 		bool need_update() const{ return builder_.need_update();};
-		bool invalidate(const bool force,const int frame_time,const map_location & src,const map_location & dst,const frame_parameters & animation_val,const frame_parameters & engine_val,const bool primary) const;
+		std::set<map_location> get_overlaped_hex(const int frame_time,const map_location & src,const map_location & dst,const frame_parameters & animation_val,const frame_parameters & engine_val,const bool primary) const;
 	private:
-		frame_builder builder_;
+		frame_parsed_parameters builder_;
 
 };
 
