@@ -1,6 +1,6 @@
-/* $Id: unit_id.cpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: unit_id.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2008 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2008 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -17,12 +17,13 @@
 
 #include <cassert>
 
-#define DBG_UT LOG_STREAM(debug, engine)
+static lg::log_domain log_unit("unit");
+#define DBG_UT LOG_STREAM(debug, log_unit)
 
 namespace n_unit {
 	id_manager id_manager::manager_;
 
-	id_manager::id_manager() : next_id_(0), fake_id_(-1)
+	id_manager::id_manager() : next_id_(0), fake_id_(size_t(-1))
 	{}
 
 	id_manager& id_manager::instance()
@@ -58,7 +59,7 @@ namespace n_unit {
 
 	void id_manager::reset_fake()
 	{
-		fake_id_ = -1;
+		fake_id_ = size_t(-1);
 	}
 
 	void id_manager::clear()
