@@ -1,6 +1,6 @@
-/* $Id: unit_types.hpp 32339 2009-01-27 20:56:56Z shadowmaster $ */
+/* $Id: unit_types.hpp 41676 2010-03-22 10:38:43Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -19,18 +19,17 @@
 #include "map_location.hpp"
 #include "portrait.hpp"
 #include "race.hpp"
-#include "util.hpp"
 
+#include <set>
 #include <string>
 #include <vector>
 
 
-class unit_type;
+class gamemap;
 class unit;
 class unit_ability_list;
+class unit_map;
 class unit_type_data;
-class gamestatus;
-class team;
 
 
 //and how much damage it does.
@@ -57,13 +56,9 @@ public:
 	std::vector<t_string> special_tooltips(bool force=false) const;
 	std::string weapon_specials(bool force=false) const;
 	void set_specials_context(const map_location& aloc,const map_location& dloc,
-                              const unit_map* unitmap,
-							  const gamemap* map, const gamestatus* game_status,
-							  const std::vector<team>* teams,bool attacker,const attack_type* other_attack) const;
+		const unit_map &unitmap, bool attacker, const attack_type *other_attack) const;
 	void set_specials_context(const map_location& loc,const map_location& dloc, const unit& un, bool attacker =true) const;
 
-	bool has_special_by_id(const std::string& special) const;
-
 	bool matches_filter(const config& cfg,bool self=false) const;
 	bool apply_modification(const config& cfg,std::string* description);
 	bool describe_modification(const config& cfg,std::string* description);
@@ -75,14 +70,11 @@ public:
 	mutable map_location aloc_,dloc_;
 	mutable bool attacker_;
 	mutable const unit_map* unitmap_;
-	mutable const gamemap* map_;
-	mutable const gamestatus* game_status_;
-	mutable const std::vector<team>* teams_;
 	mutable const attack_type* other_attack_;
 	/*
 	 * cfg: a weapon special WML structure
 	 */
-	bool special_active(const config& cfg,bool self,bool report=false) const;
+	bool special_active(const config& cfg, bool self) const;
 	bool special_affects_opponent(const config& cfg) const;
 	bool special_affects_self(const config& cfg) const;
 
@@ -91,9 +83,6 @@ public:
 	//animations for this attack. It will not return the same attack
 	//each time.
 	const unit_animation* animation(const game_display& disp, const map_location& loc,const unit* my_unit,const unit_animation::hit_type hit,const attack_type* secondary_attack,int swing_num,int damage) const;
-	// made public to ease backward compatibility for WML syntax
-	// to be removed (with all corresponding code once 1.3.6 is reached
-	std::vector<unit_animation> animation_;
 private:
 	t_string description_;
 	std::string id_;
@@ -127,8 +116,8 @@ public:
 	unit_movement_type();
 
 	const t_string& name() const;
-	int movement_cost(const gamemap& map, t_translation::t_terrain terrain, int recurse_count=0) const;
-	int defense_modifier(const gamemap& map, t_translation::t_terrain terrain, int recurse_count=0) const;
+	int movement_cost(const gamemap& map, t_translation::t_terrain terrain) const;
+	int defense_modifier(const gamemap& map, t_translation::t_terrain terrain) const;
 	int damage_against(const attack_type& attack) const { return resistance_against(attack); }
 	int resistance_against(const attack_type& attack) const;
 
@@ -151,6 +140,15 @@ private:
 	config cfg_;
 };
 
+int movement_cost_internal(std::map<t_translation::t_terrain, int>& move_costs,
+		const config& cfg, const unit_movement_type* parent,
+		const gamemap& map,	t_translation::t_terrain terrain, int recurse_count = 0);
+
+int defense_modifier_internal(std::map<t_translation::t_terrain, int>& defense_mods,
+		const config& cfg, const unit_movement_type* parent,
+		const gamemap& map, t_translation::t_terrain terrain, int recurse_count = 0);
+
+
 typedef std::map<std::string,unit_movement_type> movement_type_map;
 
 class unit_type
@@ -161,20 +159,20 @@ public:
 
 	unit_type();
 	unit_type(const config& cfg, const movement_type_map& movement_types,
-	          const race_map& races, const std::vector<config*>& traits);
+	          const race_map &races, const config::const_child_itors &traits);
 	unit_type(const unit_type& o);
 
 	~unit_type();
 
 	/** Load data into an empty unit_type */
 	void build_full(const config& cfg, const movement_type_map& movement_types,
-	          const race_map& races, const std::vector<config*>& traits);
+	                const race_map &races, const config::const_child_itors &traits);
 	/** Partially load data into an empty unit_type */
     void build_help_index(const config& cfg, const movement_type_map& movement_types,
-	          const race_map& races, const std::vector<config*>& traits);
+	                      const race_map &races, const config::const_child_itors &traits);
 	/** Load the most needed data into an empty unit_type */
     void build_created(const config& cfg, const movement_type_map& mv_types,
-                         const race_map& races, const std::vector<config*>& traits);
+	                   const race_map &races, const config::const_child_itors &traits);
 
 	/**
 	 * Adds an additional advancement path to a unit type.
@@ -182,9 +180,17 @@ public:
 	 */
 	void add_advancement(const unit_type &advance_to,int experience);
 
-	/** Adds units that this unit advances from, for help file purposes. */
-	void add_advancesfrom(const std::string& unit_id);
+	/** Get the advancement tree
+	 *  Build a set of unit type's id of this unit type's advancement tree */
+	std::set<std::string> advancement_tree() const;
 
+	const std::vector<std::string>& advances_to() const { return advances_to_; }
+	const std::vector<std::string> advances_from() const;
+
+	config::const_child_itors modification_advancements() const
+	{ return cfg_.child_range("advancement"); }
+
+	const unit_type& get_gender_unit_type(std::string gender) const;
 	const unit_type& get_gender_unit_type(unit_race::GENDER gender) const;
 	const unit_type& get_variation(const std::string& name) const;
 	/** Info on the type of unit that the unit reanimates as. */
@@ -216,9 +222,6 @@ public:
 	const unit_movement_type& movement_type() const { return movementType_; }
 
 	int experience_needed(bool with_acceleration=true) const;
-	std::vector<std::string> advances_to() const { return advances_to_; }
-	std::vector<std::string> advances_from() const { return advances_from_; }
-	const config::child_list& modification_advancements() const { return cfg_.get_children("advancement"); }
 
 	struct experience_accelerator {
 		experience_accelerator(int modifier);
@@ -249,17 +252,17 @@ public:
 
 	bool has_zoc() const { return zoc_; }
 
-	bool has_ability(const std::string& ability) const;
 	bool has_ability_by_id(const std::string& ability) const;
 	std::vector<std::string> get_ability_list() const;
 
-	const std::vector<config*> possible_traits() const { return possibleTraits_.get_children("trait"); }
-	bool has_random_traits() const { return (num_traits() > 0 && possible_traits().size() > 1); }
+	config::const_child_itors possible_traits() const
+	{ return possibleTraits_.child_range("trait"); }
+	bool has_random_traits() const;
 
 	const std::vector<unit_race::GENDER>& genders() const { return genders_; }
 
-	const std::string& race() const;
-	bool hide_help() const { return hide_help_; }
+	const std::string& race() const { return race_->id(); }
+	bool hide_help() const;
 
     enum BUILD_STATUS {NOT_BUILT, CREATED, HELP_INDEX, WITHOUT_ANIMATIONS, FULL};
 
@@ -267,7 +270,7 @@ public:
 
 	const std::vector<tportrait>& portraits() const { return portraits_; }
 
-    const config& get_cfg() { return cfg_; }
+	const config &get_cfg() const { return cfg_; }
 
 	void set_config(const config& cfg);
 
@@ -308,7 +311,6 @@ private:
 	bool zoc_, hide_help_;
 
 	std::vector<std::string> advances_to_;
-	std::vector<std::string> advances_from_;
 	int experience_needed_;
 
 
@@ -332,71 +334,53 @@ private:
 class unit_type_data
 {
 public:
-    static unit_type_data& instance() {
-        if (instance_ == NULL)
-            instance_ = new unit_type_data();
+	unit_type_data();
 
-        return *instance_;
-    }
+	typedef std::map<std::string,unit_type> unit_type_map;
 
-    typedef std::map<std::string,unit_type> unit_type_map;
+	const unit_type_map &types() const { return types_; }
+	const race_map &races() const { return races_; }
+	void set_config(config &cfg);
 
-	class unit_type_map_wrapper
-	{
-        friend class unit_type_data;
+	bool unit_type_exists(const std::string &key) const;
+	const unit_type *find(const std::string &key, unit_type::BUILD_STATUS status = unit_type::FULL) const;
+	const unit_race *find_race(const std::string &) const;
 
-		public:
-            const race_map& races() const { return races_; }
-            void set_config(const config& cfg);
+	void build_all(unit_type::BUILD_STATUS status);
 
-			unit_type_map::const_iterator begin() const { return types_.begin(); }
-			unit_type_map::const_iterator end() const { return types_.end(); }
-			bool unit_type_exists(const std::string& key) const;
-            unit_type_map::const_iterator find_unit_type(const std::string& key, unit_type::BUILD_STATUS status = unit_type::FULL) const;
-
-            void build_all(unit_type::BUILD_STATUS status) const;
-
-        private:
-            unit_type_map_wrapper();
-            unit_type_map_wrapper(unit_type_map_wrapper& /*utf*/) :
-				types_(),
-				dummy_unit_map_(),
-				movement_types_(),
-				races_(),
-				unit_traits_(),
-				unit_cfg_(0)
-			{}
-
-            void set_unit_config(const config& unit_cfg) { unit_cfg_ = &unit_cfg; }
-            void set_unit_traits(const config::child_list unit_traits) { unit_traits_ = unit_traits; }
-
-			const config& find_config(const std::string& key) const;
-			std::pair<unit_type_map::iterator, bool> insert(const std::pair<std::string,unit_type>& utype) { return types_.insert(utype); }
-			void clear() {
-			    types_.clear();
-			    movement_types_.clear();
-			    races_.clear();
-            }
-
-            unit_type& build_unit_type(const std::string& key, unit_type::BUILD_STATUS status) const;
-            void add_advancefrom(const config& unit_cfg) const;
-            void add_advancement(unit_type& to_unit) const;
-
-            mutable unit_type_map types_;
-            mutable unit_type_map dummy_unit_map_;
-            movement_type_map movement_types_;
-            race_map races_;
-            config::child_list unit_traits_;
-            const config* unit_cfg_;
-	};
-
-    static unit_type_map_wrapper& types() { return instance().unit_types_; }
+	/** Checks if the [hide_help] tag contains these IDs. */
+	bool hide_help(const std::string &type_id, const std::string &race_id) const;
 
 private:
-    static unit_type_data* instance_;
-    mutable unit_type_map_wrapper unit_types_;
+	unit_type_data(const unit_type_data &);
 
-    unit_type_data();
+	/** Parses the [hide_help] tag. */
+	void read_hide_help(const config &cfg);
+
+	void set_unit_config(const config& unit_cfg) { unit_cfg_ = &unit_cfg; }
+
+	const config &find_config(const std::string &key) const;
+	std::pair<unit_type_map::iterator, bool> insert(const std::pair<std::string, unit_type> &utype) { return types_.insert(utype); }
+	void clear();
+
+	unit_type& build_unit_type(const unit_type_map::iterator &ut, unit_type::BUILD_STATUS status) const;
+	void add_advancefrom(const config& unit_cfg) const;
+	void add_advancement(unit_type& to_unit) const;
+
+	mutable unit_type_map types_;
+	movement_type_map movement_types_;
+	race_map races_;
+
+	/** True if [hide_help] contains a 'all=yes' at its root. */
+	bool hide_help_all_;
+	// vectors containing the [hide_help] and its sub-tags [not]
+	std::vector< std::set<std::string> > hide_help_type_;
+	std::vector< std::set<std::string> > hide_help_race_;
+
+	const config *unit_cfg_;
+	unit_type::BUILD_STATUS build_status_;
 };
 
+extern unit_type_data unit_types;
+
 #endif
