@@ -1,7 +1,7 @@
-/* $Id: variable.cpp 33100 2009-02-26 01:34:22Z sapient $ */
+/* $Id: variable.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2009 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2010 by Philippe Plantier <ayin@anathas.org>
 
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -21,14 +21,21 @@
 
 #include "global.hpp"
 
+#include "variable.hpp"
+
+#include "foreach.hpp"
+#include "formula_string_utils.hpp"
 #include "gamestatus.hpp"
 #include "log.hpp"
-#include "formula_string_utils.hpp"
+#include "resources.hpp"
+#include "unit.hpp"
+#include "unit_map.hpp"
+#include "team.hpp"
 
-
-#define LOG_NG LOG_STREAM(info, engine)
-#define WRN_NG LOG_STREAM(warn, engine)
-#define ERR_NG LOG_STREAM(err, engine)
+static lg::log_domain log_engine("engine");
+#define LOG_NG LOG_STREAM(info, log_engine)
+#define WRN_NG LOG_STREAM(warn, log_engine)
+#define ERR_NG LOG_STREAM(err, log_engine)
 
 namespace
 {
@@ -36,7 +43,7 @@ namespace
 	 * @todo FIXME: the variable repository should be
 	 * a class of variable.hpp, and not the game_state.
 	 */
-	game_state* repos = NULL;
+	#define repos (resources::state_of_game)
 
 	// keeps track of insert_tag variables used by get_parsed_config
 	std::set<std::string> vconfig_recursion;
@@ -174,12 +181,12 @@ vconfig::vconfig(const config* cfg, const config * cache_key) :
 	}
 }
 
-vconfig::vconfig(const config* cfg, bool is_volatile) :
-	cfg_(cfg), cache_key_(cfg)
+vconfig::vconfig(const config &cfg, bool is_volatile) :
+	cfg_(&cfg), cache_key_(&cfg)
 {
 	if(is_volatile) {
 		increment_config_usage(cache_key_);
-		if(cache_key_ != cfg) {
+		if (cache_key_ != &cfg) {
 			//location of volatile cfg has moved
 			cfg_ = cache_key_;
 		}
@@ -209,10 +216,10 @@ vconfig& vconfig::operator=(const vconfig& cfg)
 	return *this;
 }
 
-vconfig& vconfig::operator=(const config* cfg)
+vconfig& vconfig::operator=(const config &cfg)
 {
-	if(cfg_ != cfg) {
-		cfg_ = cfg;
+	if (cfg_ != &cfg) {
+		cfg_ = &cfg;
 		decrement_config_usage(cache_key_);
 		cache_key_ = NULL;
 	}
@@ -223,18 +230,14 @@ const config vconfig::get_parsed_config() const
 {
 	config res;
 
-	for(string_map::const_iterator itor = cfg_->values.begin();
-		itor != cfg_->values.end(); ++itor)
-	{
-		res[itor->first] = expand(itor->first);
+	foreach (const config::attribute &i, cfg_->attribute_range()) {
+		res[i.first] = expand(i.first);
 	}
 
-	for(config::all_children_iterator child = cfg_->ordered_begin();
-		child != cfg_->ordered_end(); ++child)
+	foreach (const config::any_child &child, cfg_->all_children_range())
 	{
-		const std::string &child_key = *(*child).first;
-		if(child_key == "insert_tag") {
-			vconfig insert_cfg(child->second);
+		if (child.key == "insert_tag") {
+			vconfig insert_cfg(child.cfg);
 			const t_string& name = insert_cfg["name"];
 			const t_string& vname = insert_cfg["variable"];
 			if(!vconfig_recursion.insert(vname).second) {
@@ -245,14 +248,14 @@ const config vconfig::get_parsed_config() const
 				if(!vinfo.is_valid) {
 					res.add_child(name); //add empty tag
 				} else if(vinfo.explicit_index) {
-					res.add_child(name, vconfig(&(vinfo.as_container())).get_parsed_config());
+					res.add_child(name, vconfig(vinfo.as_container()).get_parsed_config());
 				} else {
 					variable_info::array_range range = vinfo.as_array();
 					if(range.first == range.second) {
 						res.add_child(name); //add empty tag
 					}
 					while(range.first != range.second) {
-						res.add_child(name, vconfig(*range.first++).get_parsed_config());
+						res.add_child(name, vconfig(**range.first++).get_parsed_config());
 					}
 				}
 				vconfig_recursion.erase(vname);
@@ -267,7 +270,7 @@ const config vconfig::get_parsed_config() const
 				}
 			}
 		} else {
-			res.add_child(child_key, vconfig((*child).second).get_parsed_config());
+			res.add_child(child.key, vconfig(child.cfg).get_parsed_config());
 		}
 	}
 	return res;
@@ -277,19 +280,17 @@ vconfig::child_list vconfig::get_children(const std::string& key) const
 {
 	vconfig::child_list res;
 
-	for(config::all_children_iterator child = cfg_->ordered_begin();
-		child != cfg_->ordered_end(); ++child)
+	foreach (const config::any_child &child, cfg_->all_children_range())
 	{
-		const std::string &child_key = *(*child).first;
-		if(child_key == key) {
-			res.push_back(vconfig(child->second, cache_key_));
-		} else if(child_key == "insert_tag") {
-			vconfig insert_cfg(child->second);
+		if (child.key == key) {
+			res.push_back(vconfig(&child.cfg, cache_key_));
+		} else if (child.key == "insert_tag") {
+			vconfig insert_cfg(child.cfg);
 			if(insert_cfg["name"] == key) {
 				variable_info vinfo(insert_cfg["variable"], false, variable_info::TYPE_CONTAINER);
 				if(!vinfo.is_valid) {
 					//push back an empty tag
-					res.push_back(vconfig(&empty_config));
+					res.push_back(vconfig(empty_config));
 				} else if(vinfo.explicit_index) {
 					config * cp = &(vinfo.as_container());
 					res.push_back(vconfig(cp, cp));
@@ -297,7 +298,7 @@ vconfig::child_list vconfig::get_children(const std::string& key) const
 					variable_info::array_range range = vinfo.as_array();
 					if(range.first == range.second) {
 						//push back an empty tag
-						res.push_back(vconfig(&empty_config));
+						res.push_back(vconfig(empty_config));
 					}
 					while(range.first != range.second) {
 						config * cp = *range.first++;
@@ -312,19 +313,16 @@ vconfig::child_list vconfig::get_children(const std::string& key) const
 
 vconfig vconfig::child(const std::string& key) const
 {
-	const config *natural = cfg_->child(key);
-	if(natural)
-	{
-		return vconfig(natural, cache_key_);
+	if (const config &natural = cfg_->child(key)) {
+		return vconfig(&natural, cache_key_);
 	}
-	for(config::const_child_itors chitors = cfg_->child_range("insert_tag");
-		chitors.first != chitors.second; ++chitors.first)
+	foreach (const config &ins, cfg_->child_range("insert_tag"))
 	{
-		vconfig insert_cfg(*chitors.first);
+		vconfig insert_cfg(ins);
 		if(insert_cfg["name"] == key) {
 			variable_info vinfo(insert_cfg["variable"], false, variable_info::TYPE_CONTAINER);
 			if(!vinfo.is_valid) {
-				return vconfig(&empty_config);
+				return vconfig(empty_config);
 			}
 			config * cp = &(vinfo.as_container());
 			return vconfig(cp, cp);
@@ -338,13 +336,12 @@ bool vconfig::has_child(const std::string& key) const
 	if(!cfg_) {
 		return false;
 	}
-	if(cfg_->child(key) != NULL) {
+	if (cfg_->child(key)) {
 		return true;
 	}
-	for(config::const_child_itors chitors = cfg_->child_range("insert_tag");
-		chitors.first != chitors.second; ++chitors.first)
+	foreach (const config &ins, cfg_->child_range("insert_tag"))
 	{
-		vconfig insert_cfg(*chitors.first);
+		vconfig insert_cfg(ins);
 		if(insert_cfg["name"] == key) {
 			return true;
 		}
@@ -367,15 +364,16 @@ const t_string vconfig::expand(const std::string& key) const
 	return t_string(val);
 }
 
-vconfig::all_children_iterator::all_children_iterator(config::all_children_iterator i, const config *cache_key)
+vconfig::all_children_iterator::all_children_iterator(const Itor &i, const config *cache_key)
 : i_(i), inner_index_(0), index_offset_(0), cache_key_(cache_key)
 {
 }
 
 vconfig::all_children_iterator& vconfig::all_children_iterator::operator++()
 {
-	if(i_.get_key() == "insert_tag") {
-		variable_info vinfo(vconfig(&i_.get_child())["variable"], false, variable_info::TYPE_CONTAINER);
+	if (i_->key == "insert_tag")
+	{
+		variable_info vinfo(vconfig(i_->cfg)["variable"], false, variable_info::TYPE_CONTAINER);
 		if(vinfo.is_valid && !vinfo.explicit_index) {
 			variable_info::array_range range = vinfo.as_array();
 			if(range.first != range.second && range.first + (++inner_index_) != range.second) {
@@ -408,20 +406,21 @@ vconfig::all_children_iterator::pointer vconfig::all_children_iterator::operator
 
 std::string vconfig::all_children_iterator::get_key() const
 {
-	const std::string& key = i_.get_key();
-	if(key == "insert_tag") {
-		return vconfig(&i_.get_child())["name"];
+	const std::string &key = i_->key;
+	if (key == "insert_tag") {
+		return vconfig(i_->cfg)["name"];
 	}
 	return key;
 }
 
 const vconfig vconfig::all_children_iterator::get_child() const
 {
-	if(i_.get_key() == "insert_tag") {
+	if (i_->key == "insert_tag")
+	{
 		config * cp;
-		variable_info vinfo(vconfig(&i_.get_child())["variable"], false, variable_info::TYPE_CONTAINER);
+		variable_info vinfo(vconfig(i_->cfg)["variable"], false, variable_info::TYPE_CONTAINER);
 		if(!vinfo.is_valid) {
-			return vconfig(&empty_config);
+			return vconfig(empty_config);
 		} else if(inner_index_ == 0) {
 			cp = &(vinfo.as_container());
 			return vconfig(cp, cp);
@@ -429,7 +428,7 @@ const vconfig vconfig::all_children_iterator::get_child() const
 		cp = *(vinfo.as_array().first + inner_index_);
 		return vconfig(cp, cp);
 	}
-	return vconfig(&i_.get_child(), cache_key_);
+	return vconfig(&i_->cfg, cache_key_);
 }
 
 size_t vconfig::all_children_iterator::get_index() const
@@ -459,14 +458,8 @@ vconfig::all_children_iterator vconfig::ordered_end() const
 
 namespace variable
 {
-	manager::manager(game_state* repository)
-	{
-		repos = repository;
-	}
-
 	manager::~manager()
 	{
-		repos = NULL;
 		hash_memory.clear();
 	}
 }
@@ -481,9 +474,8 @@ scoped_wml_variable::scoped_wml_variable(const std::string& var_name) :
 
 void scoped_wml_variable::store(const config& var_value)
 {
-	const config::child_list& children = repos->get_variables().get_children(var_name_);
-	for(config::child_list::const_iterator i = children.begin(); i != children.end(); ++i) {
-		previous_val_.append(**i);
+	foreach (const config &i, repos->get_variables().child_range(var_name_)) {
+		previous_val_.add_child(var_name_, i);
 	}
 	repos->clear_variable_cfg(var_name_);
 	repos->add_variable_cfg(var_name_, var_value);
@@ -495,9 +487,8 @@ scoped_wml_variable::~scoped_wml_variable()
 {
 	if(activated_) {
 		repos->clear_variable_cfg(var_name_);
-		config::child_list old_val =previous_val_.get_children(var_name_);
-		for(config::child_list::iterator j=old_val.begin(); j != old_val.end() ; j++){
-			repos->add_variable_cfg(var_name_,**j);
+		foreach (const config &i, previous_val_.child_range(var_name_)) {
+			repos->add_variable_cfg(var_name_, i);
 		}
 		LOG_NG << "scoped_wml_variable: var_name \"" << var_name_ << "\" has been reverted.\n";
 	}
@@ -507,33 +498,40 @@ scoped_wml_variable::~scoped_wml_variable()
 
 void scoped_xy_unit::activate()
 {
-	unit_map::const_iterator itor = umap_.find(map_location(x_,y_));
+	map_location loc = map_location(x_, y_);
+	unit_map::const_iterator itor = umap_.find(loc);
 	if(itor != umap_.end()) {
 		config tmp_cfg;
 		itor->second.write(tmp_cfg);
 		tmp_cfg["x"] = lexical_cast<std::string,int>(x_ + 1);
 		tmp_cfg["y"] = lexical_cast<std::string,int>(y_ + 1);
-		LOG_NG << "auto-storing $" << name() << " at (" << x_ << ',' << y_ << ")\n";
+		LOG_NG << "auto-storing $" << name() << " at (" << loc << ")\n";
 		store(tmp_cfg);
 	} else {
-		ERR_NG << "failed to auto-store $" << name() << " at (" << x_ << ',' << y_ << ")\n";
+		ERR_NG << "failed to auto-store $" << name() << " at (" << loc << ")\n";
 	}
 }
 
 void scoped_weapon_info::activate()
 {
-	if(data_ != NULL) {
-		store(*data_);
+	if (data_) {
+		store(data_);
 	}
 }
 
 void scoped_recall_unit::activate()
 {
-	player_info* const player = repos->get_player(player_);
-	if(player != NULL) {
-		if(player->available_units.size() > recall_index_) {
+	const std::vector<team>& teams = teams_manager::get_teams();
+	std::vector<team>::const_iterator team_it;
+	for (team_it = teams.begin(); team_it != teams.end(); ++team_it) {
+		if (team_it->save_id() == player_ )
+			break;
+	}
+
+	if(team_it != teams.end()) {
+		if(team_it->recall_list().size() > recall_index_) {
 			config tmp_cfg;
-			player->available_units[recall_index_].write(tmp_cfg);
+			team_it->recall_list()[recall_index_].write(tmp_cfg);
 			tmp_cfg["x"] = "recall";
 			tmp_cfg["y"] = "recall";
 			LOG_NG << "auto-storing $" << name() << " for player: " << player_
@@ -635,7 +633,7 @@ variable_info::variable_info(const std::string& varname,
 			case variable_info::TYPE_CONTAINER:
 				WRN_NG << "variable_info: using reserved WML variable as wrong type, "
 					<< varname << std::endl;
-				is_valid = force_valid || repos->temporaries.child(varname) != NULL;
+				is_valid = force_valid || repos->temporaries.child(varname);
 				break;
 			case variable_info::TYPE_SCALAR:
 			default:
@@ -681,7 +679,7 @@ variable_info::variable_info(const std::string& varname,
 		case variable_info::TYPE_ARRAY:
 			vars = vars->get_children(key)[index];
 			key = "__array";
-			is_valid = force_valid || vars->child(key) != NULL;
+			is_valid = force_valid || vars->child(key);
 			break;
 		case variable_info::TYPE_SCALAR:
 			vars = vars->get_children(key)[index];
@@ -694,8 +692,10 @@ variable_info::variable_info(const std::string& varname,
 			is_valid = true;
 			return;
 		}
-		WRN_NG << "variable_info: using explicitly indexed Container as wrong WML type, "
-			<< varname << std::endl;
+		if (force_valid) {
+			WRN_NG << "variable_info: using explicitly indexed "
+				"container as wrong WML type, " << varname << '\n';
+		}
 		explicit_index = false;
 		index = 0;
 	} else {
@@ -718,7 +718,7 @@ variable_info::variable_info(const std::string& varname,
 
 t_string& variable_info::as_scalar() {
 	assert(is_valid);
-	return vars->values[key];
+	return (*vars)[key];
 }
 
 config& variable_info::as_container() {
@@ -727,10 +727,9 @@ config& variable_info::as_container() {
 		// Empty data for explicit index was already created if it was needed
 		return *vars->get_children(key)[index];
 	}
-	config *temp = vars->child(key);
-	if(temp) {
+	if (config &temp = vars->child(key)) {
 		// The container exists, index not specified, return index 0
-		return *temp;
+		return temp;
 	}
 	// Add empty data for the new variable, since it does not exist yet
 	return vars->add_child(key);
@@ -738,5 +737,5 @@ config& variable_info::as_container() {
 
 variable_info::array_range variable_info::as_array() {
 	assert(is_valid);
-	return vars->child_range(key);
+	return vars->child_range_bak(key);
 }
