@@ -1,6 +1,6 @@
-/* $Id: video.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: video.cpp 40558 2010-01-03 18:45:08Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,74 +12,35 @@
    See the COPYING file for more details.
 */
 
-//! @file video.cpp
-//! Video-testprogram, standalone
+/**
+ *  @file video.cpp
+ *  Video-testprogram, standalone
+ */
 
 #include "global.hpp"
 
-#include <stdio.h>
-#include <iostream>
-#include <vector>
-
-#include "events.hpp"
 #include "font.hpp"
+#include "foreach.hpp"
 #include "image.hpp"
 #include "log.hpp"
 #include "preferences_display.hpp"
 #include "video.hpp"
 
-#define LOG_DP LOG_STREAM(info, display)
-#define ERR_DP LOG_STREAM(err, display)
+#include <vector>
+#include <map>
+#include <algorithm>
 
-#define TEST_VIDEO_ON 0
-
-#if (TEST_VIDEO_ON==1)
-
-#include <stdlib.h>
-
-// Testprogram takes three args: x-res y-res colour-depth
-int main( int argc, char** argv )
-{
-	if( argc != 4 ) {
-		printf( "usage: %s x-res y-res bitperpixel\n", argv[0] );
-		return 1;
-	}
-	SDL_Init(SDL_INIT_VIDEO);
-	CVideo video;
-
-	printf( "args: %s, %s, %s\n", argv[1], argv[2], argv[3] );
-
-	printf( "(%d,%d,%d)\n", strtoul(argv[1],0,10), strtoul(argv[2],0,10),
-	                        strtoul(argv[3],0,10) );
-
-	if( video.setMode( strtoul(argv[1],0,10), strtoul(argv[2],0,10),
-	                        strtoul(argv[3],0,10), FULL_SCREEN ) ) {
-		printf( "video mode possible\n" );
-	} else  printf( "video mode NOT possible\n" );
-	printf( "%d, %d, %d\n", video.getx(), video.gety(),
-	                        video.getBitsPerPixel() );
-
-	for( int s = 0; s < 50; s++ ) {
-		video.lock();
-		for( int i = 0; i < video.getx(); i++ )
-				video.setPixel( i, 90, 255, 0, 0 );
-		if( s%10==0)
-			printf( "frame %d\n", s );
-		video.unlock();
-		video.update( 0, 90, video.getx(), 1 );
-	}
-	return 0;
-}
-
-#endif
+static lg::log_domain log_display("display");
+#define LOG_DP LOG_STREAM(info, log_display)
+#define ERR_DP LOG_STREAM(err, log_display)
 
 namespace {
 	bool fullScreen = false;
 	int disallow_resize = 0;
 }
 void resize_monitor::process(events::pump_info &info) {
-	if(info.resize_dimensions.first >= min_allowed_width
-	&& info.resize_dimensions.second >= min_allowed_height
+	if(info.resize_dimensions.first >= preferences::min_allowed_width()
+	&& info.resize_dimensions.second >= preferences::min_allowed_height()
 	&& disallow_resize == 0) {
 		preferences::set_resolution(info.resize_dimensions);
 	}
@@ -109,12 +70,129 @@ static unsigned int get_flags(unsigned int flags)
 }
 
 namespace {
-std::vector<SDL_Rect> update_rects;
-bool update_all = false;
+struct event {
+	int x, y, w, h;
+	bool in;
+	event(const SDL_Rect& rect, bool i) : x(i ? rect.x : rect.x + rect.w), y(rect.y), w(rect.w), h(rect.h), in(i) { }
+};
+bool operator<(const event& a, const event& b) {
+	if (a.x != b.x) return a.x < b.x;
+	if (a.in != b.in) return a.in;
+	if (a.y != b.y) return a.y < b.y;
+	if (a.h != b.h) return a.h < b.h;
+	if (a.w != b.w) return a.w < b.w;
+	return false;
+}
+bool operator==(const event& a, const event& b) {
+	return a.x == b.x && a.y == b.y && a.w == b.w && a.h == b.h && a.in == b.in;
 }
 
-static bool rect_contains(const SDL_Rect& a, const SDL_Rect& b) {
-	return a.x <= b.x && a.y <= b.y && a.x+a.w >= b.x+b.w && a.y+a.h >= b.y+b.h;
+struct segment {
+	int x, count;
+	segment() : x(0), count(0) { }
+	segment(int x, int count) : x(x), count(count) { }
+};
+
+
+std::vector<SDL_Rect> update_rects;
+std::vector<event> events;
+std::map<int, segment> segments;
+
+static void calc_rects()
+{
+	events.clear();
+
+	foreach (SDL_Rect const &rect, update_rects) {
+		events.push_back(event(rect, true));
+		events.push_back(event(rect, false));
+	}
+
+	std::sort(events.begin(), events.end());
+	std::vector<event>::iterator events_end = std::unique(events.begin(), events.end());
+
+	segments.clear();
+	update_rects.clear();
+
+	for (std::vector<event>::iterator iter = events.begin(); iter != events_end; ++iter) {
+		std::map<int, segment>::iterator lower = segments.find(iter->y);
+		if (lower == segments.end()) {
+			lower = segments.insert(std::make_pair(iter->y, segment())).first;
+			if (lower != segments.begin()) {
+				std::map<int, segment>::iterator prev = lower;
+				--prev;
+				lower->second = prev->second;
+			}
+		}
+
+		if (lower->second.count == 0) {
+			lower->second.x = iter->x;
+		}
+
+		std::map<int, segment>::iterator upper = segments.find(iter->y + iter->h);
+		if (upper == segments.end()) {
+			upper = segments.insert(std::make_pair(iter->y + iter->h, segment())).first;
+			std::map<int, segment>::iterator prev = upper;
+			--prev;
+			upper->second = prev->second;
+		}
+
+		if (iter->in) {
+			while (lower != upper) {
+				++lower->second.count;
+				++lower;
+			}
+		} else {
+			while (lower != upper) {
+				lower->second.count--;
+				if (lower->second.count == 0) {
+					std::map<int, segment>::iterator next = lower;
+					++next;
+
+					int x = lower->second.x, y = lower->first;
+					unsigned w = iter->x - x;
+					unsigned h = next->first - y;
+					SDL_Rect a = {x, y, w, h};
+
+					if (update_rects.empty()) {
+						update_rects.push_back(a);
+					} else {
+						SDL_Rect& p = update_rects.back(), n;
+						int pa = p.w * p.h, aa = w * h, s = pa + aa;
+						int thresh = 51;
+
+						n.w = std::max<int>(x + w, p.x + p.w);
+						n.x = std::min<int>(p.x, x);
+						n.w -= n.x;
+						n.h = std::max<int>(y + h, p.y + p.h);
+						n.y = std::min<int>(p.y, y);
+						n.h -= n.y;
+
+						if (s * 100 < thresh * n.w * n.h) {
+							update_rects.push_back(a);
+						} else {
+							p = n;
+						}
+					}
+
+					if (lower == segments.begin()) {
+						segments.erase(lower);
+					} else {
+						std::map<int, segment>::iterator prev = lower;
+						--prev;
+						if (prev->second.count == 0) segments.erase(lower);
+					}
+
+					lower = next;
+				} else {
+					++lower;
+				}
+			}
+		}
+	}
+}
+
+
+bool update_all = false;
 }
 
 static void clear_updates()
@@ -126,11 +204,13 @@ static void clear_updates()
 namespace {
 
 surface frameBuffer = NULL;
-
+bool fake_interactive = false;
 }
 
 bool non_interactive()
 {
+	if (fake_interactive)
+		return false;
 	return SDL_GetVideoSurface() == NULL;
 }
 
@@ -203,24 +283,6 @@ void update_rect(const SDL_Rect& rect_value)
 		}
 	}
 
-	for(std::vector<SDL_Rect>::iterator i = update_rects.begin();
-	    i != update_rects.end(); ++i) {
-		if(rect_contains(*i,rect)) {
-			return;
-		}
-
-		if(rect_contains(rect,*i)) {
-			*i = rect;
-			for(++i; i != update_rects.end(); ++i) {
-				if(rect_contains(rect,*i)) {
-					i->w = 0;
-				}
-			}
-
-			return;
-		}
-	}
-
 	update_rects.push_back(rect);
 }
 
@@ -228,28 +290,28 @@ void update_whole_screen()
 {
 	update_all = true;
 }
-CVideo::CVideo() : bpp(0), fake_screen(false), help_string_(0), updatesLocked_(0)
+CVideo::CVideo(FAKE_TYPES type) : mode_changed_(false), bpp_(0), fake_screen_(false), help_string_(0), updatesLocked_(0)
 {
-	const int res = SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE);
-
-	if(res < 0) {
-		ERR_DP << "Could not initialize SDL: " << SDL_GetError() << "\n";
-		throw CVideo::error();
+	initSDL();
+	switch(type)
+	{
+		case NO_FAKE:
+			break;
+		case FAKE:
+			make_fake();
+			break;
+		case FAKE_TEST:
+			make_test_fake();
+			break;
 	}
 }
 
-CVideo::CVideo( int x, int y, int bits_per_pixel, int flags)
-		 : bpp(0), fake_screen(false), help_string_(0), updatesLocked_(0)
+void CVideo::initSDL()
 {
-	const int res = SDL_Init( SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE);
-	if(res < 0) {
-		ERR_DP << "Could not initialize SDL: " << SDL_GetError() << "\n";
-		throw CVideo::error();
-	}
+	const int res = SDL_InitSubSystem(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE);
 
-	const int mode_res = setMode( x, y, bits_per_pixel, flags );
-	if (mode_res == 0) {
-		ERR_DP << "Could not set Video Mode\n";
+	if(res < 0) {
+		ERR_DP << "Could not initialize SDL_video: " << SDL_GetError() << "\n";
 		throw CVideo::error();
 	}
 }
@@ -276,11 +338,22 @@ void CVideo::blit_surface(int x, int y, surface surf, SDL_Rect* srcrect, SDL_Rec
 
 void CVideo::make_fake()
 {
-	fake_screen = true;
+	fake_screen_ = true;
 	frameBuffer = SDL_CreateRGBSurface(SDL_SWSURFACE,16,16,24,0xFF0000,0xFF00,0xFF,0);
 	image::set_pixel_format(frameBuffer->format);
 }
 
+void CVideo::make_test_fake(const unsigned width,
+			const unsigned height, const unsigned bpp)
+{
+	frameBuffer = SDL_CreateRGBSurface(SDL_SWSURFACE,
+			width, height, bpp, 0xFF0000, 0xFF00, 0xFF, 0);
+	image::set_pixel_format(frameBuffer->format);
+
+	fake_interactive = true;
+
+}
+
 int CVideo::modePossible( int x, int y, int bits_per_pixel, int flags )
 {
 	return SDL_VideoModeOK( x, y, bits_per_pixel, get_flags(flags) );
@@ -289,7 +362,7 @@ int CVideo::modePossible( int x, int y, int bits_per_pixel, int flags )
 int CVideo::setMode( int x, int y, int bits_per_pixel, int flags )
 {
 	update_rects.clear();
-	if (fake_screen) return 0;
+	if (fake_screen_) return 0;
 	mode_changed_ = true;
 
 	flags = get_flags(flags);
@@ -314,13 +387,6 @@ bool CVideo::modeChanged()
 	return ret;
 }
 
-int CVideo::setGamma(float gamma)
-{
-	SDL_SetGamma(gamma, gamma, gamma);
-
-	return 0;
-}
-
 int CVideo::getx() const
 {
 	return frameBuffer->w;
@@ -331,49 +397,17 @@ int CVideo::gety() const
 	return frameBuffer->h;
 }
 
-int CVideo::getBitsPerPixel()
-{
-	return frameBuffer->format->BitsPerPixel;
-}
-
-int CVideo::getBytesPerPixel()
-{
-	return frameBuffer->format->BytesPerPixel;
-}
-
-int CVideo::getRedMask()
-{
-	return frameBuffer->format->Rmask;
-}
-
-int CVideo::getGreenMask()
-{
-	return frameBuffer->format->Gmask;
-}
-
-int CVideo::getBlueMask()
-{
-	return frameBuffer->format->Bmask;
-}
-
 void CVideo::flip()
 {
-	if(fake_screen)
+	if(fake_screen_)
 		return;
 
 	if(update_all) {
 		::SDL_Flip(frameBuffer);
 	} else if(update_rects.empty() == false) {
-		size_t sum = 0;
-		for(size_t n = 0; n != update_rects.size(); ++n) {
-			sum += update_rects[n].w*update_rects[n].h;
-		}
-
-		const size_t redraw_whole_screen_threshold = 80;
-		if(sum > ((getx()*gety())*redraw_whole_screen_threshold)/100) {
-			::SDL_Flip(frameBuffer);
-		} else {
-			SDL_UpdateRects(frameBuffer,update_rects.size(),&update_rects[0]);
+		calc_rects();
+		if(!update_rects.empty()) {
+			SDL_UpdateRects(frameBuffer, update_rects.size(), &update_rects[0]);
 		}
 	}
 
@@ -393,24 +427,7 @@ bool CVideo::update_locked() const
 	return updatesLocked_ > 0;
 }
 
-void CVideo::lock()
-{
-	if( SDL_MUSTLOCK(frameBuffer) )
-		SDL_LockSurface( frameBuffer );
-}
-
-void CVideo::unlock()
-{
-	if( SDL_MUSTLOCK(frameBuffer) )
-		SDL_UnlockSurface( frameBuffer );
-}
-
-int CVideo::mustLock()
-{
-	return SDL_MUSTLOCK(frameBuffer);
-}
-
-surface CVideo::getSurface( void )
+surface CVideo::getSurface()
 {
 	return frameBuffer;
 }
@@ -419,19 +436,19 @@ bool CVideo::isFullScreen() const { return fullScreen; }
 
 void CVideo::setBpp( int bpp )
 {
-	this->bpp = bpp;
+	bpp_ = bpp;
 }
 
-int CVideo::getBpp( void )
+int CVideo::getBpp()
 {
-	return bpp;
+	return bpp_;
 }
 
 int CVideo::set_help_string(const std::string& str)
 {
 	font::remove_floating_label(help_string_);
 
-	const SDL_Color colour = {0x0,0x00,0x00,0x77};
+	const SDL_Color colour = { 0, 0, 0, 0xbb };
 
 #ifdef USE_TINY_GUI
 	int size = font::SIZE_NORMAL;
