@@ -1,6 +1,6 @@
-/* $Id: video.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: video.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,8 @@
 #include "SDL.h"
 #include "sdl_utils.hpp"
 
+#include <boost/utility.hpp>
+
 //possible flags when setting video modes
 #define FULL_SCREEN SDL_FULLSCREEN
 #define VIDEO_MEMORY SDL_HWSURFACE
@@ -34,39 +36,32 @@ void update_rect(size_t x, size_t y, size_t w, size_t h);
 void update_rect(const SDL_Rect& rect);
 void update_whole_screen();
 
-class CVideo {
+class CVideo : private boost::noncopyable {
      public:
-	CVideo();
-	CVideo(int x, int y, int bits_per_pixel, int flags);
+		 enum FAKE_TYPES {
+			 NO_FAKE,
+			 FAKE,
+			 FAKE_TEST
+		 };
+	CVideo(FAKE_TYPES type = NO_FAKE);
 	~CVideo();
 
+
 	int modePossible( int x, int y, int bits_per_pixel, int flags );
 	int setMode( int x, int y, int bits_per_pixel, int flags );
 
 	//did the mode change, since the last call to the modeChanged() method?
 	bool modeChanged();
 
-	int setGamma(float gamma);
-
 	//functions to get the dimensions of the current video-mode
 	int getx() const;
 	int gety() const;
-	int getBitsPerPixel();
-	int getBytesPerPixel();
-	int getRedMask();
-	int getGreenMask();
-	int getBlueMask();
-
-	//functions to access the screen
-	void lock();
-	void unlock();
-	int mustLock();
 
 	//blits a surface with black as alpha
 	void blit_surface(int x, int y, surface surf, SDL_Rect* srcrect=NULL, SDL_Rect* clip_rect=NULL);
 	void flip();
 
-	surface getSurface( void );
+	surface getSurface();
 
 	bool isFullScreen() const;
 
@@ -79,7 +74,16 @@ class CVideo {
 	int getBpp();
 
 	void make_fake();
-	bool faked() const { return fake_screen; }
+	/**
+	 * Creates a fake frame buffer for the unit tests.
+	 *
+	 * @param width               The width of the buffer.
+	 * @param height              The heigth of the buffer.
+	 * @param bpp                 The bpp of the buffer.
+	 */
+	void make_test_fake(const unsigned width = 1024,
+			const unsigned height = 768, const unsigned bpp = 32);
+	bool faked() const { return fake_screen_; }
 
 	//functions to set and clear 'help strings'. A 'help string' is like a tooltip, but it appears
 	//at the bottom of the screen, so as to not be intrusive. Setting a help string sets what
@@ -98,12 +102,14 @@ class CVideo {
 
 private:
 
+	void initSDL();
+
 	bool mode_changed_;
 
-	int bpp;	// Store real bits per pixel
+	int bpp_;	// Store real bits per pixel
 
 	//if there is no display at all, but we 'fake' it for clients
-	bool fake_screen;
+	bool fake_screen_;
 
 	//variables for help strings
 	int help_string_;
