@@ -1,6 +1,6 @@
-/* $Id: button.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: button.cpp 46300 2010-09-07 07:50:59Z silene $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,6 +12,8 @@
    See the COPYING file for more details.
 */
 
+#define GETTEXT_DOMAIN "wesnoth-lib"
+
 #include "global.hpp"
 
 #include "widgets/button.hpp"
@@ -20,13 +22,13 @@
 #include "marked-up_text.hpp"
 #include "image.hpp"
 #include "log.hpp"
+#include "serialization/string_utils.hpp"
 #include "sound.hpp"
-#include "util.hpp"
 #include "video.hpp"
 #include "wml_separators.hpp"
-#include "serialization/string_utils.hpp"
 
-#define ERR_DP LOG_STREAM(err, display)
+static lg::log_domain log_display("display");
+#define ERR_DP LOG_STREAM(err, log_display)
 
 namespace gui {
 
@@ -109,8 +111,17 @@ void button::calculate_size()
 
 	if (!change_size) {
 		unsigned w = loc.w - (type_ == TYPE_PRESS ? horizontal_padding : checkbox_horizontal_padding + base_width_);
-		if (type_ != TYPE_IMAGE){
-			label_ = font::make_text_ellipsis(label_, font_size, w, false);
+		if (type_ != TYPE_IMAGE)
+		{
+			int fs = font_size;
+			int style = TTF_STYLE_NORMAL;
+			std::string::const_iterator i_beg = label_.begin(), i_end = label_.end(),
+				i = font::parse_markup(i_beg, i_end, &fs, NULL, &style);
+			if (i != i_end) {
+				std::string tmp(i, i_end);
+				label_.erase(i - i_beg, i_end - i_beg);
+				label_ += font::make_text_ellipsis(tmp, fs, w, style);
+			}
 		}
 	}
 
@@ -125,7 +136,7 @@ void button::calculate_size()
 #ifdef USE_TINY_GUI
 	set_height(textRect_.h+vertical_padding);
 #else
-	set_height(maximum(textRect_.h+vertical_padding,base_height_));
+	set_height(std::max(textRect_.h+vertical_padding,base_height_));
 #endif
 	if(type_ == TYPE_PRESS) {
 #ifdef USE_TINY_GUI
@@ -134,7 +145,7 @@ void button::calculate_size()
 		if(spacing_ == MINIMUM_SPACE) {
 			set_width(textRect_.w + horizontal_padding);
 		} else {
-			set_width(maximum(textRect_.w+horizontal_padding,base_width_));
+			set_width(std::max(textRect_.w+horizontal_padding,base_width_));
 		}
 #endif
 	} else {
@@ -157,6 +168,17 @@ void button::set_check(bool check)
 	}
 }
 
+void button::set_active(bool active)
+{
+	if ((state_ == NORMAL) && active) {
+		state_ = ACTIVE;
+		set_dirty();
+	} else if ((state_ == ACTIVE) && !active) {
+		state_ = NORMAL;
+		set_dirty();
+	}
+}
+
 bool button::checked() const
 {
 	return state_ == PRESSED || state_ == PRESSED_ACTIVE;
@@ -267,7 +289,7 @@ void button::mouse_motion(SDL_MouseMotionEvent const &event)
 		// the cursor is not over the widget
 		if (state_ == PRESSED_ACTIVE)
 			state_ = PRESSED;
-		else if (type_ != TYPE_CHECK && type_ != TYPE_IMAGE || state_ != PRESSED)
+		else if ((type_ != TYPE_CHECK && type_ != TYPE_IMAGE) || state_ != PRESSED)
 			state_ = NORMAL;
 	}
 }
@@ -318,18 +340,21 @@ void button::handle_event(const SDL_Event& event)
 
 	STATE start_state = state_;
 
-	switch(event.type) {
-	case SDL_MOUSEBUTTONDOWN:
-		mouse_down(event.button);
-		break;
-	case SDL_MOUSEBUTTONUP:
-		mouse_up(event.button);
-		break;
-	case SDL_MOUSEMOTION:
-		mouse_motion(event.motion);
-		break;
-	default:
-		return;
+	if (!mouse_locked())
+	{
+		switch(event.type) {
+			case SDL_MOUSEBUTTONDOWN:
+				mouse_down(event.button);
+				break;
+			case SDL_MOUSEBUTTONUP:
+				mouse_up(event.button);
+				break;
+			case SDL_MOUSEMOTION:
+				mouse_motion(event.motion);
+				break;
+			default:
+				return;
+		}
 	}
 
 	if (start_state != state_)
