@@ -1,6 +1,6 @@
-/* $Id: file_menu.cpp 32971 2009-02-21 15:53:26Z ilor $ */
+/* $Id: file_menu.cpp 40562 2010-01-03 19:22:37Z loonycyborg $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -58,18 +58,18 @@ void file_menu::display_current_files() {
 		to_show.push_back(path_up);
 	}
 	std::vector<std::string>::iterator it;
-	for (it = dirs_in_current_dir_.begin(); it != dirs_in_current_dir_.end(); it++) {
+	for (it = dirs_in_current_dir_.begin(); it != dirs_in_current_dir_.end(); ++it) {
 		// Add an image to show that these are directories.
 		std::stringstream ss;
 		ss << font::IMAGE << dir_picture << COLUMN_SEPARATOR << *it;
 		to_show.push_back(ss.str());
 	}
-	for (it = files_in_current_dir_.begin(); it != files_in_current_dir_.end(); it++) {
+	for (it = files_in_current_dir_.begin(); it != files_in_current_dir_.end(); ++it) {
 		const std::string display_string = COLUMN_SEPARATOR + *it;
 		to_show.push_back(display_string);
 	}
 	const int menu_font_size = font::SIZE_NORMAL; // Known from menu.cpp.
-	for (it = to_show.begin(); it != to_show.end(); it++) {
+	for (it = to_show.begin(); it != to_show.end(); ++it) {
 		// Make sure that all lines fit.
 		// Guess the width of the scrollbar to be 30 since it is not accessible from here.
 		// -25 to compensate for the picture column.
@@ -85,7 +85,7 @@ void file_menu::display_current_files() {
 int file_menu::delete_chosen_file() {
 	const int ret = remove(chosen_file_.c_str());
 	if (ret == -1) {
-	//	gui::message_dialog(disp_, "", _("Deletion of the file failed.")).show();
+	//	gui2::show_transient_message(disp_.video(), "", _("Deletion of the file failed."));
 	}
 	else {
 		last_selection_ = -1;
@@ -98,7 +98,7 @@ int file_menu::delete_chosen_file() {
 bool file_menu::make_directory(const std::string& subdir_name) {
 	bool ret = ::make_directory(add_path(current_dir_, subdir_name));
 	if (ret == false) {
-	//	gui::message_dialog(disp_, "", _("Creation of the directory failed.")).show();
+	//	gui2::show_transient_message(disp_.video(), "", _("Creation of the directory failed."));
 	}
 	else {
 		last_selection_ = -1;
@@ -147,7 +147,7 @@ bool file_menu::is_directory(const std::string& fname) const {
 	return ::is_directory(fname);
 }
 
-void file_menu::change_directory(const std::string path) {
+void file_menu::change_directory(const std::string& path) {
 	if(path == path_up)
 	{
 		// Parent dir wanted.
@@ -174,7 +174,7 @@ std::string file_menu::get_choice() const {
 }
 
 
-std::string file_menu::get_path(const std::string file_or_dir) const {
+std::string file_menu::get_path(const std::string& file_or_dir) const {
 	std::string res_path = file_or_dir;
 	if (!::is_directory(file_or_dir)) {
 		size_t index = file_or_dir.find_last_of(path_delim);
@@ -185,7 +185,7 @@ std::string file_menu::get_path(const std::string file_or_dir) const {
 	return res_path;
 }
 
-std::string file_menu::get_path_up(const std::string path, const unsigned levels) const {
+std::string file_menu::get_path_up(const std::string& path, const unsigned levels) const {
 	std::string curr_path = get_path(path);
 	for (unsigned i = 0; i < levels; i++) {
 		if (is_root(curr_path)) {
@@ -214,7 +214,7 @@ std::string file_menu::get_path_up(const std::string path, const unsigned levels
 	return curr_path;
 }
 
-std::string file_menu::strip_last_delim(const std::string path) const {
+std::string file_menu::strip_last_delim(const std::string& path) const {
 	std::string res_string = path;
 	if (path[path.size() - 1] == path_delim) {
 		res_string = path.substr(0, path.size() - 1);
@@ -222,7 +222,7 @@ std::string file_menu::strip_last_delim(const std::string path) const {
 	return res_string;
 }
 
-bool file_menu::is_root(const std::string path) const {
+bool file_menu::is_root(const std::string& path) const {
 #ifdef __AMIGAOS4__
 	return path.size() == 0 || path[path.size()-1] == ':';
 #else
@@ -230,7 +230,7 @@ bool file_menu::is_root(const std::string path) const {
 #endif
 }
 
-std::string file_menu::add_path(const std::string path, const std::string to_add) const
+std::string file_menu::add_path(const std::string& path, const std::string& to_add) const
 {
 	std::string joined_path = strip_last_delim(path);
 	if (to_add.size() > 0) {
@@ -245,9 +245,15 @@ std::string file_menu::add_path(const std::string path, const std::string to_add
 				joined_path += to_add;
 		}
 #endif
-		else if (to_add[0] == path_delim) {
-			joined_path += to_add;
+#ifdef _WIN32
+		else if (to_add.size() > 1 && to_add[1] == ':') {
+			joined_path = to_add;
 		}
+#else
+		else if (to_add[0] == path_delim) {
+			joined_path = to_add;
+		}
+#endif
 		else {
 			joined_path += "/" + to_add;
 		}
