@@ -1,6 +1,6 @@
-/* $Id: label.cpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: label.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2004 - 2009 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2010 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -42,19 +42,6 @@ const std::string& label::get_text() const
 	return text_;
 }
 
-int label::set_size(int size)
-{
-	size_ = size;
-	update_label_size();
-	set_dirty();
-	return size_;
-}
-
-int label::get_size() const
-{
-	return size_;
-}
-
 const SDL_Color& label::set_colour(const SDL_Color& colour)
 {
 	colour_ = colour;
