@@ -1,6 +1,6 @@
-/* $Id: label.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: label.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2010 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -29,9 +29,6 @@ public:
 	const std::string& set_text(const std::string& text);
 	const std::string& get_text() const;
 
-	int set_size(int size);
-	int get_size() const;
-
 	const SDL_Color& set_colour(const SDL_Color& colour);
 	const SDL_Color& get_colour() const;
 
