@@ -1,6 +1,6 @@
-/* $Id: menu.cpp 32635 2009-02-07 13:00:40Z baufo $ */
+/* $Id: menu.cpp 46300 2010-09-07 07:50:59Z silene $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -47,6 +47,19 @@ menu::basic_sorter& menu::basic_sorter::set_numeric_sort(int column)
 	return *this;
 }
 
+menu::basic_sorter& menu::basic_sorter::set_xp_sort(int column)
+{
+	xp_sort_.insert(column);
+	return *this;
+}
+
+menu::basic_sorter& menu::basic_sorter::set_level_sort(int level_column, int xp_column)
+{
+	level_sort_.insert(level_column);
+	xp_col_ = xp_column;
+	return *this;
+}
+
 menu::basic_sorter& menu::basic_sorter::set_id_sort(int column)
 {
 	id_sort_.insert(column);
@@ -76,7 +89,27 @@ bool menu::basic_sorter::column_sortable(int column) const
 	}
 
 	return alpha_sort_.count(column) == 1 || numeric_sort_.count(column) == 1 ||
-		   pos_sort_.count(column) == 1 || id_sort_.count(column) == 1;
+		   pos_sort_.count(column) == 1 || id_sort_.count(column) == 1 ||
+		    xp_sort_.count(column) == 1 || level_sort_.count(column) == 1;
+}
+
+static std::pair<int, int> parse_fraction(const std::string& s)
+{
+	std::vector<std::string> parts = utils::split(s, '/', 0);
+	parts.resize(2);
+	int num = lexical_cast_default<int>(parts[0], 0);
+	int denom = lexical_cast_default<int>(parts[1], 0);
+	return std::make_pair(num, denom);
+}
+
+static int xp_to_advance(const std::string& s) {
+	std::pair<int,int> xp_frac = parse_fraction(s);
+
+	//consider units without AMLA or advancement as having xp_max=1000000
+	if(xp_frac.second == 0)
+		xp_frac.second = 1000000;
+
+	return xp_frac.second - xp_frac.first;
 }
 
 bool menu::basic_sorter::less(int column, const item& row1, const item& row2) const
@@ -98,10 +131,10 @@ bool menu::basic_sorter::less(int column, const item& row1, const item& row2) co
 		return true;
 	}
 
-	if(alpha_sort_.count(column) == 1) {
-		const std::string& item1 = font::del_tags(row1.fields[column]);
-		const std::string& item2 = font::del_tags(row2.fields[column]);
+	const std::string& item1 = font::del_tags(row1.fields[column]);
+	const std::string& item2 = font::del_tags(row2.fields[column]);
 
+	if(alpha_sort_.count(column) == 1) {
 		std::string::const_iterator begin1 = item1.begin(), end1 = item1.end(),
 		                            begin2 = item2.begin(), end2 = item2.end();
 		while(begin1 != end1 && is_wml_separator(*begin1)) {
@@ -114,21 +147,22 @@ bool menu::basic_sorter::less(int column, const item& row1, const item& row2) co
 
 		return std::lexicographical_compare(begin1,end1,begin2,end2,chars_less_insensitive);
 	} else if(numeric_sort_.count(column) == 1) {
-		const std::string& item1 = font::del_tags(row1.fields[column]);
-		const std::string& item2 = font::del_tags(row2.fields[column]);
+		int val_1 = lexical_cast_default<int>(item1, 0);
+		int val_2 = lexical_cast_default<int>(item2, 0);
 
-		const char* a = item1.c_str();
-		const char* b = item2.c_str();
-
-		while(*a != 0 && !isdigit(*a)) {
-			++a;
+		return val_1 > val_2;
+	} else if(xp_sort_.count(column) == 1) {
+		return xp_to_advance(item1) < xp_to_advance(item2);
+	} else if(level_sort_.count(column) == 1) {
+		int level_1 = lexical_cast_default<int>(item1, 0);
+		int level_2 = lexical_cast_default<int>(item2, 0);
+		if (level_1 == level_2) {
+			//break tie using xp
+			const std::string& xp_item1 = font::del_tags(row1.fields[xp_col_]);
+			const std::string& xp_item2 = font::del_tags(row2.fields[xp_col_]);
+			return xp_to_advance(xp_item1) < xp_to_advance(xp_item2);
 		}
-
-		while(*b != 0 && !isdigit(*b)) {
-			++b;
-		}
-
-		return atoi(a) > atoi(b);
+		return level_1 > level_2;
 	}
 
 	const std::map<int,std::vector<int> >::const_iterator itor = pos_sort_.find(column);
@@ -617,8 +651,8 @@ void menu::handle_event(const SDL_Event& event)
 			x = event.button.x;
 			y = event.button.y;
 		} else {
-			x = (long)event.user.data1;
-			y = (long)event.user.data2;
+			x = reinterpret_cast<long>(event.user.data1);
+			y = reinterpret_cast<long>(event.user.data2);
 		}
 
 		const int item = hit(x,y);
@@ -749,9 +783,9 @@ void menu::sort_by(int column)
 
 SDL_Rect menu::style::item_size(const std::string& item) const {
 	SDL_Rect res = {0,0,0,0};
-	std::vector<std::string> img_text_items = utils::split(item, IMG_TEXT_SEPARATOR);
+	std::vector<std::string> img_text_items = utils::split(item, IMG_TEXT_SEPARATOR, utils::REMOVE_EMPTY);
 	for (std::vector<std::string>::const_iterator it = img_text_items.begin();
-		 it != img_text_items.end(); it++) {
+		 it != img_text_items.end(); ++it) {
 		if (res.w > 0 || res.h > 0) {
 			// Not the first item, add the spacing.
 			res.w += 5;
@@ -887,9 +921,9 @@ void menu::draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
 		const int last_x = xpos;
 		column.w = widths[i];
 		std::string str = row[i];
-		std::vector<std::string> img_text_items = utils::split(str, IMG_TEXT_SEPARATOR);
+		std::vector<std::string> img_text_items = utils::split(str, IMG_TEXT_SEPARATOR, utils::REMOVE_EMPTY);
 		for (std::vector<std::string>::const_iterator it = img_text_items.begin();
-			 it != img_text_items.end(); it++) {
+			 it != img_text_items.end(); ++it) {
 			str = *it;
 			if (!str.empty() && str[0] == IMAGE_PREFIX) {
 				const std::string image_name(str.begin()+1,str.end());
@@ -910,11 +944,20 @@ void menu::draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
 				column.x = xpos;
 				const bool has_wrap = (str.find_first_of("\r\n") != std::string::npos);
 				//prevent ellipsis calculation if there is any line wrapping
-				const std::string to_show =
-					(use_ellipsis_ && !has_wrap) ?
-						font::make_text_ellipsis(str, style_->get_font_size(),
-							loc.w - (xpos - rect.x) - 2*style_->get_thickness(), false, true)
-						: str;
+				std::string to_show = str;
+				if (use_ellipsis_ && !has_wrap)
+				{
+					int fs = style_->get_font_size();
+					int style = TTF_STYLE_NORMAL;
+					int w = loc.w - (xpos - rect.x) - 2 * style_->get_thickness();
+					std::string::const_iterator i_beg = to_show.begin(), i_end = to_show.end(),
+						i = font::parse_markup(i_beg, i_end, &fs, NULL, &style);
+					if (i != i_end) {
+						std::string tmp(i, i_end);
+						to_show.erase(i - i_beg, i_end - i_beg);
+						to_show += font::make_text_ellipsis(tmp, fs, w, style);
+					}
+				}
 				const SDL_Rect& text_size = font::text_area(str,style_->get_font_size());
 				const size_t y = rect.y + (rect.h - text_size.h)/2;
 				font::draw_text(&video(),column,style_->get_font_size(),font::NORMAL_COLOUR,to_show,xpos,y);
@@ -994,36 +1037,6 @@ void menu::draw()
 	set_dirty(false);
 }
 
-void menu::wrap_words()
-{
-	int total_width = max_width_ - (style_->get_cell_padding() + (2 * style_->get_thickness()));
-	if(has_scrollbar()) {
-		total_width -= scrollbar_width();
-	}
-	if(total_width <= 0) {
-		return;
-	}
-	std::vector<int> const &widths = column_widths();
-	for(std::vector<item>::iterator i = items_.begin(); i != items_.end(); ++i) {
-		int space_remaining = total_width;
-		for(size_t col = 0; col < i->fields.size() && col < widths.size(); ++col) {
-			std::string &to_wrap = i->fields[col];
-			if (!to_wrap.empty()) {
-				if(widths[col] > space_remaining && to_wrap[0] != IMAGE_PREFIX) {
-					to_wrap = font::word_wrap_text(to_wrap, style_->get_font_size(), space_remaining);
-					break;
-				}
-				space_remaining -= widths[col] + 5;
-			}
-		}
-	}
-	itemRects_.clear();
-	column_widths_.clear();
-	max_items_ = -1; // Force recalculation of the max items.
-	item_height_ = -1; // Force recalculation of the item height.
-	update_size();
-}
-
 int menu::hit(int x, int y) const
 {
 	SDL_Rect const &loc = inner_location();
