@@ -1,6 +1,6 @@
-/* $Id: menu.hpp 32028 2009-01-10 11:37:02Z baufo $ */
+/* $Id: menu.hpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -140,6 +140,8 @@ public:
 
 		basic_sorter& set_alpha_sort(int column);
 		basic_sorter& set_numeric_sort(int column);
+		basic_sorter& set_xp_sort(int column);
+		basic_sorter& set_level_sort(int level_column, int xp_column);
 		basic_sorter& set_id_sort(int column);
 		basic_sorter& set_redirect_sort(int column, int to);
 		basic_sorter& set_position_sort(int column, const std::vector<int>& pos);
@@ -148,9 +150,10 @@ public:
 		virtual bool less(int column, const item& row1, const item& row2) const;
 
 	private:
-		std::set<int> alpha_sort_, numeric_sort_, id_sort_;
+		std::set<int> alpha_sort_, numeric_sort_, id_sort_, xp_sort_, level_sort_;
 		std::map<int,int> redirect_sort_;
 		std::map<int,std::vector<int> > pos_sort_;
+		int xp_col_; //used by level sort
 	};
 
 	menu(CVideo& video, const std::vector<std::string>& items,
@@ -198,7 +201,6 @@ public:
 	//this should be changed to a more object-oriented approach
 	void set_sorter(sorter *s);
 	void sort_by(int column);
-	void wrap_words();
 
 protected:
 	bool item_ends_with_image(const std::string& item) const;
