@@ -1,7 +1,7 @@
-/* $Id: scrollbar.cpp 23842 2008-02-16 08:47:16Z mordante $*/
+/* $Id: scrollbar.cpp 40489 2010-01-01 13:16:49Z mordante $*/
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-                 2004 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+                 2004 - 2010 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,8 +13,9 @@
    See the COPYING file for more details.
 */
 
-//! @file widgets/scrollbar.cpp
-//!
+/** @file src/widgets/scrollbar.cpp */
+
+#define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "global.hpp"
 
@@ -22,7 +23,6 @@
 #include "image.hpp"
 #include "video.hpp"
 
-#include <algorithm>
 #include <iostream>
 
 namespace {
@@ -284,13 +284,14 @@ void scrollbar::draw_contents()
 
 void scrollbar::handle_event(const SDL_Event& event)
 {
-	if (hidden())
+	if (mouse_locked() || hidden())
 		return;
 
 	STATE new_state = state_;
 	SDL_Rect const &grip = grip_area();
 	SDL_Rect const &groove = groove_area();
 
+
 	switch (event.type) {
 	case SDL_MOUSEBUTTONUP:
 	{
@@ -313,7 +314,7 @@ void scrollbar::handle_event(const SDL_Event& event)
 			new_state = DRAGGED;
 		} else if (on_groove && e.button == SDL_BUTTON_LEFT && groove.h != grip.h) {
 			if (e.y < grip.y)
-				move_position(-grip_height_);
+				move_position(-static_cast<int>(grip_height_));
 			else
 				move_position(grip_height_);
 		} else if (on_groove && e.button == SDL_BUTTON_MIDDLE) {
