@@ -1,6 +1,6 @@
-/* $Id: slider.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: slider.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,6 +12,8 @@
    See the COPYING file for more details.
 */
 
+#define GETTEXT_DOMAIN "wesnoth-lib"
+
 #include "global.hpp"
 
 #include "widgets/slider.hpp"
@@ -21,9 +23,6 @@
 #include "sound.hpp"
 #include "video.hpp"
 
-#include <algorithm>
-#include <iostream>
-#include <sstream>
 
 namespace {
 	const std::string slider_image = "buttons/slider.png";
@@ -244,13 +243,16 @@ void slider::handle_event(const SDL_Event& event)
 
 	switch(event.type) {
 	case SDL_MOUSEBUTTONUP:
-		state_ = NORMAL;
+		if (!mouse_locked())
+			state_ = NORMAL;
 		break;
 	case SDL_MOUSEBUTTONDOWN:
-		mouse_down(event.button);
+		if (!mouse_locked())
+			mouse_down(event.button);
 		break;
 	case SDL_MOUSEMOTION:
-		mouse_motion(event.motion);
+		if (!mouse_locked())
+			mouse_motion(event.motion);
 		break;
 	case SDL_KEYDOWN:
 		if(focus(&event)) {
