@@ -1,6 +1,6 @@
-/* $Id: textbox.cpp 33261 2009-03-01 18:12:59Z silene $ */
+/* $Id: textbox.cpp 42225 2010-04-23 13:57:22Z ilor $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -21,10 +21,9 @@
 #include "log.hpp"
 #include "video.hpp"
 
-
-
-#define WRN_DISPLAY LOG_STREAM(warn, display)
-#define DBG_G LOG_STREAM(debug, general)
+static lg::log_domain log_display("display");
+#define WRN_DP LOG_STREAM(warn, log_display)
+#define DBG_G LOG_STREAM(debug, lg::general)
 
 namespace gui {
 
@@ -195,25 +194,6 @@ void textbox::draw_contents()
 	update_rect(loc);
 }
 
-void textbox::process()
-{
-	if(editable_) {
-		if(focus(NULL)) {
-			const int ticks = SDL_GetTicks();
-			if(ticks > show_cursor_at_+500) {
-				show_cursor_ = !show_cursor_;
-				show_cursor_at_ = ticks;
-				set_dirty();
-			}
-		} else if(show_cursor_ == true) {
-			show_cursor_ = false;
-			set_dirty();
-		}
-	}
-
-	draw();
-}
-
 void textbox::set_editable(bool value)
 {
 	editable_ = value;
@@ -402,7 +382,7 @@ void textbox::handle_event(const SDL_Event& event)
 	//Sanity check: verify that selection start and end are within text
 	//boundaries
 	if(is_selection() && !(size_t(selstart_) <= text_.size() && size_t(selend_) <= text_.size())) {
-		WRN_DISPLAY << "out-of-boundary selection\n";
+		WRN_DP << "out-of-boundary selection\n";
 		selstart_ = selend_ = -1;
 	}
 
