@@ -1,6 +1,6 @@
-/* $Id: wml_exception.cpp 32337 2009-01-27 19:24:01Z shadowmaster $ */
+/* $Id: wml_exception.cpp 40489 2010-01-01 13:16:49Z mordante $ */
 /*
-   Copyright (C) 2007 - 2009 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2007 - 2010 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -22,8 +22,9 @@
 #include "global.hpp"
 #include "wml_exception.hpp"
 
+#include "display.hpp"
 #include "gettext.hpp"
-#include "show_dialog.hpp"
+#include "gui/dialogs/message.hpp"
 #include "formula_string_utils.hpp"
 
 
@@ -54,7 +55,7 @@ void twml_exception::show(display &disp)
 		<< _("When reporting the bug please include the following error message :")
 		<< "\n" << dev_message;
 
-	gui::show_error_message(disp, sstr.str());
+	gui2::show_error_message(disp.video(), sstr.str());
 }
 
 t_string missing_mandatory_wml_key(const std::string& section, const std::string& key,
